# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2008 Alex Holkner
# Copyright (c) 2008-2020 pyglet contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------

"""Wrapper for http://oss.sgi.com/projects/ogl-sample/ABI/glext.h

Generated by tools/gengl.py.
Do not modify this file.
"""

from ctypes import *
from pyglet.gl.lib import link_GL as _link_function
from pyglet.gl.lib import c_ptrdiff_t

# BEGIN GENERATED CONTENT (do not edit below this line)

# This content is generated by gengl.py.
# Wrapper for http://www.opengl.org/registry/api/glext.h


# ARB_multitexture (/usr/include/GL/gl.h:1962)
GL_GLEXT_VERSION = 82 	# GL/glext.h:34
# VERSION_1_2 (GL/glext.h:54)
# VERSION_1_2_DEPRECATED (GL/glext.h:93)
GL_RESCALE_NORMAL = 32826 	# GL/glext.h:94
GL_LIGHT_MODEL_COLOR_CONTROL = 33272 	# GL/glext.h:95
GL_SINGLE_COLOR = 33273 	# GL/glext.h:96
GL_SEPARATE_SPECULAR_COLOR = 33274 	# GL/glext.h:97
GL_ALIASED_POINT_SIZE_RANGE = 33901 	# GL/glext.h:98
# ARB_imaging (GL/glext.h:101)
# ARB_imaging_DEPRECATED (GL/glext.h:115)
GL_CONVOLUTION_1D = 32784 	# GL/glext.h:116
GL_CONVOLUTION_2D = 32785 	# GL/glext.h:117
GL_SEPARABLE_2D = 32786 	# GL/glext.h:118
GL_CONVOLUTION_BORDER_MODE = 32787 	# GL/glext.h:119
GL_CONVOLUTION_FILTER_SCALE = 32788 	# GL/glext.h:120
GL_CONVOLUTION_FILTER_BIAS = 32789 	# GL/glext.h:121
GL_REDUCE = 32790 	# GL/glext.h:122
GL_CONVOLUTION_FORMAT = 32791 	# GL/glext.h:123
GL_CONVOLUTION_WIDTH = 32792 	# GL/glext.h:124
GL_CONVOLUTION_HEIGHT = 32793 	# GL/glext.h:125
GL_MAX_CONVOLUTION_WIDTH = 32794 	# GL/glext.h:126
GL_MAX_CONVOLUTION_HEIGHT = 32795 	# GL/glext.h:127
GL_POST_CONVOLUTION_RED_SCALE = 32796 	# GL/glext.h:128
GL_POST_CONVOLUTION_GREEN_SCALE = 32797 	# GL/glext.h:129
GL_POST_CONVOLUTION_BLUE_SCALE = 32798 	# GL/glext.h:130
GL_POST_CONVOLUTION_ALPHA_SCALE = 32799 	# GL/glext.h:131
GL_POST_CONVOLUTION_RED_BIAS = 32800 	# GL/glext.h:132
GL_POST_CONVOLUTION_GREEN_BIAS = 32801 	# GL/glext.h:133
GL_POST_CONVOLUTION_BLUE_BIAS = 32802 	# GL/glext.h:134
GL_POST_CONVOLUTION_ALPHA_BIAS = 32803 	# GL/glext.h:135
GL_HISTOGRAM = 32804 	# GL/glext.h:136
GL_PROXY_HISTOGRAM = 32805 	# GL/glext.h:137
GL_HISTOGRAM_WIDTH = 32806 	# GL/glext.h:138
GL_HISTOGRAM_FORMAT = 32807 	# GL/glext.h:139
GL_HISTOGRAM_RED_SIZE = 32808 	# GL/glext.h:140
GL_HISTOGRAM_GREEN_SIZE = 32809 	# GL/glext.h:141
GL_HISTOGRAM_BLUE_SIZE = 32810 	# GL/glext.h:142
GL_HISTOGRAM_ALPHA_SIZE = 32811 	# GL/glext.h:143
GL_HISTOGRAM_LUMINANCE_SIZE = 32812 	# GL/glext.h:144
GL_HISTOGRAM_SINK = 32813 	# GL/glext.h:145
GL_MINMAX = 32814 	# GL/glext.h:146
GL_MINMAX_FORMAT = 32815 	# GL/glext.h:147
GL_MINMAX_SINK = 32816 	# GL/glext.h:148
GL_TABLE_TOO_LARGE = 32817 	# GL/glext.h:149
GL_COLOR_MATRIX = 32945 	# GL/glext.h:150
GL_COLOR_MATRIX_STACK_DEPTH = 32946 	# GL/glext.h:151
GL_MAX_COLOR_MATRIX_STACK_DEPTH = 32947 	# GL/glext.h:152
GL_POST_COLOR_MATRIX_RED_SCALE = 32948 	# GL/glext.h:153
GL_POST_COLOR_MATRIX_GREEN_SCALE = 32949 	# GL/glext.h:154
GL_POST_COLOR_MATRIX_BLUE_SCALE = 32950 	# GL/glext.h:155
GL_POST_COLOR_MATRIX_ALPHA_SCALE = 32951 	# GL/glext.h:156
GL_POST_COLOR_MATRIX_RED_BIAS = 32952 	# GL/glext.h:157
GL_POST_COLOR_MATRIX_GREEN_BIAS = 32953 	# GL/glext.h:158
GL_POST_COLOR_MATRIX_BLUE_BIAS = 32954 	# GL/glext.h:159
GL_POST_COLOR_MATRIX_ALPHA_BIAS = 32955 	# GL/glext.h:160
GL_COLOR_TABLE = 32976 	# GL/glext.h:161
GL_POST_CONVOLUTION_COLOR_TABLE = 32977 	# GL/glext.h:162
GL_POST_COLOR_MATRIX_COLOR_TABLE = 32978 	# GL/glext.h:163
GL_PROXY_COLOR_TABLE = 32979 	# GL/glext.h:164
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE = 32980 	# GL/glext.h:165
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE = 32981 	# GL/glext.h:166
GL_COLOR_TABLE_SCALE = 32982 	# GL/glext.h:167
GL_COLOR_TABLE_BIAS = 32983 	# GL/glext.h:168
GL_COLOR_TABLE_FORMAT = 32984 	# GL/glext.h:169
GL_COLOR_TABLE_WIDTH = 32985 	# GL/glext.h:170
GL_COLOR_TABLE_RED_SIZE = 32986 	# GL/glext.h:171
GL_COLOR_TABLE_GREEN_SIZE = 32987 	# GL/glext.h:172
GL_COLOR_TABLE_BLUE_SIZE = 32988 	# GL/glext.h:173
GL_COLOR_TABLE_ALPHA_SIZE = 32989 	# GL/glext.h:174
GL_COLOR_TABLE_LUMINANCE_SIZE = 32990 	# GL/glext.h:175
GL_COLOR_TABLE_INTENSITY_SIZE = 32991 	# GL/glext.h:176
GL_CONSTANT_BORDER = 33105 	# GL/glext.h:177
GL_REPLICATE_BORDER = 33107 	# GL/glext.h:178
GL_CONVOLUTION_BORDER_COLOR = 33108 	# GL/glext.h:179
# VERSION_1_3 (GL/glext.h:182)
# VERSION_1_3_DEPRECATED (GL/glext.h:244)
GL_CLIENT_ACTIVE_TEXTURE = 34017 	# GL/glext.h:245
GL_MAX_TEXTURE_UNITS = 34018 	# GL/glext.h:246
GL_TRANSPOSE_MODELVIEW_MATRIX = 34019 	# GL/glext.h:247
GL_TRANSPOSE_PROJECTION_MATRIX = 34020 	# GL/glext.h:248
GL_TRANSPOSE_TEXTURE_MATRIX = 34021 	# GL/glext.h:249
GL_TRANSPOSE_COLOR_MATRIX = 34022 	# GL/glext.h:250
GL_MULTISAMPLE_BIT = 536870912 	# GL/glext.h:251
GL_NORMAL_MAP = 34065 	# GL/glext.h:252
GL_REFLECTION_MAP = 34066 	# GL/glext.h:253
GL_COMPRESSED_ALPHA = 34025 	# GL/glext.h:254
GL_COMPRESSED_LUMINANCE = 34026 	# GL/glext.h:255
GL_COMPRESSED_LUMINANCE_ALPHA = 34027 	# GL/glext.h:256
GL_COMPRESSED_INTENSITY = 34028 	# GL/glext.h:257
GL_COMBINE = 34160 	# GL/glext.h:258
GL_COMBINE_RGB = 34161 	# GL/glext.h:259
GL_COMBINE_ALPHA = 34162 	# GL/glext.h:260
GL_SOURCE0_RGB = 34176 	# GL/glext.h:261
GL_SOURCE1_RGB = 34177 	# GL/glext.h:262
GL_SOURCE2_RGB = 34178 	# GL/glext.h:263
GL_SOURCE0_ALPHA = 34184 	# GL/glext.h:264
GL_SOURCE1_ALPHA = 34185 	# GL/glext.h:265
GL_SOURCE2_ALPHA = 34186 	# GL/glext.h:266
GL_OPERAND0_RGB = 34192 	# GL/glext.h:267
GL_OPERAND1_RGB = 34193 	# GL/glext.h:268
GL_OPERAND2_RGB = 34194 	# GL/glext.h:269
GL_OPERAND0_ALPHA = 34200 	# GL/glext.h:270
GL_OPERAND1_ALPHA = 34201 	# GL/glext.h:271
GL_OPERAND2_ALPHA = 34202 	# GL/glext.h:272
GL_RGB_SCALE = 34163 	# GL/glext.h:273
GL_ADD_SIGNED = 34164 	# GL/glext.h:274
GL_INTERPOLATE = 34165 	# GL/glext.h:275
GL_SUBTRACT = 34023 	# GL/glext.h:276
GL_CONSTANT = 34166 	# GL/glext.h:277
GL_PRIMARY_COLOR = 34167 	# GL/glext.h:278
GL_PREVIOUS = 34168 	# GL/glext.h:279
GL_DOT3_RGB = 34478 	# GL/glext.h:280
GL_DOT3_RGBA = 34479 	# GL/glext.h:281
# VERSION_1_4 (GL/glext.h:284)
GL_BLEND_DST_RGB = 32968 	# GL/glext.h:285
GL_BLEND_SRC_RGB = 32969 	# GL/glext.h:286
GL_BLEND_DST_ALPHA = 32970 	# GL/glext.h:287
GL_BLEND_SRC_ALPHA = 32971 	# GL/glext.h:288
GL_POINT_FADE_THRESHOLD_SIZE = 33064 	# GL/glext.h:289
GL_DEPTH_COMPONENT16 = 33189 	# GL/glext.h:290
GL_DEPTH_COMPONENT24 = 33190 	# GL/glext.h:291
GL_DEPTH_COMPONENT32 = 33191 	# GL/glext.h:292
GL_MIRRORED_REPEAT = 33648 	# GL/glext.h:293
GL_MAX_TEXTURE_LOD_BIAS = 34045 	# GL/glext.h:294
GL_TEXTURE_LOD_BIAS = 34049 	# GL/glext.h:295
GL_INCR_WRAP = 34055 	# GL/glext.h:296
GL_DECR_WRAP = 34056 	# GL/glext.h:297
GL_TEXTURE_DEPTH_SIZE = 34890 	# GL/glext.h:298
GL_TEXTURE_COMPARE_MODE = 34892 	# GL/glext.h:299
GL_TEXTURE_COMPARE_FUNC = 34893 	# GL/glext.h:300
# VERSION_1_4_DEPRECATED (GL/glext.h:303)
GL_POINT_SIZE_MIN = 33062 	# GL/glext.h:304
GL_POINT_SIZE_MAX = 33063 	# GL/glext.h:305
GL_POINT_DISTANCE_ATTENUATION = 33065 	# GL/glext.h:306
GL_GENERATE_MIPMAP = 33169 	# GL/glext.h:307
GL_GENERATE_MIPMAP_HINT = 33170 	# GL/glext.h:308
GL_FOG_COORDINATE_SOURCE = 33872 	# GL/glext.h:309
GL_FOG_COORDINATE = 33873 	# GL/glext.h:310
GL_FRAGMENT_DEPTH = 33874 	# GL/glext.h:311
GL_CURRENT_FOG_COORDINATE = 33875 	# GL/glext.h:312
GL_FOG_COORDINATE_ARRAY_TYPE = 33876 	# GL/glext.h:313
GL_FOG_COORDINATE_ARRAY_STRIDE = 33877 	# GL/glext.h:314
GL_FOG_COORDINATE_ARRAY_POINTER = 33878 	# GL/glext.h:315
GL_FOG_COORDINATE_ARRAY = 33879 	# GL/glext.h:316
GL_COLOR_SUM = 33880 	# GL/glext.h:317
GL_CURRENT_SECONDARY_COLOR = 33881 	# GL/glext.h:318
GL_SECONDARY_COLOR_ARRAY_SIZE = 33882 	# GL/glext.h:319
GL_SECONDARY_COLOR_ARRAY_TYPE = 33883 	# GL/glext.h:320
GL_SECONDARY_COLOR_ARRAY_STRIDE = 33884 	# GL/glext.h:321
GL_SECONDARY_COLOR_ARRAY_POINTER = 33885 	# GL/glext.h:322
GL_SECONDARY_COLOR_ARRAY = 33886 	# GL/glext.h:323
GL_TEXTURE_FILTER_CONTROL = 34048 	# GL/glext.h:324
GL_DEPTH_TEXTURE_MODE = 34891 	# GL/glext.h:325
GL_COMPARE_R_TO_TEXTURE = 34894 	# GL/glext.h:326
# VERSION_1_5 (GL/glext.h:329)
GL_BUFFER_SIZE = 34660 	# GL/glext.h:330
GL_BUFFER_USAGE = 34661 	# GL/glext.h:331
GL_QUERY_COUNTER_BITS = 34916 	# GL/glext.h:332
GL_CURRENT_QUERY = 34917 	# GL/glext.h:333
GL_QUERY_RESULT = 34918 	# GL/glext.h:334
GL_QUERY_RESULT_AVAILABLE = 34919 	# GL/glext.h:335
GL_ARRAY_BUFFER = 34962 	# GL/glext.h:336
GL_ELEMENT_ARRAY_BUFFER = 34963 	# GL/glext.h:337
GL_ARRAY_BUFFER_BINDING = 34964 	# GL/glext.h:338
GL_ELEMENT_ARRAY_BUFFER_BINDING = 34965 	# GL/glext.h:339
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = 34975 	# GL/glext.h:340
GL_READ_ONLY = 35000 	# GL/glext.h:341
GL_WRITE_ONLY = 35001 	# GL/glext.h:342
GL_READ_WRITE = 35002 	# GL/glext.h:343
GL_BUFFER_ACCESS = 35003 	# GL/glext.h:344
GL_BUFFER_MAPPED = 35004 	# GL/glext.h:345
GL_BUFFER_MAP_POINTER = 35005 	# GL/glext.h:346
GL_STREAM_DRAW = 35040 	# GL/glext.h:347
GL_STREAM_READ = 35041 	# GL/glext.h:348
GL_STREAM_COPY = 35042 	# GL/glext.h:349
GL_STATIC_DRAW = 35044 	# GL/glext.h:350
GL_STATIC_READ = 35045 	# GL/glext.h:351
GL_STATIC_COPY = 35046 	# GL/glext.h:352
GL_DYNAMIC_DRAW = 35048 	# GL/glext.h:353
GL_DYNAMIC_READ = 35049 	# GL/glext.h:354
GL_DYNAMIC_COPY = 35050 	# GL/glext.h:355
GL_SAMPLES_PASSED = 35092 	# GL/glext.h:356
# VERSION_1_5_DEPRECATED (GL/glext.h:359)
GL_VERTEX_ARRAY_BUFFER_BINDING = 34966 	# GL/glext.h:360
GL_NORMAL_ARRAY_BUFFER_BINDING = 34967 	# GL/glext.h:361
GL_COLOR_ARRAY_BUFFER_BINDING = 34968 	# GL/glext.h:362
GL_INDEX_ARRAY_BUFFER_BINDING = 34969 	# GL/glext.h:363
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING = 34970 	# GL/glext.h:364
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING = 34971 	# GL/glext.h:365
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING = 34972 	# GL/glext.h:366
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:367
GL_WEIGHT_ARRAY_BUFFER_BINDING = 34974 	# GL/glext.h:368
GL_FOG_COORD_SRC = 33872 	# GL/glext.h:369
GL_FOG_COORD = 33873 	# GL/glext.h:370
GL_CURRENT_FOG_COORD = 33875 	# GL/glext.h:371
GL_FOG_COORD_ARRAY_TYPE = 33876 	# GL/glext.h:372
GL_FOG_COORD_ARRAY_STRIDE = 33877 	# GL/glext.h:373
GL_FOG_COORD_ARRAY_POINTER = 33878 	# GL/glext.h:374
GL_FOG_COORD_ARRAY = 33879 	# GL/glext.h:375
GL_FOG_COORD_ARRAY_BUFFER_BINDING = 34973 	# GL/glext.h:376
GL_SRC0_RGB = 34176 	# GL/glext.h:377
GL_SRC1_RGB = 34177 	# GL/glext.h:378
GL_SRC2_RGB = 34178 	# GL/glext.h:379
GL_SRC0_ALPHA = 34184 	# GL/glext.h:380
GL_SRC1_ALPHA = 34185 	# GL/glext.h:381
GL_SRC2_ALPHA = 34186 	# GL/glext.h:382
# VERSION_2_0 (GL/glext.h:385)
GL_BLEND_EQUATION_RGB = 32777 	# GL/glext.h:386
GL_VERTEX_ATTRIB_ARRAY_ENABLED = 34338 	# GL/glext.h:387
GL_VERTEX_ATTRIB_ARRAY_SIZE = 34339 	# GL/glext.h:388
GL_VERTEX_ATTRIB_ARRAY_STRIDE = 34340 	# GL/glext.h:389
GL_VERTEX_ATTRIB_ARRAY_TYPE = 34341 	# GL/glext.h:390
GL_CURRENT_VERTEX_ATTRIB = 34342 	# GL/glext.h:391
GL_VERTEX_PROGRAM_POINT_SIZE = 34370 	# GL/glext.h:392
GL_VERTEX_ATTRIB_ARRAY_POINTER = 34373 	# GL/glext.h:393
GL_STENCIL_BACK_FUNC = 34816 	# GL/glext.h:394
GL_STENCIL_BACK_FAIL = 34817 	# GL/glext.h:395
GL_STENCIL_BACK_PASS_DEPTH_FAIL = 34818 	# GL/glext.h:396
GL_STENCIL_BACK_PASS_DEPTH_PASS = 34819 	# GL/glext.h:397
GL_MAX_DRAW_BUFFERS = 34852 	# GL/glext.h:398
GL_DRAW_BUFFER0 = 34853 	# GL/glext.h:399
GL_DRAW_BUFFER1 = 34854 	# GL/glext.h:400
GL_DRAW_BUFFER2 = 34855 	# GL/glext.h:401
GL_DRAW_BUFFER3 = 34856 	# GL/glext.h:402
GL_DRAW_BUFFER4 = 34857 	# GL/glext.h:403
GL_DRAW_BUFFER5 = 34858 	# GL/glext.h:404
GL_DRAW_BUFFER6 = 34859 	# GL/glext.h:405
GL_DRAW_BUFFER7 = 34860 	# GL/glext.h:406
GL_DRAW_BUFFER8 = 34861 	# GL/glext.h:407
GL_DRAW_BUFFER9 = 34862 	# GL/glext.h:408
GL_DRAW_BUFFER10 = 34863 	# GL/glext.h:409
GL_DRAW_BUFFER11 = 34864 	# GL/glext.h:410
GL_DRAW_BUFFER12 = 34865 	# GL/glext.h:411
GL_DRAW_BUFFER13 = 34866 	# GL/glext.h:412
GL_DRAW_BUFFER14 = 34867 	# GL/glext.h:413
GL_DRAW_BUFFER15 = 34868 	# GL/glext.h:414
GL_BLEND_EQUATION_ALPHA = 34877 	# GL/glext.h:415
GL_MAX_VERTEX_ATTRIBS = 34921 	# GL/glext.h:416
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = 34922 	# GL/glext.h:417
GL_MAX_TEXTURE_IMAGE_UNITS = 34930 	# GL/glext.h:418
GL_FRAGMENT_SHADER = 35632 	# GL/glext.h:419
GL_VERTEX_SHADER = 35633 	# GL/glext.h:420
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS = 35657 	# GL/glext.h:421
GL_MAX_VERTEX_UNIFORM_COMPONENTS = 35658 	# GL/glext.h:422
GL_MAX_VARYING_FLOATS = 35659 	# GL/glext.h:423
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = 35660 	# GL/glext.h:424
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = 35661 	# GL/glext.h:425
GL_SHADER_TYPE = 35663 	# GL/glext.h:426
GL_FLOAT_VEC2 = 35664 	# GL/glext.h:427
GL_FLOAT_VEC3 = 35665 	# GL/glext.h:428
GL_FLOAT_VEC4 = 35666 	# GL/glext.h:429
GL_INT_VEC2 = 35667 	# GL/glext.h:430
GL_INT_VEC3 = 35668 	# GL/glext.h:431
GL_INT_VEC4 = 35669 	# GL/glext.h:432
GL_BOOL = 35670 	# GL/glext.h:433
GL_BOOL_VEC2 = 35671 	# GL/glext.h:434
GL_BOOL_VEC3 = 35672 	# GL/glext.h:435
GL_BOOL_VEC4 = 35673 	# GL/glext.h:436
GL_FLOAT_MAT2 = 35674 	# GL/glext.h:437
GL_FLOAT_MAT3 = 35675 	# GL/glext.h:438
GL_FLOAT_MAT4 = 35676 	# GL/glext.h:439
GL_SAMPLER_1D = 35677 	# GL/glext.h:440
GL_SAMPLER_2D = 35678 	# GL/glext.h:441
GL_SAMPLER_3D = 35679 	# GL/glext.h:442
GL_SAMPLER_CUBE = 35680 	# GL/glext.h:443
GL_SAMPLER_1D_SHADOW = 35681 	# GL/glext.h:444
GL_SAMPLER_2D_SHADOW = 35682 	# GL/glext.h:445
GL_DELETE_STATUS = 35712 	# GL/glext.h:446
GL_COMPILE_STATUS = 35713 	# GL/glext.h:447
GL_LINK_STATUS = 35714 	# GL/glext.h:448
GL_VALIDATE_STATUS = 35715 	# GL/glext.h:449
GL_INFO_LOG_LENGTH = 35716 	# GL/glext.h:450
GL_ATTACHED_SHADERS = 35717 	# GL/glext.h:451
GL_ACTIVE_UNIFORMS = 35718 	# GL/glext.h:452
GL_ACTIVE_UNIFORM_MAX_LENGTH = 35719 	# GL/glext.h:453
GL_SHADER_SOURCE_LENGTH = 35720 	# GL/glext.h:454
GL_ACTIVE_ATTRIBUTES = 35721 	# GL/glext.h:455
GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = 35722 	# GL/glext.h:456
GL_FRAGMENT_SHADER_DERIVATIVE_HINT = 35723 	# GL/glext.h:457
GL_SHADING_LANGUAGE_VERSION = 35724 	# GL/glext.h:458
GL_CURRENT_PROGRAM = 35725 	# GL/glext.h:459
GL_POINT_SPRITE_COORD_ORIGIN = 36000 	# GL/glext.h:460
GL_LOWER_LEFT = 36001 	# GL/glext.h:461
GL_UPPER_LEFT = 36002 	# GL/glext.h:462
GL_STENCIL_BACK_REF = 36003 	# GL/glext.h:463
GL_STENCIL_BACK_VALUE_MASK = 36004 	# GL/glext.h:464
GL_STENCIL_BACK_WRITEMASK = 36005 	# GL/glext.h:465
# VERSION_2_0_DEPRECATED (GL/glext.h:468)
GL_VERTEX_PROGRAM_TWO_SIDE = 34371 	# GL/glext.h:469
GL_POINT_SPRITE = 34913 	# GL/glext.h:470
GL_COORD_REPLACE = 34914 	# GL/glext.h:471
GL_MAX_TEXTURE_COORDS = 34929 	# GL/glext.h:472
# VERSION_2_1 (GL/glext.h:475)
GL_PIXEL_PACK_BUFFER = 35051 	# GL/glext.h:476
GL_PIXEL_UNPACK_BUFFER = 35052 	# GL/glext.h:477
GL_PIXEL_PACK_BUFFER_BINDING = 35053 	# GL/glext.h:478
GL_PIXEL_UNPACK_BUFFER_BINDING = 35055 	# GL/glext.h:479
GL_FLOAT_MAT2x3 = 35685 	# GL/glext.h:480
GL_FLOAT_MAT2x4 = 35686 	# GL/glext.h:481
GL_FLOAT_MAT3x2 = 35687 	# GL/glext.h:482
GL_FLOAT_MAT3x4 = 35688 	# GL/glext.h:483
GL_FLOAT_MAT4x2 = 35689 	# GL/glext.h:484
GL_FLOAT_MAT4x3 = 35690 	# GL/glext.h:485
GL_SRGB = 35904 	# GL/glext.h:486
GL_SRGB8 = 35905 	# GL/glext.h:487
GL_SRGB_ALPHA = 35906 	# GL/glext.h:488
GL_SRGB8_ALPHA8 = 35907 	# GL/glext.h:489
GL_COMPRESSED_SRGB = 35912 	# GL/glext.h:490
GL_COMPRESSED_SRGB_ALPHA = 35913 	# GL/glext.h:491
# VERSION_2_1_DEPRECATED (GL/glext.h:494)
GL_CURRENT_RASTER_SECONDARY_COLOR = 33887 	# GL/glext.h:495
GL_SLUMINANCE_ALPHA = 35908 	# GL/glext.h:496
GL_SLUMINANCE8_ALPHA8 = 35909 	# GL/glext.h:497
GL_SLUMINANCE = 35910 	# GL/glext.h:498
GL_SLUMINANCE8 = 35911 	# GL/glext.h:499
GL_COMPRESSED_SLUMINANCE = 35914 	# GL/glext.h:500
GL_COMPRESSED_SLUMINANCE_ALPHA = 35915 	# GL/glext.h:501
# VERSION_3_0 (GL/glext.h:504)
GL_COMPARE_REF_TO_TEXTURE = 34894 	# GL/glext.h:505
GL_CLIP_DISTANCE0 = 12288 	# GL/glext.h:506
GL_CLIP_DISTANCE1 = 12289 	# GL/glext.h:507
GL_CLIP_DISTANCE2 = 12290 	# GL/glext.h:508
GL_CLIP_DISTANCE3 = 12291 	# GL/glext.h:509
GL_CLIP_DISTANCE4 = 12292 	# GL/glext.h:510
GL_CLIP_DISTANCE5 = 12293 	# GL/glext.h:511
GL_CLIP_DISTANCE6 = 12294 	# GL/glext.h:512
GL_CLIP_DISTANCE7 = 12295 	# GL/glext.h:513
GL_MAX_CLIP_DISTANCES = 3378 	# GL/glext.h:514
GL_MAJOR_VERSION = 33307 	# GL/glext.h:515
GL_MINOR_VERSION = 33308 	# GL/glext.h:516
GL_NUM_EXTENSIONS = 33309 	# GL/glext.h:517
GL_CONTEXT_FLAGS = 33310 	# GL/glext.h:518
GL_COMPRESSED_RED = 33317 	# GL/glext.h:519
GL_COMPRESSED_RG = 33318 	# GL/glext.h:520
GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT = 1 	# GL/glext.h:521
GL_RGBA32F = 34836 	# GL/glext.h:522
GL_RGB32F = 34837 	# GL/glext.h:523
GL_RGBA16F = 34842 	# GL/glext.h:524
GL_RGB16F = 34843 	# GL/glext.h:525
GL_VERTEX_ATTRIB_ARRAY_INTEGER = 35069 	# GL/glext.h:526
GL_MAX_ARRAY_TEXTURE_LAYERS = 35071 	# GL/glext.h:527
GL_MIN_PROGRAM_TEXEL_OFFSET = 35076 	# GL/glext.h:528
GL_MAX_PROGRAM_TEXEL_OFFSET = 35077 	# GL/glext.h:529
GL_CLAMP_READ_COLOR = 35100 	# GL/glext.h:530
GL_FIXED_ONLY = 35101 	# GL/glext.h:531
GL_MAX_VARYING_COMPONENTS = 35659 	# GL/glext.h:532
GL_TEXTURE_1D_ARRAY = 35864 	# GL/glext.h:533
GL_PROXY_TEXTURE_1D_ARRAY = 35865 	# GL/glext.h:534
GL_TEXTURE_2D_ARRAY = 35866 	# GL/glext.h:535
GL_PROXY_TEXTURE_2D_ARRAY = 35867 	# GL/glext.h:536
GL_TEXTURE_BINDING_1D_ARRAY = 35868 	# GL/glext.h:537
GL_TEXTURE_BINDING_2D_ARRAY = 35869 	# GL/glext.h:538
GL_R11F_G11F_B10F = 35898 	# GL/glext.h:539
GL_UNSIGNED_INT_10F_11F_11F_REV = 35899 	# GL/glext.h:540
GL_RGB9_E5 = 35901 	# GL/glext.h:541
GL_UNSIGNED_INT_5_9_9_9_REV = 35902 	# GL/glext.h:542
GL_TEXTURE_SHARED_SIZE = 35903 	# GL/glext.h:543
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH = 35958 	# GL/glext.h:544
GL_TRANSFORM_FEEDBACK_BUFFER_MODE = 35967 	# GL/glext.h:545
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS = 35968 	# GL/glext.h:546
GL_TRANSFORM_FEEDBACK_VARYINGS = 35971 	# GL/glext.h:547
GL_TRANSFORM_FEEDBACK_BUFFER_START = 35972 	# GL/glext.h:548
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE = 35973 	# GL/glext.h:549
GL_PRIMITIVES_GENERATED = 35975 	# GL/glext.h:550
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN = 35976 	# GL/glext.h:551
GL_RASTERIZER_DISCARD = 35977 	# GL/glext.h:552
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS = 35978 	# GL/glext.h:553
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS = 35979 	# GL/glext.h:554
GL_INTERLEAVED_ATTRIBS = 35980 	# GL/glext.h:555
GL_SEPARATE_ATTRIBS = 35981 	# GL/glext.h:556
GL_TRANSFORM_FEEDBACK_BUFFER = 35982 	# GL/glext.h:557
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING = 35983 	# GL/glext.h:558
GL_RGBA32UI = 36208 	# GL/glext.h:559
GL_RGB32UI = 36209 	# GL/glext.h:560
GL_RGBA16UI = 36214 	# GL/glext.h:561
GL_RGB16UI = 36215 	# GL/glext.h:562
GL_RGBA8UI = 36220 	# GL/glext.h:563
GL_RGB8UI = 36221 	# GL/glext.h:564
GL_RGBA32I = 36226 	# GL/glext.h:565
GL_RGB32I = 36227 	# GL/glext.h:566
GL_RGBA16I = 36232 	# GL/glext.h:567
GL_RGB16I = 36233 	# GL/glext.h:568
GL_RGBA8I = 36238 	# GL/glext.h:569
GL_RGB8I = 36239 	# GL/glext.h:570
GL_RED_INTEGER = 36244 	# GL/glext.h:571
GL_GREEN_INTEGER = 36245 	# GL/glext.h:572
GL_BLUE_INTEGER = 36246 	# GL/glext.h:573
GL_RGB_INTEGER = 36248 	# GL/glext.h:574
GL_RGBA_INTEGER = 36249 	# GL/glext.h:575
GL_BGR_INTEGER = 36250 	# GL/glext.h:576
GL_BGRA_INTEGER = 36251 	# GL/glext.h:577
GL_SAMPLER_1D_ARRAY = 36288 	# GL/glext.h:578
GL_SAMPLER_2D_ARRAY = 36289 	# GL/glext.h:579
GL_SAMPLER_1D_ARRAY_SHADOW = 36291 	# GL/glext.h:580
GL_SAMPLER_2D_ARRAY_SHADOW = 36292 	# GL/glext.h:581
GL_SAMPLER_CUBE_SHADOW = 36293 	# GL/glext.h:582
GL_UNSIGNED_INT_VEC2 = 36294 	# GL/glext.h:583
GL_UNSIGNED_INT_VEC3 = 36295 	# GL/glext.h:584
GL_UNSIGNED_INT_VEC4 = 36296 	# GL/glext.h:585
GL_INT_SAMPLER_1D = 36297 	# GL/glext.h:586
GL_INT_SAMPLER_2D = 36298 	# GL/glext.h:587
GL_INT_SAMPLER_3D = 36299 	# GL/glext.h:588
GL_INT_SAMPLER_CUBE = 36300 	# GL/glext.h:589
GL_INT_SAMPLER_1D_ARRAY = 36302 	# GL/glext.h:590
GL_INT_SAMPLER_2D_ARRAY = 36303 	# GL/glext.h:591
GL_UNSIGNED_INT_SAMPLER_1D = 36305 	# GL/glext.h:592
GL_UNSIGNED_INT_SAMPLER_2D = 36306 	# GL/glext.h:593
GL_UNSIGNED_INT_SAMPLER_3D = 36307 	# GL/glext.h:594
GL_UNSIGNED_INT_SAMPLER_CUBE = 36308 	# GL/glext.h:595
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY = 36310 	# GL/glext.h:596
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY = 36311 	# GL/glext.h:597
GL_QUERY_WAIT = 36371 	# GL/glext.h:598
GL_QUERY_NO_WAIT = 36372 	# GL/glext.h:599
GL_QUERY_BY_REGION_WAIT = 36373 	# GL/glext.h:600
GL_QUERY_BY_REGION_NO_WAIT = 36374 	# GL/glext.h:601
GL_BUFFER_ACCESS_FLAGS = 37151 	# GL/glext.h:602
GL_BUFFER_MAP_LENGTH = 37152 	# GL/glext.h:603
GL_BUFFER_MAP_OFFSET = 37153 	# GL/glext.h:604
# VERSION_3_0_DEPRECATED (GL/glext.h:731)
GL_CLAMP_VERTEX_COLOR = 35098 	# GL/glext.h:732
GL_CLAMP_FRAGMENT_COLOR = 35099 	# GL/glext.h:733
GL_ALPHA_INTEGER = 36247 	# GL/glext.h:734
# VERSION_3_1 (GL/glext.h:740)
GL_SAMPLER_2D_RECT = 35683 	# GL/glext.h:741
GL_SAMPLER_2D_RECT_SHADOW = 35684 	# GL/glext.h:742
GL_SAMPLER_BUFFER = 36290 	# GL/glext.h:743
GL_INT_SAMPLER_2D_RECT = 36301 	# GL/glext.h:744
GL_INT_SAMPLER_BUFFER = 36304 	# GL/glext.h:745
GL_UNSIGNED_INT_SAMPLER_2D_RECT = 36309 	# GL/glext.h:746
GL_UNSIGNED_INT_SAMPLER_BUFFER = 36312 	# GL/glext.h:747
GL_TEXTURE_BUFFER = 35882 	# GL/glext.h:748
GL_MAX_TEXTURE_BUFFER_SIZE = 35883 	# GL/glext.h:749
GL_TEXTURE_BINDING_BUFFER = 35884 	# GL/glext.h:750
GL_TEXTURE_BUFFER_DATA_STORE_BINDING = 35885 	# GL/glext.h:751
GL_TEXTURE_BUFFER_FORMAT = 35886 	# GL/glext.h:752
GL_TEXTURE_RECTANGLE = 34037 	# GL/glext.h:753
GL_TEXTURE_BINDING_RECTANGLE = 34038 	# GL/glext.h:754
GL_PROXY_TEXTURE_RECTANGLE = 34039 	# GL/glext.h:755
GL_MAX_RECTANGLE_TEXTURE_SIZE = 34040 	# GL/glext.h:756
GL_RED_SNORM = 36752 	# GL/glext.h:757
GL_RG_SNORM = 36753 	# GL/glext.h:758
GL_RGB_SNORM = 36754 	# GL/glext.h:759
GL_RGBA_SNORM = 36755 	# GL/glext.h:760
GL_R8_SNORM = 36756 	# GL/glext.h:761
GL_RG8_SNORM = 36757 	# GL/glext.h:762
GL_RGB8_SNORM = 36758 	# GL/glext.h:763
GL_RGBA8_SNORM = 36759 	# GL/glext.h:764
GL_R16_SNORM = 36760 	# GL/glext.h:765
GL_RG16_SNORM = 36761 	# GL/glext.h:766
GL_RGB16_SNORM = 36762 	# GL/glext.h:767
GL_RGBA16_SNORM = 36763 	# GL/glext.h:768
GL_SIGNED_NORMALIZED = 36764 	# GL/glext.h:769
GL_PRIMITIVE_RESTART = 36765 	# GL/glext.h:770
GL_PRIMITIVE_RESTART_INDEX = 36766 	# GL/glext.h:771
# VERSION_3_2 (GL/glext.h:809)
GL_CONTEXT_CORE_PROFILE_BIT = 1 	# GL/glext.h:810
GL_CONTEXT_COMPATIBILITY_PROFILE_BIT = 2 	# GL/glext.h:811
GL_LINES_ADJACENCY = 10 	# GL/glext.h:812
GL_LINE_STRIP_ADJACENCY = 11 	# GL/glext.h:813
GL_TRIANGLES_ADJACENCY = 12 	# GL/glext.h:814
GL_TRIANGLE_STRIP_ADJACENCY = 13 	# GL/glext.h:815
GL_PROGRAM_POINT_SIZE = 34370 	# GL/glext.h:816
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS = 35881 	# GL/glext.h:817
GL_FRAMEBUFFER_ATTACHMENT_LAYERED = 36263 	# GL/glext.h:818
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS = 36264 	# GL/glext.h:819
GL_GEOMETRY_SHADER = 36313 	# GL/glext.h:820
GL_GEOMETRY_VERTICES_OUT = 35094 	# GL/glext.h:821
GL_GEOMETRY_INPUT_TYPE = 35095 	# GL/glext.h:822
GL_GEOMETRY_OUTPUT_TYPE = 35096 	# GL/glext.h:823
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS = 36319 	# GL/glext.h:824
GL_MAX_GEOMETRY_OUTPUT_VERTICES = 36320 	# GL/glext.h:825
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS = 36321 	# GL/glext.h:826
GL_MAX_VERTEX_OUTPUT_COMPONENTS = 37154 	# GL/glext.h:827
GL_MAX_GEOMETRY_INPUT_COMPONENTS = 37155 	# GL/glext.h:828
GL_MAX_GEOMETRY_OUTPUT_COMPONENTS = 37156 	# GL/glext.h:829
GL_MAX_FRAGMENT_INPUT_COMPONENTS = 37157 	# GL/glext.h:830
GL_CONTEXT_PROFILE_MASK = 37158 	# GL/glext.h:831
# VERSION_3_3 (GL/glext.h:887)
GL_VERTEX_ATTRIB_ARRAY_DIVISOR = 35070 	# GL/glext.h:888
# VERSION_4_0 (GL/glext.h:915)
GL_SAMPLE_SHADING = 35894 	# GL/glext.h:916
GL_MIN_SAMPLE_SHADING_VALUE = 35895 	# GL/glext.h:917
GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET = 36446 	# GL/glext.h:918
GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET = 36447 	# GL/glext.h:919
GL_TEXTURE_CUBE_MAP_ARRAY = 36873 	# GL/glext.h:920
GL_TEXTURE_BINDING_CUBE_MAP_ARRAY = 36874 	# GL/glext.h:921
GL_PROXY_TEXTURE_CUBE_MAP_ARRAY = 36875 	# GL/glext.h:922
GL_SAMPLER_CUBE_MAP_ARRAY = 36876 	# GL/glext.h:923
GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW = 36877 	# GL/glext.h:924
GL_INT_SAMPLER_CUBE_MAP_ARRAY = 36878 	# GL/glext.h:925
GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY = 36879 	# GL/glext.h:926
# VERSION_4_1 (GL/glext.h:1006)
# VERSION_4_2 (GL/glext.h:1049)
# ARB_multitexture (GL/glext.h:1167)
# ARB_transpose_matrix (GL/glext.h:1205)
GL_TRANSPOSE_MODELVIEW_MATRIX_ARB = 34019 	# GL/glext.h:1206
GL_TRANSPOSE_PROJECTION_MATRIX_ARB = 34020 	# GL/glext.h:1207
GL_TRANSPOSE_TEXTURE_MATRIX_ARB = 34021 	# GL/glext.h:1208
GL_TRANSPOSE_COLOR_MATRIX_ARB = 34022 	# GL/glext.h:1209
# ARB_multisample (GL/glext.h:1212)
GL_MULTISAMPLE_ARB = 32925 	# GL/glext.h:1213
GL_SAMPLE_ALPHA_TO_COVERAGE_ARB = 32926 	# GL/glext.h:1214
GL_SAMPLE_ALPHA_TO_ONE_ARB = 32927 	# GL/glext.h:1215
GL_SAMPLE_COVERAGE_ARB = 32928 	# GL/glext.h:1216
GL_SAMPLE_BUFFERS_ARB = 32936 	# GL/glext.h:1217
GL_SAMPLES_ARB = 32937 	# GL/glext.h:1218
GL_SAMPLE_COVERAGE_VALUE_ARB = 32938 	# GL/glext.h:1219
GL_SAMPLE_COVERAGE_INVERT_ARB = 32939 	# GL/glext.h:1220
GL_MULTISAMPLE_BIT_ARB = 536870912 	# GL/glext.h:1221
# ARB_texture_env_add (GL/glext.h:1224)
# ARB_texture_cube_map (GL/glext.h:1227)
GL_NORMAL_MAP_ARB = 34065 	# GL/glext.h:1228
GL_REFLECTION_MAP_ARB = 34066 	# GL/glext.h:1229
GL_TEXTURE_CUBE_MAP_ARB = 34067 	# GL/glext.h:1230
GL_TEXTURE_BINDING_CUBE_MAP_ARB = 34068 	# GL/glext.h:1231
GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB = 34069 	# GL/glext.h:1232
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB = 34070 	# GL/glext.h:1233
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB = 34071 	# GL/glext.h:1234
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB = 34072 	# GL/glext.h:1235
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB = 34073 	# GL/glext.h:1236
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB = 34074 	# GL/glext.h:1237
GL_PROXY_TEXTURE_CUBE_MAP_ARB = 34075 	# GL/glext.h:1238
GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB = 34076 	# GL/glext.h:1239
# ARB_texture_compression (GL/glext.h:1242)
GL_COMPRESSED_ALPHA_ARB = 34025 	# GL/glext.h:1243
GL_COMPRESSED_LUMINANCE_ARB = 34026 	# GL/glext.h:1244
GL_COMPRESSED_LUMINANCE_ALPHA_ARB = 34027 	# GL/glext.h:1245
GL_COMPRESSED_INTENSITY_ARB = 34028 	# GL/glext.h:1246
GL_COMPRESSED_RGB_ARB = 34029 	# GL/glext.h:1247
GL_COMPRESSED_RGBA_ARB = 34030 	# GL/glext.h:1248
GL_TEXTURE_COMPRESSION_HINT_ARB = 34031 	# GL/glext.h:1249
GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB = 34464 	# GL/glext.h:1250
GL_TEXTURE_COMPRESSED_ARB = 34465 	# GL/glext.h:1251
GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB = 34466 	# GL/glext.h:1252
GL_COMPRESSED_TEXTURE_FORMATS_ARB = 34467 	# GL/glext.h:1253
# ARB_texture_border_clamp (GL/glext.h:1256)
GL_CLAMP_TO_BORDER_ARB = 33069 	# GL/glext.h:1257
# ARB_point_parameters (GL/glext.h:1260)
GL_POINT_SIZE_MIN_ARB = 33062 	# GL/glext.h:1261
GL_POINT_SIZE_MAX_ARB = 33063 	# GL/glext.h:1262
GL_POINT_FADE_THRESHOLD_SIZE_ARB = 33064 	# GL/glext.h:1263
GL_POINT_DISTANCE_ATTENUATION_ARB = 33065 	# GL/glext.h:1264
# ARB_vertex_blend (GL/glext.h:1267)
GL_MAX_VERTEX_UNITS_ARB = 34468 	# GL/glext.h:1268
GL_ACTIVE_VERTEX_UNITS_ARB = 34469 	# GL/glext.h:1269
GL_WEIGHT_SUM_UNITY_ARB = 34470 	# GL/glext.h:1270
GL_VERTEX_BLEND_ARB = 34471 	# GL/glext.h:1271
GL_CURRENT_WEIGHT_ARB = 34472 	# GL/glext.h:1272
GL_WEIGHT_ARRAY_TYPE_ARB = 34473 	# GL/glext.h:1273
GL_WEIGHT_ARRAY_STRIDE_ARB = 34474 	# GL/glext.h:1274
GL_WEIGHT_ARRAY_SIZE_ARB = 34475 	# GL/glext.h:1275
GL_WEIGHT_ARRAY_POINTER_ARB = 34476 	# GL/glext.h:1276
GL_WEIGHT_ARRAY_ARB = 34477 	# GL/glext.h:1277
GL_MODELVIEW0_ARB = 5888 	# GL/glext.h:1278
GL_MODELVIEW1_ARB = 34058 	# GL/glext.h:1279
GL_MODELVIEW2_ARB = 34594 	# GL/glext.h:1280
GL_MODELVIEW3_ARB = 34595 	# GL/glext.h:1281
GL_MODELVIEW4_ARB = 34596 	# GL/glext.h:1282
GL_MODELVIEW5_ARB = 34597 	# GL/glext.h:1283
GL_MODELVIEW6_ARB = 34598 	# GL/glext.h:1284
GL_MODELVIEW7_ARB = 34599 	# GL/glext.h:1285
GL_MODELVIEW8_ARB = 34600 	# GL/glext.h:1286
GL_MODELVIEW9_ARB = 34601 	# GL/glext.h:1287
GL_MODELVIEW10_ARB = 34602 	# GL/glext.h:1288
GL_MODELVIEW11_ARB = 34603 	# GL/glext.h:1289
GL_MODELVIEW12_ARB = 34604 	# GL/glext.h:1290
GL_MODELVIEW13_ARB = 34605 	# GL/glext.h:1291
GL_MODELVIEW14_ARB = 34606 	# GL/glext.h:1292
GL_MODELVIEW15_ARB = 34607 	# GL/glext.h:1293
GL_MODELVIEW16_ARB = 34608 	# GL/glext.h:1294
GL_MODELVIEW17_ARB = 34609 	# GL/glext.h:1295
GL_MODELVIEW18_ARB = 34610 	# GL/glext.h:1296
GL_MODELVIEW19_ARB = 34611 	# GL/glext.h:1297
GL_MODELVIEW20_ARB = 34612 	# GL/glext.h:1298
GL_MODELVIEW21_ARB = 34613 	# GL/glext.h:1299
GL_MODELVIEW22_ARB = 34614 	# GL/glext.h:1300
GL_MODELVIEW23_ARB = 34615 	# GL/glext.h:1301
GL_MODELVIEW24_ARB = 34616 	# GL/glext.h:1302
GL_MODELVIEW25_ARB = 34617 	# GL/glext.h:1303
GL_MODELVIEW26_ARB = 34618 	# GL/glext.h:1304
GL_MODELVIEW27_ARB = 34619 	# GL/glext.h:1305
GL_MODELVIEW28_ARB = 34620 	# GL/glext.h:1306
GL_MODELVIEW29_ARB = 34621 	# GL/glext.h:1307
GL_MODELVIEW30_ARB = 34622 	# GL/glext.h:1308
GL_MODELVIEW31_ARB = 34623 	# GL/glext.h:1309
# ARB_matrix_palette (GL/glext.h:1312)
GL_MATRIX_PALETTE_ARB = 34880 	# GL/glext.h:1313
GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB = 34881 	# GL/glext.h:1314
GL_MAX_PALETTE_MATRICES_ARB = 34882 	# GL/glext.h:1315
GL_CURRENT_PALETTE_MATRIX_ARB = 34883 	# GL/glext.h:1316
GL_MATRIX_INDEX_ARRAY_ARB = 34884 	# GL/glext.h:1317
GL_CURRENT_MATRIX_INDEX_ARB = 34885 	# GL/glext.h:1318
GL_MATRIX_INDEX_ARRAY_SIZE_ARB = 34886 	# GL/glext.h:1319
GL_MATRIX_INDEX_ARRAY_TYPE_ARB = 34887 	# GL/glext.h:1320
GL_MATRIX_INDEX_ARRAY_STRIDE_ARB = 34888 	# GL/glext.h:1321
GL_MATRIX_INDEX_ARRAY_POINTER_ARB = 34889 	# GL/glext.h:1322
# ARB_texture_env_combine (GL/glext.h:1325)
GL_COMBINE_ARB = 34160 	# GL/glext.h:1326
GL_COMBINE_RGB_ARB = 34161 	# GL/glext.h:1327
GL_COMBINE_ALPHA_ARB = 34162 	# GL/glext.h:1328
GL_SOURCE0_RGB_ARB = 34176 	# GL/glext.h:1329
GL_SOURCE1_RGB_ARB = 34177 	# GL/glext.h:1330
GL_SOURCE2_RGB_ARB = 34178 	# GL/glext.h:1331
GL_SOURCE0_ALPHA_ARB = 34184 	# GL/glext.h:1332
GL_SOURCE1_ALPHA_ARB = 34185 	# GL/glext.h:1333
GL_SOURCE2_ALPHA_ARB = 34186 	# GL/glext.h:1334
GL_OPERAND0_RGB_ARB = 34192 	# GL/glext.h:1335
GL_OPERAND1_RGB_ARB = 34193 	# GL/glext.h:1336
GL_OPERAND2_RGB_ARB = 34194 	# GL/glext.h:1337
GL_OPERAND0_ALPHA_ARB = 34200 	# GL/glext.h:1338
GL_OPERAND1_ALPHA_ARB = 34201 	# GL/glext.h:1339
GL_OPERAND2_ALPHA_ARB = 34202 	# GL/glext.h:1340
GL_RGB_SCALE_ARB = 34163 	# GL/glext.h:1341
GL_ADD_SIGNED_ARB = 34164 	# GL/glext.h:1342
GL_INTERPOLATE_ARB = 34165 	# GL/glext.h:1343
GL_SUBTRACT_ARB = 34023 	# GL/glext.h:1344
GL_CONSTANT_ARB = 34166 	# GL/glext.h:1345
GL_PRIMARY_COLOR_ARB = 34167 	# GL/glext.h:1346
GL_PREVIOUS_ARB = 34168 	# GL/glext.h:1347
# ARB_texture_env_crossbar (GL/glext.h:1350)
# ARB_texture_env_dot3 (GL/glext.h:1353)
GL_DOT3_RGB_ARB = 34478 	# GL/glext.h:1354
GL_DOT3_RGBA_ARB = 34479 	# GL/glext.h:1355
# ARB_texture_mirrored_repeat (GL/glext.h:1358)
GL_MIRRORED_REPEAT_ARB = 33648 	# GL/glext.h:1359
# ARB_depth_texture (GL/glext.h:1362)
GL_DEPTH_COMPONENT16_ARB = 33189 	# GL/glext.h:1363
GL_DEPTH_COMPONENT24_ARB = 33190 	# GL/glext.h:1364
GL_DEPTH_COMPONENT32_ARB = 33191 	# GL/glext.h:1365
GL_TEXTURE_DEPTH_SIZE_ARB = 34890 	# GL/glext.h:1366
GL_DEPTH_TEXTURE_MODE_ARB = 34891 	# GL/glext.h:1367
# ARB_shadow (GL/glext.h:1370)
GL_TEXTURE_COMPARE_MODE_ARB = 34892 	# GL/glext.h:1371
GL_TEXTURE_COMPARE_FUNC_ARB = 34893 	# GL/glext.h:1372
GL_COMPARE_R_TO_TEXTURE_ARB = 34894 	# GL/glext.h:1373
# ARB_shadow_ambient (GL/glext.h:1376)
GL_TEXTURE_COMPARE_FAIL_VALUE_ARB = 32959 	# GL/glext.h:1377
# ARB_window_pos (GL/glext.h:1380)
# ARB_vertex_program (GL/glext.h:1383)
GL_COLOR_SUM_ARB = 33880 	# GL/glext.h:1384
GL_VERTEX_PROGRAM_ARB = 34336 	# GL/glext.h:1385
GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB = 34338 	# GL/glext.h:1386
GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB = 34339 	# GL/glext.h:1387
GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB = 34340 	# GL/glext.h:1388
GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB = 34341 	# GL/glext.h:1389
GL_CURRENT_VERTEX_ATTRIB_ARB = 34342 	# GL/glext.h:1390
GL_PROGRAM_LENGTH_ARB = 34343 	# GL/glext.h:1391
GL_PROGRAM_STRING_ARB = 34344 	# GL/glext.h:1392
GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB = 34350 	# GL/glext.h:1393
GL_MAX_PROGRAM_MATRICES_ARB = 34351 	# GL/glext.h:1394
GL_CURRENT_MATRIX_STACK_DEPTH_ARB = 34368 	# GL/glext.h:1395
GL_CURRENT_MATRIX_ARB = 34369 	# GL/glext.h:1396
GL_VERTEX_PROGRAM_POINT_SIZE_ARB = 34370 	# GL/glext.h:1397
GL_VERTEX_PROGRAM_TWO_SIDE_ARB = 34371 	# GL/glext.h:1398
GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB = 34373 	# GL/glext.h:1399
GL_PROGRAM_ERROR_POSITION_ARB = 34379 	# GL/glext.h:1400
GL_PROGRAM_BINDING_ARB = 34423 	# GL/glext.h:1401
GL_MAX_VERTEX_ATTRIBS_ARB = 34921 	# GL/glext.h:1402
GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB = 34922 	# GL/glext.h:1403
GL_PROGRAM_ERROR_STRING_ARB = 34932 	# GL/glext.h:1404
GL_PROGRAM_FORMAT_ASCII_ARB = 34933 	# GL/glext.h:1405
GL_PROGRAM_FORMAT_ARB = 34934 	# GL/glext.h:1406
GL_PROGRAM_INSTRUCTIONS_ARB = 34976 	# GL/glext.h:1407
GL_MAX_PROGRAM_INSTRUCTIONS_ARB = 34977 	# GL/glext.h:1408
GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34978 	# GL/glext.h:1409
GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB = 34979 	# GL/glext.h:1410
GL_PROGRAM_TEMPORARIES_ARB = 34980 	# GL/glext.h:1411
GL_MAX_PROGRAM_TEMPORARIES_ARB = 34981 	# GL/glext.h:1412
GL_PROGRAM_NATIVE_TEMPORARIES_ARB = 34982 	# GL/glext.h:1413
GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB = 34983 	# GL/glext.h:1414
GL_PROGRAM_PARAMETERS_ARB = 34984 	# GL/glext.h:1415
GL_MAX_PROGRAM_PARAMETERS_ARB = 34985 	# GL/glext.h:1416
GL_PROGRAM_NATIVE_PARAMETERS_ARB = 34986 	# GL/glext.h:1417
GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB = 34987 	# GL/glext.h:1418
GL_PROGRAM_ATTRIBS_ARB = 34988 	# GL/glext.h:1419
GL_MAX_PROGRAM_ATTRIBS_ARB = 34989 	# GL/glext.h:1420
GL_PROGRAM_NATIVE_ATTRIBS_ARB = 34990 	# GL/glext.h:1421
GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB = 34991 	# GL/glext.h:1422
GL_PROGRAM_ADDRESS_REGISTERS_ARB = 34992 	# GL/glext.h:1423
GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB = 34993 	# GL/glext.h:1424
GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34994 	# GL/glext.h:1425
GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB = 34995 	# GL/glext.h:1426
GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB = 34996 	# GL/glext.h:1427
GL_MAX_PROGRAM_ENV_PARAMETERS_ARB = 34997 	# GL/glext.h:1428
GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB = 34998 	# GL/glext.h:1429
GL_TRANSPOSE_CURRENT_MATRIX_ARB = 34999 	# GL/glext.h:1430
GL_MATRIX0_ARB = 35008 	# GL/glext.h:1431
GL_MATRIX1_ARB = 35009 	# GL/glext.h:1432
GL_MATRIX2_ARB = 35010 	# GL/glext.h:1433
GL_MATRIX3_ARB = 35011 	# GL/glext.h:1434
GL_MATRIX4_ARB = 35012 	# GL/glext.h:1435
GL_MATRIX5_ARB = 35013 	# GL/glext.h:1436
GL_MATRIX6_ARB = 35014 	# GL/glext.h:1437
GL_MATRIX7_ARB = 35015 	# GL/glext.h:1438
GL_MATRIX8_ARB = 35016 	# GL/glext.h:1439
GL_MATRIX9_ARB = 35017 	# GL/glext.h:1440
GL_MATRIX10_ARB = 35018 	# GL/glext.h:1441
GL_MATRIX11_ARB = 35019 	# GL/glext.h:1442
GL_MATRIX12_ARB = 35020 	# GL/glext.h:1443
GL_MATRIX13_ARB = 35021 	# GL/glext.h:1444
GL_MATRIX14_ARB = 35022 	# GL/glext.h:1445
GL_MATRIX15_ARB = 35023 	# GL/glext.h:1446
GL_MATRIX16_ARB = 35024 	# GL/glext.h:1447
GL_MATRIX17_ARB = 35025 	# GL/glext.h:1448
GL_MATRIX18_ARB = 35026 	# GL/glext.h:1449
GL_MATRIX19_ARB = 35027 	# GL/glext.h:1450
GL_MATRIX20_ARB = 35028 	# GL/glext.h:1451
GL_MATRIX21_ARB = 35029 	# GL/glext.h:1452
GL_MATRIX22_ARB = 35030 	# GL/glext.h:1453
GL_MATRIX23_ARB = 35031 	# GL/glext.h:1454
GL_MATRIX24_ARB = 35032 	# GL/glext.h:1455
GL_MATRIX25_ARB = 35033 	# GL/glext.h:1456
GL_MATRIX26_ARB = 35034 	# GL/glext.h:1457
GL_MATRIX27_ARB = 35035 	# GL/glext.h:1458
GL_MATRIX28_ARB = 35036 	# GL/glext.h:1459
GL_MATRIX29_ARB = 35037 	# GL/glext.h:1460
GL_MATRIX30_ARB = 35038 	# GL/glext.h:1461
GL_MATRIX31_ARB = 35039 	# GL/glext.h:1462
# ARB_fragment_program (GL/glext.h:1465)
GL_FRAGMENT_PROGRAM_ARB = 34820 	# GL/glext.h:1466
GL_PROGRAM_ALU_INSTRUCTIONS_ARB = 34821 	# GL/glext.h:1467
GL_PROGRAM_TEX_INSTRUCTIONS_ARB = 34822 	# GL/glext.h:1468
GL_PROGRAM_TEX_INDIRECTIONS_ARB = 34823 	# GL/glext.h:1469
GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34824 	# GL/glext.h:1470
GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34825 	# GL/glext.h:1471
GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34826 	# GL/glext.h:1472
GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB = 34827 	# GL/glext.h:1473
GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB = 34828 	# GL/glext.h:1474
GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB = 34829 	# GL/glext.h:1475
GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB = 34830 	# GL/glext.h:1476
GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB = 34831 	# GL/glext.h:1477
GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB = 34832 	# GL/glext.h:1478
GL_MAX_TEXTURE_COORDS_ARB = 34929 	# GL/glext.h:1479
GL_MAX_TEXTURE_IMAGE_UNITS_ARB = 34930 	# GL/glext.h:1480
# ARB_vertex_buffer_object (GL/glext.h:1483)
GL_BUFFER_SIZE_ARB = 34660 	# GL/glext.h:1484
GL_BUFFER_USAGE_ARB = 34661 	# GL/glext.h:1485
GL_ARRAY_BUFFER_ARB = 34962 	# GL/glext.h:1486
GL_ELEMENT_ARRAY_BUFFER_ARB = 34963 	# GL/glext.h:1487
GL_ARRAY_BUFFER_BINDING_ARB = 34964 	# GL/glext.h:1488
GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB = 34965 	# GL/glext.h:1489
GL_VERTEX_ARRAY_BUFFER_BINDING_ARB = 34966 	# GL/glext.h:1490
GL_NORMAL_ARRAY_BUFFER_BINDING_ARB = 34967 	# GL/glext.h:1491
GL_COLOR_ARRAY_BUFFER_BINDING_ARB = 34968 	# GL/glext.h:1492
GL_INDEX_ARRAY_BUFFER_BINDING_ARB = 34969 	# GL/glext.h:1493
GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB = 34970 	# GL/glext.h:1494
GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB = 34971 	# GL/glext.h:1495
GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB = 34972 	# GL/glext.h:1496
GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB = 34973 	# GL/glext.h:1497
GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB = 34974 	# GL/glext.h:1498
GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB = 34975 	# GL/glext.h:1499
GL_READ_ONLY_ARB = 35000 	# GL/glext.h:1500
GL_WRITE_ONLY_ARB = 35001 	# GL/glext.h:1501
GL_READ_WRITE_ARB = 35002 	# GL/glext.h:1502
GL_BUFFER_ACCESS_ARB = 35003 	# GL/glext.h:1503
GL_BUFFER_MAPPED_ARB = 35004 	# GL/glext.h:1504
GL_BUFFER_MAP_POINTER_ARB = 35005 	# GL/glext.h:1505
GL_STREAM_DRAW_ARB = 35040 	# GL/glext.h:1506
GL_STREAM_READ_ARB = 35041 	# GL/glext.h:1507
GL_STREAM_COPY_ARB = 35042 	# GL/glext.h:1508
GL_STATIC_DRAW_ARB = 35044 	# GL/glext.h:1509
GL_STATIC_READ_ARB = 35045 	# GL/glext.h:1510
GL_STATIC_COPY_ARB = 35046 	# GL/glext.h:1511
GL_DYNAMIC_DRAW_ARB = 35048 	# GL/glext.h:1512
GL_DYNAMIC_READ_ARB = 35049 	# GL/glext.h:1513
GL_DYNAMIC_COPY_ARB = 35050 	# GL/glext.h:1514
# ARB_occlusion_query (GL/glext.h:1517)
GL_QUERY_COUNTER_BITS_ARB = 34916 	# GL/glext.h:1518
GL_CURRENT_QUERY_ARB = 34917 	# GL/glext.h:1519
GL_QUERY_RESULT_ARB = 34918 	# GL/glext.h:1520
GL_QUERY_RESULT_AVAILABLE_ARB = 34919 	# GL/glext.h:1521
GL_SAMPLES_PASSED_ARB = 35092 	# GL/glext.h:1522
# ARB_shader_objects (GL/glext.h:1525)
GL_PROGRAM_OBJECT_ARB = 35648 	# GL/glext.h:1526
GL_SHADER_OBJECT_ARB = 35656 	# GL/glext.h:1527
GL_OBJECT_TYPE_ARB = 35662 	# GL/glext.h:1528
GL_OBJECT_SUBTYPE_ARB = 35663 	# GL/glext.h:1529
GL_FLOAT_VEC2_ARB = 35664 	# GL/glext.h:1530
GL_FLOAT_VEC3_ARB = 35665 	# GL/glext.h:1531
GL_FLOAT_VEC4_ARB = 35666 	# GL/glext.h:1532
GL_INT_VEC2_ARB = 35667 	# GL/glext.h:1533
GL_INT_VEC3_ARB = 35668 	# GL/glext.h:1534
GL_INT_VEC4_ARB = 35669 	# GL/glext.h:1535
GL_BOOL_ARB = 35670 	# GL/glext.h:1536
GL_BOOL_VEC2_ARB = 35671 	# GL/glext.h:1537
GL_BOOL_VEC3_ARB = 35672 	# GL/glext.h:1538
GL_BOOL_VEC4_ARB = 35673 	# GL/glext.h:1539
GL_FLOAT_MAT2_ARB = 35674 	# GL/glext.h:1540
GL_FLOAT_MAT3_ARB = 35675 	# GL/glext.h:1541
GL_FLOAT_MAT4_ARB = 35676 	# GL/glext.h:1542
GL_SAMPLER_1D_ARB = 35677 	# GL/glext.h:1543
GL_SAMPLER_2D_ARB = 35678 	# GL/glext.h:1544
GL_SAMPLER_3D_ARB = 35679 	# GL/glext.h:1545
GL_SAMPLER_CUBE_ARB = 35680 	# GL/glext.h:1546
GL_SAMPLER_1D_SHADOW_ARB = 35681 	# GL/glext.h:1547
GL_SAMPLER_2D_SHADOW_ARB = 35682 	# GL/glext.h:1548
GL_SAMPLER_2D_RECT_ARB = 35683 	# GL/glext.h:1549
GL_SAMPLER_2D_RECT_SHADOW_ARB = 35684 	# GL/glext.h:1550
GL_OBJECT_DELETE_STATUS_ARB = 35712 	# GL/glext.h:1551
GL_OBJECT_COMPILE_STATUS_ARB = 35713 	# GL/glext.h:1552
GL_OBJECT_LINK_STATUS_ARB = 35714 	# GL/glext.h:1553
GL_OBJECT_VALIDATE_STATUS_ARB = 35715 	# GL/glext.h:1554
GL_OBJECT_INFO_LOG_LENGTH_ARB = 35716 	# GL/glext.h:1555
GL_OBJECT_ATTACHED_OBJECTS_ARB = 35717 	# GL/glext.h:1556
GL_OBJECT_ACTIVE_UNIFORMS_ARB = 35718 	# GL/glext.h:1557
GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB = 35719 	# GL/glext.h:1558
GL_OBJECT_SHADER_SOURCE_LENGTH_ARB = 35720 	# GL/glext.h:1559
# ARB_vertex_shader (GL/glext.h:1562)
GL_VERTEX_SHADER_ARB = 35633 	# GL/glext.h:1563
GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB = 35658 	# GL/glext.h:1564
GL_MAX_VARYING_FLOATS_ARB = 35659 	# GL/glext.h:1565
GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB = 35660 	# GL/glext.h:1566
GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB = 35661 	# GL/glext.h:1567
GL_OBJECT_ACTIVE_ATTRIBUTES_ARB = 35721 	# GL/glext.h:1568
GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB = 35722 	# GL/glext.h:1569
# ARB_fragment_shader (GL/glext.h:1572)
GL_FRAGMENT_SHADER_ARB = 35632 	# GL/glext.h:1573
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB = 35657 	# GL/glext.h:1574
GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB = 35723 	# GL/glext.h:1575
# ARB_shading_language_100 (GL/glext.h:1578)
GL_SHADING_LANGUAGE_VERSION_ARB = 35724 	# GL/glext.h:1579
# ARB_texture_non_power_of_two (GL/glext.h:1582)
# ARB_point_sprite (GL/glext.h:1585)
GL_POINT_SPRITE_ARB = 34913 	# GL/glext.h:1586
GL_COORD_REPLACE_ARB = 34914 	# GL/glext.h:1587
# ARB_fragment_program_shadow (GL/glext.h:1590)
# ARB_draw_buffers (GL/glext.h:1593)
GL_MAX_DRAW_BUFFERS_ARB = 34852 	# GL/glext.h:1594
GL_DRAW_BUFFER0_ARB = 34853 	# GL/glext.h:1595
GL_DRAW_BUFFER1_ARB = 34854 	# GL/glext.h:1596
GL_DRAW_BUFFER2_ARB = 34855 	# GL/glext.h:1597
GL_DRAW_BUFFER3_ARB = 34856 	# GL/glext.h:1598
GL_DRAW_BUFFER4_ARB = 34857 	# GL/glext.h:1599
GL_DRAW_BUFFER5_ARB = 34858 	# GL/glext.h:1600
GL_DRAW_BUFFER6_ARB = 34859 	# GL/glext.h:1601
GL_DRAW_BUFFER7_ARB = 34860 	# GL/glext.h:1602
GL_DRAW_BUFFER8_ARB = 34861 	# GL/glext.h:1603
GL_DRAW_BUFFER9_ARB = 34862 	# GL/glext.h:1604
GL_DRAW_BUFFER10_ARB = 34863 	# GL/glext.h:1605
GL_DRAW_BUFFER11_ARB = 34864 	# GL/glext.h:1606
GL_DRAW_BUFFER12_ARB = 34865 	# GL/glext.h:1607
GL_DRAW_BUFFER13_ARB = 34866 	# GL/glext.h:1608
GL_DRAW_BUFFER14_ARB = 34867 	# GL/glext.h:1609
GL_DRAW_BUFFER15_ARB = 34868 	# GL/glext.h:1610
# ARB_texture_rectangle (GL/glext.h:1613)
GL_TEXTURE_RECTANGLE_ARB = 34037 	# GL/glext.h:1614
GL_TEXTURE_BINDING_RECTANGLE_ARB = 34038 	# GL/glext.h:1615
GL_PROXY_TEXTURE_RECTANGLE_ARB = 34039 	# GL/glext.h:1616
GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB = 34040 	# GL/glext.h:1617
# ARB_color_buffer_float (GL/glext.h:1620)
GL_RGBA_FLOAT_MODE_ARB = 34848 	# GL/glext.h:1621
GL_CLAMP_VERTEX_COLOR_ARB = 35098 	# GL/glext.h:1622
GL_CLAMP_FRAGMENT_COLOR_ARB = 35099 	# GL/glext.h:1623
GL_CLAMP_READ_COLOR_ARB = 35100 	# GL/glext.h:1624
GL_FIXED_ONLY_ARB = 35101 	# GL/glext.h:1625
# ARB_half_float_pixel (GL/glext.h:1628)
GL_HALF_FLOAT_ARB = 5131 	# GL/glext.h:1629
# ARB_texture_float (GL/glext.h:1632)
GL_TEXTURE_RED_TYPE_ARB = 35856 	# GL/glext.h:1633
GL_TEXTURE_GREEN_TYPE_ARB = 35857 	# GL/glext.h:1634
GL_TEXTURE_BLUE_TYPE_ARB = 35858 	# GL/glext.h:1635
GL_TEXTURE_ALPHA_TYPE_ARB = 35859 	# GL/glext.h:1636
GL_TEXTURE_LUMINANCE_TYPE_ARB = 35860 	# GL/glext.h:1637
GL_TEXTURE_INTENSITY_TYPE_ARB = 35861 	# GL/glext.h:1638
GL_TEXTURE_DEPTH_TYPE_ARB = 35862 	# GL/glext.h:1639
GL_UNSIGNED_NORMALIZED_ARB = 35863 	# GL/glext.h:1640
GL_RGBA32F_ARB = 34836 	# GL/glext.h:1641
GL_RGB32F_ARB = 34837 	# GL/glext.h:1642
GL_ALPHA32F_ARB = 34838 	# GL/glext.h:1643
GL_INTENSITY32F_ARB = 34839 	# GL/glext.h:1644
GL_LUMINANCE32F_ARB = 34840 	# GL/glext.h:1645
GL_LUMINANCE_ALPHA32F_ARB = 34841 	# GL/glext.h:1646
GL_RGBA16F_ARB = 34842 	# GL/glext.h:1647
GL_RGB16F_ARB = 34843 	# GL/glext.h:1648
GL_ALPHA16F_ARB = 34844 	# GL/glext.h:1649
GL_INTENSITY16F_ARB = 34845 	# GL/glext.h:1650
GL_LUMINANCE16F_ARB = 34846 	# GL/glext.h:1651
GL_LUMINANCE_ALPHA16F_ARB = 34847 	# GL/glext.h:1652
# ARB_pixel_buffer_object (GL/glext.h:1655)
GL_PIXEL_PACK_BUFFER_ARB = 35051 	# GL/glext.h:1656
GL_PIXEL_UNPACK_BUFFER_ARB = 35052 	# GL/glext.h:1657
GL_PIXEL_PACK_BUFFER_BINDING_ARB = 35053 	# GL/glext.h:1658
GL_PIXEL_UNPACK_BUFFER_BINDING_ARB = 35055 	# GL/glext.h:1659
# ARB_depth_buffer_float (GL/glext.h:1662)
GL_DEPTH_COMPONENT32F = 36012 	# GL/glext.h:1663
GL_DEPTH32F_STENCIL8 = 36013 	# GL/glext.h:1664
GL_FLOAT_32_UNSIGNED_INT_24_8_REV = 36269 	# GL/glext.h:1665
# ARB_draw_instanced (GL/glext.h:1668)
# ARB_framebuffer_object (GL/glext.h:1671)
GL_INVALID_FRAMEBUFFER_OPERATION = 1286 	# GL/glext.h:1672
GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING = 33296 	# GL/glext.h:1673
GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE = 33297 	# GL/glext.h:1674
GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE = 33298 	# GL/glext.h:1675
GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE = 33299 	# GL/glext.h:1676
GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE = 33300 	# GL/glext.h:1677
GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE = 33301 	# GL/glext.h:1678
GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE = 33302 	# GL/glext.h:1679
GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE = 33303 	# GL/glext.h:1680
GL_FRAMEBUFFER_DEFAULT = 33304 	# GL/glext.h:1681
GL_FRAMEBUFFER_UNDEFINED = 33305 	# GL/glext.h:1682
GL_DEPTH_STENCIL_ATTACHMENT = 33306 	# GL/glext.h:1683
GL_MAX_RENDERBUFFER_SIZE = 34024 	# GL/glext.h:1684
GL_DEPTH_STENCIL = 34041 	# GL/glext.h:1685
GL_UNSIGNED_INT_24_8 = 34042 	# GL/glext.h:1686
GL_DEPTH24_STENCIL8 = 35056 	# GL/glext.h:1687
GL_TEXTURE_STENCIL_SIZE = 35057 	# GL/glext.h:1688
GL_TEXTURE_RED_TYPE = 35856 	# GL/glext.h:1689
GL_TEXTURE_GREEN_TYPE = 35857 	# GL/glext.h:1690
GL_TEXTURE_BLUE_TYPE = 35858 	# GL/glext.h:1691
GL_TEXTURE_ALPHA_TYPE = 35859 	# GL/glext.h:1692
GL_TEXTURE_DEPTH_TYPE = 35862 	# GL/glext.h:1693
GL_UNSIGNED_NORMALIZED = 35863 	# GL/glext.h:1694
GL_FRAMEBUFFER_BINDING = 36006 	# GL/glext.h:1695
GL_DRAW_FRAMEBUFFER_BINDING = 36006 	# GL/glext.h:1696
GL_RENDERBUFFER_BINDING = 36007 	# GL/glext.h:1697
GL_READ_FRAMEBUFFER = 36008 	# GL/glext.h:1698
GL_DRAW_FRAMEBUFFER = 36009 	# GL/glext.h:1699
GL_READ_FRAMEBUFFER_BINDING = 36010 	# GL/glext.h:1700
GL_RENDERBUFFER_SAMPLES = 36011 	# GL/glext.h:1701
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = 36048 	# GL/glext.h:1702
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = 36049 	# GL/glext.h:1703
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = 36050 	# GL/glext.h:1704
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = 36051 	# GL/glext.h:1705
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER = 36052 	# GL/glext.h:1706
GL_FRAMEBUFFER_COMPLETE = 36053 	# GL/glext.h:1707
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = 36054 	# GL/glext.h:1708
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = 36055 	# GL/glext.h:1709
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER = 36059 	# GL/glext.h:1710
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER = 36060 	# GL/glext.h:1711
GL_FRAMEBUFFER_UNSUPPORTED = 36061 	# GL/glext.h:1712
GL_MAX_COLOR_ATTACHMENTS = 36063 	# GL/glext.h:1713
GL_COLOR_ATTACHMENT0 = 36064 	# GL/glext.h:1714
GL_COLOR_ATTACHMENT1 = 36065 	# GL/glext.h:1715
GL_COLOR_ATTACHMENT2 = 36066 	# GL/glext.h:1716
GL_COLOR_ATTACHMENT3 = 36067 	# GL/glext.h:1717
GL_COLOR_ATTACHMENT4 = 36068 	# GL/glext.h:1718
GL_COLOR_ATTACHMENT5 = 36069 	# GL/glext.h:1719
GL_COLOR_ATTACHMENT6 = 36070 	# GL/glext.h:1720
GL_COLOR_ATTACHMENT7 = 36071 	# GL/glext.h:1721
GL_COLOR_ATTACHMENT8 = 36072 	# GL/glext.h:1722
GL_COLOR_ATTACHMENT9 = 36073 	# GL/glext.h:1723
GL_COLOR_ATTACHMENT10 = 36074 	# GL/glext.h:1724
GL_COLOR_ATTACHMENT11 = 36075 	# GL/glext.h:1725
GL_COLOR_ATTACHMENT12 = 36076 	# GL/glext.h:1726
GL_COLOR_ATTACHMENT13 = 36077 	# GL/glext.h:1727
GL_COLOR_ATTACHMENT14 = 36078 	# GL/glext.h:1728
GL_COLOR_ATTACHMENT15 = 36079 	# GL/glext.h:1729
GL_DEPTH_ATTACHMENT = 36096 	# GL/glext.h:1730
GL_STENCIL_ATTACHMENT = 36128 	# GL/glext.h:1731
GL_FRAMEBUFFER = 36160 	# GL/glext.h:1732
GL_RENDERBUFFER = 36161 	# GL/glext.h:1733
GL_RENDERBUFFER_WIDTH = 36162 	# GL/glext.h:1734
GL_RENDERBUFFER_HEIGHT = 36163 	# GL/glext.h:1735
GL_RENDERBUFFER_INTERNAL_FORMAT = 36164 	# GL/glext.h:1736
GL_STENCIL_INDEX1 = 36166 	# GL/glext.h:1737
GL_STENCIL_INDEX4 = 36167 	# GL/glext.h:1738
GL_STENCIL_INDEX8 = 36168 	# GL/glext.h:1739
GL_STENCIL_INDEX16 = 36169 	# GL/glext.h:1740
GL_RENDERBUFFER_RED_SIZE = 36176 	# GL/glext.h:1741
GL_RENDERBUFFER_GREEN_SIZE = 36177 	# GL/glext.h:1742
GL_RENDERBUFFER_BLUE_SIZE = 36178 	# GL/glext.h:1743
GL_RENDERBUFFER_ALPHA_SIZE = 36179 	# GL/glext.h:1744
GL_RENDERBUFFER_DEPTH_SIZE = 36180 	# GL/glext.h:1745
GL_RENDERBUFFER_STENCIL_SIZE = 36181 	# GL/glext.h:1746
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE = 36182 	# GL/glext.h:1747
GL_MAX_SAMPLES = 36183 	# GL/glext.h:1748
# ARB_framebuffer_object_DEPRECATED (GL/glext.h:1751)
GL_INDEX = 33314 	# GL/glext.h:1752
GL_TEXTURE_LUMINANCE_TYPE = 35860 	# GL/glext.h:1753
GL_TEXTURE_INTENSITY_TYPE = 35861 	# GL/glext.h:1754
# ARB_framebuffer_sRGB (GL/glext.h:1757)
GL_FRAMEBUFFER_SRGB = 36281 	# GL/glext.h:1758
# ARB_geometry_shader4 (GL/glext.h:1761)
GL_LINES_ADJACENCY_ARB = 10 	# GL/glext.h:1762
GL_LINE_STRIP_ADJACENCY_ARB = 11 	# GL/glext.h:1763
GL_TRIANGLES_ADJACENCY_ARB = 12 	# GL/glext.h:1764
GL_TRIANGLE_STRIP_ADJACENCY_ARB = 13 	# GL/glext.h:1765
GL_PROGRAM_POINT_SIZE_ARB = 34370 	# GL/glext.h:1766
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB = 35881 	# GL/glext.h:1767
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB = 36263 	# GL/glext.h:1768
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB = 36264 	# GL/glext.h:1769
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB = 36265 	# GL/glext.h:1770
GL_GEOMETRY_SHADER_ARB = 36313 	# GL/glext.h:1771
GL_GEOMETRY_VERTICES_OUT_ARB = 36314 	# GL/glext.h:1772
GL_GEOMETRY_INPUT_TYPE_ARB = 36315 	# GL/glext.h:1773
GL_GEOMETRY_OUTPUT_TYPE_ARB = 36316 	# GL/glext.h:1774
GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB = 36317 	# GL/glext.h:1775
GL_MAX_VERTEX_VARYING_COMPONENTS_ARB = 36318 	# GL/glext.h:1776
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB = 36319 	# GL/glext.h:1777
GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB = 36320 	# GL/glext.h:1778
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB = 36321 	# GL/glext.h:1779
# ARB_half_float_vertex (GL/glext.h:1784)
GL_HALF_FLOAT = 5131 	# GL/glext.h:1785
# ARB_instanced_arrays (GL/glext.h:1788)
GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ARB = 35070 	# GL/glext.h:1789
# ARB_map_buffer_range (GL/glext.h:1792)
GL_MAP_READ_BIT = 1 	# GL/glext.h:1793
GL_MAP_WRITE_BIT = 2 	# GL/glext.h:1794
GL_MAP_INVALIDATE_RANGE_BIT = 4 	# GL/glext.h:1795
GL_MAP_INVALIDATE_BUFFER_BIT = 8 	# GL/glext.h:1796
GL_MAP_FLUSH_EXPLICIT_BIT = 16 	# GL/glext.h:1797
GL_MAP_UNSYNCHRONIZED_BIT = 32 	# GL/glext.h:1798
# ARB_texture_buffer_object (GL/glext.h:1801)
GL_TEXTURE_BUFFER_ARB = 35882 	# GL/glext.h:1802
GL_MAX_TEXTURE_BUFFER_SIZE_ARB = 35883 	# GL/glext.h:1803
GL_TEXTURE_BINDING_BUFFER_ARB = 35884 	# GL/glext.h:1804
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB = 35885 	# GL/glext.h:1805
GL_TEXTURE_BUFFER_FORMAT_ARB = 35886 	# GL/glext.h:1806
# ARB_texture_compression_rgtc (GL/glext.h:1809)
GL_COMPRESSED_RED_RGTC1 = 36283 	# GL/glext.h:1810
GL_COMPRESSED_SIGNED_RED_RGTC1 = 36284 	# GL/glext.h:1811
GL_COMPRESSED_RG_RGTC2 = 36285 	# GL/glext.h:1812
GL_COMPRESSED_SIGNED_RG_RGTC2 = 36286 	# GL/glext.h:1813
# ARB_texture_rg (GL/glext.h:1816)
GL_RG = 33319 	# GL/glext.h:1817
GL_RG_INTEGER = 33320 	# GL/glext.h:1818
GL_R8 = 33321 	# GL/glext.h:1819
GL_R16 = 33322 	# GL/glext.h:1820
GL_RG8 = 33323 	# GL/glext.h:1821
GL_RG16 = 33324 	# GL/glext.h:1822
GL_R16F = 33325 	# GL/glext.h:1823
GL_R32F = 33326 	# GL/glext.h:1824
GL_RG16F = 33327 	# GL/glext.h:1825
GL_RG32F = 33328 	# GL/glext.h:1826
GL_R8I = 33329 	# GL/glext.h:1827
GL_R8UI = 33330 	# GL/glext.h:1828
GL_R16I = 33331 	# GL/glext.h:1829
GL_R16UI = 33332 	# GL/glext.h:1830
GL_R32I = 33333 	# GL/glext.h:1831
GL_R32UI = 33334 	# GL/glext.h:1832
GL_RG8I = 33335 	# GL/glext.h:1833
GL_RG8UI = 33336 	# GL/glext.h:1834
GL_RG16I = 33337 	# GL/glext.h:1835
GL_RG16UI = 33338 	# GL/glext.h:1836
GL_RG32I = 33339 	# GL/glext.h:1837
GL_RG32UI = 33340 	# GL/glext.h:1838
# ARB_vertex_array_object (GL/glext.h:1841)
GL_VERTEX_ARRAY_BINDING = 34229 	# GL/glext.h:1842
# ARB_uniform_buffer_object (GL/glext.h:1845)
GL_UNIFORM_BUFFER = 35345 	# GL/glext.h:1846
GL_UNIFORM_BUFFER_BINDING = 35368 	# GL/glext.h:1847
GL_UNIFORM_BUFFER_START = 35369 	# GL/glext.h:1848
GL_UNIFORM_BUFFER_SIZE = 35370 	# GL/glext.h:1849
GL_MAX_VERTEX_UNIFORM_BLOCKS = 35371 	# GL/glext.h:1850
GL_MAX_GEOMETRY_UNIFORM_BLOCKS = 35372 	# GL/glext.h:1851
GL_MAX_FRAGMENT_UNIFORM_BLOCKS = 35373 	# GL/glext.h:1852
GL_MAX_COMBINED_UNIFORM_BLOCKS = 35374 	# GL/glext.h:1853
GL_MAX_UNIFORM_BUFFER_BINDINGS = 35375 	# GL/glext.h:1854
GL_MAX_UNIFORM_BLOCK_SIZE = 35376 	# GL/glext.h:1855
GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS = 35377 	# GL/glext.h:1856
GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS = 35378 	# GL/glext.h:1857
GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS = 35379 	# GL/glext.h:1858
GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT = 35380 	# GL/glext.h:1859
GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH = 35381 	# GL/glext.h:1860
GL_ACTIVE_UNIFORM_BLOCKS = 35382 	# GL/glext.h:1861
GL_UNIFORM_TYPE = 35383 	# GL/glext.h:1862
GL_UNIFORM_SIZE = 35384 	# GL/glext.h:1863
GL_UNIFORM_NAME_LENGTH = 35385 	# GL/glext.h:1864
GL_UNIFORM_BLOCK_INDEX = 35386 	# GL/glext.h:1865
GL_UNIFORM_OFFSET = 35387 	# GL/glext.h:1866
GL_UNIFORM_ARRAY_STRIDE = 35388 	# GL/glext.h:1867
GL_UNIFORM_MATRIX_STRIDE = 35389 	# GL/glext.h:1868
GL_UNIFORM_IS_ROW_MAJOR = 35390 	# GL/glext.h:1869
GL_UNIFORM_BLOCK_BINDING = 35391 	# GL/glext.h:1870
GL_UNIFORM_BLOCK_DATA_SIZE = 35392 	# GL/glext.h:1871
GL_UNIFORM_BLOCK_NAME_LENGTH = 35393 	# GL/glext.h:1872
GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS = 35394 	# GL/glext.h:1873
GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES = 35395 	# GL/glext.h:1874
GL_UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER = 35396 	# GL/glext.h:1875
GL_UNIFORM_BLOCK_REFERENCED_BY_GEOMETRY_SHADER = 35397 	# GL/glext.h:1876
GL_UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER = 35398 	# GL/glext.h:1877
GL_INVALID_INDEX = 4294967295 	# GL/glext.h:1878
# ARB_compatibility (GL/glext.h:1881)
# ARB_copy_buffer (GL/glext.h:1885)
GL_COPY_READ_BUFFER_BINDING = 36662 	# GL/glext.h:1886
GL_COPY_READ_BUFFER = 36662 	# GL/glext.h:1887
GL_COPY_WRITE_BUFFER_BINDING = 36663 	# GL/glext.h:1888
GL_COPY_WRITE_BUFFER = 36663 	# GL/glext.h:1889
# ARB_shader_texture_lod (GL/glext.h:1892)
# ARB_depth_clamp (GL/glext.h:1895)
GL_DEPTH_CLAMP = 34383 	# GL/glext.h:1896
# ARB_draw_elements_base_vertex (GL/glext.h:1899)
# ARB_fragment_coord_conventions (GL/glext.h:1902)
# ARB_provoking_vertex (GL/glext.h:1905)
GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION = 36428 	# GL/glext.h:1906
GL_FIRST_VERTEX_CONVENTION = 36429 	# GL/glext.h:1907
GL_LAST_VERTEX_CONVENTION = 36430 	# GL/glext.h:1908
GL_PROVOKING_VERTEX = 36431 	# GL/glext.h:1909
# ARB_seamless_cube_map (GL/glext.h:1912)
GL_TEXTURE_CUBE_MAP_SEAMLESS = 34895 	# GL/glext.h:1913
# ARB_sync (GL/glext.h:1916)
GL_MAX_SERVER_WAIT_TIMEOUT = 37137 	# GL/glext.h:1917
GL_OBJECT_TYPE = 37138 	# GL/glext.h:1918
GL_SYNC_CONDITION = 37139 	# GL/glext.h:1919
GL_SYNC_STATUS = 37140 	# GL/glext.h:1920
GL_SYNC_FLAGS = 37141 	# GL/glext.h:1921
GL_SYNC_FENCE = 37142 	# GL/glext.h:1922
GL_SYNC_GPU_COMMANDS_COMPLETE = 37143 	# GL/glext.h:1923
GL_UNSIGNALED = 37144 	# GL/glext.h:1924
GL_SIGNALED = 37145 	# GL/glext.h:1925
GL_ALREADY_SIGNALED = 37146 	# GL/glext.h:1926
GL_TIMEOUT_EXPIRED = 37147 	# GL/glext.h:1927
GL_CONDITION_SATISFIED = 37148 	# GL/glext.h:1928
GL_WAIT_FAILED = 37149 	# GL/glext.h:1929
GL_SYNC_FLUSH_COMMANDS_BIT = 1 	# GL/glext.h:1930
GL_TIMEOUT_IGNORED = 18446744073709551615 	# GL/glext.h:1931
# ARB_texture_multisample (GL/glext.h:1934)
GL_SAMPLE_POSITION = 36432 	# GL/glext.h:1935
GL_SAMPLE_MASK = 36433 	# GL/glext.h:1936
GL_SAMPLE_MASK_VALUE = 36434 	# GL/glext.h:1937
GL_MAX_SAMPLE_MASK_WORDS = 36441 	# GL/glext.h:1938
GL_TEXTURE_2D_MULTISAMPLE = 37120 	# GL/glext.h:1939
GL_PROXY_TEXTURE_2D_MULTISAMPLE = 37121 	# GL/glext.h:1940
GL_TEXTURE_2D_MULTISAMPLE_ARRAY = 37122 	# GL/glext.h:1941
GL_PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY = 37123 	# GL/glext.h:1942
GL_TEXTURE_BINDING_2D_MULTISAMPLE = 37124 	# GL/glext.h:1943
GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY = 37125 	# GL/glext.h:1944
GL_TEXTURE_SAMPLES = 37126 	# GL/glext.h:1945
GL_TEXTURE_FIXED_SAMPLE_LOCATIONS = 37127 	# GL/glext.h:1946
GL_SAMPLER_2D_MULTISAMPLE = 37128 	# GL/glext.h:1947
GL_INT_SAMPLER_2D_MULTISAMPLE = 37129 	# GL/glext.h:1948
GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE = 37130 	# GL/glext.h:1949
GL_SAMPLER_2D_MULTISAMPLE_ARRAY = 37131 	# GL/glext.h:1950
GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY = 37132 	# GL/glext.h:1951
GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY = 37133 	# GL/glext.h:1952
GL_MAX_COLOR_TEXTURE_SAMPLES = 37134 	# GL/glext.h:1953
GL_MAX_DEPTH_TEXTURE_SAMPLES = 37135 	# GL/glext.h:1954
GL_MAX_INTEGER_SAMPLES = 37136 	# GL/glext.h:1955
# ARB_vertex_array_bgra (GL/glext.h:1958)
# ARB_draw_buffers_blend (GL/glext.h:1962)
# ARB_sample_shading (GL/glext.h:1965)
GL_SAMPLE_SHADING_ARB = 35894 	# GL/glext.h:1966
GL_MIN_SAMPLE_SHADING_VALUE_ARB = 35895 	# GL/glext.h:1967
# ARB_texture_cube_map_array (GL/glext.h:1970)
GL_TEXTURE_CUBE_MAP_ARRAY_ARB = 36873 	# GL/glext.h:1971
GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_ARB = 36874 	# GL/glext.h:1972
GL_PROXY_TEXTURE_CUBE_MAP_ARRAY_ARB = 36875 	# GL/glext.h:1973
GL_SAMPLER_CUBE_MAP_ARRAY_ARB = 36876 	# GL/glext.h:1974
GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW_ARB = 36877 	# GL/glext.h:1975
GL_INT_SAMPLER_CUBE_MAP_ARRAY_ARB = 36878 	# GL/glext.h:1976
GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY_ARB = 36879 	# GL/glext.h:1977
# ARB_texture_gather (GL/glext.h:1980)
GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_ARB = 36446 	# GL/glext.h:1981
GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_ARB = 36447 	# GL/glext.h:1982
# ARB_texture_query_lod (GL/glext.h:1985)
# ARB_shading_language_include (GL/glext.h:1988)
GL_SHADER_INCLUDE_ARB = 36270 	# GL/glext.h:1989
GL_NAMED_STRING_LENGTH_ARB = 36329 	# GL/glext.h:1990
GL_NAMED_STRING_TYPE_ARB = 36330 	# GL/glext.h:1991
# ARB_texture_compression_bptc (GL/glext.h:1994)
GL_COMPRESSED_RGBA_BPTC_UNORM_ARB = 36492 	# GL/glext.h:1995
GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM_ARB = 36493 	# GL/glext.h:1996
GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT_ARB = 36494 	# GL/glext.h:1997
GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT_ARB = 36495 	# GL/glext.h:1998
# ARB_blend_func_extended (GL/glext.h:2001)
GL_SRC1_COLOR = 35065 	# GL/glext.h:2002
GL_ONE_MINUS_SRC1_COLOR = 35066 	# GL/glext.h:2004
GL_ONE_MINUS_SRC1_ALPHA = 35067 	# GL/glext.h:2005
GL_MAX_DUAL_SOURCE_DRAW_BUFFERS = 35068 	# GL/glext.h:2006
# ARB_explicit_attrib_location (GL/glext.h:2009)
# ARB_occlusion_query2 (GL/glext.h:2012)
GL_ANY_SAMPLES_PASSED = 35887 	# GL/glext.h:2013
# ARB_sampler_objects (GL/glext.h:2016)
GL_SAMPLER_BINDING = 35097 	# GL/glext.h:2017
# ARB_shader_bit_encoding (GL/glext.h:2020)
# ARB_texture_rgb10_a2ui (GL/glext.h:2023)
GL_RGB10_A2UI = 36975 	# GL/glext.h:2024
# ARB_texture_swizzle (GL/glext.h:2027)
GL_TEXTURE_SWIZZLE_R = 36418 	# GL/glext.h:2028
GL_TEXTURE_SWIZZLE_G = 36419 	# GL/glext.h:2029
GL_TEXTURE_SWIZZLE_B = 36420 	# GL/glext.h:2030
GL_TEXTURE_SWIZZLE_A = 36421 	# GL/glext.h:2031
GL_TEXTURE_SWIZZLE_RGBA = 36422 	# GL/glext.h:2032
# ARB_timer_query (GL/glext.h:2035)
GL_TIME_ELAPSED = 35007 	# GL/glext.h:2036
GL_TIMESTAMP = 36392 	# GL/glext.h:2037
# ARB_vertex_type_2_10_10_10_rev (GL/glext.h:2040)
GL_INT_2_10_10_10_REV = 36255 	# GL/glext.h:2042
# ARB_draw_indirect (GL/glext.h:2045)
GL_DRAW_INDIRECT_BUFFER = 36671 	# GL/glext.h:2046
GL_DRAW_INDIRECT_BUFFER_BINDING = 36675 	# GL/glext.h:2047
# ARB_gpu_shader5 (GL/glext.h:2050)
GL_GEOMETRY_SHADER_INVOCATIONS = 34943 	# GL/glext.h:2051
GL_MAX_GEOMETRY_SHADER_INVOCATIONS = 36442 	# GL/glext.h:2052
GL_MIN_FRAGMENT_INTERPOLATION_OFFSET = 36443 	# GL/glext.h:2053
GL_MAX_FRAGMENT_INTERPOLATION_OFFSET = 36444 	# GL/glext.h:2054
GL_FRAGMENT_INTERPOLATION_OFFSET_BITS = 36445 	# GL/glext.h:2055
# ARB_gpu_shader_fp64 (GL/glext.h:2059)
GL_DOUBLE_VEC2 = 36860 	# GL/glext.h:2061
GL_DOUBLE_VEC3 = 36861 	# GL/glext.h:2062
GL_DOUBLE_VEC4 = 36862 	# GL/glext.h:2063
GL_DOUBLE_MAT2 = 36678 	# GL/glext.h:2064
GL_DOUBLE_MAT3 = 36679 	# GL/glext.h:2065
GL_DOUBLE_MAT4 = 36680 	# GL/glext.h:2066
GL_DOUBLE_MAT2x3 = 36681 	# GL/glext.h:2067
GL_DOUBLE_MAT2x4 = 36682 	# GL/glext.h:2068
GL_DOUBLE_MAT3x2 = 36683 	# GL/glext.h:2069
GL_DOUBLE_MAT3x4 = 36684 	# GL/glext.h:2070
GL_DOUBLE_MAT4x2 = 36685 	# GL/glext.h:2071
GL_DOUBLE_MAT4x3 = 36686 	# GL/glext.h:2072
# ARB_shader_subroutine (GL/glext.h:2075)
GL_ACTIVE_SUBROUTINES = 36325 	# GL/glext.h:2076
GL_ACTIVE_SUBROUTINE_UNIFORMS = 36326 	# GL/glext.h:2077
GL_ACTIVE_SUBROUTINE_UNIFORM_LOCATIONS = 36423 	# GL/glext.h:2078
GL_ACTIVE_SUBROUTINE_MAX_LENGTH = 36424 	# GL/glext.h:2079
GL_ACTIVE_SUBROUTINE_UNIFORM_MAX_LENGTH = 36425 	# GL/glext.h:2080
GL_MAX_SUBROUTINES = 36327 	# GL/glext.h:2081
GL_MAX_SUBROUTINE_UNIFORM_LOCATIONS = 36328 	# GL/glext.h:2082
GL_NUM_COMPATIBLE_SUBROUTINES = 36426 	# GL/glext.h:2083
GL_COMPATIBLE_SUBROUTINES = 36427 	# GL/glext.h:2084
# ARB_tessellation_shader (GL/glext.h:2089)
GL_PATCHES = 14 	# GL/glext.h:2090
GL_PATCH_VERTICES = 36466 	# GL/glext.h:2091
GL_PATCH_DEFAULT_INNER_LEVEL = 36467 	# GL/glext.h:2092
GL_PATCH_DEFAULT_OUTER_LEVEL = 36468 	# GL/glext.h:2093
GL_TESS_CONTROL_OUTPUT_VERTICES = 36469 	# GL/glext.h:2094
GL_TESS_GEN_MODE = 36470 	# GL/glext.h:2095
GL_TESS_GEN_SPACING = 36471 	# GL/glext.h:2096
GL_TESS_GEN_VERTEX_ORDER = 36472 	# GL/glext.h:2097
GL_TESS_GEN_POINT_MODE = 36473 	# GL/glext.h:2098
GL_ISOLINES = 36474 	# GL/glext.h:2101
GL_FRACTIONAL_ODD = 36475 	# GL/glext.h:2103
GL_FRACTIONAL_EVEN = 36476 	# GL/glext.h:2104
GL_MAX_PATCH_VERTICES = 36477 	# GL/glext.h:2107
GL_MAX_TESS_GEN_LEVEL = 36478 	# GL/glext.h:2108
GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS = 36479 	# GL/glext.h:2109
GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS = 36480 	# GL/glext.h:2110
GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS = 36481 	# GL/glext.h:2111
GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS = 36482 	# GL/glext.h:2112
GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS = 36483 	# GL/glext.h:2113
GL_MAX_TESS_PATCH_COMPONENTS = 36484 	# GL/glext.h:2114
GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS = 36485 	# GL/glext.h:2115
GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS = 36486 	# GL/glext.h:2116
GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS = 36489 	# GL/glext.h:2117
GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS = 36490 	# GL/glext.h:2118
GL_MAX_TESS_CONTROL_INPUT_COMPONENTS = 34924 	# GL/glext.h:2119
GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS = 34925 	# GL/glext.h:2120
GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS = 36382 	# GL/glext.h:2121
GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS = 36383 	# GL/glext.h:2122
GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_CONTROL_SHADER = 34032 	# GL/glext.h:2123
GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_EVALUATION_SHADER = 34033 	# GL/glext.h:2124
GL_TESS_EVALUATION_SHADER = 36487 	# GL/glext.h:2125
GL_TESS_CONTROL_SHADER = 36488 	# GL/glext.h:2126
# ARB_texture_buffer_object_rgb32 (GL/glext.h:2129)
# ARB_transform_feedback2 (GL/glext.h:2135)
GL_TRANSFORM_FEEDBACK = 36386 	# GL/glext.h:2136
GL_TRANSFORM_FEEDBACK_PAUSED = 36387 	# GL/glext.h:2137
GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED = 36387 	# GL/glext.h:2138
GL_TRANSFORM_FEEDBACK_ACTIVE = 36388 	# GL/glext.h:2139
GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE = 36388 	# GL/glext.h:2140
GL_TRANSFORM_FEEDBACK_BINDING = 36389 	# GL/glext.h:2141
# ARB_transform_feedback3 (GL/glext.h:2144)
GL_MAX_TRANSFORM_FEEDBACK_BUFFERS = 36464 	# GL/glext.h:2145
GL_MAX_VERTEX_STREAMS = 36465 	# GL/glext.h:2146
# ARB_ES2_compatibility (GL/glext.h:2149)
GL_FIXED = 5132 	# GL/glext.h:2150
GL_IMPLEMENTATION_COLOR_READ_TYPE = 35738 	# GL/glext.h:2151
GL_IMPLEMENTATION_COLOR_READ_FORMAT = 35739 	# GL/glext.h:2152
GL_LOW_FLOAT = 36336 	# GL/glext.h:2153
GL_MEDIUM_FLOAT = 36337 	# GL/glext.h:2154
GL_HIGH_FLOAT = 36338 	# GL/glext.h:2155
GL_LOW_INT = 36339 	# GL/glext.h:2156
GL_MEDIUM_INT = 36340 	# GL/glext.h:2157
GL_HIGH_INT = 36341 	# GL/glext.h:2158
GL_SHADER_COMPILER = 36346 	# GL/glext.h:2159
GL_NUM_SHADER_BINARY_FORMATS = 36345 	# GL/glext.h:2160
GL_MAX_VERTEX_UNIFORM_VECTORS = 36347 	# GL/glext.h:2161
GL_MAX_VARYING_VECTORS = 36348 	# GL/glext.h:2162
GL_MAX_FRAGMENT_UNIFORM_VECTORS = 36349 	# GL/glext.h:2163
GL_RGB565 = 36194 	# GL/glext.h:2164
# ARB_get_program_binary (GL/glext.h:2167)
GL_PROGRAM_BINARY_RETRIEVABLE_HINT = 33367 	# GL/glext.h:2168
GL_PROGRAM_BINARY_LENGTH = 34625 	# GL/glext.h:2169
GL_NUM_PROGRAM_BINARY_FORMATS = 34814 	# GL/glext.h:2170
GL_PROGRAM_BINARY_FORMATS = 34815 	# GL/glext.h:2171
# ARB_separate_shader_objects (GL/glext.h:2174)
GL_VERTEX_SHADER_BIT = 1 	# GL/glext.h:2175
GL_FRAGMENT_SHADER_BIT = 2 	# GL/glext.h:2176
GL_GEOMETRY_SHADER_BIT = 4 	# GL/glext.h:2177
GL_TESS_CONTROL_SHADER_BIT = 8 	# GL/glext.h:2178
GL_TESS_EVALUATION_SHADER_BIT = 16 	# GL/glext.h:2179
GL_ALL_SHADER_BITS = 4294967295 	# GL/glext.h:2180
GL_PROGRAM_SEPARABLE = 33368 	# GL/glext.h:2181
GL_ACTIVE_PROGRAM = 33369 	# GL/glext.h:2182
GL_PROGRAM_PIPELINE_BINDING = 33370 	# GL/glext.h:2183
# ARB_shader_precision (GL/glext.h:2186)
# ARB_vertex_attrib_64bit (GL/glext.h:2189)
# ARB_viewport_array (GL/glext.h:2205)
GL_MAX_VIEWPORTS = 33371 	# GL/glext.h:2210
GL_VIEWPORT_SUBPIXEL_BITS = 33372 	# GL/glext.h:2211
GL_VIEWPORT_BOUNDS_RANGE = 33373 	# GL/glext.h:2212
GL_LAYER_PROVOKING_VERTEX = 33374 	# GL/glext.h:2213
GL_VIEWPORT_INDEX_PROVOKING_VERTEX = 33375 	# GL/glext.h:2214
GL_UNDEFINED_VERTEX = 33376 	# GL/glext.h:2215
# ARB_cl_event (GL/glext.h:2221)
GL_SYNC_CL_EVENT_ARB = 33344 	# GL/glext.h:2222
GL_SYNC_CL_EVENT_COMPLETE_ARB = 33345 	# GL/glext.h:2223
# ARB_debug_output (GL/glext.h:2226)
GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB = 33346 	# GL/glext.h:2227
GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH_ARB = 33347 	# GL/glext.h:2228
GL_DEBUG_CALLBACK_FUNCTION_ARB = 33348 	# GL/glext.h:2229
GL_DEBUG_CALLBACK_USER_PARAM_ARB = 33349 	# GL/glext.h:2230
GL_DEBUG_SOURCE_API_ARB = 33350 	# GL/glext.h:2231
GL_DEBUG_SOURCE_WINDOW_SYSTEM_ARB = 33351 	# GL/glext.h:2232
GL_DEBUG_SOURCE_SHADER_COMPILER_ARB = 33352 	# GL/glext.h:2233
GL_DEBUG_SOURCE_THIRD_PARTY_ARB = 33353 	# GL/glext.h:2234
GL_DEBUG_SOURCE_APPLICATION_ARB = 33354 	# GL/glext.h:2235
GL_DEBUG_SOURCE_OTHER_ARB = 33355 	# GL/glext.h:2236
GL_DEBUG_TYPE_ERROR_ARB = 33356 	# GL/glext.h:2237
GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR_ARB = 33357 	# GL/glext.h:2238
GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR_ARB = 33358 	# GL/glext.h:2239
GL_DEBUG_TYPE_PORTABILITY_ARB = 33359 	# GL/glext.h:2240
GL_DEBUG_TYPE_PERFORMANCE_ARB = 33360 	# GL/glext.h:2241
GL_DEBUG_TYPE_OTHER_ARB = 33361 	# GL/glext.h:2242
GL_MAX_DEBUG_MESSAGE_LENGTH_ARB = 37187 	# GL/glext.h:2243
GL_MAX_DEBUG_LOGGED_MESSAGES_ARB = 37188 	# GL/glext.h:2244
GL_DEBUG_LOGGED_MESSAGES_ARB = 37189 	# GL/glext.h:2245
GL_DEBUG_SEVERITY_HIGH_ARB = 37190 	# GL/glext.h:2246
GL_DEBUG_SEVERITY_MEDIUM_ARB = 37191 	# GL/glext.h:2247
GL_DEBUG_SEVERITY_LOW_ARB = 37192 	# GL/glext.h:2248
# ARB_robustness (GL/glext.h:2251)
GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB = 4 	# GL/glext.h:2253
GL_LOSE_CONTEXT_ON_RESET_ARB = 33362 	# GL/glext.h:2254
GL_GUILTY_CONTEXT_RESET_ARB = 33363 	# GL/glext.h:2255
GL_INNOCENT_CONTEXT_RESET_ARB = 33364 	# GL/glext.h:2256
GL_UNKNOWN_CONTEXT_RESET_ARB = 33365 	# GL/glext.h:2257
GL_RESET_NOTIFICATION_STRATEGY_ARB = 33366 	# GL/glext.h:2258
GL_NO_RESET_NOTIFICATION_ARB = 33377 	# GL/glext.h:2259
# ARB_shader_stencil_export (GL/glext.h:2262)
# ARB_base_instance (GL/glext.h:2265)
# ARB_shading_language_420pack (GL/glext.h:2268)
# ARB_transform_feedback_instanced (GL/glext.h:2271)
# ARB_compressed_texture_pixel_storage (GL/glext.h:2274)
GL_UNPACK_COMPRESSED_BLOCK_WIDTH = 37159 	# GL/glext.h:2275
GL_UNPACK_COMPRESSED_BLOCK_HEIGHT = 37160 	# GL/glext.h:2276
GL_UNPACK_COMPRESSED_BLOCK_DEPTH = 37161 	# GL/glext.h:2277
GL_UNPACK_COMPRESSED_BLOCK_SIZE = 37162 	# GL/glext.h:2278
GL_PACK_COMPRESSED_BLOCK_WIDTH = 37163 	# GL/glext.h:2279
GL_PACK_COMPRESSED_BLOCK_HEIGHT = 37164 	# GL/glext.h:2280
GL_PACK_COMPRESSED_BLOCK_DEPTH = 37165 	# GL/glext.h:2281
GL_PACK_COMPRESSED_BLOCK_SIZE = 37166 	# GL/glext.h:2282
# ARB_conservative_depth (GL/glext.h:2285)
# ARB_internalformat_query (GL/glext.h:2288)
GL_NUM_SAMPLE_COUNTS = 37760 	# GL/glext.h:2289
# ARB_map_buffer_alignment (GL/glext.h:2292)
GL_MIN_MAP_BUFFER_ALIGNMENT = 37052 	# GL/glext.h:2293
# ARB_shader_atomic_counters (GL/glext.h:2296)
GL_ATOMIC_COUNTER_BUFFER = 37568 	# GL/glext.h:2297
GL_ATOMIC_COUNTER_BUFFER_BINDING = 37569 	# GL/glext.h:2298
GL_ATOMIC_COUNTER_BUFFER_START = 37570 	# GL/glext.h:2299
GL_ATOMIC_COUNTER_BUFFER_SIZE = 37571 	# GL/glext.h:2300
GL_ATOMIC_COUNTER_BUFFER_DATA_SIZE = 37572 	# GL/glext.h:2301
GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTERS = 37573 	# GL/glext.h:2302
GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTER_INDICES = 37574 	# GL/glext.h:2303
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_VERTEX_SHADER = 37575 	# GL/glext.h:2304
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_CONTROL_SHADER = 37576 	# GL/glext.h:2305
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_EVALUATION_SHADER = 37577 	# GL/glext.h:2306
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_GEOMETRY_SHADER = 37578 	# GL/glext.h:2307
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_FRAGMENT_SHADER = 37579 	# GL/glext.h:2308
GL_MAX_VERTEX_ATOMIC_COUNTER_BUFFERS = 37580 	# GL/glext.h:2309
GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS = 37581 	# GL/glext.h:2310
GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS = 37582 	# GL/glext.h:2311
GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS = 37583 	# GL/glext.h:2312
GL_MAX_FRAGMENT_ATOMIC_COUNTER_BUFFERS = 37584 	# GL/glext.h:2313
GL_MAX_COMBINED_ATOMIC_COUNTER_BUFFERS = 37585 	# GL/glext.h:2314
GL_MAX_VERTEX_ATOMIC_COUNTERS = 37586 	# GL/glext.h:2315
GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS = 37587 	# GL/glext.h:2316
GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS = 37588 	# GL/glext.h:2317
GL_MAX_GEOMETRY_ATOMIC_COUNTERS = 37589 	# GL/glext.h:2318
GL_MAX_FRAGMENT_ATOMIC_COUNTERS = 37590 	# GL/glext.h:2319
GL_MAX_COMBINED_ATOMIC_COUNTERS = 37591 	# GL/glext.h:2320
GL_MAX_ATOMIC_COUNTER_BUFFER_SIZE = 37592 	# GL/glext.h:2321
GL_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS = 37596 	# GL/glext.h:2322
GL_ACTIVE_ATOMIC_COUNTER_BUFFERS = 37593 	# GL/glext.h:2323
GL_UNIFORM_ATOMIC_COUNTER_BUFFER_INDEX = 37594 	# GL/glext.h:2324
GL_UNSIGNED_INT_ATOMIC_COUNTER = 37595 	# GL/glext.h:2325
# ARB_shader_image_load_store (GL/glext.h:2328)
GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT = 1 	# GL/glext.h:2329
GL_ELEMENT_ARRAY_BARRIER_BIT = 2 	# GL/glext.h:2330
GL_UNIFORM_BARRIER_BIT = 4 	# GL/glext.h:2331
GL_TEXTURE_FETCH_BARRIER_BIT = 8 	# GL/glext.h:2332
GL_SHADER_IMAGE_ACCESS_BARRIER_BIT = 32 	# GL/glext.h:2333
GL_COMMAND_BARRIER_BIT = 64 	# GL/glext.h:2334
GL_PIXEL_BUFFER_BARRIER_BIT = 128 	# GL/glext.h:2335
GL_TEXTURE_UPDATE_BARRIER_BIT = 256 	# GL/glext.h:2336
GL_BUFFER_UPDATE_BARRIER_BIT = 512 	# GL/glext.h:2337
GL_FRAMEBUFFER_BARRIER_BIT = 1024 	# GL/glext.h:2338
GL_TRANSFORM_FEEDBACK_BARRIER_BIT = 2048 	# GL/glext.h:2339
GL_ATOMIC_COUNTER_BARRIER_BIT = 4096 	# GL/glext.h:2340
GL_ALL_BARRIER_BITS = 4294967295 	# GL/glext.h:2341
GL_MAX_IMAGE_UNITS = 36664 	# GL/glext.h:2342
GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS = 36665 	# GL/glext.h:2343
GL_IMAGE_BINDING_NAME = 36666 	# GL/glext.h:2344
GL_IMAGE_BINDING_LEVEL = 36667 	# GL/glext.h:2345
GL_IMAGE_BINDING_LAYERED = 36668 	# GL/glext.h:2346
GL_IMAGE_BINDING_LAYER = 36669 	# GL/glext.h:2347
GL_IMAGE_BINDING_ACCESS = 36670 	# GL/glext.h:2348
GL_IMAGE_1D = 36940 	# GL/glext.h:2349
GL_IMAGE_2D = 36941 	# GL/glext.h:2350
GL_IMAGE_3D = 36942 	# GL/glext.h:2351
GL_IMAGE_2D_RECT = 36943 	# GL/glext.h:2352
GL_IMAGE_CUBE = 36944 	# GL/glext.h:2353
GL_IMAGE_BUFFER = 36945 	# GL/glext.h:2354
GL_IMAGE_1D_ARRAY = 36946 	# GL/glext.h:2355
GL_IMAGE_2D_ARRAY = 36947 	# GL/glext.h:2356
GL_IMAGE_CUBE_MAP_ARRAY = 36948 	# GL/glext.h:2357
GL_IMAGE_2D_MULTISAMPLE = 36949 	# GL/glext.h:2358
GL_IMAGE_2D_MULTISAMPLE_ARRAY = 36950 	# GL/glext.h:2359
GL_INT_IMAGE_1D = 36951 	# GL/glext.h:2360
GL_INT_IMAGE_2D = 36952 	# GL/glext.h:2361
GL_INT_IMAGE_3D = 36953 	# GL/glext.h:2362
GL_INT_IMAGE_2D_RECT = 36954 	# GL/glext.h:2363
GL_INT_IMAGE_CUBE = 36955 	# GL/glext.h:2364
GL_INT_IMAGE_BUFFER = 36956 	# GL/glext.h:2365
GL_INT_IMAGE_1D_ARRAY = 36957 	# GL/glext.h:2366
GL_INT_IMAGE_2D_ARRAY = 36958 	# GL/glext.h:2367
GL_INT_IMAGE_CUBE_MAP_ARRAY = 36959 	# GL/glext.h:2368
GL_INT_IMAGE_2D_MULTISAMPLE = 36960 	# GL/glext.h:2369
GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY = 36961 	# GL/glext.h:2370
GL_UNSIGNED_INT_IMAGE_1D = 36962 	# GL/glext.h:2371
GL_UNSIGNED_INT_IMAGE_2D = 36963 	# GL/glext.h:2372
GL_UNSIGNED_INT_IMAGE_3D = 36964 	# GL/glext.h:2373
GL_UNSIGNED_INT_IMAGE_2D_RECT = 36965 	# GL/glext.h:2374
GL_UNSIGNED_INT_IMAGE_CUBE = 36966 	# GL/glext.h:2375
GL_UNSIGNED_INT_IMAGE_BUFFER = 36967 	# GL/glext.h:2376
GL_UNSIGNED_INT_IMAGE_1D_ARRAY = 36968 	# GL/glext.h:2377
GL_UNSIGNED_INT_IMAGE_2D_ARRAY = 36969 	# GL/glext.h:2378
GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY = 36970 	# GL/glext.h:2379
GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE = 36971 	# GL/glext.h:2380
GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY = 36972 	# GL/glext.h:2381
GL_MAX_IMAGE_SAMPLES = 36973 	# GL/glext.h:2382
GL_IMAGE_BINDING_FORMAT = 36974 	# GL/glext.h:2383
GL_IMAGE_FORMAT_COMPATIBILITY_TYPE = 37063 	# GL/glext.h:2384
GL_IMAGE_FORMAT_COMPATIBILITY_BY_SIZE = 37064 	# GL/glext.h:2385
GL_IMAGE_FORMAT_COMPATIBILITY_BY_CLASS = 37065 	# GL/glext.h:2386
GL_MAX_VERTEX_IMAGE_UNIFORMS = 37066 	# GL/glext.h:2387
GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS = 37067 	# GL/glext.h:2388
GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS = 37068 	# GL/glext.h:2389
GL_MAX_GEOMETRY_IMAGE_UNIFORMS = 37069 	# GL/glext.h:2390
GL_MAX_FRAGMENT_IMAGE_UNIFORMS = 37070 	# GL/glext.h:2391
GL_MAX_COMBINED_IMAGE_UNIFORMS = 37071 	# GL/glext.h:2392
# ARB_shading_language_packing (GL/glext.h:2395)
# ARB_texture_storage (GL/glext.h:2398)
GL_TEXTURE_IMMUTABLE_FORMAT = 37167 	# GL/glext.h:2399
# EXT_abgr (GL/glext.h:2402)
GL_ABGR_EXT = 32768 	# GL/glext.h:2403
# EXT_blend_color (GL/glext.h:2406)
GL_CONSTANT_COLOR_EXT = 32769 	# GL/glext.h:2407
GL_ONE_MINUS_CONSTANT_COLOR_EXT = 32770 	# GL/glext.h:2408
GL_CONSTANT_ALPHA_EXT = 32771 	# GL/glext.h:2409
GL_ONE_MINUS_CONSTANT_ALPHA_EXT = 32772 	# GL/glext.h:2410
GL_BLEND_COLOR_EXT = 32773 	# GL/glext.h:2411
# EXT_polygon_offset (GL/glext.h:2414)
GL_POLYGON_OFFSET_EXT = 32823 	# GL/glext.h:2415
GL_POLYGON_OFFSET_FACTOR_EXT = 32824 	# GL/glext.h:2416
GL_POLYGON_OFFSET_BIAS_EXT = 32825 	# GL/glext.h:2417
# EXT_texture (GL/glext.h:2420)
GL_ALPHA4_EXT = 32827 	# GL/glext.h:2421
GL_ALPHA8_EXT = 32828 	# GL/glext.h:2422
GL_ALPHA12_EXT = 32829 	# GL/glext.h:2423
GL_ALPHA16_EXT = 32830 	# GL/glext.h:2424
GL_LUMINANCE4_EXT = 32831 	# GL/glext.h:2425
GL_LUMINANCE8_EXT = 32832 	# GL/glext.h:2426
GL_LUMINANCE12_EXT = 32833 	# GL/glext.h:2427
GL_LUMINANCE16_EXT = 32834 	# GL/glext.h:2428
GL_LUMINANCE4_ALPHA4_EXT = 32835 	# GL/glext.h:2429
GL_LUMINANCE6_ALPHA2_EXT = 32836 	# GL/glext.h:2430
GL_LUMINANCE8_ALPHA8_EXT = 32837 	# GL/glext.h:2431
GL_LUMINANCE12_ALPHA4_EXT = 32838 	# GL/glext.h:2432
GL_LUMINANCE12_ALPHA12_EXT = 32839 	# GL/glext.h:2433
GL_LUMINANCE16_ALPHA16_EXT = 32840 	# GL/glext.h:2434
GL_INTENSITY_EXT = 32841 	# GL/glext.h:2435
GL_INTENSITY4_EXT = 32842 	# GL/glext.h:2436
GL_INTENSITY8_EXT = 32843 	# GL/glext.h:2437
GL_INTENSITY12_EXT = 32844 	# GL/glext.h:2438
GL_INTENSITY16_EXT = 32845 	# GL/glext.h:2439
GL_RGB2_EXT = 32846 	# GL/glext.h:2440
GL_RGB4_EXT = 32847 	# GL/glext.h:2441
GL_RGB5_EXT = 32848 	# GL/glext.h:2442
GL_RGB8_EXT = 32849 	# GL/glext.h:2443
GL_RGB10_EXT = 32850 	# GL/glext.h:2444
GL_RGB12_EXT = 32851 	# GL/glext.h:2445
GL_RGB16_EXT = 32852 	# GL/glext.h:2446
GL_RGBA2_EXT = 32853 	# GL/glext.h:2447
GL_RGBA4_EXT = 32854 	# GL/glext.h:2448
GL_RGB5_A1_EXT = 32855 	# GL/glext.h:2449
GL_RGBA8_EXT = 32856 	# GL/glext.h:2450
GL_RGB10_A2_EXT = 32857 	# GL/glext.h:2451
GL_RGBA12_EXT = 32858 	# GL/glext.h:2452
GL_RGBA16_EXT = 32859 	# GL/glext.h:2453
GL_TEXTURE_RED_SIZE_EXT = 32860 	# GL/glext.h:2454
GL_TEXTURE_GREEN_SIZE_EXT = 32861 	# GL/glext.h:2455
GL_TEXTURE_BLUE_SIZE_EXT = 32862 	# GL/glext.h:2456
GL_TEXTURE_ALPHA_SIZE_EXT = 32863 	# GL/glext.h:2457
GL_TEXTURE_LUMINANCE_SIZE_EXT = 32864 	# GL/glext.h:2458
GL_TEXTURE_INTENSITY_SIZE_EXT = 32865 	# GL/glext.h:2459
GL_REPLACE_EXT = 32866 	# GL/glext.h:2460
GL_PROXY_TEXTURE_1D_EXT = 32867 	# GL/glext.h:2461
GL_PROXY_TEXTURE_2D_EXT = 32868 	# GL/glext.h:2462
GL_TEXTURE_TOO_LARGE_EXT = 32869 	# GL/glext.h:2463
# EXT_texture3D (GL/glext.h:2466)
GL_PACK_SKIP_IMAGES_EXT = 32875 	# GL/glext.h:2467
GL_PACK_IMAGE_HEIGHT_EXT = 32876 	# GL/glext.h:2468
GL_UNPACK_SKIP_IMAGES_EXT = 32877 	# GL/glext.h:2469
GL_UNPACK_IMAGE_HEIGHT_EXT = 32878 	# GL/glext.h:2470
GL_TEXTURE_3D_EXT = 32879 	# GL/glext.h:2471
GL_PROXY_TEXTURE_3D_EXT = 32880 	# GL/glext.h:2472
GL_TEXTURE_DEPTH_EXT = 32881 	# GL/glext.h:2473
GL_TEXTURE_WRAP_R_EXT = 32882 	# GL/glext.h:2474
GL_MAX_3D_TEXTURE_SIZE_EXT = 32883 	# GL/glext.h:2475
# SGIS_texture_filter4 (GL/glext.h:2478)
GL_FILTER4_SGIS = 33094 	# GL/glext.h:2479
GL_TEXTURE_FILTER4_SIZE_SGIS = 33095 	# GL/glext.h:2480
# EXT_subtexture (GL/glext.h:2483)
# EXT_copy_texture (GL/glext.h:2486)
# EXT_histogram (GL/glext.h:2489)
GL_HISTOGRAM_EXT = 32804 	# GL/glext.h:2490
GL_PROXY_HISTOGRAM_EXT = 32805 	# GL/glext.h:2491
GL_HISTOGRAM_WIDTH_EXT = 32806 	# GL/glext.h:2492
GL_HISTOGRAM_FORMAT_EXT = 32807 	# GL/glext.h:2493
GL_HISTOGRAM_RED_SIZE_EXT = 32808 	# GL/glext.h:2494
GL_HISTOGRAM_GREEN_SIZE_EXT = 32809 	# GL/glext.h:2495
GL_HISTOGRAM_BLUE_SIZE_EXT = 32810 	# GL/glext.h:2496
GL_HISTOGRAM_ALPHA_SIZE_EXT = 32811 	# GL/glext.h:2497
GL_HISTOGRAM_LUMINANCE_SIZE_EXT = 32812 	# GL/glext.h:2498
GL_HISTOGRAM_SINK_EXT = 32813 	# GL/glext.h:2499
GL_MINMAX_EXT = 32814 	# GL/glext.h:2500
GL_MINMAX_FORMAT_EXT = 32815 	# GL/glext.h:2501
GL_MINMAX_SINK_EXT = 32816 	# GL/glext.h:2502
GL_TABLE_TOO_LARGE_EXT = 32817 	# GL/glext.h:2503
# EXT_convolution (GL/glext.h:2506)
GL_CONVOLUTION_1D_EXT = 32784 	# GL/glext.h:2507
GL_CONVOLUTION_2D_EXT = 32785 	# GL/glext.h:2508
GL_SEPARABLE_2D_EXT = 32786 	# GL/glext.h:2509
GL_CONVOLUTION_BORDER_MODE_EXT = 32787 	# GL/glext.h:2510
GL_CONVOLUTION_FILTER_SCALE_EXT = 32788 	# GL/glext.h:2511
GL_CONVOLUTION_FILTER_BIAS_EXT = 32789 	# GL/glext.h:2512
GL_REDUCE_EXT = 32790 	# GL/glext.h:2513
GL_CONVOLUTION_FORMAT_EXT = 32791 	# GL/glext.h:2514
GL_CONVOLUTION_WIDTH_EXT = 32792 	# GL/glext.h:2515
GL_CONVOLUTION_HEIGHT_EXT = 32793 	# GL/glext.h:2516
GL_MAX_CONVOLUTION_WIDTH_EXT = 32794 	# GL/glext.h:2517
GL_MAX_CONVOLUTION_HEIGHT_EXT = 32795 	# GL/glext.h:2518
GL_POST_CONVOLUTION_RED_SCALE_EXT = 32796 	# GL/glext.h:2519
GL_POST_CONVOLUTION_GREEN_SCALE_EXT = 32797 	# GL/glext.h:2520
GL_POST_CONVOLUTION_BLUE_SCALE_EXT = 32798 	# GL/glext.h:2521
GL_POST_CONVOLUTION_ALPHA_SCALE_EXT = 32799 	# GL/glext.h:2522
GL_POST_CONVOLUTION_RED_BIAS_EXT = 32800 	# GL/glext.h:2523
GL_POST_CONVOLUTION_GREEN_BIAS_EXT = 32801 	# GL/glext.h:2524
GL_POST_CONVOLUTION_BLUE_BIAS_EXT = 32802 	# GL/glext.h:2525
GL_POST_CONVOLUTION_ALPHA_BIAS_EXT = 32803 	# GL/glext.h:2526
# SGI_color_matrix (GL/glext.h:2529)
GL_COLOR_MATRIX_SGI = 32945 	# GL/glext.h:2530
GL_COLOR_MATRIX_STACK_DEPTH_SGI = 32946 	# GL/glext.h:2531
GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI = 32947 	# GL/glext.h:2532
GL_POST_COLOR_MATRIX_RED_SCALE_SGI = 32948 	# GL/glext.h:2533
GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI = 32949 	# GL/glext.h:2534
GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI = 32950 	# GL/glext.h:2535
GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI = 32951 	# GL/glext.h:2536
GL_POST_COLOR_MATRIX_RED_BIAS_SGI = 32952 	# GL/glext.h:2537
GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI = 32953 	# GL/glext.h:2538
GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI = 32954 	# GL/glext.h:2539
GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI = 32955 	# GL/glext.h:2540
# SGI_color_table (GL/glext.h:2543)
GL_COLOR_TABLE_SGI = 32976 	# GL/glext.h:2544
GL_POST_CONVOLUTION_COLOR_TABLE_SGI = 32977 	# GL/glext.h:2545
GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32978 	# GL/glext.h:2546
GL_PROXY_COLOR_TABLE_SGI = 32979 	# GL/glext.h:2547
GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI = 32980 	# GL/glext.h:2548
GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI = 32981 	# GL/glext.h:2549
GL_COLOR_TABLE_SCALE_SGI = 32982 	# GL/glext.h:2550
GL_COLOR_TABLE_BIAS_SGI = 32983 	# GL/glext.h:2551
GL_COLOR_TABLE_FORMAT_SGI = 32984 	# GL/glext.h:2552
GL_COLOR_TABLE_WIDTH_SGI = 32985 	# GL/glext.h:2553
GL_COLOR_TABLE_RED_SIZE_SGI = 32986 	# GL/glext.h:2554
GL_COLOR_TABLE_GREEN_SIZE_SGI = 32987 	# GL/glext.h:2555
GL_COLOR_TABLE_BLUE_SIZE_SGI = 32988 	# GL/glext.h:2556
GL_COLOR_TABLE_ALPHA_SIZE_SGI = 32989 	# GL/glext.h:2557
GL_COLOR_TABLE_LUMINANCE_SIZE_SGI = 32990 	# GL/glext.h:2558
GL_COLOR_TABLE_INTENSITY_SIZE_SGI = 32991 	# GL/glext.h:2559
# SGIS_pixel_texture (GL/glext.h:2562)
GL_PIXEL_TEXTURE_SGIS = 33619 	# GL/glext.h:2563
GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS = 33620 	# GL/glext.h:2564
GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS = 33621 	# GL/glext.h:2565
GL_PIXEL_GROUP_COLOR_SGIS = 33622 	# GL/glext.h:2566
# SGIX_pixel_texture (GL/glext.h:2569)
GL_PIXEL_TEX_GEN_SGIX = 33081 	# GL/glext.h:2570
GL_PIXEL_TEX_GEN_MODE_SGIX = 33579 	# GL/glext.h:2571
# SGIS_texture4D (GL/glext.h:2574)
GL_PACK_SKIP_VOLUMES_SGIS = 33072 	# GL/glext.h:2575
GL_PACK_IMAGE_DEPTH_SGIS = 33073 	# GL/glext.h:2576
GL_UNPACK_SKIP_VOLUMES_SGIS = 33074 	# GL/glext.h:2577
GL_UNPACK_IMAGE_DEPTH_SGIS = 33075 	# GL/glext.h:2578
GL_TEXTURE_4D_SGIS = 33076 	# GL/glext.h:2579
GL_PROXY_TEXTURE_4D_SGIS = 33077 	# GL/glext.h:2580
GL_TEXTURE_4DSIZE_SGIS = 33078 	# GL/glext.h:2581
GL_TEXTURE_WRAP_Q_SGIS = 33079 	# GL/glext.h:2582
GL_MAX_4D_TEXTURE_SIZE_SGIS = 33080 	# GL/glext.h:2583
GL_TEXTURE_4D_BINDING_SGIS = 33103 	# GL/glext.h:2584
# SGI_texture_color_table (GL/glext.h:2587)
GL_TEXTURE_COLOR_TABLE_SGI = 32956 	# GL/glext.h:2588
GL_PROXY_TEXTURE_COLOR_TABLE_SGI = 32957 	# GL/glext.h:2589
# EXT_cmyka (GL/glext.h:2592)
GL_CMYK_EXT = 32780 	# GL/glext.h:2593
GL_CMYKA_EXT = 32781 	# GL/glext.h:2594
GL_PACK_CMYK_HINT_EXT = 32782 	# GL/glext.h:2595
GL_UNPACK_CMYK_HINT_EXT = 32783 	# GL/glext.h:2596
# EXT_texture_object (GL/glext.h:2599)
GL_TEXTURE_PRIORITY_EXT = 32870 	# GL/glext.h:2600
GL_TEXTURE_RESIDENT_EXT = 32871 	# GL/glext.h:2601
GL_TEXTURE_1D_BINDING_EXT = 32872 	# GL/glext.h:2602
GL_TEXTURE_2D_BINDING_EXT = 32873 	# GL/glext.h:2603
GL_TEXTURE_3D_BINDING_EXT = 32874 	# GL/glext.h:2604
# SGIS_detail_texture (GL/glext.h:2607)
GL_DETAIL_TEXTURE_2D_SGIS = 32917 	# GL/glext.h:2608
GL_DETAIL_TEXTURE_2D_BINDING_SGIS = 32918 	# GL/glext.h:2609
GL_LINEAR_DETAIL_SGIS = 32919 	# GL/glext.h:2610
GL_LINEAR_DETAIL_ALPHA_SGIS = 32920 	# GL/glext.h:2611
GL_LINEAR_DETAIL_COLOR_SGIS = 32921 	# GL/glext.h:2612
GL_DETAIL_TEXTURE_LEVEL_SGIS = 32922 	# GL/glext.h:2613
GL_DETAIL_TEXTURE_MODE_SGIS = 32923 	# GL/glext.h:2614
GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS = 32924 	# GL/glext.h:2615
# SGIS_sharpen_texture (GL/glext.h:2618)
GL_LINEAR_SHARPEN_SGIS = 32941 	# GL/glext.h:2619
GL_LINEAR_SHARPEN_ALPHA_SGIS = 32942 	# GL/glext.h:2620
GL_LINEAR_SHARPEN_COLOR_SGIS = 32943 	# GL/glext.h:2621
GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS = 32944 	# GL/glext.h:2622
# EXT_packed_pixels (GL/glext.h:2625)
GL_UNSIGNED_BYTE_3_3_2_EXT = 32818 	# GL/glext.h:2626
GL_UNSIGNED_SHORT_4_4_4_4_EXT = 32819 	# GL/glext.h:2627
GL_UNSIGNED_SHORT_5_5_5_1_EXT = 32820 	# GL/glext.h:2628
GL_UNSIGNED_INT_8_8_8_8_EXT = 32821 	# GL/glext.h:2629
GL_UNSIGNED_INT_10_10_10_2_EXT = 32822 	# GL/glext.h:2630
# SGIS_texture_lod (GL/glext.h:2633)
GL_TEXTURE_MIN_LOD_SGIS = 33082 	# GL/glext.h:2634
GL_TEXTURE_MAX_LOD_SGIS = 33083 	# GL/glext.h:2635
GL_TEXTURE_BASE_LEVEL_SGIS = 33084 	# GL/glext.h:2636
GL_TEXTURE_MAX_LEVEL_SGIS = 33085 	# GL/glext.h:2637
# SGIS_multisample (GL/glext.h:2640)
GL_MULTISAMPLE_SGIS = 32925 	# GL/glext.h:2641
GL_SAMPLE_ALPHA_TO_MASK_SGIS = 32926 	# GL/glext.h:2642
GL_SAMPLE_ALPHA_TO_ONE_SGIS = 32927 	# GL/glext.h:2643
GL_SAMPLE_MASK_SGIS = 32928 	# GL/glext.h:2644
GL_1PASS_SGIS = 32929 	# GL/glext.h:2645
GL_2PASS_0_SGIS = 32930 	# GL/glext.h:2646
GL_2PASS_1_SGIS = 32931 	# GL/glext.h:2647
GL_4PASS_0_SGIS = 32932 	# GL/glext.h:2648
GL_4PASS_1_SGIS = 32933 	# GL/glext.h:2649
GL_4PASS_2_SGIS = 32934 	# GL/glext.h:2650
GL_4PASS_3_SGIS = 32935 	# GL/glext.h:2651
GL_SAMPLE_BUFFERS_SGIS = 32936 	# GL/glext.h:2652
GL_SAMPLES_SGIS = 32937 	# GL/glext.h:2653
GL_SAMPLE_MASK_VALUE_SGIS = 32938 	# GL/glext.h:2654
GL_SAMPLE_MASK_INVERT_SGIS = 32939 	# GL/glext.h:2655
GL_SAMPLE_PATTERN_SGIS = 32940 	# GL/glext.h:2656
# EXT_rescale_normal (GL/glext.h:2659)
GL_RESCALE_NORMAL_EXT = 32826 	# GL/glext.h:2660
# EXT_vertex_array (GL/glext.h:2663)
GL_VERTEX_ARRAY_EXT = 32884 	# GL/glext.h:2664
GL_NORMAL_ARRAY_EXT = 32885 	# GL/glext.h:2665
GL_COLOR_ARRAY_EXT = 32886 	# GL/glext.h:2666
GL_INDEX_ARRAY_EXT = 32887 	# GL/glext.h:2667
GL_TEXTURE_COORD_ARRAY_EXT = 32888 	# GL/glext.h:2668
GL_EDGE_FLAG_ARRAY_EXT = 32889 	# GL/glext.h:2669
GL_VERTEX_ARRAY_SIZE_EXT = 32890 	# GL/glext.h:2670
GL_VERTEX_ARRAY_TYPE_EXT = 32891 	# GL/glext.h:2671
GL_VERTEX_ARRAY_STRIDE_EXT = 32892 	# GL/glext.h:2672
GL_VERTEX_ARRAY_COUNT_EXT = 32893 	# GL/glext.h:2673
GL_NORMAL_ARRAY_TYPE_EXT = 32894 	# GL/glext.h:2674
GL_NORMAL_ARRAY_STRIDE_EXT = 32895 	# GL/glext.h:2675
GL_NORMAL_ARRAY_COUNT_EXT = 32896 	# GL/glext.h:2676
GL_COLOR_ARRAY_SIZE_EXT = 32897 	# GL/glext.h:2677
GL_COLOR_ARRAY_TYPE_EXT = 32898 	# GL/glext.h:2678
GL_COLOR_ARRAY_STRIDE_EXT = 32899 	# GL/glext.h:2679
GL_COLOR_ARRAY_COUNT_EXT = 32900 	# GL/glext.h:2680
GL_INDEX_ARRAY_TYPE_EXT = 32901 	# GL/glext.h:2681
GL_INDEX_ARRAY_STRIDE_EXT = 32902 	# GL/glext.h:2682
GL_INDEX_ARRAY_COUNT_EXT = 32903 	# GL/glext.h:2683
GL_TEXTURE_COORD_ARRAY_SIZE_EXT = 32904 	# GL/glext.h:2684
GL_TEXTURE_COORD_ARRAY_TYPE_EXT = 32905 	# GL/glext.h:2685
GL_TEXTURE_COORD_ARRAY_STRIDE_EXT = 32906 	# GL/glext.h:2686
GL_TEXTURE_COORD_ARRAY_COUNT_EXT = 32907 	# GL/glext.h:2687
GL_EDGE_FLAG_ARRAY_STRIDE_EXT = 32908 	# GL/glext.h:2688
GL_EDGE_FLAG_ARRAY_COUNT_EXT = 32909 	# GL/glext.h:2689
GL_VERTEX_ARRAY_POINTER_EXT = 32910 	# GL/glext.h:2690
GL_NORMAL_ARRAY_POINTER_EXT = 32911 	# GL/glext.h:2691
GL_COLOR_ARRAY_POINTER_EXT = 32912 	# GL/glext.h:2692
GL_INDEX_ARRAY_POINTER_EXT = 32913 	# GL/glext.h:2693
GL_TEXTURE_COORD_ARRAY_POINTER_EXT = 32914 	# GL/glext.h:2694
GL_EDGE_FLAG_ARRAY_POINTER_EXT = 32915 	# GL/glext.h:2695
# EXT_misc_attribute (GL/glext.h:2698)
# SGIS_generate_mipmap (GL/glext.h:2701)
GL_GENERATE_MIPMAP_SGIS = 33169 	# GL/glext.h:2702
GL_GENERATE_MIPMAP_HINT_SGIS = 33170 	# GL/glext.h:2703
# SGIX_clipmap (GL/glext.h:2706)
GL_LINEAR_CLIPMAP_LINEAR_SGIX = 33136 	# GL/glext.h:2707
GL_TEXTURE_CLIPMAP_CENTER_SGIX = 33137 	# GL/glext.h:2708
GL_TEXTURE_CLIPMAP_FRAME_SGIX = 33138 	# GL/glext.h:2709
GL_TEXTURE_CLIPMAP_OFFSET_SGIX = 33139 	# GL/glext.h:2710
GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33140 	# GL/glext.h:2711
GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX = 33141 	# GL/glext.h:2712
GL_TEXTURE_CLIPMAP_DEPTH_SGIX = 33142 	# GL/glext.h:2713
GL_MAX_CLIPMAP_DEPTH_SGIX = 33143 	# GL/glext.h:2714
GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX = 33144 	# GL/glext.h:2715
GL_NEAREST_CLIPMAP_NEAREST_SGIX = 33869 	# GL/glext.h:2716
GL_NEAREST_CLIPMAP_LINEAR_SGIX = 33870 	# GL/glext.h:2717
GL_LINEAR_CLIPMAP_NEAREST_SGIX = 33871 	# GL/glext.h:2718
# SGIX_shadow (GL/glext.h:2721)
GL_TEXTURE_COMPARE_SGIX = 33178 	# GL/glext.h:2722
GL_TEXTURE_COMPARE_OPERATOR_SGIX = 33179 	# GL/glext.h:2723
GL_TEXTURE_LEQUAL_R_SGIX = 33180 	# GL/glext.h:2724
GL_TEXTURE_GEQUAL_R_SGIX = 33181 	# GL/glext.h:2725
# SGIS_texture_edge_clamp (GL/glext.h:2728)
GL_CLAMP_TO_EDGE_SGIS = 33071 	# GL/glext.h:2729
# SGIS_texture_border_clamp (GL/glext.h:2732)
GL_CLAMP_TO_BORDER_SGIS = 33069 	# GL/glext.h:2733
# EXT_blend_minmax (GL/glext.h:2736)
GL_FUNC_ADD_EXT = 32774 	# GL/glext.h:2737
GL_MIN_EXT = 32775 	# GL/glext.h:2738
GL_MAX_EXT = 32776 	# GL/glext.h:2739
GL_BLEND_EQUATION_EXT = 32777 	# GL/glext.h:2740
# EXT_blend_subtract (GL/glext.h:2743)
GL_FUNC_SUBTRACT_EXT = 32778 	# GL/glext.h:2744
GL_FUNC_REVERSE_SUBTRACT_EXT = 32779 	# GL/glext.h:2745
# EXT_blend_logic_op (GL/glext.h:2748)
# SGIX_interlace (GL/glext.h:2751)
GL_INTERLACE_SGIX = 32916 	# GL/glext.h:2752
# SGIX_pixel_tiles (GL/glext.h:2755)
GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX = 33086 	# GL/glext.h:2756
GL_PIXEL_TILE_CACHE_INCREMENT_SGIX = 33087 	# GL/glext.h:2757
GL_PIXEL_TILE_WIDTH_SGIX = 33088 	# GL/glext.h:2758
GL_PIXEL_TILE_HEIGHT_SGIX = 33089 	# GL/glext.h:2759
GL_PIXEL_TILE_GRID_WIDTH_SGIX = 33090 	# GL/glext.h:2760
GL_PIXEL_TILE_GRID_HEIGHT_SGIX = 33091 	# GL/glext.h:2761
GL_PIXEL_TILE_GRID_DEPTH_SGIX = 33092 	# GL/glext.h:2762
GL_PIXEL_TILE_CACHE_SIZE_SGIX = 33093 	# GL/glext.h:2763
# SGIS_texture_select (GL/glext.h:2766)
GL_DUAL_ALPHA4_SGIS = 33040 	# GL/glext.h:2767
GL_DUAL_ALPHA8_SGIS = 33041 	# GL/glext.h:2768
GL_DUAL_ALPHA12_SGIS = 33042 	# GL/glext.h:2769
GL_DUAL_ALPHA16_SGIS = 33043 	# GL/glext.h:2770
GL_DUAL_LUMINANCE4_SGIS = 33044 	# GL/glext.h:2771
GL_DUAL_LUMINANCE8_SGIS = 33045 	# GL/glext.h:2772
GL_DUAL_LUMINANCE12_SGIS = 33046 	# GL/glext.h:2773
GL_DUAL_LUMINANCE16_SGIS = 33047 	# GL/glext.h:2774
GL_DUAL_INTENSITY4_SGIS = 33048 	# GL/glext.h:2775
GL_DUAL_INTENSITY8_SGIS = 33049 	# GL/glext.h:2776
GL_DUAL_INTENSITY12_SGIS = 33050 	# GL/glext.h:2777
GL_DUAL_INTENSITY16_SGIS = 33051 	# GL/glext.h:2778
GL_DUAL_LUMINANCE_ALPHA4_SGIS = 33052 	# GL/glext.h:2779
GL_DUAL_LUMINANCE_ALPHA8_SGIS = 33053 	# GL/glext.h:2780
GL_QUAD_ALPHA4_SGIS = 33054 	# GL/glext.h:2781
GL_QUAD_ALPHA8_SGIS = 33055 	# GL/glext.h:2782
GL_QUAD_LUMINANCE4_SGIS = 33056 	# GL/glext.h:2783
GL_QUAD_LUMINANCE8_SGIS = 33057 	# GL/glext.h:2784
GL_QUAD_INTENSITY4_SGIS = 33058 	# GL/glext.h:2785
GL_QUAD_INTENSITY8_SGIS = 33059 	# GL/glext.h:2786
GL_DUAL_TEXTURE_SELECT_SGIS = 33060 	# GL/glext.h:2787
GL_QUAD_TEXTURE_SELECT_SGIS = 33061 	# GL/glext.h:2788
# SGIX_sprite (GL/glext.h:2791)
GL_SPRITE_SGIX = 33096 	# GL/glext.h:2792
GL_SPRITE_MODE_SGIX = 33097 	# GL/glext.h:2793
GL_SPRITE_AXIS_SGIX = 33098 	# GL/glext.h:2794
GL_SPRITE_TRANSLATION_SGIX = 33099 	# GL/glext.h:2795
GL_SPRITE_AXIAL_SGIX = 33100 	# GL/glext.h:2796
GL_SPRITE_OBJECT_ALIGNED_SGIX = 33101 	# GL/glext.h:2797
GL_SPRITE_EYE_ALIGNED_SGIX = 33102 	# GL/glext.h:2798
# SGIX_texture_multi_buffer (GL/glext.h:2801)
GL_TEXTURE_MULTI_BUFFER_HINT_SGIX = 33070 	# GL/glext.h:2802
# EXT_point_parameters (GL/glext.h:2805)
GL_POINT_SIZE_MIN_EXT = 33062 	# GL/glext.h:2806
GL_POINT_SIZE_MAX_EXT = 33063 	# GL/glext.h:2807
GL_POINT_FADE_THRESHOLD_SIZE_EXT = 33064 	# GL/glext.h:2808
GL_DISTANCE_ATTENUATION_EXT = 33065 	# GL/glext.h:2809
# SGIS_point_parameters (GL/glext.h:2812)
GL_POINT_SIZE_MIN_SGIS = 33062 	# GL/glext.h:2813
GL_POINT_SIZE_MAX_SGIS = 33063 	# GL/glext.h:2814
GL_POINT_FADE_THRESHOLD_SIZE_SGIS = 33064 	# GL/glext.h:2815
GL_DISTANCE_ATTENUATION_SGIS = 33065 	# GL/glext.h:2816
# SGIX_instruments (GL/glext.h:2819)
GL_INSTRUMENT_BUFFER_POINTER_SGIX = 33152 	# GL/glext.h:2820
GL_INSTRUMENT_MEASUREMENTS_SGIX = 33153 	# GL/glext.h:2821
# SGIX_texture_scale_bias (GL/glext.h:2824)
GL_POST_TEXTURE_FILTER_BIAS_SGIX = 33145 	# GL/glext.h:2825
GL_POST_TEXTURE_FILTER_SCALE_SGIX = 33146 	# GL/glext.h:2826
GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX = 33147 	# GL/glext.h:2827
GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX = 33148 	# GL/glext.h:2828
# SGIX_framezoom (GL/glext.h:2831)
GL_FRAMEZOOM_SGIX = 33163 	# GL/glext.h:2832
GL_FRAMEZOOM_FACTOR_SGIX = 33164 	# GL/glext.h:2833
GL_MAX_FRAMEZOOM_FACTOR_SGIX = 33165 	# GL/glext.h:2834
# SGIX_tag_sample_buffer (GL/glext.h:2837)
# FfdMaskSGIX (GL/glext.h:2840)
GL_TEXTURE_DEFORMATION_BIT_SGIX = 1 	# GL/glext.h:2841
GL_GEOMETRY_DEFORMATION_BIT_SGIX = 2 	# GL/glext.h:2842
# SGIX_polynomial_ffd (GL/glext.h:2845)
GL_GEOMETRY_DEFORMATION_SGIX = 33172 	# GL/glext.h:2846
GL_TEXTURE_DEFORMATION_SGIX = 33173 	# GL/glext.h:2847
GL_DEFORMATIONS_MASK_SGIX = 33174 	# GL/glext.h:2848
GL_MAX_DEFORMATION_ORDER_SGIX = 33175 	# GL/glext.h:2849
# SGIX_reference_plane (GL/glext.h:2852)
GL_REFERENCE_PLANE_SGIX = 33149 	# GL/glext.h:2853
GL_REFERENCE_PLANE_EQUATION_SGIX = 33150 	# GL/glext.h:2854
# SGIX_flush_raster (GL/glext.h:2857)
# SGIX_depth_texture (GL/glext.h:2860)
GL_DEPTH_COMPONENT16_SGIX = 33189 	# GL/glext.h:2861
GL_DEPTH_COMPONENT24_SGIX = 33190 	# GL/glext.h:2862
GL_DEPTH_COMPONENT32_SGIX = 33191 	# GL/glext.h:2863
# SGIS_fog_function (GL/glext.h:2866)
GL_FOG_FUNC_SGIS = 33066 	# GL/glext.h:2867
GL_FOG_FUNC_POINTS_SGIS = 33067 	# GL/glext.h:2868
GL_MAX_FOG_FUNC_POINTS_SGIS = 33068 	# GL/glext.h:2869
# SGIX_fog_offset (GL/glext.h:2872)
GL_FOG_OFFSET_SGIX = 33176 	# GL/glext.h:2873
GL_FOG_OFFSET_VALUE_SGIX = 33177 	# GL/glext.h:2874
# HP_image_transform (GL/glext.h:2877)
GL_IMAGE_SCALE_X_HP = 33109 	# GL/glext.h:2878
GL_IMAGE_SCALE_Y_HP = 33110 	# GL/glext.h:2879
GL_IMAGE_TRANSLATE_X_HP = 33111 	# GL/glext.h:2880
GL_IMAGE_TRANSLATE_Y_HP = 33112 	# GL/glext.h:2881
GL_IMAGE_ROTATE_ANGLE_HP = 33113 	# GL/glext.h:2882
GL_IMAGE_ROTATE_ORIGIN_X_HP = 33114 	# GL/glext.h:2883
GL_IMAGE_ROTATE_ORIGIN_Y_HP = 33115 	# GL/glext.h:2884
GL_IMAGE_MAG_FILTER_HP = 33116 	# GL/glext.h:2885
GL_IMAGE_MIN_FILTER_HP = 33117 	# GL/glext.h:2886
GL_IMAGE_CUBIC_WEIGHT_HP = 33118 	# GL/glext.h:2887
GL_CUBIC_HP = 33119 	# GL/glext.h:2888
GL_AVERAGE_HP = 33120 	# GL/glext.h:2889
GL_IMAGE_TRANSFORM_2D_HP = 33121 	# GL/glext.h:2890
GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33122 	# GL/glext.h:2891
GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP = 33123 	# GL/glext.h:2892
# HP_convolution_border_modes (GL/glext.h:2895)
GL_IGNORE_BORDER_HP = 33104 	# GL/glext.h:2896
GL_CONSTANT_BORDER_HP = 33105 	# GL/glext.h:2897
GL_REPLICATE_BORDER_HP = 33107 	# GL/glext.h:2898
GL_CONVOLUTION_BORDER_COLOR_HP = 33108 	# GL/glext.h:2899
# INGR_palette_buffer (GL/glext.h:2902)
# SGIX_texture_add_env (GL/glext.h:2905)
GL_TEXTURE_ENV_BIAS_SGIX = 32958 	# GL/glext.h:2906
# EXT_color_subtable (GL/glext.h:2909)
# PGI_vertex_hints (GL/glext.h:2912)
GL_VERTEX_DATA_HINT_PGI = 107050 	# GL/glext.h:2913
GL_VERTEX_CONSISTENT_HINT_PGI = 107051 	# GL/glext.h:2914
GL_MATERIAL_SIDE_HINT_PGI = 107052 	# GL/glext.h:2915
GL_MAX_VERTEX_HINT_PGI = 107053 	# GL/glext.h:2916
GL_COLOR3_BIT_PGI = 65536 	# GL/glext.h:2917
GL_COLOR4_BIT_PGI = 131072 	# GL/glext.h:2918
GL_EDGEFLAG_BIT_PGI = 262144 	# GL/glext.h:2919
GL_INDEX_BIT_PGI = 524288 	# GL/glext.h:2920
GL_MAT_AMBIENT_BIT_PGI = 1048576 	# GL/glext.h:2921
GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI = 2097152 	# GL/glext.h:2922
GL_MAT_DIFFUSE_BIT_PGI = 4194304 	# GL/glext.h:2923
GL_MAT_EMISSION_BIT_PGI = 8388608 	# GL/glext.h:2924
GL_MAT_COLOR_INDEXES_BIT_PGI = 16777216 	# GL/glext.h:2925
GL_MAT_SHININESS_BIT_PGI = 33554432 	# GL/glext.h:2926
GL_MAT_SPECULAR_BIT_PGI = 67108864 	# GL/glext.h:2927
GL_NORMAL_BIT_PGI = 134217728 	# GL/glext.h:2928
GL_TEXCOORD1_BIT_PGI = 268435456 	# GL/glext.h:2929
GL_TEXCOORD2_BIT_PGI = 536870912 	# GL/glext.h:2930
GL_TEXCOORD3_BIT_PGI = 1073741824 	# GL/glext.h:2931
GL_TEXCOORD4_BIT_PGI = 2147483648 	# GL/glext.h:2932
GL_VERTEX23_BIT_PGI = 4 	# GL/glext.h:2933
GL_VERTEX4_BIT_PGI = 8 	# GL/glext.h:2934
# PGI_misc_hints (GL/glext.h:2937)
GL_PREFER_DOUBLEBUFFER_HINT_PGI = 107000 	# GL/glext.h:2938
GL_CONSERVE_MEMORY_HINT_PGI = 107005 	# GL/glext.h:2939
GL_RECLAIM_MEMORY_HINT_PGI = 107006 	# GL/glext.h:2940
GL_NATIVE_GRAPHICS_HANDLE_PGI = 107010 	# GL/glext.h:2941
GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI = 107011 	# GL/glext.h:2942
GL_NATIVE_GRAPHICS_END_HINT_PGI = 107012 	# GL/glext.h:2943
GL_ALWAYS_FAST_HINT_PGI = 107020 	# GL/glext.h:2944
GL_ALWAYS_SOFT_HINT_PGI = 107021 	# GL/glext.h:2945
GL_ALLOW_DRAW_OBJ_HINT_PGI = 107022 	# GL/glext.h:2946
GL_ALLOW_DRAW_WIN_HINT_PGI = 107023 	# GL/glext.h:2947
GL_ALLOW_DRAW_FRG_HINT_PGI = 107024 	# GL/glext.h:2948
GL_ALLOW_DRAW_MEM_HINT_PGI = 107025 	# GL/glext.h:2949
GL_STRICT_DEPTHFUNC_HINT_PGI = 107030 	# GL/glext.h:2950
GL_STRICT_LIGHTING_HINT_PGI = 107031 	# GL/glext.h:2951
GL_STRICT_SCISSOR_HINT_PGI = 107032 	# GL/glext.h:2952
GL_FULL_STIPPLE_HINT_PGI = 107033 	# GL/glext.h:2953
GL_CLIP_NEAR_HINT_PGI = 107040 	# GL/glext.h:2954
GL_CLIP_FAR_HINT_PGI = 107041 	# GL/glext.h:2955
GL_WIDE_LINE_HINT_PGI = 107042 	# GL/glext.h:2956
GL_BACK_NORMALS_HINT_PGI = 107043 	# GL/glext.h:2957
# EXT_paletted_texture (GL/glext.h:2960)
GL_COLOR_INDEX1_EXT = 32994 	# GL/glext.h:2961
GL_COLOR_INDEX2_EXT = 32995 	# GL/glext.h:2962
GL_COLOR_INDEX4_EXT = 32996 	# GL/glext.h:2963
GL_COLOR_INDEX8_EXT = 32997 	# GL/glext.h:2964
GL_COLOR_INDEX12_EXT = 32998 	# GL/glext.h:2965
GL_COLOR_INDEX16_EXT = 32999 	# GL/glext.h:2966
GL_TEXTURE_INDEX_SIZE_EXT = 33005 	# GL/glext.h:2967
# EXT_clip_volume_hint (GL/glext.h:2970)
GL_CLIP_VOLUME_CLIPPING_HINT_EXT = 33008 	# GL/glext.h:2971
# SGIX_list_priority (GL/glext.h:2974)
GL_LIST_PRIORITY_SGIX = 33154 	# GL/glext.h:2975
# SGIX_ir_instrument1 (GL/glext.h:2978)
GL_IR_INSTRUMENT1_SGIX = 33151 	# GL/glext.h:2979
# SGIX_calligraphic_fragment (GL/glext.h:2982)
GL_CALLIGRAPHIC_FRAGMENT_SGIX = 33155 	# GL/glext.h:2983
# SGIX_texture_lod_bias (GL/glext.h:2986)
GL_TEXTURE_LOD_BIAS_S_SGIX = 33166 	# GL/glext.h:2987
GL_TEXTURE_LOD_BIAS_T_SGIX = 33167 	# GL/glext.h:2988
GL_TEXTURE_LOD_BIAS_R_SGIX = 33168 	# GL/glext.h:2989
# SGIX_shadow_ambient (GL/glext.h:2992)
GL_SHADOW_AMBIENT_SGIX = 32959 	# GL/glext.h:2993
# EXT_index_texture (GL/glext.h:2996)
# EXT_index_material (GL/glext.h:2999)
GL_INDEX_MATERIAL_EXT = 33208 	# GL/glext.h:3000
GL_INDEX_MATERIAL_PARAMETER_EXT = 33209 	# GL/glext.h:3001
GL_INDEX_MATERIAL_FACE_EXT = 33210 	# GL/glext.h:3002
# EXT_index_func (GL/glext.h:3005)
GL_INDEX_TEST_EXT = 33205 	# GL/glext.h:3006
GL_INDEX_TEST_FUNC_EXT = 33206 	# GL/glext.h:3007
GL_INDEX_TEST_REF_EXT = 33207 	# GL/glext.h:3008
# EXT_index_array_formats (GL/glext.h:3011)
GL_IUI_V2F_EXT = 33197 	# GL/glext.h:3012
GL_IUI_V3F_EXT = 33198 	# GL/glext.h:3013
GL_IUI_N3F_V2F_EXT = 33199 	# GL/glext.h:3014
GL_IUI_N3F_V3F_EXT = 33200 	# GL/glext.h:3015
GL_T2F_IUI_V2F_EXT = 33201 	# GL/glext.h:3016
GL_T2F_IUI_V3F_EXT = 33202 	# GL/glext.h:3017
GL_T2F_IUI_N3F_V2F_EXT = 33203 	# GL/glext.h:3018
GL_T2F_IUI_N3F_V3F_EXT = 33204 	# GL/glext.h:3019
# EXT_compiled_vertex_array (GL/glext.h:3022)
GL_ARRAY_ELEMENT_LOCK_FIRST_EXT = 33192 	# GL/glext.h:3023
GL_ARRAY_ELEMENT_LOCK_COUNT_EXT = 33193 	# GL/glext.h:3024
# EXT_cull_vertex (GL/glext.h:3027)
GL_CULL_VERTEX_EXT = 33194 	# GL/glext.h:3028
GL_CULL_VERTEX_EYE_POSITION_EXT = 33195 	# GL/glext.h:3029
GL_CULL_VERTEX_OBJECT_POSITION_EXT = 33196 	# GL/glext.h:3030
# SGIX_ycrcb (GL/glext.h:3033)
GL_YCRCB_422_SGIX = 33211 	# GL/glext.h:3034
GL_YCRCB_444_SGIX = 33212 	# GL/glext.h:3035
# SGIX_fragment_lighting (GL/glext.h:3038)
GL_FRAGMENT_LIGHTING_SGIX = 33792 	# GL/glext.h:3039
GL_FRAGMENT_COLOR_MATERIAL_SGIX = 33793 	# GL/glext.h:3040
GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX = 33794 	# GL/glext.h:3041
GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX = 33795 	# GL/glext.h:3042
GL_MAX_FRAGMENT_LIGHTS_SGIX = 33796 	# GL/glext.h:3043
GL_MAX_ACTIVE_LIGHTS_SGIX = 33797 	# GL/glext.h:3044
GL_CURRENT_RASTER_NORMAL_SGIX = 33798 	# GL/glext.h:3045
GL_LIGHT_ENV_MODE_SGIX = 33799 	# GL/glext.h:3046
GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX = 33800 	# GL/glext.h:3047
GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX = 33801 	# GL/glext.h:3048
GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX = 33802 	# GL/glext.h:3049
GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX = 33803 	# GL/glext.h:3050
GL_FRAGMENT_LIGHT0_SGIX = 33804 	# GL/glext.h:3051
GL_FRAGMENT_LIGHT1_SGIX = 33805 	# GL/glext.h:3052
GL_FRAGMENT_LIGHT2_SGIX = 33806 	# GL/glext.h:3053
GL_FRAGMENT_LIGHT3_SGIX = 33807 	# GL/glext.h:3054
GL_FRAGMENT_LIGHT4_SGIX = 33808 	# GL/glext.h:3055
GL_FRAGMENT_LIGHT5_SGIX = 33809 	# GL/glext.h:3056
GL_FRAGMENT_LIGHT6_SGIX = 33810 	# GL/glext.h:3057
GL_FRAGMENT_LIGHT7_SGIX = 33811 	# GL/glext.h:3058
# IBM_rasterpos_clip (GL/glext.h:3061)
GL_RASTER_POSITION_UNCLIPPED_IBM = 103010 	# GL/glext.h:3062
# HP_texture_lighting (GL/glext.h:3065)
GL_TEXTURE_LIGHTING_MODE_HP = 33127 	# GL/glext.h:3066
GL_TEXTURE_POST_SPECULAR_HP = 33128 	# GL/glext.h:3067
GL_TEXTURE_PRE_SPECULAR_HP = 33129 	# GL/glext.h:3068
# EXT_draw_range_elements (GL/glext.h:3071)
GL_MAX_ELEMENTS_VERTICES_EXT = 33000 	# GL/glext.h:3072
GL_MAX_ELEMENTS_INDICES_EXT = 33001 	# GL/glext.h:3073
# WIN_phong_shading (GL/glext.h:3076)
GL_PHONG_WIN = 33002 	# GL/glext.h:3077
GL_PHONG_HINT_WIN = 33003 	# GL/glext.h:3078
# WIN_specular_fog (GL/glext.h:3081)
GL_FOG_SPECULAR_TEXTURE_WIN = 33004 	# GL/glext.h:3082
# EXT_light_texture (GL/glext.h:3085)
GL_FRAGMENT_MATERIAL_EXT = 33609 	# GL/glext.h:3086
GL_FRAGMENT_NORMAL_EXT = 33610 	# GL/glext.h:3087
GL_FRAGMENT_COLOR_EXT = 33612 	# GL/glext.h:3088
GL_ATTENUATION_EXT = 33613 	# GL/glext.h:3089
GL_SHADOW_ATTENUATION_EXT = 33614 	# GL/glext.h:3090
GL_TEXTURE_APPLICATION_MODE_EXT = 33615 	# GL/glext.h:3091
GL_TEXTURE_LIGHT_EXT = 33616 	# GL/glext.h:3092
GL_TEXTURE_MATERIAL_FACE_EXT = 33617 	# GL/glext.h:3093
GL_TEXTURE_MATERIAL_PARAMETER_EXT = 33618 	# GL/glext.h:3094
# SGIX_blend_alpha_minmax (GL/glext.h:3098)
GL_ALPHA_MIN_SGIX = 33568 	# GL/glext.h:3099
GL_ALPHA_MAX_SGIX = 33569 	# GL/glext.h:3100
# SGIX_impact_pixel_texture (GL/glext.h:3103)
GL_PIXEL_TEX_GEN_Q_CEILING_SGIX = 33156 	# GL/glext.h:3104
GL_PIXEL_TEX_GEN_Q_ROUND_SGIX = 33157 	# GL/glext.h:3105
GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX = 33158 	# GL/glext.h:3106
GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX = 33159 	# GL/glext.h:3107
GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX = 33160 	# GL/glext.h:3108
GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX = 33161 	# GL/glext.h:3109
GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX = 33162 	# GL/glext.h:3110
# EXT_bgra (GL/glext.h:3113)
GL_BGR_EXT = 32992 	# GL/glext.h:3114
GL_BGRA_EXT = 32993 	# GL/glext.h:3115
# SGIX_async (GL/glext.h:3118)
GL_ASYNC_MARKER_SGIX = 33577 	# GL/glext.h:3119
# SGIX_async_pixel (GL/glext.h:3122)
GL_ASYNC_TEX_IMAGE_SGIX = 33628 	# GL/glext.h:3123
GL_ASYNC_DRAW_PIXELS_SGIX = 33629 	# GL/glext.h:3124
GL_ASYNC_READ_PIXELS_SGIX = 33630 	# GL/glext.h:3125
GL_MAX_ASYNC_TEX_IMAGE_SGIX = 33631 	# GL/glext.h:3126
GL_MAX_ASYNC_DRAW_PIXELS_SGIX = 33632 	# GL/glext.h:3127
GL_MAX_ASYNC_READ_PIXELS_SGIX = 33633 	# GL/glext.h:3128
# SGIX_async_histogram (GL/glext.h:3131)
GL_ASYNC_HISTOGRAM_SGIX = 33580 	# GL/glext.h:3132
GL_MAX_ASYNC_HISTOGRAM_SGIX = 33581 	# GL/glext.h:3133
# INTEL_texture_scissor (GL/glext.h:3136)
# INTEL_parallel_arrays (GL/glext.h:3139)
GL_PARALLEL_ARRAYS_INTEL = 33780 	# GL/glext.h:3140
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL = 33781 	# GL/glext.h:3141
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL = 33782 	# GL/glext.h:3142
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL = 33783 	# GL/glext.h:3143
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL = 33784 	# GL/glext.h:3144
# HP_occlusion_test (GL/glext.h:3147)
GL_OCCLUSION_TEST_HP = 33125 	# GL/glext.h:3148
GL_OCCLUSION_TEST_RESULT_HP = 33126 	# GL/glext.h:3149
# EXT_pixel_transform (GL/glext.h:3152)
GL_PIXEL_TRANSFORM_2D_EXT = 33584 	# GL/glext.h:3153
GL_PIXEL_MAG_FILTER_EXT = 33585 	# GL/glext.h:3154
GL_PIXEL_MIN_FILTER_EXT = 33586 	# GL/glext.h:3155
GL_PIXEL_CUBIC_WEIGHT_EXT = 33587 	# GL/glext.h:3156
GL_CUBIC_EXT = 33588 	# GL/glext.h:3157
GL_AVERAGE_EXT = 33589 	# GL/glext.h:3158
GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33590 	# GL/glext.h:3159
GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT = 33591 	# GL/glext.h:3160
GL_PIXEL_TRANSFORM_2D_MATRIX_EXT = 33592 	# GL/glext.h:3161
# EXT_pixel_transform_color_table (GL/glext.h:3164)
# EXT_shared_texture_palette (GL/glext.h:3167)
GL_SHARED_TEXTURE_PALETTE_EXT = 33275 	# GL/glext.h:3168
# EXT_separate_specular_color (GL/glext.h:3171)
GL_LIGHT_MODEL_COLOR_CONTROL_EXT = 33272 	# GL/glext.h:3172
GL_SINGLE_COLOR_EXT = 33273 	# GL/glext.h:3173
GL_SEPARATE_SPECULAR_COLOR_EXT = 33274 	# GL/glext.h:3174
# EXT_secondary_color (GL/glext.h:3177)
GL_COLOR_SUM_EXT = 33880 	# GL/glext.h:3178
GL_CURRENT_SECONDARY_COLOR_EXT = 33881 	# GL/glext.h:3179
GL_SECONDARY_COLOR_ARRAY_SIZE_EXT = 33882 	# GL/glext.h:3180
GL_SECONDARY_COLOR_ARRAY_TYPE_EXT = 33883 	# GL/glext.h:3181
GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT = 33884 	# GL/glext.h:3182
GL_SECONDARY_COLOR_ARRAY_POINTER_EXT = 33885 	# GL/glext.h:3183
GL_SECONDARY_COLOR_ARRAY_EXT = 33886 	# GL/glext.h:3184
# EXT_texture_perturb_normal (GL/glext.h:3187)
GL_PERTURB_EXT = 34222 	# GL/glext.h:3188
GL_TEXTURE_NORMAL_EXT = 34223 	# GL/glext.h:3189
# EXT_multi_draw_arrays (GL/glext.h:3192)
# EXT_fog_coord (GL/glext.h:3195)
GL_FOG_COORDINATE_SOURCE_EXT = 33872 	# GL/glext.h:3196
GL_FOG_COORDINATE_EXT = 33873 	# GL/glext.h:3197
GL_FRAGMENT_DEPTH_EXT = 33874 	# GL/glext.h:3198
GL_CURRENT_FOG_COORDINATE_EXT = 33875 	# GL/glext.h:3199
GL_FOG_COORDINATE_ARRAY_TYPE_EXT = 33876 	# GL/glext.h:3200
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT = 33877 	# GL/glext.h:3201
GL_FOG_COORDINATE_ARRAY_POINTER_EXT = 33878 	# GL/glext.h:3202
GL_FOG_COORDINATE_ARRAY_EXT = 33879 	# GL/glext.h:3203
# REND_screen_coordinates (GL/glext.h:3206)
GL_SCREEN_COORDINATES_REND = 33936 	# GL/glext.h:3207
GL_INVERTED_SCREEN_W_REND = 33937 	# GL/glext.h:3208
# EXT_coordinate_frame (GL/glext.h:3211)
GL_TANGENT_ARRAY_EXT = 33849 	# GL/glext.h:3212
GL_BINORMAL_ARRAY_EXT = 33850 	# GL/glext.h:3213
GL_CURRENT_TANGENT_EXT = 33851 	# GL/glext.h:3214
GL_CURRENT_BINORMAL_EXT = 33852 	# GL/glext.h:3215
GL_TANGENT_ARRAY_TYPE_EXT = 33854 	# GL/glext.h:3216
GL_TANGENT_ARRAY_STRIDE_EXT = 33855 	# GL/glext.h:3217
GL_BINORMAL_ARRAY_TYPE_EXT = 33856 	# GL/glext.h:3218
GL_BINORMAL_ARRAY_STRIDE_EXT = 33857 	# GL/glext.h:3219
GL_TANGENT_ARRAY_POINTER_EXT = 33858 	# GL/glext.h:3220
GL_BINORMAL_ARRAY_POINTER_EXT = 33859 	# GL/glext.h:3221
GL_MAP1_TANGENT_EXT = 33860 	# GL/glext.h:3222
GL_MAP2_TANGENT_EXT = 33861 	# GL/glext.h:3223
GL_MAP1_BINORMAL_EXT = 33862 	# GL/glext.h:3224
GL_MAP2_BINORMAL_EXT = 33863 	# GL/glext.h:3225
# EXT_texture_env_combine (GL/glext.h:3228)
GL_COMBINE_EXT = 34160 	# GL/glext.h:3229
GL_COMBINE_RGB_EXT = 34161 	# GL/glext.h:3230
GL_COMBINE_ALPHA_EXT = 34162 	# GL/glext.h:3231
GL_RGB_SCALE_EXT = 34163 	# GL/glext.h:3232
GL_ADD_SIGNED_EXT = 34164 	# GL/glext.h:3233
GL_INTERPOLATE_EXT = 34165 	# GL/glext.h:3234
GL_CONSTANT_EXT = 34166 	# GL/glext.h:3235
GL_PRIMARY_COLOR_EXT = 34167 	# GL/glext.h:3236
GL_PREVIOUS_EXT = 34168 	# GL/glext.h:3237
GL_SOURCE0_RGB_EXT = 34176 	# GL/glext.h:3238
GL_SOURCE1_RGB_EXT = 34177 	# GL/glext.h:3239
GL_SOURCE2_RGB_EXT = 34178 	# GL/glext.h:3240
GL_SOURCE0_ALPHA_EXT = 34184 	# GL/glext.h:3241
GL_SOURCE1_ALPHA_EXT = 34185 	# GL/glext.h:3242
GL_SOURCE2_ALPHA_EXT = 34186 	# GL/glext.h:3243
GL_OPERAND0_RGB_EXT = 34192 	# GL/glext.h:3244
GL_OPERAND1_RGB_EXT = 34193 	# GL/glext.h:3245
GL_OPERAND2_RGB_EXT = 34194 	# GL/glext.h:3246
GL_OPERAND0_ALPHA_EXT = 34200 	# GL/glext.h:3247
GL_OPERAND1_ALPHA_EXT = 34201 	# GL/glext.h:3248
GL_OPERAND2_ALPHA_EXT = 34202 	# GL/glext.h:3249
# APPLE_specular_vector (GL/glext.h:3252)
GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE = 34224 	# GL/glext.h:3253
# APPLE_transform_hint (GL/glext.h:3256)
GL_TRANSFORM_HINT_APPLE = 34225 	# GL/glext.h:3257
# SGIX_fog_scale (GL/glext.h:3260)
GL_FOG_SCALE_SGIX = 33276 	# GL/glext.h:3261
GL_FOG_SCALE_VALUE_SGIX = 33277 	# GL/glext.h:3262
# SUNX_constant_data (GL/glext.h:3265)
GL_UNPACK_CONSTANT_DATA_SUNX = 33237 	# GL/glext.h:3266
GL_TEXTURE_CONSTANT_DATA_SUNX = 33238 	# GL/glext.h:3267
# SUN_global_alpha (GL/glext.h:3270)
GL_GLOBAL_ALPHA_SUN = 33241 	# GL/glext.h:3271
GL_GLOBAL_ALPHA_FACTOR_SUN = 33242 	# GL/glext.h:3272
# SUN_triangle_list (GL/glext.h:3275)
GL_RESTART_SUN = 1 	# GL/glext.h:3276
GL_REPLACE_MIDDLE_SUN = 2 	# GL/glext.h:3277
GL_REPLACE_OLDEST_SUN = 3 	# GL/glext.h:3278
GL_TRIANGLE_LIST_SUN = 33239 	# GL/glext.h:3279
GL_REPLACEMENT_CODE_SUN = 33240 	# GL/glext.h:3280
GL_REPLACEMENT_CODE_ARRAY_SUN = 34240 	# GL/glext.h:3281
GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN = 34241 	# GL/glext.h:3282
GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN = 34242 	# GL/glext.h:3283
GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN = 34243 	# GL/glext.h:3284
GL_R1UI_V3F_SUN = 34244 	# GL/glext.h:3285
GL_R1UI_C4UB_V3F_SUN = 34245 	# GL/glext.h:3286
GL_R1UI_C3F_V3F_SUN = 34246 	# GL/glext.h:3287
GL_R1UI_N3F_V3F_SUN = 34247 	# GL/glext.h:3288
GL_R1UI_C4F_N3F_V3F_SUN = 34248 	# GL/glext.h:3289
GL_R1UI_T2F_V3F_SUN = 34249 	# GL/glext.h:3290
GL_R1UI_T2F_N3F_V3F_SUN = 34250 	# GL/glext.h:3291
GL_R1UI_T2F_C4F_N3F_V3F_SUN = 34251 	# GL/glext.h:3292
# SUN_vertex (GL/glext.h:3295)
# EXT_blend_func_separate (GL/glext.h:3298)
GL_BLEND_DST_RGB_EXT = 32968 	# GL/glext.h:3299
GL_BLEND_SRC_RGB_EXT = 32969 	# GL/glext.h:3300
GL_BLEND_DST_ALPHA_EXT = 32970 	# GL/glext.h:3301
GL_BLEND_SRC_ALPHA_EXT = 32971 	# GL/glext.h:3302
# INGR_color_clamp (GL/glext.h:3305)
GL_RED_MIN_CLAMP_INGR = 34144 	# GL/glext.h:3306
GL_GREEN_MIN_CLAMP_INGR = 34145 	# GL/glext.h:3307
GL_BLUE_MIN_CLAMP_INGR = 34146 	# GL/glext.h:3308
GL_ALPHA_MIN_CLAMP_INGR = 34147 	# GL/glext.h:3309
GL_RED_MAX_CLAMP_INGR = 34148 	# GL/glext.h:3310
GL_GREEN_MAX_CLAMP_INGR = 34149 	# GL/glext.h:3311
GL_BLUE_MAX_CLAMP_INGR = 34150 	# GL/glext.h:3312
GL_ALPHA_MAX_CLAMP_INGR = 34151 	# GL/glext.h:3313
# INGR_interlace_read (GL/glext.h:3316)
GL_INTERLACE_READ_INGR = 34152 	# GL/glext.h:3317
# EXT_stencil_wrap (GL/glext.h:3320)
GL_INCR_WRAP_EXT = 34055 	# GL/glext.h:3321
GL_DECR_WRAP_EXT = 34056 	# GL/glext.h:3322
# EXT_422_pixels (GL/glext.h:3325)
GL_422_EXT = 32972 	# GL/glext.h:3326
GL_422_REV_EXT = 32973 	# GL/glext.h:3327
GL_422_AVERAGE_EXT = 32974 	# GL/glext.h:3328
GL_422_REV_AVERAGE_EXT = 32975 	# GL/glext.h:3329
# NV_texgen_reflection (GL/glext.h:3332)
GL_NORMAL_MAP_NV = 34065 	# GL/glext.h:3333
GL_REFLECTION_MAP_NV = 34066 	# GL/glext.h:3334
# EXT_texture_cube_map (GL/glext.h:3337)
GL_NORMAL_MAP_EXT = 34065 	# GL/glext.h:3338
GL_REFLECTION_MAP_EXT = 34066 	# GL/glext.h:3339
GL_TEXTURE_CUBE_MAP_EXT = 34067 	# GL/glext.h:3340
GL_TEXTURE_BINDING_CUBE_MAP_EXT = 34068 	# GL/glext.h:3341
GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT = 34069 	# GL/glext.h:3342
GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT = 34070 	# GL/glext.h:3343
GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT = 34071 	# GL/glext.h:3344
GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT = 34072 	# GL/glext.h:3345
GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT = 34073 	# GL/glext.h:3346
GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT = 34074 	# GL/glext.h:3347
GL_PROXY_TEXTURE_CUBE_MAP_EXT = 34075 	# GL/glext.h:3348
GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT = 34076 	# GL/glext.h:3349
# SUN_convolution_border_modes (GL/glext.h:3352)
GL_WRAP_BORDER_SUN = 33236 	# GL/glext.h:3353
# EXT_texture_env_add (GL/glext.h:3356)
# EXT_texture_lod_bias (GL/glext.h:3359)
GL_MAX_TEXTURE_LOD_BIAS_EXT = 34045 	# GL/glext.h:3360
GL_TEXTURE_FILTER_CONTROL_EXT = 34048 	# GL/glext.h:3361
GL_TEXTURE_LOD_BIAS_EXT = 34049 	# GL/glext.h:3362
# EXT_texture_filter_anisotropic (GL/glext.h:3365)
GL_TEXTURE_MAX_ANISOTROPY_EXT = 34046 	# GL/glext.h:3366
GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT = 34047 	# GL/glext.h:3367
# EXT_vertex_weighting (GL/glext.h:3370)
GL_MODELVIEW0_STACK_DEPTH_EXT = 2979 	# GL/glext.h:3371
GL_MODELVIEW1_STACK_DEPTH_EXT = 34050 	# GL/glext.h:3372
GL_MODELVIEW0_MATRIX_EXT = 2982 	# GL/glext.h:3373
GL_MODELVIEW1_MATRIX_EXT = 34054 	# GL/glext.h:3374
GL_VERTEX_WEIGHTING_EXT = 34057 	# GL/glext.h:3375
GL_MODELVIEW0_EXT = 5888 	# GL/glext.h:3376
GL_MODELVIEW1_EXT = 34058 	# GL/glext.h:3377
GL_CURRENT_VERTEX_WEIGHT_EXT = 34059 	# GL/glext.h:3378
GL_VERTEX_WEIGHT_ARRAY_EXT = 34060 	# GL/glext.h:3379
GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT = 34061 	# GL/glext.h:3380
GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT = 34062 	# GL/glext.h:3381
GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT = 34063 	# GL/glext.h:3382
GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT = 34064 	# GL/glext.h:3383
# NV_light_max_exponent (GL/glext.h:3386)
GL_MAX_SHININESS_NV = 34052 	# GL/glext.h:3387
GL_MAX_SPOT_EXPONENT_NV = 34053 	# GL/glext.h:3388
# NV_vertex_array_range (GL/glext.h:3391)
GL_VERTEX_ARRAY_RANGE_NV = 34077 	# GL/glext.h:3392
GL_VERTEX_ARRAY_RANGE_LENGTH_NV = 34078 	# GL/glext.h:3393
GL_VERTEX_ARRAY_RANGE_VALID_NV = 34079 	# GL/glext.h:3394
GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV = 34080 	# GL/glext.h:3395
GL_VERTEX_ARRAY_RANGE_POINTER_NV = 34081 	# GL/glext.h:3396
# NV_register_combiners (GL/glext.h:3399)
GL_REGISTER_COMBINERS_NV = 34082 	# GL/glext.h:3400
GL_VARIABLE_A_NV = 34083 	# GL/glext.h:3401
GL_VARIABLE_B_NV = 34084 	# GL/glext.h:3402
GL_VARIABLE_C_NV = 34085 	# GL/glext.h:3403
GL_VARIABLE_D_NV = 34086 	# GL/glext.h:3404
GL_VARIABLE_E_NV = 34087 	# GL/glext.h:3405
GL_VARIABLE_F_NV = 34088 	# GL/glext.h:3406
GL_VARIABLE_G_NV = 34089 	# GL/glext.h:3407
GL_CONSTANT_COLOR0_NV = 34090 	# GL/glext.h:3408
GL_CONSTANT_COLOR1_NV = 34091 	# GL/glext.h:3409
GL_PRIMARY_COLOR_NV = 34092 	# GL/glext.h:3410
GL_SECONDARY_COLOR_NV = 34093 	# GL/glext.h:3411
GL_SPARE0_NV = 34094 	# GL/glext.h:3412
GL_SPARE1_NV = 34095 	# GL/glext.h:3413
GL_DISCARD_NV = 34096 	# GL/glext.h:3414
GL_E_TIMES_F_NV = 34097 	# GL/glext.h:3415
GL_SPARE0_PLUS_SECONDARY_COLOR_NV = 34098 	# GL/glext.h:3416
GL_UNSIGNED_IDENTITY_NV = 34102 	# GL/glext.h:3417
GL_UNSIGNED_INVERT_NV = 34103 	# GL/glext.h:3418
GL_EXPAND_NORMAL_NV = 34104 	# GL/glext.h:3419
GL_EXPAND_NEGATE_NV = 34105 	# GL/glext.h:3420
GL_HALF_BIAS_NORMAL_NV = 34106 	# GL/glext.h:3421
GL_HALF_BIAS_NEGATE_NV = 34107 	# GL/glext.h:3422
GL_SIGNED_IDENTITY_NV = 34108 	# GL/glext.h:3423
GL_SIGNED_NEGATE_NV = 34109 	# GL/glext.h:3424
GL_SCALE_BY_TWO_NV = 34110 	# GL/glext.h:3425
GL_SCALE_BY_FOUR_NV = 34111 	# GL/glext.h:3426
GL_SCALE_BY_ONE_HALF_NV = 34112 	# GL/glext.h:3427
GL_BIAS_BY_NEGATIVE_ONE_HALF_NV = 34113 	# GL/glext.h:3428
GL_COMBINER_INPUT_NV = 34114 	# GL/glext.h:3429
GL_COMBINER_MAPPING_NV = 34115 	# GL/glext.h:3430
GL_COMBINER_COMPONENT_USAGE_NV = 34116 	# GL/glext.h:3431
GL_COMBINER_AB_DOT_PRODUCT_NV = 34117 	# GL/glext.h:3432
GL_COMBINER_CD_DOT_PRODUCT_NV = 34118 	# GL/glext.h:3433
GL_COMBINER_MUX_SUM_NV = 34119 	# GL/glext.h:3434
GL_COMBINER_SCALE_NV = 34120 	# GL/glext.h:3435
GL_COMBINER_BIAS_NV = 34121 	# GL/glext.h:3436
GL_COMBINER_AB_OUTPUT_NV = 34122 	# GL/glext.h:3437
GL_COMBINER_CD_OUTPUT_NV = 34123 	# GL/glext.h:3438
GL_COMBINER_SUM_OUTPUT_NV = 34124 	# GL/glext.h:3439
GL_MAX_GENERAL_COMBINERS_NV = 34125 	# GL/glext.h:3440
GL_NUM_GENERAL_COMBINERS_NV = 34126 	# GL/glext.h:3441
GL_COLOR_SUM_CLAMP_NV = 34127 	# GL/glext.h:3442
GL_COMBINER0_NV = 34128 	# GL/glext.h:3443
GL_COMBINER1_NV = 34129 	# GL/glext.h:3444
GL_COMBINER2_NV = 34130 	# GL/glext.h:3445
GL_COMBINER3_NV = 34131 	# GL/glext.h:3446
GL_COMBINER4_NV = 34132 	# GL/glext.h:3447
GL_COMBINER5_NV = 34133 	# GL/glext.h:3448
GL_COMBINER6_NV = 34134 	# GL/glext.h:3449
GL_COMBINER7_NV = 34135 	# GL/glext.h:3450
# NV_fog_distance (GL/glext.h:3458)
GL_FOG_DISTANCE_MODE_NV = 34138 	# GL/glext.h:3459
GL_EYE_RADIAL_NV = 34139 	# GL/glext.h:3460
GL_EYE_PLANE_ABSOLUTE_NV = 34140 	# GL/glext.h:3461
# NV_texgen_emboss (GL/glext.h:3465)
GL_EMBOSS_LIGHT_NV = 34141 	# GL/glext.h:3466
GL_EMBOSS_CONSTANT_NV = 34142 	# GL/glext.h:3467
GL_EMBOSS_MAP_NV = 34143 	# GL/glext.h:3468
# NV_blend_square (GL/glext.h:3471)
# NV_texture_env_combine4 (GL/glext.h:3474)
GL_COMBINE4_NV = 34051 	# GL/glext.h:3475
GL_SOURCE3_RGB_NV = 34179 	# GL/glext.h:3476
GL_SOURCE3_ALPHA_NV = 34187 	# GL/glext.h:3477
GL_OPERAND3_RGB_NV = 34195 	# GL/glext.h:3478
GL_OPERAND3_ALPHA_NV = 34203 	# GL/glext.h:3479
# MESA_resize_buffers (GL/glext.h:3482)
# MESA_window_pos (GL/glext.h:3485)
# EXT_texture_compression_s3tc (GL/glext.h:3488)
GL_COMPRESSED_RGB_S3TC_DXT1_EXT = 33776 	# GL/glext.h:3489
GL_COMPRESSED_RGBA_S3TC_DXT1_EXT = 33777 	# GL/glext.h:3490
GL_COMPRESSED_RGBA_S3TC_DXT3_EXT = 33778 	# GL/glext.h:3491
GL_COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779 	# GL/glext.h:3492
# IBM_cull_vertex (GL/glext.h:3495)
GL_CULL_VERTEX_IBM = 103050 	# GL/glext.h:3496
# IBM_multimode_draw_arrays (GL/glext.h:3499)
# IBM_vertex_array_lists (GL/glext.h:3502)
GL_VERTEX_ARRAY_LIST_IBM = 103070 	# GL/glext.h:3503
GL_NORMAL_ARRAY_LIST_IBM = 103071 	# GL/glext.h:3504
GL_COLOR_ARRAY_LIST_IBM = 103072 	# GL/glext.h:3505
GL_INDEX_ARRAY_LIST_IBM = 103073 	# GL/glext.h:3506
GL_TEXTURE_COORD_ARRAY_LIST_IBM = 103074 	# GL/glext.h:3507
GL_EDGE_FLAG_ARRAY_LIST_IBM = 103075 	# GL/glext.h:3508
GL_FOG_COORDINATE_ARRAY_LIST_IBM = 103076 	# GL/glext.h:3509
GL_SECONDARY_COLOR_ARRAY_LIST_IBM = 103077 	# GL/glext.h:3510
GL_VERTEX_ARRAY_LIST_STRIDE_IBM = 103080 	# GL/glext.h:3511
GL_NORMAL_ARRAY_LIST_STRIDE_IBM = 103081 	# GL/glext.h:3512
GL_COLOR_ARRAY_LIST_STRIDE_IBM = 103082 	# GL/glext.h:3513
GL_INDEX_ARRAY_LIST_STRIDE_IBM = 103083 	# GL/glext.h:3514
GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM = 103084 	# GL/glext.h:3515
GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM = 103085 	# GL/glext.h:3516
GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM = 103086 	# GL/glext.h:3517
GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM = 103087 	# GL/glext.h:3518
# SGIX_subsample (GL/glext.h:3521)
GL_PACK_SUBSAMPLE_RATE_SGIX = 34208 	# GL/glext.h:3522
GL_UNPACK_SUBSAMPLE_RATE_SGIX = 34209 	# GL/glext.h:3523
GL_PIXEL_SUBSAMPLE_4444_SGIX = 34210 	# GL/glext.h:3524
GL_PIXEL_SUBSAMPLE_2424_SGIX = 34211 	# GL/glext.h:3525
GL_PIXEL_SUBSAMPLE_4242_SGIX = 34212 	# GL/glext.h:3526
# SGIX_ycrcb_subsample (GL/glext.h:3529)
# SGIX_ycrcba (GL/glext.h:3532)
GL_YCRCB_SGIX = 33560 	# GL/glext.h:3533
GL_YCRCBA_SGIX = 33561 	# GL/glext.h:3534
# SGI_depth_pass_instrument (GL/glext.h:3537)
GL_DEPTH_PASS_INSTRUMENT_SGIX = 33552 	# GL/glext.h:3538
GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX = 33553 	# GL/glext.h:3539
GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX = 33554 	# GL/glext.h:3540
# 3DFX_texture_compression_FXT1 (GL/glext.h:3543)
GL_COMPRESSED_RGB_FXT1_3DFX = 34480 	# GL/glext.h:3544
GL_COMPRESSED_RGBA_FXT1_3DFX = 34481 	# GL/glext.h:3545
# 3DFX_multisample (GL/glext.h:3548)
GL_MULTISAMPLE_3DFX = 34482 	# GL/glext.h:3549
GL_SAMPLE_BUFFERS_3DFX = 34483 	# GL/glext.h:3550
GL_SAMPLES_3DFX = 34484 	# GL/glext.h:3551
GL_MULTISAMPLE_BIT_3DFX = 536870912 	# GL/glext.h:3552
# 3DFX_tbuffer (GL/glext.h:3555)
# EXT_multisample (GL/glext.h:3558)
GL_MULTISAMPLE_EXT = 32925 	# GL/glext.h:3559
GL_SAMPLE_ALPHA_TO_MASK_EXT = 32926 	# GL/glext.h:3560
GL_SAMPLE_ALPHA_TO_ONE_EXT = 32927 	# GL/glext.h:3561
GL_SAMPLE_MASK_EXT = 32928 	# GL/glext.h:3562
GL_1PASS_EXT = 32929 	# GL/glext.h:3563
GL_2PASS_0_EXT = 32930 	# GL/glext.h:3564
GL_2PASS_1_EXT = 32931 	# GL/glext.h:3565
GL_4PASS_0_EXT = 32932 	# GL/glext.h:3566
GL_4PASS_1_EXT = 32933 	# GL/glext.h:3567
GL_4PASS_2_EXT = 32934 	# GL/glext.h:3568
GL_4PASS_3_EXT = 32935 	# GL/glext.h:3569
GL_SAMPLE_BUFFERS_EXT = 32936 	# GL/glext.h:3570
GL_SAMPLES_EXT = 32937 	# GL/glext.h:3571
GL_SAMPLE_MASK_VALUE_EXT = 32938 	# GL/glext.h:3572
GL_SAMPLE_MASK_INVERT_EXT = 32939 	# GL/glext.h:3573
GL_SAMPLE_PATTERN_EXT = 32940 	# GL/glext.h:3574
GL_MULTISAMPLE_BIT_EXT = 536870912 	# GL/glext.h:3575
# SGIX_vertex_preclip (GL/glext.h:3578)
GL_VERTEX_PRECLIP_SGIX = 33774 	# GL/glext.h:3579
GL_VERTEX_PRECLIP_HINT_SGIX = 33775 	# GL/glext.h:3580
# SGIX_convolution_accuracy (GL/glext.h:3583)
GL_CONVOLUTION_HINT_SGIX = 33558 	# GL/glext.h:3584
# SGIX_resample (GL/glext.h:3587)
GL_PACK_RESAMPLE_SGIX = 33836 	# GL/glext.h:3588
GL_UNPACK_RESAMPLE_SGIX = 33837 	# GL/glext.h:3589
GL_RESAMPLE_REPLICATE_SGIX = 33838 	# GL/glext.h:3590
GL_RESAMPLE_ZERO_FILL_SGIX = 33839 	# GL/glext.h:3591
GL_RESAMPLE_DECIMATE_SGIX = 33840 	# GL/glext.h:3592
# SGIS_point_line_texgen (GL/glext.h:3595)
GL_EYE_DISTANCE_TO_POINT_SGIS = 33264 	# GL/glext.h:3596
GL_OBJECT_DISTANCE_TO_POINT_SGIS = 33265 	# GL/glext.h:3597
GL_EYE_DISTANCE_TO_LINE_SGIS = 33266 	# GL/glext.h:3598
GL_OBJECT_DISTANCE_TO_LINE_SGIS = 33267 	# GL/glext.h:3599
GL_EYE_POINT_SGIS = 33268 	# GL/glext.h:3600
GL_OBJECT_POINT_SGIS = 33269 	# GL/glext.h:3601
GL_EYE_LINE_SGIS = 33270 	# GL/glext.h:3602
GL_OBJECT_LINE_SGIS = 33271 	# GL/glext.h:3603
# SGIS_texture_color_mask (GL/glext.h:3606)
GL_TEXTURE_COLOR_WRITEMASK_SGIS = 33263 	# GL/glext.h:3607
# EXT_texture_env_dot3 (GL/glext.h:3610)
GL_DOT3_RGB_EXT = 34624 	# GL/glext.h:3611
GL_DOT3_RGBA_EXT = 34625 	# GL/glext.h:3612
# ATI_texture_mirror_once (GL/glext.h:3615)
GL_MIRROR_CLAMP_ATI = 34626 	# GL/glext.h:3616
GL_MIRROR_CLAMP_TO_EDGE_ATI = 34627 	# GL/glext.h:3617
# NV_fence (GL/glext.h:3620)
GL_ALL_COMPLETED_NV = 34034 	# GL/glext.h:3621
GL_FENCE_STATUS_NV = 34035 	# GL/glext.h:3622
GL_FENCE_CONDITION_NV = 34036 	# GL/glext.h:3623
# IBM_texture_mirrored_repeat (GL/glext.h:3626)
GL_MIRRORED_REPEAT_IBM = 33648 	# GL/glext.h:3627
# NV_evaluators (GL/glext.h:3630)
GL_EVAL_2D_NV = 34496 	# GL/glext.h:3631
GL_EVAL_TRIANGULAR_2D_NV = 34497 	# GL/glext.h:3632
GL_MAP_TESSELLATION_NV = 34498 	# GL/glext.h:3633
GL_MAP_ATTRIB_U_ORDER_NV = 34499 	# GL/glext.h:3634
GL_MAP_ATTRIB_V_ORDER_NV = 34500 	# GL/glext.h:3635
GL_EVAL_FRACTIONAL_TESSELLATION_NV = 34501 	# GL/glext.h:3636
GL_EVAL_VERTEX_ATTRIB0_NV = 34502 	# GL/glext.h:3637
GL_EVAL_VERTEX_ATTRIB1_NV = 34503 	# GL/glext.h:3638
GL_EVAL_VERTEX_ATTRIB2_NV = 34504 	# GL/glext.h:3639
GL_EVAL_VERTEX_ATTRIB3_NV = 34505 	# GL/glext.h:3640
GL_EVAL_VERTEX_ATTRIB4_NV = 34506 	# GL/glext.h:3641
GL_EVAL_VERTEX_ATTRIB5_NV = 34507 	# GL/glext.h:3642
GL_EVAL_VERTEX_ATTRIB6_NV = 34508 	# GL/glext.h:3643
GL_EVAL_VERTEX_ATTRIB7_NV = 34509 	# GL/glext.h:3644
GL_EVAL_VERTEX_ATTRIB8_NV = 34510 	# GL/glext.h:3645
GL_EVAL_VERTEX_ATTRIB9_NV = 34511 	# GL/glext.h:3646
GL_EVAL_VERTEX_ATTRIB10_NV = 34512 	# GL/glext.h:3647
GL_EVAL_VERTEX_ATTRIB11_NV = 34513 	# GL/glext.h:3648
GL_EVAL_VERTEX_ATTRIB12_NV = 34514 	# GL/glext.h:3649
GL_EVAL_VERTEX_ATTRIB13_NV = 34515 	# GL/glext.h:3650
GL_EVAL_VERTEX_ATTRIB14_NV = 34516 	# GL/glext.h:3651
GL_EVAL_VERTEX_ATTRIB15_NV = 34517 	# GL/glext.h:3652
GL_MAX_MAP_TESSELLATION_NV = 34518 	# GL/glext.h:3653
GL_MAX_RATIONAL_EVAL_ORDER_NV = 34519 	# GL/glext.h:3654
# NV_packed_depth_stencil (GL/glext.h:3657)
GL_DEPTH_STENCIL_NV = 34041 	# GL/glext.h:3658
GL_UNSIGNED_INT_24_8_NV = 34042 	# GL/glext.h:3659
# NV_register_combiners2 (GL/glext.h:3662)
GL_PER_STAGE_CONSTANTS_NV = 34101 	# GL/glext.h:3663
# NV_texture_compression_vtc (GL/glext.h:3666)
# NV_texture_rectangle (GL/glext.h:3669)
GL_TEXTURE_RECTANGLE_NV = 34037 	# GL/glext.h:3670
GL_TEXTURE_BINDING_RECTANGLE_NV = 34038 	# GL/glext.h:3671
GL_PROXY_TEXTURE_RECTANGLE_NV = 34039 	# GL/glext.h:3672
GL_MAX_RECTANGLE_TEXTURE_SIZE_NV = 34040 	# GL/glext.h:3673
# NV_texture_shader (GL/glext.h:3676)
GL_OFFSET_TEXTURE_RECTANGLE_NV = 34380 	# GL/glext.h:3677
GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV = 34381 	# GL/glext.h:3678
GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV = 34382 	# GL/glext.h:3679
GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV = 34521 	# GL/glext.h:3680
GL_UNSIGNED_INT_S8_S8_8_8_NV = 34522 	# GL/glext.h:3681
GL_UNSIGNED_INT_8_8_S8_S8_REV_NV = 34523 	# GL/glext.h:3682
GL_DSDT_MAG_INTENSITY_NV = 34524 	# GL/glext.h:3683
GL_SHADER_CONSISTENT_NV = 34525 	# GL/glext.h:3684
GL_TEXTURE_SHADER_NV = 34526 	# GL/glext.h:3685
GL_SHADER_OPERATION_NV = 34527 	# GL/glext.h:3686
GL_CULL_MODES_NV = 34528 	# GL/glext.h:3687
GL_OFFSET_TEXTURE_MATRIX_NV = 34529 	# GL/glext.h:3688
GL_OFFSET_TEXTURE_SCALE_NV = 34530 	# GL/glext.h:3689
GL_OFFSET_TEXTURE_BIAS_NV = 34531 	# GL/glext.h:3690
GL_OFFSET_TEXTURE_2D_MATRIX_NV = 34529 	# GL/glext.h:3691
GL_OFFSET_TEXTURE_2D_SCALE_NV = 34530 	# GL/glext.h:3692
GL_OFFSET_TEXTURE_2D_BIAS_NV = 34531 	# GL/glext.h:3693
GL_PREVIOUS_TEXTURE_INPUT_NV = 34532 	# GL/glext.h:3694
GL_CONST_EYE_NV = 34533 	# GL/glext.h:3695
GL_PASS_THROUGH_NV = 34534 	# GL/glext.h:3696
GL_CULL_FRAGMENT_NV = 34535 	# GL/glext.h:3697
GL_OFFSET_TEXTURE_2D_NV = 34536 	# GL/glext.h:3698
GL_DEPENDENT_AR_TEXTURE_2D_NV = 34537 	# GL/glext.h:3699
GL_DEPENDENT_GB_TEXTURE_2D_NV = 34538 	# GL/glext.h:3700
GL_DOT_PRODUCT_NV = 34540 	# GL/glext.h:3701
GL_DOT_PRODUCT_DEPTH_REPLACE_NV = 34541 	# GL/glext.h:3702
GL_DOT_PRODUCT_TEXTURE_2D_NV = 34542 	# GL/glext.h:3703
GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV = 34544 	# GL/glext.h:3704
GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV = 34545 	# GL/glext.h:3705
GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV = 34546 	# GL/glext.h:3706
GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV = 34547 	# GL/glext.h:3707
GL_HILO_NV = 34548 	# GL/glext.h:3708
GL_DSDT_NV = 34549 	# GL/glext.h:3709
GL_DSDT_MAG_NV = 34550 	# GL/glext.h:3710
GL_DSDT_MAG_VIB_NV = 34551 	# GL/glext.h:3711
GL_HILO16_NV = 34552 	# GL/glext.h:3712
GL_SIGNED_HILO_NV = 34553 	# GL/glext.h:3713
GL_SIGNED_HILO16_NV = 34554 	# GL/glext.h:3714
GL_SIGNED_RGBA_NV = 34555 	# GL/glext.h:3715
GL_SIGNED_RGBA8_NV = 34556 	# GL/glext.h:3716
GL_SIGNED_RGB_NV = 34558 	# GL/glext.h:3717
GL_SIGNED_RGB8_NV = 34559 	# GL/glext.h:3718
GL_SIGNED_LUMINANCE_NV = 34561 	# GL/glext.h:3719
GL_SIGNED_LUMINANCE8_NV = 34562 	# GL/glext.h:3720
GL_SIGNED_LUMINANCE_ALPHA_NV = 34563 	# GL/glext.h:3721
GL_SIGNED_LUMINANCE8_ALPHA8_NV = 34564 	# GL/glext.h:3722
GL_SIGNED_ALPHA_NV = 34565 	# GL/glext.h:3723
GL_SIGNED_ALPHA8_NV = 34566 	# GL/glext.h:3724
GL_SIGNED_INTENSITY_NV = 34567 	# GL/glext.h:3725
GL_SIGNED_INTENSITY8_NV = 34568 	# GL/glext.h:3726
GL_DSDT8_NV = 34569 	# GL/glext.h:3727
GL_DSDT8_MAG8_NV = 34570 	# GL/glext.h:3728
GL_DSDT8_MAG8_INTENSITY8_NV = 34571 	# GL/glext.h:3729
GL_SIGNED_RGB_UNSIGNED_ALPHA_NV = 34572 	# GL/glext.h:3730
GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV = 34573 	# GL/glext.h:3731
GL_HI_SCALE_NV = 34574 	# GL/glext.h:3732
GL_LO_SCALE_NV = 34575 	# GL/glext.h:3733
GL_DS_SCALE_NV = 34576 	# GL/glext.h:3734
GL_DT_SCALE_NV = 34577 	# GL/glext.h:3735
GL_MAGNITUDE_SCALE_NV = 34578 	# GL/glext.h:3736
GL_VIBRANCE_SCALE_NV = 34579 	# GL/glext.h:3737
GL_HI_BIAS_NV = 34580 	# GL/glext.h:3738
GL_LO_BIAS_NV = 34581 	# GL/glext.h:3739
GL_DS_BIAS_NV = 34582 	# GL/glext.h:3740
GL_DT_BIAS_NV = 34583 	# GL/glext.h:3741
GL_MAGNITUDE_BIAS_NV = 34584 	# GL/glext.h:3742
GL_VIBRANCE_BIAS_NV = 34585 	# GL/glext.h:3743
GL_TEXTURE_BORDER_VALUES_NV = 34586 	# GL/glext.h:3744
GL_TEXTURE_HI_SIZE_NV = 34587 	# GL/glext.h:3745
GL_TEXTURE_LO_SIZE_NV = 34588 	# GL/glext.h:3746
GL_TEXTURE_DS_SIZE_NV = 34589 	# GL/glext.h:3747
GL_TEXTURE_DT_SIZE_NV = 34590 	# GL/glext.h:3748
GL_TEXTURE_MAG_SIZE_NV = 34591 	# GL/glext.h:3749
# NV_texture_shader2 (GL/glext.h:3752)
GL_DOT_PRODUCT_TEXTURE_3D_NV = 34543 	# GL/glext.h:3753
# NV_vertex_array_range2 (GL/glext.h:3756)
GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV = 34099 	# GL/glext.h:3757
# NV_vertex_program (GL/glext.h:3760)
GL_VERTEX_PROGRAM_NV = 34336 	# GL/glext.h:3761
GL_VERTEX_STATE_PROGRAM_NV = 34337 	# GL/glext.h:3762
GL_ATTRIB_ARRAY_SIZE_NV = 34339 	# GL/glext.h:3763
GL_ATTRIB_ARRAY_STRIDE_NV = 34340 	# GL/glext.h:3764
GL_ATTRIB_ARRAY_TYPE_NV = 34341 	# GL/glext.h:3765
GL_CURRENT_ATTRIB_NV = 34342 	# GL/glext.h:3766
GL_PROGRAM_LENGTH_NV = 34343 	# GL/glext.h:3767
GL_PROGRAM_STRING_NV = 34344 	# GL/glext.h:3768
GL_MODELVIEW_PROJECTION_NV = 34345 	# GL/glext.h:3769
GL_IDENTITY_NV = 34346 	# GL/glext.h:3770
GL_INVERSE_NV = 34347 	# GL/glext.h:3771
GL_TRANSPOSE_NV = 34348 	# GL/glext.h:3772
GL_INVERSE_TRANSPOSE_NV = 34349 	# GL/glext.h:3773
GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV = 34350 	# GL/glext.h:3774
GL_MAX_TRACK_MATRICES_NV = 34351 	# GL/glext.h:3775
GL_MATRIX0_NV = 34352 	# GL/glext.h:3776
GL_MATRIX1_NV = 34353 	# GL/glext.h:3777
GL_MATRIX2_NV = 34354 	# GL/glext.h:3778
GL_MATRIX3_NV = 34355 	# GL/glext.h:3779
GL_MATRIX4_NV = 34356 	# GL/glext.h:3780
GL_MATRIX5_NV = 34357 	# GL/glext.h:3781
GL_MATRIX6_NV = 34358 	# GL/glext.h:3782
GL_MATRIX7_NV = 34359 	# GL/glext.h:3783
GL_CURRENT_MATRIX_STACK_DEPTH_NV = 34368 	# GL/glext.h:3784
GL_CURRENT_MATRIX_NV = 34369 	# GL/glext.h:3785
GL_VERTEX_PROGRAM_POINT_SIZE_NV = 34370 	# GL/glext.h:3786
GL_VERTEX_PROGRAM_TWO_SIDE_NV = 34371 	# GL/glext.h:3787
GL_PROGRAM_PARAMETER_NV = 34372 	# GL/glext.h:3788
GL_ATTRIB_ARRAY_POINTER_NV = 34373 	# GL/glext.h:3789
GL_PROGRAM_TARGET_NV = 34374 	# GL/glext.h:3790
GL_PROGRAM_RESIDENT_NV = 34375 	# GL/glext.h:3791
GL_TRACK_MATRIX_NV = 34376 	# GL/glext.h:3792
GL_TRACK_MATRIX_TRANSFORM_NV = 34377 	# GL/glext.h:3793
GL_VERTEX_PROGRAM_BINDING_NV = 34378 	# GL/glext.h:3794
GL_PROGRAM_ERROR_POSITION_NV = 34379 	# GL/glext.h:3795
GL_VERTEX_ATTRIB_ARRAY0_NV = 34384 	# GL/glext.h:3796
GL_VERTEX_ATTRIB_ARRAY1_NV = 34385 	# GL/glext.h:3797
GL_VERTEX_ATTRIB_ARRAY2_NV = 34386 	# GL/glext.h:3798
GL_VERTEX_ATTRIB_ARRAY3_NV = 34387 	# GL/glext.h:3799
GL_VERTEX_ATTRIB_ARRAY4_NV = 34388 	# GL/glext.h:3800
GL_VERTEX_ATTRIB_ARRAY5_NV = 34389 	# GL/glext.h:3801
GL_VERTEX_ATTRIB_ARRAY6_NV = 34390 	# GL/glext.h:3802
GL_VERTEX_ATTRIB_ARRAY7_NV = 34391 	# GL/glext.h:3803
GL_VERTEX_ATTRIB_ARRAY8_NV = 34392 	# GL/glext.h:3804
GL_VERTEX_ATTRIB_ARRAY9_NV = 34393 	# GL/glext.h:3805
GL_VERTEX_ATTRIB_ARRAY10_NV = 34394 	# GL/glext.h:3806
GL_VERTEX_ATTRIB_ARRAY11_NV = 34395 	# GL/glext.h:3807
GL_VERTEX_ATTRIB_ARRAY12_NV = 34396 	# GL/glext.h:3808
GL_VERTEX_ATTRIB_ARRAY13_NV = 34397 	# GL/glext.h:3809
GL_VERTEX_ATTRIB_ARRAY14_NV = 34398 	# GL/glext.h:3810
GL_VERTEX_ATTRIB_ARRAY15_NV = 34399 	# GL/glext.h:3811
GL_MAP1_VERTEX_ATTRIB0_4_NV = 34400 	# GL/glext.h:3812
GL_MAP1_VERTEX_ATTRIB1_4_NV = 34401 	# GL/glext.h:3813
GL_MAP1_VERTEX_ATTRIB2_4_NV = 34402 	# GL/glext.h:3814
GL_MAP1_VERTEX_ATTRIB3_4_NV = 34403 	# GL/glext.h:3815
GL_MAP1_VERTEX_ATTRIB4_4_NV = 34404 	# GL/glext.h:3816
GL_MAP1_VERTEX_ATTRIB5_4_NV = 34405 	# GL/glext.h:3817
GL_MAP1_VERTEX_ATTRIB6_4_NV = 34406 	# GL/glext.h:3818
GL_MAP1_VERTEX_ATTRIB7_4_NV = 34407 	# GL/glext.h:3819
GL_MAP1_VERTEX_ATTRIB8_4_NV = 34408 	# GL/glext.h:3820
GL_MAP1_VERTEX_ATTRIB9_4_NV = 34409 	# GL/glext.h:3821
GL_MAP1_VERTEX_ATTRIB10_4_NV = 34410 	# GL/glext.h:3822
GL_MAP1_VERTEX_ATTRIB11_4_NV = 34411 	# GL/glext.h:3823
GL_MAP1_VERTEX_ATTRIB12_4_NV = 34412 	# GL/glext.h:3824
GL_MAP1_VERTEX_ATTRIB13_4_NV = 34413 	# GL/glext.h:3825
GL_MAP1_VERTEX_ATTRIB14_4_NV = 34414 	# GL/glext.h:3826
GL_MAP1_VERTEX_ATTRIB15_4_NV = 34415 	# GL/glext.h:3827
GL_MAP2_VERTEX_ATTRIB0_4_NV = 34416 	# GL/glext.h:3828
GL_MAP2_VERTEX_ATTRIB1_4_NV = 34417 	# GL/glext.h:3829
GL_MAP2_VERTEX_ATTRIB2_4_NV = 34418 	# GL/glext.h:3830
GL_MAP2_VERTEX_ATTRIB3_4_NV = 34419 	# GL/glext.h:3831
GL_MAP2_VERTEX_ATTRIB4_4_NV = 34420 	# GL/glext.h:3832
GL_MAP2_VERTEX_ATTRIB5_4_NV = 34421 	# GL/glext.h:3833
GL_MAP2_VERTEX_ATTRIB6_4_NV = 34422 	# GL/glext.h:3834
GL_MAP2_VERTEX_ATTRIB7_4_NV = 34423 	# GL/glext.h:3835
GL_MAP2_VERTEX_ATTRIB8_4_NV = 34424 	# GL/glext.h:3836
GL_MAP2_VERTEX_ATTRIB9_4_NV = 34425 	# GL/glext.h:3837
GL_MAP2_VERTEX_ATTRIB10_4_NV = 34426 	# GL/glext.h:3838
GL_MAP2_VERTEX_ATTRIB11_4_NV = 34427 	# GL/glext.h:3839
GL_MAP2_VERTEX_ATTRIB12_4_NV = 34428 	# GL/glext.h:3840
GL_MAP2_VERTEX_ATTRIB13_4_NV = 34429 	# GL/glext.h:3841
GL_MAP2_VERTEX_ATTRIB14_4_NV = 34430 	# GL/glext.h:3842
GL_MAP2_VERTEX_ATTRIB15_4_NV = 34431 	# GL/glext.h:3843
# SGIX_texture_coordinate_clamp (GL/glext.h:3846)
GL_TEXTURE_MAX_CLAMP_S_SGIX = 33641 	# GL/glext.h:3847
GL_TEXTURE_MAX_CLAMP_T_SGIX = 33642 	# GL/glext.h:3848
GL_TEXTURE_MAX_CLAMP_R_SGIX = 33643 	# GL/glext.h:3849
# SGIX_scalebias_hint (GL/glext.h:3852)
GL_SCALEBIAS_HINT_SGIX = 33570 	# GL/glext.h:3853
# OML_interlace (GL/glext.h:3856)
GL_INTERLACE_OML = 35200 	# GL/glext.h:3857
GL_INTERLACE_READ_OML = 35201 	# GL/glext.h:3858
# OML_subsample (GL/glext.h:3861)
GL_FORMAT_SUBSAMPLE_24_24_OML = 35202 	# GL/glext.h:3862
GL_FORMAT_SUBSAMPLE_244_244_OML = 35203 	# GL/glext.h:3863
# OML_resample (GL/glext.h:3866)
GL_PACK_RESAMPLE_OML = 35204 	# GL/glext.h:3867
GL_UNPACK_RESAMPLE_OML = 35205 	# GL/glext.h:3868
GL_RESAMPLE_REPLICATE_OML = 35206 	# GL/glext.h:3869
GL_RESAMPLE_ZERO_FILL_OML = 35207 	# GL/glext.h:3870
GL_RESAMPLE_AVERAGE_OML = 35208 	# GL/glext.h:3871
GL_RESAMPLE_DECIMATE_OML = 35209 	# GL/glext.h:3872
# NV_copy_depth_to_color (GL/glext.h:3875)
GL_DEPTH_STENCIL_TO_RGBA_NV = 34926 	# GL/glext.h:3876
GL_DEPTH_STENCIL_TO_BGRA_NV = 34927 	# GL/glext.h:3877
# ATI_envmap_bumpmap (GL/glext.h:3880)
GL_BUMP_ROT_MATRIX_ATI = 34677 	# GL/glext.h:3881
GL_BUMP_ROT_MATRIX_SIZE_ATI = 34678 	# GL/glext.h:3882
GL_BUMP_NUM_TEX_UNITS_ATI = 34679 	# GL/glext.h:3883
GL_BUMP_TEX_UNITS_ATI = 34680 	# GL/glext.h:3884
GL_DUDV_ATI = 34681 	# GL/glext.h:3885
GL_DU8DV8_ATI = 34682 	# GL/glext.h:3886
GL_BUMP_ENVMAP_ATI = 34683 	# GL/glext.h:3887
GL_BUMP_TARGET_ATI = 34684 	# GL/glext.h:3888
# ATI_fragment_shader (GL/glext.h:3891)
GL_FRAGMENT_SHADER_ATI = 35104 	# GL/glext.h:3892
GL_REG_0_ATI = 35105 	# GL/glext.h:3893
GL_REG_1_ATI = 35106 	# GL/glext.h:3894
GL_REG_2_ATI = 35107 	# GL/glext.h:3895
GL_REG_3_ATI = 35108 	# GL/glext.h:3896
GL_REG_4_ATI = 35109 	# GL/glext.h:3897
GL_REG_5_ATI = 35110 	# GL/glext.h:3898
GL_REG_6_ATI = 35111 	# GL/glext.h:3899
GL_REG_7_ATI = 35112 	# GL/glext.h:3900
GL_REG_8_ATI = 35113 	# GL/glext.h:3901
GL_REG_9_ATI = 35114 	# GL/glext.h:3902
GL_REG_10_ATI = 35115 	# GL/glext.h:3903
GL_REG_11_ATI = 35116 	# GL/glext.h:3904
GL_REG_12_ATI = 35117 	# GL/glext.h:3905
GL_REG_13_ATI = 35118 	# GL/glext.h:3906
GL_REG_14_ATI = 35119 	# GL/glext.h:3907
GL_REG_15_ATI = 35120 	# GL/glext.h:3908
GL_REG_16_ATI = 35121 	# GL/glext.h:3909
GL_REG_17_ATI = 35122 	# GL/glext.h:3910
GL_REG_18_ATI = 35123 	# GL/glext.h:3911
GL_REG_19_ATI = 35124 	# GL/glext.h:3912
GL_REG_20_ATI = 35125 	# GL/glext.h:3913
GL_REG_21_ATI = 35126 	# GL/glext.h:3914
GL_REG_22_ATI = 35127 	# GL/glext.h:3915
GL_REG_23_ATI = 35128 	# GL/glext.h:3916
GL_REG_24_ATI = 35129 	# GL/glext.h:3917
GL_REG_25_ATI = 35130 	# GL/glext.h:3918
GL_REG_26_ATI = 35131 	# GL/glext.h:3919
GL_REG_27_ATI = 35132 	# GL/glext.h:3920
GL_REG_28_ATI = 35133 	# GL/glext.h:3921
GL_REG_29_ATI = 35134 	# GL/glext.h:3922
GL_REG_30_ATI = 35135 	# GL/glext.h:3923
GL_REG_31_ATI = 35136 	# GL/glext.h:3924
GL_CON_0_ATI = 35137 	# GL/glext.h:3925
GL_CON_1_ATI = 35138 	# GL/glext.h:3926
GL_CON_2_ATI = 35139 	# GL/glext.h:3927
GL_CON_3_ATI = 35140 	# GL/glext.h:3928
GL_CON_4_ATI = 35141 	# GL/glext.h:3929
GL_CON_5_ATI = 35142 	# GL/glext.h:3930
GL_CON_6_ATI = 35143 	# GL/glext.h:3931
GL_CON_7_ATI = 35144 	# GL/glext.h:3932
GL_CON_8_ATI = 35145 	# GL/glext.h:3933
GL_CON_9_ATI = 35146 	# GL/glext.h:3934
GL_CON_10_ATI = 35147 	# GL/glext.h:3935
GL_CON_11_ATI = 35148 	# GL/glext.h:3936
GL_CON_12_ATI = 35149 	# GL/glext.h:3937
GL_CON_13_ATI = 35150 	# GL/glext.h:3938
GL_CON_14_ATI = 35151 	# GL/glext.h:3939
GL_CON_15_ATI = 35152 	# GL/glext.h:3940
GL_CON_16_ATI = 35153 	# GL/glext.h:3941
GL_CON_17_ATI = 35154 	# GL/glext.h:3942
GL_CON_18_ATI = 35155 	# GL/glext.h:3943
GL_CON_19_ATI = 35156 	# GL/glext.h:3944
GL_CON_20_ATI = 35157 	# GL/glext.h:3945
GL_CON_21_ATI = 35158 	# GL/glext.h:3946
GL_CON_22_ATI = 35159 	# GL/glext.h:3947
GL_CON_23_ATI = 35160 	# GL/glext.h:3948
GL_CON_24_ATI = 35161 	# GL/glext.h:3949
GL_CON_25_ATI = 35162 	# GL/glext.h:3950
GL_CON_26_ATI = 35163 	# GL/glext.h:3951
GL_CON_27_ATI = 35164 	# GL/glext.h:3952
GL_CON_28_ATI = 35165 	# GL/glext.h:3953
GL_CON_29_ATI = 35166 	# GL/glext.h:3954
GL_CON_30_ATI = 35167 	# GL/glext.h:3955
GL_CON_31_ATI = 35168 	# GL/glext.h:3956
GL_MOV_ATI = 35169 	# GL/glext.h:3957
GL_ADD_ATI = 35171 	# GL/glext.h:3958
GL_MUL_ATI = 35172 	# GL/glext.h:3959
GL_SUB_ATI = 35173 	# GL/glext.h:3960
GL_DOT3_ATI = 35174 	# GL/glext.h:3961
GL_DOT4_ATI = 35175 	# GL/glext.h:3962
GL_MAD_ATI = 35176 	# GL/glext.h:3963
GL_LERP_ATI = 35177 	# GL/glext.h:3964
GL_CND_ATI = 35178 	# GL/glext.h:3965
GL_CND0_ATI = 35179 	# GL/glext.h:3966
GL_DOT2_ADD_ATI = 35180 	# GL/glext.h:3967
GL_SECONDARY_INTERPOLATOR_ATI = 35181 	# GL/glext.h:3968
GL_NUM_FRAGMENT_REGISTERS_ATI = 35182 	# GL/glext.h:3969
GL_NUM_FRAGMENT_CONSTANTS_ATI = 35183 	# GL/glext.h:3970
GL_NUM_PASSES_ATI = 35184 	# GL/glext.h:3971
GL_NUM_INSTRUCTIONS_PER_PASS_ATI = 35185 	# GL/glext.h:3972
GL_NUM_INSTRUCTIONS_TOTAL_ATI = 35186 	# GL/glext.h:3973
GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI = 35187 	# GL/glext.h:3974
GL_NUM_LOOPBACK_COMPONENTS_ATI = 35188 	# GL/glext.h:3975
GL_COLOR_ALPHA_PAIRING_ATI = 35189 	# GL/glext.h:3976
GL_SWIZZLE_STR_ATI = 35190 	# GL/glext.h:3977
GL_SWIZZLE_STQ_ATI = 35191 	# GL/glext.h:3978
GL_SWIZZLE_STR_DR_ATI = 35192 	# GL/glext.h:3979
GL_SWIZZLE_STQ_DQ_ATI = 35193 	# GL/glext.h:3980
GL_SWIZZLE_STRQ_ATI = 35194 	# GL/glext.h:3981
GL_SWIZZLE_STRQ_DQ_ATI = 35195 	# GL/glext.h:3982
GL_RED_BIT_ATI = 1 	# GL/glext.h:3983
GL_GREEN_BIT_ATI = 2 	# GL/glext.h:3984
GL_BLUE_BIT_ATI = 4 	# GL/glext.h:3985
GL_2X_BIT_ATI = 1 	# GL/glext.h:3986
GL_4X_BIT_ATI = 2 	# GL/glext.h:3987
GL_8X_BIT_ATI = 4 	# GL/glext.h:3988
GL_HALF_BIT_ATI = 8 	# GL/glext.h:3989
GL_QUARTER_BIT_ATI = 16 	# GL/glext.h:3990
GL_EIGHTH_BIT_ATI = 32 	# GL/glext.h:3991
GL_SATURATE_BIT_ATI = 64 	# GL/glext.h:3992
GL_COMP_BIT_ATI = 2 	# GL/glext.h:3993
GL_NEGATE_BIT_ATI = 4 	# GL/glext.h:3994
GL_BIAS_BIT_ATI = 8 	# GL/glext.h:3995
# ATI_pn_triangles (GL/glext.h:3998)
GL_PN_TRIANGLES_ATI = 34800 	# GL/glext.h:3999
GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34801 	# GL/glext.h:4000
GL_PN_TRIANGLES_POINT_MODE_ATI = 34802 	# GL/glext.h:4001
GL_PN_TRIANGLES_NORMAL_MODE_ATI = 34803 	# GL/glext.h:4002
GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI = 34804 	# GL/glext.h:4003
GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI = 34805 	# GL/glext.h:4004
GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI = 34806 	# GL/glext.h:4005
GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI = 34807 	# GL/glext.h:4006
GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI = 34808 	# GL/glext.h:4007
# ATI_vertex_array_object (GL/glext.h:4010)
GL_STATIC_ATI = 34656 	# GL/glext.h:4011
GL_DYNAMIC_ATI = 34657 	# GL/glext.h:4012
GL_PRESERVE_ATI = 34658 	# GL/glext.h:4013
GL_DISCARD_ATI = 34659 	# GL/glext.h:4014
GL_OBJECT_BUFFER_SIZE_ATI = 34660 	# GL/glext.h:4015
GL_OBJECT_BUFFER_USAGE_ATI = 34661 	# GL/glext.h:4016
GL_ARRAY_OBJECT_BUFFER_ATI = 34662 	# GL/glext.h:4017
GL_ARRAY_OBJECT_OFFSET_ATI = 34663 	# GL/glext.h:4018
# EXT_vertex_shader (GL/glext.h:4021)
GL_VERTEX_SHADER_EXT = 34688 	# GL/glext.h:4022
GL_VERTEX_SHADER_BINDING_EXT = 34689 	# GL/glext.h:4023
GL_OP_INDEX_EXT = 34690 	# GL/glext.h:4024
GL_OP_NEGATE_EXT = 34691 	# GL/glext.h:4025
GL_OP_DOT3_EXT = 34692 	# GL/glext.h:4026
GL_OP_DOT4_EXT = 34693 	# GL/glext.h:4027
GL_OP_MUL_EXT = 34694 	# GL/glext.h:4028
GL_OP_ADD_EXT = 34695 	# GL/glext.h:4029
GL_OP_MADD_EXT = 34696 	# GL/glext.h:4030
GL_OP_FRAC_EXT = 34697 	# GL/glext.h:4031
GL_OP_MAX_EXT = 34698 	# GL/glext.h:4032
GL_OP_MIN_EXT = 34699 	# GL/glext.h:4033
GL_OP_SET_GE_EXT = 34700 	# GL/glext.h:4034
GL_OP_SET_LT_EXT = 34701 	# GL/glext.h:4035
GL_OP_CLAMP_EXT = 34702 	# GL/glext.h:4036
GL_OP_FLOOR_EXT = 34703 	# GL/glext.h:4037
GL_OP_ROUND_EXT = 34704 	# GL/glext.h:4038
GL_OP_EXP_BASE_2_EXT = 34705 	# GL/glext.h:4039
GL_OP_LOG_BASE_2_EXT = 34706 	# GL/glext.h:4040
GL_OP_POWER_EXT = 34707 	# GL/glext.h:4041
GL_OP_RECIP_EXT = 34708 	# GL/glext.h:4042
GL_OP_RECIP_SQRT_EXT = 34709 	# GL/glext.h:4043
GL_OP_SUB_EXT = 34710 	# GL/glext.h:4044
GL_OP_CROSS_PRODUCT_EXT = 34711 	# GL/glext.h:4045
GL_OP_MULTIPLY_MATRIX_EXT = 34712 	# GL/glext.h:4046
GL_OP_MOV_EXT = 34713 	# GL/glext.h:4047
GL_OUTPUT_VERTEX_EXT = 34714 	# GL/glext.h:4048
GL_OUTPUT_COLOR0_EXT = 34715 	# GL/glext.h:4049
GL_OUTPUT_COLOR1_EXT = 34716 	# GL/glext.h:4050
GL_OUTPUT_TEXTURE_COORD0_EXT = 34717 	# GL/glext.h:4051
GL_OUTPUT_TEXTURE_COORD1_EXT = 34718 	# GL/glext.h:4052
GL_OUTPUT_TEXTURE_COORD2_EXT = 34719 	# GL/glext.h:4053
GL_OUTPUT_TEXTURE_COORD3_EXT = 34720 	# GL/glext.h:4054
GL_OUTPUT_TEXTURE_COORD4_EXT = 34721 	# GL/glext.h:4055
GL_OUTPUT_TEXTURE_COORD5_EXT = 34722 	# GL/glext.h:4056
GL_OUTPUT_TEXTURE_COORD6_EXT = 34723 	# GL/glext.h:4057
GL_OUTPUT_TEXTURE_COORD7_EXT = 34724 	# GL/glext.h:4058
GL_OUTPUT_TEXTURE_COORD8_EXT = 34725 	# GL/glext.h:4059
GL_OUTPUT_TEXTURE_COORD9_EXT = 34726 	# GL/glext.h:4060
GL_OUTPUT_TEXTURE_COORD10_EXT = 34727 	# GL/glext.h:4061
GL_OUTPUT_TEXTURE_COORD11_EXT = 34728 	# GL/glext.h:4062
GL_OUTPUT_TEXTURE_COORD12_EXT = 34729 	# GL/glext.h:4063
GL_OUTPUT_TEXTURE_COORD13_EXT = 34730 	# GL/glext.h:4064
GL_OUTPUT_TEXTURE_COORD14_EXT = 34731 	# GL/glext.h:4065
GL_OUTPUT_TEXTURE_COORD15_EXT = 34732 	# GL/glext.h:4066
GL_OUTPUT_TEXTURE_COORD16_EXT = 34733 	# GL/glext.h:4067
GL_OUTPUT_TEXTURE_COORD17_EXT = 34734 	# GL/glext.h:4068
GL_OUTPUT_TEXTURE_COORD18_EXT = 34735 	# GL/glext.h:4069
GL_OUTPUT_TEXTURE_COORD19_EXT = 34736 	# GL/glext.h:4070
GL_OUTPUT_TEXTURE_COORD20_EXT = 34737 	# GL/glext.h:4071
GL_OUTPUT_TEXTURE_COORD21_EXT = 34738 	# GL/glext.h:4072
GL_OUTPUT_TEXTURE_COORD22_EXT = 34739 	# GL/glext.h:4073
GL_OUTPUT_TEXTURE_COORD23_EXT = 34740 	# GL/glext.h:4074
GL_OUTPUT_TEXTURE_COORD24_EXT = 34741 	# GL/glext.h:4075
GL_OUTPUT_TEXTURE_COORD25_EXT = 34742 	# GL/glext.h:4076
GL_OUTPUT_TEXTURE_COORD26_EXT = 34743 	# GL/glext.h:4077
GL_OUTPUT_TEXTURE_COORD27_EXT = 34744 	# GL/glext.h:4078
GL_OUTPUT_TEXTURE_COORD28_EXT = 34745 	# GL/glext.h:4079
GL_OUTPUT_TEXTURE_COORD29_EXT = 34746 	# GL/glext.h:4080
GL_OUTPUT_TEXTURE_COORD30_EXT = 34747 	# GL/glext.h:4081
GL_OUTPUT_TEXTURE_COORD31_EXT = 34748 	# GL/glext.h:4082
GL_OUTPUT_FOG_EXT = 34749 	# GL/glext.h:4083
GL_SCALAR_EXT = 34750 	# GL/glext.h:4084
GL_VECTOR_EXT = 34751 	# GL/glext.h:4085
GL_MATRIX_EXT = 34752 	# GL/glext.h:4086
GL_VARIANT_EXT = 34753 	# GL/glext.h:4087
GL_INVARIANT_EXT = 34754 	# GL/glext.h:4088
GL_LOCAL_CONSTANT_EXT = 34755 	# GL/glext.h:4089
GL_LOCAL_EXT = 34756 	# GL/glext.h:4090
GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT = 34757 	# GL/glext.h:4091
GL_MAX_VERTEX_SHADER_VARIANTS_EXT = 34758 	# GL/glext.h:4092
GL_MAX_VERTEX_SHADER_INVARIANTS_EXT = 34759 	# GL/glext.h:4093
GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34760 	# GL/glext.h:4094
GL_MAX_VERTEX_SHADER_LOCALS_EXT = 34761 	# GL/glext.h:4095
GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT = 34762 	# GL/glext.h:4096
GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT = 34763 	# GL/glext.h:4097
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34764 	# GL/glext.h:4098
GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT = 34765 	# GL/glext.h:4099
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT = 34766 	# GL/glext.h:4100
GL_VERTEX_SHADER_INSTRUCTIONS_EXT = 34767 	# GL/glext.h:4101
GL_VERTEX_SHADER_VARIANTS_EXT = 34768 	# GL/glext.h:4102
GL_VERTEX_SHADER_INVARIANTS_EXT = 34769 	# GL/glext.h:4103
GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = 34770 	# GL/glext.h:4104
GL_VERTEX_SHADER_LOCALS_EXT = 34771 	# GL/glext.h:4105
GL_VERTEX_SHADER_OPTIMIZED_EXT = 34772 	# GL/glext.h:4106
GL_X_EXT = 34773 	# GL/glext.h:4107
GL_Y_EXT = 34774 	# GL/glext.h:4108
GL_Z_EXT = 34775 	# GL/glext.h:4109
GL_W_EXT = 34776 	# GL/glext.h:4110
GL_NEGATIVE_X_EXT = 34777 	# GL/glext.h:4111
GL_NEGATIVE_Y_EXT = 34778 	# GL/glext.h:4112
GL_NEGATIVE_Z_EXT = 34779 	# GL/glext.h:4113
GL_NEGATIVE_W_EXT = 34780 	# GL/glext.h:4114
GL_ZERO_EXT = 34781 	# GL/glext.h:4115
GL_ONE_EXT = 34782 	# GL/glext.h:4116
GL_NEGATIVE_ONE_EXT = 34783 	# GL/glext.h:4117
GL_NORMALIZED_RANGE_EXT = 34784 	# GL/glext.h:4118
GL_FULL_RANGE_EXT = 34785 	# GL/glext.h:4119
GL_CURRENT_VERTEX_EXT = 34786 	# GL/glext.h:4120
GL_MVP_MATRIX_EXT = 34787 	# GL/glext.h:4121
GL_VARIANT_VALUE_EXT = 34788 	# GL/glext.h:4122
GL_VARIANT_DATATYPE_EXT = 34789 	# GL/glext.h:4123
GL_VARIANT_ARRAY_STRIDE_EXT = 34790 	# GL/glext.h:4124
GL_VARIANT_ARRAY_TYPE_EXT = 34791 	# GL/glext.h:4125
GL_VARIANT_ARRAY_EXT = 34792 	# GL/glext.h:4126
GL_VARIANT_ARRAY_POINTER_EXT = 34793 	# GL/glext.h:4127
GL_INVARIANT_VALUE_EXT = 34794 	# GL/glext.h:4128
GL_INVARIANT_DATATYPE_EXT = 34795 	# GL/glext.h:4129
GL_LOCAL_CONSTANT_VALUE_EXT = 34796 	# GL/glext.h:4130
GL_LOCAL_CONSTANT_DATATYPE_EXT = 34797 	# GL/glext.h:4131
# ATI_vertex_streams (GL/glext.h:4134)
GL_MAX_VERTEX_STREAMS_ATI = 34667 	# GL/glext.h:4135
GL_VERTEX_STREAM0_ATI = 34668 	# GL/glext.h:4136
GL_VERTEX_STREAM1_ATI = 34669 	# GL/glext.h:4137
GL_VERTEX_STREAM2_ATI = 34670 	# GL/glext.h:4138
GL_VERTEX_STREAM3_ATI = 34671 	# GL/glext.h:4139
GL_VERTEX_STREAM4_ATI = 34672 	# GL/glext.h:4140
GL_VERTEX_STREAM5_ATI = 34673 	# GL/glext.h:4141
GL_VERTEX_STREAM6_ATI = 34674 	# GL/glext.h:4142
GL_VERTEX_STREAM7_ATI = 34675 	# GL/glext.h:4143
GL_VERTEX_SOURCE_ATI = 34676 	# GL/glext.h:4144
# ATI_element_array (GL/glext.h:4147)
GL_ELEMENT_ARRAY_ATI = 34664 	# GL/glext.h:4148
GL_ELEMENT_ARRAY_TYPE_ATI = 34665 	# GL/glext.h:4149
GL_ELEMENT_ARRAY_POINTER_ATI = 34666 	# GL/glext.h:4150
# SUN_mesh_array (GL/glext.h:4153)
GL_QUAD_MESH_SUN = 34324 	# GL/glext.h:4154
GL_TRIANGLE_MESH_SUN = 34325 	# GL/glext.h:4155
# SUN_slice_accum (GL/glext.h:4158)
GL_SLICE_ACCUM_SUN = 34252 	# GL/glext.h:4159
# NV_multisample_filter_hint (GL/glext.h:4162)
GL_MULTISAMPLE_FILTER_HINT_NV = 34100 	# GL/glext.h:4163
# NV_depth_clamp (GL/glext.h:4166)
GL_DEPTH_CLAMP_NV = 34383 	# GL/glext.h:4167
# NV_occlusion_query (GL/glext.h:4170)
GL_PIXEL_COUNTER_BITS_NV = 34916 	# GL/glext.h:4171
GL_CURRENT_OCCLUSION_QUERY_ID_NV = 34917 	# GL/glext.h:4172
GL_PIXEL_COUNT_NV = 34918 	# GL/glext.h:4173
GL_PIXEL_COUNT_AVAILABLE_NV = 34919 	# GL/glext.h:4174
# NV_point_sprite (GL/glext.h:4177)
GL_POINT_SPRITE_NV = 34913 	# GL/glext.h:4178
GL_COORD_REPLACE_NV = 34914 	# GL/glext.h:4179
GL_POINT_SPRITE_R_MODE_NV = 34915 	# GL/glext.h:4180
# NV_texture_shader3 (GL/glext.h:4183)
GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV = 34896 	# GL/glext.h:4184
GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV = 34897 	# GL/glext.h:4185
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34898 	# GL/glext.h:4186
GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV = 34899 	# GL/glext.h:4187
GL_OFFSET_HILO_TEXTURE_2D_NV = 34900 	# GL/glext.h:4188
GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV = 34901 	# GL/glext.h:4189
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV = 34902 	# GL/glext.h:4190
GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV = 34903 	# GL/glext.h:4191
GL_DEPENDENT_HILO_TEXTURE_2D_NV = 34904 	# GL/glext.h:4192
GL_DEPENDENT_RGB_TEXTURE_3D_NV = 34905 	# GL/glext.h:4193
GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV = 34906 	# GL/glext.h:4194
GL_DOT_PRODUCT_PASS_THROUGH_NV = 34907 	# GL/glext.h:4195
GL_DOT_PRODUCT_TEXTURE_1D_NV = 34908 	# GL/glext.h:4196
GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV = 34909 	# GL/glext.h:4197
GL_HILO8_NV = 34910 	# GL/glext.h:4198
GL_SIGNED_HILO8_NV = 34911 	# GL/glext.h:4199
GL_FORCE_BLUE_TO_ONE_NV = 34912 	# GL/glext.h:4200
# NV_vertex_program1_1 (GL/glext.h:4203)
# EXT_shadow_funcs (GL/glext.h:4206)
# EXT_stencil_two_side (GL/glext.h:4209)
GL_STENCIL_TEST_TWO_SIDE_EXT = 35088 	# GL/glext.h:4210
GL_ACTIVE_STENCIL_FACE_EXT = 35089 	# GL/glext.h:4211
# ATI_text_fragment_shader (GL/glext.h:4214)
GL_TEXT_FRAGMENT_SHADER_ATI = 33280 	# GL/glext.h:4215
# APPLE_client_storage (GL/glext.h:4218)
GL_UNPACK_CLIENT_STORAGE_APPLE = 34226 	# GL/glext.h:4219
# APPLE_element_array (GL/glext.h:4222)
GL_ELEMENT_ARRAY_APPLE = 35340 	# GL/glext.h:4223
GL_ELEMENT_ARRAY_TYPE_APPLE = 35341 	# GL/glext.h:4224
GL_ELEMENT_ARRAY_POINTER_APPLE = 35342 	# GL/glext.h:4225
# APPLE_fence (GL/glext.h:4228)
GL_DRAW_PIXELS_APPLE = 35338 	# GL/glext.h:4229
GL_FENCE_APPLE = 35339 	# GL/glext.h:4230
# APPLE_vertex_array_object (GL/glext.h:4233)
GL_VERTEX_ARRAY_BINDING_APPLE = 34229 	# GL/glext.h:4234
# APPLE_vertex_array_range (GL/glext.h:4237)
GL_VERTEX_ARRAY_RANGE_APPLE = 34077 	# GL/glext.h:4238
GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE = 34078 	# GL/glext.h:4239
GL_VERTEX_ARRAY_STORAGE_HINT_APPLE = 34079 	# GL/glext.h:4240
GL_VERTEX_ARRAY_RANGE_POINTER_APPLE = 34081 	# GL/glext.h:4241
GL_STORAGE_CLIENT_APPLE = 34228 	# GL/glext.h:4242
GL_STORAGE_CACHED_APPLE = 34238 	# GL/glext.h:4243
GL_STORAGE_SHARED_APPLE = 34239 	# GL/glext.h:4244
# APPLE_ycbcr_422 (GL/glext.h:4247)
GL_YCBCR_422_APPLE = 34233 	# GL/glext.h:4248
GL_UNSIGNED_SHORT_8_8_APPLE = 34234 	# GL/glext.h:4249
GL_UNSIGNED_SHORT_8_8_REV_APPLE = 34235 	# GL/glext.h:4250
# S3_s3tc (GL/glext.h:4253)
GL_RGB_S3TC = 33696 	# GL/glext.h:4254
GL_RGB4_S3TC = 33697 	# GL/glext.h:4255
GL_RGBA_S3TC = 33698 	# GL/glext.h:4256
GL_RGBA4_S3TC = 33699 	# GL/glext.h:4257
# ATI_draw_buffers (GL/glext.h:4260)
GL_MAX_DRAW_BUFFERS_ATI = 34852 	# GL/glext.h:4261
GL_DRAW_BUFFER0_ATI = 34853 	# GL/glext.h:4262
GL_DRAW_BUFFER1_ATI = 34854 	# GL/glext.h:4263
GL_DRAW_BUFFER2_ATI = 34855 	# GL/glext.h:4264
GL_DRAW_BUFFER3_ATI = 34856 	# GL/glext.h:4265
GL_DRAW_BUFFER4_ATI = 34857 	# GL/glext.h:4266
GL_DRAW_BUFFER5_ATI = 34858 	# GL/glext.h:4267
GL_DRAW_BUFFER6_ATI = 34859 	# GL/glext.h:4268
GL_DRAW_BUFFER7_ATI = 34860 	# GL/glext.h:4269
GL_DRAW_BUFFER8_ATI = 34861 	# GL/glext.h:4270
GL_DRAW_BUFFER9_ATI = 34862 	# GL/glext.h:4271
GL_DRAW_BUFFER10_ATI = 34863 	# GL/glext.h:4272
GL_DRAW_BUFFER11_ATI = 34864 	# GL/glext.h:4273
GL_DRAW_BUFFER12_ATI = 34865 	# GL/glext.h:4274
GL_DRAW_BUFFER13_ATI = 34866 	# GL/glext.h:4275
GL_DRAW_BUFFER14_ATI = 34867 	# GL/glext.h:4276
GL_DRAW_BUFFER15_ATI = 34868 	# GL/glext.h:4277
# ATI_pixel_format_float (GL/glext.h:4280)
GL_TYPE_RGBA_FLOAT_ATI = 34848 	# GL/glext.h:4281
GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI = 34869 	# GL/glext.h:4282
# ATI_texture_env_combine3 (GL/glext.h:4285)
GL_MODULATE_ADD_ATI = 34628 	# GL/glext.h:4286
GL_MODULATE_SIGNED_ADD_ATI = 34629 	# GL/glext.h:4287
GL_MODULATE_SUBTRACT_ATI = 34630 	# GL/glext.h:4288
# ATI_texture_float (GL/glext.h:4291)
GL_RGBA_FLOAT32_ATI = 34836 	# GL/glext.h:4292
GL_RGB_FLOAT32_ATI = 34837 	# GL/glext.h:4293
GL_ALPHA_FLOAT32_ATI = 34838 	# GL/glext.h:4294
GL_INTENSITY_FLOAT32_ATI = 34839 	# GL/glext.h:4295
GL_LUMINANCE_FLOAT32_ATI = 34840 	# GL/glext.h:4296
GL_LUMINANCE_ALPHA_FLOAT32_ATI = 34841 	# GL/glext.h:4297
GL_RGBA_FLOAT16_ATI = 34842 	# GL/glext.h:4298
GL_RGB_FLOAT16_ATI = 34843 	# GL/glext.h:4299
GL_ALPHA_FLOAT16_ATI = 34844 	# GL/glext.h:4300
GL_INTENSITY_FLOAT16_ATI = 34845 	# GL/glext.h:4301
GL_LUMINANCE_FLOAT16_ATI = 34846 	# GL/glext.h:4302
GL_LUMINANCE_ALPHA_FLOAT16_ATI = 34847 	# GL/glext.h:4303
# NV_float_buffer (GL/glext.h:4306)
GL_FLOAT_R_NV = 34944 	# GL/glext.h:4307
GL_FLOAT_RG_NV = 34945 	# GL/glext.h:4308
GL_FLOAT_RGB_NV = 34946 	# GL/glext.h:4309
GL_FLOAT_RGBA_NV = 34947 	# GL/glext.h:4310
GL_FLOAT_R16_NV = 34948 	# GL/glext.h:4311
GL_FLOAT_R32_NV = 34949 	# GL/glext.h:4312
GL_FLOAT_RG16_NV = 34950 	# GL/glext.h:4313
GL_FLOAT_RG32_NV = 34951 	# GL/glext.h:4314
GL_FLOAT_RGB16_NV = 34952 	# GL/glext.h:4315
GL_FLOAT_RGB32_NV = 34953 	# GL/glext.h:4316
GL_FLOAT_RGBA16_NV = 34954 	# GL/glext.h:4317
GL_FLOAT_RGBA32_NV = 34955 	# GL/glext.h:4318
GL_TEXTURE_FLOAT_COMPONENTS_NV = 34956 	# GL/glext.h:4319
GL_FLOAT_CLEAR_COLOR_VALUE_NV = 34957 	# GL/glext.h:4320
GL_FLOAT_RGBA_MODE_NV = 34958 	# GL/glext.h:4321
# NV_fragment_program (GL/glext.h:4324)
GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV = 34920 	# GL/glext.h:4325
GL_FRAGMENT_PROGRAM_NV = 34928 	# GL/glext.h:4326
GL_MAX_TEXTURE_COORDS_NV = 34929 	# GL/glext.h:4327
GL_MAX_TEXTURE_IMAGE_UNITS_NV = 34930 	# GL/glext.h:4328
GL_FRAGMENT_PROGRAM_BINDING_NV = 34931 	# GL/glext.h:4329
GL_PROGRAM_ERROR_STRING_NV = 34932 	# GL/glext.h:4330
# NV_half_float (GL/glext.h:4333)
GL_HALF_FLOAT_NV = 5131 	# GL/glext.h:4334
# NV_pixel_data_range (GL/glext.h:4337)
GL_WRITE_PIXEL_DATA_RANGE_NV = 34936 	# GL/glext.h:4338
GL_READ_PIXEL_DATA_RANGE_NV = 34937 	# GL/glext.h:4339
GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV = 34938 	# GL/glext.h:4340
GL_READ_PIXEL_DATA_RANGE_LENGTH_NV = 34939 	# GL/glext.h:4341
GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV = 34940 	# GL/glext.h:4342
GL_READ_PIXEL_DATA_RANGE_POINTER_NV = 34941 	# GL/glext.h:4343
# NV_primitive_restart (GL/glext.h:4346)
GL_PRIMITIVE_RESTART_NV = 34136 	# GL/glext.h:4347
GL_PRIMITIVE_RESTART_INDEX_NV = 34137 	# GL/glext.h:4348
# NV_texture_expand_normal (GL/glext.h:4351)
GL_TEXTURE_UNSIGNED_REMAP_MODE_NV = 34959 	# GL/glext.h:4352
# NV_vertex_program2 (GL/glext.h:4355)
# ATI_map_object_buffer (GL/glext.h:4358)
# ATI_separate_stencil (GL/glext.h:4361)
GL_STENCIL_BACK_FUNC_ATI = 34816 	# GL/glext.h:4362
GL_STENCIL_BACK_FAIL_ATI = 34817 	# GL/glext.h:4363
GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI = 34818 	# GL/glext.h:4364
GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI = 34819 	# GL/glext.h:4365
# ATI_vertex_attrib_array_object (GL/glext.h:4368)
# OES_read_format (GL/glext.h:4371)
GL_IMPLEMENTATION_COLOR_READ_TYPE_OES = 35738 	# GL/glext.h:4372
GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES = 35739 	# GL/glext.h:4373
# EXT_depth_bounds_test (GL/glext.h:4376)
GL_DEPTH_BOUNDS_TEST_EXT = 34960 	# GL/glext.h:4377
GL_DEPTH_BOUNDS_EXT = 34961 	# GL/glext.h:4378
# EXT_texture_mirror_clamp (GL/glext.h:4381)
GL_MIRROR_CLAMP_EXT = 34626 	# GL/glext.h:4382
GL_MIRROR_CLAMP_TO_EDGE_EXT = 34627 	# GL/glext.h:4383
GL_MIRROR_CLAMP_TO_BORDER_EXT = 35090 	# GL/glext.h:4384
# EXT_blend_equation_separate (GL/glext.h:4387)
GL_BLEND_EQUATION_RGB_EXT = 32777 	# GL/glext.h:4388
GL_BLEND_EQUATION_ALPHA_EXT = 34877 	# GL/glext.h:4389
# MESA_pack_invert (GL/glext.h:4392)
GL_PACK_INVERT_MESA = 34648 	# GL/glext.h:4393
# MESA_ycbcr_texture (GL/glext.h:4396)
GL_UNSIGNED_SHORT_8_8_MESA = 34234 	# GL/glext.h:4397
GL_UNSIGNED_SHORT_8_8_REV_MESA = 34235 	# GL/glext.h:4398
GL_YCBCR_MESA = 34647 	# GL/glext.h:4399
# EXT_pixel_buffer_object (GL/glext.h:4402)
GL_PIXEL_PACK_BUFFER_EXT = 35051 	# GL/glext.h:4403
GL_PIXEL_UNPACK_BUFFER_EXT = 35052 	# GL/glext.h:4404
GL_PIXEL_PACK_BUFFER_BINDING_EXT = 35053 	# GL/glext.h:4405
GL_PIXEL_UNPACK_BUFFER_BINDING_EXT = 35055 	# GL/glext.h:4406
# NV_fragment_program_option (GL/glext.h:4409)
# NV_fragment_program2 (GL/glext.h:4412)
GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV = 35060 	# GL/glext.h:4413
GL_MAX_PROGRAM_CALL_DEPTH_NV = 35061 	# GL/glext.h:4414
GL_MAX_PROGRAM_IF_DEPTH_NV = 35062 	# GL/glext.h:4415
GL_MAX_PROGRAM_LOOP_DEPTH_NV = 35063 	# GL/glext.h:4416
GL_MAX_PROGRAM_LOOP_COUNT_NV = 35064 	# GL/glext.h:4417
# NV_vertex_program2_option (GL/glext.h:4420)
# NV_vertex_program3 (GL/glext.h:4425)
# EXT_framebuffer_object (GL/glext.h:4429)
GL_INVALID_FRAMEBUFFER_OPERATION_EXT = 1286 	# GL/glext.h:4430
GL_MAX_RENDERBUFFER_SIZE_EXT = 34024 	# GL/glext.h:4431
GL_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:4432
GL_RENDERBUFFER_BINDING_EXT = 36007 	# GL/glext.h:4433
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT = 36048 	# GL/glext.h:4434
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT = 36049 	# GL/glext.h:4435
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT = 36050 	# GL/glext.h:4436
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT = 36051 	# GL/glext.h:4437
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT = 36052 	# GL/glext.h:4438
GL_FRAMEBUFFER_COMPLETE_EXT = 36053 	# GL/glext.h:4439
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT = 36054 	# GL/glext.h:4440
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT = 36055 	# GL/glext.h:4441
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT = 36057 	# GL/glext.h:4442
GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT = 36058 	# GL/glext.h:4443
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT = 36059 	# GL/glext.h:4444
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT = 36060 	# GL/glext.h:4445
GL_FRAMEBUFFER_UNSUPPORTED_EXT = 36061 	# GL/glext.h:4446
GL_MAX_COLOR_ATTACHMENTS_EXT = 36063 	# GL/glext.h:4447
GL_COLOR_ATTACHMENT0_EXT = 36064 	# GL/glext.h:4448
GL_COLOR_ATTACHMENT1_EXT = 36065 	# GL/glext.h:4449
GL_COLOR_ATTACHMENT2_EXT = 36066 	# GL/glext.h:4450
GL_COLOR_ATTACHMENT3_EXT = 36067 	# GL/glext.h:4451
GL_COLOR_ATTACHMENT4_EXT = 36068 	# GL/glext.h:4452
GL_COLOR_ATTACHMENT5_EXT = 36069 	# GL/glext.h:4453
GL_COLOR_ATTACHMENT6_EXT = 36070 	# GL/glext.h:4454
GL_COLOR_ATTACHMENT7_EXT = 36071 	# GL/glext.h:4455
GL_COLOR_ATTACHMENT8_EXT = 36072 	# GL/glext.h:4456
GL_COLOR_ATTACHMENT9_EXT = 36073 	# GL/glext.h:4457
GL_COLOR_ATTACHMENT10_EXT = 36074 	# GL/glext.h:4458
GL_COLOR_ATTACHMENT11_EXT = 36075 	# GL/glext.h:4459
GL_COLOR_ATTACHMENT12_EXT = 36076 	# GL/glext.h:4460
GL_COLOR_ATTACHMENT13_EXT = 36077 	# GL/glext.h:4461
GL_COLOR_ATTACHMENT14_EXT = 36078 	# GL/glext.h:4462
GL_COLOR_ATTACHMENT15_EXT = 36079 	# GL/glext.h:4463
GL_DEPTH_ATTACHMENT_EXT = 36096 	# GL/glext.h:4464
GL_STENCIL_ATTACHMENT_EXT = 36128 	# GL/glext.h:4465
GL_FRAMEBUFFER_EXT = 36160 	# GL/glext.h:4466
GL_RENDERBUFFER_EXT = 36161 	# GL/glext.h:4467
GL_RENDERBUFFER_WIDTH_EXT = 36162 	# GL/glext.h:4468
GL_RENDERBUFFER_HEIGHT_EXT = 36163 	# GL/glext.h:4469
GL_RENDERBUFFER_INTERNAL_FORMAT_EXT = 36164 	# GL/glext.h:4470
GL_STENCIL_INDEX1_EXT = 36166 	# GL/glext.h:4471
GL_STENCIL_INDEX4_EXT = 36167 	# GL/glext.h:4472
GL_STENCIL_INDEX8_EXT = 36168 	# GL/glext.h:4473
GL_STENCIL_INDEX16_EXT = 36169 	# GL/glext.h:4474
GL_RENDERBUFFER_RED_SIZE_EXT = 36176 	# GL/glext.h:4475
GL_RENDERBUFFER_GREEN_SIZE_EXT = 36177 	# GL/glext.h:4476
GL_RENDERBUFFER_BLUE_SIZE_EXT = 36178 	# GL/glext.h:4477
GL_RENDERBUFFER_ALPHA_SIZE_EXT = 36179 	# GL/glext.h:4478
GL_RENDERBUFFER_DEPTH_SIZE_EXT = 36180 	# GL/glext.h:4479
GL_RENDERBUFFER_STENCIL_SIZE_EXT = 36181 	# GL/glext.h:4480
# GREMEDY_string_marker (GL/glext.h:4483)
# EXT_packed_depth_stencil (GL/glext.h:4486)
GL_DEPTH_STENCIL_EXT = 34041 	# GL/glext.h:4487
GL_UNSIGNED_INT_24_8_EXT = 34042 	# GL/glext.h:4488
GL_DEPTH24_STENCIL8_EXT = 35056 	# GL/glext.h:4489
GL_TEXTURE_STENCIL_SIZE_EXT = 35057 	# GL/glext.h:4490
# EXT_stencil_clear_tag (GL/glext.h:4493)
GL_STENCIL_TAG_BITS_EXT = 35058 	# GL/glext.h:4494
GL_STENCIL_CLEAR_TAG_VALUE_EXT = 35059 	# GL/glext.h:4495
# EXT_texture_sRGB (GL/glext.h:4498)
GL_SRGB_EXT = 35904 	# GL/glext.h:4499
GL_SRGB8_EXT = 35905 	# GL/glext.h:4500
GL_SRGB_ALPHA_EXT = 35906 	# GL/glext.h:4501
GL_SRGB8_ALPHA8_EXT = 35907 	# GL/glext.h:4502
GL_SLUMINANCE_ALPHA_EXT = 35908 	# GL/glext.h:4503
GL_SLUMINANCE8_ALPHA8_EXT = 35909 	# GL/glext.h:4504
GL_SLUMINANCE_EXT = 35910 	# GL/glext.h:4505
GL_SLUMINANCE8_EXT = 35911 	# GL/glext.h:4506
GL_COMPRESSED_SRGB_EXT = 35912 	# GL/glext.h:4507
GL_COMPRESSED_SRGB_ALPHA_EXT = 35913 	# GL/glext.h:4508
GL_COMPRESSED_SLUMINANCE_EXT = 35914 	# GL/glext.h:4509
GL_COMPRESSED_SLUMINANCE_ALPHA_EXT = 35915 	# GL/glext.h:4510
GL_COMPRESSED_SRGB_S3TC_DXT1_EXT = 35916 	# GL/glext.h:4511
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT = 35917 	# GL/glext.h:4512
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT = 35918 	# GL/glext.h:4513
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT = 35919 	# GL/glext.h:4514
# EXT_framebuffer_blit (GL/glext.h:4517)
GL_READ_FRAMEBUFFER_EXT = 36008 	# GL/glext.h:4518
GL_DRAW_FRAMEBUFFER_EXT = 36009 	# GL/glext.h:4519
GL_DRAW_FRAMEBUFFER_BINDING_EXT = 36006 	# GL/glext.h:4520
GL_READ_FRAMEBUFFER_BINDING_EXT = 36010 	# GL/glext.h:4521
# EXT_framebuffer_multisample (GL/glext.h:4524)
GL_RENDERBUFFER_SAMPLES_EXT = 36011 	# GL/glext.h:4525
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT = 36182 	# GL/glext.h:4526
GL_MAX_SAMPLES_EXT = 36183 	# GL/glext.h:4527
# MESAX_texture_stack (GL/glext.h:4530)
GL_TEXTURE_1D_STACK_MESAX = 34649 	# GL/glext.h:4531
GL_TEXTURE_2D_STACK_MESAX = 34650 	# GL/glext.h:4532
GL_PROXY_TEXTURE_1D_STACK_MESAX = 34651 	# GL/glext.h:4533
GL_PROXY_TEXTURE_2D_STACK_MESAX = 34652 	# GL/glext.h:4534
GL_TEXTURE_1D_STACK_BINDING_MESAX = 34653 	# GL/glext.h:4535
GL_TEXTURE_2D_STACK_BINDING_MESAX = 34654 	# GL/glext.h:4536
# EXT_timer_query (GL/glext.h:4539)
GL_TIME_ELAPSED_EXT = 35007 	# GL/glext.h:4540
# EXT_gpu_program_parameters (GL/glext.h:4543)
# APPLE_flush_buffer_range (GL/glext.h:4546)
GL_BUFFER_SERIALIZED_MODIFY_APPLE = 35346 	# GL/glext.h:4547
GL_BUFFER_FLUSHING_UNMAP_APPLE = 35347 	# GL/glext.h:4548
# NV_gpu_program4 (GL/glext.h:4551)
GL_MIN_PROGRAM_TEXEL_OFFSET_NV = 35076 	# GL/glext.h:4552
GL_MAX_PROGRAM_TEXEL_OFFSET_NV = 35077 	# GL/glext.h:4553
GL_PROGRAM_ATTRIB_COMPONENTS_NV = 35078 	# GL/glext.h:4554
GL_PROGRAM_RESULT_COMPONENTS_NV = 35079 	# GL/glext.h:4555
GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV = 35080 	# GL/glext.h:4556
GL_MAX_PROGRAM_RESULT_COMPONENTS_NV = 35081 	# GL/glext.h:4557
GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV = 36261 	# GL/glext.h:4558
GL_MAX_PROGRAM_GENERIC_RESULTS_NV = 36262 	# GL/glext.h:4559
# NV_geometry_program4 (GL/glext.h:4562)
GL_LINES_ADJACENCY_EXT = 10 	# GL/glext.h:4563
GL_LINE_STRIP_ADJACENCY_EXT = 11 	# GL/glext.h:4564
GL_TRIANGLES_ADJACENCY_EXT = 12 	# GL/glext.h:4565
GL_TRIANGLE_STRIP_ADJACENCY_EXT = 13 	# GL/glext.h:4566
GL_GEOMETRY_PROGRAM_NV = 35878 	# GL/glext.h:4567
GL_MAX_PROGRAM_OUTPUT_VERTICES_NV = 35879 	# GL/glext.h:4568
GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV = 35880 	# GL/glext.h:4569
GL_GEOMETRY_VERTICES_OUT_EXT = 36314 	# GL/glext.h:4570
GL_GEOMETRY_INPUT_TYPE_EXT = 36315 	# GL/glext.h:4571
GL_GEOMETRY_OUTPUT_TYPE_EXT = 36316 	# GL/glext.h:4572
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT = 35881 	# GL/glext.h:4573
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT = 36263 	# GL/glext.h:4574
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT = 36264 	# GL/glext.h:4575
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT = 36265 	# GL/glext.h:4576
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT = 36052 	# GL/glext.h:4577
GL_PROGRAM_POINT_SIZE_EXT = 34370 	# GL/glext.h:4578
# EXT_geometry_shader4 (GL/glext.h:4581)
GL_GEOMETRY_SHADER_EXT = 36313 	# GL/glext.h:4582
GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT = 36317 	# GL/glext.h:4587
GL_MAX_VERTEX_VARYING_COMPONENTS_EXT = 36318 	# GL/glext.h:4588
GL_MAX_VARYING_COMPONENTS_EXT = 35659 	# GL/glext.h:4589
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT = 36319 	# GL/glext.h:4590
GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT = 36320 	# GL/glext.h:4591
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT = 36321 	# GL/glext.h:4592
# NV_vertex_program4 (GL/glext.h:4604)
GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV = 35069 	# GL/glext.h:4605
# EXT_gpu_shader4 (GL/glext.h:4608)
GL_SAMPLER_1D_ARRAY_EXT = 36288 	# GL/glext.h:4609
GL_SAMPLER_2D_ARRAY_EXT = 36289 	# GL/glext.h:4610
GL_SAMPLER_BUFFER_EXT = 36290 	# GL/glext.h:4611
GL_SAMPLER_1D_ARRAY_SHADOW_EXT = 36291 	# GL/glext.h:4612
GL_SAMPLER_2D_ARRAY_SHADOW_EXT = 36292 	# GL/glext.h:4613
GL_SAMPLER_CUBE_SHADOW_EXT = 36293 	# GL/glext.h:4614
GL_UNSIGNED_INT_VEC2_EXT = 36294 	# GL/glext.h:4615
GL_UNSIGNED_INT_VEC3_EXT = 36295 	# GL/glext.h:4616
GL_UNSIGNED_INT_VEC4_EXT = 36296 	# GL/glext.h:4617
GL_INT_SAMPLER_1D_EXT = 36297 	# GL/glext.h:4618
GL_INT_SAMPLER_2D_EXT = 36298 	# GL/glext.h:4619
GL_INT_SAMPLER_3D_EXT = 36299 	# GL/glext.h:4620
GL_INT_SAMPLER_CUBE_EXT = 36300 	# GL/glext.h:4621
GL_INT_SAMPLER_2D_RECT_EXT = 36301 	# GL/glext.h:4622
GL_INT_SAMPLER_1D_ARRAY_EXT = 36302 	# GL/glext.h:4623
GL_INT_SAMPLER_2D_ARRAY_EXT = 36303 	# GL/glext.h:4624
GL_INT_SAMPLER_BUFFER_EXT = 36304 	# GL/glext.h:4625
GL_UNSIGNED_INT_SAMPLER_1D_EXT = 36305 	# GL/glext.h:4626
GL_UNSIGNED_INT_SAMPLER_2D_EXT = 36306 	# GL/glext.h:4627
GL_UNSIGNED_INT_SAMPLER_3D_EXT = 36307 	# GL/glext.h:4628
GL_UNSIGNED_INT_SAMPLER_CUBE_EXT = 36308 	# GL/glext.h:4629
GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT = 36309 	# GL/glext.h:4630
GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT = 36310 	# GL/glext.h:4631
GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT = 36311 	# GL/glext.h:4632
GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT = 36312 	# GL/glext.h:4633
# EXT_draw_instanced (GL/glext.h:4636)
# EXT_packed_float (GL/glext.h:4639)
GL_R11F_G11F_B10F_EXT = 35898 	# GL/glext.h:4640
GL_UNSIGNED_INT_10F_11F_11F_REV_EXT = 35899 	# GL/glext.h:4641
GL_RGBA_SIGNED_COMPONENTS_EXT = 35900 	# GL/glext.h:4642
# EXT_texture_array (GL/glext.h:4645)
GL_TEXTURE_1D_ARRAY_EXT = 35864 	# GL/glext.h:4646
GL_PROXY_TEXTURE_1D_ARRAY_EXT = 35865 	# GL/glext.h:4647
GL_TEXTURE_2D_ARRAY_EXT = 35866 	# GL/glext.h:4648
GL_PROXY_TEXTURE_2D_ARRAY_EXT = 35867 	# GL/glext.h:4649
GL_TEXTURE_BINDING_1D_ARRAY_EXT = 35868 	# GL/glext.h:4650
GL_TEXTURE_BINDING_2D_ARRAY_EXT = 35869 	# GL/glext.h:4651
GL_MAX_ARRAY_TEXTURE_LAYERS_EXT = 35071 	# GL/glext.h:4652
GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT = 34894 	# GL/glext.h:4653
# EXT_texture_buffer_object (GL/glext.h:4657)
GL_TEXTURE_BUFFER_EXT = 35882 	# GL/glext.h:4658
GL_MAX_TEXTURE_BUFFER_SIZE_EXT = 35883 	# GL/glext.h:4659
GL_TEXTURE_BINDING_BUFFER_EXT = 35884 	# GL/glext.h:4660
GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT = 35885 	# GL/glext.h:4661
GL_TEXTURE_BUFFER_FORMAT_EXT = 35886 	# GL/glext.h:4662
# EXT_texture_compression_latc (GL/glext.h:4665)
GL_COMPRESSED_LUMINANCE_LATC1_EXT = 35952 	# GL/glext.h:4666
GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT = 35953 	# GL/glext.h:4667
GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT = 35954 	# GL/glext.h:4668
GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT = 35955 	# GL/glext.h:4669
# EXT_texture_compression_rgtc (GL/glext.h:4672)
GL_COMPRESSED_RED_RGTC1_EXT = 36283 	# GL/glext.h:4673
GL_COMPRESSED_SIGNED_RED_RGTC1_EXT = 36284 	# GL/glext.h:4674
GL_COMPRESSED_RED_GREEN_RGTC2_EXT = 36285 	# GL/glext.h:4675
GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT = 36286 	# GL/glext.h:4676
# EXT_texture_shared_exponent (GL/glext.h:4679)
GL_RGB9_E5_EXT = 35901 	# GL/glext.h:4680
GL_UNSIGNED_INT_5_9_9_9_REV_EXT = 35902 	# GL/glext.h:4681
GL_TEXTURE_SHARED_SIZE_EXT = 35903 	# GL/glext.h:4682
# NV_depth_buffer_float (GL/glext.h:4685)
GL_DEPTH_COMPONENT32F_NV = 36267 	# GL/glext.h:4686
GL_DEPTH32F_STENCIL8_NV = 36268 	# GL/glext.h:4687
GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV = 36269 	# GL/glext.h:4688
GL_DEPTH_BUFFER_FLOAT_MODE_NV = 36271 	# GL/glext.h:4689
# NV_fragment_program4 (GL/glext.h:4692)
# NV_framebuffer_multisample_coverage (GL/glext.h:4695)
GL_RENDERBUFFER_COVERAGE_SAMPLES_NV = 36011 	# GL/glext.h:4696
GL_RENDERBUFFER_COLOR_SAMPLES_NV = 36368 	# GL/glext.h:4697
GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV = 36369 	# GL/glext.h:4698
GL_MULTISAMPLE_COVERAGE_MODES_NV = 36370 	# GL/glext.h:4699
# EXT_framebuffer_sRGB (GL/glext.h:4702)
GL_FRAMEBUFFER_SRGB_EXT = 36281 	# GL/glext.h:4703
GL_FRAMEBUFFER_SRGB_CAPABLE_EXT = 36282 	# GL/glext.h:4704
# NV_geometry_shader4 (GL/glext.h:4707)
# NV_parameter_buffer_object (GL/glext.h:4710)
GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV = 36256 	# GL/glext.h:4711
GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV = 36257 	# GL/glext.h:4712
GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV = 36258 	# GL/glext.h:4713
GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV = 36259 	# GL/glext.h:4714
GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV = 36260 	# GL/glext.h:4715
# EXT_draw_buffers2 (GL/glext.h:4718)
# NV_transform_feedback (GL/glext.h:4721)
GL_BACK_PRIMARY_COLOR_NV = 35959 	# GL/glext.h:4722
GL_BACK_SECONDARY_COLOR_NV = 35960 	# GL/glext.h:4723
GL_TEXTURE_COORD_NV = 35961 	# GL/glext.h:4724
GL_CLIP_DISTANCE_NV = 35962 	# GL/glext.h:4725
GL_VERTEX_ID_NV = 35963 	# GL/glext.h:4726
GL_PRIMITIVE_ID_NV = 35964 	# GL/glext.h:4727
GL_GENERIC_ATTRIB_NV = 35965 	# GL/glext.h:4728
GL_TRANSFORM_FEEDBACK_ATTRIBS_NV = 35966 	# GL/glext.h:4729
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV = 35967 	# GL/glext.h:4730
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV = 35968 	# GL/glext.h:4731
GL_ACTIVE_VARYINGS_NV = 35969 	# GL/glext.h:4732
GL_ACTIVE_VARYING_MAX_LENGTH_NV = 35970 	# GL/glext.h:4733
GL_TRANSFORM_FEEDBACK_VARYINGS_NV = 35971 	# GL/glext.h:4734
GL_TRANSFORM_FEEDBACK_BUFFER_START_NV = 35972 	# GL/glext.h:4735
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV = 35973 	# GL/glext.h:4736
GL_TRANSFORM_FEEDBACK_RECORD_NV = 35974 	# GL/glext.h:4737
GL_PRIMITIVES_GENERATED_NV = 35975 	# GL/glext.h:4738
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV = 35976 	# GL/glext.h:4739
GL_RASTERIZER_DISCARD_NV = 35977 	# GL/glext.h:4740
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV = 35978 	# GL/glext.h:4741
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV = 35979 	# GL/glext.h:4742
GL_INTERLEAVED_ATTRIBS_NV = 35980 	# GL/glext.h:4743
GL_SEPARATE_ATTRIBS_NV = 35981 	# GL/glext.h:4744
GL_TRANSFORM_FEEDBACK_BUFFER_NV = 35982 	# GL/glext.h:4745
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV = 35983 	# GL/glext.h:4746
GL_LAYER_NV = 36266 	# GL/glext.h:4747
GL_NEXT_BUFFER_NV = -2 	# GL/glext.h:4748
GL_SKIP_COMPONENTS4_NV = -3 	# GL/glext.h:4749
GL_SKIP_COMPONENTS3_NV = -4 	# GL/glext.h:4750
GL_SKIP_COMPONENTS2_NV = -5 	# GL/glext.h:4751
GL_SKIP_COMPONENTS1_NV = -6 	# GL/glext.h:4752
# EXT_bindable_uniform (GL/glext.h:4755)
GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT = 36322 	# GL/glext.h:4756
GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT = 36323 	# GL/glext.h:4757
GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT = 36324 	# GL/glext.h:4758
GL_MAX_BINDABLE_UNIFORM_SIZE_EXT = 36333 	# GL/glext.h:4759
GL_UNIFORM_BUFFER_EXT = 36334 	# GL/glext.h:4760
GL_UNIFORM_BUFFER_BINDING_EXT = 36335 	# GL/glext.h:4761
# EXT_texture_integer (GL/glext.h:4764)
GL_RGBA32UI_EXT = 36208 	# GL/glext.h:4765
GL_RGB32UI_EXT = 36209 	# GL/glext.h:4766
GL_ALPHA32UI_EXT = 36210 	# GL/glext.h:4767
GL_INTENSITY32UI_EXT = 36211 	# GL/glext.h:4768
GL_LUMINANCE32UI_EXT = 36212 	# GL/glext.h:4769
GL_LUMINANCE_ALPHA32UI_EXT = 36213 	# GL/glext.h:4770
GL_RGBA16UI_EXT = 36214 	# GL/glext.h:4771
GL_RGB16UI_EXT = 36215 	# GL/glext.h:4772
GL_ALPHA16UI_EXT = 36216 	# GL/glext.h:4773
GL_INTENSITY16UI_EXT = 36217 	# GL/glext.h:4774
GL_LUMINANCE16UI_EXT = 36218 	# GL/glext.h:4775
GL_LUMINANCE_ALPHA16UI_EXT = 36219 	# GL/glext.h:4776
GL_RGBA8UI_EXT = 36220 	# GL/glext.h:4777
GL_RGB8UI_EXT = 36221 	# GL/glext.h:4778
GL_ALPHA8UI_EXT = 36222 	# GL/glext.h:4779
GL_INTENSITY8UI_EXT = 36223 	# GL/glext.h:4780
GL_LUMINANCE8UI_EXT = 36224 	# GL/glext.h:4781
GL_LUMINANCE_ALPHA8UI_EXT = 36225 	# GL/glext.h:4782
GL_RGBA32I_EXT = 36226 	# GL/glext.h:4783
GL_RGB32I_EXT = 36227 	# GL/glext.h:4784
GL_ALPHA32I_EXT = 36228 	# GL/glext.h:4785
GL_INTENSITY32I_EXT = 36229 	# GL/glext.h:4786
GL_LUMINANCE32I_EXT = 36230 	# GL/glext.h:4787
GL_LUMINANCE_ALPHA32I_EXT = 36231 	# GL/glext.h:4788
GL_RGBA16I_EXT = 36232 	# GL/glext.h:4789
GL_RGB16I_EXT = 36233 	# GL/glext.h:4790
GL_ALPHA16I_EXT = 36234 	# GL/glext.h:4791
GL_INTENSITY16I_EXT = 36235 	# GL/glext.h:4792
GL_LUMINANCE16I_EXT = 36236 	# GL/glext.h:4793
GL_LUMINANCE_ALPHA16I_EXT = 36237 	# GL/glext.h:4794
GL_RGBA8I_EXT = 36238 	# GL/glext.h:4795
GL_RGB8I_EXT = 36239 	# GL/glext.h:4796
GL_ALPHA8I_EXT = 36240 	# GL/glext.h:4797
GL_INTENSITY8I_EXT = 36241 	# GL/glext.h:4798
GL_LUMINANCE8I_EXT = 36242 	# GL/glext.h:4799
GL_LUMINANCE_ALPHA8I_EXT = 36243 	# GL/glext.h:4800
GL_RED_INTEGER_EXT = 36244 	# GL/glext.h:4801
GL_GREEN_INTEGER_EXT = 36245 	# GL/glext.h:4802
GL_BLUE_INTEGER_EXT = 36246 	# GL/glext.h:4803
GL_ALPHA_INTEGER_EXT = 36247 	# GL/glext.h:4804
GL_RGB_INTEGER_EXT = 36248 	# GL/glext.h:4805
GL_RGBA_INTEGER_EXT = 36249 	# GL/glext.h:4806
GL_BGR_INTEGER_EXT = 36250 	# GL/glext.h:4807
GL_BGRA_INTEGER_EXT = 36251 	# GL/glext.h:4808
GL_LUMINANCE_INTEGER_EXT = 36252 	# GL/glext.h:4809
GL_LUMINANCE_ALPHA_INTEGER_EXT = 36253 	# GL/glext.h:4810
GL_RGBA_INTEGER_MODE_EXT = 36254 	# GL/glext.h:4811
# GREMEDY_frame_terminator (GL/glext.h:4814)
# NV_conditional_render (GL/glext.h:4817)
GL_QUERY_WAIT_NV = 36371 	# GL/glext.h:4818
GL_QUERY_NO_WAIT_NV = 36372 	# GL/glext.h:4819
GL_QUERY_BY_REGION_WAIT_NV = 36373 	# GL/glext.h:4820
GL_QUERY_BY_REGION_NO_WAIT_NV = 36374 	# GL/glext.h:4821
# NV_present_video (GL/glext.h:4824)
GL_FRAME_NV = 36390 	# GL/glext.h:4825
GL_FIELDS_NV = 36391 	# GL/glext.h:4826
GL_CURRENT_TIME_NV = 36392 	# GL/glext.h:4827
GL_NUM_FILL_STREAMS_NV = 36393 	# GL/glext.h:4828
GL_PRESENT_TIME_NV = 36394 	# GL/glext.h:4829
GL_PRESENT_DURATION_NV = 36395 	# GL/glext.h:4830
# EXT_transform_feedback (GL/glext.h:4833)
GL_TRANSFORM_FEEDBACK_BUFFER_EXT = 35982 	# GL/glext.h:4834
GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT = 35972 	# GL/glext.h:4835
GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT = 35973 	# GL/glext.h:4836
GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT = 35983 	# GL/glext.h:4837
GL_INTERLEAVED_ATTRIBS_EXT = 35980 	# GL/glext.h:4838
GL_SEPARATE_ATTRIBS_EXT = 35981 	# GL/glext.h:4839
GL_PRIMITIVES_GENERATED_EXT = 35975 	# GL/glext.h:4840
GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT = 35976 	# GL/glext.h:4841
GL_RASTERIZER_DISCARD_EXT = 35977 	# GL/glext.h:4842
GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT = 35978 	# GL/glext.h:4843
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT = 35979 	# GL/glext.h:4844
GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT = 35968 	# GL/glext.h:4845
GL_TRANSFORM_FEEDBACK_VARYINGS_EXT = 35971 	# GL/glext.h:4846
GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT = 35967 	# GL/glext.h:4847
GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT = 35958 	# GL/glext.h:4848
# EXT_direct_state_access (GL/glext.h:4851)
GL_PROGRAM_MATRIX_EXT = 36397 	# GL/glext.h:4852
GL_TRANSPOSE_PROGRAM_MATRIX_EXT = 36398 	# GL/glext.h:4853
GL_PROGRAM_MATRIX_STACK_DEPTH_EXT = 36399 	# GL/glext.h:4854
# EXT_vertex_array_bgra (GL/glext.h:4857)
# EXT_texture_swizzle (GL/glext.h:4861)
GL_TEXTURE_SWIZZLE_R_EXT = 36418 	# GL/glext.h:4862
GL_TEXTURE_SWIZZLE_G_EXT = 36419 	# GL/glext.h:4863
GL_TEXTURE_SWIZZLE_B_EXT = 36420 	# GL/glext.h:4864
GL_TEXTURE_SWIZZLE_A_EXT = 36421 	# GL/glext.h:4865
GL_TEXTURE_SWIZZLE_RGBA_EXT = 36422 	# GL/glext.h:4866
# NV_explicit_multisample (GL/glext.h:4869)
GL_SAMPLE_POSITION_NV = 36432 	# GL/glext.h:4870
GL_SAMPLE_MASK_NV = 36433 	# GL/glext.h:4871
GL_SAMPLE_MASK_VALUE_NV = 36434 	# GL/glext.h:4872
GL_TEXTURE_BINDING_RENDERBUFFER_NV = 36435 	# GL/glext.h:4873
GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV = 36436 	# GL/glext.h:4874
GL_TEXTURE_RENDERBUFFER_NV = 36437 	# GL/glext.h:4875
GL_SAMPLER_RENDERBUFFER_NV = 36438 	# GL/glext.h:4876
GL_INT_SAMPLER_RENDERBUFFER_NV = 36439 	# GL/glext.h:4877
GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV = 36440 	# GL/glext.h:4878
GL_MAX_SAMPLE_MASK_WORDS_NV = 36441 	# GL/glext.h:4879
# NV_transform_feedback2 (GL/glext.h:4882)
GL_TRANSFORM_FEEDBACK_NV = 36386 	# GL/glext.h:4883
GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV = 36387 	# GL/glext.h:4884
GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV = 36388 	# GL/glext.h:4885
GL_TRANSFORM_FEEDBACK_BINDING_NV = 36389 	# GL/glext.h:4886
# ATI_meminfo (GL/glext.h:4889)
GL_VBO_FREE_MEMORY_ATI = 34811 	# GL/glext.h:4890
GL_TEXTURE_FREE_MEMORY_ATI = 34812 	# GL/glext.h:4891
GL_RENDERBUFFER_FREE_MEMORY_ATI = 34813 	# GL/glext.h:4892
# AMD_performance_monitor (GL/glext.h:4895)
GL_COUNTER_TYPE_AMD = 35776 	# GL/glext.h:4896
GL_COUNTER_RANGE_AMD = 35777 	# GL/glext.h:4897
GL_UNSIGNED_INT64_AMD = 35778 	# GL/glext.h:4898
GL_PERCENTAGE_AMD = 35779 	# GL/glext.h:4899
GL_PERFMON_RESULT_AVAILABLE_AMD = 35780 	# GL/glext.h:4900
GL_PERFMON_RESULT_SIZE_AMD = 35781 	# GL/glext.h:4901
GL_PERFMON_RESULT_AMD = 35782 	# GL/glext.h:4902
# AMD_texture_texture4 (GL/glext.h:4905)
# AMD_vertex_shader_tesselator (GL/glext.h:4908)
GL_SAMPLER_BUFFER_AMD = 36865 	# GL/glext.h:4909
GL_INT_SAMPLER_BUFFER_AMD = 36866 	# GL/glext.h:4910
GL_UNSIGNED_INT_SAMPLER_BUFFER_AMD = 36867 	# GL/glext.h:4911
GL_TESSELLATION_MODE_AMD = 36868 	# GL/glext.h:4912
GL_TESSELLATION_FACTOR_AMD = 36869 	# GL/glext.h:4913
GL_DISCRETE_AMD = 36870 	# GL/glext.h:4914
GL_CONTINUOUS_AMD = 36871 	# GL/glext.h:4915
# EXT_provoking_vertex (GL/glext.h:4918)
GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION_EXT = 36428 	# GL/glext.h:4919
GL_FIRST_VERTEX_CONVENTION_EXT = 36429 	# GL/glext.h:4920
GL_LAST_VERTEX_CONVENTION_EXT = 36430 	# GL/glext.h:4921
GL_PROVOKING_VERTEX_EXT = 36431 	# GL/glext.h:4922
# EXT_texture_snorm (GL/glext.h:4925)
GL_ALPHA_SNORM = 36880 	# GL/glext.h:4926
GL_LUMINANCE_SNORM = 36881 	# GL/glext.h:4927
GL_LUMINANCE_ALPHA_SNORM = 36882 	# GL/glext.h:4928
GL_INTENSITY_SNORM = 36883 	# GL/glext.h:4929
GL_ALPHA8_SNORM = 36884 	# GL/glext.h:4930
GL_LUMINANCE8_SNORM = 36885 	# GL/glext.h:4931
GL_LUMINANCE8_ALPHA8_SNORM = 36886 	# GL/glext.h:4932
GL_INTENSITY8_SNORM = 36887 	# GL/glext.h:4933
GL_ALPHA16_SNORM = 36888 	# GL/glext.h:4934
GL_LUMINANCE16_SNORM = 36889 	# GL/glext.h:4935
GL_LUMINANCE16_ALPHA16_SNORM = 36890 	# GL/glext.h:4936
GL_INTENSITY16_SNORM = 36891 	# GL/glext.h:4937
# AMD_draw_buffers_blend (GL/glext.h:4953)
# APPLE_texture_range (GL/glext.h:4956)
GL_TEXTURE_RANGE_LENGTH_APPLE = 34231 	# GL/glext.h:4957
GL_TEXTURE_RANGE_POINTER_APPLE = 34232 	# GL/glext.h:4958
GL_TEXTURE_STORAGE_HINT_APPLE = 34236 	# GL/glext.h:4959
GL_STORAGE_PRIVATE_APPLE = 34237 	# GL/glext.h:4960
# APPLE_float_pixels (GL/glext.h:4965)
GL_HALF_APPLE = 5131 	# GL/glext.h:4966
GL_RGBA_FLOAT32_APPLE = 34836 	# GL/glext.h:4967
GL_RGB_FLOAT32_APPLE = 34837 	# GL/glext.h:4968
GL_ALPHA_FLOAT32_APPLE = 34838 	# GL/glext.h:4969
GL_INTENSITY_FLOAT32_APPLE = 34839 	# GL/glext.h:4970
GL_LUMINANCE_FLOAT32_APPLE = 34840 	# GL/glext.h:4971
GL_LUMINANCE_ALPHA_FLOAT32_APPLE = 34841 	# GL/glext.h:4972
GL_RGBA_FLOAT16_APPLE = 34842 	# GL/glext.h:4973
GL_RGB_FLOAT16_APPLE = 34843 	# GL/glext.h:4974
GL_ALPHA_FLOAT16_APPLE = 34844 	# GL/glext.h:4975
GL_INTENSITY_FLOAT16_APPLE = 34845 	# GL/glext.h:4976
GL_LUMINANCE_FLOAT16_APPLE = 34846 	# GL/glext.h:4977
GL_LUMINANCE_ALPHA_FLOAT16_APPLE = 34847 	# GL/glext.h:4978
GL_COLOR_FLOAT_APPLE = 35343 	# GL/glext.h:4979
# APPLE_vertex_program_evaluators (GL/glext.h:4982)
GL_VERTEX_ATTRIB_MAP1_APPLE = 35328 	# GL/glext.h:4983
GL_VERTEX_ATTRIB_MAP2_APPLE = 35329 	# GL/glext.h:4984
GL_VERTEX_ATTRIB_MAP1_SIZE_APPLE = 35330 	# GL/glext.h:4985
GL_VERTEX_ATTRIB_MAP1_COEFF_APPLE = 35331 	# GL/glext.h:4986
GL_VERTEX_ATTRIB_MAP1_ORDER_APPLE = 35332 	# GL/glext.h:4987
GL_VERTEX_ATTRIB_MAP1_DOMAIN_APPLE = 35333 	# GL/glext.h:4988
GL_VERTEX_ATTRIB_MAP2_SIZE_APPLE = 35334 	# GL/glext.h:4989
GL_VERTEX_ATTRIB_MAP2_COEFF_APPLE = 35335 	# GL/glext.h:4990
GL_VERTEX_ATTRIB_MAP2_ORDER_APPLE = 35336 	# GL/glext.h:4991
GL_VERTEX_ATTRIB_MAP2_DOMAIN_APPLE = 35337 	# GL/glext.h:4992
# APPLE_aux_depth_stencil (GL/glext.h:4995)
GL_AUX_DEPTH_STENCIL_APPLE = 35348 	# GL/glext.h:4996
# APPLE_object_purgeable (GL/glext.h:4999)
GL_BUFFER_OBJECT_APPLE = 34227 	# GL/glext.h:5000
GL_RELEASED_APPLE = 35353 	# GL/glext.h:5001
GL_VOLATILE_APPLE = 35354 	# GL/glext.h:5002
GL_RETAINED_APPLE = 35355 	# GL/glext.h:5003
GL_UNDEFINED_APPLE = 35356 	# GL/glext.h:5004
GL_PURGEABLE_APPLE = 35357 	# GL/glext.h:5005
# APPLE_row_bytes (GL/glext.h:5008)
GL_PACK_ROW_BYTES_APPLE = 35349 	# GL/glext.h:5009
GL_UNPACK_ROW_BYTES_APPLE = 35350 	# GL/glext.h:5010
# APPLE_rgb_422 (GL/glext.h:5013)
GL_RGB_422_APPLE = 35359 	# GL/glext.h:5014
# NV_video_capture (GL/glext.h:5019)
GL_VIDEO_BUFFER_NV = 36896 	# GL/glext.h:5020
GL_VIDEO_BUFFER_BINDING_NV = 36897 	# GL/glext.h:5021
GL_FIELD_UPPER_NV = 36898 	# GL/glext.h:5022
GL_FIELD_LOWER_NV = 36899 	# GL/glext.h:5023
GL_NUM_VIDEO_CAPTURE_STREAMS_NV = 36900 	# GL/glext.h:5024
GL_NEXT_VIDEO_CAPTURE_BUFFER_STATUS_NV = 36901 	# GL/glext.h:5025
GL_VIDEO_CAPTURE_TO_422_SUPPORTED_NV = 36902 	# GL/glext.h:5026
GL_LAST_VIDEO_CAPTURE_STATUS_NV = 36903 	# GL/glext.h:5027
GL_VIDEO_BUFFER_PITCH_NV = 36904 	# GL/glext.h:5028
GL_VIDEO_COLOR_CONVERSION_MATRIX_NV = 36905 	# GL/glext.h:5029
GL_VIDEO_COLOR_CONVERSION_MAX_NV = 36906 	# GL/glext.h:5030
GL_VIDEO_COLOR_CONVERSION_MIN_NV = 36907 	# GL/glext.h:5031
GL_VIDEO_COLOR_CONVERSION_OFFSET_NV = 36908 	# GL/glext.h:5032
GL_VIDEO_BUFFER_INTERNAL_FORMAT_NV = 36909 	# GL/glext.h:5033
GL_PARTIAL_SUCCESS_NV = 36910 	# GL/glext.h:5034
GL_SUCCESS_NV = 36911 	# GL/glext.h:5035
GL_FAILURE_NV = 36912 	# GL/glext.h:5036
GL_YCBYCR8_422_NV = 36913 	# GL/glext.h:5037
GL_YCBAYCR8A_4224_NV = 36914 	# GL/glext.h:5038
GL_Z6Y10Z6CB10Z6Y10Z6CR10_422_NV = 36915 	# GL/glext.h:5039
GL_Z6Y10Z6CB10Z6A10Z6Y10Z6CR10Z6A10_4224_NV = 36916 	# GL/glext.h:5040
GL_Z4Y12Z4CB12Z4Y12Z4CR12_422_NV = 36917 	# GL/glext.h:5041
GL_Z4Y12Z4CB12Z4A12Z4Y12Z4CR12Z4A12_4224_NV = 36918 	# GL/glext.h:5042
GL_Z4Y12Z4CB12Z4CR12_444_NV = 36919 	# GL/glext.h:5043
GL_VIDEO_CAPTURE_FRAME_WIDTH_NV = 36920 	# GL/glext.h:5044
GL_VIDEO_CAPTURE_FRAME_HEIGHT_NV = 36921 	# GL/glext.h:5045
GL_VIDEO_CAPTURE_FIELD_UPPER_HEIGHT_NV = 36922 	# GL/glext.h:5046
GL_VIDEO_CAPTURE_FIELD_LOWER_HEIGHT_NV = 36923 	# GL/glext.h:5047
GL_VIDEO_CAPTURE_SURFACE_ORIGIN_NV = 36924 	# GL/glext.h:5048
# NV_copy_image (GL/glext.h:5051)
# EXT_separate_shader_objects (GL/glext.h:5054)
GL_ACTIVE_PROGRAM_EXT = 35725 	# GL/glext.h:5055
# NV_parameter_buffer_object2 (GL/glext.h:5058)
# NV_shader_buffer_load (GL/glext.h:5061)
GL_BUFFER_GPU_ADDRESS_NV = 36637 	# GL/glext.h:5062
GL_GPU_ADDRESS_NV = 36660 	# GL/glext.h:5063
GL_MAX_SHADER_BUFFER_ADDRESS_NV = 36661 	# GL/glext.h:5064
# NV_vertex_buffer_unified_memory (GL/glext.h:5067)
GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV = 36638 	# GL/glext.h:5068
GL_ELEMENT_ARRAY_UNIFIED_NV = 36639 	# GL/glext.h:5069
GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV = 36640 	# GL/glext.h:5070
GL_VERTEX_ARRAY_ADDRESS_NV = 36641 	# GL/glext.h:5071
GL_NORMAL_ARRAY_ADDRESS_NV = 36642 	# GL/glext.h:5072
GL_COLOR_ARRAY_ADDRESS_NV = 36643 	# GL/glext.h:5073
GL_INDEX_ARRAY_ADDRESS_NV = 36644 	# GL/glext.h:5074
GL_TEXTURE_COORD_ARRAY_ADDRESS_NV = 36645 	# GL/glext.h:5075
GL_EDGE_FLAG_ARRAY_ADDRESS_NV = 36646 	# GL/glext.h:5076
GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV = 36647 	# GL/glext.h:5077
GL_FOG_COORD_ARRAY_ADDRESS_NV = 36648 	# GL/glext.h:5078
GL_ELEMENT_ARRAY_ADDRESS_NV = 36649 	# GL/glext.h:5079
GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV = 36650 	# GL/glext.h:5080
GL_VERTEX_ARRAY_LENGTH_NV = 36651 	# GL/glext.h:5081
GL_NORMAL_ARRAY_LENGTH_NV = 36652 	# GL/glext.h:5082
GL_COLOR_ARRAY_LENGTH_NV = 36653 	# GL/glext.h:5083
GL_INDEX_ARRAY_LENGTH_NV = 36654 	# GL/glext.h:5084
GL_TEXTURE_COORD_ARRAY_LENGTH_NV = 36655 	# GL/glext.h:5085
GL_EDGE_FLAG_ARRAY_LENGTH_NV = 36656 	# GL/glext.h:5086
GL_SECONDARY_COLOR_ARRAY_LENGTH_NV = 36657 	# GL/glext.h:5087
GL_FOG_COORD_ARRAY_LENGTH_NV = 36658 	# GL/glext.h:5088
GL_ELEMENT_ARRAY_LENGTH_NV = 36659 	# GL/glext.h:5089
GL_DRAW_INDIRECT_UNIFIED_NV = 36672 	# GL/glext.h:5090
GL_DRAW_INDIRECT_ADDRESS_NV = 36673 	# GL/glext.h:5091
GL_DRAW_INDIRECT_LENGTH_NV = 36674 	# GL/glext.h:5092
# NV_texture_barrier (GL/glext.h:5095)
# AMD_shader_stencil_export (GL/glext.h:5098)
# AMD_seamless_cubemap_per_texture (GL/glext.h:5101)
# AMD_conservative_depth (GL/glext.h:5105)
# EXT_shader_image_load_store (GL/glext.h:5108)
GL_MAX_IMAGE_UNITS_EXT = 36664 	# GL/glext.h:5109
GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS_EXT = 36665 	# GL/glext.h:5110
GL_IMAGE_BINDING_NAME_EXT = 36666 	# GL/glext.h:5111
GL_IMAGE_BINDING_LEVEL_EXT = 36667 	# GL/glext.h:5112
GL_IMAGE_BINDING_LAYERED_EXT = 36668 	# GL/glext.h:5113
GL_IMAGE_BINDING_LAYER_EXT = 36669 	# GL/glext.h:5114
GL_IMAGE_BINDING_ACCESS_EXT = 36670 	# GL/glext.h:5115
GL_IMAGE_1D_EXT = 36940 	# GL/glext.h:5116
GL_IMAGE_2D_EXT = 36941 	# GL/glext.h:5117
GL_IMAGE_3D_EXT = 36942 	# GL/glext.h:5118
GL_IMAGE_2D_RECT_EXT = 36943 	# GL/glext.h:5119
GL_IMAGE_CUBE_EXT = 36944 	# GL/glext.h:5120
GL_IMAGE_BUFFER_EXT = 36945 	# GL/glext.h:5121
GL_IMAGE_1D_ARRAY_EXT = 36946 	# GL/glext.h:5122
GL_IMAGE_2D_ARRAY_EXT = 36947 	# GL/glext.h:5123
GL_IMAGE_CUBE_MAP_ARRAY_EXT = 36948 	# GL/glext.h:5124
GL_IMAGE_2D_MULTISAMPLE_EXT = 36949 	# GL/glext.h:5125
GL_IMAGE_2D_MULTISAMPLE_ARRAY_EXT = 36950 	# GL/glext.h:5126
GL_INT_IMAGE_1D_EXT = 36951 	# GL/glext.h:5127
GL_INT_IMAGE_2D_EXT = 36952 	# GL/glext.h:5128
GL_INT_IMAGE_3D_EXT = 36953 	# GL/glext.h:5129
GL_INT_IMAGE_2D_RECT_EXT = 36954 	# GL/glext.h:5130
GL_INT_IMAGE_CUBE_EXT = 36955 	# GL/glext.h:5131
GL_INT_IMAGE_BUFFER_EXT = 36956 	# GL/glext.h:5132
GL_INT_IMAGE_1D_ARRAY_EXT = 36957 	# GL/glext.h:5133
GL_INT_IMAGE_2D_ARRAY_EXT = 36958 	# GL/glext.h:5134
GL_INT_IMAGE_CUBE_MAP_ARRAY_EXT = 36959 	# GL/glext.h:5135
GL_INT_IMAGE_2D_MULTISAMPLE_EXT = 36960 	# GL/glext.h:5136
GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY_EXT = 36961 	# GL/glext.h:5137
GL_UNSIGNED_INT_IMAGE_1D_EXT = 36962 	# GL/glext.h:5138
GL_UNSIGNED_INT_IMAGE_2D_EXT = 36963 	# GL/glext.h:5139
GL_UNSIGNED_INT_IMAGE_3D_EXT = 36964 	# GL/glext.h:5140
GL_UNSIGNED_INT_IMAGE_2D_RECT_EXT = 36965 	# GL/glext.h:5141
GL_UNSIGNED_INT_IMAGE_CUBE_EXT = 36966 	# GL/glext.h:5142
GL_UNSIGNED_INT_IMAGE_BUFFER_EXT = 36967 	# GL/glext.h:5143
GL_UNSIGNED_INT_IMAGE_1D_ARRAY_EXT = 36968 	# GL/glext.h:5144
GL_UNSIGNED_INT_IMAGE_2D_ARRAY_EXT = 36969 	# GL/glext.h:5145
GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY_EXT = 36970 	# GL/glext.h:5146
GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_EXT = 36971 	# GL/glext.h:5147
GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY_EXT = 36972 	# GL/glext.h:5148
GL_MAX_IMAGE_SAMPLES_EXT = 36973 	# GL/glext.h:5149
GL_IMAGE_BINDING_FORMAT_EXT = 36974 	# GL/glext.h:5150
GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT_EXT = 1 	# GL/glext.h:5151
GL_ELEMENT_ARRAY_BARRIER_BIT_EXT = 2 	# GL/glext.h:5152
GL_UNIFORM_BARRIER_BIT_EXT = 4 	# GL/glext.h:5153
GL_TEXTURE_FETCH_BARRIER_BIT_EXT = 8 	# GL/glext.h:5154
GL_SHADER_IMAGE_ACCESS_BARRIER_BIT_EXT = 32 	# GL/glext.h:5155
GL_COMMAND_BARRIER_BIT_EXT = 64 	# GL/glext.h:5156
GL_PIXEL_BUFFER_BARRIER_BIT_EXT = 128 	# GL/glext.h:5157
GL_TEXTURE_UPDATE_BARRIER_BIT_EXT = 256 	# GL/glext.h:5158
GL_BUFFER_UPDATE_BARRIER_BIT_EXT = 512 	# GL/glext.h:5159
GL_FRAMEBUFFER_BARRIER_BIT_EXT = 1024 	# GL/glext.h:5160
GL_TRANSFORM_FEEDBACK_BARRIER_BIT_EXT = 2048 	# GL/glext.h:5161
GL_ATOMIC_COUNTER_BARRIER_BIT_EXT = 4096 	# GL/glext.h:5162
GL_ALL_BARRIER_BITS_EXT = 4294967295 	# GL/glext.h:5163
# EXT_vertex_attrib_64bit (GL/glext.h:5166)
GL_DOUBLE_VEC2_EXT = 36860 	# GL/glext.h:5168
GL_DOUBLE_VEC3_EXT = 36861 	# GL/glext.h:5169
GL_DOUBLE_VEC4_EXT = 36862 	# GL/glext.h:5170
GL_DOUBLE_MAT2_EXT = 36678 	# GL/glext.h:5171
GL_DOUBLE_MAT3_EXT = 36679 	# GL/glext.h:5172
GL_DOUBLE_MAT4_EXT = 36680 	# GL/glext.h:5173
GL_DOUBLE_MAT2x3_EXT = 36681 	# GL/glext.h:5174
GL_DOUBLE_MAT2x4_EXT = 36682 	# GL/glext.h:5175
GL_DOUBLE_MAT3x2_EXT = 36683 	# GL/glext.h:5176
GL_DOUBLE_MAT3x4_EXT = 36684 	# GL/glext.h:5177
GL_DOUBLE_MAT4x2_EXT = 36685 	# GL/glext.h:5178
GL_DOUBLE_MAT4x3_EXT = 36686 	# GL/glext.h:5179
# NV_gpu_program5 (GL/glext.h:5182)
GL_MAX_GEOMETRY_PROGRAM_INVOCATIONS_NV = 36442 	# GL/glext.h:5183
GL_MIN_FRAGMENT_INTERPOLATION_OFFSET_NV = 36443 	# GL/glext.h:5184
GL_MAX_FRAGMENT_INTERPOLATION_OFFSET_NV = 36444 	# GL/glext.h:5185
GL_FRAGMENT_PROGRAM_INTERPOLATION_OFFSET_BITS_NV = 36445 	# GL/glext.h:5186
GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_NV = 36446 	# GL/glext.h:5187
GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_NV = 36447 	# GL/glext.h:5188
GL_MAX_PROGRAM_SUBROUTINE_PARAMETERS_NV = 36676 	# GL/glext.h:5189
GL_MAX_PROGRAM_SUBROUTINE_NUM_NV = 36677 	# GL/glext.h:5190
# NV_gpu_shader5 (GL/glext.h:5193)
GL_INT64_NV = 5134 	# GL/glext.h:5194
GL_UNSIGNED_INT64_NV = 5135 	# GL/glext.h:5195
GL_INT8_NV = 36832 	# GL/glext.h:5196
GL_INT8_VEC2_NV = 36833 	# GL/glext.h:5197
GL_INT8_VEC3_NV = 36834 	# GL/glext.h:5198
GL_INT8_VEC4_NV = 36835 	# GL/glext.h:5199
GL_INT16_NV = 36836 	# GL/glext.h:5200
GL_INT16_VEC2_NV = 36837 	# GL/glext.h:5201
GL_INT16_VEC3_NV = 36838 	# GL/glext.h:5202
GL_INT16_VEC4_NV = 36839 	# GL/glext.h:5203
GL_INT64_VEC2_NV = 36841 	# GL/glext.h:5204
GL_INT64_VEC3_NV = 36842 	# GL/glext.h:5205
GL_INT64_VEC4_NV = 36843 	# GL/glext.h:5206
GL_UNSIGNED_INT8_NV = 36844 	# GL/glext.h:5207
GL_UNSIGNED_INT8_VEC2_NV = 36845 	# GL/glext.h:5208
GL_UNSIGNED_INT8_VEC3_NV = 36846 	# GL/glext.h:5209
GL_UNSIGNED_INT8_VEC4_NV = 36847 	# GL/glext.h:5210
GL_UNSIGNED_INT16_NV = 36848 	# GL/glext.h:5211
GL_UNSIGNED_INT16_VEC2_NV = 36849 	# GL/glext.h:5212
GL_UNSIGNED_INT16_VEC3_NV = 36850 	# GL/glext.h:5213
GL_UNSIGNED_INT16_VEC4_NV = 36851 	# GL/glext.h:5214
GL_UNSIGNED_INT64_VEC2_NV = 36853 	# GL/glext.h:5215
GL_UNSIGNED_INT64_VEC3_NV = 36854 	# GL/glext.h:5216
GL_UNSIGNED_INT64_VEC4_NV = 36855 	# GL/glext.h:5217
GL_FLOAT16_NV = 36856 	# GL/glext.h:5218
GL_FLOAT16_VEC2_NV = 36857 	# GL/glext.h:5219
GL_FLOAT16_VEC3_NV = 36858 	# GL/glext.h:5220
GL_FLOAT16_VEC4_NV = 36859 	# GL/glext.h:5221
# NV_shader_buffer_store (GL/glext.h:5225)
GL_SHADER_GLOBAL_ACCESS_BARRIER_BIT_NV = 16 	# GL/glext.h:5226
# NV_tessellation_program5 (GL/glext.h:5231)
GL_MAX_PROGRAM_PATCH_ATTRIBS_NV = 34520 	# GL/glext.h:5232
GL_TESS_CONTROL_PROGRAM_NV = 35102 	# GL/glext.h:5233
GL_TESS_EVALUATION_PROGRAM_NV = 35103 	# GL/glext.h:5234
GL_TESS_CONTROL_PROGRAM_PARAMETER_BUFFER_NV = 35956 	# GL/glext.h:5235
GL_TESS_EVALUATION_PROGRAM_PARAMETER_BUFFER_NV = 35957 	# GL/glext.h:5236
# NV_vertex_attrib_integer_64bit (GL/glext.h:5239)
# NV_multisample_coverage (GL/glext.h:5244)
GL_COVERAGE_SAMPLES_NV = 32937 	# GL/glext.h:5245
GL_COLOR_SAMPLES_NV = 36384 	# GL/glext.h:5246
# AMD_name_gen_delete (GL/glext.h:5249)
GL_DATA_BUFFER_AMD = 37201 	# GL/glext.h:5250
GL_PERFORMANCE_MONITOR_AMD = 37202 	# GL/glext.h:5251
GL_QUERY_OBJECT_AMD = 37203 	# GL/glext.h:5252
GL_VERTEX_ARRAY_OBJECT_AMD = 37204 	# GL/glext.h:5253
GL_SAMPLER_OBJECT_AMD = 37205 	# GL/glext.h:5254
# AMD_debug_output (GL/glext.h:5257)
GL_MAX_DEBUG_LOGGED_MESSAGES_AMD = 37188 	# GL/glext.h:5258
GL_DEBUG_LOGGED_MESSAGES_AMD = 37189 	# GL/glext.h:5259
GL_DEBUG_SEVERITY_HIGH_AMD = 37190 	# GL/glext.h:5260
GL_DEBUG_SEVERITY_MEDIUM_AMD = 37191 	# GL/glext.h:5261
GL_DEBUG_SEVERITY_LOW_AMD = 37192 	# GL/glext.h:5262
GL_DEBUG_CATEGORY_API_ERROR_AMD = 37193 	# GL/glext.h:5263
GL_DEBUG_CATEGORY_WINDOW_SYSTEM_AMD = 37194 	# GL/glext.h:5264
GL_DEBUG_CATEGORY_DEPRECATION_AMD = 37195 	# GL/glext.h:5265
GL_DEBUG_CATEGORY_UNDEFINED_BEHAVIOR_AMD = 37196 	# GL/glext.h:5266
GL_DEBUG_CATEGORY_PERFORMANCE_AMD = 37197 	# GL/glext.h:5267
GL_DEBUG_CATEGORY_SHADER_COMPILER_AMD = 37198 	# GL/glext.h:5268
GL_DEBUG_CATEGORY_APPLICATION_AMD = 37199 	# GL/glext.h:5269
GL_DEBUG_CATEGORY_OTHER_AMD = 37200 	# GL/glext.h:5270
# NV_vdpau_interop (GL/glext.h:5273)
GL_SURFACE_STATE_NV = 34539 	# GL/glext.h:5274
GL_SURFACE_REGISTERED_NV = 34557 	# GL/glext.h:5275
GL_SURFACE_MAPPED_NV = 34560 	# GL/glext.h:5276
GL_WRITE_DISCARD_NV = 35006 	# GL/glext.h:5277
# AMD_transform_feedback3_lines_triangles (GL/glext.h:5280)
# AMD_depth_clamp_separate (GL/glext.h:5283)
GL_DEPTH_CLAMP_NEAR_AMD = 36894 	# GL/glext.h:5284
GL_DEPTH_CLAMP_FAR_AMD = 36895 	# GL/glext.h:5285
# EXT_texture_sRGB_decode (GL/glext.h:5288)
GL_TEXTURE_SRGB_DECODE_EXT = 35400 	# GL/glext.h:5289
GL_DECODE_EXT = 35401 	# GL/glext.h:5290
GL_SKIP_DECODE_EXT = 35402 	# GL/glext.h:5291
# NV_texture_multisample (GL/glext.h:5294)
GL_TEXTURE_COVERAGE_SAMPLES_NV = 36933 	# GL/glext.h:5295
GL_TEXTURE_COLOR_SAMPLES_NV = 36934 	# GL/glext.h:5296
# AMD_blend_minmax_factor (GL/glext.h:5299)
GL_FACTOR_MIN_AMD = 36892 	# GL/glext.h:5300
GL_FACTOR_MAX_AMD = 36893 	# GL/glext.h:5301
# AMD_sample_positions (GL/glext.h:5304)
GL_SUBSAMPLE_DISTANCE_AMD = 34879 	# GL/glext.h:5305
# EXT_x11_sync_object (GL/glext.h:5308)
GL_SYNC_X11_FENCE_EXT = 37089 	# GL/glext.h:5309
# AMD_multi_draw_indirect (GL/glext.h:5312)
# EXT_framebuffer_multisample_blit_scaled (GL/glext.h:5315)
GL_SCALED_RESOLVE_FASTEST_EXT = 37050 	# GL/glext.h:5316
GL_SCALED_RESOLVE_NICEST_EXT = 37051 	# GL/glext.h:5317
# NV_path_rendering (GL/glext.h:5320)
GL_PATH_FORMAT_SVG_NV = 36976 	# GL/glext.h:5321
GL_PATH_FORMAT_PS_NV = 36977 	# GL/glext.h:5322
GL_STANDARD_FONT_NAME_NV = 36978 	# GL/glext.h:5323
GL_SYSTEM_FONT_NAME_NV = 36979 	# GL/glext.h:5324
GL_FILE_NAME_NV = 36980 	# GL/glext.h:5325
GL_PATH_STROKE_WIDTH_NV = 36981 	# GL/glext.h:5326
GL_PATH_END_CAPS_NV = 36982 	# GL/glext.h:5327
GL_PATH_INITIAL_END_CAP_NV = 36983 	# GL/glext.h:5328
GL_PATH_TERMINAL_END_CAP_NV = 36984 	# GL/glext.h:5329
GL_PATH_JOIN_STYLE_NV = 36985 	# GL/glext.h:5330
GL_PATH_MITER_LIMIT_NV = 36986 	# GL/glext.h:5331
GL_PATH_DASH_CAPS_NV = 36987 	# GL/glext.h:5332
GL_PATH_INITIAL_DASH_CAP_NV = 36988 	# GL/glext.h:5333
GL_PATH_TERMINAL_DASH_CAP_NV = 36989 	# GL/glext.h:5334
GL_PATH_DASH_OFFSET_NV = 36990 	# GL/glext.h:5335
GL_PATH_CLIENT_LENGTH_NV = 36991 	# GL/glext.h:5336
GL_PATH_FILL_MODE_NV = 36992 	# GL/glext.h:5337
GL_PATH_FILL_MASK_NV = 36993 	# GL/glext.h:5338
GL_PATH_FILL_COVER_MODE_NV = 36994 	# GL/glext.h:5339
GL_PATH_STROKE_COVER_MODE_NV = 36995 	# GL/glext.h:5340
GL_PATH_STROKE_MASK_NV = 36996 	# GL/glext.h:5341
GL_PATH_SAMPLE_QUALITY_NV = 36997 	# GL/glext.h:5342
GL_PATH_STROKE_BOUND_NV = 36998 	# GL/glext.h:5343
GL_PATH_STROKE_OVERSAMPLE_COUNT_NV = 36999 	# GL/glext.h:5344
GL_COUNT_UP_NV = 37000 	# GL/glext.h:5345
GL_COUNT_DOWN_NV = 37001 	# GL/glext.h:5346
GL_PATH_OBJECT_BOUNDING_BOX_NV = 37002 	# GL/glext.h:5347
GL_CONVEX_HULL_NV = 37003 	# GL/glext.h:5348
GL_MULTI_HULLS_NV = 37004 	# GL/glext.h:5349
GL_BOUNDING_BOX_NV = 37005 	# GL/glext.h:5350
GL_TRANSLATE_X_NV = 37006 	# GL/glext.h:5351
GL_TRANSLATE_Y_NV = 37007 	# GL/glext.h:5352
GL_TRANSLATE_2D_NV = 37008 	# GL/glext.h:5353
GL_TRANSLATE_3D_NV = 37009 	# GL/glext.h:5354
GL_AFFINE_2D_NV = 37010 	# GL/glext.h:5355
GL_PROJECTIVE_2D_NV = 37011 	# GL/glext.h:5356
GL_AFFINE_3D_NV = 37012 	# GL/glext.h:5357
GL_PROJECTIVE_3D_NV = 37013 	# GL/glext.h:5358
GL_TRANSPOSE_AFFINE_2D_NV = 37014 	# GL/glext.h:5359
GL_TRANSPOSE_PROJECTIVE_2D_NV = 37015 	# GL/glext.h:5360
GL_TRANSPOSE_AFFINE_3D_NV = 37016 	# GL/glext.h:5361
GL_TRANSPOSE_PROJECTIVE_3D_NV = 37017 	# GL/glext.h:5362
GL_UTF8_NV = 37018 	# GL/glext.h:5363
GL_UTF16_NV = 37019 	# GL/glext.h:5364
GL_BOUNDING_BOX_OF_BOUNDING_BOXES_NV = 37020 	# GL/glext.h:5365
GL_PATH_COMMAND_COUNT_NV = 37021 	# GL/glext.h:5366
GL_PATH_COORD_COUNT_NV = 37022 	# GL/glext.h:5367
GL_PATH_DASH_ARRAY_COUNT_NV = 37023 	# GL/glext.h:5368
GL_PATH_COMPUTED_LENGTH_NV = 37024 	# GL/glext.h:5369
GL_PATH_FILL_BOUNDING_BOX_NV = 37025 	# GL/glext.h:5370
GL_PATH_STROKE_BOUNDING_BOX_NV = 37026 	# GL/glext.h:5371
GL_SQUARE_NV = 37027 	# GL/glext.h:5372
GL_ROUND_NV = 37028 	# GL/glext.h:5373
GL_TRIANGULAR_NV = 37029 	# GL/glext.h:5374
GL_BEVEL_NV = 37030 	# GL/glext.h:5375
GL_MITER_REVERT_NV = 37031 	# GL/glext.h:5376
GL_MITER_TRUNCATE_NV = 37032 	# GL/glext.h:5377
GL_SKIP_MISSING_GLYPH_NV = 37033 	# GL/glext.h:5378
GL_USE_MISSING_GLYPH_NV = 37034 	# GL/glext.h:5379
GL_PATH_ERROR_POSITION_NV = 37035 	# GL/glext.h:5380
GL_PATH_FOG_GEN_MODE_NV = 37036 	# GL/glext.h:5381
GL_ACCUM_ADJACENT_PAIRS_NV = 37037 	# GL/glext.h:5382
GL_ADJACENT_PAIRS_NV = 37038 	# GL/glext.h:5383
GL_FIRST_TO_REST_NV = 37039 	# GL/glext.h:5384
GL_PATH_GEN_MODE_NV = 37040 	# GL/glext.h:5385
GL_PATH_GEN_COEFF_NV = 37041 	# GL/glext.h:5386
GL_PATH_GEN_COLOR_FORMAT_NV = 37042 	# GL/glext.h:5387
GL_PATH_GEN_COMPONENTS_NV = 37043 	# GL/glext.h:5388
GL_PATH_STENCIL_FUNC_NV = 37047 	# GL/glext.h:5389
GL_PATH_STENCIL_REF_NV = 37048 	# GL/glext.h:5390
GL_PATH_STENCIL_VALUE_MASK_NV = 37049 	# GL/glext.h:5391
GL_PATH_STENCIL_DEPTH_OFFSET_FACTOR_NV = 37053 	# GL/glext.h:5392
GL_PATH_STENCIL_DEPTH_OFFSET_UNITS_NV = 37054 	# GL/glext.h:5393
GL_PATH_COVER_DEPTH_FUNC_NV = 37055 	# GL/glext.h:5394
GL_PATH_DASH_OFFSET_RESET_NV = 37044 	# GL/glext.h:5395
GL_MOVE_TO_RESETS_NV = 37045 	# GL/glext.h:5396
GL_MOVE_TO_CONTINUES_NV = 37046 	# GL/glext.h:5397
GL_CLOSE_PATH_NV = 0 	# GL/glext.h:5398
GL_MOVE_TO_NV = 2 	# GL/glext.h:5399
GL_RELATIVE_MOVE_TO_NV = 3 	# GL/glext.h:5400
GL_LINE_TO_NV = 4 	# GL/glext.h:5401
GL_RELATIVE_LINE_TO_NV = 5 	# GL/glext.h:5402
GL_HORIZONTAL_LINE_TO_NV = 6 	# GL/glext.h:5403
GL_RELATIVE_HORIZONTAL_LINE_TO_NV = 7 	# GL/glext.h:5404
GL_VERTICAL_LINE_TO_NV = 8 	# GL/glext.h:5405
GL_RELATIVE_VERTICAL_LINE_TO_NV = 9 	# GL/glext.h:5406
GL_QUADRATIC_CURVE_TO_NV = 10 	# GL/glext.h:5407
GL_RELATIVE_QUADRATIC_CURVE_TO_NV = 11 	# GL/glext.h:5408
GL_CUBIC_CURVE_TO_NV = 12 	# GL/glext.h:5409
GL_RELATIVE_CUBIC_CURVE_TO_NV = 13 	# GL/glext.h:5410
GL_SMOOTH_QUADRATIC_CURVE_TO_NV = 14 	# GL/glext.h:5411
GL_RELATIVE_SMOOTH_QUADRATIC_CURVE_TO_NV = 15 	# GL/glext.h:5412
GL_SMOOTH_CUBIC_CURVE_TO_NV = 16 	# GL/glext.h:5413
GL_RELATIVE_SMOOTH_CUBIC_CURVE_TO_NV = 17 	# GL/glext.h:5414
GL_SMALL_CCW_ARC_TO_NV = 18 	# GL/glext.h:5415
GL_RELATIVE_SMALL_CCW_ARC_TO_NV = 19 	# GL/glext.h:5416
GL_SMALL_CW_ARC_TO_NV = 20 	# GL/glext.h:5417
GL_RELATIVE_SMALL_CW_ARC_TO_NV = 21 	# GL/glext.h:5418
GL_LARGE_CCW_ARC_TO_NV = 22 	# GL/glext.h:5419
GL_RELATIVE_LARGE_CCW_ARC_TO_NV = 23 	# GL/glext.h:5420
GL_LARGE_CW_ARC_TO_NV = 24 	# GL/glext.h:5421
GL_RELATIVE_LARGE_CW_ARC_TO_NV = 25 	# GL/glext.h:5422
GL_RESTART_PATH_NV = 240 	# GL/glext.h:5423
GL_DUP_FIRST_CUBIC_CURVE_TO_NV = 242 	# GL/glext.h:5424
GL_DUP_LAST_CUBIC_CURVE_TO_NV = 244 	# GL/glext.h:5425
GL_RECT_NV = 246 	# GL/glext.h:5426
GL_CIRCULAR_CCW_ARC_TO_NV = 248 	# GL/glext.h:5427
GL_CIRCULAR_CW_ARC_TO_NV = 250 	# GL/glext.h:5428
GL_CIRCULAR_TANGENT_ARC_TO_NV = 252 	# GL/glext.h:5429
GL_ARC_TO_NV = 254 	# GL/glext.h:5430
GL_RELATIVE_ARC_TO_NV = 255 	# GL/glext.h:5431
GL_BOLD_BIT_NV = 1 	# GL/glext.h:5432
GL_ITALIC_BIT_NV = 2 	# GL/glext.h:5433
GL_GLYPH_WIDTH_BIT_NV = 1 	# GL/glext.h:5434
GL_GLYPH_HEIGHT_BIT_NV = 2 	# GL/glext.h:5435
GL_GLYPH_HORIZONTAL_BEARING_X_BIT_NV = 4 	# GL/glext.h:5436
GL_GLYPH_HORIZONTAL_BEARING_Y_BIT_NV = 8 	# GL/glext.h:5437
GL_GLYPH_HORIZONTAL_BEARING_ADVANCE_BIT_NV = 16 	# GL/glext.h:5438
GL_GLYPH_VERTICAL_BEARING_X_BIT_NV = 32 	# GL/glext.h:5439
GL_GLYPH_VERTICAL_BEARING_Y_BIT_NV = 64 	# GL/glext.h:5440
GL_GLYPH_VERTICAL_BEARING_ADVANCE_BIT_NV = 128 	# GL/glext.h:5441
GL_GLYPH_HAS_KERNING_NV = 256 	# GL/glext.h:5442
GL_FONT_X_MIN_BOUNDS_NV = 65536 	# GL/glext.h:5443
GL_FONT_Y_MIN_BOUNDS_NV = 131072 	# GL/glext.h:5444
GL_FONT_X_MAX_BOUNDS_NV = 262144 	# GL/glext.h:5445
GL_FONT_Y_MAX_BOUNDS_NV = 524288 	# GL/glext.h:5446
GL_FONT_UNITS_PER_EM_NV = 1048576 	# GL/glext.h:5447
GL_FONT_ASCENDER_NV = 2097152 	# GL/glext.h:5448
GL_FONT_DESCENDER_NV = 4194304 	# GL/glext.h:5449
GL_FONT_HEIGHT_NV = 8388608 	# GL/glext.h:5450
GL_FONT_MAX_ADVANCE_WIDTH_NV = 16777216 	# GL/glext.h:5451
GL_FONT_MAX_ADVANCE_HEIGHT_NV = 33554432 	# GL/glext.h:5452
GL_FONT_UNDERLINE_POSITION_NV = 67108864 	# GL/glext.h:5453
GL_FONT_UNDERLINE_THICKNESS_NV = 134217728 	# GL/glext.h:5454
GL_FONT_HAS_KERNING_NV = 268435456 	# GL/glext.h:5455
# AMD_pinned_memory (GL/glext.h:5458)
GL_EXTERNAL_VIRTUAL_MEMORY_BUFFER_AMD = 37216 	# GL/glext.h:5459
# AMD_stencil_operation_extended (GL/glext.h:5462)
GL_SET_AMD = 34634 	# GL/glext.h:5463
GL_REPLACE_VALUE_AMD = 34635 	# GL/glext.h:5464
GL_STENCIL_OP_VALUE_AMD = 34636 	# GL/glext.h:5465
GL_STENCIL_BACK_OP_VALUE_AMD = 34637 	# GL/glext.h:5466
# AMD_vertex_shader_viewport_index (GL/glext.h:5469)
# AMD_vertex_shader_layer (GL/glext.h:5472)
# NV_bindless_texture (GL/glext.h:5475)
# NV_shader_atomic_float (GL/glext.h:5478)
# AMD_query_buffer_object (GL/glext.h:5481)
GL_QUERY_BUFFER_AMD = 37266 	# GL/glext.h:5482
GL_QUERY_BUFFER_BINDING_AMD = 37267 	# GL/glext.h:5483
GL_QUERY_RESULT_NO_WAIT_AMD = 37268 	# GL/glext.h:5484
# VERSION_2_0 (GL/glext.h:5491)
GLchar = c_char 	# GL/glext.h:5493
# VERSION_1_5 (GL/glext.h:5496)
GLintptr = c_ptrdiff_t 	# GL/glext.h:5498
GLsizeiptr = c_ptrdiff_t 	# GL/glext.h:5499
# ARB_vertex_buffer_object (GL/glext.h:5502)
GLintptrARB = c_ptrdiff_t 	# GL/glext.h:5504
GLsizeiptrARB = c_ptrdiff_t 	# GL/glext.h:5505
# ARB_shader_objects (GL/glext.h:5508)
GLcharARB = c_char 	# GL/glext.h:5510
GLhandleARB = c_uint 	# GL/glext.h:5511
# ARB_half_float_pixel (GL/glext.h:5515)
GLhalfARB = c_ushort 	# GL/glext.h:5516
# NV_half_float (GL/glext.h:5519)
GLhalfNV = c_ushort 	# GL/glext.h:5520
# EXT_timer_query (GL/glext.h:5561)
GLint64EXT = c_int64 	# GL/glext.h:5562
GLuint64EXT = c_uint64 	# GL/glext.h:5563
# ARB_sync (GL/glext.h:5566)
GLint64 = c_int64 	# GL/glext.h:5567
GLuint64 = c_uint64 	# GL/glext.h:5568
class struct___GLsync(Structure):
    __slots__ = [
    ]
struct___GLsync._fields_ = [
    ('_opaque_struct', c_int)
]

class struct___GLsync(Structure):
    __slots__ = [
    ]
struct___GLsync._fields_ = [
    ('_opaque_struct', c_int)
]

GLsync = POINTER(struct___GLsync) 	# GL/glext.h:5569
# ARB_cl_event (GL/glext.h:5572)
# ARB_debug_output (GL/glext.h:5578)
GLenum = c_uint 	# /usr/include/GL/gl.h:149
GLuint = c_uint 	# /usr/include/GL/gl.h:158
GLsizei = c_int 	# /usr/include/GL/gl.h:159
GLvoid = None 	# /usr/include/GL/gl.h:152
GLDEBUGPROCARB = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLenum, GLsizei, POINTER(GLchar), POINTER(GLvoid)) 	# GL/glext.h:5579
# AMD_debug_output (GL/glext.h:5582)
GLDEBUGPROCAMD = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLsizei, POINTER(GLchar), POINTER(GLvoid)) 	# GL/glext.h:5583
# NV_vdpau_interop (GL/glext.h:5586)
GLvdpauSurfaceNV = GLintptr 	# GL/glext.h:5587
# VERSION_1_2 (GL/glext.h:5590)
# VERSION_1_2_DEPRECATED (GL/glext.h:5608)
GL_VERSION_1_2_DEPRECATED = 1 	# GL/glext.h:5609
# GL/glext.h:5611
glColorTable = _link_function('glColorTable', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

GLfloat = c_float 	# /usr/include/GL/gl.h:160
# GL/glext.h:5612
glColorTableParameterfv = _link_function('glColorTableParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2_DEPRECATED')

GLint = c_int 	# /usr/include/GL/gl.h:155
# GL/glext.h:5613
glColorTableParameteriv = _link_function('glColorTableParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5614
glCopyColorTable = _link_function('glCopyColorTable', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5615
glGetColorTable = _link_function('glGetColorTable', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5616
glGetColorTableParameterfv = _link_function('glGetColorTableParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5617
glGetColorTableParameteriv = _link_function('glGetColorTableParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5618
glColorSubTable = _link_function('glColorSubTable', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5619
glCopyColorSubTable = _link_function('glCopyColorSubTable', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5620
glConvolutionFilter1D = _link_function('glConvolutionFilter1D', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5621
glConvolutionFilter2D = _link_function('glConvolutionFilter2D', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5622
glConvolutionParameterf = _link_function('glConvolutionParameterf', None, [GLenum, GLenum, GLfloat], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5623
glConvolutionParameterfv = _link_function('glConvolutionParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5624
glConvolutionParameteri = _link_function('glConvolutionParameteri', None, [GLenum, GLenum, GLint], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5625
glConvolutionParameteriv = _link_function('glConvolutionParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5626
glCopyConvolutionFilter1D = _link_function('glCopyConvolutionFilter1D', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5627
glCopyConvolutionFilter2D = _link_function('glCopyConvolutionFilter2D', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5628
glGetConvolutionFilter = _link_function('glGetConvolutionFilter', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5629
glGetConvolutionParameterfv = _link_function('glGetConvolutionParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5630
glGetConvolutionParameteriv = _link_function('glGetConvolutionParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5631
glGetSeparableFilter = _link_function('glGetSeparableFilter', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5632
glSeparableFilter2D = _link_function('glSeparableFilter2D', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

GLboolean = c_ubyte 	# /usr/include/GL/gl.h:150
# GL/glext.h:5633
glGetHistogram = _link_function('glGetHistogram', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5634
glGetHistogramParameterfv = _link_function('glGetHistogramParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5635
glGetHistogramParameteriv = _link_function('glGetHistogramParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5636
glGetMinmax = _link_function('glGetMinmax', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5637
glGetMinmaxParameterfv = _link_function('glGetMinmaxParameterfv', None, [GLenum, GLenum, POINTER(GLfloat)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5638
glGetMinmaxParameteriv = _link_function('glGetMinmaxParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5639
glHistogram = _link_function('glHistogram', None, [GLenum, GLsizei, GLenum, GLboolean], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5640
glMinmax = _link_function('glMinmax', None, [GLenum, GLenum, GLboolean], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5641
glResetHistogram = _link_function('glResetHistogram', None, [GLenum], 'VERSION_1_2_DEPRECATED')

# GL/glext.h:5642
glResetMinmax = _link_function('glResetMinmax', None, [GLenum], 'VERSION_1_2_DEPRECATED')

PFNGLCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5644
PFNGLCOLORTABLEPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5645
PFNGLCOLORTABLEPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5646
PFNGLCOPYCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:5647
PFNGLGETCOLORTABLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5648
PFNGLGETCOLORTABLEPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5649
PFNGLGETCOLORTABLEPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5650
PFNGLCOLORSUBTABLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5651
PFNGLCOPYCOLORSUBTABLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:5652
PFNGLCONVOLUTIONFILTER1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5653
PFNGLCONVOLUTIONFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5654
PFNGLCONVOLUTIONPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:5655
PFNGLCONVOLUTIONPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5656
PFNGLCONVOLUTIONPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:5657
PFNGLCONVOLUTIONPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5658
PFNGLCOPYCONVOLUTIONFILTER1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:5659
PFNGLCOPYCONVOLUTIONFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:5660
PFNGLGETCONVOLUTIONFILTERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5661
PFNGLGETCONVOLUTIONPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5662
PFNGLGETCONVOLUTIONPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5663
PFNGLGETSEPARABLEFILTERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:5664
PFNGLSEPARABLEFILTER2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:5665
PFNGLGETHISTOGRAMPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5666
PFNGLGETHISTOGRAMPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5667
PFNGLGETHISTOGRAMPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5668
PFNGLGETMINMAXPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:5669
PFNGLGETMINMAXPARAMETERFVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5670
PFNGLGETMINMAXPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5671
PFNGLHISTOGRAMPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:5672
PFNGLMINMAXPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:5673
PFNGLRESETHISTOGRAMPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5674
PFNGLRESETMINMAXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5675
# VERSION_1_3 (GL/glext.h:5678)
# VERSION_1_3_DEPRECATED (GL/glext.h:5702)
GL_VERSION_1_3_DEPRECATED = 1 	# GL/glext.h:5703
# GL/glext.h:5705
glClientActiveTexture = _link_function('glClientActiveTexture', None, [GLenum], 'VERSION_1_3_DEPRECATED')

GLdouble = c_double 	# /usr/include/GL/gl.h:162
# GL/glext.h:5706
glMultiTexCoord1d = _link_function('glMultiTexCoord1d', None, [GLenum, GLdouble], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5707
glMultiTexCoord1dv = _link_function('glMultiTexCoord1dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5708
glMultiTexCoord1f = _link_function('glMultiTexCoord1f', None, [GLenum, GLfloat], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5709
glMultiTexCoord1fv = _link_function('glMultiTexCoord1fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5710
glMultiTexCoord1i = _link_function('glMultiTexCoord1i', None, [GLenum, GLint], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5711
glMultiTexCoord1iv = _link_function('glMultiTexCoord1iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3_DEPRECATED')

GLshort = c_short 	# /usr/include/GL/gl.h:154
# GL/glext.h:5712
glMultiTexCoord1s = _link_function('glMultiTexCoord1s', None, [GLenum, GLshort], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5713
glMultiTexCoord1sv = _link_function('glMultiTexCoord1sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5714
glMultiTexCoord2d = _link_function('glMultiTexCoord2d', None, [GLenum, GLdouble, GLdouble], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5715
glMultiTexCoord2dv = _link_function('glMultiTexCoord2dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5716
glMultiTexCoord2f = _link_function('glMultiTexCoord2f', None, [GLenum, GLfloat, GLfloat], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5717
glMultiTexCoord2fv = _link_function('glMultiTexCoord2fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5718
glMultiTexCoord2i = _link_function('glMultiTexCoord2i', None, [GLenum, GLint, GLint], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5719
glMultiTexCoord2iv = _link_function('glMultiTexCoord2iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5720
glMultiTexCoord2s = _link_function('glMultiTexCoord2s', None, [GLenum, GLshort, GLshort], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5721
glMultiTexCoord2sv = _link_function('glMultiTexCoord2sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5722
glMultiTexCoord3d = _link_function('glMultiTexCoord3d', None, [GLenum, GLdouble, GLdouble, GLdouble], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5723
glMultiTexCoord3dv = _link_function('glMultiTexCoord3dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5724
glMultiTexCoord3f = _link_function('glMultiTexCoord3f', None, [GLenum, GLfloat, GLfloat, GLfloat], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5725
glMultiTexCoord3fv = _link_function('glMultiTexCoord3fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5726
glMultiTexCoord3i = _link_function('glMultiTexCoord3i', None, [GLenum, GLint, GLint, GLint], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5727
glMultiTexCoord3iv = _link_function('glMultiTexCoord3iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5728
glMultiTexCoord3s = _link_function('glMultiTexCoord3s', None, [GLenum, GLshort, GLshort, GLshort], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5729
glMultiTexCoord3sv = _link_function('glMultiTexCoord3sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5730
glMultiTexCoord4d = _link_function('glMultiTexCoord4d', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5731
glMultiTexCoord4dv = _link_function('glMultiTexCoord4dv', None, [GLenum, POINTER(GLdouble)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5732
glMultiTexCoord4f = _link_function('glMultiTexCoord4f', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5733
glMultiTexCoord4fv = _link_function('glMultiTexCoord4fv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5734
glMultiTexCoord4i = _link_function('glMultiTexCoord4i', None, [GLenum, GLint, GLint, GLint, GLint], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5735
glMultiTexCoord4iv = _link_function('glMultiTexCoord4iv', None, [GLenum, POINTER(GLint)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5736
glMultiTexCoord4s = _link_function('glMultiTexCoord4s', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5737
glMultiTexCoord4sv = _link_function('glMultiTexCoord4sv', None, [GLenum, POINTER(GLshort)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5738
glLoadTransposeMatrixf = _link_function('glLoadTransposeMatrixf', None, [POINTER(GLfloat)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5739
glLoadTransposeMatrixd = _link_function('glLoadTransposeMatrixd', None, [POINTER(GLdouble)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5740
glMultTransposeMatrixf = _link_function('glMultTransposeMatrixf', None, [POINTER(GLfloat)], 'VERSION_1_3_DEPRECATED')

# GL/glext.h:5741
glMultTransposeMatrixd = _link_function('glMultTransposeMatrixd', None, [POINTER(GLdouble)], 'VERSION_1_3_DEPRECATED')

PFNGLCLIENTACTIVETEXTUREPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5743
PFNGLMULTITEXCOORD1DPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:5744
PFNGLMULTITEXCOORD1DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5745
PFNGLMULTITEXCOORD1FPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5746
PFNGLMULTITEXCOORD1FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5747
PFNGLMULTITEXCOORD1IPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5748
PFNGLMULTITEXCOORD1IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5749
PFNGLMULTITEXCOORD1SPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:5750
PFNGLMULTITEXCOORD1SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5751
PFNGLMULTITEXCOORD2DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:5752
PFNGLMULTITEXCOORD2DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5753
PFNGLMULTITEXCOORD2FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:5754
PFNGLMULTITEXCOORD2FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5755
PFNGLMULTITEXCOORD2IPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:5756
PFNGLMULTITEXCOORD2IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5757
PFNGLMULTITEXCOORD2SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:5758
PFNGLMULTITEXCOORD2SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5759
PFNGLMULTITEXCOORD3DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5760
PFNGLMULTITEXCOORD3DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5761
PFNGLMULTITEXCOORD3FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5762
PFNGLMULTITEXCOORD3FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5763
PFNGLMULTITEXCOORD3IPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:5764
PFNGLMULTITEXCOORD3IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5765
PFNGLMULTITEXCOORD3SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:5766
PFNGLMULTITEXCOORD3SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5767
PFNGLMULTITEXCOORD4DPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5768
PFNGLMULTITEXCOORD4DVPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:5769
PFNGLMULTITEXCOORD4FPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5770
PFNGLMULTITEXCOORD4FVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5771
PFNGLMULTITEXCOORD4IPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:5772
PFNGLMULTITEXCOORD4IVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5773
PFNGLMULTITEXCOORD4SPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:5774
PFNGLMULTITEXCOORD4SVPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:5775
PFNGLLOADTRANSPOSEMATRIXFPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5776
PFNGLLOADTRANSPOSEMATRIXDPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5777
PFNGLMULTTRANSPOSEMATRIXFPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5778
PFNGLMULTTRANSPOSEMATRIXDPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5779
# VERSION_1_4 (GL/glext.h:5782)
GL_VERSION_1_4 = 1 	# GL/glext.h:5783
# GL/glext.h:5785
glBlendFuncSeparate = _link_function('glBlendFuncSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_1_4')

# GL/glext.h:5786
glMultiDrawArrays = _link_function('glMultiDrawArrays', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'VERSION_1_4')

# GL/glext.h:5787
glMultiDrawElements = _link_function('glMultiDrawElements', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'VERSION_1_4')

# GL/glext.h:5788
glPointParameterf = _link_function('glPointParameterf', None, [GLenum, GLfloat], 'VERSION_1_4')

# GL/glext.h:5789
glPointParameterfv = _link_function('glPointParameterfv', None, [GLenum, POINTER(GLfloat)], 'VERSION_1_4')

# GL/glext.h:5790
glPointParameteri = _link_function('glPointParameteri', None, [GLenum, GLint], 'VERSION_1_4')

# GL/glext.h:5791
glPointParameteriv = _link_function('glPointParameteriv', None, [GLenum, POINTER(GLint)], 'VERSION_1_4')

PFNGLBLENDFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:5793
PFNGLMULTIDRAWARRAYSPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:5794
PFNGLMULTIDRAWELEMENTSPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:5795
PFNGLPOINTPARAMETERFPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:5796
PFNGLPOINTPARAMETERFVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:5797
PFNGLPOINTPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:5798
PFNGLPOINTPARAMETERIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:5799
# VERSION_1_4_DEPRECATED (GL/glext.h:5802)
GL_VERSION_1_4_DEPRECATED = 1 	# GL/glext.h:5803
# GL/glext.h:5805
glFogCoordf = _link_function('glFogCoordf', None, [GLfloat], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5806
glFogCoordfv = _link_function('glFogCoordfv', None, [POINTER(GLfloat)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5807
glFogCoordd = _link_function('glFogCoordd', None, [GLdouble], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5808
glFogCoorddv = _link_function('glFogCoorddv', None, [POINTER(GLdouble)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5809
glFogCoordPointer = _link_function('glFogCoordPointer', None, [GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4_DEPRECATED')

GLbyte = c_char 	# /usr/include/GL/gl.h:153
# GL/glext.h:5810
glSecondaryColor3b = _link_function('glSecondaryColor3b', None, [GLbyte, GLbyte, GLbyte], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5811
glSecondaryColor3bv = _link_function('glSecondaryColor3bv', None, [POINTER(GLbyte)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5812
glSecondaryColor3d = _link_function('glSecondaryColor3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5813
glSecondaryColor3dv = _link_function('glSecondaryColor3dv', None, [POINTER(GLdouble)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5814
glSecondaryColor3f = _link_function('glSecondaryColor3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5815
glSecondaryColor3fv = _link_function('glSecondaryColor3fv', None, [POINTER(GLfloat)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5816
glSecondaryColor3i = _link_function('glSecondaryColor3i', None, [GLint, GLint, GLint], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5817
glSecondaryColor3iv = _link_function('glSecondaryColor3iv', None, [POINTER(GLint)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5818
glSecondaryColor3s = _link_function('glSecondaryColor3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5819
glSecondaryColor3sv = _link_function('glSecondaryColor3sv', None, [POINTER(GLshort)], 'VERSION_1_4_DEPRECATED')

GLubyte = c_ubyte 	# /usr/include/GL/gl.h:156
# GL/glext.h:5820
glSecondaryColor3ub = _link_function('glSecondaryColor3ub', None, [GLubyte, GLubyte, GLubyte], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5821
glSecondaryColor3ubv = _link_function('glSecondaryColor3ubv', None, [POINTER(GLubyte)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5822
glSecondaryColor3ui = _link_function('glSecondaryColor3ui', None, [GLuint, GLuint, GLuint], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5823
glSecondaryColor3uiv = _link_function('glSecondaryColor3uiv', None, [POINTER(GLuint)], 'VERSION_1_4_DEPRECATED')

GLushort = c_ushort 	# /usr/include/GL/gl.h:157
# GL/glext.h:5824
glSecondaryColor3us = _link_function('glSecondaryColor3us', None, [GLushort, GLushort, GLushort], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5825
glSecondaryColor3usv = _link_function('glSecondaryColor3usv', None, [POINTER(GLushort)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5826
glSecondaryColorPointer = _link_function('glSecondaryColorPointer', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5827
glWindowPos2d = _link_function('glWindowPos2d', None, [GLdouble, GLdouble], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5828
glWindowPos2dv = _link_function('glWindowPos2dv', None, [POINTER(GLdouble)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5829
glWindowPos2f = _link_function('glWindowPos2f', None, [GLfloat, GLfloat], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5830
glWindowPos2fv = _link_function('glWindowPos2fv', None, [POINTER(GLfloat)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5831
glWindowPos2i = _link_function('glWindowPos2i', None, [GLint, GLint], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5832
glWindowPos2iv = _link_function('glWindowPos2iv', None, [POINTER(GLint)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5833
glWindowPos2s = _link_function('glWindowPos2s', None, [GLshort, GLshort], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5834
glWindowPos2sv = _link_function('glWindowPos2sv', None, [POINTER(GLshort)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5835
glWindowPos3d = _link_function('glWindowPos3d', None, [GLdouble, GLdouble, GLdouble], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5836
glWindowPos3dv = _link_function('glWindowPos3dv', None, [POINTER(GLdouble)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5837
glWindowPos3f = _link_function('glWindowPos3f', None, [GLfloat, GLfloat, GLfloat], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5838
glWindowPos3fv = _link_function('glWindowPos3fv', None, [POINTER(GLfloat)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5839
glWindowPos3i = _link_function('glWindowPos3i', None, [GLint, GLint, GLint], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5840
glWindowPos3iv = _link_function('glWindowPos3iv', None, [POINTER(GLint)], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5841
glWindowPos3s = _link_function('glWindowPos3s', None, [GLshort, GLshort, GLshort], 'VERSION_1_4_DEPRECATED')

# GL/glext.h:5842
glWindowPos3sv = _link_function('glWindowPos3sv', None, [POINTER(GLshort)], 'VERSION_1_4_DEPRECATED')

PFNGLFOGCOORDFPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:5844
PFNGLFOGCOORDFVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5845
PFNGLFOGCOORDDPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:5846
PFNGLFOGCOORDDVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5847
PFNGLFOGCOORDPOINTERPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5848
PFNGLSECONDARYCOLOR3BPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:5849
PFNGLSECONDARYCOLOR3BVPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:5850
PFNGLSECONDARYCOLOR3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5851
PFNGLSECONDARYCOLOR3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5852
PFNGLSECONDARYCOLOR3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5853
PFNGLSECONDARYCOLOR3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5854
PFNGLSECONDARYCOLOR3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5855
PFNGLSECONDARYCOLOR3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5856
PFNGLSECONDARYCOLOR3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5857
PFNGLSECONDARYCOLOR3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5858
PFNGLSECONDARYCOLOR3UBPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:5859
PFNGLSECONDARYCOLOR3UBVPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:5860
PFNGLSECONDARYCOLOR3UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:5861
PFNGLSECONDARYCOLOR3UIVPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:5862
PFNGLSECONDARYCOLOR3USPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:5863
PFNGLSECONDARYCOLOR3USVPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:5864
PFNGLSECONDARYCOLORPOINTERPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:5865
PFNGLWINDOWPOS2DPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:5866
PFNGLWINDOWPOS2DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5867
PFNGLWINDOWPOS2FPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:5868
PFNGLWINDOWPOS2FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5869
PFNGLWINDOWPOS2IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:5870
PFNGLWINDOWPOS2IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5871
PFNGLWINDOWPOS2SPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:5872
PFNGLWINDOWPOS2SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5873
PFNGLWINDOWPOS3DPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:5874
PFNGLWINDOWPOS3DVPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:5875
PFNGLWINDOWPOS3FPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:5876
PFNGLWINDOWPOS3FVPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:5877
PFNGLWINDOWPOS3IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:5878
PFNGLWINDOWPOS3IVPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:5879
PFNGLWINDOWPOS3SPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:5880
PFNGLWINDOWPOS3SVPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:5881
# VERSION_1_5 (GL/glext.h:5884)
GL_VERSION_1_5 = 1 	# GL/glext.h:5885
# GL/glext.h:5887
glGenQueries = _link_function('glGenQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:5888
glDeleteQueries = _link_function('glDeleteQueries', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:5889
glIsQuery = _link_function('glIsQuery', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:5890
glBeginQuery = _link_function('glBeginQuery', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:5891
glEndQuery = _link_function('glEndQuery', None, [GLenum], 'VERSION_1_5')

# GL/glext.h:5892
glGetQueryiv = _link_function('glGetQueryiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:5893
glGetQueryObjectiv = _link_function('glGetQueryObjectiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:5894
glGetQueryObjectuiv = _link_function('glGetQueryObjectuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:5895
glBindBuffer = _link_function('glBindBuffer', None, [GLenum, GLuint], 'VERSION_1_5')

# GL/glext.h:5896
glDeleteBuffers = _link_function('glDeleteBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:5897
glGenBuffers = _link_function('glGenBuffers', None, [GLsizei, POINTER(GLuint)], 'VERSION_1_5')

# GL/glext.h:5898
glIsBuffer = _link_function('glIsBuffer', GLboolean, [GLuint], 'VERSION_1_5')

# GL/glext.h:5899
glBufferData = _link_function('glBufferData', None, [GLenum, GLsizeiptr, POINTER(GLvoid), GLenum], 'VERSION_1_5')

# GL/glext.h:5900
glBufferSubData = _link_function('glBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:5901
glGetBufferSubData = _link_function('glGetBufferSubData', None, [GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'VERSION_1_5')

# GL/glext.h:5902
glMapBuffer = _link_function('glMapBuffer', POINTER(GLvoid), [GLenum, GLenum], 'VERSION_1_5')

# GL/glext.h:5903
glUnmapBuffer = _link_function('glUnmapBuffer', GLboolean, [GLenum], 'VERSION_1_5')

# GL/glext.h:5904
glGetBufferParameteriv = _link_function('glGetBufferParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_1_5')

# GL/glext.h:5905
glGetBufferPointerv = _link_function('glGetBufferPointerv', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_1_5')

PFNGLGENQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5907
PFNGLDELETEQUERIESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5908
PFNGLISQUERYPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5909
PFNGLBEGINQUERYPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5910
PFNGLENDQUERYPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:5911
PFNGLGETQUERYIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5912
PFNGLGETQUERYOBJECTIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:5913
PFNGLGETQUERYOBJECTUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:5914
PFNGLBINDBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:5915
PFNGLDELETEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5916
PFNGLGENBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:5917
PFNGLISBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:5918
PFNGLBUFFERDATAPROC = CFUNCTYPE(None, GLenum, GLsizeiptr, POINTER(GLvoid), GLenum) 	# GL/glext.h:5919
PFNGLBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:5920
PFNGLGETBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:5921
PFNGLMAPBUFFERPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:5922
PFNGLUNMAPBUFFERPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:5923
PFNGLGETBUFFERPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:5924
PFNGLGETBUFFERPOINTERVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:5925
# VERSION_2_0 (GL/glext.h:5928)
GL_VERSION_2_0 = 1 	# GL/glext.h:5929
# GL/glext.h:5931
glBlendEquationSeparate = _link_function('glBlendEquationSeparate', None, [GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:5932
glDrawBuffers = _link_function('glDrawBuffers', None, [GLsizei, POINTER(GLenum)], 'VERSION_2_0')

# GL/glext.h:5933
glStencilOpSeparate = _link_function('glStencilOpSeparate', None, [GLenum, GLenum, GLenum, GLenum], 'VERSION_2_0')

# GL/glext.h:5934
glStencilFuncSeparate = _link_function('glStencilFuncSeparate', None, [GLenum, GLenum, GLint, GLuint], 'VERSION_2_0')

# GL/glext.h:5935
glStencilMaskSeparate = _link_function('glStencilMaskSeparate', None, [GLenum, GLuint], 'VERSION_2_0')

# GL/glext.h:5936
glAttachShader = _link_function('glAttachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:5937
glBindAttribLocation = _link_function('glBindAttribLocation', None, [GLuint, GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5938
glCompileShader = _link_function('glCompileShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5939
glCreateProgram = _link_function('glCreateProgram', GLuint, [], 'VERSION_2_0')

# GL/glext.h:5940
glCreateShader = _link_function('glCreateShader', GLuint, [GLenum], 'VERSION_2_0')

# GL/glext.h:5941
glDeleteProgram = _link_function('glDeleteProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5942
glDeleteShader = _link_function('glDeleteShader', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5943
glDetachShader = _link_function('glDetachShader', None, [GLuint, GLuint], 'VERSION_2_0')

# GL/glext.h:5944
glDisableVertexAttribArray = _link_function('glDisableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5945
glEnableVertexAttribArray = _link_function('glEnableVertexAttribArray', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5946
glGetActiveAttrib = _link_function('glGetActiveAttrib', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5947
glGetActiveUniform = _link_function('glGetActiveUniform', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5948
glGetAttachedShaders = _link_function('glGetAttachedShaders', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:5949
glGetAttribLocation = _link_function('glGetAttribLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5950
glGetProgramiv = _link_function('glGetProgramiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5951
glGetProgramInfoLog = _link_function('glGetProgramInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5952
glGetShaderiv = _link_function('glGetShaderiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5953
glGetShaderInfoLog = _link_function('glGetShaderInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5954
glGetShaderSource = _link_function('glGetShaderSource', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5955
glGetUniformLocation = _link_function('glGetUniformLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_2_0')

# GL/glext.h:5956
glGetUniformfv = _link_function('glGetUniformfv', None, [GLuint, GLint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5957
glGetUniformiv = _link_function('glGetUniformiv', None, [GLuint, GLint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5958
glGetVertexAttribdv = _link_function('glGetVertexAttribdv', None, [GLuint, GLenum, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:5959
glGetVertexAttribfv = _link_function('glGetVertexAttribfv', None, [GLuint, GLenum, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5960
glGetVertexAttribiv = _link_function('glGetVertexAttribiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5961
glGetVertexAttribPointerv = _link_function('glGetVertexAttribPointerv', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'VERSION_2_0')

# GL/glext.h:5962
glIsProgram = _link_function('glIsProgram', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:5963
glIsShader = _link_function('glIsShader', GLboolean, [GLuint], 'VERSION_2_0')

# GL/glext.h:5964
glLinkProgram = _link_function('glLinkProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5965
glShaderSource = _link_function('glShaderSource', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5966
glUseProgram = _link_function('glUseProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5967
glUniform1f = _link_function('glUniform1f', None, [GLint, GLfloat], 'VERSION_2_0')

# GL/glext.h:5968
glUniform2f = _link_function('glUniform2f', None, [GLint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:5969
glUniform3f = _link_function('glUniform3f', None, [GLint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:5970
glUniform4f = _link_function('glUniform4f', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:5971
glUniform1i = _link_function('glUniform1i', None, [GLint, GLint], 'VERSION_2_0')

# GL/glext.h:5972
glUniform2i = _link_function('glUniform2i', None, [GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:5973
glUniform3i = _link_function('glUniform3i', None, [GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:5974
glUniform4i = _link_function('glUniform4i', None, [GLint, GLint, GLint, GLint, GLint], 'VERSION_2_0')

# GL/glext.h:5975
glUniform1fv = _link_function('glUniform1fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5976
glUniform2fv = _link_function('glUniform2fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5977
glUniform3fv = _link_function('glUniform3fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5978
glUniform4fv = _link_function('glUniform4fv', None, [GLint, GLsizei, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5979
glUniform1iv = _link_function('glUniform1iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5980
glUniform2iv = _link_function('glUniform2iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5981
glUniform3iv = _link_function('glUniform3iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5982
glUniform4iv = _link_function('glUniform4iv', None, [GLint, GLsizei, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:5983
glUniformMatrix2fv = _link_function('glUniformMatrix2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5984
glUniformMatrix3fv = _link_function('glUniformMatrix3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5985
glUniformMatrix4fv = _link_function('glUniformMatrix4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5986
glValidateProgram = _link_function('glValidateProgram', None, [GLuint], 'VERSION_2_0')

# GL/glext.h:5987
glVertexAttrib1d = _link_function('glVertexAttrib1d', None, [GLuint, GLdouble], 'VERSION_2_0')

# GL/glext.h:5988
glVertexAttrib1dv = _link_function('glVertexAttrib1dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:5989
glVertexAttrib1f = _link_function('glVertexAttrib1f', None, [GLuint, GLfloat], 'VERSION_2_0')

# GL/glext.h:5990
glVertexAttrib1fv = _link_function('glVertexAttrib1fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5991
glVertexAttrib1s = _link_function('glVertexAttrib1s', None, [GLuint, GLshort], 'VERSION_2_0')

# GL/glext.h:5992
glVertexAttrib1sv = _link_function('glVertexAttrib1sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:5993
glVertexAttrib2d = _link_function('glVertexAttrib2d', None, [GLuint, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:5994
glVertexAttrib2dv = _link_function('glVertexAttrib2dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:5995
glVertexAttrib2f = _link_function('glVertexAttrib2f', None, [GLuint, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:5996
glVertexAttrib2fv = _link_function('glVertexAttrib2fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:5997
glVertexAttrib2s = _link_function('glVertexAttrib2s', None, [GLuint, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:5998
glVertexAttrib2sv = _link_function('glVertexAttrib2sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:5999
glVertexAttrib3d = _link_function('glVertexAttrib3d', None, [GLuint, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:6000
glVertexAttrib3dv = _link_function('glVertexAttrib3dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:6001
glVertexAttrib3f = _link_function('glVertexAttrib3f', None, [GLuint, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:6002
glVertexAttrib3fv = _link_function('glVertexAttrib3fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:6003
glVertexAttrib3s = _link_function('glVertexAttrib3s', None, [GLuint, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:6004
glVertexAttrib3sv = _link_function('glVertexAttrib3sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:6005
glVertexAttrib4Nbv = _link_function('glVertexAttrib4Nbv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:6006
glVertexAttrib4Niv = _link_function('glVertexAttrib4Niv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:6007
glVertexAttrib4Nsv = _link_function('glVertexAttrib4Nsv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:6008
glVertexAttrib4Nub = _link_function('glVertexAttrib4Nub', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'VERSION_2_0')

# GL/glext.h:6009
glVertexAttrib4Nubv = _link_function('glVertexAttrib4Nubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:6010
glVertexAttrib4Nuiv = _link_function('glVertexAttrib4Nuiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:6011
glVertexAttrib4Nusv = _link_function('glVertexAttrib4Nusv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:6012
glVertexAttrib4bv = _link_function('glVertexAttrib4bv', None, [GLuint, POINTER(GLbyte)], 'VERSION_2_0')

# GL/glext.h:6013
glVertexAttrib4d = _link_function('glVertexAttrib4d', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'VERSION_2_0')

# GL/glext.h:6014
glVertexAttrib4dv = _link_function('glVertexAttrib4dv', None, [GLuint, POINTER(GLdouble)], 'VERSION_2_0')

# GL/glext.h:6015
glVertexAttrib4f = _link_function('glVertexAttrib4f', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'VERSION_2_0')

# GL/glext.h:6016
glVertexAttrib4fv = _link_function('glVertexAttrib4fv', None, [GLuint, POINTER(GLfloat)], 'VERSION_2_0')

# GL/glext.h:6017
glVertexAttrib4iv = _link_function('glVertexAttrib4iv', None, [GLuint, POINTER(GLint)], 'VERSION_2_0')

# GL/glext.h:6018
glVertexAttrib4s = _link_function('glVertexAttrib4s', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'VERSION_2_0')

# GL/glext.h:6019
glVertexAttrib4sv = _link_function('glVertexAttrib4sv', None, [GLuint, POINTER(GLshort)], 'VERSION_2_0')

# GL/glext.h:6020
glVertexAttrib4ubv = _link_function('glVertexAttrib4ubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_2_0')

# GL/glext.h:6021
glVertexAttrib4uiv = _link_function('glVertexAttrib4uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_2_0')

# GL/glext.h:6022
glVertexAttrib4usv = _link_function('glVertexAttrib4usv', None, [GLuint, POINTER(GLushort)], 'VERSION_2_0')

# GL/glext.h:6023
glVertexAttribPointer = _link_function('glVertexAttribPointer', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'VERSION_2_0')

PFNGLBLENDEQUATIONSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:6025
PFNGLDRAWBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:6026
PFNGLSTENCILOPSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6027
PFNGLSTENCILFUNCSEPARATEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:6028
PFNGLSTENCILMASKSEPARATEPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6029
PFNGLATTACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:6030
PFNGLBINDATTRIBLOCATIONPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6031
PFNGLCOMPILESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6032
PFNGLCREATEPROGRAMPROC = CFUNCTYPE(GLuint) 	# GL/glext.h:6033
PFNGLCREATESHADERPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:6034
PFNGLDELETEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6035
PFNGLDELETESHADERPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6036
PFNGLDETACHSHADERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:6037
PFNGLDISABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6038
PFNGLENABLEVERTEXATTRIBARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6039
PFNGLGETACTIVEATTRIBPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:6040
PFNGLGETACTIVEUNIFORMPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:6041
PFNGLGETATTACHEDSHADERSPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLuint)) 	# GL/glext.h:6042
PFNGLGETATTRIBLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6043
PFNGLGETPROGRAMIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6044
PFNGLGETPROGRAMINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:6045
PFNGLGETSHADERIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6046
PFNGLGETSHADERINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:6047
PFNGLGETSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:6048
PFNGLGETUNIFORMLOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6049
PFNGLGETUNIFORMFVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLfloat)) 	# GL/glext.h:6050
PFNGLGETUNIFORMIVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLint)) 	# GL/glext.h:6051
PFNGLGETVERTEXATTRIBDVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6052
PFNGLGETVERTEXATTRIBFVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6053
PFNGLGETVERTEXATTRIBIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6054
PFNGLGETVERTEXATTRIBPOINTERVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:6055
PFNGLISPROGRAMPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6056
PFNGLISSHADERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6057
PFNGLLINKPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6058
PFNGLSHADERSOURCEPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)) 	# GL/glext.h:6059
PFNGLUSEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6060
PFNGLUNIFORM1FPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:6061
PFNGLUNIFORM2FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:6062
PFNGLUNIFORM3FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6063
PFNGLUNIFORM4FPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6064
PFNGLUNIFORM1IPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:6065
PFNGLUNIFORM2IPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6066
PFNGLUNIFORM3IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:6067
PFNGLUNIFORM4IPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:6068
PFNGLUNIFORM1FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6069
PFNGLUNIFORM2FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6070
PFNGLUNIFORM3FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6071
PFNGLUNIFORM4FVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6072
PFNGLUNIFORM1IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6073
PFNGLUNIFORM2IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6074
PFNGLUNIFORM3IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6075
PFNGLUNIFORM4IVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6076
PFNGLUNIFORMMATRIX2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6077
PFNGLUNIFORMMATRIX3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6078
PFNGLUNIFORMMATRIX4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6079
PFNGLVALIDATEPROGRAMPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6080
PFNGLVERTEXATTRIB1DPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:6081
PFNGLVERTEXATTRIB1DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6082
PFNGLVERTEXATTRIB1FPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:6083
PFNGLVERTEXATTRIB1FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6084
PFNGLVERTEXATTRIB1SPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:6085
PFNGLVERTEXATTRIB1SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6086
PFNGLVERTEXATTRIB2DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:6087
PFNGLVERTEXATTRIB2DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6088
PFNGLVERTEXATTRIB2FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:6089
PFNGLVERTEXATTRIB2FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6090
PFNGLVERTEXATTRIB2SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:6091
PFNGLVERTEXATTRIB2SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6092
PFNGLVERTEXATTRIB3DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6093
PFNGLVERTEXATTRIB3DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6094
PFNGLVERTEXATTRIB3FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6095
PFNGLVERTEXATTRIB3FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6096
PFNGLVERTEXATTRIB3SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:6097
PFNGLVERTEXATTRIB3SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6098
PFNGLVERTEXATTRIB4NBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:6099
PFNGLVERTEXATTRIB4NIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6100
PFNGLVERTEXATTRIB4NSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6101
PFNGLVERTEXATTRIB4NUBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:6102
PFNGLVERTEXATTRIB4NUBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6103
PFNGLVERTEXATTRIB4NUIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6104
PFNGLVERTEXATTRIB4NUSVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:6105
PFNGLVERTEXATTRIB4BVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:6106
PFNGLVERTEXATTRIB4DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6107
PFNGLVERTEXATTRIB4DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6108
PFNGLVERTEXATTRIB4FPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6109
PFNGLVERTEXATTRIB4FVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6110
PFNGLVERTEXATTRIB4IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6111
PFNGLVERTEXATTRIB4SPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6112
PFNGLVERTEXATTRIB4SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6113
PFNGLVERTEXATTRIB4UBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6114
PFNGLVERTEXATTRIB4UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6115
PFNGLVERTEXATTRIB4USVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:6116
PFNGLVERTEXATTRIBPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6117
# VERSION_2_1 (GL/glext.h:6120)
GL_VERSION_2_1 = 1 	# GL/glext.h:6121
# GL/glext.h:6123
glUniformMatrix2x3fv = _link_function('glUniformMatrix2x3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:6124
glUniformMatrix3x2fv = _link_function('glUniformMatrix3x2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:6125
glUniformMatrix2x4fv = _link_function('glUniformMatrix2x4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:6126
glUniformMatrix4x2fv = _link_function('glUniformMatrix4x2fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:6127
glUniformMatrix3x4fv = _link_function('glUniformMatrix3x4fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

# GL/glext.h:6128
glUniformMatrix4x3fv = _link_function('glUniformMatrix4x3fv', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'VERSION_2_1')

PFNGLUNIFORMMATRIX2X3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6130
PFNGLUNIFORMMATRIX3X2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6131
PFNGLUNIFORMMATRIX2X4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6132
PFNGLUNIFORMMATRIX4X2FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6133
PFNGLUNIFORMMATRIX3X4FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6134
PFNGLUNIFORMMATRIX4X3FVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6135
# VERSION_3_0 (GL/glext.h:6138)
GL_VERSION_3_0 = 1 	# GL/glext.h:6139
# GL/glext.h:6145
glColorMaski = _link_function('glColorMaski', None, [GLuint, GLboolean, GLboolean, GLboolean, GLboolean], 'VERSION_3_0')

# GL/glext.h:6146
glGetBooleani_v = _link_function('glGetBooleani_v', None, [GLenum, GLuint, POINTER(GLboolean)], 'VERSION_3_0')

# GL/glext.h:6147
glGetIntegeri_v = _link_function('glGetIntegeri_v', None, [GLenum, GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6148
glEnablei = _link_function('glEnablei', None, [GLenum, GLuint], 'VERSION_3_0')

# GL/glext.h:6149
glDisablei = _link_function('glDisablei', None, [GLenum, GLuint], 'VERSION_3_0')

# GL/glext.h:6150
glIsEnabledi = _link_function('glIsEnabledi', GLboolean, [GLenum, GLuint], 'VERSION_3_0')

# GL/glext.h:6151
glBeginTransformFeedback = _link_function('glBeginTransformFeedback', None, [GLenum], 'VERSION_3_0')

# GL/glext.h:6152
glEndTransformFeedback = _link_function('glEndTransformFeedback', None, [], 'VERSION_3_0')

# GL/glext.h:6153
glBindBufferRange = _link_function('glBindBufferRange', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'VERSION_3_0')

# GL/glext.h:6154
glBindBufferBase = _link_function('glBindBufferBase', None, [GLenum, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6155
glTransformFeedbackVaryings = _link_function('glTransformFeedbackVaryings', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), GLenum], 'VERSION_3_0')

# GL/glext.h:6156
glGetTransformFeedbackVarying = _link_function('glGetTransformFeedbackVarying', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)], 'VERSION_3_0')

# GL/glext.h:6157
glClampColor = _link_function('glClampColor', None, [GLenum, GLenum], 'VERSION_3_0')

# GL/glext.h:6158
glBeginConditionalRender = _link_function('glBeginConditionalRender', None, [GLuint, GLenum], 'VERSION_3_0')

# GL/glext.h:6159
glEndConditionalRender = _link_function('glEndConditionalRender', None, [], 'VERSION_3_0')

# GL/glext.h:6160
glVertexAttribIPointer = _link_function('glVertexAttribIPointer', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'VERSION_3_0')

# GL/glext.h:6161
glGetVertexAttribIiv = _link_function('glGetVertexAttribIiv', None, [GLuint, GLenum, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6162
glGetVertexAttribIuiv = _link_function('glGetVertexAttribIuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6163
glVertexAttribI1i = _link_function('glVertexAttribI1i', None, [GLuint, GLint], 'VERSION_3_0')

# GL/glext.h:6164
glVertexAttribI2i = _link_function('glVertexAttribI2i', None, [GLuint, GLint, GLint], 'VERSION_3_0')

# GL/glext.h:6165
glVertexAttribI3i = _link_function('glVertexAttribI3i', None, [GLuint, GLint, GLint, GLint], 'VERSION_3_0')

# GL/glext.h:6166
glVertexAttribI4i = _link_function('glVertexAttribI4i', None, [GLuint, GLint, GLint, GLint, GLint], 'VERSION_3_0')

# GL/glext.h:6167
glVertexAttribI1ui = _link_function('glVertexAttribI1ui', None, [GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6168
glVertexAttribI2ui = _link_function('glVertexAttribI2ui', None, [GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6169
glVertexAttribI3ui = _link_function('glVertexAttribI3ui', None, [GLuint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6170
glVertexAttribI4ui = _link_function('glVertexAttribI4ui', None, [GLuint, GLuint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6171
glVertexAttribI1iv = _link_function('glVertexAttribI1iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6172
glVertexAttribI2iv = _link_function('glVertexAttribI2iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6173
glVertexAttribI3iv = _link_function('glVertexAttribI3iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6174
glVertexAttribI4iv = _link_function('glVertexAttribI4iv', None, [GLuint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6175
glVertexAttribI1uiv = _link_function('glVertexAttribI1uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6176
glVertexAttribI2uiv = _link_function('glVertexAttribI2uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6177
glVertexAttribI3uiv = _link_function('glVertexAttribI3uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6178
glVertexAttribI4uiv = _link_function('glVertexAttribI4uiv', None, [GLuint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6179
glVertexAttribI4bv = _link_function('glVertexAttribI4bv', None, [GLuint, POINTER(GLbyte)], 'VERSION_3_0')

# GL/glext.h:6180
glVertexAttribI4sv = _link_function('glVertexAttribI4sv', None, [GLuint, POINTER(GLshort)], 'VERSION_3_0')

# GL/glext.h:6181
glVertexAttribI4ubv = _link_function('glVertexAttribI4ubv', None, [GLuint, POINTER(GLubyte)], 'VERSION_3_0')

# GL/glext.h:6182
glVertexAttribI4usv = _link_function('glVertexAttribI4usv', None, [GLuint, POINTER(GLushort)], 'VERSION_3_0')

# GL/glext.h:6183
glGetUniformuiv = _link_function('glGetUniformuiv', None, [GLuint, GLint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6184
glBindFragDataLocation = _link_function('glBindFragDataLocation', None, [GLuint, GLuint, POINTER(GLchar)], 'VERSION_3_0')

# GL/glext.h:6185
glGetFragDataLocation = _link_function('glGetFragDataLocation', GLint, [GLuint, POINTER(GLchar)], 'VERSION_3_0')

# GL/glext.h:6186
glUniform1ui = _link_function('glUniform1ui', None, [GLint, GLuint], 'VERSION_3_0')

# GL/glext.h:6187
glUniform2ui = _link_function('glUniform2ui', None, [GLint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6188
glUniform3ui = _link_function('glUniform3ui', None, [GLint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6189
glUniform4ui = _link_function('glUniform4ui', None, [GLint, GLuint, GLuint, GLuint, GLuint], 'VERSION_3_0')

# GL/glext.h:6190
glUniform1uiv = _link_function('glUniform1uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6191
glUniform2uiv = _link_function('glUniform2uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6192
glUniform3uiv = _link_function('glUniform3uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6193
glUniform4uiv = _link_function('glUniform4uiv', None, [GLint, GLsizei, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6194
glTexParameterIiv = _link_function('glTexParameterIiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6195
glTexParameterIuiv = _link_function('glTexParameterIuiv', None, [GLenum, GLenum, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6196
glGetTexParameterIiv = _link_function('glGetTexParameterIiv', None, [GLenum, GLenum, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6197
glGetTexParameterIuiv = _link_function('glGetTexParameterIuiv', None, [GLenum, GLenum, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6198
glClearBufferiv = _link_function('glClearBufferiv', None, [GLenum, GLint, POINTER(GLint)], 'VERSION_3_0')

# GL/glext.h:6199
glClearBufferuiv = _link_function('glClearBufferuiv', None, [GLenum, GLint, POINTER(GLuint)], 'VERSION_3_0')

# GL/glext.h:6200
glClearBufferfv = _link_function('glClearBufferfv', None, [GLenum, GLint, POINTER(GLfloat)], 'VERSION_3_0')

# GL/glext.h:6201
glClearBufferfi = _link_function('glClearBufferfi', None, [GLenum, GLint, GLfloat, GLint], 'VERSION_3_0')

# GL/glext.h:6202
glGetStringi = _link_function('glGetStringi', POINTER(GLubyte), [GLenum, GLuint], 'VERSION_3_0')

PFNGLCOLORMASKIPROC = CFUNCTYPE(None, GLuint, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:6204
PFNGLGETBOOLEANI_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLboolean)) 	# GL/glext.h:6205
PFNGLGETINTEGERI_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:6206
PFNGLENABLEIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6207
PFNGLDISABLEIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6208
PFNGLISENABLEDIPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:6209
PFNGLBEGINTRANSFORMFEEDBACKPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6210
PFNGLENDTRANSFORMFEEDBACKPROC = CFUNCTYPE(None) 	# GL/glext.h:6211
PFNGLBINDBUFFERRANGEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:6212
PFNGLBINDBUFFERBASEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:6213
PFNGLTRANSFORMFEEDBACKVARYINGSPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), GLenum) 	# GL/glext.h:6214
PFNGLGETTRANSFORMFEEDBACKVARYINGPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:6215
PFNGLCLAMPCOLORPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:6216
PFNGLBEGINCONDITIONALRENDERPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:6217
PFNGLENDCONDITIONALRENDERPROC = CFUNCTYPE(None) 	# GL/glext.h:6218
PFNGLVERTEXATTRIBIPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6219
PFNGLGETVERTEXATTRIBIIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6220
PFNGLGETVERTEXATTRIBIUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:6221
PFNGLVERTEXATTRIBI1IPROC = CFUNCTYPE(None, GLuint, GLint) 	# GL/glext.h:6222
PFNGLVERTEXATTRIBI2IPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:6223
PFNGLVERTEXATTRIBI3IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:6224
PFNGLVERTEXATTRIBI4IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:6225
PFNGLVERTEXATTRIBI1UIPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:6226
PFNGLVERTEXATTRIBI2UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:6227
PFNGLVERTEXATTRIBI3UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6228
PFNGLVERTEXATTRIBI4UIPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6229
PFNGLVERTEXATTRIBI1IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6230
PFNGLVERTEXATTRIBI2IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6231
PFNGLVERTEXATTRIBI3IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6232
PFNGLVERTEXATTRIBI4IVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6233
PFNGLVERTEXATTRIBI1UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6234
PFNGLVERTEXATTRIBI2UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6235
PFNGLVERTEXATTRIBI3UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6236
PFNGLVERTEXATTRIBI4UIVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6237
PFNGLVERTEXATTRIBI4BVPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:6238
PFNGLVERTEXATTRIBI4SVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6239
PFNGLVERTEXATTRIBI4UBVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6240
PFNGLVERTEXATTRIBI4USVPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:6241
PFNGLGETUNIFORMUIVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint)) 	# GL/glext.h:6242
PFNGLBINDFRAGDATALOCATIONPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6243
PFNGLGETFRAGDATALOCATIONPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:6244
PFNGLUNIFORM1UIPROC = CFUNCTYPE(None, GLint, GLuint) 	# GL/glext.h:6245
PFNGLUNIFORM2UIPROC = CFUNCTYPE(None, GLint, GLuint, GLuint) 	# GL/glext.h:6246
PFNGLUNIFORM3UIPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:6247
PFNGLUNIFORM4UIPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:6248
PFNGLUNIFORM1UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6249
PFNGLUNIFORM2UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6250
PFNGLUNIFORM3UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6251
PFNGLUNIFORM4UIVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6252
PFNGLTEXPARAMETERIIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6253
PFNGLTEXPARAMETERIUIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:6254
PFNGLGETTEXPARAMETERIIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6255
PFNGLGETTEXPARAMETERIUIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:6256
PFNGLCLEARBUFFERIVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLint)) 	# GL/glext.h:6257
PFNGLCLEARBUFFERUIVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLuint)) 	# GL/glext.h:6258
PFNGLCLEARBUFFERFVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLfloat)) 	# GL/glext.h:6259
PFNGLCLEARBUFFERFIPROC = CFUNCTYPE(None, GLenum, GLint, GLfloat, GLint) 	# GL/glext.h:6260
PFNGLGETSTRINGIPROC = CFUNCTYPE(POINTER(GLubyte), GLenum, GLuint) 	# GL/glext.h:6261
# VERSION_3_1 (GL/glext.h:6264)
GL_VERSION_3_1 = 1 	# GL/glext.h:6265
# GL/glext.h:6270
glDrawArraysInstanced = _link_function('glDrawArraysInstanced', None, [GLenum, GLint, GLsizei, GLsizei], 'VERSION_3_1')

# GL/glext.h:6271
glDrawElementsInstanced = _link_function('glDrawElementsInstanced', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei], 'VERSION_3_1')

# GL/glext.h:6272
glTexBuffer = _link_function('glTexBuffer', None, [GLenum, GLenum, GLuint], 'VERSION_3_1')

# GL/glext.h:6273
glPrimitiveRestartIndex = _link_function('glPrimitiveRestartIndex', None, [GLuint], 'VERSION_3_1')

PFNGLDRAWARRAYSINSTANCEDPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:6275
PFNGLDRAWELEMENTSINSTANCEDPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:6276
PFNGLTEXBUFFERPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:6277
PFNGLPRIMITIVERESTARTINDEXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6278
# VERSION_3_2 (GL/glext.h:6281)
GL_VERSION_3_2 = 1 	# GL/glext.h:6282
# GL/glext.h:6289
glGetInteger64i_v = _link_function('glGetInteger64i_v', None, [GLenum, GLuint, POINTER(GLint64)], 'VERSION_3_2')

# GL/glext.h:6290
glGetBufferParameteri64v = _link_function('glGetBufferParameteri64v', None, [GLenum, GLenum, POINTER(GLint64)], 'VERSION_3_2')

# GL/glext.h:6291
glFramebufferTexture = _link_function('glFramebufferTexture', None, [GLenum, GLenum, GLuint, GLint], 'VERSION_3_2')

PFNGLGETINTEGER64I_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint64)) 	# GL/glext.h:6293
PFNGLGETBUFFERPARAMETERI64VPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint64)) 	# GL/glext.h:6294
PFNGLFRAMEBUFFERTEXTUREPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:6295
# VERSION_3_3 (GL/glext.h:6298)
GL_VERSION_3_3 = 1 	# GL/glext.h:6299
# GL/glext.h:6311
glVertexAttribDivisor = _link_function('glVertexAttribDivisor', None, [GLuint, GLuint], 'VERSION_3_3')

PFNGLVERTEXATTRIBDIVISORPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:6313
# VERSION_4_0 (GL/glext.h:6316)
GL_VERSION_4_0 = 1 	# GL/glext.h:6317
# GL/glext.h:6331
glMinSampleShading = _link_function('glMinSampleShading', None, [GLfloat], 'VERSION_4_0')

# GL/glext.h:6332
glBlendEquationi = _link_function('glBlendEquationi', None, [GLuint, GLenum], 'VERSION_4_0')

# GL/glext.h:6333
glBlendEquationSeparatei = _link_function('glBlendEquationSeparatei', None, [GLuint, GLenum, GLenum], 'VERSION_4_0')

# GL/glext.h:6334
glBlendFunci = _link_function('glBlendFunci', None, [GLuint, GLenum, GLenum], 'VERSION_4_0')

# GL/glext.h:6335
glBlendFuncSeparatei = _link_function('glBlendFuncSeparatei', None, [GLuint, GLenum, GLenum, GLenum, GLenum], 'VERSION_4_0')

PFNGLMINSAMPLESHADINGPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:6337
PFNGLBLENDEQUATIONIPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:6338
PFNGLBLENDEQUATIONSEPARATEIPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum) 	# GL/glext.h:6339
PFNGLBLENDFUNCIPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum) 	# GL/glext.h:6340
PFNGLBLENDFUNCSEPARATEIPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:6341
# VERSION_4_1 (GL/glext.h:6344)
GL_VERSION_4_1 = 1 	# GL/glext.h:6345
# VERSION_4_2 (GL/glext.h:6355)
GL_VERSION_4_2 = 1 	# GL/glext.h:6356
# ARB_multitexture (GL/glext.h:6371)
# ARB_transpose_matrix (GL/glext.h:6445)
GL_ARB_transpose_matrix = 1 	# GL/glext.h:6446
# GL/glext.h:6448
glLoadTransposeMatrixfARB = _link_function('glLoadTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:6449
glLoadTransposeMatrixdARB = _link_function('glLoadTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

# GL/glext.h:6450
glMultTransposeMatrixfARB = _link_function('glMultTransposeMatrixfARB', None, [POINTER(GLfloat)], 'ARB_transpose_matrix')

# GL/glext.h:6451
glMultTransposeMatrixdARB = _link_function('glMultTransposeMatrixdARB', None, [POINTER(GLdouble)], 'ARB_transpose_matrix')

PFNGLLOADTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6453
PFNGLLOADTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6454
PFNGLMULTTRANSPOSEMATRIXFARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6455
PFNGLMULTTRANSPOSEMATRIXDARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6456
# ARB_multisample (GL/glext.h:6459)
GL_ARB_multisample = 1 	# GL/glext.h:6460
# GL/glext.h:6462
glSampleCoverageARB = _link_function('glSampleCoverageARB', None, [GLfloat, GLboolean], 'ARB_multisample')

PFNGLSAMPLECOVERAGEARBPROC = CFUNCTYPE(None, GLfloat, GLboolean) 	# GL/glext.h:6464
# ARB_texture_env_add (GL/glext.h:6467)
GL_ARB_texture_env_add = 1 	# GL/glext.h:6468
# ARB_texture_cube_map (GL/glext.h:6471)
GL_ARB_texture_cube_map = 1 	# GL/glext.h:6472
# ARB_texture_compression (GL/glext.h:6475)
GL_ARB_texture_compression = 1 	# GL/glext.h:6476
# GL/glext.h:6478
glCompressedTexImage3DARB = _link_function('glCompressedTexImage3DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:6479
glCompressedTexImage2DARB = _link_function('glCompressedTexImage2DARB', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:6480
glCompressedTexImage1DARB = _link_function('glCompressedTexImage1DARB', None, [GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:6481
glCompressedTexSubImage3DARB = _link_function('glCompressedTexSubImage3DARB', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:6482
glCompressedTexSubImage2DARB = _link_function('glCompressedTexSubImage2DARB', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:6483
glCompressedTexSubImage1DARB = _link_function('glCompressedTexSubImage1DARB', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_texture_compression')

# GL/glext.h:6484
glGetCompressedTexImageARB = _link_function('glGetCompressedTexImageARB', None, [GLenum, GLint, POINTER(GLvoid)], 'ARB_texture_compression')

PFNGLCOMPRESSEDTEXIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6486
PFNGLCOMPRESSEDTEXIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6487
PFNGLCOMPRESSEDTEXIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6488
PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6489
PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6490
PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6491
PFNGLGETCOMPRESSEDTEXIMAGEARBPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:6492
# ARB_texture_border_clamp (GL/glext.h:6495)
GL_ARB_texture_border_clamp = 1 	# GL/glext.h:6496
# ARB_point_parameters (GL/glext.h:6499)
GL_ARB_point_parameters = 1 	# GL/glext.h:6500
# GL/glext.h:6502
glPointParameterfARB = _link_function('glPointParameterfARB', None, [GLenum, GLfloat], 'ARB_point_parameters')

# GL/glext.h:6503
glPointParameterfvARB = _link_function('glPointParameterfvARB', None, [GLenum, POINTER(GLfloat)], 'ARB_point_parameters')

PFNGLPOINTPARAMETERFARBPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:6505
PFNGLPOINTPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6506
# ARB_vertex_blend (GL/glext.h:6509)
GL_ARB_vertex_blend = 1 	# GL/glext.h:6510
# GL/glext.h:6512
glWeightbvARB = _link_function('glWeightbvARB', None, [GLint, POINTER(GLbyte)], 'ARB_vertex_blend')

# GL/glext.h:6513
glWeightsvARB = _link_function('glWeightsvARB', None, [GLint, POINTER(GLshort)], 'ARB_vertex_blend')

# GL/glext.h:6514
glWeightivARB = _link_function('glWeightivARB', None, [GLint, POINTER(GLint)], 'ARB_vertex_blend')

# GL/glext.h:6515
glWeightfvARB = _link_function('glWeightfvARB', None, [GLint, POINTER(GLfloat)], 'ARB_vertex_blend')

# GL/glext.h:6516
glWeightdvARB = _link_function('glWeightdvARB', None, [GLint, POINTER(GLdouble)], 'ARB_vertex_blend')

# GL/glext.h:6517
glWeightubvARB = _link_function('glWeightubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_vertex_blend')

# GL/glext.h:6518
glWeightusvARB = _link_function('glWeightusvARB', None, [GLint, POINTER(GLushort)], 'ARB_vertex_blend')

# GL/glext.h:6519
glWeightuivARB = _link_function('glWeightuivARB', None, [GLint, POINTER(GLuint)], 'ARB_vertex_blend')

# GL/glext.h:6520
glWeightPointerARB = _link_function('glWeightPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_blend')

# GL/glext.h:6521
glVertexBlendARB = _link_function('glVertexBlendARB', None, [GLint], 'ARB_vertex_blend')

PFNGLWEIGHTBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLbyte)) 	# GL/glext.h:6523
PFNGLWEIGHTSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLshort)) 	# GL/glext.h:6524
PFNGLWEIGHTIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLint)) 	# GL/glext.h:6525
PFNGLWEIGHTFVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLfloat)) 	# GL/glext.h:6526
PFNGLWEIGHTDVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLdouble)) 	# GL/glext.h:6527
PFNGLWEIGHTUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:6528
PFNGLWEIGHTUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:6529
PFNGLWEIGHTUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:6530
PFNGLWEIGHTPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6531
PFNGLVERTEXBLENDARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:6532
# ARB_matrix_palette (GL/glext.h:6535)
GL_ARB_matrix_palette = 1 	# GL/glext.h:6536
# GL/glext.h:6538
glCurrentPaletteMatrixARB = _link_function('glCurrentPaletteMatrixARB', None, [GLint], 'ARB_matrix_palette')

# GL/glext.h:6539
glMatrixIndexubvARB = _link_function('glMatrixIndexubvARB', None, [GLint, POINTER(GLubyte)], 'ARB_matrix_palette')

# GL/glext.h:6540
glMatrixIndexusvARB = _link_function('glMatrixIndexusvARB', None, [GLint, POINTER(GLushort)], 'ARB_matrix_palette')

# GL/glext.h:6541
glMatrixIndexuivARB = _link_function('glMatrixIndexuivARB', None, [GLint, POINTER(GLuint)], 'ARB_matrix_palette')

# GL/glext.h:6542
glMatrixIndexPointerARB = _link_function('glMatrixIndexPointerARB', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_matrix_palette')

PFNGLCURRENTPALETTEMATRIXARBPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:6544
PFNGLMATRIXINDEXUBVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLubyte)) 	# GL/glext.h:6545
PFNGLMATRIXINDEXUSVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLushort)) 	# GL/glext.h:6546
PFNGLMATRIXINDEXUIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLuint)) 	# GL/glext.h:6547
PFNGLMATRIXINDEXPOINTERARBPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6548
# ARB_texture_env_combine (GL/glext.h:6551)
GL_ARB_texture_env_combine = 1 	# GL/glext.h:6552
# ARB_texture_env_crossbar (GL/glext.h:6555)
GL_ARB_texture_env_crossbar = 1 	# GL/glext.h:6556
# ARB_texture_env_dot3 (GL/glext.h:6559)
GL_ARB_texture_env_dot3 = 1 	# GL/glext.h:6560
# ARB_texture_mirrored_repeat (GL/glext.h:6563)
GL_ARB_texture_mirrored_repeat = 1 	# GL/glext.h:6564
# ARB_depth_texture (GL/glext.h:6567)
GL_ARB_depth_texture = 1 	# GL/glext.h:6568
# ARB_shadow (GL/glext.h:6571)
GL_ARB_shadow = 1 	# GL/glext.h:6572
# ARB_shadow_ambient (GL/glext.h:6575)
GL_ARB_shadow_ambient = 1 	# GL/glext.h:6576
# ARB_window_pos (GL/glext.h:6579)
GL_ARB_window_pos = 1 	# GL/glext.h:6580
# GL/glext.h:6582
glWindowPos2dARB = _link_function('glWindowPos2dARB', None, [GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:6583
glWindowPos2dvARB = _link_function('glWindowPos2dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:6584
glWindowPos2fARB = _link_function('glWindowPos2fARB', None, [GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:6585
glWindowPos2fvARB = _link_function('glWindowPos2fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:6586
glWindowPos2iARB = _link_function('glWindowPos2iARB', None, [GLint, GLint], 'ARB_window_pos')

# GL/glext.h:6587
glWindowPos2ivARB = _link_function('glWindowPos2ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:6588
glWindowPos2sARB = _link_function('glWindowPos2sARB', None, [GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:6589
glWindowPos2svARB = _link_function('glWindowPos2svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

# GL/glext.h:6590
glWindowPos3dARB = _link_function('glWindowPos3dARB', None, [GLdouble, GLdouble, GLdouble], 'ARB_window_pos')

# GL/glext.h:6591
glWindowPos3dvARB = _link_function('glWindowPos3dvARB', None, [POINTER(GLdouble)], 'ARB_window_pos')

# GL/glext.h:6592
glWindowPos3fARB = _link_function('glWindowPos3fARB', None, [GLfloat, GLfloat, GLfloat], 'ARB_window_pos')

# GL/glext.h:6593
glWindowPos3fvARB = _link_function('glWindowPos3fvARB', None, [POINTER(GLfloat)], 'ARB_window_pos')

# GL/glext.h:6594
glWindowPos3iARB = _link_function('glWindowPos3iARB', None, [GLint, GLint, GLint], 'ARB_window_pos')

# GL/glext.h:6595
glWindowPos3ivARB = _link_function('glWindowPos3ivARB', None, [POINTER(GLint)], 'ARB_window_pos')

# GL/glext.h:6596
glWindowPos3sARB = _link_function('glWindowPos3sARB', None, [GLshort, GLshort, GLshort], 'ARB_window_pos')

# GL/glext.h:6597
glWindowPos3svARB = _link_function('glWindowPos3svARB', None, [POINTER(GLshort)], 'ARB_window_pos')

PFNGLWINDOWPOS2DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:6599
PFNGLWINDOWPOS2DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6600
PFNGLWINDOWPOS2FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:6601
PFNGLWINDOWPOS2FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6602
PFNGLWINDOWPOS2IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:6603
PFNGLWINDOWPOS2IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6604
PFNGLWINDOWPOS2SARBPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:6605
PFNGLWINDOWPOS2SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6606
PFNGLWINDOWPOS3DARBPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6607
PFNGLWINDOWPOS3DVARBPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:6608
PFNGLWINDOWPOS3FARBPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6609
PFNGLWINDOWPOS3FVARBPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:6610
PFNGLWINDOWPOS3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6611
PFNGLWINDOWPOS3IVARBPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:6612
PFNGLWINDOWPOS3SARBPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:6613
PFNGLWINDOWPOS3SVARBPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:6614
# ARB_vertex_program (GL/glext.h:6617)
GL_ARB_vertex_program = 1 	# GL/glext.h:6618
# GL/glext.h:6620
glVertexAttrib1dARB = _link_function('glVertexAttrib1dARB', None, [GLuint, GLdouble], 'ARB_vertex_program')

# GL/glext.h:6621
glVertexAttrib1dvARB = _link_function('glVertexAttrib1dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6622
glVertexAttrib1fARB = _link_function('glVertexAttrib1fARB', None, [GLuint, GLfloat], 'ARB_vertex_program')

# GL/glext.h:6623
glVertexAttrib1fvARB = _link_function('glVertexAttrib1fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6624
glVertexAttrib1sARB = _link_function('glVertexAttrib1sARB', None, [GLuint, GLshort], 'ARB_vertex_program')

# GL/glext.h:6625
glVertexAttrib1svARB = _link_function('glVertexAttrib1svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:6626
glVertexAttrib2dARB = _link_function('glVertexAttrib2dARB', None, [GLuint, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:6627
glVertexAttrib2dvARB = _link_function('glVertexAttrib2dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6628
glVertexAttrib2fARB = _link_function('glVertexAttrib2fARB', None, [GLuint, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:6629
glVertexAttrib2fvARB = _link_function('glVertexAttrib2fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6630
glVertexAttrib2sARB = _link_function('glVertexAttrib2sARB', None, [GLuint, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:6631
glVertexAttrib2svARB = _link_function('glVertexAttrib2svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:6632
glVertexAttrib3dARB = _link_function('glVertexAttrib3dARB', None, [GLuint, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:6633
glVertexAttrib3dvARB = _link_function('glVertexAttrib3dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6634
glVertexAttrib3fARB = _link_function('glVertexAttrib3fARB', None, [GLuint, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:6635
glVertexAttrib3fvARB = _link_function('glVertexAttrib3fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6636
glVertexAttrib3sARB = _link_function('glVertexAttrib3sARB', None, [GLuint, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:6637
glVertexAttrib3svARB = _link_function('glVertexAttrib3svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:6638
glVertexAttrib4NbvARB = _link_function('glVertexAttrib4NbvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:6639
glVertexAttrib4NivARB = _link_function('glVertexAttrib4NivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:6640
glVertexAttrib4NsvARB = _link_function('glVertexAttrib4NsvARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:6641
glVertexAttrib4NubARB = _link_function('glVertexAttrib4NubARB', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'ARB_vertex_program')

# GL/glext.h:6642
glVertexAttrib4NubvARB = _link_function('glVertexAttrib4NubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:6643
glVertexAttrib4NuivARB = _link_function('glVertexAttrib4NuivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:6644
glVertexAttrib4NusvARB = _link_function('glVertexAttrib4NusvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:6645
glVertexAttrib4bvARB = _link_function('glVertexAttrib4bvARB', None, [GLuint, POINTER(GLbyte)], 'ARB_vertex_program')

# GL/glext.h:6646
glVertexAttrib4dARB = _link_function('glVertexAttrib4dARB', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:6647
glVertexAttrib4dvARB = _link_function('glVertexAttrib4dvARB', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6648
glVertexAttrib4fARB = _link_function('glVertexAttrib4fARB', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:6649
glVertexAttrib4fvARB = _link_function('glVertexAttrib4fvARB', None, [GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6650
glVertexAttrib4ivARB = _link_function('glVertexAttrib4ivARB', None, [GLuint, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:6651
glVertexAttrib4sARB = _link_function('glVertexAttrib4sARB', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'ARB_vertex_program')

# GL/glext.h:6652
glVertexAttrib4svARB = _link_function('glVertexAttrib4svARB', None, [GLuint, POINTER(GLshort)], 'ARB_vertex_program')

# GL/glext.h:6653
glVertexAttrib4ubvARB = _link_function('glVertexAttrib4ubvARB', None, [GLuint, POINTER(GLubyte)], 'ARB_vertex_program')

# GL/glext.h:6654
glVertexAttrib4uivARB = _link_function('glVertexAttrib4uivARB', None, [GLuint, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:6655
glVertexAttrib4usvARB = _link_function('glVertexAttrib4usvARB', None, [GLuint, POINTER(GLushort)], 'ARB_vertex_program')

# GL/glext.h:6656
glVertexAttribPointerARB = _link_function('glVertexAttribPointerARB', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:6657
glEnableVertexAttribArrayARB = _link_function('glEnableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:6658
glDisableVertexAttribArrayARB = _link_function('glDisableVertexAttribArrayARB', None, [GLuint], 'ARB_vertex_program')

# GL/glext.h:6659
glProgramStringARB = _link_function('glProgramStringARB', None, [GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:6660
glBindProgramARB = _link_function('glBindProgramARB', None, [GLenum, GLuint], 'ARB_vertex_program')

# GL/glext.h:6661
glDeleteProgramsARB = _link_function('glDeleteProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:6662
glGenProgramsARB = _link_function('glGenProgramsARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_program')

# GL/glext.h:6663
glProgramEnvParameter4dARB = _link_function('glProgramEnvParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:6664
glProgramEnvParameter4dvARB = _link_function('glProgramEnvParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6665
glProgramEnvParameter4fARB = _link_function('glProgramEnvParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:6666
glProgramEnvParameter4fvARB = _link_function('glProgramEnvParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6667
glProgramLocalParameter4dARB = _link_function('glProgramLocalParameter4dARB', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_program')

# GL/glext.h:6668
glProgramLocalParameter4dvARB = _link_function('glProgramLocalParameter4dvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6669
glProgramLocalParameter4fARB = _link_function('glProgramLocalParameter4fARB', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_vertex_program')

# GL/glext.h:6670
glProgramLocalParameter4fvARB = _link_function('glProgramLocalParameter4fvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6671
glGetProgramEnvParameterdvARB = _link_function('glGetProgramEnvParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6672
glGetProgramEnvParameterfvARB = _link_function('glGetProgramEnvParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6673
glGetProgramLocalParameterdvARB = _link_function('glGetProgramLocalParameterdvARB', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6674
glGetProgramLocalParameterfvARB = _link_function('glGetProgramLocalParameterfvARB', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6675
glGetProgramivARB = _link_function('glGetProgramivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:6676
glGetProgramStringARB = _link_function('glGetProgramStringARB', None, [GLenum, GLenum, POINTER(GLvoid)], 'ARB_vertex_program')

# GL/glext.h:6677
glGetVertexAttribdvARB = _link_function('glGetVertexAttribdvARB', None, [GLuint, GLenum, POINTER(GLdouble)], 'ARB_vertex_program')

# GL/glext.h:6678
glGetVertexAttribfvARB = _link_function('glGetVertexAttribfvARB', None, [GLuint, GLenum, POINTER(GLfloat)], 'ARB_vertex_program')

# GL/glext.h:6679
glGetVertexAttribivARB = _link_function('glGetVertexAttribivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_vertex_program')

# GL/glext.h:6680
glGetVertexAttribPointervARB = _link_function('glGetVertexAttribPointervARB', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_program')

# GL/glext.h:6681
glIsProgramARB = _link_function('glIsProgramARB', GLboolean, [GLuint], 'ARB_vertex_program')

PFNGLVERTEXATTRIB1DARBPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:6683
PFNGLVERTEXATTRIB1DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6684
PFNGLVERTEXATTRIB1FARBPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:6685
PFNGLVERTEXATTRIB1FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6686
PFNGLVERTEXATTRIB1SARBPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:6687
PFNGLVERTEXATTRIB1SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6688
PFNGLVERTEXATTRIB2DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:6689
PFNGLVERTEXATTRIB2DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6690
PFNGLVERTEXATTRIB2FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:6691
PFNGLVERTEXATTRIB2FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6692
PFNGLVERTEXATTRIB2SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:6693
PFNGLVERTEXATTRIB2SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6694
PFNGLVERTEXATTRIB3DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6695
PFNGLVERTEXATTRIB3DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6696
PFNGLVERTEXATTRIB3FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6697
PFNGLVERTEXATTRIB3FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6698
PFNGLVERTEXATTRIB3SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:6699
PFNGLVERTEXATTRIB3SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6700
PFNGLVERTEXATTRIB4NBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:6701
PFNGLVERTEXATTRIB4NIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6702
PFNGLVERTEXATTRIB4NSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6703
PFNGLVERTEXATTRIB4NUBARBPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:6704
PFNGLVERTEXATTRIB4NUBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6705
PFNGLVERTEXATTRIB4NUIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6706
PFNGLVERTEXATTRIB4NUSVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:6707
PFNGLVERTEXATTRIB4BVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:6708
PFNGLVERTEXATTRIB4DARBPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6709
PFNGLVERTEXATTRIB4DVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6710
PFNGLVERTEXATTRIB4FARBPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6711
PFNGLVERTEXATTRIB4FVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6712
PFNGLVERTEXATTRIB4IVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:6713
PFNGLVERTEXATTRIB4SARBPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:6714
PFNGLVERTEXATTRIB4SVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:6715
PFNGLVERTEXATTRIB4UBVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:6716
PFNGLVERTEXATTRIB4UIVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:6717
PFNGLVERTEXATTRIB4USVARBPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:6718
PFNGLVERTEXATTRIBPOINTERARBPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6719
PFNGLENABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6720
PFNGLDISABLEVERTEXATTRIBARRAYARBPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:6721
PFNGLPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:6722
PFNGLBINDPROGRAMARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6723
PFNGLDELETEPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6724
PFNGLGENPROGRAMSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6725
PFNGLPROGRAMENVPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6726
PFNGLPROGRAMENVPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6727
PFNGLPROGRAMENVPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6728
PFNGLPROGRAMENVPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6729
PFNGLPROGRAMLOCALPARAMETER4DARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:6730
PFNGLPROGRAMLOCALPARAMETER4DVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6731
PFNGLPROGRAMLOCALPARAMETER4FARBPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6732
PFNGLPROGRAMLOCALPARAMETER4FVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6733
PFNGLGETPROGRAMENVPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6734
PFNGLGETPROGRAMENVPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6735
PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:6736
PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:6737
PFNGLGETPROGRAMIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6738
PFNGLGETPROGRAMSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:6739
PFNGLGETVERTEXATTRIBDVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:6740
PFNGLGETVERTEXATTRIBFVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6741
PFNGLGETVERTEXATTRIBIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6742
PFNGLGETVERTEXATTRIBPOINTERVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:6743
PFNGLISPROGRAMARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6744
# ARB_fragment_program (GL/glext.h:6747)
GL_ARB_fragment_program = 1 	# GL/glext.h:6748
# ARB_vertex_buffer_object (GL/glext.h:6752)
GL_ARB_vertex_buffer_object = 1 	# GL/glext.h:6753
# GL/glext.h:6755
glBindBufferARB = _link_function('glBindBufferARB', None, [GLenum, GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:6756
glDeleteBuffersARB = _link_function('glDeleteBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:6757
glGenBuffersARB = _link_function('glGenBuffersARB', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_buffer_object')

# GL/glext.h:6758
glIsBufferARB = _link_function('glIsBufferARB', GLboolean, [GLuint], 'ARB_vertex_buffer_object')

# GL/glext.h:6759
glBufferDataARB = _link_function('glBufferDataARB', None, [GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:6760
glBufferSubDataARB = _link_function('glBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:6761
glGetBufferSubDataARB = _link_function('glGetBufferSubDataARB', None, [GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)], 'ARB_vertex_buffer_object')

# GL/glext.h:6762
glMapBufferARB = _link_function('glMapBufferARB', POINTER(GLvoid), [GLenum, GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:6763
glUnmapBufferARB = _link_function('glUnmapBufferARB', GLboolean, [GLenum], 'ARB_vertex_buffer_object')

# GL/glext.h:6764
glGetBufferParameterivARB = _link_function('glGetBufferParameterivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_vertex_buffer_object')

# GL/glext.h:6765
glGetBufferPointervARB = _link_function('glGetBufferPointervARB', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'ARB_vertex_buffer_object')

PFNGLBINDBUFFERARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6767
PFNGLDELETEBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6768
PFNGLGENBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6769
PFNGLISBUFFERARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6770
PFNGLBUFFERDATAARBPROC = CFUNCTYPE(None, GLenum, GLsizeiptrARB, POINTER(GLvoid), GLenum) 	# GL/glext.h:6771
PFNGLBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:6772
PFNGLGETBUFFERSUBDATAARBPROC = CFUNCTYPE(None, GLenum, GLintptrARB, GLsizeiptrARB, POINTER(GLvoid)) 	# GL/glext.h:6773
PFNGLMAPBUFFERARBPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLenum) 	# GL/glext.h:6774
PFNGLUNMAPBUFFERARBPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:6775
PFNGLGETBUFFERPARAMETERIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6776
PFNGLGETBUFFERPOINTERVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:6777
# ARB_occlusion_query (GL/glext.h:6780)
GL_ARB_occlusion_query = 1 	# GL/glext.h:6781
# GL/glext.h:6783
glGenQueriesARB = _link_function('glGenQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:6784
glDeleteQueriesARB = _link_function('glDeleteQueriesARB', None, [GLsizei, POINTER(GLuint)], 'ARB_occlusion_query')

# GL/glext.h:6785
glIsQueryARB = _link_function('glIsQueryARB', GLboolean, [GLuint], 'ARB_occlusion_query')

# GL/glext.h:6786
glBeginQueryARB = _link_function('glBeginQueryARB', None, [GLenum, GLuint], 'ARB_occlusion_query')

# GL/glext.h:6787
glEndQueryARB = _link_function('glEndQueryARB', None, [GLenum], 'ARB_occlusion_query')

# GL/glext.h:6788
glGetQueryivARB = _link_function('glGetQueryivARB', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:6789
glGetQueryObjectivARB = _link_function('glGetQueryObjectivARB', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_occlusion_query')

# GL/glext.h:6790
glGetQueryObjectuivARB = _link_function('glGetQueryObjectuivARB', None, [GLuint, GLenum, POINTER(GLuint)], 'ARB_occlusion_query')

PFNGLGENQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6792
PFNGLDELETEQUERIESARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6793
PFNGLISQUERYARBPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6794
PFNGLBEGINQUERYARBPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6795
PFNGLENDQUERYARBPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:6796
PFNGLGETQUERYIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6797
PFNGLGETQUERYOBJECTIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:6798
PFNGLGETQUERYOBJECTUIVARBPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:6799
# ARB_shader_objects (GL/glext.h:6802)
GL_ARB_shader_objects = 1 	# GL/glext.h:6803
# GL/glext.h:6805
glDeleteObjectARB = _link_function('glDeleteObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6806
glGetHandleARB = _link_function('glGetHandleARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:6807
glDetachObjectARB = _link_function('glDetachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6808
glCreateShaderObjectARB = _link_function('glCreateShaderObjectARB', GLhandleARB, [GLenum], 'ARB_shader_objects')

# GL/glext.h:6809
glShaderSourceARB = _link_function('glShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6810
glCompileShaderARB = _link_function('glCompileShaderARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6811
glCreateProgramObjectARB = _link_function('glCreateProgramObjectARB', GLhandleARB, [], 'ARB_shader_objects')

# GL/glext.h:6812
glAttachObjectARB = _link_function('glAttachObjectARB', None, [GLhandleARB, GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6813
glLinkProgramARB = _link_function('glLinkProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6814
glUseProgramObjectARB = _link_function('glUseProgramObjectARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6815
glValidateProgramARB = _link_function('glValidateProgramARB', None, [GLhandleARB], 'ARB_shader_objects')

# GL/glext.h:6816
glUniform1fARB = _link_function('glUniform1fARB', None, [GLint, GLfloat], 'ARB_shader_objects')

# GL/glext.h:6817
glUniform2fARB = _link_function('glUniform2fARB', None, [GLint, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:6818
glUniform3fARB = _link_function('glUniform3fARB', None, [GLint, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:6819
glUniform4fARB = _link_function('glUniform4fARB', None, [GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_shader_objects')

# GL/glext.h:6820
glUniform1iARB = _link_function('glUniform1iARB', None, [GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:6821
glUniform2iARB = _link_function('glUniform2iARB', None, [GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:6822
glUniform3iARB = _link_function('glUniform3iARB', None, [GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:6823
glUniform4iARB = _link_function('glUniform4iARB', None, [GLint, GLint, GLint, GLint, GLint], 'ARB_shader_objects')

# GL/glext.h:6824
glUniform1fvARB = _link_function('glUniform1fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6825
glUniform2fvARB = _link_function('glUniform2fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6826
glUniform3fvARB = _link_function('glUniform3fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6827
glUniform4fvARB = _link_function('glUniform4fvARB', None, [GLint, GLsizei, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6828
glUniform1ivARB = _link_function('glUniform1ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6829
glUniform2ivARB = _link_function('glUniform2ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6830
glUniform3ivARB = _link_function('glUniform3ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6831
glUniform4ivARB = _link_function('glUniform4ivARB', None, [GLint, GLsizei, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6832
glUniformMatrix2fvARB = _link_function('glUniformMatrix2fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6833
glUniformMatrix3fvARB = _link_function('glUniformMatrix3fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6834
glUniformMatrix4fvARB = _link_function('glUniformMatrix4fvARB', None, [GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6835
glGetObjectParameterfvARB = _link_function('glGetObjectParameterfvARB', None, [GLhandleARB, GLenum, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6836
glGetObjectParameterivARB = _link_function('glGetObjectParameterivARB', None, [GLhandleARB, GLenum, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6837
glGetInfoLogARB = _link_function('glGetInfoLogARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:6838
glGetAttachedObjectsARB = _link_function('glGetAttachedObjectsARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)], 'ARB_shader_objects')

# GL/glext.h:6839
glGetUniformLocationARB = _link_function('glGetUniformLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:6840
glGetActiveUniformARB = _link_function('glGetActiveUniformARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_shader_objects')

# GL/glext.h:6841
glGetUniformfvARB = _link_function('glGetUniformfvARB', None, [GLhandleARB, GLint, POINTER(GLfloat)], 'ARB_shader_objects')

# GL/glext.h:6842
glGetUniformivARB = _link_function('glGetUniformivARB', None, [GLhandleARB, GLint, POINTER(GLint)], 'ARB_shader_objects')

# GL/glext.h:6843
glGetShaderSourceARB = _link_function('glGetShaderSourceARB', None, [GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)], 'ARB_shader_objects')

PFNGLDELETEOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:6845
PFNGLGETHANDLEARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:6846
PFNGLDETACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:6847
PFNGLCREATESHADEROBJECTARBPROC = CFUNCTYPE(GLhandleARB, GLenum) 	# GL/glext.h:6848
PFNGLSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(POINTER(GLcharARB)), POINTER(GLint)) 	# GL/glext.h:6849
PFNGLCOMPILESHADERARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:6850
PFNGLCREATEPROGRAMOBJECTARBPROC = CFUNCTYPE(GLhandleARB) 	# GL/glext.h:6851
PFNGLATTACHOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB, GLhandleARB) 	# GL/glext.h:6852
PFNGLLINKPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:6853
PFNGLUSEPROGRAMOBJECTARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:6854
PFNGLVALIDATEPROGRAMARBPROC = CFUNCTYPE(None, GLhandleARB) 	# GL/glext.h:6855
PFNGLUNIFORM1FARBPROC = CFUNCTYPE(None, GLint, GLfloat) 	# GL/glext.h:6856
PFNGLUNIFORM2FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat) 	# GL/glext.h:6857
PFNGLUNIFORM3FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6858
PFNGLUNIFORM4FARBPROC = CFUNCTYPE(None, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:6859
PFNGLUNIFORM1IARBPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:6860
PFNGLUNIFORM2IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:6861
PFNGLUNIFORM3IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:6862
PFNGLUNIFORM4IARBPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:6863
PFNGLUNIFORM1FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6864
PFNGLUNIFORM2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6865
PFNGLUNIFORM3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6866
PFNGLUNIFORM4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:6867
PFNGLUNIFORM1IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6868
PFNGLUNIFORM2IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6869
PFNGLUNIFORM3IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6870
PFNGLUNIFORM4IVARBPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:6871
PFNGLUNIFORMMATRIX2FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6872
PFNGLUNIFORMMATRIX3FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6873
PFNGLUNIFORMMATRIX4FVARBPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:6874
PFNGLGETOBJECTPARAMETERFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLfloat)) 	# GL/glext.h:6875
PFNGLGETOBJECTPARAMETERIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLenum, POINTER(GLint)) 	# GL/glext.h:6876
PFNGLGETINFOLOGARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:6877
PFNGLGETATTACHEDOBJECTSARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLhandleARB)) 	# GL/glext.h:6878
PFNGLGETUNIFORMLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:6879
PFNGLGETACTIVEUNIFORMARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:6880
PFNGLGETUNIFORMFVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLfloat)) 	# GL/glext.h:6881
PFNGLGETUNIFORMIVARBPROC = CFUNCTYPE(None, GLhandleARB, GLint, POINTER(GLint)) 	# GL/glext.h:6882
PFNGLGETSHADERSOURCEARBPROC = CFUNCTYPE(None, GLhandleARB, GLsizei, POINTER(GLsizei), POINTER(GLcharARB)) 	# GL/glext.h:6883
# ARB_vertex_shader (GL/glext.h:6886)
GL_ARB_vertex_shader = 1 	# GL/glext.h:6887
# GL/glext.h:6889
glBindAttribLocationARB = _link_function('glBindAttribLocationARB', None, [GLhandleARB, GLuint, POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:6890
glGetActiveAttribARB = _link_function('glGetActiveAttribARB', None, [GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)], 'ARB_vertex_shader')

# GL/glext.h:6891
glGetAttribLocationARB = _link_function('glGetAttribLocationARB', GLint, [GLhandleARB, POINTER(GLcharARB)], 'ARB_vertex_shader')

PFNGLBINDATTRIBLOCATIONARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, POINTER(GLcharARB)) 	# GL/glext.h:6893
PFNGLGETACTIVEATTRIBARBPROC = CFUNCTYPE(None, GLhandleARB, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLint), POINTER(GLenum), POINTER(GLcharARB)) 	# GL/glext.h:6894
PFNGLGETATTRIBLOCATIONARBPROC = CFUNCTYPE(GLint, GLhandleARB, POINTER(GLcharARB)) 	# GL/glext.h:6895
# ARB_fragment_shader (GL/glext.h:6898)
GL_ARB_fragment_shader = 1 	# GL/glext.h:6899
# ARB_shading_language_100 (GL/glext.h:6902)
GL_ARB_shading_language_100 = 1 	# GL/glext.h:6903
# ARB_texture_non_power_of_two (GL/glext.h:6906)
GL_ARB_texture_non_power_of_two = 1 	# GL/glext.h:6907
# ARB_point_sprite (GL/glext.h:6910)
GL_ARB_point_sprite = 1 	# GL/glext.h:6911
# ARB_fragment_program_shadow (GL/glext.h:6914)
GL_ARB_fragment_program_shadow = 1 	# GL/glext.h:6915
# ARB_draw_buffers (GL/glext.h:6918)
GL_ARB_draw_buffers = 1 	# GL/glext.h:6919
# GL/glext.h:6921
glDrawBuffersARB = _link_function('glDrawBuffersARB', None, [GLsizei, POINTER(GLenum)], 'ARB_draw_buffers')

PFNGLDRAWBUFFERSARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:6923
# ARB_texture_rectangle (GL/glext.h:6926)
GL_ARB_texture_rectangle = 1 	# GL/glext.h:6927
# ARB_color_buffer_float (GL/glext.h:6930)
GL_ARB_color_buffer_float = 1 	# GL/glext.h:6931
# GL/glext.h:6933
glClampColorARB = _link_function('glClampColorARB', None, [GLenum, GLenum], 'ARB_color_buffer_float')

PFNGLCLAMPCOLORARBPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:6935
# ARB_half_float_pixel (GL/glext.h:6938)
GL_ARB_half_float_pixel = 1 	# GL/glext.h:6939
# ARB_texture_float (GL/glext.h:6942)
GL_ARB_texture_float = 1 	# GL/glext.h:6943
# ARB_pixel_buffer_object (GL/glext.h:6946)
GL_ARB_pixel_buffer_object = 1 	# GL/glext.h:6947
# ARB_depth_buffer_float (GL/glext.h:6950)
GL_ARB_depth_buffer_float = 1 	# GL/glext.h:6951
# ARB_draw_instanced (GL/glext.h:6954)
GL_ARB_draw_instanced = 1 	# GL/glext.h:6955
# GL/glext.h:6957
glDrawArraysInstancedARB = _link_function('glDrawArraysInstancedARB', None, [GLenum, GLint, GLsizei, GLsizei], 'ARB_draw_instanced')

# GL/glext.h:6958
glDrawElementsInstancedARB = _link_function('glDrawElementsInstancedARB', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei], 'ARB_draw_instanced')

PFNGLDRAWARRAYSINSTANCEDARBPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:6960
PFNGLDRAWELEMENTSINSTANCEDARBPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:6961
# ARB_framebuffer_object (GL/glext.h:6964)
GL_ARB_framebuffer_object = 1 	# GL/glext.h:6965
# GL/glext.h:6967
glIsRenderbuffer = _link_function('glIsRenderbuffer', GLboolean, [GLuint], 'ARB_framebuffer_object')

# GL/glext.h:6968
glBindRenderbuffer = _link_function('glBindRenderbuffer', None, [GLenum, GLuint], 'ARB_framebuffer_object')

# GL/glext.h:6969
glDeleteRenderbuffers = _link_function('glDeleteRenderbuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:6970
glGenRenderbuffers = _link_function('glGenRenderbuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:6971
glRenderbufferStorage = _link_function('glRenderbufferStorage', None, [GLenum, GLenum, GLsizei, GLsizei], 'ARB_framebuffer_object')

# GL/glext.h:6972
glGetRenderbufferParameteriv = _link_function('glGetRenderbufferParameteriv', None, [GLenum, GLenum, POINTER(GLint)], 'ARB_framebuffer_object')

# GL/glext.h:6973
glIsFramebuffer = _link_function('glIsFramebuffer', GLboolean, [GLuint], 'ARB_framebuffer_object')

# GL/glext.h:6974
glBindFramebuffer = _link_function('glBindFramebuffer', None, [GLenum, GLuint], 'ARB_framebuffer_object')

# GL/glext.h:6975
glDeleteFramebuffers = _link_function('glDeleteFramebuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:6976
glGenFramebuffers = _link_function('glGenFramebuffers', None, [GLsizei, POINTER(GLuint)], 'ARB_framebuffer_object')

# GL/glext.h:6977
glCheckFramebufferStatus = _link_function('glCheckFramebufferStatus', GLenum, [GLenum], 'ARB_framebuffer_object')

# GL/glext.h:6978
glFramebufferTexture1D = _link_function('glFramebufferTexture1D', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'ARB_framebuffer_object')

# GL/glext.h:6979
glFramebufferTexture2D = _link_function('glFramebufferTexture2D', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'ARB_framebuffer_object')

# GL/glext.h:6980
glFramebufferTexture3D = _link_function('glFramebufferTexture3D', None, [GLenum, GLenum, GLenum, GLuint, GLint, GLint], 'ARB_framebuffer_object')

# GL/glext.h:6981
glFramebufferRenderbuffer = _link_function('glFramebufferRenderbuffer', None, [GLenum, GLenum, GLenum, GLuint], 'ARB_framebuffer_object')

# GL/glext.h:6982
glGetFramebufferAttachmentParameteriv = _link_function('glGetFramebufferAttachmentParameteriv', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'ARB_framebuffer_object')

# GL/glext.h:6983
glGenerateMipmap = _link_function('glGenerateMipmap', None, [GLenum], 'ARB_framebuffer_object')

GLbitfield = c_uint 	# /usr/include/GL/gl.h:151
# GL/glext.h:6984
glBlitFramebuffer = _link_function('glBlitFramebuffer', None, [GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum], 'ARB_framebuffer_object')

# GL/glext.h:6985
glRenderbufferStorageMultisample = _link_function('glRenderbufferStorageMultisample', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'ARB_framebuffer_object')

# GL/glext.h:6986
glFramebufferTextureLayer = _link_function('glFramebufferTextureLayer', None, [GLenum, GLenum, GLuint, GLint, GLint], 'ARB_framebuffer_object')

PFNGLISRENDERBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6988
PFNGLBINDRENDERBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6989
PFNGLDELETERENDERBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6990
PFNGLGENRENDERBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6991
PFNGLRENDERBUFFERSTORAGEPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei) 	# GL/glext.h:6992
PFNGLGETRENDERBUFFERPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:6993
PFNGLISFRAMEBUFFERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:6994
PFNGLBINDFRAMEBUFFERPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:6995
PFNGLDELETEFRAMEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6996
PFNGLGENFRAMEBUFFERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:6997
PFNGLCHECKFRAMEBUFFERSTATUSPROC = CFUNCTYPE(GLenum, GLenum) 	# GL/glext.h:6998
PFNGLFRAMEBUFFERTEXTURE1DPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:6999
PFNGLFRAMEBUFFERTEXTURE2DPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:7000
PFNGLFRAMEBUFFERTEXTURE3DPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:7001
PFNGLFRAMEBUFFERRENDERBUFFERPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:7002
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7003
PFNGLGENERATEMIPMAPPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7004
PFNGLBLITFRAMEBUFFERPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum) 	# GL/glext.h:7005
PFNGLRENDERBUFFERSTORAGEMULTISAMPLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7006
PFNGLFRAMEBUFFERTEXTURELAYERPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:7007
# ARB_framebuffer_sRGB (GL/glext.h:7010)
GL_ARB_framebuffer_sRGB = 1 	# GL/glext.h:7011
# ARB_geometry_shader4 (GL/glext.h:7014)
GL_ARB_geometry_shader4 = 1 	# GL/glext.h:7015
# GL/glext.h:7017
glProgramParameteriARB = _link_function('glProgramParameteriARB', None, [GLuint, GLenum, GLint], 'ARB_geometry_shader4')

# GL/glext.h:7018
glFramebufferTextureARB = _link_function('glFramebufferTextureARB', None, [GLenum, GLenum, GLuint, GLint], 'ARB_geometry_shader4')

# GL/glext.h:7019
glFramebufferTextureLayerARB = _link_function('glFramebufferTextureLayerARB', None, [GLenum, GLenum, GLuint, GLint, GLint], 'ARB_geometry_shader4')

# GL/glext.h:7020
glFramebufferTextureFaceARB = _link_function('glFramebufferTextureFaceARB', None, [GLenum, GLenum, GLuint, GLint, GLenum], 'ARB_geometry_shader4')

PFNGLPROGRAMPARAMETERIARBPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:7022
PFNGLFRAMEBUFFERTEXTUREARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:7023
PFNGLFRAMEBUFFERTEXTURELAYERARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:7024
PFNGLFRAMEBUFFERTEXTUREFACEARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:7025
# ARB_half_float_vertex (GL/glext.h:7028)
GL_ARB_half_float_vertex = 1 	# GL/glext.h:7029
# ARB_instanced_arrays (GL/glext.h:7032)
GL_ARB_instanced_arrays = 1 	# GL/glext.h:7033
# GL/glext.h:7035
glVertexAttribDivisorARB = _link_function('glVertexAttribDivisorARB', None, [GLuint, GLuint], 'ARB_instanced_arrays')

PFNGLVERTEXATTRIBDIVISORARBPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:7037
# ARB_map_buffer_range (GL/glext.h:7040)
GL_ARB_map_buffer_range = 1 	# GL/glext.h:7041
# GL/glext.h:7043
glMapBufferRange = _link_function('glMapBufferRange', POINTER(GLvoid), [GLenum, GLintptr, GLsizeiptr, GLbitfield], 'ARB_map_buffer_range')

# GL/glext.h:7044
glFlushMappedBufferRange = _link_function('glFlushMappedBufferRange', None, [GLenum, GLintptr, GLsizeiptr], 'ARB_map_buffer_range')

PFNGLMAPBUFFERRANGEPROC = CFUNCTYPE(POINTER(GLvoid), GLenum, GLintptr, GLsizeiptr, GLbitfield) 	# GL/glext.h:7046
PFNGLFLUSHMAPPEDBUFFERRANGEPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr) 	# GL/glext.h:7047
# ARB_texture_buffer_object (GL/glext.h:7050)
GL_ARB_texture_buffer_object = 1 	# GL/glext.h:7051
# GL/glext.h:7053
glTexBufferARB = _link_function('glTexBufferARB', None, [GLenum, GLenum, GLuint], 'ARB_texture_buffer_object')

PFNGLTEXBUFFERARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:7055
# ARB_texture_compression_rgtc (GL/glext.h:7058)
GL_ARB_texture_compression_rgtc = 1 	# GL/glext.h:7059
# ARB_texture_rg (GL/glext.h:7062)
GL_ARB_texture_rg = 1 	# GL/glext.h:7063
# ARB_vertex_array_object (GL/glext.h:7066)
GL_ARB_vertex_array_object = 1 	# GL/glext.h:7067
# GL/glext.h:7069
glBindVertexArray = _link_function('glBindVertexArray', None, [GLuint], 'ARB_vertex_array_object')

# GL/glext.h:7070
glDeleteVertexArrays = _link_function('glDeleteVertexArrays', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_array_object')

# GL/glext.h:7071
glGenVertexArrays = _link_function('glGenVertexArrays', None, [GLsizei, POINTER(GLuint)], 'ARB_vertex_array_object')

# GL/glext.h:7072
glIsVertexArray = _link_function('glIsVertexArray', GLboolean, [GLuint], 'ARB_vertex_array_object')

PFNGLBINDVERTEXARRAYPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7074
PFNGLDELETEVERTEXARRAYSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7075
PFNGLGENVERTEXARRAYSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7076
PFNGLISVERTEXARRAYPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7077
# ARB_uniform_buffer_object (GL/glext.h:7080)
GL_ARB_uniform_buffer_object = 1 	# GL/glext.h:7081
# GL/glext.h:7083
glGetUniformIndices = _link_function('glGetUniformIndices', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLuint)], 'ARB_uniform_buffer_object')

# GL/glext.h:7084
glGetActiveUniformsiv = _link_function('glGetActiveUniformsiv', None, [GLuint, GLsizei, POINTER(GLuint), GLenum, POINTER(GLint)], 'ARB_uniform_buffer_object')

# GL/glext.h:7085
glGetActiveUniformName = _link_function('glGetActiveUniformName', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'ARB_uniform_buffer_object')

# GL/glext.h:7086
glGetUniformBlockIndex = _link_function('glGetUniformBlockIndex', GLuint, [GLuint, POINTER(GLchar)], 'ARB_uniform_buffer_object')

# GL/glext.h:7087
glGetActiveUniformBlockiv = _link_function('glGetActiveUniformBlockiv', None, [GLuint, GLuint, GLenum, POINTER(GLint)], 'ARB_uniform_buffer_object')

# GL/glext.h:7088
glGetActiveUniformBlockName = _link_function('glGetActiveUniformBlockName', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'ARB_uniform_buffer_object')

# GL/glext.h:7089
glUniformBlockBinding = _link_function('glUniformBlockBinding', None, [GLuint, GLuint, GLuint], 'ARB_uniform_buffer_object')

PFNGLGETUNIFORMINDICESPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLuint)) 	# GL/glext.h:7091
PFNGLGETACTIVEUNIFORMSIVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLuint), GLenum, POINTER(GLint)) 	# GL/glext.h:7092
PFNGLGETACTIVEUNIFORMNAMEPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:7093
PFNGLGETUNIFORMBLOCKINDEXPROC = CFUNCTYPE(GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:7094
PFNGLGETACTIVEUNIFORMBLOCKIVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7095
PFNGLGETACTIVEUNIFORMBLOCKNAMEPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:7096
PFNGLUNIFORMBLOCKBINDINGPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:7097
# ARB_compatibility (GL/glext.h:7100)
GL_ARB_compatibility = 1 	# GL/glext.h:7101
# ARB_copy_buffer (GL/glext.h:7104)
GL_ARB_copy_buffer = 1 	# GL/glext.h:7105
# GL/glext.h:7107
glCopyBufferSubData = _link_function('glCopyBufferSubData', None, [GLenum, GLenum, GLintptr, GLintptr, GLsizeiptr], 'ARB_copy_buffer')

PFNGLCOPYBUFFERSUBDATAPROC = CFUNCTYPE(None, GLenum, GLenum, GLintptr, GLintptr, GLsizeiptr) 	# GL/glext.h:7109
# ARB_shader_texture_lod (GL/glext.h:7112)
GL_ARB_shader_texture_lod = 1 	# GL/glext.h:7113
# ARB_depth_clamp (GL/glext.h:7116)
GL_ARB_depth_clamp = 1 	# GL/glext.h:7117
# ARB_draw_elements_base_vertex (GL/glext.h:7120)
GL_ARB_draw_elements_base_vertex = 1 	# GL/glext.h:7121
# GL/glext.h:7123
glDrawElementsBaseVertex = _link_function('glDrawElementsBaseVertex', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLint], 'ARB_draw_elements_base_vertex')

# GL/glext.h:7124
glDrawRangeElementsBaseVertex = _link_function('glDrawRangeElementsBaseVertex', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid), GLint], 'ARB_draw_elements_base_vertex')

# GL/glext.h:7125
glDrawElementsInstancedBaseVertex = _link_function('glDrawElementsInstancedBaseVertex', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei, GLint], 'ARB_draw_elements_base_vertex')

# GL/glext.h:7126
glMultiDrawElementsBaseVertex = _link_function('glMultiDrawElementsBaseVertex', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, POINTER(GLint)], 'ARB_draw_elements_base_vertex')

PFNGLDRAWELEMENTSBASEVERTEXPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLint) 	# GL/glext.h:7128
PFNGLDRAWRANGEELEMENTSBASEVERTEXPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid), GLint) 	# GL/glext.h:7129
PFNGLDRAWELEMENTSINSTANCEDBASEVERTEXPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei, GLint) 	# GL/glext.h:7130
PFNGLMULTIDRAWELEMENTSBASEVERTEXPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, POINTER(GLint)) 	# GL/glext.h:7131
# ARB_fragment_coord_conventions (GL/glext.h:7134)
GL_ARB_fragment_coord_conventions = 1 	# GL/glext.h:7135
# ARB_provoking_vertex (GL/glext.h:7138)
GL_ARB_provoking_vertex = 1 	# GL/glext.h:7139
# GL/glext.h:7141
glProvokingVertex = _link_function('glProvokingVertex', None, [GLenum], 'ARB_provoking_vertex')

PFNGLPROVOKINGVERTEXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7143
# ARB_seamless_cube_map (GL/glext.h:7146)
GL_ARB_seamless_cube_map = 1 	# GL/glext.h:7147
# ARB_sync (GL/glext.h:7150)
GL_ARB_sync = 1 	# GL/glext.h:7151
# GL/glext.h:7153
glFenceSync = _link_function('glFenceSync', GLsync, [GLenum, GLbitfield], 'ARB_sync')

# GL/glext.h:7154
glIsSync = _link_function('glIsSync', GLboolean, [GLsync], 'ARB_sync')

# GL/glext.h:7155
glDeleteSync = _link_function('glDeleteSync', None, [GLsync], 'ARB_sync')

# GL/glext.h:7156
glClientWaitSync = _link_function('glClientWaitSync', GLenum, [GLsync, GLbitfield, GLuint64], 'ARB_sync')

# GL/glext.h:7157
glWaitSync = _link_function('glWaitSync', None, [GLsync, GLbitfield, GLuint64], 'ARB_sync')

# GL/glext.h:7158
glGetInteger64v = _link_function('glGetInteger64v', None, [GLenum, POINTER(GLint64)], 'ARB_sync')

# GL/glext.h:7159
glGetSynciv = _link_function('glGetSynciv', None, [GLsync, GLenum, GLsizei, POINTER(GLsizei), POINTER(GLint)], 'ARB_sync')

PFNGLFENCESYNCPROC = CFUNCTYPE(GLsync, GLenum, GLbitfield) 	# GL/glext.h:7161
PFNGLISSYNCPROC = CFUNCTYPE(GLboolean, GLsync) 	# GL/glext.h:7162
PFNGLDELETESYNCPROC = CFUNCTYPE(None, GLsync) 	# GL/glext.h:7163
PFNGLCLIENTWAITSYNCPROC = CFUNCTYPE(GLenum, GLsync, GLbitfield, GLuint64) 	# GL/glext.h:7164
PFNGLWAITSYNCPROC = CFUNCTYPE(None, GLsync, GLbitfield, GLuint64) 	# GL/glext.h:7165
PFNGLGETINTEGER64VPROC = CFUNCTYPE(None, GLenum, POINTER(GLint64)) 	# GL/glext.h:7166
PFNGLGETSYNCIVPROC = CFUNCTYPE(None, GLsync, GLenum, GLsizei, POINTER(GLsizei), POINTER(GLint)) 	# GL/glext.h:7167
# ARB_texture_multisample (GL/glext.h:7170)
GL_ARB_texture_multisample = 1 	# GL/glext.h:7171
# GL/glext.h:7173
glTexImage2DMultisample = _link_function('glTexImage2DMultisample', None, [GLenum, GLsizei, GLint, GLsizei, GLsizei, GLboolean], 'ARB_texture_multisample')

# GL/glext.h:7174
glTexImage3DMultisample = _link_function('glTexImage3DMultisample', None, [GLenum, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean], 'ARB_texture_multisample')

# GL/glext.h:7175
glGetMultisamplefv = _link_function('glGetMultisamplefv', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_texture_multisample')

# GL/glext.h:7176
glSampleMaski = _link_function('glSampleMaski', None, [GLuint, GLbitfield], 'ARB_texture_multisample')

PFNGLTEXIMAGE2DMULTISAMPLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:7178
PFNGLTEXIMAGE3DMULTISAMPLEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:7179
PFNGLGETMULTISAMPLEFVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:7180
PFNGLSAMPLEMASKIPROC = CFUNCTYPE(None, GLuint, GLbitfield) 	# GL/glext.h:7181
# ARB_vertex_array_bgra (GL/glext.h:7184)
GL_ARB_vertex_array_bgra = 1 	# GL/glext.h:7185
# ARB_draw_buffers_blend (GL/glext.h:7188)
GL_ARB_draw_buffers_blend = 1 	# GL/glext.h:7189
# GL/glext.h:7191
glBlendEquationiARB = _link_function('glBlendEquationiARB', None, [GLuint, GLenum], 'ARB_draw_buffers_blend')

# GL/glext.h:7192
glBlendEquationSeparateiARB = _link_function('glBlendEquationSeparateiARB', None, [GLuint, GLenum, GLenum], 'ARB_draw_buffers_blend')

# GL/glext.h:7193
glBlendFunciARB = _link_function('glBlendFunciARB', None, [GLuint, GLenum, GLenum], 'ARB_draw_buffers_blend')

# GL/glext.h:7194
glBlendFuncSeparateiARB = _link_function('glBlendFuncSeparateiARB', None, [GLuint, GLenum, GLenum, GLenum, GLenum], 'ARB_draw_buffers_blend')

PFNGLBLENDEQUATIONIARBPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:7196
PFNGLBLENDEQUATIONSEPARATEIARBPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum) 	# GL/glext.h:7197
PFNGLBLENDFUNCIARBPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum) 	# GL/glext.h:7198
PFNGLBLENDFUNCSEPARATEIARBPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:7199
# ARB_sample_shading (GL/glext.h:7202)
GL_ARB_sample_shading = 1 	# GL/glext.h:7203
# GL/glext.h:7205
glMinSampleShadingARB = _link_function('glMinSampleShadingARB', None, [GLfloat], 'ARB_sample_shading')

PFNGLMINSAMPLESHADINGARBPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:7207
# ARB_texture_cube_map_array (GL/glext.h:7210)
GL_ARB_texture_cube_map_array = 1 	# GL/glext.h:7211
# ARB_texture_gather (GL/glext.h:7214)
GL_ARB_texture_gather = 1 	# GL/glext.h:7215
# ARB_texture_query_lod (GL/glext.h:7218)
GL_ARB_texture_query_lod = 1 	# GL/glext.h:7219
# ARB_shading_language_include (GL/glext.h:7222)
GL_ARB_shading_language_include = 1 	# GL/glext.h:7223
# GL/glext.h:7225
glNamedStringARB = _link_function('glNamedStringARB', None, [GLenum, GLint, POINTER(GLchar), GLint, POINTER(GLchar)], 'ARB_shading_language_include')

# GL/glext.h:7226
glDeleteNamedStringARB = _link_function('glDeleteNamedStringARB', None, [GLint, POINTER(GLchar)], 'ARB_shading_language_include')

# GL/glext.h:7227
glCompileShaderIncludeARB = _link_function('glCompileShaderIncludeARB', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)], 'ARB_shading_language_include')

# GL/glext.h:7228
glIsNamedStringARB = _link_function('glIsNamedStringARB', GLboolean, [GLint, POINTER(GLchar)], 'ARB_shading_language_include')

# GL/glext.h:7229
glGetNamedStringARB = _link_function('glGetNamedStringARB', None, [GLint, POINTER(GLchar), GLsizei, POINTER(GLint), POINTER(GLchar)], 'ARB_shading_language_include')

# GL/glext.h:7230
glGetNamedStringivARB = _link_function('glGetNamedStringivARB', None, [GLint, POINTER(GLchar), GLenum, POINTER(GLint)], 'ARB_shading_language_include')

PFNGLNAMEDSTRINGARBPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLchar), GLint, POINTER(GLchar)) 	# GL/glext.h:7232
PFNGLDELETENAMEDSTRINGARBPROC = CFUNCTYPE(None, GLint, POINTER(GLchar)) 	# GL/glext.h:7233
PFNGLCOMPILESHADERINCLUDEARBPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), POINTER(GLint)) 	# GL/glext.h:7234
PFNGLISNAMEDSTRINGARBPROC = CFUNCTYPE(GLboolean, GLint, POINTER(GLchar)) 	# GL/glext.h:7235
PFNGLGETNAMEDSTRINGARBPROC = CFUNCTYPE(None, GLint, POINTER(GLchar), GLsizei, POINTER(GLint), POINTER(GLchar)) 	# GL/glext.h:7236
PFNGLGETNAMEDSTRINGIVARBPROC = CFUNCTYPE(None, GLint, POINTER(GLchar), GLenum, POINTER(GLint)) 	# GL/glext.h:7237
# ARB_texture_compression_bptc (GL/glext.h:7240)
GL_ARB_texture_compression_bptc = 1 	# GL/glext.h:7241
# ARB_blend_func_extended (GL/glext.h:7244)
GL_ARB_blend_func_extended = 1 	# GL/glext.h:7245
# GL/glext.h:7247
glBindFragDataLocationIndexed = _link_function('glBindFragDataLocationIndexed', None, [GLuint, GLuint, GLuint, POINTER(GLchar)], 'ARB_blend_func_extended')

# GL/glext.h:7248
glGetFragDataIndex = _link_function('glGetFragDataIndex', GLint, [GLuint, POINTER(GLchar)], 'ARB_blend_func_extended')

PFNGLBINDFRAGDATALOCATIONINDEXEDPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:7250
PFNGLGETFRAGDATAINDEXPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:7251
# ARB_explicit_attrib_location (GL/glext.h:7254)
GL_ARB_explicit_attrib_location = 1 	# GL/glext.h:7255
# ARB_occlusion_query2 (GL/glext.h:7258)
GL_ARB_occlusion_query2 = 1 	# GL/glext.h:7259
# ARB_sampler_objects (GL/glext.h:7262)
GL_ARB_sampler_objects = 1 	# GL/glext.h:7263
# GL/glext.h:7265
glGenSamplers = _link_function('glGenSamplers', None, [GLsizei, POINTER(GLuint)], 'ARB_sampler_objects')

# GL/glext.h:7266
glDeleteSamplers = _link_function('glDeleteSamplers', None, [GLsizei, POINTER(GLuint)], 'ARB_sampler_objects')

# GL/glext.h:7267
glIsSampler = _link_function('glIsSampler', GLboolean, [GLuint], 'ARB_sampler_objects')

# GL/glext.h:7268
glBindSampler = _link_function('glBindSampler', None, [GLuint, GLuint], 'ARB_sampler_objects')

# GL/glext.h:7269
glSamplerParameteri = _link_function('glSamplerParameteri', None, [GLuint, GLenum, GLint], 'ARB_sampler_objects')

# GL/glext.h:7270
glSamplerParameteriv = _link_function('glSamplerParameteriv', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_sampler_objects')

# GL/glext.h:7271
glSamplerParameterf = _link_function('glSamplerParameterf', None, [GLuint, GLenum, GLfloat], 'ARB_sampler_objects')

# GL/glext.h:7272
glSamplerParameterfv = _link_function('glSamplerParameterfv', None, [GLuint, GLenum, POINTER(GLfloat)], 'ARB_sampler_objects')

# GL/glext.h:7273
glSamplerParameterIiv = _link_function('glSamplerParameterIiv', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_sampler_objects')

# GL/glext.h:7274
glSamplerParameterIuiv = _link_function('glSamplerParameterIuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'ARB_sampler_objects')

# GL/glext.h:7275
glGetSamplerParameteriv = _link_function('glGetSamplerParameteriv', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_sampler_objects')

# GL/glext.h:7276
glGetSamplerParameterIiv = _link_function('glGetSamplerParameterIiv', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_sampler_objects')

# GL/glext.h:7277
glGetSamplerParameterfv = _link_function('glGetSamplerParameterfv', None, [GLuint, GLenum, POINTER(GLfloat)], 'ARB_sampler_objects')

# GL/glext.h:7278
glGetSamplerParameterIuiv = _link_function('glGetSamplerParameterIuiv', None, [GLuint, GLenum, POINTER(GLuint)], 'ARB_sampler_objects')

PFNGLGENSAMPLERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7280
PFNGLDELETESAMPLERSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7281
PFNGLISSAMPLERPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7282
PFNGLBINDSAMPLERPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:7283
PFNGLSAMPLERPARAMETERIPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:7284
PFNGLSAMPLERPARAMETERIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7285
PFNGLSAMPLERPARAMETERFPROC = CFUNCTYPE(None, GLuint, GLenum, GLfloat) 	# GL/glext.h:7286
PFNGLSAMPLERPARAMETERFVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7287
PFNGLSAMPLERPARAMETERIIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7288
PFNGLSAMPLERPARAMETERIUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:7289
PFNGLGETSAMPLERPARAMETERIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7290
PFNGLGETSAMPLERPARAMETERIIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7291
PFNGLGETSAMPLERPARAMETERFVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7292
PFNGLGETSAMPLERPARAMETERIUIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:7293
# ARB_shader_bit_encoding (GL/glext.h:7296)
GL_ARB_shader_bit_encoding = 1 	# GL/glext.h:7297
# ARB_texture_rgb10_a2ui (GL/glext.h:7300)
GL_ARB_texture_rgb10_a2ui = 1 	# GL/glext.h:7301
# ARB_texture_swizzle (GL/glext.h:7304)
GL_ARB_texture_swizzle = 1 	# GL/glext.h:7305
# ARB_timer_query (GL/glext.h:7308)
GL_ARB_timer_query = 1 	# GL/glext.h:7309
# GL/glext.h:7311
glQueryCounter = _link_function('glQueryCounter', None, [GLuint, GLenum], 'ARB_timer_query')

# GL/glext.h:7312
glGetQueryObjecti64v = _link_function('glGetQueryObjecti64v', None, [GLuint, GLenum, POINTER(GLint64)], 'ARB_timer_query')

# GL/glext.h:7313
glGetQueryObjectui64v = _link_function('glGetQueryObjectui64v', None, [GLuint, GLenum, POINTER(GLuint64)], 'ARB_timer_query')

PFNGLQUERYCOUNTERPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:7315
PFNGLGETQUERYOBJECTI64VPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64)) 	# GL/glext.h:7316
PFNGLGETQUERYOBJECTUI64VPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64)) 	# GL/glext.h:7317
# ARB_vertex_type_2_10_10_10_rev (GL/glext.h:7320)
GL_ARB_vertex_type_2_10_10_10_rev = 1 	# GL/glext.h:7321
# GL/glext.h:7323
glVertexP2ui = _link_function('glVertexP2ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7324
glVertexP2uiv = _link_function('glVertexP2uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7325
glVertexP3ui = _link_function('glVertexP3ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7326
glVertexP3uiv = _link_function('glVertexP3uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7327
glVertexP4ui = _link_function('glVertexP4ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7328
glVertexP4uiv = _link_function('glVertexP4uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7329
glTexCoordP1ui = _link_function('glTexCoordP1ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7330
glTexCoordP1uiv = _link_function('glTexCoordP1uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7331
glTexCoordP2ui = _link_function('glTexCoordP2ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7332
glTexCoordP2uiv = _link_function('glTexCoordP2uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7333
glTexCoordP3ui = _link_function('glTexCoordP3ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7334
glTexCoordP3uiv = _link_function('glTexCoordP3uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7335
glTexCoordP4ui = _link_function('glTexCoordP4ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7336
glTexCoordP4uiv = _link_function('glTexCoordP4uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7337
glMultiTexCoordP1ui = _link_function('glMultiTexCoordP1ui', None, [GLenum, GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7338
glMultiTexCoordP1uiv = _link_function('glMultiTexCoordP1uiv', None, [GLenum, GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7339
glMultiTexCoordP2ui = _link_function('glMultiTexCoordP2ui', None, [GLenum, GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7340
glMultiTexCoordP2uiv = _link_function('glMultiTexCoordP2uiv', None, [GLenum, GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7341
glMultiTexCoordP3ui = _link_function('glMultiTexCoordP3ui', None, [GLenum, GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7342
glMultiTexCoordP3uiv = _link_function('glMultiTexCoordP3uiv', None, [GLenum, GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7343
glMultiTexCoordP4ui = _link_function('glMultiTexCoordP4ui', None, [GLenum, GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7344
glMultiTexCoordP4uiv = _link_function('glMultiTexCoordP4uiv', None, [GLenum, GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7345
glNormalP3ui = _link_function('glNormalP3ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7346
glNormalP3uiv = _link_function('glNormalP3uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7347
glColorP3ui = _link_function('glColorP3ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7348
glColorP3uiv = _link_function('glColorP3uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7349
glColorP4ui = _link_function('glColorP4ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7350
glColorP4uiv = _link_function('glColorP4uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7351
glSecondaryColorP3ui = _link_function('glSecondaryColorP3ui', None, [GLenum, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7352
glSecondaryColorP3uiv = _link_function('glSecondaryColorP3uiv', None, [GLenum, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7353
glVertexAttribP1ui = _link_function('glVertexAttribP1ui', None, [GLuint, GLenum, GLboolean, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7354
glVertexAttribP1uiv = _link_function('glVertexAttribP1uiv', None, [GLuint, GLenum, GLboolean, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7355
glVertexAttribP2ui = _link_function('glVertexAttribP2ui', None, [GLuint, GLenum, GLboolean, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7356
glVertexAttribP2uiv = _link_function('glVertexAttribP2uiv', None, [GLuint, GLenum, GLboolean, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7357
glVertexAttribP3ui = _link_function('glVertexAttribP3ui', None, [GLuint, GLenum, GLboolean, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7358
glVertexAttribP3uiv = _link_function('glVertexAttribP3uiv', None, [GLuint, GLenum, GLboolean, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7359
glVertexAttribP4ui = _link_function('glVertexAttribP4ui', None, [GLuint, GLenum, GLboolean, GLuint], 'ARB_vertex_type_2_10_10_10_rev')

# GL/glext.h:7360
glVertexAttribP4uiv = _link_function('glVertexAttribP4uiv', None, [GLuint, GLenum, GLboolean, POINTER(GLuint)], 'ARB_vertex_type_2_10_10_10_rev')

PFNGLVERTEXP2UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7362
PFNGLVERTEXP2UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7363
PFNGLVERTEXP3UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7364
PFNGLVERTEXP3UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7365
PFNGLVERTEXP4UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7366
PFNGLVERTEXP4UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7367
PFNGLTEXCOORDP1UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7368
PFNGLTEXCOORDP1UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7369
PFNGLTEXCOORDP2UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7370
PFNGLTEXCOORDP2UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7371
PFNGLTEXCOORDP3UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7372
PFNGLTEXCOORDP3UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7373
PFNGLTEXCOORDP4UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7374
PFNGLTEXCOORDP4UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7375
PFNGLMULTITEXCOORDP1UIPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:7376
PFNGLMULTITEXCOORDP1UIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:7377
PFNGLMULTITEXCOORDP2UIPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:7378
PFNGLMULTITEXCOORDP2UIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:7379
PFNGLMULTITEXCOORDP3UIPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:7380
PFNGLMULTITEXCOORDP3UIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:7381
PFNGLMULTITEXCOORDP4UIPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:7382
PFNGLMULTITEXCOORDP4UIVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:7383
PFNGLNORMALP3UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7384
PFNGLNORMALP3UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7385
PFNGLCOLORP3UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7386
PFNGLCOLORP3UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7387
PFNGLCOLORP4UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7388
PFNGLCOLORP4UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7389
PFNGLSECONDARYCOLORP3UIPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7390
PFNGLSECONDARYCOLORP3UIVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint)) 	# GL/glext.h:7391
PFNGLVERTEXATTRIBP1UIPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, GLuint) 	# GL/glext.h:7392
PFNGLVERTEXATTRIBP1UIVPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, POINTER(GLuint)) 	# GL/glext.h:7393
PFNGLVERTEXATTRIBP2UIPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, GLuint) 	# GL/glext.h:7394
PFNGLVERTEXATTRIBP2UIVPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, POINTER(GLuint)) 	# GL/glext.h:7395
PFNGLVERTEXATTRIBP3UIPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, GLuint) 	# GL/glext.h:7396
PFNGLVERTEXATTRIBP3UIVPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, POINTER(GLuint)) 	# GL/glext.h:7397
PFNGLVERTEXATTRIBP4UIPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, GLuint) 	# GL/glext.h:7398
PFNGLVERTEXATTRIBP4UIVPROC = CFUNCTYPE(None, GLuint, GLenum, GLboolean, POINTER(GLuint)) 	# GL/glext.h:7399
# ARB_draw_indirect (GL/glext.h:7402)
GL_ARB_draw_indirect = 1 	# GL/glext.h:7403
# GL/glext.h:7405
glDrawArraysIndirect = _link_function('glDrawArraysIndirect', None, [GLenum, POINTER(GLvoid)], 'ARB_draw_indirect')

# GL/glext.h:7406
glDrawElementsIndirect = _link_function('glDrawElementsIndirect', None, [GLenum, GLenum, POINTER(GLvoid)], 'ARB_draw_indirect')

PFNGLDRAWARRAYSINDIRECTPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7408
PFNGLDRAWELEMENTSINDIRECTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7409
# ARB_gpu_shader5 (GL/glext.h:7412)
GL_ARB_gpu_shader5 = 1 	# GL/glext.h:7413
# ARB_gpu_shader_fp64 (GL/glext.h:7416)
GL_ARB_gpu_shader_fp64 = 1 	# GL/glext.h:7417
# GL/glext.h:7419
glUniform1d = _link_function('glUniform1d', None, [GLint, GLdouble], 'ARB_gpu_shader_fp64')

# GL/glext.h:7420
glUniform2d = _link_function('glUniform2d', None, [GLint, GLdouble, GLdouble], 'ARB_gpu_shader_fp64')

# GL/glext.h:7421
glUniform3d = _link_function('glUniform3d', None, [GLint, GLdouble, GLdouble, GLdouble], 'ARB_gpu_shader_fp64')

# GL/glext.h:7422
glUniform4d = _link_function('glUniform4d', None, [GLint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_gpu_shader_fp64')

# GL/glext.h:7423
glUniform1dv = _link_function('glUniform1dv', None, [GLint, GLsizei, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7424
glUniform2dv = _link_function('glUniform2dv', None, [GLint, GLsizei, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7425
glUniform3dv = _link_function('glUniform3dv', None, [GLint, GLsizei, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7426
glUniform4dv = _link_function('glUniform4dv', None, [GLint, GLsizei, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7427
glUniformMatrix2dv = _link_function('glUniformMatrix2dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7428
glUniformMatrix3dv = _link_function('glUniformMatrix3dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7429
glUniformMatrix4dv = _link_function('glUniformMatrix4dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7430
glUniformMatrix2x3dv = _link_function('glUniformMatrix2x3dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7431
glUniformMatrix2x4dv = _link_function('glUniformMatrix2x4dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7432
glUniformMatrix3x2dv = _link_function('glUniformMatrix3x2dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7433
glUniformMatrix3x4dv = _link_function('glUniformMatrix3x4dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7434
glUniformMatrix4x2dv = _link_function('glUniformMatrix4x2dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7435
glUniformMatrix4x3dv = _link_function('glUniformMatrix4x3dv', None, [GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

# GL/glext.h:7436
glGetUniformdv = _link_function('glGetUniformdv', None, [GLuint, GLint, POINTER(GLdouble)], 'ARB_gpu_shader_fp64')

PFNGLUNIFORM1DPROC = CFUNCTYPE(None, GLint, GLdouble) 	# GL/glext.h:7438
PFNGLUNIFORM2DPROC = CFUNCTYPE(None, GLint, GLdouble, GLdouble) 	# GL/glext.h:7439
PFNGLUNIFORM3DPROC = CFUNCTYPE(None, GLint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7440
PFNGLUNIFORM4DPROC = CFUNCTYPE(None, GLint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7441
PFNGLUNIFORM1DVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7442
PFNGLUNIFORM2DVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7443
PFNGLUNIFORM3DVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7444
PFNGLUNIFORM4DVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7445
PFNGLUNIFORMMATRIX2DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7446
PFNGLUNIFORMMATRIX3DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7447
PFNGLUNIFORMMATRIX4DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7448
PFNGLUNIFORMMATRIX2X3DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7449
PFNGLUNIFORMMATRIX2X4DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7450
PFNGLUNIFORMMATRIX3X2DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7451
PFNGLUNIFORMMATRIX3X4DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7452
PFNGLUNIFORMMATRIX4X2DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7453
PFNGLUNIFORMMATRIX4X3DVPROC = CFUNCTYPE(None, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7454
PFNGLGETUNIFORMDVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLdouble)) 	# GL/glext.h:7455
# ARB_shader_subroutine (GL/glext.h:7458)
GL_ARB_shader_subroutine = 1 	# GL/glext.h:7459
# GL/glext.h:7461
glGetSubroutineUniformLocation = _link_function('glGetSubroutineUniformLocation', GLint, [GLuint, GLenum, POINTER(GLchar)], 'ARB_shader_subroutine')

# GL/glext.h:7462
glGetSubroutineIndex = _link_function('glGetSubroutineIndex', GLuint, [GLuint, GLenum, POINTER(GLchar)], 'ARB_shader_subroutine')

# GL/glext.h:7463
glGetActiveSubroutineUniformiv = _link_function('glGetActiveSubroutineUniformiv', None, [GLuint, GLenum, GLuint, GLenum, POINTER(GLint)], 'ARB_shader_subroutine')

# GL/glext.h:7464
glGetActiveSubroutineUniformName = _link_function('glGetActiveSubroutineUniformName', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'ARB_shader_subroutine')

# GL/glext.h:7465
glGetActiveSubroutineName = _link_function('glGetActiveSubroutineName', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'ARB_shader_subroutine')

# GL/glext.h:7466
glUniformSubroutinesuiv = _link_function('glUniformSubroutinesuiv', None, [GLenum, GLsizei, POINTER(GLuint)], 'ARB_shader_subroutine')

# GL/glext.h:7467
glGetUniformSubroutineuiv = _link_function('glGetUniformSubroutineuiv', None, [GLenum, GLint, POINTER(GLuint)], 'ARB_shader_subroutine')

# GL/glext.h:7468
glGetProgramStageiv = _link_function('glGetProgramStageiv', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'ARB_shader_subroutine')

PFNGLGETSUBROUTINEUNIFORMLOCATIONPROC = CFUNCTYPE(GLint, GLuint, GLenum, POINTER(GLchar)) 	# GL/glext.h:7470
PFNGLGETSUBROUTINEINDEXPROC = CFUNCTYPE(GLuint, GLuint, GLenum, POINTER(GLchar)) 	# GL/glext.h:7471
PFNGLGETACTIVESUBROUTINEUNIFORMIVPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7472
PFNGLGETACTIVESUBROUTINEUNIFORMNAMEPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:7473
PFNGLGETACTIVESUBROUTINENAMEPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:7474
PFNGLUNIFORMSUBROUTINESUIVPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7475
PFNGLGETUNIFORMSUBROUTINEUIVPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(GLuint)) 	# GL/glext.h:7476
PFNGLGETPROGRAMSTAGEIVPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7477
# ARB_tessellation_shader (GL/glext.h:7480)
GL_ARB_tessellation_shader = 1 	# GL/glext.h:7481
# GL/glext.h:7483
glPatchParameteri = _link_function('glPatchParameteri', None, [GLenum, GLint], 'ARB_tessellation_shader')

# GL/glext.h:7484
glPatchParameterfv = _link_function('glPatchParameterfv', None, [GLenum, POINTER(GLfloat)], 'ARB_tessellation_shader')

PFNGLPATCHPARAMETERIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:7486
PFNGLPATCHPARAMETERFVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7487
# ARB_texture_buffer_object_rgb32 (GL/glext.h:7490)
GL_ARB_texture_buffer_object_rgb32 = 1 	# GL/glext.h:7491
# ARB_transform_feedback2 (GL/glext.h:7494)
GL_ARB_transform_feedback2 = 1 	# GL/glext.h:7495
# GL/glext.h:7497
glBindTransformFeedback = _link_function('glBindTransformFeedback', None, [GLenum, GLuint], 'ARB_transform_feedback2')

# GL/glext.h:7498
glDeleteTransformFeedbacks = _link_function('glDeleteTransformFeedbacks', None, [GLsizei, POINTER(GLuint)], 'ARB_transform_feedback2')

# GL/glext.h:7499
glGenTransformFeedbacks = _link_function('glGenTransformFeedbacks', None, [GLsizei, POINTER(GLuint)], 'ARB_transform_feedback2')

# GL/glext.h:7500
glIsTransformFeedback = _link_function('glIsTransformFeedback', GLboolean, [GLuint], 'ARB_transform_feedback2')

# GL/glext.h:7501
glPauseTransformFeedback = _link_function('glPauseTransformFeedback', None, [], 'ARB_transform_feedback2')

# GL/glext.h:7502
glResumeTransformFeedback = _link_function('glResumeTransformFeedback', None, [], 'ARB_transform_feedback2')

# GL/glext.h:7503
glDrawTransformFeedback = _link_function('glDrawTransformFeedback', None, [GLenum, GLuint], 'ARB_transform_feedback2')

PFNGLBINDTRANSFORMFEEDBACKPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7505
PFNGLDELETETRANSFORMFEEDBACKSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7506
PFNGLGENTRANSFORMFEEDBACKSPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7507
PFNGLISTRANSFORMFEEDBACKPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7508
PFNGLPAUSETRANSFORMFEEDBACKPROC = CFUNCTYPE(None) 	# GL/glext.h:7509
PFNGLRESUMETRANSFORMFEEDBACKPROC = CFUNCTYPE(None) 	# GL/glext.h:7510
PFNGLDRAWTRANSFORMFEEDBACKPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7511
# ARB_transform_feedback3 (GL/glext.h:7514)
GL_ARB_transform_feedback3 = 1 	# GL/glext.h:7515
# GL/glext.h:7517
glDrawTransformFeedbackStream = _link_function('glDrawTransformFeedbackStream', None, [GLenum, GLuint, GLuint], 'ARB_transform_feedback3')

# GL/glext.h:7518
glBeginQueryIndexed = _link_function('glBeginQueryIndexed', None, [GLenum, GLuint, GLuint], 'ARB_transform_feedback3')

# GL/glext.h:7519
glEndQueryIndexed = _link_function('glEndQueryIndexed', None, [GLenum, GLuint], 'ARB_transform_feedback3')

# GL/glext.h:7520
glGetQueryIndexediv = _link_function('glGetQueryIndexediv', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'ARB_transform_feedback3')

PFNGLDRAWTRANSFORMFEEDBACKSTREAMPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:7522
PFNGLBEGINQUERYINDEXEDPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:7523
PFNGLENDQUERYINDEXEDPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:7524
PFNGLGETQUERYINDEXEDIVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7525
# ARB_ES2_compatibility (GL/glext.h:7528)
GL_ARB_ES2_compatibility = 1 	# GL/glext.h:7529
# GL/glext.h:7531
glReleaseShaderCompiler = _link_function('glReleaseShaderCompiler', None, [], 'ARB_ES2_compatibility')

# GL/glext.h:7532
glShaderBinary = _link_function('glShaderBinary', None, [GLsizei, POINTER(GLuint), GLenum, POINTER(GLvoid), GLsizei], 'ARB_ES2_compatibility')

# GL/glext.h:7533
glGetShaderPrecisionFormat = _link_function('glGetShaderPrecisionFormat', None, [GLenum, GLenum, POINTER(GLint), POINTER(GLint)], 'ARB_ES2_compatibility')

# GL/glext.h:7534
glDepthRangef = _link_function('glDepthRangef', None, [GLfloat, GLfloat], 'ARB_ES2_compatibility')

# GL/glext.h:7535
glClearDepthf = _link_function('glClearDepthf', None, [GLfloat], 'ARB_ES2_compatibility')

PFNGLRELEASESHADERCOMPILERPROC = CFUNCTYPE(None) 	# GL/glext.h:7537
PFNGLSHADERBINARYPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint), GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:7538
PFNGLGETSHADERPRECISIONFORMATPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint), POINTER(GLint)) 	# GL/glext.h:7539
PFNGLDEPTHRANGEFPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:7540
PFNGLCLEARDEPTHFPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:7541
# ARB_get_program_binary (GL/glext.h:7544)
GL_ARB_get_program_binary = 1 	# GL/glext.h:7545
# GL/glext.h:7547
glGetProgramBinary = _link_function('glGetProgramBinary', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLenum), POINTER(GLvoid)], 'ARB_get_program_binary')

# GL/glext.h:7548
glProgramBinary = _link_function('glProgramBinary', None, [GLuint, GLenum, POINTER(GLvoid), GLsizei], 'ARB_get_program_binary')

# GL/glext.h:7549
glProgramParameteri = _link_function('glProgramParameteri', None, [GLuint, GLenum, GLint], 'ARB_get_program_binary')

PFNGLGETPROGRAMBINARYPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLenum), POINTER(GLvoid)) 	# GL/glext.h:7551
PFNGLPROGRAMBINARYPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:7552
PFNGLPROGRAMPARAMETERIPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:7553
# ARB_separate_shader_objects (GL/glext.h:7556)
GL_ARB_separate_shader_objects = 1 	# GL/glext.h:7557
# GL/glext.h:7559
glUseProgramStages = _link_function('glUseProgramStages', None, [GLuint, GLbitfield, GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7560
glActiveShaderProgram = _link_function('glActiveShaderProgram', None, [GLuint, GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7561
glCreateShaderProgramv = _link_function('glCreateShaderProgramv', GLuint, [GLenum, GLsizei, POINTER(POINTER(GLchar))], 'ARB_separate_shader_objects')

# GL/glext.h:7562
glBindProgramPipeline = _link_function('glBindProgramPipeline', None, [GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7563
glDeleteProgramPipelines = _link_function('glDeleteProgramPipelines', None, [GLsizei, POINTER(GLuint)], 'ARB_separate_shader_objects')

# GL/glext.h:7564
glGenProgramPipelines = _link_function('glGenProgramPipelines', None, [GLsizei, POINTER(GLuint)], 'ARB_separate_shader_objects')

# GL/glext.h:7565
glIsProgramPipeline = _link_function('glIsProgramPipeline', GLboolean, [GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7566
glGetProgramPipelineiv = _link_function('glGetProgramPipelineiv', None, [GLuint, GLenum, POINTER(GLint)], 'ARB_separate_shader_objects')

# GL/glext.h:7567
glProgramUniform1i = _link_function('glProgramUniform1i', None, [GLuint, GLint, GLint], 'ARB_separate_shader_objects')

# GL/glext.h:7568
glProgramUniform1iv = _link_function('glProgramUniform1iv', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'ARB_separate_shader_objects')

# GL/glext.h:7569
glProgramUniform1f = _link_function('glProgramUniform1f', None, [GLuint, GLint, GLfloat], 'ARB_separate_shader_objects')

# GL/glext.h:7570
glProgramUniform1fv = _link_function('glProgramUniform1fv', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7571
glProgramUniform1d = _link_function('glProgramUniform1d', None, [GLuint, GLint, GLdouble], 'ARB_separate_shader_objects')

# GL/glext.h:7572
glProgramUniform1dv = _link_function('glProgramUniform1dv', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7573
glProgramUniform1ui = _link_function('glProgramUniform1ui', None, [GLuint, GLint, GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7574
glProgramUniform1uiv = _link_function('glProgramUniform1uiv', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'ARB_separate_shader_objects')

# GL/glext.h:7575
glProgramUniform2i = _link_function('glProgramUniform2i', None, [GLuint, GLint, GLint, GLint], 'ARB_separate_shader_objects')

# GL/glext.h:7576
glProgramUniform2iv = _link_function('glProgramUniform2iv', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'ARB_separate_shader_objects')

# GL/glext.h:7577
glProgramUniform2f = _link_function('glProgramUniform2f', None, [GLuint, GLint, GLfloat, GLfloat], 'ARB_separate_shader_objects')

# GL/glext.h:7578
glProgramUniform2fv = _link_function('glProgramUniform2fv', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7579
glProgramUniform2d = _link_function('glProgramUniform2d', None, [GLuint, GLint, GLdouble, GLdouble], 'ARB_separate_shader_objects')

# GL/glext.h:7580
glProgramUniform2dv = _link_function('glProgramUniform2dv', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7581
glProgramUniform2ui = _link_function('glProgramUniform2ui', None, [GLuint, GLint, GLuint, GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7582
glProgramUniform2uiv = _link_function('glProgramUniform2uiv', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'ARB_separate_shader_objects')

# GL/glext.h:7583
glProgramUniform3i = _link_function('glProgramUniform3i', None, [GLuint, GLint, GLint, GLint, GLint], 'ARB_separate_shader_objects')

# GL/glext.h:7584
glProgramUniform3iv = _link_function('glProgramUniform3iv', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'ARB_separate_shader_objects')

# GL/glext.h:7585
glProgramUniform3f = _link_function('glProgramUniform3f', None, [GLuint, GLint, GLfloat, GLfloat, GLfloat], 'ARB_separate_shader_objects')

# GL/glext.h:7586
glProgramUniform3fv = _link_function('glProgramUniform3fv', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7587
glProgramUniform3d = _link_function('glProgramUniform3d', None, [GLuint, GLint, GLdouble, GLdouble, GLdouble], 'ARB_separate_shader_objects')

# GL/glext.h:7588
glProgramUniform3dv = _link_function('glProgramUniform3dv', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7589
glProgramUniform3ui = _link_function('glProgramUniform3ui', None, [GLuint, GLint, GLuint, GLuint, GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7590
glProgramUniform3uiv = _link_function('glProgramUniform3uiv', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'ARB_separate_shader_objects')

# GL/glext.h:7591
glProgramUniform4i = _link_function('glProgramUniform4i', None, [GLuint, GLint, GLint, GLint, GLint, GLint], 'ARB_separate_shader_objects')

# GL/glext.h:7592
glProgramUniform4iv = _link_function('glProgramUniform4iv', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'ARB_separate_shader_objects')

# GL/glext.h:7593
glProgramUniform4f = _link_function('glProgramUniform4f', None, [GLuint, GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_separate_shader_objects')

# GL/glext.h:7594
glProgramUniform4fv = _link_function('glProgramUniform4fv', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7595
glProgramUniform4d = _link_function('glProgramUniform4d', None, [GLuint, GLint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_separate_shader_objects')

# GL/glext.h:7596
glProgramUniform4dv = _link_function('glProgramUniform4dv', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7597
glProgramUniform4ui = _link_function('glProgramUniform4ui', None, [GLuint, GLint, GLuint, GLuint, GLuint, GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7598
glProgramUniform4uiv = _link_function('glProgramUniform4uiv', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'ARB_separate_shader_objects')

# GL/glext.h:7599
glProgramUniformMatrix2fv = _link_function('glProgramUniformMatrix2fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7600
glProgramUniformMatrix3fv = _link_function('glProgramUniformMatrix3fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7601
glProgramUniformMatrix4fv = _link_function('glProgramUniformMatrix4fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7602
glProgramUniformMatrix2dv = _link_function('glProgramUniformMatrix2dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7603
glProgramUniformMatrix3dv = _link_function('glProgramUniformMatrix3dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7604
glProgramUniformMatrix4dv = _link_function('glProgramUniformMatrix4dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7605
glProgramUniformMatrix2x3fv = _link_function('glProgramUniformMatrix2x3fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7606
glProgramUniformMatrix3x2fv = _link_function('glProgramUniformMatrix3x2fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7607
glProgramUniformMatrix2x4fv = _link_function('glProgramUniformMatrix2x4fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7608
glProgramUniformMatrix4x2fv = _link_function('glProgramUniformMatrix4x2fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7609
glProgramUniformMatrix3x4fv = _link_function('glProgramUniformMatrix3x4fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7610
glProgramUniformMatrix4x3fv = _link_function('glProgramUniformMatrix4x3fv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'ARB_separate_shader_objects')

# GL/glext.h:7611
glProgramUniformMatrix2x3dv = _link_function('glProgramUniformMatrix2x3dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7612
glProgramUniformMatrix3x2dv = _link_function('glProgramUniformMatrix3x2dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7613
glProgramUniformMatrix2x4dv = _link_function('glProgramUniformMatrix2x4dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7614
glProgramUniformMatrix4x2dv = _link_function('glProgramUniformMatrix4x2dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7615
glProgramUniformMatrix3x4dv = _link_function('glProgramUniformMatrix3x4dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7616
glProgramUniformMatrix4x3dv = _link_function('glProgramUniformMatrix4x3dv', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'ARB_separate_shader_objects')

# GL/glext.h:7617
glValidateProgramPipeline = _link_function('glValidateProgramPipeline', None, [GLuint], 'ARB_separate_shader_objects')

# GL/glext.h:7618
glGetProgramPipelineInfoLog = _link_function('glGetProgramPipelineInfoLog', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'ARB_separate_shader_objects')

PFNGLUSEPROGRAMSTAGESPROC = CFUNCTYPE(None, GLuint, GLbitfield, GLuint) 	# GL/glext.h:7620
PFNGLACTIVESHADERPROGRAMPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:7621
PFNGLCREATESHADERPROGRAMVPROC = CFUNCTYPE(GLuint, GLenum, GLsizei, POINTER(POINTER(GLchar))) 	# GL/glext.h:7622
PFNGLBINDPROGRAMPIPELINEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7623
PFNGLDELETEPROGRAMPIPELINESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7624
PFNGLGENPROGRAMPIPELINESPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7625
PFNGLISPROGRAMPIPELINEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:7626
PFNGLGETPROGRAMPIPELINEIVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7627
PFNGLPROGRAMUNIFORM1IPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:7628
PFNGLPROGRAMUNIFORM1IVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7629
PFNGLPROGRAMUNIFORM1FPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat) 	# GL/glext.h:7630
PFNGLPROGRAMUNIFORM1FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7631
PFNGLPROGRAMUNIFORM1DPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble) 	# GL/glext.h:7632
PFNGLPROGRAMUNIFORM1DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7633
PFNGLPROGRAMUNIFORM1UIPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:7634
PFNGLPROGRAMUNIFORM1UIVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7635
PFNGLPROGRAMUNIFORM2IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:7636
PFNGLPROGRAMUNIFORM2IVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7637
PFNGLPROGRAMUNIFORM2FPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat) 	# GL/glext.h:7638
PFNGLPROGRAMUNIFORM2FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7639
PFNGLPROGRAMUNIFORM2DPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble, GLdouble) 	# GL/glext.h:7640
PFNGLPROGRAMUNIFORM2DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7641
PFNGLPROGRAMUNIFORM2UIPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint) 	# GL/glext.h:7642
PFNGLPROGRAMUNIFORM2UIVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7643
PFNGLPROGRAMUNIFORM3IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7644
PFNGLPROGRAMUNIFORM3IVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7645
PFNGLPROGRAMUNIFORM3FPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7646
PFNGLPROGRAMUNIFORM3FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7647
PFNGLPROGRAMUNIFORM3DPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7648
PFNGLPROGRAMUNIFORM3DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7649
PFNGLPROGRAMUNIFORM3UIPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:7650
PFNGLPROGRAMUNIFORM3UIVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7651
PFNGLPROGRAMUNIFORM4IPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:7652
PFNGLPROGRAMUNIFORM4IVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7653
PFNGLPROGRAMUNIFORM4FPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7654
PFNGLPROGRAMUNIFORM4FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7655
PFNGLPROGRAMUNIFORM4DPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7656
PFNGLPROGRAMUNIFORM4DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7657
PFNGLPROGRAMUNIFORM4UIPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:7658
PFNGLPROGRAMUNIFORM4UIVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7659
PFNGLPROGRAMUNIFORMMATRIX2FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7660
PFNGLPROGRAMUNIFORMMATRIX3FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7661
PFNGLPROGRAMUNIFORMMATRIX4FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7662
PFNGLPROGRAMUNIFORMMATRIX2DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7663
PFNGLPROGRAMUNIFORMMATRIX3DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7664
PFNGLPROGRAMUNIFORMMATRIX4DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7665
PFNGLPROGRAMUNIFORMMATRIX2X3FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7666
PFNGLPROGRAMUNIFORMMATRIX3X2FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7667
PFNGLPROGRAMUNIFORMMATRIX2X4FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7668
PFNGLPROGRAMUNIFORMMATRIX4X2FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7669
PFNGLPROGRAMUNIFORMMATRIX3X4FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7670
PFNGLPROGRAMUNIFORMMATRIX4X3FVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:7671
PFNGLPROGRAMUNIFORMMATRIX2X3DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7672
PFNGLPROGRAMUNIFORMMATRIX3X2DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7673
PFNGLPROGRAMUNIFORMMATRIX2X4DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7674
PFNGLPROGRAMUNIFORMMATRIX4X2DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7675
PFNGLPROGRAMUNIFORMMATRIX3X4DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7676
PFNGLPROGRAMUNIFORMMATRIX4X3DVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:7677
PFNGLVALIDATEPROGRAMPIPELINEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:7678
PFNGLGETPROGRAMPIPELINEINFOLOGPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:7679
# ARB_vertex_attrib_64bit (GL/glext.h:7682)
GL_ARB_vertex_attrib_64bit = 1 	# GL/glext.h:7683
# GL/glext.h:7685
glVertexAttribL1d = _link_function('glVertexAttribL1d', None, [GLuint, GLdouble], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7686
glVertexAttribL2d = _link_function('glVertexAttribL2d', None, [GLuint, GLdouble, GLdouble], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7687
glVertexAttribL3d = _link_function('glVertexAttribL3d', None, [GLuint, GLdouble, GLdouble, GLdouble], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7688
glVertexAttribL4d = _link_function('glVertexAttribL4d', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7689
glVertexAttribL1dv = _link_function('glVertexAttribL1dv', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7690
glVertexAttribL2dv = _link_function('glVertexAttribL2dv', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7691
glVertexAttribL3dv = _link_function('glVertexAttribL3dv', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7692
glVertexAttribL4dv = _link_function('glVertexAttribL4dv', None, [GLuint, POINTER(GLdouble)], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7693
glVertexAttribLPointer = _link_function('glVertexAttribLPointer', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_vertex_attrib_64bit')

# GL/glext.h:7694
glGetVertexAttribLdv = _link_function('glGetVertexAttribLdv', None, [GLuint, GLenum, POINTER(GLdouble)], 'ARB_vertex_attrib_64bit')

PFNGLVERTEXATTRIBL1DPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:7696
PFNGLVERTEXATTRIBL2DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:7697
PFNGLVERTEXATTRIBL3DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7698
PFNGLVERTEXATTRIBL4DPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:7699
PFNGLVERTEXATTRIBL1DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:7700
PFNGLVERTEXATTRIBL2DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:7701
PFNGLVERTEXATTRIBL3DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:7702
PFNGLVERTEXATTRIBL4DVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:7703
PFNGLVERTEXATTRIBLPOINTERPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7704
PFNGLGETVERTEXATTRIBLDVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:7705
# ARB_viewport_array (GL/glext.h:7708)
GL_ARB_viewport_array = 1 	# GL/glext.h:7709
# GL/glext.h:7711
glViewportArrayv = _link_function('glViewportArrayv', None, [GLuint, GLsizei, POINTER(GLfloat)], 'ARB_viewport_array')

# GL/glext.h:7712
glViewportIndexedf = _link_function('glViewportIndexedf', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'ARB_viewport_array')

# GL/glext.h:7713
glViewportIndexedfv = _link_function('glViewportIndexedfv', None, [GLuint, POINTER(GLfloat)], 'ARB_viewport_array')

# GL/glext.h:7714
glScissorArrayv = _link_function('glScissorArrayv', None, [GLuint, GLsizei, POINTER(GLint)], 'ARB_viewport_array')

# GL/glext.h:7715
glScissorIndexed = _link_function('glScissorIndexed', None, [GLuint, GLint, GLint, GLsizei, GLsizei], 'ARB_viewport_array')

# GL/glext.h:7716
glScissorIndexedv = _link_function('glScissorIndexedv', None, [GLuint, POINTER(GLint)], 'ARB_viewport_array')

# GL/glext.h:7717
glDepthRangeArrayv = _link_function('glDepthRangeArrayv', None, [GLuint, GLsizei, POINTER(GLdouble)], 'ARB_viewport_array')

# GL/glext.h:7718
glDepthRangeIndexed = _link_function('glDepthRangeIndexed', None, [GLuint, GLdouble, GLdouble], 'ARB_viewport_array')

# GL/glext.h:7719
glGetFloati_v = _link_function('glGetFloati_v', None, [GLenum, GLuint, POINTER(GLfloat)], 'ARB_viewport_array')

# GL/glext.h:7720
glGetDoublei_v = _link_function('glGetDoublei_v', None, [GLenum, GLuint, POINTER(GLdouble)], 'ARB_viewport_array')

PFNGLVIEWPORTARRAYVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7722
PFNGLVIEWPORTINDEXEDFPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7723
PFNGLVIEWPORTINDEXEDFVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:7724
PFNGLSCISSORARRAYVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7725
PFNGLSCISSORINDEXEDPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:7726
PFNGLSCISSORINDEXEDVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:7727
PFNGLDEPTHRANGEARRAYVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7728
PFNGLDEPTHRANGEINDEXEDPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:7729
PFNGLGETFLOATI_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:7730
PFNGLGETDOUBLEI_VPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:7731
# ARB_cl_event (GL/glext.h:7734)
GL_ARB_cl_event = 1 	# GL/glext.h:7735
class struct__cl_context(Structure):
    __slots__ = [
    ]
struct__cl_context._fields_ = [
    ('_opaque_struct', c_int)
]

class struct__cl_event(Structure):
    __slots__ = [
    ]
struct__cl_event._fields_ = [
    ('_opaque_struct', c_int)
]

# GL/glext.h:7737
glCreateSyncFromCLeventARB = _link_function('glCreateSyncFromCLeventARB', GLsync, [POINTER(struct__cl_context), POINTER(struct__cl_event), GLbitfield], 'ARB_cl_event')

class struct__cl_context(Structure):
    __slots__ = [
    ]
struct__cl_context._fields_ = [
    ('_opaque_struct', c_int)
]

class struct__cl_event(Structure):
    __slots__ = [
    ]
struct__cl_event._fields_ = [
    ('_opaque_struct', c_int)
]

class struct__cl_context(Structure):
    __slots__ = [
    ]
struct__cl_context._fields_ = [
    ('_opaque_struct', c_int)
]

class struct__cl_event(Structure):
    __slots__ = [
    ]
struct__cl_event._fields_ = [
    ('_opaque_struct', c_int)
]

PFNGLCREATESYNCFROMCLEVENTARBPROC = CFUNCTYPE(GLsync, POINTER(struct__cl_context), POINTER(struct__cl_event), GLbitfield) 	# GL/glext.h:7739
# ARB_debug_output (GL/glext.h:7742)
GL_ARB_debug_output = 1 	# GL/glext.h:7743
# GL/glext.h:7745
glDebugMessageControlARB = _link_function('glDebugMessageControlARB', None, [GLenum, GLenum, GLenum, GLsizei, POINTER(GLuint), GLboolean], 'ARB_debug_output')

# GL/glext.h:7746
glDebugMessageInsertARB = _link_function('glDebugMessageInsertARB', None, [GLenum, GLenum, GLuint, GLenum, GLsizei, POINTER(GLchar)], 'ARB_debug_output')

# GL/glext.h:7747
glDebugMessageCallbackARB = _link_function('glDebugMessageCallbackARB', None, [GLDEBUGPROCARB, POINTER(GLvoid)], 'ARB_debug_output')

# GL/glext.h:7748
glGetDebugMessageLogARB = _link_function('glGetDebugMessageLogARB', GLuint, [GLuint, GLsizei, POINTER(GLenum), POINTER(GLenum), POINTER(GLuint), POINTER(GLenum), POINTER(GLsizei), POINTER(GLchar)], 'ARB_debug_output')

PFNGLDEBUGMESSAGECONTROLARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLsizei, POINTER(GLuint), GLboolean) 	# GL/glext.h:7750
PFNGLDEBUGMESSAGEINSERTARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLenum, GLsizei, POINTER(GLchar)) 	# GL/glext.h:7751
PFNGLDEBUGMESSAGECALLBACKARBPROC = CFUNCTYPE(None, GLDEBUGPROCARB, POINTER(GLvoid)) 	# GL/glext.h:7752
PFNGLGETDEBUGMESSAGELOGARBPROC = CFUNCTYPE(GLuint, GLuint, GLsizei, POINTER(GLenum), POINTER(GLenum), POINTER(GLuint), POINTER(GLenum), POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:7753
# ARB_robustness (GL/glext.h:7756)
GL_ARB_robustness = 1 	# GL/glext.h:7757
# GL/glext.h:7759
glGetGraphicsResetStatusARB = _link_function('glGetGraphicsResetStatusARB', GLenum, [], 'ARB_robustness')

# GL/glext.h:7760
glGetnMapdvARB = _link_function('glGetnMapdvARB', None, [GLenum, GLenum, GLsizei, POINTER(GLdouble)], 'ARB_robustness')

# GL/glext.h:7761
glGetnMapfvARB = _link_function('glGetnMapfvARB', None, [GLenum, GLenum, GLsizei, POINTER(GLfloat)], 'ARB_robustness')

# GL/glext.h:7762
glGetnMapivARB = _link_function('glGetnMapivARB', None, [GLenum, GLenum, GLsizei, POINTER(GLint)], 'ARB_robustness')

# GL/glext.h:7763
glGetnPixelMapfvARB = _link_function('glGetnPixelMapfvARB', None, [GLenum, GLsizei, POINTER(GLfloat)], 'ARB_robustness')

# GL/glext.h:7764
glGetnPixelMapuivARB = _link_function('glGetnPixelMapuivARB', None, [GLenum, GLsizei, POINTER(GLuint)], 'ARB_robustness')

# GL/glext.h:7765
glGetnPixelMapusvARB = _link_function('glGetnPixelMapusvARB', None, [GLenum, GLsizei, POINTER(GLushort)], 'ARB_robustness')

# GL/glext.h:7766
glGetnPolygonStippleARB = _link_function('glGetnPolygonStippleARB', None, [GLsizei, POINTER(GLubyte)], 'ARB_robustness')

# GL/glext.h:7767
glGetnColorTableARB = _link_function('glGetnColorTableARB', None, [GLenum, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7768
glGetnConvolutionFilterARB = _link_function('glGetnConvolutionFilterARB', None, [GLenum, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7769
glGetnSeparableFilterARB = _link_function('glGetnSeparableFilterARB', None, [GLenum, GLenum, GLenum, GLsizei, POINTER(GLvoid), GLsizei, POINTER(GLvoid), POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7770
glGetnHistogramARB = _link_function('glGetnHistogramARB', None, [GLenum, GLboolean, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7771
glGetnMinmaxARB = _link_function('glGetnMinmaxARB', None, [GLenum, GLboolean, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7772
glGetnTexImageARB = _link_function('glGetnTexImageARB', None, [GLenum, GLint, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7773
glReadnPixelsARB = _link_function('glReadnPixelsARB', None, [GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7774
glGetnCompressedTexImageARB = _link_function('glGetnCompressedTexImageARB', None, [GLenum, GLint, GLsizei, POINTER(GLvoid)], 'ARB_robustness')

# GL/glext.h:7775
glGetnUniformfvARB = _link_function('glGetnUniformfvARB', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'ARB_robustness')

# GL/glext.h:7776
glGetnUniformivARB = _link_function('glGetnUniformivARB', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'ARB_robustness')

# GL/glext.h:7777
glGetnUniformuivARB = _link_function('glGetnUniformuivARB', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'ARB_robustness')

# GL/glext.h:7778
glGetnUniformdvARB = _link_function('glGetnUniformdvARB', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'ARB_robustness')

PFNGLGETGRAPHICSRESETSTATUSARBPROC = CFUNCTYPE(GLenum) 	# GL/glext.h:7780
PFNGLGETNMAPDVARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7781
PFNGLGETNMAPFVARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7782
PFNGLGETNMAPIVARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLint)) 	# GL/glext.h:7783
PFNGLGETNPIXELMAPFVARBPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7784
PFNGLGETNPIXELMAPUIVARBPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7785
PFNGLGETNPIXELMAPUSVARBPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLushort)) 	# GL/glext.h:7786
PFNGLGETNPOLYGONSTIPPLEARBPROC = CFUNCTYPE(None, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:7787
PFNGLGETNCOLORTABLEARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7788
PFNGLGETNCONVOLUTIONFILTERARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7789
PFNGLGETNSEPARABLEFILTERARBPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLsizei, POINTER(GLvoid), GLsizei, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:7790
PFNGLGETNHISTOGRAMARBPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7791
PFNGLGETNMINMAXARBPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7792
PFNGLGETNTEXIMAGEARBPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7793
PFNGLREADNPIXELSARBPROC = CFUNCTYPE(None, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7794
PFNGLGETNCOMPRESSEDTEXIMAGEARBPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:7795
PFNGLGETNUNIFORMFVARBPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7796
PFNGLGETNUNIFORMIVARBPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:7797
PFNGLGETNUNIFORMUIVARBPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:7798
PFNGLGETNUNIFORMDVARBPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:7799
# ARB_shader_stencil_export (GL/glext.h:7802)
GL_ARB_shader_stencil_export = 1 	# GL/glext.h:7803
# ARB_base_instance (GL/glext.h:7806)
GL_ARB_base_instance = 1 	# GL/glext.h:7807
# GL/glext.h:7809
glDrawArraysInstancedBaseInstance = _link_function('glDrawArraysInstancedBaseInstance', None, [GLenum, GLint, GLsizei, GLsizei, GLuint], 'ARB_base_instance')

# GL/glext.h:7810
glDrawElementsInstancedBaseInstance = _link_function('glDrawElementsInstancedBaseInstance', None, [GLenum, GLsizei, GLenum, POINTER(None), GLsizei, GLuint], 'ARB_base_instance')

# GL/glext.h:7811
glDrawElementsInstancedBaseVertexBaseInstance = _link_function('glDrawElementsInstancedBaseVertexBaseInstance', None, [GLenum, GLsizei, GLenum, POINTER(None), GLsizei, GLint, GLuint], 'ARB_base_instance')

PFNGLDRAWARRAYSINSTANCEDBASEINSTANCEPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei, GLuint) 	# GL/glext.h:7813
PFNGLDRAWELEMENTSINSTANCEDBASEINSTANCEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(None), GLsizei, GLuint) 	# GL/glext.h:7814
PFNGLDRAWELEMENTSINSTANCEDBASEVERTEXBASEINSTANCEPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(None), GLsizei, GLint, GLuint) 	# GL/glext.h:7815
# ARB_shading_language_420pack (GL/glext.h:7818)
GL_ARB_shading_language_420pack = 1 	# GL/glext.h:7819
# ARB_transform_feedback_instanced (GL/glext.h:7822)
GL_ARB_transform_feedback_instanced = 1 	# GL/glext.h:7823
# GL/glext.h:7825
glDrawTransformFeedbackInstanced = _link_function('glDrawTransformFeedbackInstanced', None, [GLenum, GLuint, GLsizei], 'ARB_transform_feedback_instanced')

# GL/glext.h:7826
glDrawTransformFeedbackStreamInstanced = _link_function('glDrawTransformFeedbackStreamInstanced', None, [GLenum, GLuint, GLuint, GLsizei], 'ARB_transform_feedback_instanced')

PFNGLDRAWTRANSFORMFEEDBACKINSTANCEDPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei) 	# GL/glext.h:7828
PFNGLDRAWTRANSFORMFEEDBACKSTREAMINSTANCEDPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei) 	# GL/glext.h:7829
# ARB_compressed_texture_pixel_storage (GL/glext.h:7832)
GL_ARB_compressed_texture_pixel_storage = 1 	# GL/glext.h:7833
# ARB_conservative_depth (GL/glext.h:7836)
GL_ARB_conservative_depth = 1 	# GL/glext.h:7837
# ARB_internalformat_query (GL/glext.h:7840)
GL_ARB_internalformat_query = 1 	# GL/glext.h:7841
# GL/glext.h:7843
glGetInternalformativ = _link_function('glGetInternalformativ', None, [GLenum, GLenum, GLenum, GLsizei, POINTER(GLint)], 'ARB_internalformat_query')

PFNGLGETINTERNALFORMATIVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLsizei, POINTER(GLint)) 	# GL/glext.h:7845
# ARB_map_buffer_alignment (GL/glext.h:7848)
GL_ARB_map_buffer_alignment = 1 	# GL/glext.h:7849
# ARB_shader_atomic_counters (GL/glext.h:7852)
GL_ARB_shader_atomic_counters = 1 	# GL/glext.h:7853
# GL/glext.h:7855
glGetActiveAtomicCounterBufferiv = _link_function('glGetActiveAtomicCounterBufferiv', None, [GLuint, GLuint, GLenum, POINTER(GLint)], 'ARB_shader_atomic_counters')

PFNGLGETACTIVEATOMICCOUNTERBUFFERIVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:7857
# ARB_shader_image_load_store (GL/glext.h:7860)
GL_ARB_shader_image_load_store = 1 	# GL/glext.h:7861
# GL/glext.h:7863
glBindImageTexture = _link_function('glBindImageTexture', None, [GLuint, GLuint, GLint, GLboolean, GLint, GLenum, GLenum], 'ARB_shader_image_load_store')

# GL/glext.h:7864
glMemoryBarrier = _link_function('glMemoryBarrier', None, [GLbitfield], 'ARB_shader_image_load_store')

PFNGLBINDIMAGETEXTUREPROC = CFUNCTYPE(None, GLuint, GLuint, GLint, GLboolean, GLint, GLenum, GLenum) 	# GL/glext.h:7866
PFNGLMEMORYBARRIERPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:7867
# ARB_shading_language_packing (GL/glext.h:7870)
GL_ARB_shading_language_packing = 1 	# GL/glext.h:7871
# ARB_texture_storage (GL/glext.h:7874)
GL_ARB_texture_storage = 1 	# GL/glext.h:7875
# GL/glext.h:7877
glTexStorage1D = _link_function('glTexStorage1D', None, [GLenum, GLsizei, GLenum, GLsizei], 'ARB_texture_storage')

# GL/glext.h:7878
glTexStorage2D = _link_function('glTexStorage2D', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'ARB_texture_storage')

# GL/glext.h:7879
glTexStorage3D = _link_function('glTexStorage3D', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei, GLsizei], 'ARB_texture_storage')

# GL/glext.h:7880
glTextureStorage1DEXT = _link_function('glTextureStorage1DEXT', None, [GLuint, GLenum, GLsizei, GLenum, GLsizei], 'ARB_texture_storage')

# GL/glext.h:7881
glTextureStorage2DEXT = _link_function('glTextureStorage2DEXT', None, [GLuint, GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'ARB_texture_storage')

# GL/glext.h:7882
glTextureStorage3DEXT = _link_function('glTextureStorage3DEXT', None, [GLuint, GLenum, GLsizei, GLenum, GLsizei, GLsizei, GLsizei], 'ARB_texture_storage')

PFNGLTEXSTORAGE1DPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei) 	# GL/glext.h:7884
PFNGLTEXSTORAGE2DPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7885
PFNGLTEXSTORAGE3DPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei, GLsizei) 	# GL/glext.h:7886
PFNGLTEXTURESTORAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLenum, GLsizei) 	# GL/glext.h:7887
PFNGLTEXTURESTORAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:7888
PFNGLTEXTURESTORAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLenum, GLsizei, GLsizei, GLsizei) 	# GL/glext.h:7889
# EXT_abgr (GL/glext.h:7892)
GL_EXT_abgr = 1 	# GL/glext.h:7893
# EXT_blend_color (GL/glext.h:7896)
GL_EXT_blend_color = 1 	# GL/glext.h:7897
# GL/glext.h:7899
glBlendColorEXT = _link_function('glBlendColorEXT', None, [GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_blend_color')

PFNGLBLENDCOLOREXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:7901
# EXT_polygon_offset (GL/glext.h:7904)
GL_EXT_polygon_offset = 1 	# GL/glext.h:7905
# GL/glext.h:7907
glPolygonOffsetEXT = _link_function('glPolygonOffsetEXT', None, [GLfloat, GLfloat], 'EXT_polygon_offset')

PFNGLPOLYGONOFFSETEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:7909
# EXT_texture (GL/glext.h:7912)
GL_EXT_texture = 1 	# GL/glext.h:7913
# EXT_texture3D (GL/glext.h:7916)
GL_EXT_texture3D = 1 	# GL/glext.h:7917
# GL/glext.h:7919
glTexImage3DEXT = _link_function('glTexImage3DEXT', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

# GL/glext.h:7920
glTexSubImage3DEXT = _link_function('glTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_texture3D')

PFNGLTEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7922
PFNGLTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7923
# SGIS_texture_filter4 (GL/glext.h:7926)
GL_SGIS_texture_filter4 = 1 	# GL/glext.h:7927
# GL/glext.h:7929
glGetTexFilterFuncSGIS = _link_function('glGetTexFilterFuncSGIS', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIS_texture_filter4')

# GL/glext.h:7930
glTexFilterFuncSGIS = _link_function('glTexFilterFuncSGIS', None, [GLenum, GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_texture_filter4')

PFNGLGETTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7932
PFNGLTEXFILTERFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:7933
# EXT_subtexture (GL/glext.h:7936)
GL_EXT_subtexture = 1 	# GL/glext.h:7937
# GL/glext.h:7939
glTexSubImage1DEXT = _link_function('glTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

# GL/glext.h:7940
glTexSubImage2DEXT = _link_function('glTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_subtexture')

PFNGLTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7942
PFNGLTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7943
# EXT_copy_texture (GL/glext.h:7946)
GL_EXT_copy_texture = 1 	# GL/glext.h:7947
# GL/glext.h:7949
glCopyTexImage1DEXT = _link_function('glCopyTexImage1DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:7950
glCopyTexImage2DEXT = _link_function('glCopyTexImage2DEXT', None, [GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_copy_texture')

# GL/glext.h:7951
glCopyTexSubImage1DEXT = _link_function('glCopyTexSubImage1DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_copy_texture')

# GL/glext.h:7952
glCopyTexSubImage2DEXT = _link_function('glCopyTexSubImage2DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

# GL/glext.h:7953
glCopyTexSubImage3DEXT = _link_function('glCopyTexSubImage3DEXT', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_copy_texture')

PFNGLCOPYTEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:7955
PFNGLCOPYTEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:7956
PFNGLCOPYTEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:7957
PFNGLCOPYTEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:7958
PFNGLCOPYTEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:7959
# EXT_histogram (GL/glext.h:7962)
GL_EXT_histogram = 1 	# GL/glext.h:7963
# GL/glext.h:7965
glGetHistogramEXT = _link_function('glGetHistogramEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:7966
glGetHistogramParameterfvEXT = _link_function('glGetHistogramParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:7967
glGetHistogramParameterivEXT = _link_function('glGetHistogramParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:7968
glGetMinmaxEXT = _link_function('glGetMinmaxEXT', None, [GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)], 'EXT_histogram')

# GL/glext.h:7969
glGetMinmaxParameterfvEXT = _link_function('glGetMinmaxParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_histogram')

# GL/glext.h:7970
glGetMinmaxParameterivEXT = _link_function('glGetMinmaxParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_histogram')

# GL/glext.h:7971
glHistogramEXT = _link_function('glHistogramEXT', None, [GLenum, GLsizei, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:7972
glMinmaxEXT = _link_function('glMinmaxEXT', None, [GLenum, GLenum, GLboolean], 'EXT_histogram')

# GL/glext.h:7973
glResetHistogramEXT = _link_function('glResetHistogramEXT', None, [GLenum], 'EXT_histogram')

# GL/glext.h:7974
glResetMinmaxEXT = _link_function('glResetMinmaxEXT', None, [GLenum], 'EXT_histogram')

PFNGLGETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7976
PFNGLGETHISTOGRAMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7977
PFNGLGETHISTOGRAMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7978
PFNGLGETMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLboolean, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:7979
PFNGLGETMINMAXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:7980
PFNGLGETMINMAXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:7981
PFNGLHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLboolean) 	# GL/glext.h:7982
PFNGLMINMAXEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLboolean) 	# GL/glext.h:7983
PFNGLRESETHISTOGRAMEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7984
PFNGLRESETMINMAXEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:7985
# EXT_convolution (GL/glext.h:7988)
GL_EXT_convolution = 1 	# GL/glext.h:7989
# GL/glext.h:7991
glConvolutionFilter1DEXT = _link_function('glConvolutionFilter1DEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:7992
glConvolutionFilter2DEXT = _link_function('glConvolutionFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:7993
glConvolutionParameterfEXT = _link_function('glConvolutionParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_convolution')

# GL/glext.h:7994
glConvolutionParameterfvEXT = _link_function('glConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:7995
glConvolutionParameteriEXT = _link_function('glConvolutionParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_convolution')

# GL/glext.h:7996
glConvolutionParameterivEXT = _link_function('glConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:7997
glCopyConvolutionFilter1DEXT = _link_function('glCopyConvolutionFilter1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'EXT_convolution')

# GL/glext.h:7998
glCopyConvolutionFilter2DEXT = _link_function('glCopyConvolutionFilter2DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLsizei], 'EXT_convolution')

# GL/glext.h:7999
glGetConvolutionFilterEXT = _link_function('glGetConvolutionFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:8000
glGetConvolutionParameterfvEXT = _link_function('glGetConvolutionParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_convolution')

# GL/glext.h:8001
glGetConvolutionParameterivEXT = _link_function('glGetConvolutionParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_convolution')

# GL/glext.h:8002
glGetSeparableFilterEXT = _link_function('glGetSeparableFilterEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

# GL/glext.h:8003
glSeparableFilter2DEXT = _link_function('glSeparableFilter2DEXT', None, [GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)], 'EXT_convolution')

PFNGLCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8005
PFNGLCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8006
PFNGLCONVOLUTIONPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:8007
PFNGLCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8008
PFNGLCONVOLUTIONPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:8009
PFNGLCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8010
PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:8011
PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:8012
PFNGLGETCONVOLUTIONFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8013
PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8014
PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8015
PFNGLGETSEPARABLEFILTEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:8016
PFNGLSEPARABLEFILTER2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:8017
# SGI_color_matrix (GL/glext.h:8020)
GL_SGI_color_matrix = 1 	# GL/glext.h:8021
# SGI_color_table (GL/glext.h:8024)
GL_SGI_color_table = 1 	# GL/glext.h:8025
# GL/glext.h:8027
glColorTableSGI = _link_function('glColorTableSGI', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:8028
glColorTableParameterfvSGI = _link_function('glColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:8029
glColorTableParameterivSGI = _link_function('glColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

# GL/glext.h:8030
glCopyColorTableSGI = _link_function('glCopyColorTableSGI', None, [GLenum, GLenum, GLint, GLint, GLsizei], 'SGI_color_table')

# GL/glext.h:8031
glGetColorTableSGI = _link_function('glGetColorTableSGI', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'SGI_color_table')

# GL/glext.h:8032
glGetColorTableParameterfvSGI = _link_function('glGetColorTableParameterfvSGI', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGI_color_table')

# GL/glext.h:8033
glGetColorTableParameterivSGI = _link_function('glGetColorTableParameterivSGI', None, [GLenum, GLenum, POINTER(GLint)], 'SGI_color_table')

PFNGLCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8035
PFNGLCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8036
PFNGLCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8037
PFNGLCOPYCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei) 	# GL/glext.h:8038
PFNGLGETCOLORTABLESGIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8039
PFNGLGETCOLORTABLEPARAMETERFVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8040
PFNGLGETCOLORTABLEPARAMETERIVSGIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8041
# SGIX_pixel_texture (GL/glext.h:8044)
GL_SGIX_pixel_texture = 1 	# GL/glext.h:8045
# GL/glext.h:8047
glPixelTexGenSGIX = _link_function('glPixelTexGenSGIX', None, [GLenum], 'SGIX_pixel_texture')

PFNGLPIXELTEXGENSGIXPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8049
# SGIS_pixel_texture (GL/glext.h:8052)
GL_SGIS_pixel_texture = 1 	# GL/glext.h:8053
# GL/glext.h:8055
glPixelTexGenParameteriSGIS = _link_function('glPixelTexGenParameteriSGIS', None, [GLenum, GLint], 'SGIS_pixel_texture')

# GL/glext.h:8056
glPixelTexGenParameterivSGIS = _link_function('glPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:8057
glPixelTexGenParameterfSGIS = _link_function('glPixelTexGenParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_pixel_texture')

# GL/glext.h:8058
glPixelTexGenParameterfvSGIS = _link_function('glPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

# GL/glext.h:8059
glGetPixelTexGenParameterivSGIS = _link_function('glGetPixelTexGenParameterivSGIS', None, [GLenum, POINTER(GLint)], 'SGIS_pixel_texture')

# GL/glext.h:8060
glGetPixelTexGenParameterfvSGIS = _link_function('glGetPixelTexGenParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_pixel_texture')

PFNGLPIXELTEXGENPARAMETERISGISPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:8062
PFNGLPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:8063
PFNGLPIXELTEXGENPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:8064
PFNGLPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8065
PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:8066
PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8067
# SGIS_texture4D (GL/glext.h:8070)
GL_SGIS_texture4D = 1 	# GL/glext.h:8071
# GL/glext.h:8073
glTexImage4DSGIS = _link_function('glTexImage4DSGIS', None, [GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

# GL/glext.h:8074
glTexSubImage4DSGIS = _link_function('glTexSubImage4DSGIS', None, [GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'SGIS_texture4D')

PFNGLTEXIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8076
PFNGLTEXSUBIMAGE4DSGISPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8077
# SGI_texture_color_table (GL/glext.h:8080)
GL_SGI_texture_color_table = 1 	# GL/glext.h:8081
# EXT_cmyka (GL/glext.h:8084)
GL_EXT_cmyka = 1 	# GL/glext.h:8085
# EXT_texture_object (GL/glext.h:8088)
GL_EXT_texture_object = 1 	# GL/glext.h:8089
# GL/glext.h:8091
glAreTexturesResidentEXT = _link_function('glAreTexturesResidentEXT', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'EXT_texture_object')

# GL/glext.h:8092
glBindTextureEXT = _link_function('glBindTextureEXT', None, [GLenum, GLuint], 'EXT_texture_object')

# GL/glext.h:8093
glDeleteTexturesEXT = _link_function('glDeleteTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:8094
glGenTexturesEXT = _link_function('glGenTexturesEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_texture_object')

# GL/glext.h:8095
glIsTextureEXT = _link_function('glIsTextureEXT', GLboolean, [GLuint], 'EXT_texture_object')

GLclampf = c_float 	# /usr/include/GL/gl.h:161
# GL/glext.h:8096
glPrioritizeTexturesEXT = _link_function('glPrioritizeTexturesEXT', None, [GLsizei, POINTER(GLuint), POINTER(GLclampf)], 'EXT_texture_object')

PFNGLARETEXTURESRESIDENTEXTPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:8098
PFNGLBINDTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:8099
PFNGLDELETETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8100
PFNGLGENTEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:8101
PFNGLISTEXTUREEXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:8102
PFNGLPRIORITIZETEXTURESEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint), POINTER(GLclampf)) 	# GL/glext.h:8103
# SGIS_detail_texture (GL/glext.h:8106)
GL_SGIS_detail_texture = 1 	# GL/glext.h:8107
# GL/glext.h:8109
glDetailTexFuncSGIS = _link_function('glDetailTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_detail_texture')

# GL/glext.h:8110
glGetDetailTexFuncSGIS = _link_function('glGetDetailTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_detail_texture')

PFNGLDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8112
PFNGLGETDETAILTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8113
# SGIS_sharpen_texture (GL/glext.h:8116)
GL_SGIS_sharpen_texture = 1 	# GL/glext.h:8117
# GL/glext.h:8119
glSharpenTexFuncSGIS = _link_function('glSharpenTexFuncSGIS', None, [GLenum, GLsizei, POINTER(GLfloat)], 'SGIS_sharpen_texture')

# GL/glext.h:8120
glGetSharpenTexFuncSGIS = _link_function('glGetSharpenTexFuncSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_sharpen_texture')

PFNGLSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8122
PFNGLGETSHARPENTEXFUNCSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8123
# EXT_packed_pixels (GL/glext.h:8126)
GL_EXT_packed_pixels = 1 	# GL/glext.h:8127
# SGIS_texture_lod (GL/glext.h:8130)
GL_SGIS_texture_lod = 1 	# GL/glext.h:8131
# SGIS_multisample (GL/glext.h:8134)
GL_SGIS_multisample = 1 	# GL/glext.h:8135
# GL/glext.h:8137
glSampleMaskSGIS = _link_function('glSampleMaskSGIS', None, [GLclampf, GLboolean], 'SGIS_multisample')

# GL/glext.h:8138
glSamplePatternSGIS = _link_function('glSamplePatternSGIS', None, [GLenum], 'SGIS_multisample')

PFNGLSAMPLEMASKSGISPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:8140
PFNGLSAMPLEPATTERNSGISPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8141
# EXT_rescale_normal (GL/glext.h:8144)
GL_EXT_rescale_normal = 1 	# GL/glext.h:8145
# EXT_vertex_array (GL/glext.h:8148)
GL_EXT_vertex_array = 1 	# GL/glext.h:8149
# GL/glext.h:8151
glArrayElementEXT = _link_function('glArrayElementEXT', None, [GLint], 'EXT_vertex_array')

# GL/glext.h:8152
glColorPointerEXT = _link_function('glColorPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:8153
glDrawArraysEXT = _link_function('glDrawArraysEXT', None, [GLenum, GLint, GLsizei], 'EXT_vertex_array')

# GL/glext.h:8154
glEdgeFlagPointerEXT = _link_function('glEdgeFlagPointerEXT', None, [GLsizei, GLsizei, POINTER(GLboolean)], 'EXT_vertex_array')

# GL/glext.h:8155
glGetPointervEXT = _link_function('glGetPointervEXT', None, [GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_array')

# GL/glext.h:8156
glIndexPointerEXT = _link_function('glIndexPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:8157
glNormalPointerEXT = _link_function('glNormalPointerEXT', None, [GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:8158
glTexCoordPointerEXT = _link_function('glTexCoordPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

# GL/glext.h:8159
glVertexPointerEXT = _link_function('glVertexPointerEXT', None, [GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)], 'EXT_vertex_array')

PFNGLARRAYELEMENTEXTPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:8161
PFNGLCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8162
PFNGLDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:8163
PFNGLEDGEFLAGPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLsizei, POINTER(GLboolean)) 	# GL/glext.h:8164
PFNGLGETPOINTERVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8165
PFNGLINDEXPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8166
PFNGLNORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8167
PFNGLTEXCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8168
PFNGLVERTEXPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8169
# EXT_misc_attribute (GL/glext.h:8172)
GL_EXT_misc_attribute = 1 	# GL/glext.h:8173
# SGIS_generate_mipmap (GL/glext.h:8176)
GL_SGIS_generate_mipmap = 1 	# GL/glext.h:8177
# SGIX_clipmap (GL/glext.h:8180)
GL_SGIX_clipmap = 1 	# GL/glext.h:8181
# SGIX_shadow (GL/glext.h:8184)
GL_SGIX_shadow = 1 	# GL/glext.h:8185
# SGIS_texture_edge_clamp (GL/glext.h:8188)
GL_SGIS_texture_edge_clamp = 1 	# GL/glext.h:8189
# SGIS_texture_border_clamp (GL/glext.h:8192)
GL_SGIS_texture_border_clamp = 1 	# GL/glext.h:8193
# EXT_blend_minmax (GL/glext.h:8196)
GL_EXT_blend_minmax = 1 	# GL/glext.h:8197
# GL/glext.h:8199
glBlendEquationEXT = _link_function('glBlendEquationEXT', None, [GLenum], 'EXT_blend_minmax')

PFNGLBLENDEQUATIONEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8201
# EXT_blend_subtract (GL/glext.h:8204)
GL_EXT_blend_subtract = 1 	# GL/glext.h:8205
# EXT_blend_logic_op (GL/glext.h:8208)
GL_EXT_blend_logic_op = 1 	# GL/glext.h:8209
# SGIX_interlace (GL/glext.h:8212)
GL_SGIX_interlace = 1 	# GL/glext.h:8213
# SGIX_pixel_tiles (GL/glext.h:8216)
GL_SGIX_pixel_tiles = 1 	# GL/glext.h:8217
# SGIX_texture_select (GL/glext.h:8220)
GL_SGIX_texture_select = 1 	# GL/glext.h:8221
# SGIX_sprite (GL/glext.h:8224)
GL_SGIX_sprite = 1 	# GL/glext.h:8225
# GL/glext.h:8227
glSpriteParameterfSGIX = _link_function('glSpriteParameterfSGIX', None, [GLenum, GLfloat], 'SGIX_sprite')

# GL/glext.h:8228
glSpriteParameterfvSGIX = _link_function('glSpriteParameterfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_sprite')

# GL/glext.h:8229
glSpriteParameteriSGIX = _link_function('glSpriteParameteriSGIX', None, [GLenum, GLint], 'SGIX_sprite')

# GL/glext.h:8230
glSpriteParameterivSGIX = _link_function('glSpriteParameterivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_sprite')

PFNGLSPRITEPARAMETERFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:8232
PFNGLSPRITEPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8233
PFNGLSPRITEPARAMETERISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:8234
PFNGLSPRITEPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:8235
# SGIX_texture_multi_buffer (GL/glext.h:8238)
GL_SGIX_texture_multi_buffer = 1 	# GL/glext.h:8239
# EXT_point_parameters (GL/glext.h:8242)
GL_EXT_point_parameters = 1 	# GL/glext.h:8243
# GL/glext.h:8245
glPointParameterfEXT = _link_function('glPointParameterfEXT', None, [GLenum, GLfloat], 'EXT_point_parameters')

# GL/glext.h:8246
glPointParameterfvEXT = _link_function('glPointParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_point_parameters')

PFNGLPOINTPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:8248
PFNGLPOINTPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8249
# SGIS_point_parameters (GL/glext.h:8252)
GL_SGIS_point_parameters = 1 	# GL/glext.h:8253
# GL/glext.h:8255
glPointParameterfSGIS = _link_function('glPointParameterfSGIS', None, [GLenum, GLfloat], 'SGIS_point_parameters')

# GL/glext.h:8256
glPointParameterfvSGIS = _link_function('glPointParameterfvSGIS', None, [GLenum, POINTER(GLfloat)], 'SGIS_point_parameters')

PFNGLPOINTPARAMETERFSGISPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:8258
PFNGLPOINTPARAMETERFVSGISPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8259
# SGIX_instruments (GL/glext.h:8262)
GL_SGIX_instruments = 1 	# GL/glext.h:8263
# GL/glext.h:8265
glGetInstrumentsSGIX = _link_function('glGetInstrumentsSGIX', GLint, [], 'SGIX_instruments')

# GL/glext.h:8266
glInstrumentsBufferSGIX = _link_function('glInstrumentsBufferSGIX', None, [GLsizei, POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:8267
glPollInstrumentsSGIX = _link_function('glPollInstrumentsSGIX', GLint, [POINTER(GLint)], 'SGIX_instruments')

# GL/glext.h:8268
glReadInstrumentsSGIX = _link_function('glReadInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

# GL/glext.h:8269
glStartInstrumentsSGIX = _link_function('glStartInstrumentsSGIX', None, [], 'SGIX_instruments')

# GL/glext.h:8270
glStopInstrumentsSGIX = _link_function('glStopInstrumentsSGIX', None, [GLint], 'SGIX_instruments')

PFNGLGETINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint) 	# GL/glext.h:8272
PFNGLINSTRUMENTSBUFFERSGIXPROC = CFUNCTYPE(None, GLsizei, POINTER(GLint)) 	# GL/glext.h:8273
PFNGLPOLLINSTRUMENTSSGIXPROC = CFUNCTYPE(GLint, POINTER(GLint)) 	# GL/glext.h:8274
PFNGLREADINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:8275
PFNGLSTARTINSTRUMENTSSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:8276
PFNGLSTOPINSTRUMENTSSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:8277
# SGIX_texture_scale_bias (GL/glext.h:8280)
GL_SGIX_texture_scale_bias = 1 	# GL/glext.h:8281
# SGIX_framezoom (GL/glext.h:8284)
GL_SGIX_framezoom = 1 	# GL/glext.h:8285
# GL/glext.h:8287
glFrameZoomSGIX = _link_function('glFrameZoomSGIX', None, [GLint], 'SGIX_framezoom')

PFNGLFRAMEZOOMSGIXPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:8289
# SGIX_tag_sample_buffer (GL/glext.h:8292)
GL_SGIX_tag_sample_buffer = 1 	# GL/glext.h:8293
# GL/glext.h:8295
glTagSampleBufferSGIX = _link_function('glTagSampleBufferSGIX', None, [], 'SGIX_tag_sample_buffer')

PFNGLTAGSAMPLEBUFFERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:8297
# SGIX_polynomial_ffd (GL/glext.h:8300)
GL_SGIX_polynomial_ffd = 1 	# GL/glext.h:8301
# GL/glext.h:8303
glDeformationMap3dSGIX = _link_function('glDeformationMap3dSGIX', None, [GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)], 'SGIX_polynomial_ffd')

# GL/glext.h:8304
glDeformationMap3fSGIX = _link_function('glDeformationMap3fSGIX', None, [GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)], 'SGIX_polynomial_ffd')

# GL/glext.h:8305
glDeformSGIX = _link_function('glDeformSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

# GL/glext.h:8306
glLoadIdentityDeformationMapSGIX = _link_function('glLoadIdentityDeformationMapSGIX', None, [GLbitfield], 'SGIX_polynomial_ffd')

PFNGLDEFORMATIONMAP3DSGIXPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)) 	# GL/glext.h:8308
PFNGLDEFORMATIONMAP3FSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)) 	# GL/glext.h:8309
PFNGLDEFORMSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:8310
PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:8311
# SGIX_reference_plane (GL/glext.h:8314)
GL_SGIX_reference_plane = 1 	# GL/glext.h:8315
# GL/glext.h:8317
glReferencePlaneSGIX = _link_function('glReferencePlaneSGIX', None, [POINTER(GLdouble)], 'SGIX_reference_plane')

PFNGLREFERENCEPLANESGIXPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:8319
# SGIX_flush_raster (GL/glext.h:8322)
GL_SGIX_flush_raster = 1 	# GL/glext.h:8323
# GL/glext.h:8325
glFlushRasterSGIX = _link_function('glFlushRasterSGIX', None, [], 'SGIX_flush_raster')

PFNGLFLUSHRASTERSGIXPROC = CFUNCTYPE(None) 	# GL/glext.h:8327
# SGIX_depth_texture (GL/glext.h:8330)
GL_SGIX_depth_texture = 1 	# GL/glext.h:8331
# SGIS_fog_function (GL/glext.h:8334)
GL_SGIS_fog_function = 1 	# GL/glext.h:8335
# GL/glext.h:8337
glFogFuncSGIS = _link_function('glFogFuncSGIS', None, [GLsizei, POINTER(GLfloat)], 'SGIS_fog_function')

# GL/glext.h:8338
glGetFogFuncSGIS = _link_function('glGetFogFuncSGIS', None, [POINTER(GLfloat)], 'SGIS_fog_function')

PFNGLFOGFUNCSGISPROC = CFUNCTYPE(None, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:8340
PFNGLGETFOGFUNCSGISPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:8341
# SGIX_fog_offset (GL/glext.h:8344)
GL_SGIX_fog_offset = 1 	# GL/glext.h:8345
# HP_image_transform (GL/glext.h:8348)
GL_HP_image_transform = 1 	# GL/glext.h:8349
# GL/glext.h:8351
glImageTransformParameteriHP = _link_function('glImageTransformParameteriHP', None, [GLenum, GLenum, GLint], 'HP_image_transform')

# GL/glext.h:8352
glImageTransformParameterfHP = _link_function('glImageTransformParameterfHP', None, [GLenum, GLenum, GLfloat], 'HP_image_transform')

# GL/glext.h:8353
glImageTransformParameterivHP = _link_function('glImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:8354
glImageTransformParameterfvHP = _link_function('glImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

# GL/glext.h:8355
glGetImageTransformParameterivHP = _link_function('glGetImageTransformParameterivHP', None, [GLenum, GLenum, POINTER(GLint)], 'HP_image_transform')

# GL/glext.h:8356
glGetImageTransformParameterfvHP = _link_function('glGetImageTransformParameterfvHP', None, [GLenum, GLenum, POINTER(GLfloat)], 'HP_image_transform')

PFNGLIMAGETRANSFORMPARAMETERIHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:8358
PFNGLIMAGETRANSFORMPARAMETERFHPPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:8359
PFNGLIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8360
PFNGLIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8361
PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8362
PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8363
# HP_convolution_border_modes (GL/glext.h:8366)
GL_HP_convolution_border_modes = 1 	# GL/glext.h:8367
# SGIX_texture_add_env (GL/glext.h:8370)
GL_SGIX_texture_add_env = 1 	# GL/glext.h:8371
# EXT_color_subtable (GL/glext.h:8374)
GL_EXT_color_subtable = 1 	# GL/glext.h:8375
# GL/glext.h:8377
glColorSubTableEXT = _link_function('glColorSubTableEXT', None, [GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_color_subtable')

# GL/glext.h:8378
glCopyColorSubTableEXT = _link_function('glCopyColorSubTableEXT', None, [GLenum, GLsizei, GLint, GLint, GLsizei], 'EXT_color_subtable')

PFNGLCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8380
PFNGLCOPYCOLORSUBTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLint, GLint, GLsizei) 	# GL/glext.h:8381
# PGI_vertex_hints (GL/glext.h:8384)
GL_PGI_vertex_hints = 1 	# GL/glext.h:8385
# PGI_misc_hints (GL/glext.h:8388)
GL_PGI_misc_hints = 1 	# GL/glext.h:8389
# GL/glext.h:8391
glHintPGI = _link_function('glHintPGI', None, [GLenum, GLint], 'PGI_misc_hints')

PFNGLHINTPGIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:8393
# EXT_paletted_texture (GL/glext.h:8396)
GL_EXT_paletted_texture = 1 	# GL/glext.h:8397
# GL/glext.h:8399
glColorTableEXT = _link_function('glColorTableEXT', None, [GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:8400
glGetColorTableEXT = _link_function('glGetColorTableEXT', None, [GLenum, GLenum, GLenum, POINTER(GLvoid)], 'EXT_paletted_texture')

# GL/glext.h:8401
glGetColorTableParameterivEXT = _link_function('glGetColorTableParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_paletted_texture')

# GL/glext.h:8402
glGetColorTableParameterfvEXT = _link_function('glGetColorTableParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_paletted_texture')

PFNGLCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8404
PFNGLGETCOLORTABLEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8405
PFNGLGETCOLORTABLEPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8406
PFNGLGETCOLORTABLEPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8407
# EXT_clip_volume_hint (GL/glext.h:8410)
GL_EXT_clip_volume_hint = 1 	# GL/glext.h:8411
# SGIX_list_priority (GL/glext.h:8414)
GL_SGIX_list_priority = 1 	# GL/glext.h:8415
# GL/glext.h:8417
glGetListParameterfvSGIX = _link_function('glGetListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:8418
glGetListParameterivSGIX = _link_function('glGetListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

# GL/glext.h:8419
glListParameterfSGIX = _link_function('glListParameterfSGIX', None, [GLuint, GLenum, GLfloat], 'SGIX_list_priority')

# GL/glext.h:8420
glListParameterfvSGIX = _link_function('glListParameterfvSGIX', None, [GLuint, GLenum, POINTER(GLfloat)], 'SGIX_list_priority')

# GL/glext.h:8421
glListParameteriSGIX = _link_function('glListParameteriSGIX', None, [GLuint, GLenum, GLint], 'SGIX_list_priority')

# GL/glext.h:8422
glListParameterivSGIX = _link_function('glListParameterivSGIX', None, [GLuint, GLenum, POINTER(GLint)], 'SGIX_list_priority')

PFNGLGETLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8424
PFNGLGETLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:8425
PFNGLLISTPARAMETERFSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLfloat) 	# GL/glext.h:8426
PFNGLLISTPARAMETERFVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8427
PFNGLLISTPARAMETERISGIXPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:8428
PFNGLLISTPARAMETERIVSGIXPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:8429
# SGIX_ir_instrument1 (GL/glext.h:8432)
GL_SGIX_ir_instrument1 = 1 	# GL/glext.h:8433
# SGIX_calligraphic_fragment (GL/glext.h:8436)
GL_SGIX_calligraphic_fragment = 1 	# GL/glext.h:8437
# SGIX_texture_lod_bias (GL/glext.h:8440)
GL_SGIX_texture_lod_bias = 1 	# GL/glext.h:8441
# SGIX_shadow_ambient (GL/glext.h:8444)
GL_SGIX_shadow_ambient = 1 	# GL/glext.h:8445
# EXT_index_texture (GL/glext.h:8448)
GL_EXT_index_texture = 1 	# GL/glext.h:8449
# EXT_index_material (GL/glext.h:8452)
GL_EXT_index_material = 1 	# GL/glext.h:8453
# GL/glext.h:8455
glIndexMaterialEXT = _link_function('glIndexMaterialEXT', None, [GLenum, GLenum], 'EXT_index_material')

PFNGLINDEXMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:8457
# EXT_index_func (GL/glext.h:8460)
GL_EXT_index_func = 1 	# GL/glext.h:8461
# GL/glext.h:8463
glIndexFuncEXT = _link_function('glIndexFuncEXT', None, [GLenum, GLclampf], 'EXT_index_func')

PFNGLINDEXFUNCEXTPROC = CFUNCTYPE(None, GLenum, GLclampf) 	# GL/glext.h:8465
# EXT_index_array_formats (GL/glext.h:8468)
GL_EXT_index_array_formats = 1 	# GL/glext.h:8469
# EXT_compiled_vertex_array (GL/glext.h:8472)
GL_EXT_compiled_vertex_array = 1 	# GL/glext.h:8473
# GL/glext.h:8475
glLockArraysEXT = _link_function('glLockArraysEXT', None, [GLint, GLsizei], 'EXT_compiled_vertex_array')

# GL/glext.h:8476
glUnlockArraysEXT = _link_function('glUnlockArraysEXT', None, [], 'EXT_compiled_vertex_array')

PFNGLLOCKARRAYSEXTPROC = CFUNCTYPE(None, GLint, GLsizei) 	# GL/glext.h:8478
PFNGLUNLOCKARRAYSEXTPROC = CFUNCTYPE(None) 	# GL/glext.h:8479
# EXT_cull_vertex (GL/glext.h:8482)
GL_EXT_cull_vertex = 1 	# GL/glext.h:8483
# GL/glext.h:8485
glCullParameterdvEXT = _link_function('glCullParameterdvEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_cull_vertex')

# GL/glext.h:8486
glCullParameterfvEXT = _link_function('glCullParameterfvEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_cull_vertex')

PFNGLCULLPARAMETERDVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:8488
PFNGLCULLPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8489
# SGIX_ycrcb (GL/glext.h:8492)
GL_SGIX_ycrcb = 1 	# GL/glext.h:8493
# SGIX_fragment_lighting (GL/glext.h:8496)
GL_SGIX_fragment_lighting = 1 	# GL/glext.h:8497
# GL/glext.h:8499
glFragmentColorMaterialSGIX = _link_function('glFragmentColorMaterialSGIX', None, [GLenum, GLenum], 'SGIX_fragment_lighting')

# GL/glext.h:8500
glFragmentLightfSGIX = _link_function('glFragmentLightfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:8501
glFragmentLightfvSGIX = _link_function('glFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:8502
glFragmentLightiSGIX = _link_function('glFragmentLightiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:8503
glFragmentLightivSGIX = _link_function('glFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:8504
glFragmentLightModelfSGIX = _link_function('glFragmentLightModelfSGIX', None, [GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:8505
glFragmentLightModelfvSGIX = _link_function('glFragmentLightModelfvSGIX', None, [GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:8506
glFragmentLightModeliSGIX = _link_function('glFragmentLightModeliSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:8507
glFragmentLightModelivSGIX = _link_function('glFragmentLightModelivSGIX', None, [GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:8508
glFragmentMaterialfSGIX = _link_function('glFragmentMaterialfSGIX', None, [GLenum, GLenum, GLfloat], 'SGIX_fragment_lighting')

# GL/glext.h:8509
glFragmentMaterialfvSGIX = _link_function('glFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:8510
glFragmentMaterialiSGIX = _link_function('glFragmentMaterialiSGIX', None, [GLenum, GLenum, GLint], 'SGIX_fragment_lighting')

# GL/glext.h:8511
glFragmentMaterialivSGIX = _link_function('glFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:8512
glGetFragmentLightfvSGIX = _link_function('glGetFragmentLightfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:8513
glGetFragmentLightivSGIX = _link_function('glGetFragmentLightivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:8514
glGetFragmentMaterialfvSGIX = _link_function('glGetFragmentMaterialfvSGIX', None, [GLenum, GLenum, POINTER(GLfloat)], 'SGIX_fragment_lighting')

# GL/glext.h:8515
glGetFragmentMaterialivSGIX = _link_function('glGetFragmentMaterialivSGIX', None, [GLenum, GLenum, POINTER(GLint)], 'SGIX_fragment_lighting')

# GL/glext.h:8516
glLightEnviSGIX = _link_function('glLightEnviSGIX', None, [GLenum, GLint], 'SGIX_fragment_lighting')

PFNGLFRAGMENTCOLORMATERIALSGIXPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:8518
PFNGLFRAGMENTLIGHTFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:8519
PFNGLFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8520
PFNGLFRAGMENTLIGHTISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:8521
PFNGLFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8522
PFNGLFRAGMENTLIGHTMODELFSGIXPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:8523
PFNGLFRAGMENTLIGHTMODELFVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8524
PFNGLFRAGMENTLIGHTMODELISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:8525
PFNGLFRAGMENTLIGHTMODELIVSGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:8526
PFNGLFRAGMENTMATERIALFSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:8527
PFNGLFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8528
PFNGLFRAGMENTMATERIALISGIXPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:8529
PFNGLFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8530
PFNGLGETFRAGMENTLIGHTFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8531
PFNGLGETFRAGMENTLIGHTIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8532
PFNGLGETFRAGMENTMATERIALFVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8533
PFNGLGETFRAGMENTMATERIALIVSGIXPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8534
PFNGLLIGHTENVISGIXPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:8535
# IBM_rasterpos_clip (GL/glext.h:8538)
GL_IBM_rasterpos_clip = 1 	# GL/glext.h:8539
# HP_texture_lighting (GL/glext.h:8542)
GL_HP_texture_lighting = 1 	# GL/glext.h:8543
# EXT_draw_range_elements (GL/glext.h:8546)
GL_EXT_draw_range_elements = 1 	# GL/glext.h:8547
# GL/glext.h:8549
glDrawRangeElementsEXT = _link_function('glDrawRangeElementsEXT', None, [GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'EXT_draw_range_elements')

PFNGLDRAWRANGEELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:8551
# WIN_phong_shading (GL/glext.h:8554)
GL_WIN_phong_shading = 1 	# GL/glext.h:8555
# WIN_specular_fog (GL/glext.h:8558)
GL_WIN_specular_fog = 1 	# GL/glext.h:8559
# EXT_light_texture (GL/glext.h:8562)
GL_EXT_light_texture = 1 	# GL/glext.h:8563
# GL/glext.h:8565
glApplyTextureEXT = _link_function('glApplyTextureEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:8566
glTextureLightEXT = _link_function('glTextureLightEXT', None, [GLenum], 'EXT_light_texture')

# GL/glext.h:8567
glTextureMaterialEXT = _link_function('glTextureMaterialEXT', None, [GLenum, GLenum], 'EXT_light_texture')

PFNGLAPPLYTEXTUREEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8569
PFNGLTEXTURELIGHTEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8570
PFNGLTEXTUREMATERIALEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:8571
# SGIX_blend_alpha_minmax (GL/glext.h:8574)
GL_SGIX_blend_alpha_minmax = 1 	# GL/glext.h:8575
# EXT_bgra (GL/glext.h:8578)
GL_EXT_bgra = 1 	# GL/glext.h:8579
# SGIX_async (GL/glext.h:8582)
GL_SGIX_async = 1 	# GL/glext.h:8583
# GL/glext.h:8585
glAsyncMarkerSGIX = _link_function('glAsyncMarkerSGIX', None, [GLuint], 'SGIX_async')

# GL/glext.h:8586
glFinishAsyncSGIX = _link_function('glFinishAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:8587
glPollAsyncSGIX = _link_function('glPollAsyncSGIX', GLint, [POINTER(GLuint)], 'SGIX_async')

# GL/glext.h:8588
glGenAsyncMarkersSGIX = _link_function('glGenAsyncMarkersSGIX', GLuint, [GLsizei], 'SGIX_async')

# GL/glext.h:8589
glDeleteAsyncMarkersSGIX = _link_function('glDeleteAsyncMarkersSGIX', None, [GLuint, GLsizei], 'SGIX_async')

# GL/glext.h:8590
glIsAsyncMarkerSGIX = _link_function('glIsAsyncMarkerSGIX', GLboolean, [GLuint], 'SGIX_async')

PFNGLASYNCMARKERSGIXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:8592
PFNGLFINISHASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:8593
PFNGLPOLLASYNCSGIXPROC = CFUNCTYPE(GLint, POINTER(GLuint)) 	# GL/glext.h:8594
PFNGLGENASYNCMARKERSSGIXPROC = CFUNCTYPE(GLuint, GLsizei) 	# GL/glext.h:8595
PFNGLDELETEASYNCMARKERSSGIXPROC = CFUNCTYPE(None, GLuint, GLsizei) 	# GL/glext.h:8596
PFNGLISASYNCMARKERSGIXPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:8597
# SGIX_async_pixel (GL/glext.h:8600)
GL_SGIX_async_pixel = 1 	# GL/glext.h:8601
# SGIX_async_histogram (GL/glext.h:8604)
GL_SGIX_async_histogram = 1 	# GL/glext.h:8605
# INTEL_parallel_arrays (GL/glext.h:8608)
GL_INTEL_parallel_arrays = 1 	# GL/glext.h:8609
# GL/glext.h:8611
glVertexPointervINTEL = _link_function('glVertexPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:8612
glNormalPointervINTEL = _link_function('glNormalPointervINTEL', None, [GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:8613
glColorPointervINTEL = _link_function('glColorPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

# GL/glext.h:8614
glTexCoordPointervINTEL = _link_function('glTexCoordPointervINTEL', None, [GLint, GLenum, POINTER(POINTER(GLvoid))], 'INTEL_parallel_arrays')

PFNGLVERTEXPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8616
PFNGLNORMALPOINTERVINTELPROC = CFUNCTYPE(None, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8617
PFNGLCOLORPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8618
PFNGLTEXCOORDPOINTERVINTELPROC = CFUNCTYPE(None, GLint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8619
# HP_occlusion_test (GL/glext.h:8622)
GL_HP_occlusion_test = 1 	# GL/glext.h:8623
# EXT_pixel_transform (GL/glext.h:8626)
GL_EXT_pixel_transform = 1 	# GL/glext.h:8627
# GL/glext.h:8629
glPixelTransformParameteriEXT = _link_function('glPixelTransformParameteriEXT', None, [GLenum, GLenum, GLint], 'EXT_pixel_transform')

# GL/glext.h:8630
glPixelTransformParameterfEXT = _link_function('glPixelTransformParameterfEXT', None, [GLenum, GLenum, GLfloat], 'EXT_pixel_transform')

# GL/glext.h:8631
glPixelTransformParameterivEXT = _link_function('glPixelTransformParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_pixel_transform')

# GL/glext.h:8632
glPixelTransformParameterfvEXT = _link_function('glPixelTransformParameterfvEXT', None, [GLenum, GLenum, POINTER(GLfloat)], 'EXT_pixel_transform')

PFNGLPIXELTRANSFORMPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:8634
PFNGLPIXELTRANSFORMPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLfloat) 	# GL/glext.h:8635
PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:8636
PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:8637
# EXT_pixel_transform_color_table (GL/glext.h:8640)
GL_EXT_pixel_transform_color_table = 1 	# GL/glext.h:8641
# EXT_shared_texture_palette (GL/glext.h:8644)
GL_EXT_shared_texture_palette = 1 	# GL/glext.h:8645
# EXT_separate_specular_color (GL/glext.h:8648)
GL_EXT_separate_specular_color = 1 	# GL/glext.h:8649
# EXT_secondary_color (GL/glext.h:8652)
GL_EXT_secondary_color = 1 	# GL/glext.h:8653
# GL/glext.h:8655
glSecondaryColor3bEXT = _link_function('glSecondaryColor3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_secondary_color')

# GL/glext.h:8656
glSecondaryColor3bvEXT = _link_function('glSecondaryColor3bvEXT', None, [POINTER(GLbyte)], 'EXT_secondary_color')

# GL/glext.h:8657
glSecondaryColor3dEXT = _link_function('glSecondaryColor3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_secondary_color')

# GL/glext.h:8658
glSecondaryColor3dvEXT = _link_function('glSecondaryColor3dvEXT', None, [POINTER(GLdouble)], 'EXT_secondary_color')

# GL/glext.h:8659
glSecondaryColor3fEXT = _link_function('glSecondaryColor3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_secondary_color')

# GL/glext.h:8660
glSecondaryColor3fvEXT = _link_function('glSecondaryColor3fvEXT', None, [POINTER(GLfloat)], 'EXT_secondary_color')

# GL/glext.h:8661
glSecondaryColor3iEXT = _link_function('glSecondaryColor3iEXT', None, [GLint, GLint, GLint], 'EXT_secondary_color')

# GL/glext.h:8662
glSecondaryColor3ivEXT = _link_function('glSecondaryColor3ivEXT', None, [POINTER(GLint)], 'EXT_secondary_color')

# GL/glext.h:8663
glSecondaryColor3sEXT = _link_function('glSecondaryColor3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_secondary_color')

# GL/glext.h:8664
glSecondaryColor3svEXT = _link_function('glSecondaryColor3svEXT', None, [POINTER(GLshort)], 'EXT_secondary_color')

# GL/glext.h:8665
glSecondaryColor3ubEXT = _link_function('glSecondaryColor3ubEXT', None, [GLubyte, GLubyte, GLubyte], 'EXT_secondary_color')

# GL/glext.h:8666
glSecondaryColor3ubvEXT = _link_function('glSecondaryColor3ubvEXT', None, [POINTER(GLubyte)], 'EXT_secondary_color')

# GL/glext.h:8667
glSecondaryColor3uiEXT = _link_function('glSecondaryColor3uiEXT', None, [GLuint, GLuint, GLuint], 'EXT_secondary_color')

# GL/glext.h:8668
glSecondaryColor3uivEXT = _link_function('glSecondaryColor3uivEXT', None, [POINTER(GLuint)], 'EXT_secondary_color')

# GL/glext.h:8669
glSecondaryColor3usEXT = _link_function('glSecondaryColor3usEXT', None, [GLushort, GLushort, GLushort], 'EXT_secondary_color')

# GL/glext.h:8670
glSecondaryColor3usvEXT = _link_function('glSecondaryColor3usvEXT', None, [POINTER(GLushort)], 'EXT_secondary_color')

# GL/glext.h:8671
glSecondaryColorPointerEXT = _link_function('glSecondaryColorPointerEXT', None, [GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_secondary_color')

PFNGLSECONDARYCOLOR3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:8673
PFNGLSECONDARYCOLOR3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:8674
PFNGLSECONDARYCOLOR3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8675
PFNGLSECONDARYCOLOR3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:8676
PFNGLSECONDARYCOLOR3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8677
PFNGLSECONDARYCOLOR3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:8678
PFNGLSECONDARYCOLOR3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:8679
PFNGLSECONDARYCOLOR3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:8680
PFNGLSECONDARYCOLOR3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:8681
PFNGLSECONDARYCOLOR3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:8682
PFNGLSECONDARYCOLOR3UBEXTPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:8683
PFNGLSECONDARYCOLOR3UBVEXTPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:8684
PFNGLSECONDARYCOLOR3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:8685
PFNGLSECONDARYCOLOR3UIVEXTPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:8686
PFNGLSECONDARYCOLOR3USEXTPROC = CFUNCTYPE(None, GLushort, GLushort, GLushort) 	# GL/glext.h:8687
PFNGLSECONDARYCOLOR3USVEXTPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:8688
PFNGLSECONDARYCOLORPOINTEREXTPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8689
# EXT_texture_perturb_normal (GL/glext.h:8692)
GL_EXT_texture_perturb_normal = 1 	# GL/glext.h:8693
# GL/glext.h:8695
glTextureNormalEXT = _link_function('glTextureNormalEXT', None, [GLenum], 'EXT_texture_perturb_normal')

PFNGLTEXTURENORMALEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:8697
# EXT_multi_draw_arrays (GL/glext.h:8700)
GL_EXT_multi_draw_arrays = 1 	# GL/glext.h:8701
# GL/glext.h:8703
glMultiDrawArraysEXT = _link_function('glMultiDrawArraysEXT', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'EXT_multi_draw_arrays')

# GL/glext.h:8704
glMultiDrawElementsEXT = _link_function('glMultiDrawElementsEXT', None, [GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei], 'EXT_multi_draw_arrays')

PFNGLMULTIDRAWARRAYSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:8706
PFNGLMULTIDRAWELEMENTSEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei) 	# GL/glext.h:8707
# EXT_fog_coord (GL/glext.h:8710)
GL_EXT_fog_coord = 1 	# GL/glext.h:8711
# GL/glext.h:8713
glFogCoordfEXT = _link_function('glFogCoordfEXT', None, [GLfloat], 'EXT_fog_coord')

# GL/glext.h:8714
glFogCoordfvEXT = _link_function('glFogCoordfvEXT', None, [POINTER(GLfloat)], 'EXT_fog_coord')

# GL/glext.h:8715
glFogCoorddEXT = _link_function('glFogCoorddEXT', None, [GLdouble], 'EXT_fog_coord')

# GL/glext.h:8716
glFogCoorddvEXT = _link_function('glFogCoorddvEXT', None, [POINTER(GLdouble)], 'EXT_fog_coord')

# GL/glext.h:8717
glFogCoordPointerEXT = _link_function('glFogCoordPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_fog_coord')

PFNGLFOGCOORDFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:8719
PFNGLFOGCOORDFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:8720
PFNGLFOGCOORDDEXTPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:8721
PFNGLFOGCOORDDVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:8722
PFNGLFOGCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8723
# REND_screen_coordinates (GL/glext.h:8726)
GL_REND_screen_coordinates = 1 	# GL/glext.h:8727
# EXT_coordinate_frame (GL/glext.h:8730)
GL_EXT_coordinate_frame = 1 	# GL/glext.h:8731
# GL/glext.h:8733
glTangent3bEXT = _link_function('glTangent3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:8734
glTangent3bvEXT = _link_function('glTangent3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:8735
glTangent3dEXT = _link_function('glTangent3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:8736
glTangent3dvEXT = _link_function('glTangent3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:8737
glTangent3fEXT = _link_function('glTangent3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:8738
glTangent3fvEXT = _link_function('glTangent3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:8739
glTangent3iEXT = _link_function('glTangent3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:8740
glTangent3ivEXT = _link_function('glTangent3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:8741
glTangent3sEXT = _link_function('glTangent3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:8742
glTangent3svEXT = _link_function('glTangent3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:8743
glBinormal3bEXT = _link_function('glBinormal3bEXT', None, [GLbyte, GLbyte, GLbyte], 'EXT_coordinate_frame')

# GL/glext.h:8744
glBinormal3bvEXT = _link_function('glBinormal3bvEXT', None, [POINTER(GLbyte)], 'EXT_coordinate_frame')

# GL/glext.h:8745
glBinormal3dEXT = _link_function('glBinormal3dEXT', None, [GLdouble, GLdouble, GLdouble], 'EXT_coordinate_frame')

# GL/glext.h:8746
glBinormal3dvEXT = _link_function('glBinormal3dvEXT', None, [POINTER(GLdouble)], 'EXT_coordinate_frame')

# GL/glext.h:8747
glBinormal3fEXT = _link_function('glBinormal3fEXT', None, [GLfloat, GLfloat, GLfloat], 'EXT_coordinate_frame')

# GL/glext.h:8748
glBinormal3fvEXT = _link_function('glBinormal3fvEXT', None, [POINTER(GLfloat)], 'EXT_coordinate_frame')

# GL/glext.h:8749
glBinormal3iEXT = _link_function('glBinormal3iEXT', None, [GLint, GLint, GLint], 'EXT_coordinate_frame')

# GL/glext.h:8750
glBinormal3ivEXT = _link_function('glBinormal3ivEXT', None, [POINTER(GLint)], 'EXT_coordinate_frame')

# GL/glext.h:8751
glBinormal3sEXT = _link_function('glBinormal3sEXT', None, [GLshort, GLshort, GLshort], 'EXT_coordinate_frame')

# GL/glext.h:8752
glBinormal3svEXT = _link_function('glBinormal3svEXT', None, [POINTER(GLshort)], 'EXT_coordinate_frame')

# GL/glext.h:8753
glTangentPointerEXT = _link_function('glTangentPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

# GL/glext.h:8754
glBinormalPointerEXT = _link_function('glBinormalPointerEXT', None, [GLenum, GLsizei, POINTER(GLvoid)], 'EXT_coordinate_frame')

PFNGLTANGENT3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:8756
PFNGLTANGENT3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:8757
PFNGLTANGENT3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8758
PFNGLTANGENT3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:8759
PFNGLTANGENT3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8760
PFNGLTANGENT3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:8761
PFNGLTANGENT3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:8762
PFNGLTANGENT3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:8763
PFNGLTANGENT3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:8764
PFNGLTANGENT3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:8765
PFNGLBINORMAL3BEXTPROC = CFUNCTYPE(None, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:8766
PFNGLBINORMAL3BVEXTPROC = CFUNCTYPE(None, POINTER(GLbyte)) 	# GL/glext.h:8767
PFNGLBINORMAL3DEXTPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:8768
PFNGLBINORMAL3DVEXTPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:8769
PFNGLBINORMAL3FEXTPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8770
PFNGLBINORMAL3FVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:8771
PFNGLBINORMAL3IEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:8772
PFNGLBINORMAL3IVEXTPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:8773
PFNGLBINORMAL3SEXTPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:8774
PFNGLBINORMAL3SVEXTPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:8775
PFNGLTANGENTPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8776
PFNGLBINORMALPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8777
# EXT_texture_env_combine (GL/glext.h:8780)
GL_EXT_texture_env_combine = 1 	# GL/glext.h:8781
# APPLE_specular_vector (GL/glext.h:8784)
GL_APPLE_specular_vector = 1 	# GL/glext.h:8785
# APPLE_transform_hint (GL/glext.h:8788)
GL_APPLE_transform_hint = 1 	# GL/glext.h:8789
# SGIX_fog_scale (GL/glext.h:8792)
GL_SGIX_fog_scale = 1 	# GL/glext.h:8793
# SUNX_constant_data (GL/glext.h:8796)
GL_SUNX_constant_data = 1 	# GL/glext.h:8797
# GL/glext.h:8799
glFinishTextureSUNX = _link_function('glFinishTextureSUNX', None, [], 'SUNX_constant_data')

PFNGLFINISHTEXTURESUNXPROC = CFUNCTYPE(None) 	# GL/glext.h:8801
# SUN_global_alpha (GL/glext.h:8804)
GL_SUN_global_alpha = 1 	# GL/glext.h:8805
# GL/glext.h:8807
glGlobalAlphaFactorbSUN = _link_function('glGlobalAlphaFactorbSUN', None, [GLbyte], 'SUN_global_alpha')

# GL/glext.h:8808
glGlobalAlphaFactorsSUN = _link_function('glGlobalAlphaFactorsSUN', None, [GLshort], 'SUN_global_alpha')

# GL/glext.h:8809
glGlobalAlphaFactoriSUN = _link_function('glGlobalAlphaFactoriSUN', None, [GLint], 'SUN_global_alpha')

# GL/glext.h:8810
glGlobalAlphaFactorfSUN = _link_function('glGlobalAlphaFactorfSUN', None, [GLfloat], 'SUN_global_alpha')

# GL/glext.h:8811
glGlobalAlphaFactordSUN = _link_function('glGlobalAlphaFactordSUN', None, [GLdouble], 'SUN_global_alpha')

# GL/glext.h:8812
glGlobalAlphaFactorubSUN = _link_function('glGlobalAlphaFactorubSUN', None, [GLubyte], 'SUN_global_alpha')

# GL/glext.h:8813
glGlobalAlphaFactorusSUN = _link_function('glGlobalAlphaFactorusSUN', None, [GLushort], 'SUN_global_alpha')

# GL/glext.h:8814
glGlobalAlphaFactoruiSUN = _link_function('glGlobalAlphaFactoruiSUN', None, [GLuint], 'SUN_global_alpha')

PFNGLGLOBALALPHAFACTORBSUNPROC = CFUNCTYPE(None, GLbyte) 	# GL/glext.h:8816
PFNGLGLOBALALPHAFACTORSSUNPROC = CFUNCTYPE(None, GLshort) 	# GL/glext.h:8817
PFNGLGLOBALALPHAFACTORISUNPROC = CFUNCTYPE(None, GLint) 	# GL/glext.h:8818
PFNGLGLOBALALPHAFACTORFSUNPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:8819
PFNGLGLOBALALPHAFACTORDSUNPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:8820
PFNGLGLOBALALPHAFACTORUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:8821
PFNGLGLOBALALPHAFACTORUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:8822
PFNGLGLOBALALPHAFACTORUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:8823
# SUN_triangle_list (GL/glext.h:8826)
GL_SUN_triangle_list = 1 	# GL/glext.h:8827
# GL/glext.h:8829
glReplacementCodeuiSUN = _link_function('glReplacementCodeuiSUN', None, [GLuint], 'SUN_triangle_list')

# GL/glext.h:8830
glReplacementCodeusSUN = _link_function('glReplacementCodeusSUN', None, [GLushort], 'SUN_triangle_list')

# GL/glext.h:8831
glReplacementCodeubSUN = _link_function('glReplacementCodeubSUN', None, [GLubyte], 'SUN_triangle_list')

# GL/glext.h:8832
glReplacementCodeuivSUN = _link_function('glReplacementCodeuivSUN', None, [POINTER(GLuint)], 'SUN_triangle_list')

# GL/glext.h:8833
glReplacementCodeusvSUN = _link_function('glReplacementCodeusvSUN', None, [POINTER(GLushort)], 'SUN_triangle_list')

# GL/glext.h:8834
glReplacementCodeubvSUN = _link_function('glReplacementCodeubvSUN', None, [POINTER(GLubyte)], 'SUN_triangle_list')

# GL/glext.h:8835
glReplacementCodePointerSUN = _link_function('glReplacementCodePointerSUN', None, [GLenum, GLsizei, POINTER(POINTER(GLvoid))], 'SUN_triangle_list')

PFNGLREPLACEMENTCODEUISUNPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:8837
PFNGLREPLACEMENTCODEUSSUNPROC = CFUNCTYPE(None, GLushort) 	# GL/glext.h:8838
PFNGLREPLACEMENTCODEUBSUNPROC = CFUNCTYPE(None, GLubyte) 	# GL/glext.h:8839
PFNGLREPLACEMENTCODEUIVSUNPROC = CFUNCTYPE(None, POINTER(GLuint)) 	# GL/glext.h:8840
PFNGLREPLACEMENTCODEUSVSUNPROC = CFUNCTYPE(None, POINTER(GLushort)) 	# GL/glext.h:8841
PFNGLREPLACEMENTCODEUBVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte)) 	# GL/glext.h:8842
PFNGLREPLACEMENTCODEPOINTERSUNPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(POINTER(GLvoid))) 	# GL/glext.h:8843
# SUN_vertex (GL/glext.h:8846)
GL_SUN_vertex = 1 	# GL/glext.h:8847
# GL/glext.h:8849
glColor4ubVertex2fSUN = _link_function('glColor4ubVertex2fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8850
glColor4ubVertex2fvSUN = _link_function('glColor4ubVertex2fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8851
glColor4ubVertex3fSUN = _link_function('glColor4ubVertex3fSUN', None, [GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8852
glColor4ubVertex3fvSUN = _link_function('glColor4ubVertex3fvSUN', None, [POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8853
glColor3fVertex3fSUN = _link_function('glColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8854
glColor3fVertex3fvSUN = _link_function('glColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8855
glNormal3fVertex3fSUN = _link_function('glNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8856
glNormal3fVertex3fvSUN = _link_function('glNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8857
glColor4fNormal3fVertex3fSUN = _link_function('glColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8858
glColor4fNormal3fVertex3fvSUN = _link_function('glColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8859
glTexCoord2fVertex3fSUN = _link_function('glTexCoord2fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8860
glTexCoord2fVertex3fvSUN = _link_function('glTexCoord2fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8861
glTexCoord4fVertex4fSUN = _link_function('glTexCoord4fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8862
glTexCoord4fVertex4fvSUN = _link_function('glTexCoord4fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8863
glTexCoord2fColor4ubVertex3fSUN = _link_function('glTexCoord2fColor4ubVertex3fSUN', None, [GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8864
glTexCoord2fColor4ubVertex3fvSUN = _link_function('glTexCoord2fColor4ubVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8865
glTexCoord2fColor3fVertex3fSUN = _link_function('glTexCoord2fColor3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8866
glTexCoord2fColor3fVertex3fvSUN = _link_function('glTexCoord2fColor3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8867
glTexCoord2fNormal3fVertex3fSUN = _link_function('glTexCoord2fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8868
glTexCoord2fNormal3fVertex3fvSUN = _link_function('glTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8869
glTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8870
glTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8871
glTexCoord4fColor4fNormal3fVertex4fSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fSUN', None, [GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8872
glTexCoord4fColor4fNormal3fVertex4fvSUN = _link_function('glTexCoord4fColor4fNormal3fVertex4fvSUN', None, [POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8873
glReplacementCodeuiVertex3fSUN = _link_function('glReplacementCodeuiVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8874
glReplacementCodeuiVertex3fvSUN = _link_function('glReplacementCodeuiVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8875
glReplacementCodeuiColor4ubVertex3fSUN = _link_function('glReplacementCodeuiColor4ubVertex3fSUN', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8876
glReplacementCodeuiColor4ubVertex3fvSUN = _link_function('glReplacementCodeuiColor4ubVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8877
glReplacementCodeuiColor3fVertex3fSUN = _link_function('glReplacementCodeuiColor3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8878
glReplacementCodeuiColor3fVertex3fvSUN = _link_function('glReplacementCodeuiColor3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8879
glReplacementCodeuiNormal3fVertex3fSUN = _link_function('glReplacementCodeuiNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8880
glReplacementCodeuiNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8881
glReplacementCodeuiColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8882
glReplacementCodeuiColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8883
glReplacementCodeuiTexCoord2fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8884
glReplacementCodeuiTexCoord2fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8885
glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8886
glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

# GL/glext.h:8887
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat], 'SUN_vertex')

# GL/glext.h:8888
glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN = _link_function('glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN', None, [POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'SUN_vertex')

PFNGLCOLOR4UBVERTEX2FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat) 	# GL/glext.h:8890
PFNGLCOLOR4UBVERTEX2FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:8891
PFNGLCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8892
PFNGLCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:8893
PFNGLCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8894
PFNGLCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8895
PFNGLNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8896
PFNGLNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8897
PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8898
PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8899
PFNGLTEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8900
PFNGLTEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8901
PFNGLTEXCOORD4FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8902
PFNGLTEXCOORD4FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8903
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8904
PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:8905
PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8906
PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8907
PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8908
PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8909
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8910
PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8911
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8912
PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC = CFUNCTYPE(None, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8913
PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8914
PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat)) 	# GL/glext.h:8915
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8916
PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:8917
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8918
PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8919
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8920
PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8921
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8922
PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8923
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8924
PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8925
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8926
PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8927
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:8928
PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC = CFUNCTYPE(None, POINTER(GLuint), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:8929
# EXT_blend_func_separate (GL/glext.h:8932)
GL_EXT_blend_func_separate = 1 	# GL/glext.h:8933
# GL/glext.h:8935
glBlendFuncSeparateEXT = _link_function('glBlendFuncSeparateEXT', None, [GLenum, GLenum, GLenum, GLenum], 'EXT_blend_func_separate')

PFNGLBLENDFUNCSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:8937
# INGR_blend_func_separate (GL/glext.h:8940)
GL_INGR_blend_func_separate = 1 	# GL/glext.h:8941
# GL/glext.h:8943
glBlendFuncSeparateINGR = _link_function('glBlendFuncSeparateINGR', None, [GLenum, GLenum, GLenum, GLenum], 'INGR_blend_func_separate')

PFNGLBLENDFUNCSEPARATEINGRPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:8945
# INGR_color_clamp (GL/glext.h:8948)
GL_INGR_color_clamp = 1 	# GL/glext.h:8949
# INGR_interlace_read (GL/glext.h:8952)
GL_INGR_interlace_read = 1 	# GL/glext.h:8953
# EXT_stencil_wrap (GL/glext.h:8956)
GL_EXT_stencil_wrap = 1 	# GL/glext.h:8957
# EXT_422_pixels (GL/glext.h:8960)
GL_EXT_422_pixels = 1 	# GL/glext.h:8961
# NV_texgen_reflection (GL/glext.h:8964)
GL_NV_texgen_reflection = 1 	# GL/glext.h:8965
# SUN_convolution_border_modes (GL/glext.h:8968)
GL_SUN_convolution_border_modes = 1 	# GL/glext.h:8969
# EXT_texture_env_add (GL/glext.h:8972)
GL_EXT_texture_env_add = 1 	# GL/glext.h:8973
# EXT_texture_lod_bias (GL/glext.h:8976)
GL_EXT_texture_lod_bias = 1 	# GL/glext.h:8977
# EXT_texture_filter_anisotropic (GL/glext.h:8980)
GL_EXT_texture_filter_anisotropic = 1 	# GL/glext.h:8981
# EXT_vertex_weighting (GL/glext.h:8984)
GL_EXT_vertex_weighting = 1 	# GL/glext.h:8985
# GL/glext.h:8987
glVertexWeightfEXT = _link_function('glVertexWeightfEXT', None, [GLfloat], 'EXT_vertex_weighting')

# GL/glext.h:8988
glVertexWeightfvEXT = _link_function('glVertexWeightfvEXT', None, [POINTER(GLfloat)], 'EXT_vertex_weighting')

# GL/glext.h:8989
glVertexWeightPointerEXT = _link_function('glVertexWeightPointerEXT', None, [GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_vertex_weighting')

PFNGLVERTEXWEIGHTFEXTPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:8991
PFNGLVERTEXWEIGHTFVEXTPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:8992
PFNGLVERTEXWEIGHTPOINTEREXTPROC = CFUNCTYPE(None, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:8993
# NV_light_max_exponent (GL/glext.h:8996)
GL_NV_light_max_exponent = 1 	# GL/glext.h:8997
# NV_vertex_array_range (GL/glext.h:9000)
GL_NV_vertex_array_range = 1 	# GL/glext.h:9001
# GL/glext.h:9003
glFlushVertexArrayRangeNV = _link_function('glFlushVertexArrayRangeNV', None, [], 'NV_vertex_array_range')

# GL/glext.h:9004
glVertexArrayRangeNV = _link_function('glVertexArrayRangeNV', None, [GLsizei, POINTER(GLvoid)], 'NV_vertex_array_range')

PFNGLFLUSHVERTEXARRAYRANGENVPROC = CFUNCTYPE(None) 	# GL/glext.h:9006
PFNGLVERTEXARRAYRANGENVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:9007
# NV_register_combiners (GL/glext.h:9010)
GL_NV_register_combiners = 1 	# GL/glext.h:9011
# GL/glext.h:9013
glCombinerParameterfvNV = _link_function('glCombinerParameterfvNV', None, [GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:9014
glCombinerParameterfNV = _link_function('glCombinerParameterfNV', None, [GLenum, GLfloat], 'NV_register_combiners')

# GL/glext.h:9015
glCombinerParameterivNV = _link_function('glCombinerParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:9016
glCombinerParameteriNV = _link_function('glCombinerParameteriNV', None, [GLenum, GLint], 'NV_register_combiners')

# GL/glext.h:9017
glCombinerInputNV = _link_function('glCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:9018
glCombinerOutputNV = _link_function('glCombinerOutputNV', None, [GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean], 'NV_register_combiners')

# GL/glext.h:9019
glFinalCombinerInputNV = _link_function('glFinalCombinerInputNV', None, [GLenum, GLenum, GLenum, GLenum], 'NV_register_combiners')

# GL/glext.h:9020
glGetCombinerInputParameterfvNV = _link_function('glGetCombinerInputParameterfvNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:9021
glGetCombinerInputParameterivNV = _link_function('glGetCombinerInputParameterivNV', None, [GLenum, GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:9022
glGetCombinerOutputParameterfvNV = _link_function('glGetCombinerOutputParameterfvNV', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:9023
glGetCombinerOutputParameterivNV = _link_function('glGetCombinerOutputParameterivNV', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

# GL/glext.h:9024
glGetFinalCombinerInputParameterfvNV = _link_function('glGetFinalCombinerInputParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners')

# GL/glext.h:9025
glGetFinalCombinerInputParameterivNV = _link_function('glGetFinalCombinerInputParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_register_combiners')

PFNGLCOMBINERPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9027
PFNGLCOMBINERPARAMETERFNVPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:9028
PFNGLCOMBINERPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9029
PFNGLCOMBINERPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9030
PFNGLCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:9031
PFNGLCOMBINEROUTPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLenum, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:9032
PFNGLFINALCOMBINERINPUTNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:9033
PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9034
PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:9035
PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9036
PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:9037
PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9038
PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:9039
# NV_fog_distance (GL/glext.h:9042)
GL_NV_fog_distance = 1 	# GL/glext.h:9043
# NV_texgen_emboss (GL/glext.h:9046)
GL_NV_texgen_emboss = 1 	# GL/glext.h:9047
# NV_blend_square (GL/glext.h:9050)
GL_NV_blend_square = 1 	# GL/glext.h:9051
# NV_texture_env_combine4 (GL/glext.h:9054)
GL_NV_texture_env_combine4 = 1 	# GL/glext.h:9055
# MESA_resize_buffers (GL/glext.h:9058)
GL_MESA_resize_buffers = 1 	# GL/glext.h:9059
# GL/glext.h:9061
glResizeBuffersMESA = _link_function('glResizeBuffersMESA', None, [], 'MESA_resize_buffers')

PFNGLRESIZEBUFFERSMESAPROC = CFUNCTYPE(None) 	# GL/glext.h:9063
# MESA_window_pos (GL/glext.h:9066)
GL_MESA_window_pos = 1 	# GL/glext.h:9067
# GL/glext.h:9069
glWindowPos2dMESA = _link_function('glWindowPos2dMESA', None, [GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:9070
glWindowPos2dvMESA = _link_function('glWindowPos2dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:9071
glWindowPos2fMESA = _link_function('glWindowPos2fMESA', None, [GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:9072
glWindowPos2fvMESA = _link_function('glWindowPos2fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:9073
glWindowPos2iMESA = _link_function('glWindowPos2iMESA', None, [GLint, GLint], 'MESA_window_pos')

# GL/glext.h:9074
glWindowPos2ivMESA = _link_function('glWindowPos2ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:9075
glWindowPos2sMESA = _link_function('glWindowPos2sMESA', None, [GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:9076
glWindowPos2svMESA = _link_function('glWindowPos2svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:9077
glWindowPos3dMESA = _link_function('glWindowPos3dMESA', None, [GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:9078
glWindowPos3dvMESA = _link_function('glWindowPos3dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:9079
glWindowPos3fMESA = _link_function('glWindowPos3fMESA', None, [GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:9080
glWindowPos3fvMESA = _link_function('glWindowPos3fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:9081
glWindowPos3iMESA = _link_function('glWindowPos3iMESA', None, [GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:9082
glWindowPos3ivMESA = _link_function('glWindowPos3ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:9083
glWindowPos3sMESA = _link_function('glWindowPos3sMESA', None, [GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:9084
glWindowPos3svMESA = _link_function('glWindowPos3svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

# GL/glext.h:9085
glWindowPos4dMESA = _link_function('glWindowPos4dMESA', None, [GLdouble, GLdouble, GLdouble, GLdouble], 'MESA_window_pos')

# GL/glext.h:9086
glWindowPos4dvMESA = _link_function('glWindowPos4dvMESA', None, [POINTER(GLdouble)], 'MESA_window_pos')

# GL/glext.h:9087
glWindowPos4fMESA = _link_function('glWindowPos4fMESA', None, [GLfloat, GLfloat, GLfloat, GLfloat], 'MESA_window_pos')

# GL/glext.h:9088
glWindowPos4fvMESA = _link_function('glWindowPos4fvMESA', None, [POINTER(GLfloat)], 'MESA_window_pos')

# GL/glext.h:9089
glWindowPos4iMESA = _link_function('glWindowPos4iMESA', None, [GLint, GLint, GLint, GLint], 'MESA_window_pos')

# GL/glext.h:9090
glWindowPos4ivMESA = _link_function('glWindowPos4ivMESA', None, [POINTER(GLint)], 'MESA_window_pos')

# GL/glext.h:9091
glWindowPos4sMESA = _link_function('glWindowPos4sMESA', None, [GLshort, GLshort, GLshort, GLshort], 'MESA_window_pos')

# GL/glext.h:9092
glWindowPos4svMESA = _link_function('glWindowPos4svMESA', None, [POINTER(GLshort)], 'MESA_window_pos')

PFNGLWINDOWPOS2DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:9094
PFNGLWINDOWPOS2DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:9095
PFNGLWINDOWPOS2FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:9096
PFNGLWINDOWPOS2FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:9097
PFNGLWINDOWPOS2IMESAPROC = CFUNCTYPE(None, GLint, GLint) 	# GL/glext.h:9098
PFNGLWINDOWPOS2IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:9099
PFNGLWINDOWPOS2SMESAPROC = CFUNCTYPE(None, GLshort, GLshort) 	# GL/glext.h:9100
PFNGLWINDOWPOS2SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:9101
PFNGLWINDOWPOS3DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9102
PFNGLWINDOWPOS3DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:9103
PFNGLWINDOWPOS3FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9104
PFNGLWINDOWPOS3FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:9105
PFNGLWINDOWPOS3IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint) 	# GL/glext.h:9106
PFNGLWINDOWPOS3IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:9107
PFNGLWINDOWPOS3SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort) 	# GL/glext.h:9108
PFNGLWINDOWPOS3SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:9109
PFNGLWINDOWPOS4DMESAPROC = CFUNCTYPE(None, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9110
PFNGLWINDOWPOS4DVMESAPROC = CFUNCTYPE(None, POINTER(GLdouble)) 	# GL/glext.h:9111
PFNGLWINDOWPOS4FMESAPROC = CFUNCTYPE(None, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9112
PFNGLWINDOWPOS4FVMESAPROC = CFUNCTYPE(None, POINTER(GLfloat)) 	# GL/glext.h:9113
PFNGLWINDOWPOS4IMESAPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:9114
PFNGLWINDOWPOS4IVMESAPROC = CFUNCTYPE(None, POINTER(GLint)) 	# GL/glext.h:9115
PFNGLWINDOWPOS4SMESAPROC = CFUNCTYPE(None, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:9116
PFNGLWINDOWPOS4SVMESAPROC = CFUNCTYPE(None, POINTER(GLshort)) 	# GL/glext.h:9117
# IBM_cull_vertex (GL/glext.h:9120)
GL_IBM_cull_vertex = 1 	# GL/glext.h:9121
# IBM_multimode_draw_arrays (GL/glext.h:9124)
GL_IBM_multimode_draw_arrays = 1 	# GL/glext.h:9125
# GL/glext.h:9127
glMultiModeDrawArraysIBM = _link_function('glMultiModeDrawArraysIBM', None, [POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint], 'IBM_multimode_draw_arrays')

# GL/glext.h:9128
glMultiModeDrawElementsIBM = _link_function('glMultiModeDrawElementsIBM', None, [POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint], 'IBM_multimode_draw_arrays')

PFNGLMULTIMODEDRAWARRAYSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLint), POINTER(GLsizei), GLsizei, GLint) 	# GL/glext.h:9130
PFNGLMULTIMODEDRAWELEMENTSIBMPROC = CFUNCTYPE(None, POINTER(GLenum), POINTER(GLsizei), GLenum, POINTER(POINTER(GLvoid)), GLsizei, GLint) 	# GL/glext.h:9131
# IBM_vertex_array_lists (GL/glext.h:9134)
GL_IBM_vertex_array_lists = 1 	# GL/glext.h:9135
# GL/glext.h:9137
glColorPointerListIBM = _link_function('glColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9138
glSecondaryColorPointerListIBM = _link_function('glSecondaryColorPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9139
glEdgeFlagPointerListIBM = _link_function('glEdgeFlagPointerListIBM', None, [GLint, POINTER(POINTER(GLboolean)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9140
glFogCoordPointerListIBM = _link_function('glFogCoordPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9141
glIndexPointerListIBM = _link_function('glIndexPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9142
glNormalPointerListIBM = _link_function('glNormalPointerListIBM', None, [GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9143
glTexCoordPointerListIBM = _link_function('glTexCoordPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

# GL/glext.h:9144
glVertexPointerListIBM = _link_function('glVertexPointerListIBM', None, [GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint], 'IBM_vertex_array_lists')

PFNGLCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9146
PFNGLSECONDARYCOLORPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9147
PFNGLEDGEFLAGPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, POINTER(POINTER(GLboolean)), GLint) 	# GL/glext.h:9148
PFNGLFOGCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9149
PFNGLINDEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9150
PFNGLNORMALPOINTERLISTIBMPROC = CFUNCTYPE(None, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9151
PFNGLTEXCOORDPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9152
PFNGLVERTEXPOINTERLISTIBMPROC = CFUNCTYPE(None, GLint, GLenum, GLint, POINTER(POINTER(GLvoid)), GLint) 	# GL/glext.h:9153
# SGIX_subsample (GL/glext.h:9156)
GL_SGIX_subsample = 1 	# GL/glext.h:9157
# SGIX_ycrcba (GL/glext.h:9160)
GL_SGIX_ycrcba = 1 	# GL/glext.h:9161
# SGIX_ycrcb_subsample (GL/glext.h:9164)
GL_SGIX_ycrcb_subsample = 1 	# GL/glext.h:9165
# SGIX_depth_pass_instrument (GL/glext.h:9168)
GL_SGIX_depth_pass_instrument = 1 	# GL/glext.h:9169
# 3DFX_texture_compression_FXT1 (GL/glext.h:9172)
GL_3DFX_texture_compression_FXT1 = 1 	# GL/glext.h:9173
# 3DFX_multisample (GL/glext.h:9176)
GL_3DFX_multisample = 1 	# GL/glext.h:9177
# 3DFX_tbuffer (GL/glext.h:9180)
GL_3DFX_tbuffer = 1 	# GL/glext.h:9181
# GL/glext.h:9183
glTbufferMask3DFX = _link_function('glTbufferMask3DFX', None, [GLuint], '3DFX_tbuffer')

PFNGLTBUFFERMASK3DFXPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9185
# EXT_multisample (GL/glext.h:9188)
GL_EXT_multisample = 1 	# GL/glext.h:9189
# GL/glext.h:9191
glSampleMaskEXT = _link_function('glSampleMaskEXT', None, [GLclampf, GLboolean], 'EXT_multisample')

# GL/glext.h:9192
glSamplePatternEXT = _link_function('glSamplePatternEXT', None, [GLenum], 'EXT_multisample')

PFNGLSAMPLEMASKEXTPROC = CFUNCTYPE(None, GLclampf, GLboolean) 	# GL/glext.h:9194
PFNGLSAMPLEPATTERNEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:9195
# SGIX_vertex_preclip (GL/glext.h:9198)
GL_SGIX_vertex_preclip = 1 	# GL/glext.h:9199
# SGIX_convolution_accuracy (GL/glext.h:9202)
GL_SGIX_convolution_accuracy = 1 	# GL/glext.h:9203
# SGIX_resample (GL/glext.h:9206)
GL_SGIX_resample = 1 	# GL/glext.h:9207
# SGIS_point_line_texgen (GL/glext.h:9210)
GL_SGIS_point_line_texgen = 1 	# GL/glext.h:9211
# SGIS_texture_color_mask (GL/glext.h:9214)
GL_SGIS_texture_color_mask = 1 	# GL/glext.h:9215
# GL/glext.h:9217
glTextureColorMaskSGIS = _link_function('glTextureColorMaskSGIS', None, [GLboolean, GLboolean, GLboolean, GLboolean], 'SGIS_texture_color_mask')

PFNGLTEXTURECOLORMASKSGISPROC = CFUNCTYPE(None, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:9219
# SGIX_igloo_interface (GL/glext.h:9222)
GL_SGIX_igloo_interface = 1 	# GL/glext.h:9223
# GL/glext.h:9225
glIglooInterfaceSGIX = _link_function('glIglooInterfaceSGIX', None, [GLenum, POINTER(GLvoid)], 'SGIX_igloo_interface')

PFNGLIGLOOINTERFACESGIXPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:9227
# EXT_texture_env_dot3 (GL/glext.h:9230)
GL_EXT_texture_env_dot3 = 1 	# GL/glext.h:9231
# ATI_texture_mirror_once (GL/glext.h:9234)
GL_ATI_texture_mirror_once = 1 	# GL/glext.h:9235
# NV_fence (GL/glext.h:9238)
GL_NV_fence = 1 	# GL/glext.h:9239
# GL/glext.h:9241
glDeleteFencesNV = _link_function('glDeleteFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:9242
glGenFencesNV = _link_function('glGenFencesNV', None, [GLsizei, POINTER(GLuint)], 'NV_fence')

# GL/glext.h:9243
glIsFenceNV = _link_function('glIsFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:9244
glTestFenceNV = _link_function('glTestFenceNV', GLboolean, [GLuint], 'NV_fence')

# GL/glext.h:9245
glGetFenceivNV = _link_function('glGetFenceivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_fence')

# GL/glext.h:9246
glFinishFenceNV = _link_function('glFinishFenceNV', None, [GLuint], 'NV_fence')

# GL/glext.h:9247
glSetFenceNV = _link_function('glSetFenceNV', None, [GLuint, GLenum], 'NV_fence')

PFNGLDELETEFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9249
PFNGLGENFENCESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9250
PFNGLISFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9251
PFNGLTESTFENCENVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9252
PFNGLGETFENCEIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9253
PFNGLFINISHFENCENVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9254
PFNGLSETFENCENVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:9255
# NV_evaluators (GL/glext.h:9258)
GL_NV_evaluators = 1 	# GL/glext.h:9259
# GL/glext.h:9261
glMapControlPointsNV = _link_function('glMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:9262
glMapParameterivNV = _link_function('glMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:9263
glMapParameterfvNV = _link_function('glMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:9264
glGetMapControlPointsNV = _link_function('glGetMapControlPointsNV', None, [GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)], 'NV_evaluators')

# GL/glext.h:9265
glGetMapParameterivNV = _link_function('glGetMapParameterivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:9266
glGetMapParameterfvNV = _link_function('glGetMapParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:9267
glGetMapAttribParameterivNV = _link_function('glGetMapAttribParameterivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_evaluators')

# GL/glext.h:9268
glGetMapAttribParameterfvNV = _link_function('glGetMapAttribParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_evaluators')

# GL/glext.h:9269
glEvalMapsNV = _link_function('glEvalMapsNV', None, [GLenum, GLenum], 'NV_evaluators')

PFNGLMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:9271
PFNGLMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:9272
PFNGLMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9273
PFNGLGETMAPCONTROLPOINTSNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, POINTER(GLvoid)) 	# GL/glext.h:9274
PFNGLGETMAPPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:9275
PFNGLGETMAPPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9276
PFNGLGETMAPATTRIBPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9277
PFNGLGETMAPATTRIBPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9278
PFNGLEVALMAPSNVPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:9279
# NV_packed_depth_stencil (GL/glext.h:9282)
GL_NV_packed_depth_stencil = 1 	# GL/glext.h:9283
# NV_register_combiners2 (GL/glext.h:9286)
GL_NV_register_combiners2 = 1 	# GL/glext.h:9287
# GL/glext.h:9289
glCombinerStageParameterfvNV = _link_function('glCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

# GL/glext.h:9290
glGetCombinerStageParameterfvNV = _link_function('glGetCombinerStageParameterfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_register_combiners2')

PFNGLCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9292
PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9293
# NV_texture_compression_vtc (GL/glext.h:9296)
GL_NV_texture_compression_vtc = 1 	# GL/glext.h:9297
# NV_texture_rectangle (GL/glext.h:9300)
GL_NV_texture_rectangle = 1 	# GL/glext.h:9301
# NV_texture_shader (GL/glext.h:9304)
GL_NV_texture_shader = 1 	# GL/glext.h:9305
# NV_texture_shader2 (GL/glext.h:9308)
GL_NV_texture_shader2 = 1 	# GL/glext.h:9309
# NV_vertex_array_range2 (GL/glext.h:9312)
GL_NV_vertex_array_range2 = 1 	# GL/glext.h:9313
# NV_vertex_program (GL/glext.h:9316)
GL_NV_vertex_program = 1 	# GL/glext.h:9317
# GL/glext.h:9319
glAreProgramsResidentNV = _link_function('glAreProgramsResidentNV', GLboolean, [GLsizei, POINTER(GLuint), POINTER(GLboolean)], 'NV_vertex_program')

# GL/glext.h:9320
glBindProgramNV = _link_function('glBindProgramNV', None, [GLenum, GLuint], 'NV_vertex_program')

# GL/glext.h:9321
glDeleteProgramsNV = _link_function('glDeleteProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:9322
glExecuteProgramNV = _link_function('glExecuteProgramNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9323
glGenProgramsNV = _link_function('glGenProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:9324
glGetProgramParameterdvNV = _link_function('glGetProgramParameterdvNV', None, [GLenum, GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9325
glGetProgramParameterfvNV = _link_function('glGetProgramParameterfvNV', None, [GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9326
glGetProgramivNV = _link_function('glGetProgramivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:9327
glGetProgramStringNV = _link_function('glGetProgramStringNV', None, [GLuint, GLenum, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:9328
glGetTrackMatrixivNV = _link_function('glGetTrackMatrixivNV', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:9329
glGetVertexAttribdvNV = _link_function('glGetVertexAttribdvNV', None, [GLuint, GLenum, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9330
glGetVertexAttribfvNV = _link_function('glGetVertexAttribfvNV', None, [GLuint, GLenum, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9331
glGetVertexAttribivNV = _link_function('glGetVertexAttribivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program')

# GL/glext.h:9332
glGetVertexAttribPointervNV = _link_function('glGetVertexAttribPointervNV', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'NV_vertex_program')

# GL/glext.h:9333
glIsProgramNV = _link_function('glIsProgramNV', GLboolean, [GLuint], 'NV_vertex_program')

# GL/glext.h:9334
glLoadProgramNV = _link_function('glLoadProgramNV', None, [GLenum, GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:9335
glProgramParameter4dNV = _link_function('glProgramParameter4dNV', None, [GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:9336
glProgramParameter4dvNV = _link_function('glProgramParameter4dvNV', None, [GLenum, GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9337
glProgramParameter4fNV = _link_function('glProgramParameter4fNV', None, [GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:9338
glProgramParameter4fvNV = _link_function('glProgramParameter4fvNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9339
glProgramParameters4dvNV = _link_function('glProgramParameters4dvNV', None, [GLenum, GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9340
glProgramParameters4fvNV = _link_function('glProgramParameters4fvNV', None, [GLenum, GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9341
glRequestResidentProgramsNV = _link_function('glRequestResidentProgramsNV', None, [GLsizei, POINTER(GLuint)], 'NV_vertex_program')

# GL/glext.h:9342
glTrackMatrixNV = _link_function('glTrackMatrixNV', None, [GLenum, GLuint, GLenum, GLenum], 'NV_vertex_program')

# GL/glext.h:9343
glVertexAttribPointerNV = _link_function('glVertexAttribPointerNV', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'NV_vertex_program')

# GL/glext.h:9344
glVertexAttrib1dNV = _link_function('glVertexAttrib1dNV', None, [GLuint, GLdouble], 'NV_vertex_program')

# GL/glext.h:9345
glVertexAttrib1dvNV = _link_function('glVertexAttrib1dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9346
glVertexAttrib1fNV = _link_function('glVertexAttrib1fNV', None, [GLuint, GLfloat], 'NV_vertex_program')

# GL/glext.h:9347
glVertexAttrib1fvNV = _link_function('glVertexAttrib1fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9348
glVertexAttrib1sNV = _link_function('glVertexAttrib1sNV', None, [GLuint, GLshort], 'NV_vertex_program')

# GL/glext.h:9349
glVertexAttrib1svNV = _link_function('glVertexAttrib1svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9350
glVertexAttrib2dNV = _link_function('glVertexAttrib2dNV', None, [GLuint, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:9351
glVertexAttrib2dvNV = _link_function('glVertexAttrib2dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9352
glVertexAttrib2fNV = _link_function('glVertexAttrib2fNV', None, [GLuint, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:9353
glVertexAttrib2fvNV = _link_function('glVertexAttrib2fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9354
glVertexAttrib2sNV = _link_function('glVertexAttrib2sNV', None, [GLuint, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:9355
glVertexAttrib2svNV = _link_function('glVertexAttrib2svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9356
glVertexAttrib3dNV = _link_function('glVertexAttrib3dNV', None, [GLuint, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:9357
glVertexAttrib3dvNV = _link_function('glVertexAttrib3dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9358
glVertexAttrib3fNV = _link_function('glVertexAttrib3fNV', None, [GLuint, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:9359
glVertexAttrib3fvNV = _link_function('glVertexAttrib3fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9360
glVertexAttrib3sNV = _link_function('glVertexAttrib3sNV', None, [GLuint, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:9361
glVertexAttrib3svNV = _link_function('glVertexAttrib3svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9362
glVertexAttrib4dNV = _link_function('glVertexAttrib4dNV', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'NV_vertex_program')

# GL/glext.h:9363
glVertexAttrib4dvNV = _link_function('glVertexAttrib4dvNV', None, [GLuint, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9364
glVertexAttrib4fNV = _link_function('glVertexAttrib4fNV', None, [GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'NV_vertex_program')

# GL/glext.h:9365
glVertexAttrib4fvNV = _link_function('glVertexAttrib4fvNV', None, [GLuint, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9366
glVertexAttrib4sNV = _link_function('glVertexAttrib4sNV', None, [GLuint, GLshort, GLshort, GLshort, GLshort], 'NV_vertex_program')

# GL/glext.h:9367
glVertexAttrib4svNV = _link_function('glVertexAttrib4svNV', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9368
glVertexAttrib4ubNV = _link_function('glVertexAttrib4ubNV', None, [GLuint, GLubyte, GLubyte, GLubyte, GLubyte], 'NV_vertex_program')

# GL/glext.h:9369
glVertexAttrib4ubvNV = _link_function('glVertexAttrib4ubvNV', None, [GLuint, POINTER(GLubyte)], 'NV_vertex_program')

# GL/glext.h:9370
glVertexAttribs1dvNV = _link_function('glVertexAttribs1dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9371
glVertexAttribs1fvNV = _link_function('glVertexAttribs1fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9372
glVertexAttribs1svNV = _link_function('glVertexAttribs1svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9373
glVertexAttribs2dvNV = _link_function('glVertexAttribs2dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9374
glVertexAttribs2fvNV = _link_function('glVertexAttribs2fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9375
glVertexAttribs2svNV = _link_function('glVertexAttribs2svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9376
glVertexAttribs3dvNV = _link_function('glVertexAttribs3dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9377
glVertexAttribs3fvNV = _link_function('glVertexAttribs3fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9378
glVertexAttribs3svNV = _link_function('glVertexAttribs3svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9379
glVertexAttribs4dvNV = _link_function('glVertexAttribs4dvNV', None, [GLuint, GLsizei, POINTER(GLdouble)], 'NV_vertex_program')

# GL/glext.h:9380
glVertexAttribs4fvNV = _link_function('glVertexAttribs4fvNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_vertex_program')

# GL/glext.h:9381
glVertexAttribs4svNV = _link_function('glVertexAttribs4svNV', None, [GLuint, GLsizei, POINTER(GLshort)], 'NV_vertex_program')

# GL/glext.h:9382
glVertexAttribs4ubvNV = _link_function('glVertexAttribs4ubvNV', None, [GLuint, GLsizei, POINTER(GLubyte)], 'NV_vertex_program')

PFNGLAREPROGRAMSRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLsizei, POINTER(GLuint), POINTER(GLboolean)) 	# GL/glext.h:9384
PFNGLBINDPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:9385
PFNGLDELETEPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9386
PFNGLEXECUTEPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9387
PFNGLGENPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9388
PFNGLGETPROGRAMPARAMETERDVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9389
PFNGLGETPROGRAMPARAMETERFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9390
PFNGLGETPROGRAMIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9391
PFNGLGETPROGRAMSTRINGNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLubyte)) 	# GL/glext.h:9392
PFNGLGETTRACKMATRIXIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9393
PFNGLGETVERTEXATTRIBDVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9394
PFNGLGETVERTEXATTRIBFVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9395
PFNGLGETVERTEXATTRIBIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9396
PFNGLGETVERTEXATTRIBPOINTERVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:9397
PFNGLISPROGRAMNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9398
PFNGLLOADPROGRAMNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:9399
PFNGLPROGRAMPARAMETER4DNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9400
PFNGLPROGRAMPARAMETER4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:9401
PFNGLPROGRAMPARAMETER4FNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9402
PFNGLPROGRAMPARAMETER4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9403
PFNGLPROGRAMPARAMETERS4DVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:9404
PFNGLPROGRAMPARAMETERS4FVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:9405
PFNGLREQUESTRESIDENTPROGRAMSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9406
PFNGLTRACKMATRIXNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, GLenum) 	# GL/glext.h:9407
PFNGLVERTEXATTRIBPOINTERNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:9408
PFNGLVERTEXATTRIB1DNVPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:9409
PFNGLVERTEXATTRIB1DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:9410
PFNGLVERTEXATTRIB1FNVPROC = CFUNCTYPE(None, GLuint, GLfloat) 	# GL/glext.h:9411
PFNGLVERTEXATTRIB1FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9412
PFNGLVERTEXATTRIB1SNVPROC = CFUNCTYPE(None, GLuint, GLshort) 	# GL/glext.h:9413
PFNGLVERTEXATTRIB1SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:9414
PFNGLVERTEXATTRIB2DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:9415
PFNGLVERTEXATTRIB2DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:9416
PFNGLVERTEXATTRIB2FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat) 	# GL/glext.h:9417
PFNGLVERTEXATTRIB2FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9418
PFNGLVERTEXATTRIB2SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort) 	# GL/glext.h:9419
PFNGLVERTEXATTRIB2SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:9420
PFNGLVERTEXATTRIB3DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9421
PFNGLVERTEXATTRIB3DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:9422
PFNGLVERTEXATTRIB3FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9423
PFNGLVERTEXATTRIB3FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9424
PFNGLVERTEXATTRIB3SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort) 	# GL/glext.h:9425
PFNGLVERTEXATTRIB3SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:9426
PFNGLVERTEXATTRIB4DNVPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9427
PFNGLVERTEXATTRIB4DVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:9428
PFNGLVERTEXATTRIB4FNVPROC = CFUNCTYPE(None, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9429
PFNGLVERTEXATTRIB4FVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9430
PFNGLVERTEXATTRIB4SNVPROC = CFUNCTYPE(None, GLuint, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:9431
PFNGLVERTEXATTRIB4SVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:9432
PFNGLVERTEXATTRIB4UBNVPROC = CFUNCTYPE(None, GLuint, GLubyte, GLubyte, GLubyte, GLubyte) 	# GL/glext.h:9433
PFNGLVERTEXATTRIB4UBVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:9434
PFNGLVERTEXATTRIBS1DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:9435
PFNGLVERTEXATTRIBS1FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:9436
PFNGLVERTEXATTRIBS1SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:9437
PFNGLVERTEXATTRIBS2DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:9438
PFNGLVERTEXATTRIBS2FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:9439
PFNGLVERTEXATTRIBS2SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:9440
PFNGLVERTEXATTRIBS3DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:9441
PFNGLVERTEXATTRIBS3FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:9442
PFNGLVERTEXATTRIBS3SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:9443
PFNGLVERTEXATTRIBS4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:9444
PFNGLVERTEXATTRIBS4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:9445
PFNGLVERTEXATTRIBS4SVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLshort)) 	# GL/glext.h:9446
PFNGLVERTEXATTRIBS4UBVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte)) 	# GL/glext.h:9447
# SGIX_texture_coordinate_clamp (GL/glext.h:9450)
GL_SGIX_texture_coordinate_clamp = 1 	# GL/glext.h:9451
# SGIX_scalebias_hint (GL/glext.h:9454)
GL_SGIX_scalebias_hint = 1 	# GL/glext.h:9455
# OML_interlace (GL/glext.h:9458)
GL_OML_interlace = 1 	# GL/glext.h:9459
# OML_subsample (GL/glext.h:9462)
GL_OML_subsample = 1 	# GL/glext.h:9463
# OML_resample (GL/glext.h:9466)
GL_OML_resample = 1 	# GL/glext.h:9467
# NV_copy_depth_to_color (GL/glext.h:9470)
GL_NV_copy_depth_to_color = 1 	# GL/glext.h:9471
# ATI_envmap_bumpmap (GL/glext.h:9474)
GL_ATI_envmap_bumpmap = 1 	# GL/glext.h:9475
# GL/glext.h:9477
glTexBumpParameterivATI = _link_function('glTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:9478
glTexBumpParameterfvATI = _link_function('glTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

# GL/glext.h:9479
glGetTexBumpParameterivATI = _link_function('glGetTexBumpParameterivATI', None, [GLenum, POINTER(GLint)], 'ATI_envmap_bumpmap')

# GL/glext.h:9480
glGetTexBumpParameterfvATI = _link_function('glGetTexBumpParameterfvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_envmap_bumpmap')

PFNGLTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9482
PFNGLTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9483
PFNGLGETTEXBUMPPARAMETERIVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9484
PFNGLGETTEXBUMPPARAMETERFVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9485
# ATI_fragment_shader (GL/glext.h:9488)
GL_ATI_fragment_shader = 1 	# GL/glext.h:9489
# GL/glext.h:9491
glGenFragmentShadersATI = _link_function('glGenFragmentShadersATI', GLuint, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:9492
glBindFragmentShaderATI = _link_function('glBindFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:9493
glDeleteFragmentShaderATI = _link_function('glDeleteFragmentShaderATI', None, [GLuint], 'ATI_fragment_shader')

# GL/glext.h:9494
glBeginFragmentShaderATI = _link_function('glBeginFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:9495
glEndFragmentShaderATI = _link_function('glEndFragmentShaderATI', None, [], 'ATI_fragment_shader')

# GL/glext.h:9496
glPassTexCoordATI = _link_function('glPassTexCoordATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:9497
glSampleMapATI = _link_function('glSampleMapATI', None, [GLuint, GLuint, GLenum], 'ATI_fragment_shader')

# GL/glext.h:9498
glColorFragmentOp1ATI = _link_function('glColorFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:9499
glColorFragmentOp2ATI = _link_function('glColorFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:9500
glColorFragmentOp3ATI = _link_function('glColorFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:9501
glAlphaFragmentOp1ATI = _link_function('glAlphaFragmentOp1ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:9502
glAlphaFragmentOp2ATI = _link_function('glAlphaFragmentOp2ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:9503
glAlphaFragmentOp3ATI = _link_function('glAlphaFragmentOp3ATI', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint], 'ATI_fragment_shader')

# GL/glext.h:9504
glSetFragmentShaderConstantATI = _link_function('glSetFragmentShaderConstantATI', None, [GLuint, POINTER(GLfloat)], 'ATI_fragment_shader')

PFNGLGENFRAGMENTSHADERSATIPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:9506
PFNGLBINDFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9507
PFNGLDELETEFRAGMENTSHADERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9508
PFNGLBEGINFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:9509
PFNGLENDFRAGMENTSHADERATIPROC = CFUNCTYPE(None) 	# GL/glext.h:9510
PFNGLPASSTEXCOORDATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:9511
PFNGLSAMPLEMAPATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum) 	# GL/glext.h:9512
PFNGLCOLORFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9513
PFNGLCOLORFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9514
PFNGLCOLORFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9515
PFNGLALPHAFRAGMENTOP1ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9516
PFNGLALPHAFRAGMENTOP2ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9517
PFNGLALPHAFRAGMENTOP3ATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9518
PFNGLSETFRAGMENTSHADERCONSTANTATIPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9519
# ATI_pn_triangles (GL/glext.h:9522)
GL_ATI_pn_triangles = 1 	# GL/glext.h:9523
# GL/glext.h:9525
glPNTrianglesiATI = _link_function('glPNTrianglesiATI', None, [GLenum, GLint], 'ATI_pn_triangles')

# GL/glext.h:9526
glPNTrianglesfATI = _link_function('glPNTrianglesfATI', None, [GLenum, GLfloat], 'ATI_pn_triangles')

PFNGLPNTRIANGLESIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9528
PFNGLPNTRIANGLESFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:9529
# ATI_vertex_array_object (GL/glext.h:9532)
GL_ATI_vertex_array_object = 1 	# GL/glext.h:9533
# GL/glext.h:9535
glNewObjectBufferATI = _link_function('glNewObjectBufferATI', GLuint, [GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:9536
glIsObjectBufferATI = _link_function('glIsObjectBufferATI', GLboolean, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:9537
glUpdateObjectBufferATI = _link_function('glUpdateObjectBufferATI', None, [GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum], 'ATI_vertex_array_object')

# GL/glext.h:9538
glGetObjectBufferfvATI = _link_function('glGetObjectBufferfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:9539
glGetObjectBufferivATI = _link_function('glGetObjectBufferivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:9540
glFreeObjectBufferATI = _link_function('glFreeObjectBufferATI', None, [GLuint], 'ATI_vertex_array_object')

# GL/glext.h:9541
glArrayObjectATI = _link_function('glArrayObjectATI', None, [GLenum, GLint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:9542
glGetArrayObjectfvATI = _link_function('glGetArrayObjectfvATI', None, [GLenum, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:9543
glGetArrayObjectivATI = _link_function('glGetArrayObjectivATI', None, [GLenum, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

# GL/glext.h:9544
glVariantArrayObjectATI = _link_function('glVariantArrayObjectATI', None, [GLuint, GLenum, GLsizei, GLuint, GLuint], 'ATI_vertex_array_object')

# GL/glext.h:9545
glGetVariantArrayObjectfvATI = _link_function('glGetVariantArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_array_object')

# GL/glext.h:9546
glGetVariantArrayObjectivATI = _link_function('glGetVariantArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_array_object')

PFNGLNEWOBJECTBUFFERATIPROC = CFUNCTYPE(GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:9548
PFNGLISOBJECTBUFFERATIPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9549
PFNGLUPDATEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLvoid), GLenum) 	# GL/glext.h:9550
PFNGLGETOBJECTBUFFERFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9551
PFNGLGETOBJECTBUFFERIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9552
PFNGLFREEOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9553
PFNGLARRAYOBJECTATIPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:9554
PFNGLGETARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9555
PFNGLGETARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:9556
PFNGLVARIANTARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLuint, GLuint) 	# GL/glext.h:9557
PFNGLGETVARIANTARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9558
PFNGLGETVARIANTARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9559
# EXT_vertex_shader (GL/glext.h:9562)
GL_EXT_vertex_shader = 1 	# GL/glext.h:9563
# GL/glext.h:9565
glBeginVertexShaderEXT = _link_function('glBeginVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:9566
glEndVertexShaderEXT = _link_function('glEndVertexShaderEXT', None, [], 'EXT_vertex_shader')

# GL/glext.h:9567
glBindVertexShaderEXT = _link_function('glBindVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:9568
glGenVertexShadersEXT = _link_function('glGenVertexShadersEXT', GLuint, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:9569
glDeleteVertexShaderEXT = _link_function('glDeleteVertexShaderEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:9570
glShaderOp1EXT = _link_function('glShaderOp1EXT', None, [GLenum, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:9571
glShaderOp2EXT = _link_function('glShaderOp2EXT', None, [GLenum, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:9572
glShaderOp3EXT = _link_function('glShaderOp3EXT', None, [GLenum, GLuint, GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:9573
glSwizzleEXT = _link_function('glSwizzleEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9574
glWriteMaskEXT = _link_function('glWriteMaskEXT', None, [GLuint, GLuint, GLenum, GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9575
glInsertComponentEXT = _link_function('glInsertComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:9576
glExtractComponentEXT = _link_function('glExtractComponentEXT', None, [GLuint, GLuint, GLuint], 'EXT_vertex_shader')

# GL/glext.h:9577
glGenSymbolsEXT = _link_function('glGenSymbolsEXT', GLuint, [GLenum, GLenum, GLenum, GLuint], 'EXT_vertex_shader')

# GL/glext.h:9578
glSetInvariantEXT = _link_function('glSetInvariantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:9579
glSetLocalConstantEXT = _link_function('glSetLocalConstantEXT', None, [GLuint, GLenum, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:9580
glVariantbvEXT = _link_function('glVariantbvEXT', None, [GLuint, POINTER(GLbyte)], 'EXT_vertex_shader')

# GL/glext.h:9581
glVariantsvEXT = _link_function('glVariantsvEXT', None, [GLuint, POINTER(GLshort)], 'EXT_vertex_shader')

# GL/glext.h:9582
glVariantivEXT = _link_function('glVariantivEXT', None, [GLuint, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:9583
glVariantfvEXT = _link_function('glVariantfvEXT', None, [GLuint, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:9584
glVariantdvEXT = _link_function('glVariantdvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_shader')

# GL/glext.h:9585
glVariantubvEXT = _link_function('glVariantubvEXT', None, [GLuint, POINTER(GLubyte)], 'EXT_vertex_shader')

# GL/glext.h:9586
glVariantusvEXT = _link_function('glVariantusvEXT', None, [GLuint, POINTER(GLushort)], 'EXT_vertex_shader')

# GL/glext.h:9587
glVariantuivEXT = _link_function('glVariantuivEXT', None, [GLuint, POINTER(GLuint)], 'EXT_vertex_shader')

# GL/glext.h:9588
glVariantPointerEXT = _link_function('glVariantPointerEXT', None, [GLuint, GLenum, GLuint, POINTER(GLvoid)], 'EXT_vertex_shader')

# GL/glext.h:9589
glEnableVariantClientStateEXT = _link_function('glEnableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:9590
glDisableVariantClientStateEXT = _link_function('glDisableVariantClientStateEXT', None, [GLuint], 'EXT_vertex_shader')

# GL/glext.h:9591
glBindLightParameterEXT = _link_function('glBindLightParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9592
glBindMaterialParameterEXT = _link_function('glBindMaterialParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9593
glBindTexGenParameterEXT = _link_function('glBindTexGenParameterEXT', GLuint, [GLenum, GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9594
glBindTextureUnitParameterEXT = _link_function('glBindTextureUnitParameterEXT', GLuint, [GLenum, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9595
glBindParameterEXT = _link_function('glBindParameterEXT', GLuint, [GLenum], 'EXT_vertex_shader')

# GL/glext.h:9596
glIsVariantEnabledEXT = _link_function('glIsVariantEnabledEXT', GLboolean, [GLuint, GLenum], 'EXT_vertex_shader')

# GL/glext.h:9597
glGetVariantBooleanvEXT = _link_function('glGetVariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:9598
glGetVariantIntegervEXT = _link_function('glGetVariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:9599
glGetVariantFloatvEXT = _link_function('glGetVariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:9600
glGetVariantPointervEXT = _link_function('glGetVariantPointervEXT', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'EXT_vertex_shader')

# GL/glext.h:9601
glGetInvariantBooleanvEXT = _link_function('glGetInvariantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:9602
glGetInvariantIntegervEXT = _link_function('glGetInvariantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:9603
glGetInvariantFloatvEXT = _link_function('glGetInvariantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

# GL/glext.h:9604
glGetLocalConstantBooleanvEXT = _link_function('glGetLocalConstantBooleanvEXT', None, [GLuint, GLenum, POINTER(GLboolean)], 'EXT_vertex_shader')

# GL/glext.h:9605
glGetLocalConstantIntegervEXT = _link_function('glGetLocalConstantIntegervEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_vertex_shader')

# GL/glext.h:9606
glGetLocalConstantFloatvEXT = _link_function('glGetLocalConstantFloatvEXT', None, [GLuint, GLenum, POINTER(GLfloat)], 'EXT_vertex_shader')

PFNGLBEGINVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:9608
PFNGLENDVERTEXSHADEREXTPROC = CFUNCTYPE(None) 	# GL/glext.h:9609
PFNGLBINDVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9610
PFNGLGENVERTEXSHADERSEXTPROC = CFUNCTYPE(GLuint, GLuint) 	# GL/glext.h:9611
PFNGLDELETEVERTEXSHADEREXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9612
PFNGLSHADEROP1EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:9613
PFNGLSHADEROP2EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint) 	# GL/glext.h:9614
PFNGLSHADEROP3EXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:9615
PFNGLSWIZZLEEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:9616
PFNGLWRITEMASKEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:9617
PFNGLINSERTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:9618
PFNGLEXTRACTCOMPONENTEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:9619
PFNGLGENSYMBOLSEXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:9620
PFNGLSETINVARIANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:9621
PFNGLSETLOCALCONSTANTEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:9622
PFNGLVARIANTBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:9623
PFNGLVARIANTSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:9624
PFNGLVARIANTIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:9625
PFNGLVARIANTFVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:9626
PFNGLVARIANTDVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:9627
PFNGLVARIANTUBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:9628
PFNGLVARIANTUSVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:9629
PFNGLVARIANTUIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:9630
PFNGLVARIANTPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLvoid)) 	# GL/glext.h:9631
PFNGLENABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9632
PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9633
PFNGLBINDLIGHTPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:9634
PFNGLBINDMATERIALPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:9635
PFNGLBINDTEXGENPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum, GLenum) 	# GL/glext.h:9636
PFNGLBINDTEXTUREUNITPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum, GLenum) 	# GL/glext.h:9637
PFNGLBINDPARAMETEREXTPROC = CFUNCTYPE(GLuint, GLenum) 	# GL/glext.h:9638
PFNGLISVARIANTENABLEDEXTPROC = CFUNCTYPE(GLboolean, GLuint, GLenum) 	# GL/glext.h:9639
PFNGLGETVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:9640
PFNGLGETVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9641
PFNGLGETVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9642
PFNGLGETVARIANTPOINTERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:9643
PFNGLGETINVARIANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:9644
PFNGLGETINVARIANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9645
PFNGLGETINVARIANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9646
PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLboolean)) 	# GL/glext.h:9647
PFNGLGETLOCALCONSTANTINTEGERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9648
PFNGLGETLOCALCONSTANTFLOATVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9649
# ATI_vertex_streams (GL/glext.h:9652)
GL_ATI_vertex_streams = 1 	# GL/glext.h:9653
# GL/glext.h:9655
glVertexStream1sATI = _link_function('glVertexStream1sATI', None, [GLenum, GLshort], 'ATI_vertex_streams')

# GL/glext.h:9656
glVertexStream1svATI = _link_function('glVertexStream1svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:9657
glVertexStream1iATI = _link_function('glVertexStream1iATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:9658
glVertexStream1ivATI = _link_function('glVertexStream1ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:9659
glVertexStream1fATI = _link_function('glVertexStream1fATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:9660
glVertexStream1fvATI = _link_function('glVertexStream1fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:9661
glVertexStream1dATI = _link_function('glVertexStream1dATI', None, [GLenum, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:9662
glVertexStream1dvATI = _link_function('glVertexStream1dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:9663
glVertexStream2sATI = _link_function('glVertexStream2sATI', None, [GLenum, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:9664
glVertexStream2svATI = _link_function('glVertexStream2svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:9665
glVertexStream2iATI = _link_function('glVertexStream2iATI', None, [GLenum, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:9666
glVertexStream2ivATI = _link_function('glVertexStream2ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:9667
glVertexStream2fATI = _link_function('glVertexStream2fATI', None, [GLenum, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:9668
glVertexStream2fvATI = _link_function('glVertexStream2fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:9669
glVertexStream2dATI = _link_function('glVertexStream2dATI', None, [GLenum, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:9670
glVertexStream2dvATI = _link_function('glVertexStream2dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:9671
glVertexStream3sATI = _link_function('glVertexStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:9672
glVertexStream3svATI = _link_function('glVertexStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:9673
glVertexStream3iATI = _link_function('glVertexStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:9674
glVertexStream3ivATI = _link_function('glVertexStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:9675
glVertexStream3fATI = _link_function('glVertexStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:9676
glVertexStream3fvATI = _link_function('glVertexStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:9677
glVertexStream3dATI = _link_function('glVertexStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:9678
glVertexStream3dvATI = _link_function('glVertexStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:9679
glVertexStream4sATI = _link_function('glVertexStream4sATI', None, [GLenum, GLshort, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:9680
glVertexStream4svATI = _link_function('glVertexStream4svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:9681
glVertexStream4iATI = _link_function('glVertexStream4iATI', None, [GLenum, GLint, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:9682
glVertexStream4ivATI = _link_function('glVertexStream4ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:9683
glVertexStream4fATI = _link_function('glVertexStream4fATI', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:9684
glVertexStream4fvATI = _link_function('glVertexStream4fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:9685
glVertexStream4dATI = _link_function('glVertexStream4dATI', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:9686
glVertexStream4dvATI = _link_function('glVertexStream4dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:9687
glNormalStream3bATI = _link_function('glNormalStream3bATI', None, [GLenum, GLbyte, GLbyte, GLbyte], 'ATI_vertex_streams')

# GL/glext.h:9688
glNormalStream3bvATI = _link_function('glNormalStream3bvATI', None, [GLenum, POINTER(GLbyte)], 'ATI_vertex_streams')

# GL/glext.h:9689
glNormalStream3sATI = _link_function('glNormalStream3sATI', None, [GLenum, GLshort, GLshort, GLshort], 'ATI_vertex_streams')

# GL/glext.h:9690
glNormalStream3svATI = _link_function('glNormalStream3svATI', None, [GLenum, POINTER(GLshort)], 'ATI_vertex_streams')

# GL/glext.h:9691
glNormalStream3iATI = _link_function('glNormalStream3iATI', None, [GLenum, GLint, GLint, GLint], 'ATI_vertex_streams')

# GL/glext.h:9692
glNormalStream3ivATI = _link_function('glNormalStream3ivATI', None, [GLenum, POINTER(GLint)], 'ATI_vertex_streams')

# GL/glext.h:9693
glNormalStream3fATI = _link_function('glNormalStream3fATI', None, [GLenum, GLfloat, GLfloat, GLfloat], 'ATI_vertex_streams')

# GL/glext.h:9694
glNormalStream3fvATI = _link_function('glNormalStream3fvATI', None, [GLenum, POINTER(GLfloat)], 'ATI_vertex_streams')

# GL/glext.h:9695
glNormalStream3dATI = _link_function('glNormalStream3dATI', None, [GLenum, GLdouble, GLdouble, GLdouble], 'ATI_vertex_streams')

# GL/glext.h:9696
glNormalStream3dvATI = _link_function('glNormalStream3dvATI', None, [GLenum, POINTER(GLdouble)], 'ATI_vertex_streams')

# GL/glext.h:9697
glClientActiveVertexStreamATI = _link_function('glClientActiveVertexStreamATI', None, [GLenum], 'ATI_vertex_streams')

# GL/glext.h:9698
glVertexBlendEnviATI = _link_function('glVertexBlendEnviATI', None, [GLenum, GLint], 'ATI_vertex_streams')

# GL/glext.h:9699
glVertexBlendEnvfATI = _link_function('glVertexBlendEnvfATI', None, [GLenum, GLfloat], 'ATI_vertex_streams')

PFNGLVERTEXSTREAM1SATIPROC = CFUNCTYPE(None, GLenum, GLshort) 	# GL/glext.h:9701
PFNGLVERTEXSTREAM1SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:9702
PFNGLVERTEXSTREAM1IATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9703
PFNGLVERTEXSTREAM1IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9704
PFNGLVERTEXSTREAM1FATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:9705
PFNGLVERTEXSTREAM1FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9706
PFNGLVERTEXSTREAM1DATIPROC = CFUNCTYPE(None, GLenum, GLdouble) 	# GL/glext.h:9707
PFNGLVERTEXSTREAM1DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9708
PFNGLVERTEXSTREAM2SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort) 	# GL/glext.h:9709
PFNGLVERTEXSTREAM2SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:9710
PFNGLVERTEXSTREAM2IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint) 	# GL/glext.h:9711
PFNGLVERTEXSTREAM2IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9712
PFNGLVERTEXSTREAM2FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat) 	# GL/glext.h:9713
PFNGLVERTEXSTREAM2FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9714
PFNGLVERTEXSTREAM2DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble) 	# GL/glext.h:9715
PFNGLVERTEXSTREAM2DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9716
PFNGLVERTEXSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:9717
PFNGLVERTEXSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:9718
PFNGLVERTEXSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:9719
PFNGLVERTEXSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9720
PFNGLVERTEXSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9721
PFNGLVERTEXSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9722
PFNGLVERTEXSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9723
PFNGLVERTEXSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9724
PFNGLVERTEXSTREAM4SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort, GLshort) 	# GL/glext.h:9725
PFNGLVERTEXSTREAM4SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:9726
PFNGLVERTEXSTREAM4IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint, GLint) 	# GL/glext.h:9727
PFNGLVERTEXSTREAM4IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9728
PFNGLVERTEXSTREAM4FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9729
PFNGLVERTEXSTREAM4FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9730
PFNGLVERTEXSTREAM4DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9731
PFNGLVERTEXSTREAM4DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9732
PFNGLNORMALSTREAM3BATIPROC = CFUNCTYPE(None, GLenum, GLbyte, GLbyte, GLbyte) 	# GL/glext.h:9733
PFNGLNORMALSTREAM3BVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLbyte)) 	# GL/glext.h:9734
PFNGLNORMALSTREAM3SATIPROC = CFUNCTYPE(None, GLenum, GLshort, GLshort, GLshort) 	# GL/glext.h:9735
PFNGLNORMALSTREAM3SVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLshort)) 	# GL/glext.h:9736
PFNGLNORMALSTREAM3IATIPROC = CFUNCTYPE(None, GLenum, GLint, GLint, GLint) 	# GL/glext.h:9737
PFNGLNORMALSTREAM3IVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9738
PFNGLNORMALSTREAM3FATIPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9739
PFNGLNORMALSTREAM3FVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:9740
PFNGLNORMALSTREAM3DATIPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9741
PFNGLNORMALSTREAM3DVATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:9742
PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:9743
PFNGLVERTEXBLENDENVIATIPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9744
PFNGLVERTEXBLENDENVFATIPROC = CFUNCTYPE(None, GLenum, GLfloat) 	# GL/glext.h:9745
# ATI_element_array (GL/glext.h:9748)
GL_ATI_element_array = 1 	# GL/glext.h:9749
# GL/glext.h:9751
glElementPointerATI = _link_function('glElementPointerATI', None, [GLenum, POINTER(GLvoid)], 'ATI_element_array')

# GL/glext.h:9752
glDrawElementArrayATI = _link_function('glDrawElementArrayATI', None, [GLenum, GLsizei], 'ATI_element_array')

# GL/glext.h:9753
glDrawRangeElementArrayATI = _link_function('glDrawRangeElementArrayATI', None, [GLenum, GLuint, GLuint, GLsizei], 'ATI_element_array')

PFNGLELEMENTPOINTERATIPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:9755
PFNGLDRAWELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:9756
PFNGLDRAWRANGEELEMENTARRAYATIPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei) 	# GL/glext.h:9757
# SUN_mesh_array (GL/glext.h:9760)
GL_SUN_mesh_array = 1 	# GL/glext.h:9761
# GL/glext.h:9763
glDrawMeshArraysSUN = _link_function('glDrawMeshArraysSUN', None, [GLenum, GLint, GLsizei, GLsizei], 'SUN_mesh_array')

PFNGLDRAWMESHARRAYSSUNPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:9765
# SUN_slice_accum (GL/glext.h:9768)
GL_SUN_slice_accum = 1 	# GL/glext.h:9769
# NV_multisample_filter_hint (GL/glext.h:9772)
GL_NV_multisample_filter_hint = 1 	# GL/glext.h:9773
# NV_depth_clamp (GL/glext.h:9776)
GL_NV_depth_clamp = 1 	# GL/glext.h:9777
# NV_occlusion_query (GL/glext.h:9780)
GL_NV_occlusion_query = 1 	# GL/glext.h:9781
# GL/glext.h:9783
glGenOcclusionQueriesNV = _link_function('glGenOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:9784
glDeleteOcclusionQueriesNV = _link_function('glDeleteOcclusionQueriesNV', None, [GLsizei, POINTER(GLuint)], 'NV_occlusion_query')

# GL/glext.h:9785
glIsOcclusionQueryNV = _link_function('glIsOcclusionQueryNV', GLboolean, [GLuint], 'NV_occlusion_query')

# GL/glext.h:9786
glBeginOcclusionQueryNV = _link_function('glBeginOcclusionQueryNV', None, [GLuint], 'NV_occlusion_query')

# GL/glext.h:9787
glEndOcclusionQueryNV = _link_function('glEndOcclusionQueryNV', None, [], 'NV_occlusion_query')

# GL/glext.h:9788
glGetOcclusionQueryivNV = _link_function('glGetOcclusionQueryivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_occlusion_query')

# GL/glext.h:9789
glGetOcclusionQueryuivNV = _link_function('glGetOcclusionQueryuivNV', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_occlusion_query')

PFNGLGENOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9791
PFNGLDELETEOCCLUSIONQUERIESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9792
PFNGLISOCCLUSIONQUERYNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9793
PFNGLBEGINOCCLUSIONQUERYNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9794
PFNGLENDOCCLUSIONQUERYNVPROC = CFUNCTYPE(None) 	# GL/glext.h:9795
PFNGLGETOCCLUSIONQUERYIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:9796
PFNGLGETOCCLUSIONQUERYUIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:9797
# NV_point_sprite (GL/glext.h:9800)
GL_NV_point_sprite = 1 	# GL/glext.h:9801
# GL/glext.h:9803
glPointParameteriNV = _link_function('glPointParameteriNV', None, [GLenum, GLint], 'NV_point_sprite')

# GL/glext.h:9804
glPointParameterivNV = _link_function('glPointParameterivNV', None, [GLenum, POINTER(GLint)], 'NV_point_sprite')

PFNGLPOINTPARAMETERINVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9806
PFNGLPOINTPARAMETERIVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLint)) 	# GL/glext.h:9807
# NV_texture_shader3 (GL/glext.h:9810)
GL_NV_texture_shader3 = 1 	# GL/glext.h:9811
# NV_vertex_program1_1 (GL/glext.h:9814)
GL_NV_vertex_program1_1 = 1 	# GL/glext.h:9815
# EXT_shadow_funcs (GL/glext.h:9818)
GL_EXT_shadow_funcs = 1 	# GL/glext.h:9819
# EXT_stencil_two_side (GL/glext.h:9822)
GL_EXT_stencil_two_side = 1 	# GL/glext.h:9823
# GL/glext.h:9825
glActiveStencilFaceEXT = _link_function('glActiveStencilFaceEXT', None, [GLenum], 'EXT_stencil_two_side')

PFNGLACTIVESTENCILFACEEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:9827
# ATI_text_fragment_shader (GL/glext.h:9830)
GL_ATI_text_fragment_shader = 1 	# GL/glext.h:9831
# APPLE_client_storage (GL/glext.h:9834)
GL_APPLE_client_storage = 1 	# GL/glext.h:9835
# APPLE_element_array (GL/glext.h:9838)
GL_APPLE_element_array = 1 	# GL/glext.h:9839
# GL/glext.h:9841
glElementPointerAPPLE = _link_function('glElementPointerAPPLE', None, [GLenum, POINTER(GLvoid)], 'APPLE_element_array')

# GL/glext.h:9842
glDrawElementArrayAPPLE = _link_function('glDrawElementArrayAPPLE', None, [GLenum, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:9843
glDrawRangeElementArrayAPPLE = _link_function('glDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, GLint, GLsizei], 'APPLE_element_array')

# GL/glext.h:9844
glMultiDrawElementArrayAPPLE = _link_function('glMultiDrawElementArrayAPPLE', None, [GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

# GL/glext.h:9845
glMultiDrawRangeElementArrayAPPLE = _link_function('glMultiDrawRangeElementArrayAPPLE', None, [GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei], 'APPLE_element_array')

PFNGLELEMENTPOINTERAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid)) 	# GL/glext.h:9847
PFNGLDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei) 	# GL/glext.h:9848
PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLint, GLsizei) 	# GL/glext.h:9849
PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:9850
PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, POINTER(GLint), POINTER(GLsizei), GLsizei) 	# GL/glext.h:9851
# APPLE_fence (GL/glext.h:9854)
GL_APPLE_fence = 1 	# GL/glext.h:9855
# GL/glext.h:9857
glGenFencesAPPLE = _link_function('glGenFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:9858
glDeleteFencesAPPLE = _link_function('glDeleteFencesAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_fence')

# GL/glext.h:9859
glSetFenceAPPLE = _link_function('glSetFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:9860
glIsFenceAPPLE = _link_function('glIsFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:9861
glTestFenceAPPLE = _link_function('glTestFenceAPPLE', GLboolean, [GLuint], 'APPLE_fence')

# GL/glext.h:9862
glFinishFenceAPPLE = _link_function('glFinishFenceAPPLE', None, [GLuint], 'APPLE_fence')

# GL/glext.h:9863
glTestObjectAPPLE = _link_function('glTestObjectAPPLE', GLboolean, [GLenum, GLuint], 'APPLE_fence')

# GL/glext.h:9864
glFinishObjectAPPLE = _link_function('glFinishObjectAPPLE', None, [GLenum, GLint], 'APPLE_fence')

PFNGLGENFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9866
PFNGLDELETEFENCESAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9867
PFNGLSETFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9868
PFNGLISFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9869
PFNGLTESTFENCEAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9870
PFNGLFINISHFENCEAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9871
PFNGLTESTOBJECTAPPLEPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:9872
PFNGLFINISHOBJECTAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9873
# APPLE_vertex_array_object (GL/glext.h:9876)
GL_APPLE_vertex_array_object = 1 	# GL/glext.h:9877
# GL/glext.h:9879
glBindVertexArrayAPPLE = _link_function('glBindVertexArrayAPPLE', None, [GLuint], 'APPLE_vertex_array_object')

# GL/glext.h:9880
glDeleteVertexArraysAPPLE = _link_function('glDeleteVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:9881
glGenVertexArraysAPPLE = _link_function('glGenVertexArraysAPPLE', None, [GLsizei, POINTER(GLuint)], 'APPLE_vertex_array_object')

# GL/glext.h:9882
glIsVertexArrayAPPLE = _link_function('glIsVertexArrayAPPLE', GLboolean, [GLuint], 'APPLE_vertex_array_object')

PFNGLBINDVERTEXARRAYAPPLEPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:9884
PFNGLDELETEVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9885
PFNGLGENVERTEXARRAYSAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:9886
PFNGLISVERTEXARRAYAPPLEPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:9887
# APPLE_vertex_array_range (GL/glext.h:9890)
GL_APPLE_vertex_array_range = 1 	# GL/glext.h:9891
# GL/glext.h:9893
glVertexArrayRangeAPPLE = _link_function('glVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:9894
glFlushVertexArrayRangeAPPLE = _link_function('glFlushVertexArrayRangeAPPLE', None, [GLsizei, POINTER(GLvoid)], 'APPLE_vertex_array_range')

# GL/glext.h:9895
glVertexArrayParameteriAPPLE = _link_function('glVertexArrayParameteriAPPLE', None, [GLenum, GLint], 'APPLE_vertex_array_range')

PFNGLVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:9897
PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:9898
PFNGLVERTEXARRAYPARAMETERIAPPLEPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:9899
# APPLE_ycbcr_422 (GL/glext.h:9902)
GL_APPLE_ycbcr_422 = 1 	# GL/glext.h:9903
# S3_s3tc (GL/glext.h:9906)
GL_S3_s3tc = 1 	# GL/glext.h:9907
# ATI_draw_buffers (GL/glext.h:9910)
GL_ATI_draw_buffers = 1 	# GL/glext.h:9911
# GL/glext.h:9913
glDrawBuffersATI = _link_function('glDrawBuffersATI', None, [GLsizei, POINTER(GLenum)], 'ATI_draw_buffers')

PFNGLDRAWBUFFERSATIPROC = CFUNCTYPE(None, GLsizei, POINTER(GLenum)) 	# GL/glext.h:9915
# ATI_pixel_format_float (GL/glext.h:9918)
GL_ATI_pixel_format_float = 1 	# GL/glext.h:9919
# ATI_texture_env_combine3 (GL/glext.h:9925)
GL_ATI_texture_env_combine3 = 1 	# GL/glext.h:9926
# ATI_texture_float (GL/glext.h:9929)
GL_ATI_texture_float = 1 	# GL/glext.h:9930
# NV_float_buffer (GL/glext.h:9933)
GL_NV_float_buffer = 1 	# GL/glext.h:9934
# NV_fragment_program (GL/glext.h:9937)
GL_NV_fragment_program = 1 	# GL/glext.h:9938
# GL/glext.h:9941
glProgramNamedParameter4fNV = _link_function('glProgramNamedParameter4fNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat], 'NV_fragment_program')

# GL/glext.h:9942
glProgramNamedParameter4dNV = _link_function('glProgramNamedParameter4dNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble], 'NV_fragment_program')

# GL/glext.h:9943
glProgramNamedParameter4fvNV = _link_function('glProgramNamedParameter4fvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:9944
glProgramNamedParameter4dvNV = _link_function('glProgramNamedParameter4dvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

# GL/glext.h:9945
glGetProgramNamedParameterfvNV = _link_function('glGetProgramNamedParameterfvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)], 'NV_fragment_program')

# GL/glext.h:9946
glGetProgramNamedParameterdvNV = _link_function('glGetProgramNamedParameterdvNV', None, [GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)], 'NV_fragment_program')

PFNGLPROGRAMNAMEDPARAMETER4FNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:9948
PFNGLPROGRAMNAMEDPARAMETER4DNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:9949
PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:9950
PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:9951
PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLfloat)) 	# GL/glext.h:9952
PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), POINTER(GLdouble)) 	# GL/glext.h:9953
# NV_half_float (GL/glext.h:9956)
GL_NV_half_float = 1 	# GL/glext.h:9957
# GL/glext.h:9959
glVertex2hNV = _link_function('glVertex2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9960
glVertex2hvNV = _link_function('glVertex2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9961
glVertex3hNV = _link_function('glVertex3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9962
glVertex3hvNV = _link_function('glVertex3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9963
glVertex4hNV = _link_function('glVertex4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9964
glVertex4hvNV = _link_function('glVertex4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9965
glNormal3hNV = _link_function('glNormal3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9966
glNormal3hvNV = _link_function('glNormal3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9967
glColor3hNV = _link_function('glColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9968
glColor3hvNV = _link_function('glColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9969
glColor4hNV = _link_function('glColor4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9970
glColor4hvNV = _link_function('glColor4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9971
glTexCoord1hNV = _link_function('glTexCoord1hNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:9972
glTexCoord1hvNV = _link_function('glTexCoord1hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9973
glTexCoord2hNV = _link_function('glTexCoord2hNV', None, [GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9974
glTexCoord2hvNV = _link_function('glTexCoord2hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9975
glTexCoord3hNV = _link_function('glTexCoord3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9976
glTexCoord3hvNV = _link_function('glTexCoord3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9977
glTexCoord4hNV = _link_function('glTexCoord4hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9978
glTexCoord4hvNV = _link_function('glTexCoord4hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9979
glMultiTexCoord1hNV = _link_function('glMultiTexCoord1hNV', None, [GLenum, GLhalfNV], 'NV_half_float')

# GL/glext.h:9980
glMultiTexCoord1hvNV = _link_function('glMultiTexCoord1hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9981
glMultiTexCoord2hNV = _link_function('glMultiTexCoord2hNV', None, [GLenum, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9982
glMultiTexCoord2hvNV = _link_function('glMultiTexCoord2hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9983
glMultiTexCoord3hNV = _link_function('glMultiTexCoord3hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9984
glMultiTexCoord3hvNV = _link_function('glMultiTexCoord3hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9985
glMultiTexCoord4hNV = _link_function('glMultiTexCoord4hNV', None, [GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9986
glMultiTexCoord4hvNV = _link_function('glMultiTexCoord4hvNV', None, [GLenum, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9987
glFogCoordhNV = _link_function('glFogCoordhNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:9988
glFogCoordhvNV = _link_function('glFogCoordhvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9989
glSecondaryColor3hNV = _link_function('glSecondaryColor3hNV', None, [GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9990
glSecondaryColor3hvNV = _link_function('glSecondaryColor3hvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9991
glVertexWeighthNV = _link_function('glVertexWeighthNV', None, [GLhalfNV], 'NV_half_float')

# GL/glext.h:9992
glVertexWeighthvNV = _link_function('glVertexWeighthvNV', None, [POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9993
glVertexAttrib1hNV = _link_function('glVertexAttrib1hNV', None, [GLuint, GLhalfNV], 'NV_half_float')

# GL/glext.h:9994
glVertexAttrib1hvNV = _link_function('glVertexAttrib1hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9995
glVertexAttrib2hNV = _link_function('glVertexAttrib2hNV', None, [GLuint, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9996
glVertexAttrib2hvNV = _link_function('glVertexAttrib2hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9997
glVertexAttrib3hNV = _link_function('glVertexAttrib3hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:9998
glVertexAttrib3hvNV = _link_function('glVertexAttrib3hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:9999
glVertexAttrib4hNV = _link_function('glVertexAttrib4hNV', None, [GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV], 'NV_half_float')

# GL/glext.h:10000
glVertexAttrib4hvNV = _link_function('glVertexAttrib4hvNV', None, [GLuint, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:10001
glVertexAttribs1hvNV = _link_function('glVertexAttribs1hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:10002
glVertexAttribs2hvNV = _link_function('glVertexAttribs2hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:10003
glVertexAttribs3hvNV = _link_function('glVertexAttribs3hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

# GL/glext.h:10004
glVertexAttribs4hvNV = _link_function('glVertexAttribs4hvNV', None, [GLuint, GLsizei, POINTER(GLhalfNV)], 'NV_half_float')

PFNGLVERTEX2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:10006
PFNGLVERTEX2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10007
PFNGLVERTEX3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10008
PFNGLVERTEX3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10009
PFNGLVERTEX4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10010
PFNGLVERTEX4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10011
PFNGLNORMAL3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10012
PFNGLNORMAL3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10013
PFNGLCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10014
PFNGLCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10015
PFNGLCOLOR4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10016
PFNGLCOLOR4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10017
PFNGLTEXCOORD1HNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:10018
PFNGLTEXCOORD1HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10019
PFNGLTEXCOORD2HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV) 	# GL/glext.h:10020
PFNGLTEXCOORD2HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10021
PFNGLTEXCOORD3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10022
PFNGLTEXCOORD3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10023
PFNGLTEXCOORD4HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10024
PFNGLTEXCOORD4HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10025
PFNGLMULTITEXCOORD1HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV) 	# GL/glext.h:10026
PFNGLMULTITEXCOORD1HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:10027
PFNGLMULTITEXCOORD2HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV) 	# GL/glext.h:10028
PFNGLMULTITEXCOORD2HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:10029
PFNGLMULTITEXCOORD3HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10030
PFNGLMULTITEXCOORD3HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:10031
PFNGLMULTITEXCOORD4HNVPROC = CFUNCTYPE(None, GLenum, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10032
PFNGLMULTITEXCOORD4HVNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLhalfNV)) 	# GL/glext.h:10033
PFNGLFOGCOORDHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:10034
PFNGLFOGCOORDHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10035
PFNGLSECONDARYCOLOR3HNVPROC = CFUNCTYPE(None, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10036
PFNGLSECONDARYCOLOR3HVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10037
PFNGLVERTEXWEIGHTHNVPROC = CFUNCTYPE(None, GLhalfNV) 	# GL/glext.h:10038
PFNGLVERTEXWEIGHTHVNVPROC = CFUNCTYPE(None, POINTER(GLhalfNV)) 	# GL/glext.h:10039
PFNGLVERTEXATTRIB1HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV) 	# GL/glext.h:10040
PFNGLVERTEXATTRIB1HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:10041
PFNGLVERTEXATTRIB2HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV) 	# GL/glext.h:10042
PFNGLVERTEXATTRIB2HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:10043
PFNGLVERTEXATTRIB3HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10044
PFNGLVERTEXATTRIB3HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:10045
PFNGLVERTEXATTRIB4HNVPROC = CFUNCTYPE(None, GLuint, GLhalfNV, GLhalfNV, GLhalfNV, GLhalfNV) 	# GL/glext.h:10046
PFNGLVERTEXATTRIB4HVNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLhalfNV)) 	# GL/glext.h:10047
PFNGLVERTEXATTRIBS1HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:10048
PFNGLVERTEXATTRIBS2HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:10049
PFNGLVERTEXATTRIBS3HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:10050
PFNGLVERTEXATTRIBS4HVNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLhalfNV)) 	# GL/glext.h:10051
# NV_pixel_data_range (GL/glext.h:10054)
GL_NV_pixel_data_range = 1 	# GL/glext.h:10055
# GL/glext.h:10057
glPixelDataRangeNV = _link_function('glPixelDataRangeNV', None, [GLenum, GLsizei, POINTER(GLvoid)], 'NV_pixel_data_range')

# GL/glext.h:10058
glFlushPixelDataRangeNV = _link_function('glFlushPixelDataRangeNV', None, [GLenum], 'NV_pixel_data_range')

PFNGLPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10060
PFNGLFLUSHPIXELDATARANGENVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10061
# NV_primitive_restart (GL/glext.h:10064)
GL_NV_primitive_restart = 1 	# GL/glext.h:10065
# GL/glext.h:10067
glPrimitiveRestartNV = _link_function('glPrimitiveRestartNV', None, [], 'NV_primitive_restart')

# GL/glext.h:10068
glPrimitiveRestartIndexNV = _link_function('glPrimitiveRestartIndexNV', None, [GLuint], 'NV_primitive_restart')

PFNGLPRIMITIVERESTARTNVPROC = CFUNCTYPE(None) 	# GL/glext.h:10070
PFNGLPRIMITIVERESTARTINDEXNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:10071
# NV_texture_expand_normal (GL/glext.h:10074)
GL_NV_texture_expand_normal = 1 	# GL/glext.h:10075
# NV_vertex_program2 (GL/glext.h:10078)
GL_NV_vertex_program2 = 1 	# GL/glext.h:10079
# ATI_map_object_buffer (GL/glext.h:10082)
GL_ATI_map_object_buffer = 1 	# GL/glext.h:10083
# GL/glext.h:10085
glMapObjectBufferATI = _link_function('glMapObjectBufferATI', POINTER(GLvoid), [GLuint], 'ATI_map_object_buffer')

# GL/glext.h:10086
glUnmapObjectBufferATI = _link_function('glUnmapObjectBufferATI', None, [GLuint], 'ATI_map_object_buffer')

PFNGLMAPOBJECTBUFFERATIPROC = CFUNCTYPE(POINTER(GLvoid), GLuint) 	# GL/glext.h:10088
PFNGLUNMAPOBJECTBUFFERATIPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:10089
# ATI_separate_stencil (GL/glext.h:10092)
GL_ATI_separate_stencil = 1 	# GL/glext.h:10093
# GL/glext.h:10095
glStencilOpSeparateATI = _link_function('glStencilOpSeparateATI', None, [GLenum, GLenum, GLenum, GLenum], 'ATI_separate_stencil')

# GL/glext.h:10096
glStencilFuncSeparateATI = _link_function('glStencilFuncSeparateATI', None, [GLenum, GLenum, GLint, GLuint], 'ATI_separate_stencil')

PFNGLSTENCILOPSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:10098
PFNGLSTENCILFUNCSEPARATEATIPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLuint) 	# GL/glext.h:10099
# ATI_vertex_attrib_array_object (GL/glext.h:10102)
GL_ATI_vertex_attrib_array_object = 1 	# GL/glext.h:10103
# GL/glext.h:10105
glVertexAttribArrayObjectATI = _link_function('glVertexAttribArrayObjectATI', None, [GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint], 'ATI_vertex_attrib_array_object')

# GL/glext.h:10106
glGetVertexAttribArrayObjectfvATI = _link_function('glGetVertexAttribArrayObjectfvATI', None, [GLuint, GLenum, POINTER(GLfloat)], 'ATI_vertex_attrib_array_object')

# GL/glext.h:10107
glGetVertexAttribArrayObjectivATI = _link_function('glGetVertexAttribArrayObjectivATI', None, [GLuint, GLenum, POINTER(GLint)], 'ATI_vertex_attrib_array_object')

PFNGLVERTEXATTRIBARRAYOBJECTATIPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei, GLuint, GLuint) 	# GL/glext.h:10109
PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10110
PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:10111
# OES_read_format (GL/glext.h:10114)
GL_OES_read_format = 1 	# GL/glext.h:10115
# EXT_depth_bounds_test (GL/glext.h:10118)
GL_EXT_depth_bounds_test = 1 	# GL/glext.h:10119
GLclampd = c_double 	# /usr/include/GL/gl.h:163
# GL/glext.h:10121
glDepthBoundsEXT = _link_function('glDepthBoundsEXT', None, [GLclampd, GLclampd], 'EXT_depth_bounds_test')

PFNGLDEPTHBOUNDSEXTPROC = CFUNCTYPE(None, GLclampd, GLclampd) 	# GL/glext.h:10123
# EXT_texture_mirror_clamp (GL/glext.h:10126)
GL_EXT_texture_mirror_clamp = 1 	# GL/glext.h:10127
# EXT_blend_equation_separate (GL/glext.h:10130)
GL_EXT_blend_equation_separate = 1 	# GL/glext.h:10131
# GL/glext.h:10133
glBlendEquationSeparateEXT = _link_function('glBlendEquationSeparateEXT', None, [GLenum, GLenum], 'EXT_blend_equation_separate')

PFNGLBLENDEQUATIONSEPARATEEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:10135
# MESA_pack_invert (GL/glext.h:10138)
GL_MESA_pack_invert = 1 	# GL/glext.h:10139
# MESA_ycbcr_texture (GL/glext.h:10142)
GL_MESA_ycbcr_texture = 1 	# GL/glext.h:10143
# EXT_pixel_buffer_object (GL/glext.h:10146)
GL_EXT_pixel_buffer_object = 1 	# GL/glext.h:10147
# NV_fragment_program_option (GL/glext.h:10150)
GL_NV_fragment_program_option = 1 	# GL/glext.h:10151
# NV_fragment_program2 (GL/glext.h:10154)
GL_NV_fragment_program2 = 1 	# GL/glext.h:10155
# NV_vertex_program2_option (GL/glext.h:10158)
GL_NV_vertex_program2_option = 1 	# GL/glext.h:10159
# NV_vertex_program3 (GL/glext.h:10162)
GL_NV_vertex_program3 = 1 	# GL/glext.h:10163
# EXT_framebuffer_object (GL/glext.h:10166)
GL_EXT_framebuffer_object = 1 	# GL/glext.h:10167
# GL/glext.h:10169
glIsRenderbufferEXT = _link_function('glIsRenderbufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:10170
glBindRenderbufferEXT = _link_function('glBindRenderbufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:10171
glDeleteRenderbuffersEXT = _link_function('glDeleteRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:10172
glGenRenderbuffersEXT = _link_function('glGenRenderbuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:10173
glRenderbufferStorageEXT = _link_function('glRenderbufferStorageEXT', None, [GLenum, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_object')

# GL/glext.h:10174
glGetRenderbufferParameterivEXT = _link_function('glGetRenderbufferParameterivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:10175
glIsFramebufferEXT = _link_function('glIsFramebufferEXT', GLboolean, [GLuint], 'EXT_framebuffer_object')

# GL/glext.h:10176
glBindFramebufferEXT = _link_function('glBindFramebufferEXT', None, [GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:10177
glDeleteFramebuffersEXT = _link_function('glDeleteFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:10178
glGenFramebuffersEXT = _link_function('glGenFramebuffersEXT', None, [GLsizei, POINTER(GLuint)], 'EXT_framebuffer_object')

# GL/glext.h:10179
glCheckFramebufferStatusEXT = _link_function('glCheckFramebufferStatusEXT', GLenum, [GLenum], 'EXT_framebuffer_object')

# GL/glext.h:10180
glFramebufferTexture1DEXT = _link_function('glFramebufferTexture1DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:10181
glFramebufferTexture2DEXT = _link_function('glFramebufferTexture2DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:10182
glFramebufferTexture3DEXT = _link_function('glFramebufferTexture3DEXT', None, [GLenum, GLenum, GLenum, GLuint, GLint, GLint], 'EXT_framebuffer_object')

# GL/glext.h:10183
glFramebufferRenderbufferEXT = _link_function('glFramebufferRenderbufferEXT', None, [GLenum, GLenum, GLenum, GLuint], 'EXT_framebuffer_object')

# GL/glext.h:10184
glGetFramebufferAttachmentParameterivEXT = _link_function('glGetFramebufferAttachmentParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_framebuffer_object')

# GL/glext.h:10185
glGenerateMipmapEXT = _link_function('glGenerateMipmapEXT', None, [GLenum], 'EXT_framebuffer_object')

PFNGLISRENDERBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:10187
PFNGLBINDRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:10188
PFNGLDELETERENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10189
PFNGLGENRENDERBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10190
PFNGLRENDERBUFFERSTORAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, GLsizei) 	# GL/glext.h:10191
PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10192
PFNGLISFRAMEBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:10193
PFNGLBINDFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:10194
PFNGLDELETEFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10195
PFNGLGENFRAMEBUFFERSEXTPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10196
PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC = CFUNCTYPE(GLenum, GLenum) 	# GL/glext.h:10197
PFNGLFRAMEBUFFERTEXTURE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:10198
PFNGLFRAMEBUFFERTEXTURE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:10199
PFNGLFRAMEBUFFERTEXTURE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:10200
PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:10201
PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10202
PFNGLGENERATEMIPMAPEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10203
# GREMEDY_string_marker (GL/glext.h:10206)
GL_GREMEDY_string_marker = 1 	# GL/glext.h:10207
# GL/glext.h:10209
glStringMarkerGREMEDY = _link_function('glStringMarkerGREMEDY', None, [GLsizei, POINTER(GLvoid)], 'GREMEDY_string_marker')

PFNGLSTRINGMARKERGREMEDYPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10211
# EXT_packed_depth_stencil (GL/glext.h:10214)
GL_EXT_packed_depth_stencil = 1 	# GL/glext.h:10215
# EXT_stencil_clear_tag (GL/glext.h:10218)
GL_EXT_stencil_clear_tag = 1 	# GL/glext.h:10219
# GL/glext.h:10221
glStencilClearTagEXT = _link_function('glStencilClearTagEXT', None, [GLsizei, GLuint], 'EXT_stencil_clear_tag')

PFNGLSTENCILCLEARTAGEXTPROC = CFUNCTYPE(None, GLsizei, GLuint) 	# GL/glext.h:10223
# EXT_texture_sRGB (GL/glext.h:10226)
GL_EXT_texture_sRGB = 1 	# GL/glext.h:10227
# EXT_framebuffer_blit (GL/glext.h:10230)
GL_EXT_framebuffer_blit = 1 	# GL/glext.h:10231
# GL/glext.h:10233
glBlitFramebufferEXT = _link_function('glBlitFramebufferEXT', None, [GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum], 'EXT_framebuffer_blit')

PFNGLBLITFRAMEBUFFEREXTPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLint, GLbitfield, GLenum) 	# GL/glext.h:10235
# EXT_framebuffer_multisample (GL/glext.h:10238)
GL_EXT_framebuffer_multisample = 1 	# GL/glext.h:10239
# GL/glext.h:10241
glRenderbufferStorageMultisampleEXT = _link_function('glRenderbufferStorageMultisampleEXT', None, [GLenum, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_framebuffer_multisample')

PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:10243
# MESAX_texture_stack (GL/glext.h:10246)
GL_MESAX_texture_stack = 1 	# GL/glext.h:10247
# EXT_timer_query (GL/glext.h:10250)
GL_EXT_timer_query = 1 	# GL/glext.h:10251
# GL/glext.h:10253
glGetQueryObjecti64vEXT = _link_function('glGetQueryObjecti64vEXT', None, [GLuint, GLenum, POINTER(GLint64EXT)], 'EXT_timer_query')

# GL/glext.h:10254
glGetQueryObjectui64vEXT = _link_function('glGetQueryObjectui64vEXT', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'EXT_timer_query')

PFNGLGETQUERYOBJECTI64VEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64EXT)) 	# GL/glext.h:10256
PFNGLGETQUERYOBJECTUI64VEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:10257
# EXT_gpu_program_parameters (GL/glext.h:10260)
GL_EXT_gpu_program_parameters = 1 	# GL/glext.h:10261
# GL/glext.h:10263
glProgramEnvParameters4fvEXT = _link_function('glProgramEnvParameters4fvEXT', None, [GLenum, GLuint, GLsizei, POINTER(GLfloat)], 'EXT_gpu_program_parameters')

# GL/glext.h:10264
glProgramLocalParameters4fvEXT = _link_function('glProgramLocalParameters4fvEXT', None, [GLenum, GLuint, GLsizei, POINTER(GLfloat)], 'EXT_gpu_program_parameters')

PFNGLPROGRAMENVPARAMETERS4FVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10266
PFNGLPROGRAMLOCALPARAMETERS4FVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10267
# APPLE_flush_buffer_range (GL/glext.h:10270)
GL_APPLE_flush_buffer_range = 1 	# GL/glext.h:10271
# GL/glext.h:10273
glBufferParameteriAPPLE = _link_function('glBufferParameteriAPPLE', None, [GLenum, GLenum, GLint], 'APPLE_flush_buffer_range')

# GL/glext.h:10274
glFlushMappedBufferRangeAPPLE = _link_function('glFlushMappedBufferRangeAPPLE', None, [GLenum, GLintptr, GLsizeiptr], 'APPLE_flush_buffer_range')

PFNGLBUFFERPARAMETERIAPPLEPROC = CFUNCTYPE(None, GLenum, GLenum, GLint) 	# GL/glext.h:10276
PFNGLFLUSHMAPPEDBUFFERRANGEAPPLEPROC = CFUNCTYPE(None, GLenum, GLintptr, GLsizeiptr) 	# GL/glext.h:10277
# NV_gpu_program4 (GL/glext.h:10280)
GL_NV_gpu_program4 = 1 	# GL/glext.h:10281
# GL/glext.h:10283
glProgramLocalParameterI4iNV = _link_function('glProgramLocalParameterI4iNV', None, [GLenum, GLuint, GLint, GLint, GLint, GLint], 'NV_gpu_program4')

# GL/glext.h:10284
glProgramLocalParameterI4ivNV = _link_function('glProgramLocalParameterI4ivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:10285
glProgramLocalParametersI4ivNV = _link_function('glProgramLocalParametersI4ivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:10286
glProgramLocalParameterI4uiNV = _link_function('glProgramLocalParameterI4uiNV', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_gpu_program4')

# GL/glext.h:10287
glProgramLocalParameterI4uivNV = _link_function('glProgramLocalParameterI4uivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:10288
glProgramLocalParametersI4uivNV = _link_function('glProgramLocalParametersI4uivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:10289
glProgramEnvParameterI4iNV = _link_function('glProgramEnvParameterI4iNV', None, [GLenum, GLuint, GLint, GLint, GLint, GLint], 'NV_gpu_program4')

# GL/glext.h:10290
glProgramEnvParameterI4ivNV = _link_function('glProgramEnvParameterI4ivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:10291
glProgramEnvParametersI4ivNV = _link_function('glProgramEnvParametersI4ivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:10292
glProgramEnvParameterI4uiNV = _link_function('glProgramEnvParameterI4uiNV', None, [GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_gpu_program4')

# GL/glext.h:10293
glProgramEnvParameterI4uivNV = _link_function('glProgramEnvParameterI4uivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:10294
glProgramEnvParametersI4uivNV = _link_function('glProgramEnvParametersI4uivNV', None, [GLenum, GLuint, GLsizei, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:10295
glGetProgramLocalParameterIivNV = _link_function('glGetProgramLocalParameterIivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:10296
glGetProgramLocalParameterIuivNV = _link_function('glGetProgramLocalParameterIuivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

# GL/glext.h:10297
glGetProgramEnvParameterIivNV = _link_function('glGetProgramEnvParameterIivNV', None, [GLenum, GLuint, POINTER(GLint)], 'NV_gpu_program4')

# GL/glext.h:10298
glGetProgramEnvParameterIuivNV = _link_function('glGetProgramEnvParameterIuivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program4')

PFNGLPROGRAMLOCALPARAMETERI4INVPROC = CFUNCTYPE(None, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:10300
PFNGLPROGRAMLOCALPARAMETERI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10301
PFNGLPROGRAMLOCALPARAMETERSI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10302
PFNGLPROGRAMLOCALPARAMETERI4UINVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10303
PFNGLPROGRAMLOCALPARAMETERI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:10304
PFNGLPROGRAMLOCALPARAMETERSI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10305
PFNGLPROGRAMENVPARAMETERI4INVPROC = CFUNCTYPE(None, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:10306
PFNGLPROGRAMENVPARAMETERI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10307
PFNGLPROGRAMENVPARAMETERSI4IVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10308
PFNGLPROGRAMENVPARAMETERI4UINVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10309
PFNGLPROGRAMENVPARAMETERI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:10310
PFNGLPROGRAMENVPARAMETERSI4UIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10311
PFNGLGETPROGRAMLOCALPARAMETERIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10312
PFNGLGETPROGRAMLOCALPARAMETERIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:10313
PFNGLGETPROGRAMENVPARAMETERIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10314
PFNGLGETPROGRAMENVPARAMETERIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:10315
# NV_geometry_program4 (GL/glext.h:10318)
GL_NV_geometry_program4 = 1 	# GL/glext.h:10319
# GL/glext.h:10321
glProgramVertexLimitNV = _link_function('glProgramVertexLimitNV', None, [GLenum, GLint], 'NV_geometry_program4')

# GL/glext.h:10322
glFramebufferTextureEXT = _link_function('glFramebufferTextureEXT', None, [GLenum, GLenum, GLuint, GLint], 'NV_geometry_program4')

# GL/glext.h:10323
glFramebufferTextureLayerEXT = _link_function('glFramebufferTextureLayerEXT', None, [GLenum, GLenum, GLuint, GLint, GLint], 'NV_geometry_program4')

# GL/glext.h:10324
glFramebufferTextureFaceEXT = _link_function('glFramebufferTextureFaceEXT', None, [GLenum, GLenum, GLuint, GLint, GLenum], 'NV_geometry_program4')

PFNGLPROGRAMVERTEXLIMITNVPROC = CFUNCTYPE(None, GLenum, GLint) 	# GL/glext.h:10326
PFNGLFRAMEBUFFERTEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:10327
PFNGLFRAMEBUFFERTEXTURELAYEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:10328
PFNGLFRAMEBUFFERTEXTUREFACEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:10329
# EXT_geometry_shader4 (GL/glext.h:10332)
GL_EXT_geometry_shader4 = 1 	# GL/glext.h:10333
# GL/glext.h:10335
glProgramParameteriEXT = _link_function('glProgramParameteriEXT', None, [GLuint, GLenum, GLint], 'EXT_geometry_shader4')

PFNGLPROGRAMPARAMETERIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:10337
# NV_vertex_program4 (GL/glext.h:10340)
GL_NV_vertex_program4 = 1 	# GL/glext.h:10341
# GL/glext.h:10343
glVertexAttribI1iEXT = _link_function('glVertexAttribI1iEXT', None, [GLuint, GLint], 'NV_vertex_program4')

# GL/glext.h:10344
glVertexAttribI2iEXT = _link_function('glVertexAttribI2iEXT', None, [GLuint, GLint, GLint], 'NV_vertex_program4')

# GL/glext.h:10345
glVertexAttribI3iEXT = _link_function('glVertexAttribI3iEXT', None, [GLuint, GLint, GLint, GLint], 'NV_vertex_program4')

# GL/glext.h:10346
glVertexAttribI4iEXT = _link_function('glVertexAttribI4iEXT', None, [GLuint, GLint, GLint, GLint, GLint], 'NV_vertex_program4')

# GL/glext.h:10347
glVertexAttribI1uiEXT = _link_function('glVertexAttribI1uiEXT', None, [GLuint, GLuint], 'NV_vertex_program4')

# GL/glext.h:10348
glVertexAttribI2uiEXT = _link_function('glVertexAttribI2uiEXT', None, [GLuint, GLuint, GLuint], 'NV_vertex_program4')

# GL/glext.h:10349
glVertexAttribI3uiEXT = _link_function('glVertexAttribI3uiEXT', None, [GLuint, GLuint, GLuint, GLuint], 'NV_vertex_program4')

# GL/glext.h:10350
glVertexAttribI4uiEXT = _link_function('glVertexAttribI4uiEXT', None, [GLuint, GLuint, GLuint, GLuint, GLuint], 'NV_vertex_program4')

# GL/glext.h:10351
glVertexAttribI1ivEXT = _link_function('glVertexAttribI1ivEXT', None, [GLuint, POINTER(GLint)], 'NV_vertex_program4')

# GL/glext.h:10352
glVertexAttribI2ivEXT = _link_function('glVertexAttribI2ivEXT', None, [GLuint, POINTER(GLint)], 'NV_vertex_program4')

# GL/glext.h:10353
glVertexAttribI3ivEXT = _link_function('glVertexAttribI3ivEXT', None, [GLuint, POINTER(GLint)], 'NV_vertex_program4')

# GL/glext.h:10354
glVertexAttribI4ivEXT = _link_function('glVertexAttribI4ivEXT', None, [GLuint, POINTER(GLint)], 'NV_vertex_program4')

# GL/glext.h:10355
glVertexAttribI1uivEXT = _link_function('glVertexAttribI1uivEXT', None, [GLuint, POINTER(GLuint)], 'NV_vertex_program4')

# GL/glext.h:10356
glVertexAttribI2uivEXT = _link_function('glVertexAttribI2uivEXT', None, [GLuint, POINTER(GLuint)], 'NV_vertex_program4')

# GL/glext.h:10357
glVertexAttribI3uivEXT = _link_function('glVertexAttribI3uivEXT', None, [GLuint, POINTER(GLuint)], 'NV_vertex_program4')

# GL/glext.h:10358
glVertexAttribI4uivEXT = _link_function('glVertexAttribI4uivEXT', None, [GLuint, POINTER(GLuint)], 'NV_vertex_program4')

# GL/glext.h:10359
glVertexAttribI4bvEXT = _link_function('glVertexAttribI4bvEXT', None, [GLuint, POINTER(GLbyte)], 'NV_vertex_program4')

# GL/glext.h:10360
glVertexAttribI4svEXT = _link_function('glVertexAttribI4svEXT', None, [GLuint, POINTER(GLshort)], 'NV_vertex_program4')

# GL/glext.h:10361
glVertexAttribI4ubvEXT = _link_function('glVertexAttribI4ubvEXT', None, [GLuint, POINTER(GLubyte)], 'NV_vertex_program4')

# GL/glext.h:10362
glVertexAttribI4usvEXT = _link_function('glVertexAttribI4usvEXT', None, [GLuint, POINTER(GLushort)], 'NV_vertex_program4')

# GL/glext.h:10363
glVertexAttribIPointerEXT = _link_function('glVertexAttribIPointerEXT', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'NV_vertex_program4')

# GL/glext.h:10364
glGetVertexAttribIivEXT = _link_function('glGetVertexAttribIivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'NV_vertex_program4')

# GL/glext.h:10365
glGetVertexAttribIuivEXT = _link_function('glGetVertexAttribIuivEXT', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_vertex_program4')

PFNGLVERTEXATTRIBI1IEXTPROC = CFUNCTYPE(None, GLuint, GLint) 	# GL/glext.h:10367
PFNGLVERTEXATTRIBI2IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:10368
PFNGLVERTEXATTRIBI3IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:10369
PFNGLVERTEXATTRIBI4IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:10370
PFNGLVERTEXATTRIBI1UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:10371
PFNGLVERTEXATTRIBI2UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint) 	# GL/glext.h:10372
PFNGLVERTEXATTRIBI3UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10373
PFNGLVERTEXATTRIBI4UIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10374
PFNGLVERTEXATTRIBI1IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:10375
PFNGLVERTEXATTRIBI2IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:10376
PFNGLVERTEXATTRIBI3IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:10377
PFNGLVERTEXATTRIBI4IVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLint)) 	# GL/glext.h:10378
PFNGLVERTEXATTRIBI1UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:10379
PFNGLVERTEXATTRIBI2UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:10380
PFNGLVERTEXATTRIBI3UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:10381
PFNGLVERTEXATTRIBI4UIVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint)) 	# GL/glext.h:10382
PFNGLVERTEXATTRIBI4BVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLbyte)) 	# GL/glext.h:10383
PFNGLVERTEXATTRIBI4SVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLshort)) 	# GL/glext.h:10384
PFNGLVERTEXATTRIBI4UBVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:10385
PFNGLVERTEXATTRIBI4USVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLushort)) 	# GL/glext.h:10386
PFNGLVERTEXATTRIBIPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10387
PFNGLGETVERTEXATTRIBIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:10388
PFNGLGETVERTEXATTRIBIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:10389
# EXT_gpu_shader4 (GL/glext.h:10392)
GL_EXT_gpu_shader4 = 1 	# GL/glext.h:10393
# GL/glext.h:10395
glGetUniformuivEXT = _link_function('glGetUniformuivEXT', None, [GLuint, GLint, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:10396
glBindFragDataLocationEXT = _link_function('glBindFragDataLocationEXT', None, [GLuint, GLuint, POINTER(GLchar)], 'EXT_gpu_shader4')

# GL/glext.h:10397
glGetFragDataLocationEXT = _link_function('glGetFragDataLocationEXT', GLint, [GLuint, POINTER(GLchar)], 'EXT_gpu_shader4')

# GL/glext.h:10398
glUniform1uiEXT = _link_function('glUniform1uiEXT', None, [GLint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:10399
glUniform2uiEXT = _link_function('glUniform2uiEXT', None, [GLint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:10400
glUniform3uiEXT = _link_function('glUniform3uiEXT', None, [GLint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:10401
glUniform4uiEXT = _link_function('glUniform4uiEXT', None, [GLint, GLuint, GLuint, GLuint, GLuint], 'EXT_gpu_shader4')

# GL/glext.h:10402
glUniform1uivEXT = _link_function('glUniform1uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:10403
glUniform2uivEXT = _link_function('glUniform2uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:10404
glUniform3uivEXT = _link_function('glUniform3uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

# GL/glext.h:10405
glUniform4uivEXT = _link_function('glUniform4uivEXT', None, [GLint, GLsizei, POINTER(GLuint)], 'EXT_gpu_shader4')

PFNGLGETUNIFORMUIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint)) 	# GL/glext.h:10407
PFNGLBINDFRAGDATALOCATIONEXTPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLchar)) 	# GL/glext.h:10408
PFNGLGETFRAGDATALOCATIONEXTPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:10409
PFNGLUNIFORM1UIEXTPROC = CFUNCTYPE(None, GLint, GLuint) 	# GL/glext.h:10410
PFNGLUNIFORM2UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint) 	# GL/glext.h:10411
PFNGLUNIFORM3UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:10412
PFNGLUNIFORM4UIEXTPROC = CFUNCTYPE(None, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10413
PFNGLUNIFORM1UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10414
PFNGLUNIFORM2UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10415
PFNGLUNIFORM3UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10416
PFNGLUNIFORM4UIVEXTPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10417
# EXT_draw_instanced (GL/glext.h:10420)
GL_EXT_draw_instanced = 1 	# GL/glext.h:10421
# GL/glext.h:10423
glDrawArraysInstancedEXT = _link_function('glDrawArraysInstancedEXT', None, [GLenum, GLint, GLsizei, GLsizei], 'EXT_draw_instanced')

# GL/glext.h:10424
glDrawElementsInstancedEXT = _link_function('glDrawElementsInstancedEXT', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei], 'EXT_draw_instanced')

PFNGLDRAWARRAYSINSTANCEDEXTPROC = CFUNCTYPE(None, GLenum, GLint, GLsizei, GLsizei) 	# GL/glext.h:10426
PFNGLDRAWELEMENTSINSTANCEDEXTPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLsizei) 	# GL/glext.h:10427
# EXT_packed_float (GL/glext.h:10430)
GL_EXT_packed_float = 1 	# GL/glext.h:10431
# EXT_texture_array (GL/glext.h:10434)
GL_EXT_texture_array = 1 	# GL/glext.h:10435
# EXT_texture_buffer_object (GL/glext.h:10438)
GL_EXT_texture_buffer_object = 1 	# GL/glext.h:10439
# GL/glext.h:10441
glTexBufferEXT = _link_function('glTexBufferEXT', None, [GLenum, GLenum, GLuint], 'EXT_texture_buffer_object')

PFNGLTEXBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:10443
# EXT_texture_compression_latc (GL/glext.h:10446)
GL_EXT_texture_compression_latc = 1 	# GL/glext.h:10447
# EXT_texture_compression_rgtc (GL/glext.h:10450)
GL_EXT_texture_compression_rgtc = 1 	# GL/glext.h:10451
# EXT_texture_shared_exponent (GL/glext.h:10454)
GL_EXT_texture_shared_exponent = 1 	# GL/glext.h:10455
# NV_depth_buffer_float (GL/glext.h:10458)
GL_NV_depth_buffer_float = 1 	# GL/glext.h:10459
# GL/glext.h:10461
glDepthRangedNV = _link_function('glDepthRangedNV', None, [GLdouble, GLdouble], 'NV_depth_buffer_float')

# GL/glext.h:10462
glClearDepthdNV = _link_function('glClearDepthdNV', None, [GLdouble], 'NV_depth_buffer_float')

# GL/glext.h:10463
glDepthBoundsdNV = _link_function('glDepthBoundsdNV', None, [GLdouble, GLdouble], 'NV_depth_buffer_float')

PFNGLDEPTHRANGEDNVPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:10465
PFNGLCLEARDEPTHDNVPROC = CFUNCTYPE(None, GLdouble) 	# GL/glext.h:10466
PFNGLDEPTHBOUNDSDNVPROC = CFUNCTYPE(None, GLdouble, GLdouble) 	# GL/glext.h:10467
# NV_fragment_program4 (GL/glext.h:10470)
GL_NV_fragment_program4 = 1 	# GL/glext.h:10471
# NV_framebuffer_multisample_coverage (GL/glext.h:10474)
GL_NV_framebuffer_multisample_coverage = 1 	# GL/glext.h:10475
# GL/glext.h:10477
glRenderbufferStorageMultisampleCoverageNV = _link_function('glRenderbufferStorageMultisampleCoverageNV', None, [GLenum, GLsizei, GLsizei, GLenum, GLsizei, GLsizei], 'NV_framebuffer_multisample_coverage')

PFNGLRENDERBUFFERSTORAGEMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:10479
# EXT_framebuffer_sRGB (GL/glext.h:10482)
GL_EXT_framebuffer_sRGB = 1 	# GL/glext.h:10483
# NV_geometry_shader4 (GL/glext.h:10486)
GL_NV_geometry_shader4 = 1 	# GL/glext.h:10487
# NV_parameter_buffer_object (GL/glext.h:10490)
GL_NV_parameter_buffer_object = 1 	# GL/glext.h:10491
# GL/glext.h:10493
glProgramBufferParametersfvNV = _link_function('glProgramBufferParametersfvNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLfloat)], 'NV_parameter_buffer_object')

# GL/glext.h:10494
glProgramBufferParametersIivNV = _link_function('glProgramBufferParametersIivNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLint)], 'NV_parameter_buffer_object')

# GL/glext.h:10495
glProgramBufferParametersIuivNV = _link_function('glProgramBufferParametersIuivNV', None, [GLenum, GLuint, GLuint, GLsizei, POINTER(GLuint)], 'NV_parameter_buffer_object')

PFNGLPROGRAMBUFFERPARAMETERSFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10497
PFNGLPROGRAMBUFFERPARAMETERSIIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10498
PFNGLPROGRAMBUFFERPARAMETERSIUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10499
# EXT_draw_buffers2 (GL/glext.h:10502)
GL_EXT_draw_buffers2 = 1 	# GL/glext.h:10503
# GL/glext.h:10505
glColorMaskIndexedEXT = _link_function('glColorMaskIndexedEXT', None, [GLuint, GLboolean, GLboolean, GLboolean, GLboolean], 'EXT_draw_buffers2')

# GL/glext.h:10506
glGetBooleanIndexedvEXT = _link_function('glGetBooleanIndexedvEXT', None, [GLenum, GLuint, POINTER(GLboolean)], 'EXT_draw_buffers2')

# GL/glext.h:10507
glGetIntegerIndexedvEXT = _link_function('glGetIntegerIndexedvEXT', None, [GLenum, GLuint, POINTER(GLint)], 'EXT_draw_buffers2')

# GL/glext.h:10508
glEnableIndexedEXT = _link_function('glEnableIndexedEXT', None, [GLenum, GLuint], 'EXT_draw_buffers2')

# GL/glext.h:10509
glDisableIndexedEXT = _link_function('glDisableIndexedEXT', None, [GLenum, GLuint], 'EXT_draw_buffers2')

# GL/glext.h:10510
glIsEnabledIndexedEXT = _link_function('glIsEnabledIndexedEXT', GLboolean, [GLenum, GLuint], 'EXT_draw_buffers2')

PFNGLCOLORMASKINDEXEDEXTPROC = CFUNCTYPE(None, GLuint, GLboolean, GLboolean, GLboolean, GLboolean) 	# GL/glext.h:10512
PFNGLGETBOOLEANINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLboolean)) 	# GL/glext.h:10513
PFNGLGETINTEGERINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10514
PFNGLENABLEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:10515
PFNGLDISABLEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:10516
PFNGLISENABLEDINDEXEDEXTPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:10517
# NV_transform_feedback (GL/glext.h:10520)
GL_NV_transform_feedback = 1 	# GL/glext.h:10521
# GL/glext.h:10523
glBeginTransformFeedbackNV = _link_function('glBeginTransformFeedbackNV', None, [GLenum], 'NV_transform_feedback')

# GL/glext.h:10524
glEndTransformFeedbackNV = _link_function('glEndTransformFeedbackNV', None, [], 'NV_transform_feedback')

# GL/glext.h:10525
glTransformFeedbackAttribsNV = _link_function('glTransformFeedbackAttribsNV', None, [GLuint, POINTER(GLint), GLenum], 'NV_transform_feedback')

# GL/glext.h:10526
glBindBufferRangeNV = _link_function('glBindBufferRangeNV', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'NV_transform_feedback')

# GL/glext.h:10527
glBindBufferOffsetNV = _link_function('glBindBufferOffsetNV', None, [GLenum, GLuint, GLuint, GLintptr], 'NV_transform_feedback')

# GL/glext.h:10528
glBindBufferBaseNV = _link_function('glBindBufferBaseNV', None, [GLenum, GLuint, GLuint], 'NV_transform_feedback')

# GL/glext.h:10529
glTransformFeedbackVaryingsNV = _link_function('glTransformFeedbackVaryingsNV', None, [GLuint, GLsizei, POINTER(GLint), GLenum], 'NV_transform_feedback')

# GL/glext.h:10530
glActiveVaryingNV = _link_function('glActiveVaryingNV', None, [GLuint, POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:10531
glGetVaryingLocationNV = _link_function('glGetVaryingLocationNV', GLint, [GLuint, POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:10532
glGetActiveVaryingNV = _link_function('glGetActiveVaryingNV', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)], 'NV_transform_feedback')

# GL/glext.h:10533
glGetTransformFeedbackVaryingNV = _link_function('glGetTransformFeedbackVaryingNV', None, [GLuint, GLuint, POINTER(GLint)], 'NV_transform_feedback')

# GL/glext.h:10534
glTransformFeedbackStreamAttribsNV = _link_function('glTransformFeedbackStreamAttribsNV', None, [GLsizei, POINTER(GLint), GLsizei, POINTER(GLint), GLenum], 'NV_transform_feedback')

PFNGLBEGINTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10536
PFNGLENDTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:10537
PFNGLTRANSFORMFEEDBACKATTRIBSNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint), GLenum) 	# GL/glext.h:10538
PFNGLBINDBUFFERRANGENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:10539
PFNGLBINDBUFFEROFFSETNVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr) 	# GL/glext.h:10540
PFNGLBINDBUFFERBASENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:10541
PFNGLTRANSFORMFEEDBACKVARYINGSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLint), GLenum) 	# GL/glext.h:10542
PFNGLACTIVEVARYINGNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLchar)) 	# GL/glext.h:10543
PFNGLGETVARYINGLOCATIONNVPROC = CFUNCTYPE(GLint, GLuint, POINTER(GLchar)) 	# GL/glext.h:10544
PFNGLGETACTIVEVARYINGNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:10545
PFNGLGETTRANSFORMFEEDBACKVARYINGNVPROC = CFUNCTYPE(None, GLuint, GLuint, POINTER(GLint)) 	# GL/glext.h:10546
PFNGLTRANSFORMFEEDBACKSTREAMATTRIBSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLint), GLsizei, POINTER(GLint), GLenum) 	# GL/glext.h:10547
# EXT_bindable_uniform (GL/glext.h:10550)
GL_EXT_bindable_uniform = 1 	# GL/glext.h:10551
# GL/glext.h:10553
glUniformBufferEXT = _link_function('glUniformBufferEXT', None, [GLuint, GLint, GLuint], 'EXT_bindable_uniform')

# GL/glext.h:10554
glGetUniformBufferSizeEXT = _link_function('glGetUniformBufferSizeEXT', GLint, [GLuint, GLint], 'EXT_bindable_uniform')

# GL/glext.h:10555
glGetUniformOffsetEXT = _link_function('glGetUniformOffsetEXT', GLintptr, [GLuint, GLint], 'EXT_bindable_uniform')

PFNGLUNIFORMBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:10557
PFNGLGETUNIFORMBUFFERSIZEEXTPROC = CFUNCTYPE(GLint, GLuint, GLint) 	# GL/glext.h:10558
PFNGLGETUNIFORMOFFSETEXTPROC = CFUNCTYPE(GLintptr, GLuint, GLint) 	# GL/glext.h:10559
# EXT_texture_integer (GL/glext.h:10562)
GL_EXT_texture_integer = 1 	# GL/glext.h:10563
# GL/glext.h:10565
glTexParameterIivEXT = _link_function('glTexParameterIivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_texture_integer')

# GL/glext.h:10566
glTexParameterIuivEXT = _link_function('glTexParameterIuivEXT', None, [GLenum, GLenum, POINTER(GLuint)], 'EXT_texture_integer')

# GL/glext.h:10567
glGetTexParameterIivEXT = _link_function('glGetTexParameterIivEXT', None, [GLenum, GLenum, POINTER(GLint)], 'EXT_texture_integer')

# GL/glext.h:10568
glGetTexParameterIuivEXT = _link_function('glGetTexParameterIuivEXT', None, [GLenum, GLenum, POINTER(GLuint)], 'EXT_texture_integer')

# GL/glext.h:10569
glClearColorIiEXT = _link_function('glClearColorIiEXT', None, [GLint, GLint, GLint, GLint], 'EXT_texture_integer')

# GL/glext.h:10570
glClearColorIuiEXT = _link_function('glClearColorIuiEXT', None, [GLuint, GLuint, GLuint, GLuint], 'EXT_texture_integer')

PFNGLTEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10572
PFNGLTEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:10573
PFNGLGETTEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10574
PFNGLGETTEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:10575
PFNGLCLEARCOLORIIEXTPROC = CFUNCTYPE(None, GLint, GLint, GLint, GLint) 	# GL/glext.h:10576
PFNGLCLEARCOLORIUIEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10577
# GREMEDY_frame_terminator (GL/glext.h:10580)
GL_GREMEDY_frame_terminator = 1 	# GL/glext.h:10581
# GL/glext.h:10583
glFrameTerminatorGREMEDY = _link_function('glFrameTerminatorGREMEDY', None, [], 'GREMEDY_frame_terminator')

PFNGLFRAMETERMINATORGREMEDYPROC = CFUNCTYPE(None) 	# GL/glext.h:10585
# NV_conditional_render (GL/glext.h:10588)
GL_NV_conditional_render = 1 	# GL/glext.h:10589
# GL/glext.h:10591
glBeginConditionalRenderNV = _link_function('glBeginConditionalRenderNV', None, [GLuint, GLenum], 'NV_conditional_render')

# GL/glext.h:10592
glEndConditionalRenderNV = _link_function('glEndConditionalRenderNV', None, [], 'NV_conditional_render')

PFNGLBEGINCONDITIONALRENDERNVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:10594
PFNGLENDCONDITIONALRENDERNVPROC = CFUNCTYPE(None) 	# GL/glext.h:10595
# NV_present_video (GL/glext.h:10598)
GL_NV_present_video = 1 	# GL/glext.h:10599
# GL/glext.h:10601
glPresentFrameKeyedNV = _link_function('glPresentFrameKeyedNV', None, [GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLuint, GLenum, GLuint, GLuint], 'NV_present_video')

# GL/glext.h:10602
glPresentFrameDualFillNV = _link_function('glPresentFrameDualFillNV', None, [GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint], 'NV_present_video')

# GL/glext.h:10603
glGetVideoivNV = _link_function('glGetVideoivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_present_video')

# GL/glext.h:10604
glGetVideouivNV = _link_function('glGetVideouivNV', None, [GLuint, GLenum, POINTER(GLuint)], 'NV_present_video')

# GL/glext.h:10605
glGetVideoi64vNV = _link_function('glGetVideoi64vNV', None, [GLuint, GLenum, POINTER(GLint64EXT)], 'NV_present_video')

# GL/glext.h:10606
glGetVideoui64vNV = _link_function('glGetVideoui64vNV', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'NV_present_video')

PFNGLPRESENTFRAMEKEYEDNVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLuint, GLenum, GLuint, GLuint) 	# GL/glext.h:10608
PFNGLPRESENTFRAMEDUALFILLNVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint, GLuint, GLenum, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint, GLenum, GLuint) 	# GL/glext.h:10609
PFNGLGETVIDEOIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:10610
PFNGLGETVIDEOUIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint)) 	# GL/glext.h:10611
PFNGLGETVIDEOI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64EXT)) 	# GL/glext.h:10612
PFNGLGETVIDEOUI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:10613
# EXT_transform_feedback (GL/glext.h:10616)
GL_EXT_transform_feedback = 1 	# GL/glext.h:10617
# GL/glext.h:10619
glBeginTransformFeedbackEXT = _link_function('glBeginTransformFeedbackEXT', None, [GLenum], 'EXT_transform_feedback')

# GL/glext.h:10620
glEndTransformFeedbackEXT = _link_function('glEndTransformFeedbackEXT', None, [], 'EXT_transform_feedback')

# GL/glext.h:10621
glBindBufferRangeEXT = _link_function('glBindBufferRangeEXT', None, [GLenum, GLuint, GLuint, GLintptr, GLsizeiptr], 'EXT_transform_feedback')

# GL/glext.h:10622
glBindBufferOffsetEXT = _link_function('glBindBufferOffsetEXT', None, [GLenum, GLuint, GLuint, GLintptr], 'EXT_transform_feedback')

# GL/glext.h:10623
glBindBufferBaseEXT = _link_function('glBindBufferBaseEXT', None, [GLenum, GLuint, GLuint], 'EXT_transform_feedback')

# GL/glext.h:10624
glTransformFeedbackVaryingsEXT = _link_function('glTransformFeedbackVaryingsEXT', None, [GLuint, GLsizei, POINTER(POINTER(GLchar)), GLenum], 'EXT_transform_feedback')

# GL/glext.h:10625
glGetTransformFeedbackVaryingEXT = _link_function('glGetTransformFeedbackVaryingEXT', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)], 'EXT_transform_feedback')

PFNGLBEGINTRANSFORMFEEDBACKEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10627
PFNGLENDTRANSFORMFEEDBACKEXTPROC = CFUNCTYPE(None) 	# GL/glext.h:10628
PFNGLBINDBUFFERRANGEEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:10629
PFNGLBINDBUFFEROFFSETEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint, GLintptr) 	# GL/glext.h:10630
PFNGLBINDBUFFERBASEEXTPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint) 	# GL/glext.h:10631
PFNGLTRANSFORMFEEDBACKVARYINGSEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(POINTER(GLchar)), GLenum) 	# GL/glext.h:10632
PFNGLGETTRANSFORMFEEDBACKVARYINGEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLsizei), POINTER(GLenum), POINTER(GLchar)) 	# GL/glext.h:10633
# EXT_direct_state_access (GL/glext.h:10636)
GL_EXT_direct_state_access = 1 	# GL/glext.h:10637
# GL/glext.h:10639
glClientAttribDefaultEXT = _link_function('glClientAttribDefaultEXT', None, [GLbitfield], 'EXT_direct_state_access')

# GL/glext.h:10640
glPushClientAttribDefaultEXT = _link_function('glPushClientAttribDefaultEXT', None, [GLbitfield], 'EXT_direct_state_access')

# GL/glext.h:10641
glMatrixLoadfEXT = _link_function('glMatrixLoadfEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10642
glMatrixLoaddEXT = _link_function('glMatrixLoaddEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10643
glMatrixMultfEXT = _link_function('glMatrixMultfEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10644
glMatrixMultdEXT = _link_function('glMatrixMultdEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10645
glMatrixLoadIdentityEXT = _link_function('glMatrixLoadIdentityEXT', None, [GLenum], 'EXT_direct_state_access')

# GL/glext.h:10646
glMatrixRotatefEXT = _link_function('glMatrixRotatefEXT', None, [GLenum, GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10647
glMatrixRotatedEXT = _link_function('glMatrixRotatedEXT', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10648
glMatrixScalefEXT = _link_function('glMatrixScalefEXT', None, [GLenum, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10649
glMatrixScaledEXT = _link_function('glMatrixScaledEXT', None, [GLenum, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10650
glMatrixTranslatefEXT = _link_function('glMatrixTranslatefEXT', None, [GLenum, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10651
glMatrixTranslatedEXT = _link_function('glMatrixTranslatedEXT', None, [GLenum, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10652
glMatrixFrustumEXT = _link_function('glMatrixFrustumEXT', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10653
glMatrixOrthoEXT = _link_function('glMatrixOrthoEXT', None, [GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10654
glMatrixPopEXT = _link_function('glMatrixPopEXT', None, [GLenum], 'EXT_direct_state_access')

# GL/glext.h:10655
glMatrixPushEXT = _link_function('glMatrixPushEXT', None, [GLenum], 'EXT_direct_state_access')

# GL/glext.h:10656
glMatrixLoadTransposefEXT = _link_function('glMatrixLoadTransposefEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10657
glMatrixLoadTransposedEXT = _link_function('glMatrixLoadTransposedEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10658
glMatrixMultTransposefEXT = _link_function('glMatrixMultTransposefEXT', None, [GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10659
glMatrixMultTransposedEXT = _link_function('glMatrixMultTransposedEXT', None, [GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10660
glTextureParameterfEXT = _link_function('glTextureParameterfEXT', None, [GLuint, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10661
glTextureParameterfvEXT = _link_function('glTextureParameterfvEXT', None, [GLuint, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10662
glTextureParameteriEXT = _link_function('glTextureParameteriEXT', None, [GLuint, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:10663
glTextureParameterivEXT = _link_function('glTextureParameterivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10664
glTextureImage1DEXT = _link_function('glTextureImage1DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10665
glTextureImage2DEXT = _link_function('glTextureImage2DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10666
glTextureSubImage1DEXT = _link_function('glTextureSubImage1DEXT', None, [GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10667
glTextureSubImage2DEXT = _link_function('glTextureSubImage2DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10668
glCopyTextureImage1DEXT = _link_function('glCopyTextureImage1DEXT', None, [GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:10669
glCopyTextureImage2DEXT = _link_function('glCopyTextureImage2DEXT', None, [GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:10670
glCopyTextureSubImage1DEXT = _link_function('glCopyTextureSubImage1DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10671
glCopyTextureSubImage2DEXT = _link_function('glCopyTextureSubImage2DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10672
glGetTextureImageEXT = _link_function('glGetTextureImageEXT', None, [GLuint, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10673
glGetTextureParameterfvEXT = _link_function('glGetTextureParameterfvEXT', None, [GLuint, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10674
glGetTextureParameterivEXT = _link_function('glGetTextureParameterivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10675
glGetTextureLevelParameterfvEXT = _link_function('glGetTextureLevelParameterfvEXT', None, [GLuint, GLenum, GLint, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10676
glGetTextureLevelParameterivEXT = _link_function('glGetTextureLevelParameterivEXT', None, [GLuint, GLenum, GLint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10677
glTextureImage3DEXT = _link_function('glTextureImage3DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10678
glTextureSubImage3DEXT = _link_function('glTextureSubImage3DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10679
glCopyTextureSubImage3DEXT = _link_function('glCopyTextureSubImage3DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10680
glMultiTexParameterfEXT = _link_function('glMultiTexParameterfEXT', None, [GLenum, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10681
glMultiTexParameterfvEXT = _link_function('glMultiTexParameterfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10682
glMultiTexParameteriEXT = _link_function('glMultiTexParameteriEXT', None, [GLenum, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:10683
glMultiTexParameterivEXT = _link_function('glMultiTexParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10684
glMultiTexImage1DEXT = _link_function('glMultiTexImage1DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10685
glMultiTexImage2DEXT = _link_function('glMultiTexImage2DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10686
glMultiTexSubImage1DEXT = _link_function('glMultiTexSubImage1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10687
glMultiTexSubImage2DEXT = _link_function('glMultiTexSubImage2DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10688
glCopyMultiTexImage1DEXT = _link_function('glCopyMultiTexImage1DEXT', None, [GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:10689
glCopyMultiTexImage2DEXT = _link_function('glCopyMultiTexImage2DEXT', None, [GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint], 'EXT_direct_state_access')

# GL/glext.h:10690
glCopyMultiTexSubImage1DEXT = _link_function('glCopyMultiTexSubImage1DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10691
glCopyMultiTexSubImage2DEXT = _link_function('glCopyMultiTexSubImage2DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10692
glGetMultiTexImageEXT = _link_function('glGetMultiTexImageEXT', None, [GLenum, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10693
glGetMultiTexParameterfvEXT = _link_function('glGetMultiTexParameterfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10694
glGetMultiTexParameterivEXT = _link_function('glGetMultiTexParameterivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10695
glGetMultiTexLevelParameterfvEXT = _link_function('glGetMultiTexLevelParameterfvEXT', None, [GLenum, GLenum, GLint, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10696
glGetMultiTexLevelParameterivEXT = _link_function('glGetMultiTexLevelParameterivEXT', None, [GLenum, GLenum, GLint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10697
glMultiTexImage3DEXT = _link_function('glMultiTexImage3DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10698
glMultiTexSubImage3DEXT = _link_function('glMultiTexSubImage3DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10699
glCopyMultiTexSubImage3DEXT = _link_function('glCopyMultiTexSubImage3DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10700
glBindMultiTextureEXT = _link_function('glBindMultiTextureEXT', None, [GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10701
glEnableClientStateIndexedEXT = _link_function('glEnableClientStateIndexedEXT', None, [GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10702
glDisableClientStateIndexedEXT = _link_function('glDisableClientStateIndexedEXT', None, [GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10703
glMultiTexCoordPointerEXT = _link_function('glMultiTexCoordPointerEXT', None, [GLenum, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10704
glMultiTexEnvfEXT = _link_function('glMultiTexEnvfEXT', None, [GLenum, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10705
glMultiTexEnvfvEXT = _link_function('glMultiTexEnvfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10706
glMultiTexEnviEXT = _link_function('glMultiTexEnviEXT', None, [GLenum, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:10707
glMultiTexEnvivEXT = _link_function('glMultiTexEnvivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10708
glMultiTexGendEXT = _link_function('glMultiTexGendEXT', None, [GLenum, GLenum, GLenum, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10709
glMultiTexGendvEXT = _link_function('glMultiTexGendvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10710
glMultiTexGenfEXT = _link_function('glMultiTexGenfEXT', None, [GLenum, GLenum, GLenum, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10711
glMultiTexGenfvEXT = _link_function('glMultiTexGenfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10712
glMultiTexGeniEXT = _link_function('glMultiTexGeniEXT', None, [GLenum, GLenum, GLenum, GLint], 'EXT_direct_state_access')

# GL/glext.h:10713
glMultiTexGenivEXT = _link_function('glMultiTexGenivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10714
glGetMultiTexEnvfvEXT = _link_function('glGetMultiTexEnvfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10715
glGetMultiTexEnvivEXT = _link_function('glGetMultiTexEnvivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10716
glGetMultiTexGendvEXT = _link_function('glGetMultiTexGendvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10717
glGetMultiTexGenfvEXT = _link_function('glGetMultiTexGenfvEXT', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10718
glGetMultiTexGenivEXT = _link_function('glGetMultiTexGenivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10719
glGetFloatIndexedvEXT = _link_function('glGetFloatIndexedvEXT', None, [GLenum, GLuint, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10720
glGetDoubleIndexedvEXT = _link_function('glGetDoubleIndexedvEXT', None, [GLenum, GLuint, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10721
glGetPointerIndexedvEXT = _link_function('glGetPointerIndexedvEXT', None, [GLenum, GLuint, POINTER(POINTER(GLvoid))], 'EXT_direct_state_access')

# GL/glext.h:10722
glCompressedTextureImage3DEXT = _link_function('glCompressedTextureImage3DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10723
glCompressedTextureImage2DEXT = _link_function('glCompressedTextureImage2DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10724
glCompressedTextureImage1DEXT = _link_function('glCompressedTextureImage1DEXT', None, [GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10725
glCompressedTextureSubImage3DEXT = _link_function('glCompressedTextureSubImage3DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10726
glCompressedTextureSubImage2DEXT = _link_function('glCompressedTextureSubImage2DEXT', None, [GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10727
glCompressedTextureSubImage1DEXT = _link_function('glCompressedTextureSubImage1DEXT', None, [GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10728
glGetCompressedTextureImageEXT = _link_function('glGetCompressedTextureImageEXT', None, [GLuint, GLenum, GLint, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10729
glCompressedMultiTexImage3DEXT = _link_function('glCompressedMultiTexImage3DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10730
glCompressedMultiTexImage2DEXT = _link_function('glCompressedMultiTexImage2DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10731
glCompressedMultiTexImage1DEXT = _link_function('glCompressedMultiTexImage1DEXT', None, [GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10732
glCompressedMultiTexSubImage3DEXT = _link_function('glCompressedMultiTexSubImage3DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10733
glCompressedMultiTexSubImage2DEXT = _link_function('glCompressedMultiTexSubImage2DEXT', None, [GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10734
glCompressedMultiTexSubImage1DEXT = _link_function('glCompressedMultiTexSubImage1DEXT', None, [GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10735
glGetCompressedMultiTexImageEXT = _link_function('glGetCompressedMultiTexImageEXT', None, [GLenum, GLenum, GLint, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10736
glNamedProgramStringEXT = _link_function('glNamedProgramStringEXT', None, [GLuint, GLenum, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10737
glNamedProgramLocalParameter4dEXT = _link_function('glNamedProgramLocalParameter4dEXT', None, [GLuint, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10738
glNamedProgramLocalParameter4dvEXT = _link_function('glNamedProgramLocalParameter4dvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10739
glNamedProgramLocalParameter4fEXT = _link_function('glNamedProgramLocalParameter4fEXT', None, [GLuint, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10740
glNamedProgramLocalParameter4fvEXT = _link_function('glNamedProgramLocalParameter4fvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10741
glGetNamedProgramLocalParameterdvEXT = _link_function('glGetNamedProgramLocalParameterdvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10742
glGetNamedProgramLocalParameterfvEXT = _link_function('glGetNamedProgramLocalParameterfvEXT', None, [GLuint, GLenum, GLuint, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10743
glGetNamedProgramivEXT = _link_function('glGetNamedProgramivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10744
glGetNamedProgramStringEXT = _link_function('glGetNamedProgramStringEXT', None, [GLuint, GLenum, GLenum, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10745
glNamedProgramLocalParameters4fvEXT = _link_function('glNamedProgramLocalParameters4fvEXT', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10746
glNamedProgramLocalParameterI4iEXT = _link_function('glNamedProgramLocalParameterI4iEXT', None, [GLuint, GLenum, GLuint, GLint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10747
glNamedProgramLocalParameterI4ivEXT = _link_function('glNamedProgramLocalParameterI4ivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10748
glNamedProgramLocalParametersI4ivEXT = _link_function('glNamedProgramLocalParametersI4ivEXT', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10749
glNamedProgramLocalParameterI4uiEXT = _link_function('glNamedProgramLocalParameterI4uiEXT', None, [GLuint, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10750
glNamedProgramLocalParameterI4uivEXT = _link_function('glNamedProgramLocalParameterI4uivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10751
glNamedProgramLocalParametersI4uivEXT = _link_function('glNamedProgramLocalParametersI4uivEXT', None, [GLuint, GLenum, GLuint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10752
glGetNamedProgramLocalParameterIivEXT = _link_function('glGetNamedProgramLocalParameterIivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10753
glGetNamedProgramLocalParameterIuivEXT = _link_function('glGetNamedProgramLocalParameterIuivEXT', None, [GLuint, GLenum, GLuint, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10754
glTextureParameterIivEXT = _link_function('glTextureParameterIivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10755
glTextureParameterIuivEXT = _link_function('glTextureParameterIuivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10756
glGetTextureParameterIivEXT = _link_function('glGetTextureParameterIivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10757
glGetTextureParameterIuivEXT = _link_function('glGetTextureParameterIuivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10758
glMultiTexParameterIivEXT = _link_function('glMultiTexParameterIivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10759
glMultiTexParameterIuivEXT = _link_function('glMultiTexParameterIuivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10760
glGetMultiTexParameterIivEXT = _link_function('glGetMultiTexParameterIivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10761
glGetMultiTexParameterIuivEXT = _link_function('glGetMultiTexParameterIuivEXT', None, [GLenum, GLenum, GLenum, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10762
glProgramUniform1fEXT = _link_function('glProgramUniform1fEXT', None, [GLuint, GLint, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10763
glProgramUniform2fEXT = _link_function('glProgramUniform2fEXT', None, [GLuint, GLint, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10764
glProgramUniform3fEXT = _link_function('glProgramUniform3fEXT', None, [GLuint, GLint, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10765
glProgramUniform4fEXT = _link_function('glProgramUniform4fEXT', None, [GLuint, GLint, GLfloat, GLfloat, GLfloat, GLfloat], 'EXT_direct_state_access')

# GL/glext.h:10766
glProgramUniform1iEXT = _link_function('glProgramUniform1iEXT', None, [GLuint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10767
glProgramUniform2iEXT = _link_function('glProgramUniform2iEXT', None, [GLuint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10768
glProgramUniform3iEXT = _link_function('glProgramUniform3iEXT', None, [GLuint, GLint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10769
glProgramUniform4iEXT = _link_function('glProgramUniform4iEXT', None, [GLuint, GLint, GLint, GLint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10770
glProgramUniform1fvEXT = _link_function('glProgramUniform1fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10771
glProgramUniform2fvEXT = _link_function('glProgramUniform2fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10772
glProgramUniform3fvEXT = _link_function('glProgramUniform3fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10773
glProgramUniform4fvEXT = _link_function('glProgramUniform4fvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10774
glProgramUniform1ivEXT = _link_function('glProgramUniform1ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10775
glProgramUniform2ivEXT = _link_function('glProgramUniform2ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10776
glProgramUniform3ivEXT = _link_function('glProgramUniform3ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10777
glProgramUniform4ivEXT = _link_function('glProgramUniform4ivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10778
glProgramUniformMatrix2fvEXT = _link_function('glProgramUniformMatrix2fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10779
glProgramUniformMatrix3fvEXT = _link_function('glProgramUniformMatrix3fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10780
glProgramUniformMatrix4fvEXT = _link_function('glProgramUniformMatrix4fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10781
glProgramUniformMatrix2x3fvEXT = _link_function('glProgramUniformMatrix2x3fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10782
glProgramUniformMatrix3x2fvEXT = _link_function('glProgramUniformMatrix3x2fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10783
glProgramUniformMatrix2x4fvEXT = _link_function('glProgramUniformMatrix2x4fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10784
glProgramUniformMatrix4x2fvEXT = _link_function('glProgramUniformMatrix4x2fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10785
glProgramUniformMatrix3x4fvEXT = _link_function('glProgramUniformMatrix3x4fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10786
glProgramUniformMatrix4x3fvEXT = _link_function('glProgramUniformMatrix4x3fvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)], 'EXT_direct_state_access')

# GL/glext.h:10787
glProgramUniform1uiEXT = _link_function('glProgramUniform1uiEXT', None, [GLuint, GLint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10788
glProgramUniform2uiEXT = _link_function('glProgramUniform2uiEXT', None, [GLuint, GLint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10789
glProgramUniform3uiEXT = _link_function('glProgramUniform3uiEXT', None, [GLuint, GLint, GLuint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10790
glProgramUniform4uiEXT = _link_function('glProgramUniform4uiEXT', None, [GLuint, GLint, GLuint, GLuint, GLuint, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10791
glProgramUniform1uivEXT = _link_function('glProgramUniform1uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10792
glProgramUniform2uivEXT = _link_function('glProgramUniform2uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10793
glProgramUniform3uivEXT = _link_function('glProgramUniform3uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10794
glProgramUniform4uivEXT = _link_function('glProgramUniform4uivEXT', None, [GLuint, GLint, GLsizei, POINTER(GLuint)], 'EXT_direct_state_access')

# GL/glext.h:10795
glNamedBufferDataEXT = _link_function('glNamedBufferDataEXT', None, [GLuint, GLsizeiptr, POINTER(GLvoid), GLenum], 'EXT_direct_state_access')

# GL/glext.h:10796
glNamedBufferSubDataEXT = _link_function('glNamedBufferSubDataEXT', None, [GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10797
glMapNamedBufferEXT = _link_function('glMapNamedBufferEXT', POINTER(GLvoid), [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10798
glUnmapNamedBufferEXT = _link_function('glUnmapNamedBufferEXT', GLboolean, [GLuint], 'EXT_direct_state_access')

# GL/glext.h:10799
glMapNamedBufferRangeEXT = _link_function('glMapNamedBufferRangeEXT', POINTER(GLvoid), [GLuint, GLintptr, GLsizeiptr, GLbitfield], 'EXT_direct_state_access')

# GL/glext.h:10800
glFlushMappedNamedBufferRangeEXT = _link_function('glFlushMappedNamedBufferRangeEXT', None, [GLuint, GLintptr, GLsizeiptr], 'EXT_direct_state_access')

# GL/glext.h:10801
glNamedCopyBufferSubDataEXT = _link_function('glNamedCopyBufferSubDataEXT', None, [GLuint, GLuint, GLintptr, GLintptr, GLsizeiptr], 'EXT_direct_state_access')

# GL/glext.h:10802
glGetNamedBufferParameterivEXT = _link_function('glGetNamedBufferParameterivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10803
glGetNamedBufferPointervEXT = _link_function('glGetNamedBufferPointervEXT', None, [GLuint, GLenum, POINTER(POINTER(GLvoid))], 'EXT_direct_state_access')

# GL/glext.h:10804
glGetNamedBufferSubDataEXT = _link_function('glGetNamedBufferSubDataEXT', None, [GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)], 'EXT_direct_state_access')

# GL/glext.h:10805
glTextureBufferEXT = _link_function('glTextureBufferEXT', None, [GLuint, GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10806
glMultiTexBufferEXT = _link_function('glMultiTexBufferEXT', None, [GLenum, GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10807
glNamedRenderbufferStorageEXT = _link_function('glNamedRenderbufferStorageEXT', None, [GLuint, GLenum, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10808
glGetNamedRenderbufferParameterivEXT = _link_function('glGetNamedRenderbufferParameterivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10809
glCheckNamedFramebufferStatusEXT = _link_function('glCheckNamedFramebufferStatusEXT', GLenum, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10810
glNamedFramebufferTexture1DEXT = _link_function('glNamedFramebufferTexture1DEXT', None, [GLuint, GLenum, GLenum, GLuint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10811
glNamedFramebufferTexture2DEXT = _link_function('glNamedFramebufferTexture2DEXT', None, [GLuint, GLenum, GLenum, GLuint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10812
glNamedFramebufferTexture3DEXT = _link_function('glNamedFramebufferTexture3DEXT', None, [GLuint, GLenum, GLenum, GLuint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10813
glNamedFramebufferRenderbufferEXT = _link_function('glNamedFramebufferRenderbufferEXT', None, [GLuint, GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10814
glGetNamedFramebufferAttachmentParameterivEXT = _link_function('glGetNamedFramebufferAttachmentParameterivEXT', None, [GLuint, GLenum, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10815
glGenerateTextureMipmapEXT = _link_function('glGenerateTextureMipmapEXT', None, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10816
glGenerateMultiTexMipmapEXT = _link_function('glGenerateMultiTexMipmapEXT', None, [GLenum, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10817
glFramebufferDrawBufferEXT = _link_function('glFramebufferDrawBufferEXT', None, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10818
glFramebufferDrawBuffersEXT = _link_function('glFramebufferDrawBuffersEXT', None, [GLuint, GLsizei, POINTER(GLenum)], 'EXT_direct_state_access')

# GL/glext.h:10819
glFramebufferReadBufferEXT = _link_function('glFramebufferReadBufferEXT', None, [GLuint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10820
glGetFramebufferParameterivEXT = _link_function('glGetFramebufferParameterivEXT', None, [GLuint, GLenum, POINTER(GLint)], 'EXT_direct_state_access')

# GL/glext.h:10821
glNamedRenderbufferStorageMultisampleEXT = _link_function('glNamedRenderbufferStorageMultisampleEXT', None, [GLuint, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10822
glNamedRenderbufferStorageMultisampleCoverageEXT = _link_function('glNamedRenderbufferStorageMultisampleCoverageEXT', None, [GLuint, GLsizei, GLsizei, GLenum, GLsizei, GLsizei], 'EXT_direct_state_access')

# GL/glext.h:10823
glNamedFramebufferTextureEXT = _link_function('glNamedFramebufferTextureEXT', None, [GLuint, GLenum, GLuint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10824
glNamedFramebufferTextureLayerEXT = _link_function('glNamedFramebufferTextureLayerEXT', None, [GLuint, GLenum, GLuint, GLint, GLint], 'EXT_direct_state_access')

# GL/glext.h:10825
glNamedFramebufferTextureFaceEXT = _link_function('glNamedFramebufferTextureFaceEXT', None, [GLuint, GLenum, GLuint, GLint, GLenum], 'EXT_direct_state_access')

# GL/glext.h:10826
glTextureRenderbufferEXT = _link_function('glTextureRenderbufferEXT', None, [GLuint, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10827
glMultiTexRenderbufferEXT = _link_function('glMultiTexRenderbufferEXT', None, [GLenum, GLenum, GLuint], 'EXT_direct_state_access')

# GL/glext.h:10828
glProgramUniform1dEXT = _link_function('glProgramUniform1dEXT', None, [GLuint, GLint, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10829
glProgramUniform2dEXT = _link_function('glProgramUniform2dEXT', None, [GLuint, GLint, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10830
glProgramUniform3dEXT = _link_function('glProgramUniform3dEXT', None, [GLuint, GLint, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10831
glProgramUniform4dEXT = _link_function('glProgramUniform4dEXT', None, [GLuint, GLint, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_direct_state_access')

# GL/glext.h:10832
glProgramUniform1dvEXT = _link_function('glProgramUniform1dvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10833
glProgramUniform2dvEXT = _link_function('glProgramUniform2dvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10834
glProgramUniform3dvEXT = _link_function('glProgramUniform3dvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10835
glProgramUniform4dvEXT = _link_function('glProgramUniform4dvEXT', None, [GLuint, GLint, GLsizei, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10836
glProgramUniformMatrix2dvEXT = _link_function('glProgramUniformMatrix2dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10837
glProgramUniformMatrix3dvEXT = _link_function('glProgramUniformMatrix3dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10838
glProgramUniformMatrix4dvEXT = _link_function('glProgramUniformMatrix4dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10839
glProgramUniformMatrix2x3dvEXT = _link_function('glProgramUniformMatrix2x3dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10840
glProgramUniformMatrix2x4dvEXT = _link_function('glProgramUniformMatrix2x4dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10841
glProgramUniformMatrix3x2dvEXT = _link_function('glProgramUniformMatrix3x2dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10842
glProgramUniformMatrix3x4dvEXT = _link_function('glProgramUniformMatrix3x4dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10843
glProgramUniformMatrix4x2dvEXT = _link_function('glProgramUniformMatrix4x2dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

# GL/glext.h:10844
glProgramUniformMatrix4x3dvEXT = _link_function('glProgramUniformMatrix4x3dvEXT', None, [GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)], 'EXT_direct_state_access')

PFNGLCLIENTATTRIBDEFAULTEXTPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:10846
PFNGLPUSHCLIENTATTRIBDEFAULTEXTPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:10847
PFNGLMATRIXLOADFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10848
PFNGLMATRIXLOADDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:10849
PFNGLMATRIXMULTFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10850
PFNGLMATRIXMULTDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:10851
PFNGLMATRIXLOADIDENTITYEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10852
PFNGLMATRIXROTATEFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:10853
PFNGLMATRIXROTATEDEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:10854
PFNGLMATRIXSCALEFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:10855
PFNGLMATRIXSCALEDEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:10856
PFNGLMATRIXTRANSLATEFEXTPROC = CFUNCTYPE(None, GLenum, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:10857
PFNGLMATRIXTRANSLATEDEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:10858
PFNGLMATRIXFRUSTUMEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:10859
PFNGLMATRIXORTHOEXTPROC = CFUNCTYPE(None, GLenum, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:10860
PFNGLMATRIXPOPEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10861
PFNGLMATRIXPUSHEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:10862
PFNGLMATRIXLOADTRANSPOSEFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10863
PFNGLMATRIXLOADTRANSPOSEDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:10864
PFNGLMATRIXMULTTRANSPOSEFEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10865
PFNGLMATRIXMULTTRANSPOSEDEXTPROC = CFUNCTYPE(None, GLenum, POINTER(GLdouble)) 	# GL/glext.h:10866
PFNGLTEXTUREPARAMETERFEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLfloat) 	# GL/glext.h:10867
PFNGLTEXTUREPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10868
PFNGLTEXTUREPARAMETERIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLint) 	# GL/glext.h:10869
PFNGLTEXTUREPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10870
PFNGLTEXTUREIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10871
PFNGLTEXTUREIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10872
PFNGLTEXTURESUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10873
PFNGLTEXTURESUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10874
PFNGLCOPYTEXTUREIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:10875
PFNGLCOPYTEXTUREIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:10876
PFNGLCOPYTEXTURESUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:10877
PFNGLCOPYTEXTURESUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:10878
PFNGLGETTEXTUREIMAGEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10879
PFNGLGETTEXTUREPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10880
PFNGLGETTEXTUREPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10881
PFNGLGETTEXTURELEVELPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10882
PFNGLGETTEXTURELEVELPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, POINTER(GLint)) 	# GL/glext.h:10883
PFNGLTEXTUREIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10884
PFNGLTEXTURESUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10885
PFNGLCOPYTEXTURESUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:10886
PFNGLMULTITEXPARAMETERFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLfloat) 	# GL/glext.h:10887
PFNGLMULTITEXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10888
PFNGLMULTITEXPARAMETERIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLint) 	# GL/glext.h:10889
PFNGLMULTITEXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10890
PFNGLMULTITEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10891
PFNGLMULTITEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10892
PFNGLMULTITEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10893
PFNGLMULTITEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10894
PFNGLCOPYMULTITEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLint) 	# GL/glext.h:10895
PFNGLCOPYMULTITEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint) 	# GL/glext.h:10896
PFNGLCOPYMULTITEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei) 	# GL/glext.h:10897
PFNGLCOPYMULTITEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:10898
PFNGLGETMULTITEXIMAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10899
PFNGLGETMULTITEXPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10900
PFNGLGETMULTITEXPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10901
PFNGLGETMULTITEXLEVELPARAMETERFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10902
PFNGLGETMULTITEXLEVELPARAMETERIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, POINTER(GLint)) 	# GL/glext.h:10903
PFNGLMULTITEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10904
PFNGLMULTITEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10905
PFNGLCOPYMULTITEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei) 	# GL/glext.h:10906
PFNGLBINDMULTITEXTUREEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:10907
PFNGLENABLECLIENTSTATEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:10908
PFNGLDISABLECLIENTSTATEINDEXEDEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:10909
PFNGLMULTITEXCOORDPOINTEREXTPROC = CFUNCTYPE(None, GLenum, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10910
PFNGLMULTITEXENVFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLfloat) 	# GL/glext.h:10911
PFNGLMULTITEXENVFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10912
PFNGLMULTITEXENVIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLint) 	# GL/glext.h:10913
PFNGLMULTITEXENVIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10914
PFNGLMULTITEXGENDEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLdouble) 	# GL/glext.h:10915
PFNGLMULTITEXGENDVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLdouble)) 	# GL/glext.h:10916
PFNGLMULTITEXGENFEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLfloat) 	# GL/glext.h:10917
PFNGLMULTITEXGENFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10918
PFNGLMULTITEXGENIEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLint) 	# GL/glext.h:10919
PFNGLMULTITEXGENIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10920
PFNGLGETMULTITEXENVFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10921
PFNGLGETMULTITEXENVIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10922
PFNGLGETMULTITEXGENDVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLdouble)) 	# GL/glext.h:10923
PFNGLGETMULTITEXGENFVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:10924
PFNGLGETMULTITEXGENIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10925
PFNGLGETFLOATINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:10926
PFNGLGETDOUBLEINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:10927
PFNGLGETPOINTERINDEXEDVEXTPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(POINTER(GLvoid))) 	# GL/glext.h:10928
PFNGLCOMPRESSEDTEXTUREIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10929
PFNGLCOMPRESSEDTEXTUREIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10930
PFNGLCOMPRESSEDTEXTUREIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10931
PFNGLCOMPRESSEDTEXTURESUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10932
PFNGLCOMPRESSEDTEXTURESUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10933
PFNGLCOMPRESSEDTEXTURESUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10934
PFNGLGETCOMPRESSEDTEXTUREIMAGEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:10935
PFNGLCOMPRESSEDMULTITEXIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10936
PFNGLCOMPRESSEDMULTITEXIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10937
PFNGLCOMPRESSEDMULTITEXIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLenum, GLsizei, GLint, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10938
PFNGLCOMPRESSEDMULTITEXSUBIMAGE3DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10939
PFNGLCOMPRESSEDMULTITEXSUBIMAGE2DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10940
PFNGLCOMPRESSEDMULTITEXSUBIMAGE1DEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, GLint, GLsizei, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10941
PFNGLGETCOMPRESSEDMULTITEXIMAGEEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, POINTER(GLvoid)) 	# GL/glext.h:10942
PFNGLNAMEDPROGRAMSTRINGEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:10943
PFNGLNAMEDPROGRAMLOCALPARAMETER4DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:10944
PFNGLNAMEDPROGRAMLOCALPARAMETER4DVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:10945
PFNGLNAMEDPROGRAMLOCALPARAMETER4FEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:10946
PFNGLNAMEDPROGRAMLOCALPARAMETER4FVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:10947
PFNGLGETNAMEDPROGRAMLOCALPARAMETERDVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLdouble)) 	# GL/glext.h:10948
PFNGLGETNAMEDPROGRAMLOCALPARAMETERFVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:10949
PFNGLGETNAMEDPROGRAMIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10950
PFNGLGETNAMEDPROGRAMSTRINGEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLvoid)) 	# GL/glext.h:10951
PFNGLNAMEDPROGRAMLOCALPARAMETERS4FVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10952
PFNGLNAMEDPROGRAMLOCALPARAMETERI4IEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:10953
PFNGLNAMEDPROGRAMLOCALPARAMETERI4IVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10954
PFNGLNAMEDPROGRAMLOCALPARAMETERSI4IVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10955
PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10956
PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:10957
PFNGLNAMEDPROGRAMLOCALPARAMETERSI4UIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10958
PFNGLGETNAMEDPROGRAMLOCALPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLint)) 	# GL/glext.h:10959
PFNGLGETNAMEDPROGRAMLOCALPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:10960
PFNGLTEXTUREPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10961
PFNGLTEXTUREPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:10962
PFNGLGETTEXTUREPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10963
PFNGLGETTEXTUREPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:10964
PFNGLMULTITEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10965
PFNGLMULTITEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:10966
PFNGLGETMULTITEXPARAMETERIIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:10967
PFNGLGETMULTITEXPARAMETERIUIVEXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLuint)) 	# GL/glext.h:10968
PFNGLPROGRAMUNIFORM1FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat) 	# GL/glext.h:10969
PFNGLPROGRAMUNIFORM2FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat) 	# GL/glext.h:10970
PFNGLPROGRAMUNIFORM3FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:10971
PFNGLPROGRAMUNIFORM4FEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLfloat, GLfloat, GLfloat, GLfloat) 	# GL/glext.h:10972
PFNGLPROGRAMUNIFORM1IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint) 	# GL/glext.h:10973
PFNGLPROGRAMUNIFORM2IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint) 	# GL/glext.h:10974
PFNGLPROGRAMUNIFORM3IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint) 	# GL/glext.h:10975
PFNGLPROGRAMUNIFORM4IEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLint, GLint, GLint, GLint) 	# GL/glext.h:10976
PFNGLPROGRAMUNIFORM1FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10977
PFNGLPROGRAMUNIFORM2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10978
PFNGLPROGRAMUNIFORM3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10979
PFNGLPROGRAMUNIFORM4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:10980
PFNGLPROGRAMUNIFORM1IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10981
PFNGLPROGRAMUNIFORM2IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10982
PFNGLPROGRAMUNIFORM3IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10983
PFNGLPROGRAMUNIFORM4IVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint)) 	# GL/glext.h:10984
PFNGLPROGRAMUNIFORMMATRIX2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10985
PFNGLPROGRAMUNIFORMMATRIX3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10986
PFNGLPROGRAMUNIFORMMATRIX4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10987
PFNGLPROGRAMUNIFORMMATRIX2X3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10988
PFNGLPROGRAMUNIFORMMATRIX3X2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10989
PFNGLPROGRAMUNIFORMMATRIX2X4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10990
PFNGLPROGRAMUNIFORMMATRIX4X2FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10991
PFNGLPROGRAMUNIFORMMATRIX3X4FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10992
PFNGLPROGRAMUNIFORMMATRIX4X3FVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLfloat)) 	# GL/glext.h:10993
PFNGLPROGRAMUNIFORM1UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:10994
PFNGLPROGRAMUNIFORM2UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint) 	# GL/glext.h:10995
PFNGLPROGRAMUNIFORM3UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint, GLuint) 	# GL/glext.h:10996
PFNGLPROGRAMUNIFORM4UIEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLuint, GLuint, GLuint, GLuint) 	# GL/glext.h:10997
PFNGLPROGRAMUNIFORM1UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10998
PFNGLPROGRAMUNIFORM2UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:10999
PFNGLPROGRAMUNIFORM3UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11000
PFNGLPROGRAMUNIFORM4UIVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11001
PFNGLNAMEDBUFFERDATAEXTPROC = CFUNCTYPE(None, GLuint, GLsizeiptr, POINTER(GLvoid), GLenum) 	# GL/glext.h:11002
PFNGLNAMEDBUFFERSUBDATAEXTPROC = CFUNCTYPE(None, GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:11003
PFNGLMAPNAMEDBUFFEREXTPROC = CFUNCTYPE(POINTER(GLvoid), GLuint, GLenum) 	# GL/glext.h:11004
PFNGLUNMAPNAMEDBUFFEREXTPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:11005
PFNGLMAPNAMEDBUFFERRANGEEXTPROC = CFUNCTYPE(POINTER(GLvoid), GLuint, GLintptr, GLsizeiptr, GLbitfield) 	# GL/glext.h:11006
PFNGLFLUSHMAPPEDNAMEDBUFFERRANGEEXTPROC = CFUNCTYPE(None, GLuint, GLintptr, GLsizeiptr) 	# GL/glext.h:11007
PFNGLNAMEDCOPYBUFFERSUBDATAEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLintptr, GLintptr, GLsizeiptr) 	# GL/glext.h:11008
PFNGLGETNAMEDBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11009
PFNGLGETNAMEDBUFFERPOINTERVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:11010
PFNGLGETNAMEDBUFFERSUBDATAEXTPROC = CFUNCTYPE(None, GLuint, GLintptr, GLsizeiptr, POINTER(GLvoid)) 	# GL/glext.h:11011
PFNGLTEXTUREBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint) 	# GL/glext.h:11012
PFNGLMULTITEXBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, GLuint) 	# GL/glext.h:11013
PFNGLNAMEDRENDERBUFFERSTORAGEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLsizei) 	# GL/glext.h:11014
PFNGLGETNAMEDRENDERBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11015
PFNGLCHECKNAMEDFRAMEBUFFERSTATUSEXTPROC = CFUNCTYPE(GLenum, GLuint, GLenum) 	# GL/glext.h:11016
PFNGLNAMEDFRAMEBUFFERTEXTURE1DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:11017
PFNGLNAMEDFRAMEBUFFERTEXTURE2DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint, GLint) 	# GL/glext.h:11018
PFNGLNAMEDFRAMEBUFFERTEXTURE3DEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:11019
PFNGLNAMEDFRAMEBUFFERRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLuint) 	# GL/glext.h:11020
PFNGLGETNAMEDFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:11021
PFNGLGENERATETEXTUREMIPMAPEXTPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11022
PFNGLGENERATEMULTITEXMIPMAPEXTPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:11023
PFNGLFRAMEBUFFERDRAWBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11024
PFNGLFRAMEBUFFERDRAWBUFFERSEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLenum)) 	# GL/glext.h:11025
PFNGLFRAMEBUFFERREADBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11026
PFNGLGETFRAMEBUFFERPARAMETERIVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11027
PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:11028
PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLECOVERAGEEXTPROC = CFUNCTYPE(None, GLuint, GLsizei, GLsizei, GLenum, GLsizei, GLsizei) 	# GL/glext.h:11029
PFNGLNAMEDFRAMEBUFFERTEXTUREEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint) 	# GL/glext.h:11030
PFNGLNAMEDFRAMEBUFFERTEXTURELAYEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint, GLint) 	# GL/glext.h:11031
PFNGLNAMEDFRAMEBUFFERTEXTUREFACEEXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint, GLint, GLenum) 	# GL/glext.h:11032
PFNGLTEXTURERENDERBUFFEREXTPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint) 	# GL/glext.h:11033
PFNGLMULTITEXRENDERBUFFEREXTPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint) 	# GL/glext.h:11034
PFNGLPROGRAMUNIFORM1DEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble) 	# GL/glext.h:11035
PFNGLPROGRAMUNIFORM2DEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble, GLdouble) 	# GL/glext.h:11036
PFNGLPROGRAMUNIFORM3DEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:11037
PFNGLPROGRAMUNIFORM4DEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:11038
PFNGLPROGRAMUNIFORM1DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:11039
PFNGLPROGRAMUNIFORM2DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:11040
PFNGLPROGRAMUNIFORM3DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:11041
PFNGLPROGRAMUNIFORM4DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLdouble)) 	# GL/glext.h:11042
PFNGLPROGRAMUNIFORMMATRIX2DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11043
PFNGLPROGRAMUNIFORMMATRIX3DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11044
PFNGLPROGRAMUNIFORMMATRIX4DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11045
PFNGLPROGRAMUNIFORMMATRIX2X3DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11046
PFNGLPROGRAMUNIFORMMATRIX2X4DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11047
PFNGLPROGRAMUNIFORMMATRIX3X2DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11048
PFNGLPROGRAMUNIFORMMATRIX3X4DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11049
PFNGLPROGRAMUNIFORMMATRIX4X2DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11050
PFNGLPROGRAMUNIFORMMATRIX4X3DVEXTPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, GLboolean, POINTER(GLdouble)) 	# GL/glext.h:11051
# EXT_vertex_array_bgra (GL/glext.h:11054)
GL_EXT_vertex_array_bgra = 1 	# GL/glext.h:11055
# EXT_texture_swizzle (GL/glext.h:11058)
GL_EXT_texture_swizzle = 1 	# GL/glext.h:11059
# NV_explicit_multisample (GL/glext.h:11062)
GL_NV_explicit_multisample = 1 	# GL/glext.h:11063
# GL/glext.h:11065
glGetMultisamplefvNV = _link_function('glGetMultisamplefvNV', None, [GLenum, GLuint, POINTER(GLfloat)], 'NV_explicit_multisample')

# GL/glext.h:11066
glSampleMaskIndexedNV = _link_function('glSampleMaskIndexedNV', None, [GLuint, GLbitfield], 'NV_explicit_multisample')

# GL/glext.h:11067
glTexRenderbufferNV = _link_function('glTexRenderbufferNV', None, [GLenum, GLuint], 'NV_explicit_multisample')

PFNGLGETMULTISAMPLEFVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:11069
PFNGLSAMPLEMASKINDEXEDNVPROC = CFUNCTYPE(None, GLuint, GLbitfield) 	# GL/glext.h:11070
PFNGLTEXRENDERBUFFERNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:11071
# NV_transform_feedback2 (GL/glext.h:11074)
GL_NV_transform_feedback2 = 1 	# GL/glext.h:11075
# GL/glext.h:11077
glBindTransformFeedbackNV = _link_function('glBindTransformFeedbackNV', None, [GLenum, GLuint], 'NV_transform_feedback2')

# GL/glext.h:11078
glDeleteTransformFeedbacksNV = _link_function('glDeleteTransformFeedbacksNV', None, [GLsizei, POINTER(GLuint)], 'NV_transform_feedback2')

# GL/glext.h:11079
glGenTransformFeedbacksNV = _link_function('glGenTransformFeedbacksNV', None, [GLsizei, POINTER(GLuint)], 'NV_transform_feedback2')

# GL/glext.h:11080
glIsTransformFeedbackNV = _link_function('glIsTransformFeedbackNV', GLboolean, [GLuint], 'NV_transform_feedback2')

# GL/glext.h:11081
glPauseTransformFeedbackNV = _link_function('glPauseTransformFeedbackNV', None, [], 'NV_transform_feedback2')

# GL/glext.h:11082
glResumeTransformFeedbackNV = _link_function('glResumeTransformFeedbackNV', None, [], 'NV_transform_feedback2')

# GL/glext.h:11083
glDrawTransformFeedbackNV = _link_function('glDrawTransformFeedbackNV', None, [GLenum, GLuint], 'NV_transform_feedback2')

PFNGLBINDTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:11085
PFNGLDELETETRANSFORMFEEDBACKSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11086
PFNGLGENTRANSFORMFEEDBACKSNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11087
PFNGLISTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:11088
PFNGLPAUSETRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:11089
PFNGLRESUMETRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None) 	# GL/glext.h:11090
PFNGLDRAWTRANSFORMFEEDBACKNVPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:11091
# ATI_meminfo (GL/glext.h:11094)
GL_ATI_meminfo = 1 	# GL/glext.h:11095
# AMD_performance_monitor (GL/glext.h:11098)
GL_AMD_performance_monitor = 1 	# GL/glext.h:11099
# GL/glext.h:11101
glGetPerfMonitorGroupsAMD = _link_function('glGetPerfMonitorGroupsAMD', None, [POINTER(GLint), GLsizei, POINTER(GLuint)], 'AMD_performance_monitor')

# GL/glext.h:11102
glGetPerfMonitorCountersAMD = _link_function('glGetPerfMonitorCountersAMD', None, [GLuint, POINTER(GLint), POINTER(GLint), GLsizei, POINTER(GLuint)], 'AMD_performance_monitor')

# GL/glext.h:11103
glGetPerfMonitorGroupStringAMD = _link_function('glGetPerfMonitorGroupStringAMD', None, [GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'AMD_performance_monitor')

# GL/glext.h:11104
glGetPerfMonitorCounterStringAMD = _link_function('glGetPerfMonitorCounterStringAMD', None, [GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)], 'AMD_performance_monitor')

# GL/glext.h:11105
glGetPerfMonitorCounterInfoAMD = _link_function('glGetPerfMonitorCounterInfoAMD', None, [GLuint, GLuint, GLenum, POINTER(GLvoid)], 'AMD_performance_monitor')

# GL/glext.h:11106
glGenPerfMonitorsAMD = _link_function('glGenPerfMonitorsAMD', None, [GLsizei, POINTER(GLuint)], 'AMD_performance_monitor')

# GL/glext.h:11107
glDeletePerfMonitorsAMD = _link_function('glDeletePerfMonitorsAMD', None, [GLsizei, POINTER(GLuint)], 'AMD_performance_monitor')

# GL/glext.h:11108
glSelectPerfMonitorCountersAMD = _link_function('glSelectPerfMonitorCountersAMD', None, [GLuint, GLboolean, GLuint, GLint, POINTER(GLuint)], 'AMD_performance_monitor')

# GL/glext.h:11109
glBeginPerfMonitorAMD = _link_function('glBeginPerfMonitorAMD', None, [GLuint], 'AMD_performance_monitor')

# GL/glext.h:11110
glEndPerfMonitorAMD = _link_function('glEndPerfMonitorAMD', None, [GLuint], 'AMD_performance_monitor')

# GL/glext.h:11111
glGetPerfMonitorCounterDataAMD = _link_function('glGetPerfMonitorCounterDataAMD', None, [GLuint, GLenum, GLsizei, POINTER(GLuint), POINTER(GLint)], 'AMD_performance_monitor')

PFNGLGETPERFMONITORGROUPSAMDPROC = CFUNCTYPE(None, POINTER(GLint), GLsizei, POINTER(GLuint)) 	# GL/glext.h:11113
PFNGLGETPERFMONITORCOUNTERSAMDPROC = CFUNCTYPE(None, GLuint, POINTER(GLint), POINTER(GLint), GLsizei, POINTER(GLuint)) 	# GL/glext.h:11114
PFNGLGETPERFMONITORGROUPSTRINGAMDPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:11115
PFNGLGETPERFMONITORCOUNTERSTRINGAMDPROC = CFUNCTYPE(None, GLuint, GLuint, GLsizei, POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:11116
PFNGLGETPERFMONITORCOUNTERINFOAMDPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLvoid)) 	# GL/glext.h:11117
PFNGLGENPERFMONITORSAMDPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11118
PFNGLDELETEPERFMONITORSAMDPROC = CFUNCTYPE(None, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11119
PFNGLSELECTPERFMONITORCOUNTERSAMDPROC = CFUNCTYPE(None, GLuint, GLboolean, GLuint, GLint, POINTER(GLuint)) 	# GL/glext.h:11120
PFNGLBEGINPERFMONITORAMDPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:11121
PFNGLENDPERFMONITORAMDPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:11122
PFNGLGETPERFMONITORCOUNTERDATAAMDPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, POINTER(GLuint), POINTER(GLint)) 	# GL/glext.h:11123
# AMD_texture_texture4 (GL/glext.h:11126)
GL_AMD_texture_texture4 = 1 	# GL/glext.h:11127
# AMD_vertex_shader_tesselator (GL/glext.h:11130)
GL_AMD_vertex_shader_tesselator = 1 	# GL/glext.h:11131
# GL/glext.h:11133
glTessellationFactorAMD = _link_function('glTessellationFactorAMD', None, [GLfloat], 'AMD_vertex_shader_tesselator')

# GL/glext.h:11134
glTessellationModeAMD = _link_function('glTessellationModeAMD', None, [GLenum], 'AMD_vertex_shader_tesselator')

PFNGLTESSELLATIONFACTORAMDPROC = CFUNCTYPE(None, GLfloat) 	# GL/glext.h:11136
PFNGLTESSELLATIONMODEAMDPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:11137
# EXT_provoking_vertex (GL/glext.h:11140)
GL_EXT_provoking_vertex = 1 	# GL/glext.h:11141
# GL/glext.h:11143
glProvokingVertexEXT = _link_function('glProvokingVertexEXT', None, [GLenum], 'EXT_provoking_vertex')

PFNGLPROVOKINGVERTEXEXTPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:11145
# EXT_texture_snorm (GL/glext.h:11148)
GL_EXT_texture_snorm = 1 	# GL/glext.h:11149
# AMD_draw_buffers_blend (GL/glext.h:11152)
GL_AMD_draw_buffers_blend = 1 	# GL/glext.h:11153
# GL/glext.h:11155
glBlendFuncIndexedAMD = _link_function('glBlendFuncIndexedAMD', None, [GLuint, GLenum, GLenum], 'AMD_draw_buffers_blend')

# GL/glext.h:11156
glBlendFuncSeparateIndexedAMD = _link_function('glBlendFuncSeparateIndexedAMD', None, [GLuint, GLenum, GLenum, GLenum, GLenum], 'AMD_draw_buffers_blend')

# GL/glext.h:11157
glBlendEquationIndexedAMD = _link_function('glBlendEquationIndexedAMD', None, [GLuint, GLenum], 'AMD_draw_buffers_blend')

# GL/glext.h:11158
glBlendEquationSeparateIndexedAMD = _link_function('glBlendEquationSeparateIndexedAMD', None, [GLuint, GLenum, GLenum], 'AMD_draw_buffers_blend')

PFNGLBLENDFUNCINDEXEDAMDPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum) 	# GL/glext.h:11160
PFNGLBLENDFUNCSEPARATEINDEXEDAMDPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum, GLenum, GLenum) 	# GL/glext.h:11161
PFNGLBLENDEQUATIONINDEXEDAMDPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11162
PFNGLBLENDEQUATIONSEPARATEINDEXEDAMDPROC = CFUNCTYPE(None, GLuint, GLenum, GLenum) 	# GL/glext.h:11163
# APPLE_texture_range (GL/glext.h:11166)
GL_APPLE_texture_range = 1 	# GL/glext.h:11167
# GL/glext.h:11169
glTextureRangeAPPLE = _link_function('glTextureRangeAPPLE', None, [GLenum, GLsizei, POINTER(GLvoid)], 'APPLE_texture_range')

# GL/glext.h:11170
glGetTexParameterPointervAPPLE = _link_function('glGetTexParameterPointervAPPLE', None, [GLenum, GLenum, POINTER(POINTER(GLvoid))], 'APPLE_texture_range')

PFNGLTEXTURERANGEAPPLEPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:11172
PFNGLGETTEXPARAMETERPOINTERVAPPLEPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(POINTER(GLvoid))) 	# GL/glext.h:11173
# APPLE_float_pixels (GL/glext.h:11176)
GL_APPLE_float_pixels = 1 	# GL/glext.h:11177
# APPLE_vertex_program_evaluators (GL/glext.h:11180)
GL_APPLE_vertex_program_evaluators = 1 	# GL/glext.h:11181
# GL/glext.h:11183
glEnableVertexAttribAPPLE = _link_function('glEnableVertexAttribAPPLE', None, [GLuint, GLenum], 'APPLE_vertex_program_evaluators')

# GL/glext.h:11184
glDisableVertexAttribAPPLE = _link_function('glDisableVertexAttribAPPLE', None, [GLuint, GLenum], 'APPLE_vertex_program_evaluators')

# GL/glext.h:11185
glIsVertexAttribEnabledAPPLE = _link_function('glIsVertexAttribEnabledAPPLE', GLboolean, [GLuint, GLenum], 'APPLE_vertex_program_evaluators')

# GL/glext.h:11186
glMapVertexAttrib1dAPPLE = _link_function('glMapVertexAttrib1dAPPLE', None, [GLuint, GLuint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)], 'APPLE_vertex_program_evaluators')

# GL/glext.h:11187
glMapVertexAttrib1fAPPLE = _link_function('glMapVertexAttrib1fAPPLE', None, [GLuint, GLuint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)], 'APPLE_vertex_program_evaluators')

# GL/glext.h:11188
glMapVertexAttrib2dAPPLE = _link_function('glMapVertexAttrib2dAPPLE', None, [GLuint, GLuint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)], 'APPLE_vertex_program_evaluators')

# GL/glext.h:11189
glMapVertexAttrib2fAPPLE = _link_function('glMapVertexAttrib2fAPPLE', None, [GLuint, GLuint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)], 'APPLE_vertex_program_evaluators')

PFNGLENABLEVERTEXATTRIBAPPLEPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11191
PFNGLDISABLEVERTEXATTRIBAPPLEPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11192
PFNGLISVERTEXATTRIBENABLEDAPPLEPROC = CFUNCTYPE(GLboolean, GLuint, GLenum) 	# GL/glext.h:11193
PFNGLMAPVERTEXATTRIB1DAPPLEPROC = CFUNCTYPE(None, GLuint, GLuint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)) 	# GL/glext.h:11194
PFNGLMAPVERTEXATTRIB1FAPPLEPROC = CFUNCTYPE(None, GLuint, GLuint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)) 	# GL/glext.h:11195
PFNGLMAPVERTEXATTRIB2DAPPLEPROC = CFUNCTYPE(None, GLuint, GLuint, GLdouble, GLdouble, GLint, GLint, GLdouble, GLdouble, GLint, GLint, POINTER(GLdouble)) 	# GL/glext.h:11196
PFNGLMAPVERTEXATTRIB2FAPPLEPROC = CFUNCTYPE(None, GLuint, GLuint, GLfloat, GLfloat, GLint, GLint, GLfloat, GLfloat, GLint, GLint, POINTER(GLfloat)) 	# GL/glext.h:11197
# APPLE_aux_depth_stencil (GL/glext.h:11200)
GL_APPLE_aux_depth_stencil = 1 	# GL/glext.h:11201
# APPLE_object_purgeable (GL/glext.h:11204)
GL_APPLE_object_purgeable = 1 	# GL/glext.h:11205
# GL/glext.h:11207
glObjectPurgeableAPPLE = _link_function('glObjectPurgeableAPPLE', GLenum, [GLenum, GLuint, GLenum], 'APPLE_object_purgeable')

# GL/glext.h:11208
glObjectUnpurgeableAPPLE = _link_function('glObjectUnpurgeableAPPLE', GLenum, [GLenum, GLuint, GLenum], 'APPLE_object_purgeable')

# GL/glext.h:11209
glGetObjectParameterivAPPLE = _link_function('glGetObjectParameterivAPPLE', None, [GLenum, GLuint, GLenum, POINTER(GLint)], 'APPLE_object_purgeable')

PFNGLOBJECTPURGEABLEAPPLEPROC = CFUNCTYPE(GLenum, GLenum, GLuint, GLenum) 	# GL/glext.h:11211
PFNGLOBJECTUNPURGEABLEAPPLEPROC = CFUNCTYPE(GLenum, GLenum, GLuint, GLenum) 	# GL/glext.h:11212
PFNGLGETOBJECTPARAMETERIVAPPLEPROC = CFUNCTYPE(None, GLenum, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11213
# APPLE_row_bytes (GL/glext.h:11216)
GL_APPLE_row_bytes = 1 	# GL/glext.h:11217
# APPLE_rgb_422 (GL/glext.h:11220)
GL_APPLE_rgb_422 = 1 	# GL/glext.h:11221
# NV_video_capture (GL/glext.h:11224)
GL_NV_video_capture = 1 	# GL/glext.h:11225
# GL/glext.h:11227
glBeginVideoCaptureNV = _link_function('glBeginVideoCaptureNV', None, [GLuint], 'NV_video_capture')

# GL/glext.h:11228
glBindVideoCaptureStreamBufferNV = _link_function('glBindVideoCaptureStreamBufferNV', None, [GLuint, GLuint, GLenum, GLintptrARB], 'NV_video_capture')

# GL/glext.h:11229
glBindVideoCaptureStreamTextureNV = _link_function('glBindVideoCaptureStreamTextureNV', None, [GLuint, GLuint, GLenum, GLenum, GLuint], 'NV_video_capture')

# GL/glext.h:11230
glEndVideoCaptureNV = _link_function('glEndVideoCaptureNV', None, [GLuint], 'NV_video_capture')

# GL/glext.h:11231
glGetVideoCaptureivNV = _link_function('glGetVideoCaptureivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_video_capture')

# GL/glext.h:11232
glGetVideoCaptureStreamivNV = _link_function('glGetVideoCaptureStreamivNV', None, [GLuint, GLuint, GLenum, POINTER(GLint)], 'NV_video_capture')

# GL/glext.h:11233
glGetVideoCaptureStreamfvNV = _link_function('glGetVideoCaptureStreamfvNV', None, [GLuint, GLuint, GLenum, POINTER(GLfloat)], 'NV_video_capture')

# GL/glext.h:11234
glGetVideoCaptureStreamdvNV = _link_function('glGetVideoCaptureStreamdvNV', None, [GLuint, GLuint, GLenum, POINTER(GLdouble)], 'NV_video_capture')

# GL/glext.h:11235
glVideoCaptureNV = _link_function('glVideoCaptureNV', GLenum, [GLuint, POINTER(GLuint), POINTER(GLuint64EXT)], 'NV_video_capture')

# GL/glext.h:11236
glVideoCaptureStreamParameterivNV = _link_function('glVideoCaptureStreamParameterivNV', None, [GLuint, GLuint, GLenum, POINTER(GLint)], 'NV_video_capture')

# GL/glext.h:11237
glVideoCaptureStreamParameterfvNV = _link_function('glVideoCaptureStreamParameterfvNV', None, [GLuint, GLuint, GLenum, POINTER(GLfloat)], 'NV_video_capture')

# GL/glext.h:11238
glVideoCaptureStreamParameterdvNV = _link_function('glVideoCaptureStreamParameterdvNV', None, [GLuint, GLuint, GLenum, POINTER(GLdouble)], 'NV_video_capture')

PFNGLBEGINVIDEOCAPTURENVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:11240
PFNGLBINDVIDEOCAPTURESTREAMBUFFERNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLintptrARB) 	# GL/glext.h:11241
PFNGLBINDVIDEOCAPTURESTREAMTEXTURENVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, GLenum, GLuint) 	# GL/glext.h:11242
PFNGLENDVIDEOCAPTURENVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:11243
PFNGLGETVIDEOCAPTUREIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11244
PFNGLGETVIDEOCAPTURESTREAMIVNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11245
PFNGLGETVIDEOCAPTURESTREAMFVNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11246
PFNGLGETVIDEOCAPTURESTREAMDVNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:11247
PFNGLVIDEOCAPTURENVPROC = CFUNCTYPE(GLenum, GLuint, POINTER(GLuint), POINTER(GLuint64EXT)) 	# GL/glext.h:11248
PFNGLVIDEOCAPTURESTREAMPARAMETERIVNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11249
PFNGLVIDEOCAPTURESTREAMPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11250
PFNGLVIDEOCAPTURESTREAMPARAMETERDVNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:11251
# NV_copy_image (GL/glext.h:11254)
GL_NV_copy_image = 1 	# GL/glext.h:11255
# GL/glext.h:11257
glCopyImageSubDataNV = _link_function('glCopyImageSubDataNV', None, [GLuint, GLenum, GLint, GLint, GLint, GLint, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei], 'NV_copy_image')

PFNGLCOPYIMAGESUBDATANVPROC = CFUNCTYPE(None, GLuint, GLenum, GLint, GLint, GLint, GLint, GLuint, GLenum, GLint, GLint, GLint, GLint, GLsizei, GLsizei, GLsizei) 	# GL/glext.h:11259
# EXT_separate_shader_objects (GL/glext.h:11262)
GL_EXT_separate_shader_objects = 1 	# GL/glext.h:11263
# GL/glext.h:11265
glUseShaderProgramEXT = _link_function('glUseShaderProgramEXT', None, [GLenum, GLuint], 'EXT_separate_shader_objects')

# GL/glext.h:11266
glActiveProgramEXT = _link_function('glActiveProgramEXT', None, [GLuint], 'EXT_separate_shader_objects')

# GL/glext.h:11267
glCreateShaderProgramEXT = _link_function('glCreateShaderProgramEXT', GLuint, [GLenum, POINTER(GLchar)], 'EXT_separate_shader_objects')

PFNGLUSESHADERPROGRAMEXTPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:11269
PFNGLACTIVEPROGRAMEXTPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:11270
PFNGLCREATESHADERPROGRAMEXTPROC = CFUNCTYPE(GLuint, GLenum, POINTER(GLchar)) 	# GL/glext.h:11271
# NV_parameter_buffer_object2 (GL/glext.h:11274)
GL_NV_parameter_buffer_object2 = 1 	# GL/glext.h:11275
# NV_shader_buffer_load (GL/glext.h:11278)
GL_NV_shader_buffer_load = 1 	# GL/glext.h:11279
# GL/glext.h:11281
glMakeBufferResidentNV = _link_function('glMakeBufferResidentNV', None, [GLenum, GLenum], 'NV_shader_buffer_load')

# GL/glext.h:11282
glMakeBufferNonResidentNV = _link_function('glMakeBufferNonResidentNV', None, [GLenum], 'NV_shader_buffer_load')

# GL/glext.h:11283
glIsBufferResidentNV = _link_function('glIsBufferResidentNV', GLboolean, [GLenum], 'NV_shader_buffer_load')

# GL/glext.h:11284
glMakeNamedBufferResidentNV = _link_function('glMakeNamedBufferResidentNV', None, [GLuint, GLenum], 'NV_shader_buffer_load')

# GL/glext.h:11285
glMakeNamedBufferNonResidentNV = _link_function('glMakeNamedBufferNonResidentNV', None, [GLuint], 'NV_shader_buffer_load')

# GL/glext.h:11286
glIsNamedBufferResidentNV = _link_function('glIsNamedBufferResidentNV', GLboolean, [GLuint], 'NV_shader_buffer_load')

# GL/glext.h:11287
glGetBufferParameterui64vNV = _link_function('glGetBufferParameterui64vNV', None, [GLenum, GLenum, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:11288
glGetNamedBufferParameterui64vNV = _link_function('glGetNamedBufferParameterui64vNV', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:11289
glGetIntegerui64vNV = _link_function('glGetIntegerui64vNV', None, [GLenum, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:11290
glUniformui64NV = _link_function('glUniformui64NV', None, [GLint, GLuint64EXT], 'NV_shader_buffer_load')

# GL/glext.h:11291
glUniformui64vNV = _link_function('glUniformui64vNV', None, [GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:11292
glGetUniformui64vNV = _link_function('glGetUniformui64vNV', None, [GLuint, GLint, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

# GL/glext.h:11293
glProgramUniformui64NV = _link_function('glProgramUniformui64NV', None, [GLuint, GLint, GLuint64EXT], 'NV_shader_buffer_load')

# GL/glext.h:11294
glProgramUniformui64vNV = _link_function('glProgramUniformui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_shader_buffer_load')

PFNGLMAKEBUFFERRESIDENTNVPROC = CFUNCTYPE(None, GLenum, GLenum) 	# GL/glext.h:11296
PFNGLMAKEBUFFERNONRESIDENTNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:11297
PFNGLISBUFFERRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLenum) 	# GL/glext.h:11298
PFNGLMAKENAMEDBUFFERRESIDENTNVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11299
PFNGLMAKENAMEDBUFFERNONRESIDENTNVPROC = CFUNCTYPE(None, GLuint) 	# GL/glext.h:11300
PFNGLISNAMEDBUFFERRESIDENTNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:11301
PFNGLGETBUFFERPARAMETERUI64VNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:11302
PFNGLGETNAMEDBUFFERPARAMETERUI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:11303
PFNGLGETINTEGERUI64VNVPROC = CFUNCTYPE(None, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:11304
PFNGLUNIFORMUI64NVPROC = CFUNCTYPE(None, GLint, GLuint64EXT) 	# GL/glext.h:11305
PFNGLUNIFORMUI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11306
PFNGLGETUNIFORMUI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLuint64EXT)) 	# GL/glext.h:11307
PFNGLPROGRAMUNIFORMUI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64EXT) 	# GL/glext.h:11308
PFNGLPROGRAMUNIFORMUI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11309
# NV_vertex_buffer_unified_memory (GL/glext.h:11312)
GL_NV_vertex_buffer_unified_memory = 1 	# GL/glext.h:11313
# GL/glext.h:11315
glBufferAddressRangeNV = _link_function('glBufferAddressRangeNV', None, [GLenum, GLuint, GLuint64EXT, GLsizeiptr], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11316
glVertexFormatNV = _link_function('glVertexFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11317
glNormalFormatNV = _link_function('glNormalFormatNV', None, [GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11318
glColorFormatNV = _link_function('glColorFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11319
glIndexFormatNV = _link_function('glIndexFormatNV', None, [GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11320
glTexCoordFormatNV = _link_function('glTexCoordFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11321
glEdgeFlagFormatNV = _link_function('glEdgeFlagFormatNV', None, [GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11322
glSecondaryColorFormatNV = _link_function('glSecondaryColorFormatNV', None, [GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11323
glFogCoordFormatNV = _link_function('glFogCoordFormatNV', None, [GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11324
glVertexAttribFormatNV = _link_function('glVertexAttribFormatNV', None, [GLuint, GLint, GLenum, GLboolean, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11325
glVertexAttribIFormatNV = _link_function('glVertexAttribIFormatNV', None, [GLuint, GLint, GLenum, GLsizei], 'NV_vertex_buffer_unified_memory')

# GL/glext.h:11326
glGetIntegerui64i_vNV = _link_function('glGetIntegerui64i_vNV', None, [GLenum, GLuint, POINTER(GLuint64EXT)], 'NV_vertex_buffer_unified_memory')

PFNGLBUFFERADDRESSRANGENVPROC = CFUNCTYPE(None, GLenum, GLuint, GLuint64EXT, GLsizeiptr) 	# GL/glext.h:11328
PFNGLVERTEXFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:11329
PFNGLNORMALFORMATNVPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:11330
PFNGLCOLORFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:11331
PFNGLINDEXFORMATNVPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:11332
PFNGLTEXCOORDFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:11333
PFNGLEDGEFLAGFORMATNVPROC = CFUNCTYPE(None, GLsizei) 	# GL/glext.h:11334
PFNGLSECONDARYCOLORFORMATNVPROC = CFUNCTYPE(None, GLint, GLenum, GLsizei) 	# GL/glext.h:11335
PFNGLFOGCOORDFORMATNVPROC = CFUNCTYPE(None, GLenum, GLsizei) 	# GL/glext.h:11336
PFNGLVERTEXATTRIBFORMATNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLboolean, GLsizei) 	# GL/glext.h:11337
PFNGLVERTEXATTRIBIFORMATNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei) 	# GL/glext.h:11338
PFNGLGETINTEGERUI64I_VNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint64EXT)) 	# GL/glext.h:11339
# NV_texture_barrier (GL/glext.h:11342)
GL_NV_texture_barrier = 1 	# GL/glext.h:11343
# GL/glext.h:11345
glTextureBarrierNV = _link_function('glTextureBarrierNV', None, [], 'NV_texture_barrier')

PFNGLTEXTUREBARRIERNVPROC = CFUNCTYPE(None) 	# GL/glext.h:11347
# AMD_shader_stencil_export (GL/glext.h:11350)
GL_AMD_shader_stencil_export = 1 	# GL/glext.h:11351
# AMD_seamless_cubemap_per_texture (GL/glext.h:11354)
GL_AMD_seamless_cubemap_per_texture = 1 	# GL/glext.h:11355
# AMD_conservative_depth (GL/glext.h:11358)
GL_AMD_conservative_depth = 1 	# GL/glext.h:11359
# EXT_shader_image_load_store (GL/glext.h:11362)
GL_EXT_shader_image_load_store = 1 	# GL/glext.h:11363
# GL/glext.h:11365
glBindImageTextureEXT = _link_function('glBindImageTextureEXT', None, [GLuint, GLuint, GLint, GLboolean, GLint, GLenum, GLint], 'EXT_shader_image_load_store')

# GL/glext.h:11366
glMemoryBarrierEXT = _link_function('glMemoryBarrierEXT', None, [GLbitfield], 'EXT_shader_image_load_store')

PFNGLBINDIMAGETEXTUREEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLint, GLboolean, GLint, GLenum, GLint) 	# GL/glext.h:11368
PFNGLMEMORYBARRIEREXTPROC = CFUNCTYPE(None, GLbitfield) 	# GL/glext.h:11369
# EXT_vertex_attrib_64bit (GL/glext.h:11372)
GL_EXT_vertex_attrib_64bit = 1 	# GL/glext.h:11373
# GL/glext.h:11375
glVertexAttribL1dEXT = _link_function('glVertexAttribL1dEXT', None, [GLuint, GLdouble], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11376
glVertexAttribL2dEXT = _link_function('glVertexAttribL2dEXT', None, [GLuint, GLdouble, GLdouble], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11377
glVertexAttribL3dEXT = _link_function('glVertexAttribL3dEXT', None, [GLuint, GLdouble, GLdouble, GLdouble], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11378
glVertexAttribL4dEXT = _link_function('glVertexAttribL4dEXT', None, [GLuint, GLdouble, GLdouble, GLdouble, GLdouble], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11379
glVertexAttribL1dvEXT = _link_function('glVertexAttribL1dvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11380
glVertexAttribL2dvEXT = _link_function('glVertexAttribL2dvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11381
glVertexAttribL3dvEXT = _link_function('glVertexAttribL3dvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11382
glVertexAttribL4dvEXT = _link_function('glVertexAttribL4dvEXT', None, [GLuint, POINTER(GLdouble)], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11383
glVertexAttribLPointerEXT = _link_function('glVertexAttribLPointerEXT', None, [GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11384
glGetVertexAttribLdvEXT = _link_function('glGetVertexAttribLdvEXT', None, [GLuint, GLenum, POINTER(GLdouble)], 'EXT_vertex_attrib_64bit')

# GL/glext.h:11385
glVertexArrayVertexAttribLOffsetEXT = _link_function('glVertexArrayVertexAttribLOffsetEXT', None, [GLuint, GLuint, GLuint, GLint, GLenum, GLsizei, GLintptr], 'EXT_vertex_attrib_64bit')

PFNGLVERTEXATTRIBL1DEXTPROC = CFUNCTYPE(None, GLuint, GLdouble) 	# GL/glext.h:11387
PFNGLVERTEXATTRIBL2DEXTPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble) 	# GL/glext.h:11388
PFNGLVERTEXATTRIBL3DEXTPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:11389
PFNGLVERTEXATTRIBL4DEXTPROC = CFUNCTYPE(None, GLuint, GLdouble, GLdouble, GLdouble, GLdouble) 	# GL/glext.h:11390
PFNGLVERTEXATTRIBL1DVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:11391
PFNGLVERTEXATTRIBL2DVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:11392
PFNGLVERTEXATTRIBL3DVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:11393
PFNGLVERTEXATTRIBL4DVEXTPROC = CFUNCTYPE(None, GLuint, POINTER(GLdouble)) 	# GL/glext.h:11394
PFNGLVERTEXATTRIBLPOINTEREXTPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:11395
PFNGLGETVERTEXATTRIBLDVEXTPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLdouble)) 	# GL/glext.h:11396
PFNGLVERTEXARRAYVERTEXATTRIBLOFFSETEXTPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLint, GLenum, GLsizei, GLintptr) 	# GL/glext.h:11397
# NV_gpu_program5 (GL/glext.h:11400)
GL_NV_gpu_program5 = 1 	# GL/glext.h:11401
# GL/glext.h:11403
glProgramSubroutineParametersuivNV = _link_function('glProgramSubroutineParametersuivNV', None, [GLenum, GLsizei, POINTER(GLuint)], 'NV_gpu_program5')

# GL/glext.h:11404
glGetProgramSubroutineParameteruivNV = _link_function('glGetProgramSubroutineParameteruivNV', None, [GLenum, GLuint, POINTER(GLuint)], 'NV_gpu_program5')

PFNGLPROGRAMSUBROUTINEPARAMETERSUIVNVPROC = CFUNCTYPE(None, GLenum, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11406
PFNGLGETPROGRAMSUBROUTINEPARAMETERUIVNVPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:11407
# NV_gpu_shader5 (GL/glext.h:11410)
GL_NV_gpu_shader5 = 1 	# GL/glext.h:11411
# GL/glext.h:11413
glUniform1i64NV = _link_function('glUniform1i64NV', None, [GLint, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11414
glUniform2i64NV = _link_function('glUniform2i64NV', None, [GLint, GLint64EXT, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11415
glUniform3i64NV = _link_function('glUniform3i64NV', None, [GLint, GLint64EXT, GLint64EXT, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11416
glUniform4i64NV = _link_function('glUniform4i64NV', None, [GLint, GLint64EXT, GLint64EXT, GLint64EXT, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11417
glUniform1i64vNV = _link_function('glUniform1i64vNV', None, [GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11418
glUniform2i64vNV = _link_function('glUniform2i64vNV', None, [GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11419
glUniform3i64vNV = _link_function('glUniform3i64vNV', None, [GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11420
glUniform4i64vNV = _link_function('glUniform4i64vNV', None, [GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11421
glUniform1ui64NV = _link_function('glUniform1ui64NV', None, [GLint, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11422
glUniform2ui64NV = _link_function('glUniform2ui64NV', None, [GLint, GLuint64EXT, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11423
glUniform3ui64NV = _link_function('glUniform3ui64NV', None, [GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11424
glUniform4ui64NV = _link_function('glUniform4ui64NV', None, [GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11425
glUniform1ui64vNV = _link_function('glUniform1ui64vNV', None, [GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11426
glUniform2ui64vNV = _link_function('glUniform2ui64vNV', None, [GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11427
glUniform3ui64vNV = _link_function('glUniform3ui64vNV', None, [GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11428
glUniform4ui64vNV = _link_function('glUniform4ui64vNV', None, [GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11429
glGetUniformi64vNV = _link_function('glGetUniformi64vNV', None, [GLuint, GLint, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11430
glProgramUniform1i64NV = _link_function('glProgramUniform1i64NV', None, [GLuint, GLint, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11431
glProgramUniform2i64NV = _link_function('glProgramUniform2i64NV', None, [GLuint, GLint, GLint64EXT, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11432
glProgramUniform3i64NV = _link_function('glProgramUniform3i64NV', None, [GLuint, GLint, GLint64EXT, GLint64EXT, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11433
glProgramUniform4i64NV = _link_function('glProgramUniform4i64NV', None, [GLuint, GLint, GLint64EXT, GLint64EXT, GLint64EXT, GLint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11434
glProgramUniform1i64vNV = _link_function('glProgramUniform1i64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11435
glProgramUniform2i64vNV = _link_function('glProgramUniform2i64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11436
glProgramUniform3i64vNV = _link_function('glProgramUniform3i64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11437
glProgramUniform4i64vNV = _link_function('glProgramUniform4i64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11438
glProgramUniform1ui64NV = _link_function('glProgramUniform1ui64NV', None, [GLuint, GLint, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11439
glProgramUniform2ui64NV = _link_function('glProgramUniform2ui64NV', None, [GLuint, GLint, GLuint64EXT, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11440
glProgramUniform3ui64NV = _link_function('glProgramUniform3ui64NV', None, [GLuint, GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11441
glProgramUniform4ui64NV = _link_function('glProgramUniform4ui64NV', None, [GLuint, GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT, GLuint64EXT], 'NV_gpu_shader5')

# GL/glext.h:11442
glProgramUniform1ui64vNV = _link_function('glProgramUniform1ui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11443
glProgramUniform2ui64vNV = _link_function('glProgramUniform2ui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11444
glProgramUniform3ui64vNV = _link_function('glProgramUniform3ui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

# GL/glext.h:11445
glProgramUniform4ui64vNV = _link_function('glProgramUniform4ui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64EXT)], 'NV_gpu_shader5')

PFNGLUNIFORM1I64NVPROC = CFUNCTYPE(None, GLint, GLint64EXT) 	# GL/glext.h:11447
PFNGLUNIFORM2I64NVPROC = CFUNCTYPE(None, GLint, GLint64EXT, GLint64EXT) 	# GL/glext.h:11448
PFNGLUNIFORM3I64NVPROC = CFUNCTYPE(None, GLint, GLint64EXT, GLint64EXT, GLint64EXT) 	# GL/glext.h:11449
PFNGLUNIFORM4I64NVPROC = CFUNCTYPE(None, GLint, GLint64EXT, GLint64EXT, GLint64EXT, GLint64EXT) 	# GL/glext.h:11450
PFNGLUNIFORM1I64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11451
PFNGLUNIFORM2I64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11452
PFNGLUNIFORM3I64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11453
PFNGLUNIFORM4I64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11454
PFNGLUNIFORM1UI64NVPROC = CFUNCTYPE(None, GLint, GLuint64EXT) 	# GL/glext.h:11455
PFNGLUNIFORM2UI64NVPROC = CFUNCTYPE(None, GLint, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11456
PFNGLUNIFORM3UI64NVPROC = CFUNCTYPE(None, GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11457
PFNGLUNIFORM4UI64NVPROC = CFUNCTYPE(None, GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11458
PFNGLUNIFORM1UI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11459
PFNGLUNIFORM2UI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11460
PFNGLUNIFORM3UI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11461
PFNGLUNIFORM4UI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11462
PFNGLGETUNIFORMI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, POINTER(GLint64EXT)) 	# GL/glext.h:11463
PFNGLPROGRAMUNIFORM1I64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLint64EXT) 	# GL/glext.h:11464
PFNGLPROGRAMUNIFORM2I64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLint64EXT, GLint64EXT) 	# GL/glext.h:11465
PFNGLPROGRAMUNIFORM3I64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLint64EXT, GLint64EXT, GLint64EXT) 	# GL/glext.h:11466
PFNGLPROGRAMUNIFORM4I64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLint64EXT, GLint64EXT, GLint64EXT, GLint64EXT) 	# GL/glext.h:11467
PFNGLPROGRAMUNIFORM1I64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11468
PFNGLPROGRAMUNIFORM2I64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11469
PFNGLPROGRAMUNIFORM3I64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11470
PFNGLPROGRAMUNIFORM4I64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLint64EXT)) 	# GL/glext.h:11471
PFNGLPROGRAMUNIFORM1UI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64EXT) 	# GL/glext.h:11472
PFNGLPROGRAMUNIFORM2UI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11473
PFNGLPROGRAMUNIFORM3UI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11474
PFNGLPROGRAMUNIFORM4UI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64EXT, GLuint64EXT, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11475
PFNGLPROGRAMUNIFORM1UI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11476
PFNGLPROGRAMUNIFORM2UI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11477
PFNGLPROGRAMUNIFORM3UI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11478
PFNGLPROGRAMUNIFORM4UI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64EXT)) 	# GL/glext.h:11479
# NV_shader_buffer_store (GL/glext.h:11482)
GL_NV_shader_buffer_store = 1 	# GL/glext.h:11483
# NV_tessellation_program5 (GL/glext.h:11486)
GL_NV_tessellation_program5 = 1 	# GL/glext.h:11487
# NV_vertex_attrib_integer_64bit (GL/glext.h:11490)
GL_NV_vertex_attrib_integer_64bit = 1 	# GL/glext.h:11491
# GL/glext.h:11493
glVertexAttribL1i64NV = _link_function('glVertexAttribL1i64NV', None, [GLuint, GLint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11494
glVertexAttribL2i64NV = _link_function('glVertexAttribL2i64NV', None, [GLuint, GLint64EXT, GLint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11495
glVertexAttribL3i64NV = _link_function('glVertexAttribL3i64NV', None, [GLuint, GLint64EXT, GLint64EXT, GLint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11496
glVertexAttribL4i64NV = _link_function('glVertexAttribL4i64NV', None, [GLuint, GLint64EXT, GLint64EXT, GLint64EXT, GLint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11497
glVertexAttribL1i64vNV = _link_function('glVertexAttribL1i64vNV', None, [GLuint, POINTER(GLint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11498
glVertexAttribL2i64vNV = _link_function('glVertexAttribL2i64vNV', None, [GLuint, POINTER(GLint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11499
glVertexAttribL3i64vNV = _link_function('glVertexAttribL3i64vNV', None, [GLuint, POINTER(GLint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11500
glVertexAttribL4i64vNV = _link_function('glVertexAttribL4i64vNV', None, [GLuint, POINTER(GLint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11501
glVertexAttribL1ui64NV = _link_function('glVertexAttribL1ui64NV', None, [GLuint, GLuint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11502
glVertexAttribL2ui64NV = _link_function('glVertexAttribL2ui64NV', None, [GLuint, GLuint64EXT, GLuint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11503
glVertexAttribL3ui64NV = _link_function('glVertexAttribL3ui64NV', None, [GLuint, GLuint64EXT, GLuint64EXT, GLuint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11504
glVertexAttribL4ui64NV = _link_function('glVertexAttribL4ui64NV', None, [GLuint, GLuint64EXT, GLuint64EXT, GLuint64EXT, GLuint64EXT], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11505
glVertexAttribL1ui64vNV = _link_function('glVertexAttribL1ui64vNV', None, [GLuint, POINTER(GLuint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11506
glVertexAttribL2ui64vNV = _link_function('glVertexAttribL2ui64vNV', None, [GLuint, POINTER(GLuint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11507
glVertexAttribL3ui64vNV = _link_function('glVertexAttribL3ui64vNV', None, [GLuint, POINTER(GLuint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11508
glVertexAttribL4ui64vNV = _link_function('glVertexAttribL4ui64vNV', None, [GLuint, POINTER(GLuint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11509
glGetVertexAttribLi64vNV = _link_function('glGetVertexAttribLi64vNV', None, [GLuint, GLenum, POINTER(GLint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11510
glGetVertexAttribLui64vNV = _link_function('glGetVertexAttribLui64vNV', None, [GLuint, GLenum, POINTER(GLuint64EXT)], 'NV_vertex_attrib_integer_64bit')

# GL/glext.h:11511
glVertexAttribLFormatNV = _link_function('glVertexAttribLFormatNV', None, [GLuint, GLint, GLenum, GLsizei], 'NV_vertex_attrib_integer_64bit')

PFNGLVERTEXATTRIBL1I64NVPROC = CFUNCTYPE(None, GLuint, GLint64EXT) 	# GL/glext.h:11513
PFNGLVERTEXATTRIBL2I64NVPROC = CFUNCTYPE(None, GLuint, GLint64EXT, GLint64EXT) 	# GL/glext.h:11514
PFNGLVERTEXATTRIBL3I64NVPROC = CFUNCTYPE(None, GLuint, GLint64EXT, GLint64EXT, GLint64EXT) 	# GL/glext.h:11515
PFNGLVERTEXATTRIBL4I64NVPROC = CFUNCTYPE(None, GLuint, GLint64EXT, GLint64EXT, GLint64EXT, GLint64EXT) 	# GL/glext.h:11516
PFNGLVERTEXATTRIBL1I64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint64EXT)) 	# GL/glext.h:11517
PFNGLVERTEXATTRIBL2I64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint64EXT)) 	# GL/glext.h:11518
PFNGLVERTEXATTRIBL3I64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint64EXT)) 	# GL/glext.h:11519
PFNGLVERTEXATTRIBL4I64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLint64EXT)) 	# GL/glext.h:11520
PFNGLVERTEXATTRIBL1UI64NVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT) 	# GL/glext.h:11521
PFNGLVERTEXATTRIBL2UI64NVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11522
PFNGLVERTEXATTRIBL3UI64NVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11523
PFNGLVERTEXATTRIBL4UI64NVPROC = CFUNCTYPE(None, GLuint, GLuint64EXT, GLuint64EXT, GLuint64EXT, GLuint64EXT) 	# GL/glext.h:11524
PFNGLVERTEXATTRIBL1UI64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint64EXT)) 	# GL/glext.h:11525
PFNGLVERTEXATTRIBL2UI64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint64EXT)) 	# GL/glext.h:11526
PFNGLVERTEXATTRIBL3UI64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint64EXT)) 	# GL/glext.h:11527
PFNGLVERTEXATTRIBL4UI64VNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLuint64EXT)) 	# GL/glext.h:11528
PFNGLGETVERTEXATTRIBLI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint64EXT)) 	# GL/glext.h:11529
PFNGLGETVERTEXATTRIBLUI64VNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLuint64EXT)) 	# GL/glext.h:11530
PFNGLVERTEXATTRIBLFORMATNVPROC = CFUNCTYPE(None, GLuint, GLint, GLenum, GLsizei) 	# GL/glext.h:11531
# NV_multisample_coverage (GL/glext.h:11534)
GL_NV_multisample_coverage = 1 	# GL/glext.h:11535
# AMD_name_gen_delete (GL/glext.h:11538)
GL_AMD_name_gen_delete = 1 	# GL/glext.h:11539
# GL/glext.h:11541
glGenNamesAMD = _link_function('glGenNamesAMD', None, [GLenum, GLuint, POINTER(GLuint)], 'AMD_name_gen_delete')

# GL/glext.h:11542
glDeleteNamesAMD = _link_function('glDeleteNamesAMD', None, [GLenum, GLuint, POINTER(GLuint)], 'AMD_name_gen_delete')

# GL/glext.h:11543
glIsNameAMD = _link_function('glIsNameAMD', GLboolean, [GLenum, GLuint], 'AMD_name_gen_delete')

PFNGLGENNAMESAMDPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:11545
PFNGLDELETENAMESAMDPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLuint)) 	# GL/glext.h:11546
PFNGLISNAMEAMDPROC = CFUNCTYPE(GLboolean, GLenum, GLuint) 	# GL/glext.h:11547
# AMD_debug_output (GL/glext.h:11550)
GL_AMD_debug_output = 1 	# GL/glext.h:11551
# GL/glext.h:11553
glDebugMessageEnableAMD = _link_function('glDebugMessageEnableAMD', None, [GLenum, GLenum, GLsizei, POINTER(GLuint), GLboolean], 'AMD_debug_output')

# GL/glext.h:11554
glDebugMessageInsertAMD = _link_function('glDebugMessageInsertAMD', None, [GLenum, GLenum, GLuint, GLsizei, POINTER(GLchar)], 'AMD_debug_output')

# GL/glext.h:11555
glDebugMessageCallbackAMD = _link_function('glDebugMessageCallbackAMD', None, [GLDEBUGPROCAMD, POINTER(GLvoid)], 'AMD_debug_output')

# GL/glext.h:11556
glGetDebugMessageLogAMD = _link_function('glGetDebugMessageLogAMD', GLuint, [GLuint, GLsizei, POINTER(GLenum), POINTER(GLuint), POINTER(GLuint), POINTER(GLsizei), POINTER(GLchar)], 'AMD_debug_output')

PFNGLDEBUGMESSAGEENABLEAMDPROC = CFUNCTYPE(None, GLenum, GLenum, GLsizei, POINTER(GLuint), GLboolean) 	# GL/glext.h:11558
PFNGLDEBUGMESSAGEINSERTAMDPROC = CFUNCTYPE(None, GLenum, GLenum, GLuint, GLsizei, POINTER(GLchar)) 	# GL/glext.h:11559
PFNGLDEBUGMESSAGECALLBACKAMDPROC = CFUNCTYPE(None, GLDEBUGPROCAMD, POINTER(GLvoid)) 	# GL/glext.h:11560
PFNGLGETDEBUGMESSAGELOGAMDPROC = CFUNCTYPE(GLuint, GLuint, GLsizei, POINTER(GLenum), POINTER(GLuint), POINTER(GLuint), POINTER(GLsizei), POINTER(GLchar)) 	# GL/glext.h:11561
# NV_vdpau_interop (GL/glext.h:11564)
GL_NV_vdpau_interop = 1 	# GL/glext.h:11565
# GL/glext.h:11567
glVDPAUInitNV = _link_function('glVDPAUInitNV', None, [POINTER(GLvoid), POINTER(GLvoid)], 'NV_vdpau_interop')

# GL/glext.h:11568
glVDPAUFiniNV = _link_function('glVDPAUFiniNV', None, [], 'NV_vdpau_interop')

# GL/glext.h:11569
glVDPAURegisterVideoSurfaceNV = _link_function('glVDPAURegisterVideoSurfaceNV', GLvdpauSurfaceNV, [POINTER(GLvoid), GLenum, GLsizei, POINTER(GLuint)], 'NV_vdpau_interop')

# GL/glext.h:11570
glVDPAURegisterOutputSurfaceNV = _link_function('glVDPAURegisterOutputSurfaceNV', GLvdpauSurfaceNV, [POINTER(GLvoid), GLenum, GLsizei, POINTER(GLuint)], 'NV_vdpau_interop')

# GL/glext.h:11571
glVDPAUIsSurfaceNV = _link_function('glVDPAUIsSurfaceNV', None, [GLvdpauSurfaceNV], 'NV_vdpau_interop')

# GL/glext.h:11572
glVDPAUUnregisterSurfaceNV = _link_function('glVDPAUUnregisterSurfaceNV', None, [GLvdpauSurfaceNV], 'NV_vdpau_interop')

# GL/glext.h:11573
glVDPAUGetSurfaceivNV = _link_function('glVDPAUGetSurfaceivNV', None, [GLvdpauSurfaceNV, GLenum, GLsizei, POINTER(GLsizei), POINTER(GLint)], 'NV_vdpau_interop')

# GL/glext.h:11574
glVDPAUSurfaceAccessNV = _link_function('glVDPAUSurfaceAccessNV', None, [GLvdpauSurfaceNV, GLenum], 'NV_vdpau_interop')

# GL/glext.h:11575
glVDPAUMapSurfacesNV = _link_function('glVDPAUMapSurfacesNV', None, [GLsizei, POINTER(GLvdpauSurfaceNV)], 'NV_vdpau_interop')

# GL/glext.h:11576
glVDPAUUnmapSurfacesNV = _link_function('glVDPAUUnmapSurfacesNV', None, [GLsizei, POINTER(GLvdpauSurfaceNV)], 'NV_vdpau_interop')

PFNGLVDPAUINITNVPROC = CFUNCTYPE(None, POINTER(GLvoid), POINTER(GLvoid)) 	# GL/glext.h:11578
PFNGLVDPAUFININVPROC = CFUNCTYPE(None) 	# GL/glext.h:11579
PFNGLVDPAUREGISTERVIDEOSURFACENVPROC = CFUNCTYPE(GLvdpauSurfaceNV, POINTER(GLvoid), GLenum, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11580
PFNGLVDPAUREGISTEROUTPUTSURFACENVPROC = CFUNCTYPE(GLvdpauSurfaceNV, POINTER(GLvoid), GLenum, GLsizei, POINTER(GLuint)) 	# GL/glext.h:11581
PFNGLVDPAUISSURFACENVPROC = CFUNCTYPE(None, GLvdpauSurfaceNV) 	# GL/glext.h:11582
PFNGLVDPAUUNREGISTERSURFACENVPROC = CFUNCTYPE(None, GLvdpauSurfaceNV) 	# GL/glext.h:11583
PFNGLVDPAUGETSURFACEIVNVPROC = CFUNCTYPE(None, GLvdpauSurfaceNV, GLenum, GLsizei, POINTER(GLsizei), POINTER(GLint)) 	# GL/glext.h:11584
PFNGLVDPAUSURFACEACCESSNVPROC = CFUNCTYPE(None, GLvdpauSurfaceNV, GLenum) 	# GL/glext.h:11585
PFNGLVDPAUMAPSURFACESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvdpauSurfaceNV)) 	# GL/glext.h:11586
PFNGLVDPAUUNMAPSURFACESNVPROC = CFUNCTYPE(None, GLsizei, POINTER(GLvdpauSurfaceNV)) 	# GL/glext.h:11587
# AMD_transform_feedback3_lines_triangles (GL/glext.h:11590)
GL_AMD_transform_feedback3_lines_triangles = 1 	# GL/glext.h:11591
# AMD_depth_clamp_separate (GL/glext.h:11594)
GL_AMD_depth_clamp_separate = 1 	# GL/glext.h:11595
# EXT_texture_sRGB_decode (GL/glext.h:11598)
GL_EXT_texture_sRGB_decode = 1 	# GL/glext.h:11599
# NV_texture_multisample (GL/glext.h:11602)
GL_NV_texture_multisample = 1 	# GL/glext.h:11603
# GL/glext.h:11605
glTexImage2DMultisampleCoverageNV = _link_function('glTexImage2DMultisampleCoverageNV', None, [GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLboolean], 'NV_texture_multisample')

# GL/glext.h:11606
glTexImage3DMultisampleCoverageNV = _link_function('glTexImage3DMultisampleCoverageNV', None, [GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean], 'NV_texture_multisample')

# GL/glext.h:11607
glTextureImage2DMultisampleNV = _link_function('glTextureImage2DMultisampleNV', None, [GLuint, GLenum, GLsizei, GLint, GLsizei, GLsizei, GLboolean], 'NV_texture_multisample')

# GL/glext.h:11608
glTextureImage3DMultisampleNV = _link_function('glTextureImage3DMultisampleNV', None, [GLuint, GLenum, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean], 'NV_texture_multisample')

# GL/glext.h:11609
glTextureImage2DMultisampleCoverageNV = _link_function('glTextureImage2DMultisampleCoverageNV', None, [GLuint, GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLboolean], 'NV_texture_multisample')

# GL/glext.h:11610
glTextureImage3DMultisampleCoverageNV = _link_function('glTextureImage3DMultisampleCoverageNV', None, [GLuint, GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean], 'NV_texture_multisample')

PFNGLTEXIMAGE2DMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:11612
PFNGLTEXIMAGE3DMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:11613
PFNGLTEXTUREIMAGE2DMULTISAMPLENVPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLint, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:11614
PFNGLTEXTUREIMAGE3DMULTISAMPLENVPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:11615
PFNGLTEXTUREIMAGE2DMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:11616
PFNGLTEXTUREIMAGE3DMULTISAMPLECOVERAGENVPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, GLsizei, GLint, GLsizei, GLsizei, GLsizei, GLboolean) 	# GL/glext.h:11617
# AMD_blend_minmax_factor (GL/glext.h:11620)
GL_AMD_blend_minmax_factor = 1 	# GL/glext.h:11621
# AMD_sample_positions (GL/glext.h:11624)
GL_AMD_sample_positions = 1 	# GL/glext.h:11625
# GL/glext.h:11627
glSetMultisamplefvAMD = _link_function('glSetMultisamplefvAMD', None, [GLenum, GLuint, POINTER(GLfloat)], 'AMD_sample_positions')

PFNGLSETMULTISAMPLEFVAMDPROC = CFUNCTYPE(None, GLenum, GLuint, POINTER(GLfloat)) 	# GL/glext.h:11629
# EXT_x11_sync_object (GL/glext.h:11632)
GL_EXT_x11_sync_object = 1 	# GL/glext.h:11633
# GL/glext.h:11635
glImportSyncEXT = _link_function('glImportSyncEXT', GLsync, [GLenum, GLintptr, GLbitfield], 'EXT_x11_sync_object')

PFNGLIMPORTSYNCEXTPROC = CFUNCTYPE(GLsync, GLenum, GLintptr, GLbitfield) 	# GL/glext.h:11637
# AMD_multi_draw_indirect (GL/glext.h:11640)
GL_AMD_multi_draw_indirect = 1 	# GL/glext.h:11641
# GL/glext.h:11643
glMultiDrawArraysIndirectAMD = _link_function('glMultiDrawArraysIndirectAMD', None, [GLenum, POINTER(GLvoid), GLsizei, GLsizei], 'AMD_multi_draw_indirect')

# GL/glext.h:11644
glMultiDrawElementsIndirectAMD = _link_function('glMultiDrawElementsIndirectAMD', None, [GLenum, GLenum, POINTER(GLvoid), GLsizei, GLsizei], 'AMD_multi_draw_indirect')

PFNGLMULTIDRAWARRAYSINDIRECTAMDPROC = CFUNCTYPE(None, GLenum, POINTER(GLvoid), GLsizei, GLsizei) 	# GL/glext.h:11646
PFNGLMULTIDRAWELEMENTSINDIRECTAMDPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLvoid), GLsizei, GLsizei) 	# GL/glext.h:11647
# EXT_framebuffer_multisample_blit_scaled (GL/glext.h:11650)
GL_EXT_framebuffer_multisample_blit_scaled = 1 	# GL/glext.h:11651
# NV_path_rendering (GL/glext.h:11654)
GL_NV_path_rendering = 1 	# GL/glext.h:11655
# GL/glext.h:11657
glGenPathsNV = _link_function('glGenPathsNV', GLuint, [GLsizei], 'NV_path_rendering')

# GL/glext.h:11658
glDeletePathsNV = _link_function('glDeletePathsNV', None, [GLuint, GLsizei], 'NV_path_rendering')

# GL/glext.h:11659
glIsPathNV = _link_function('glIsPathNV', GLboolean, [GLuint], 'NV_path_rendering')

# GL/glext.h:11660
glPathCommandsNV = _link_function('glPathCommandsNV', None, [GLuint, GLsizei, POINTER(GLubyte), GLsizei, GLenum, POINTER(GLvoid)], 'NV_path_rendering')

# GL/glext.h:11661
glPathCoordsNV = _link_function('glPathCoordsNV', None, [GLuint, GLsizei, GLenum, POINTER(GLvoid)], 'NV_path_rendering')

# GL/glext.h:11662
glPathSubCommandsNV = _link_function('glPathSubCommandsNV', None, [GLuint, GLsizei, GLsizei, GLsizei, POINTER(GLubyte), GLsizei, GLenum, POINTER(GLvoid)], 'NV_path_rendering')

# GL/glext.h:11663
glPathSubCoordsNV = _link_function('glPathSubCoordsNV', None, [GLuint, GLsizei, GLsizei, GLenum, POINTER(GLvoid)], 'NV_path_rendering')

# GL/glext.h:11664
glPathStringNV = _link_function('glPathStringNV', None, [GLuint, GLenum, GLsizei, POINTER(GLvoid)], 'NV_path_rendering')

# GL/glext.h:11665
glPathGlyphsNV = _link_function('glPathGlyphsNV', None, [GLuint, GLenum, POINTER(GLvoid), GLbitfield, GLsizei, GLenum, POINTER(GLvoid), GLenum, GLuint, GLfloat], 'NV_path_rendering')

# GL/glext.h:11666
glPathGlyphRangeNV = _link_function('glPathGlyphRangeNV', None, [GLuint, GLenum, POINTER(GLvoid), GLbitfield, GLuint, GLsizei, GLenum, GLuint, GLfloat], 'NV_path_rendering')

# GL/glext.h:11667
glWeightPathsNV = _link_function('glWeightPathsNV', None, [GLuint, GLsizei, POINTER(GLuint), POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11668
glCopyPathNV = _link_function('glCopyPathNV', None, [GLuint, GLuint], 'NV_path_rendering')

# GL/glext.h:11669
glInterpolatePathsNV = _link_function('glInterpolatePathsNV', None, [GLuint, GLuint, GLuint, GLfloat], 'NV_path_rendering')

# GL/glext.h:11670
glTransformPathNV = _link_function('glTransformPathNV', None, [GLuint, GLuint, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11671
glPathParameterivNV = _link_function('glPathParameterivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_path_rendering')

# GL/glext.h:11672
glPathParameteriNV = _link_function('glPathParameteriNV', None, [GLuint, GLenum, GLint], 'NV_path_rendering')

# GL/glext.h:11673
glPathParameterfvNV = _link_function('glPathParameterfvNV', None, [GLuint, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11674
glPathParameterfNV = _link_function('glPathParameterfNV', None, [GLuint, GLenum, GLfloat], 'NV_path_rendering')

# GL/glext.h:11675
glPathDashArrayNV = _link_function('glPathDashArrayNV', None, [GLuint, GLsizei, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11676
glPathStencilFuncNV = _link_function('glPathStencilFuncNV', None, [GLenum, GLint, GLuint], 'NV_path_rendering')

# GL/glext.h:11677
glPathStencilDepthOffsetNV = _link_function('glPathStencilDepthOffsetNV', None, [GLfloat, GLfloat], 'NV_path_rendering')

# GL/glext.h:11678
glStencilFillPathNV = _link_function('glStencilFillPathNV', None, [GLuint, GLenum, GLuint], 'NV_path_rendering')

# GL/glext.h:11679
glStencilStrokePathNV = _link_function('glStencilStrokePathNV', None, [GLuint, GLint, GLuint], 'NV_path_rendering')

# GL/glext.h:11680
glStencilFillPathInstancedNV = _link_function('glStencilFillPathInstancedNV', None, [GLsizei, GLenum, POINTER(GLvoid), GLuint, GLenum, GLuint, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11681
glStencilStrokePathInstancedNV = _link_function('glStencilStrokePathInstancedNV', None, [GLsizei, GLenum, POINTER(GLvoid), GLuint, GLint, GLuint, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11682
glPathCoverDepthFuncNV = _link_function('glPathCoverDepthFuncNV', None, [GLenum], 'NV_path_rendering')

# GL/glext.h:11683
glPathColorGenNV = _link_function('glPathColorGenNV', None, [GLenum, GLenum, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11684
glPathTexGenNV = _link_function('glPathTexGenNV', None, [GLenum, GLenum, GLint, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11685
glPathFogGenNV = _link_function('glPathFogGenNV', None, [GLenum], 'NV_path_rendering')

# GL/glext.h:11686
glCoverFillPathNV = _link_function('glCoverFillPathNV', None, [GLuint, GLenum], 'NV_path_rendering')

# GL/glext.h:11687
glCoverStrokePathNV = _link_function('glCoverStrokePathNV', None, [GLuint, GLenum], 'NV_path_rendering')

# GL/glext.h:11688
glCoverFillPathInstancedNV = _link_function('glCoverFillPathInstancedNV', None, [GLsizei, GLenum, POINTER(GLvoid), GLuint, GLenum, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11689
glCoverStrokePathInstancedNV = _link_function('glCoverStrokePathInstancedNV', None, [GLsizei, GLenum, POINTER(GLvoid), GLuint, GLenum, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11690
glGetPathParameterivNV = _link_function('glGetPathParameterivNV', None, [GLuint, GLenum, POINTER(GLint)], 'NV_path_rendering')

# GL/glext.h:11691
glGetPathParameterfvNV = _link_function('glGetPathParameterfvNV', None, [GLuint, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11692
glGetPathCommandsNV = _link_function('glGetPathCommandsNV', None, [GLuint, POINTER(GLubyte)], 'NV_path_rendering')

# GL/glext.h:11693
glGetPathCoordsNV = _link_function('glGetPathCoordsNV', None, [GLuint, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11694
glGetPathDashArrayNV = _link_function('glGetPathDashArrayNV', None, [GLuint, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11695
glGetPathMetricsNV = _link_function('glGetPathMetricsNV', None, [GLbitfield, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLsizei, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11696
glGetPathMetricRangeNV = _link_function('glGetPathMetricRangeNV', None, [GLbitfield, GLuint, GLsizei, GLsizei, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11697
glGetPathSpacingNV = _link_function('glGetPathSpacingNV', None, [GLenum, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLfloat, GLfloat, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11698
glGetPathColorGenivNV = _link_function('glGetPathColorGenivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_path_rendering')

# GL/glext.h:11699
glGetPathColorGenfvNV = _link_function('glGetPathColorGenfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11700
glGetPathTexGenivNV = _link_function('glGetPathTexGenivNV', None, [GLenum, GLenum, POINTER(GLint)], 'NV_path_rendering')

# GL/glext.h:11701
glGetPathTexGenfvNV = _link_function('glGetPathTexGenfvNV', None, [GLenum, GLenum, POINTER(GLfloat)], 'NV_path_rendering')

# GL/glext.h:11702
glIsPointInFillPathNV = _link_function('glIsPointInFillPathNV', GLboolean, [GLuint, GLuint, GLfloat, GLfloat], 'NV_path_rendering')

# GL/glext.h:11703
glIsPointInStrokePathNV = _link_function('glIsPointInStrokePathNV', GLboolean, [GLuint, GLfloat, GLfloat], 'NV_path_rendering')

# GL/glext.h:11704
glGetPathLengthNV = _link_function('glGetPathLengthNV', GLfloat, [GLuint, GLsizei, GLsizei], 'NV_path_rendering')

# GL/glext.h:11705
glPointAlongPathNV = _link_function('glPointAlongPathNV', GLboolean, [GLuint, GLsizei, GLsizei, GLfloat, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)], 'NV_path_rendering')

PFNGLGENPATHSNVPROC = CFUNCTYPE(GLuint, GLsizei) 	# GL/glext.h:11707
PFNGLDELETEPATHSNVPROC = CFUNCTYPE(None, GLuint, GLsizei) 	# GL/glext.h:11708
PFNGLISPATHNVPROC = CFUNCTYPE(GLboolean, GLuint) 	# GL/glext.h:11709
PFNGLPATHCOMMANDSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLubyte), GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:11710
PFNGLPATHCOORDSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:11711
PFNGLPATHSUBCOMMANDSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, GLsizei, GLsizei, POINTER(GLubyte), GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:11712
PFNGLPATHSUBCOORDSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, GLsizei, GLenum, POINTER(GLvoid)) 	# GL/glext.h:11713
PFNGLPATHSTRINGNVPROC = CFUNCTYPE(None, GLuint, GLenum, GLsizei, POINTER(GLvoid)) 	# GL/glext.h:11714
PFNGLPATHGLYPHSNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid), GLbitfield, GLsizei, GLenum, POINTER(GLvoid), GLenum, GLuint, GLfloat) 	# GL/glext.h:11715
PFNGLPATHGLYPHRANGENVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLvoid), GLbitfield, GLuint, GLsizei, GLenum, GLuint, GLfloat) 	# GL/glext.h:11716
PFNGLWEIGHTPATHSNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLuint), POINTER(GLfloat)) 	# GL/glext.h:11717
PFNGLCOPYPATHNVPROC = CFUNCTYPE(None, GLuint, GLuint) 	# GL/glext.h:11718
PFNGLINTERPOLATEPATHSNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLuint, GLfloat) 	# GL/glext.h:11719
PFNGLTRANSFORMPATHNVPROC = CFUNCTYPE(None, GLuint, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11720
PFNGLPATHPARAMETERIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11721
PFNGLPATHPARAMETERINVPROC = CFUNCTYPE(None, GLuint, GLenum, GLint) 	# GL/glext.h:11722
PFNGLPATHPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11723
PFNGLPATHPARAMETERFNVPROC = CFUNCTYPE(None, GLuint, GLenum, GLfloat) 	# GL/glext.h:11724
PFNGLPATHDASHARRAYNVPROC = CFUNCTYPE(None, GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:11725
PFNGLPATHSTENCILFUNCNVPROC = CFUNCTYPE(None, GLenum, GLint, GLuint) 	# GL/glext.h:11726
PFNGLPATHSTENCILDEPTHOFFSETNVPROC = CFUNCTYPE(None, GLfloat, GLfloat) 	# GL/glext.h:11727
PFNGLSTENCILFILLPATHNVPROC = CFUNCTYPE(None, GLuint, GLenum, GLuint) 	# GL/glext.h:11728
PFNGLSTENCILSTROKEPATHNVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint) 	# GL/glext.h:11729
PFNGLSTENCILFILLPATHINSTANCEDNVPROC = CFUNCTYPE(None, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLenum, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11730
PFNGLSTENCILSTROKEPATHINSTANCEDNVPROC = CFUNCTYPE(None, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLint, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11731
PFNGLPATHCOVERDEPTHFUNCNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:11732
PFNGLPATHCOLORGENNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11733
PFNGLPATHTEXGENNVPROC = CFUNCTYPE(None, GLenum, GLenum, GLint, POINTER(GLfloat)) 	# GL/glext.h:11734
PFNGLPATHFOGGENNVPROC = CFUNCTYPE(None, GLenum) 	# GL/glext.h:11735
PFNGLCOVERFILLPATHNVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11736
PFNGLCOVERSTROKEPATHNVPROC = CFUNCTYPE(None, GLuint, GLenum) 	# GL/glext.h:11737
PFNGLCOVERFILLPATHINSTANCEDNVPROC = CFUNCTYPE(None, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11738
PFNGLCOVERSTROKEPATHINSTANCEDNVPROC = CFUNCTYPE(None, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11739
PFNGLGETPATHPARAMETERIVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLint)) 	# GL/glext.h:11740
PFNGLGETPATHPARAMETERFVNVPROC = CFUNCTYPE(None, GLuint, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11741
PFNGLGETPATHCOMMANDSNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLubyte)) 	# GL/glext.h:11742
PFNGLGETPATHCOORDSNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:11743
PFNGLGETPATHDASHARRAYNVPROC = CFUNCTYPE(None, GLuint, POINTER(GLfloat)) 	# GL/glext.h:11744
PFNGLGETPATHMETRICSNVPROC = CFUNCTYPE(None, GLbitfield, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:11745
PFNGLGETPATHMETRICRANGENVPROC = CFUNCTYPE(None, GLbitfield, GLuint, GLsizei, GLsizei, POINTER(GLfloat)) 	# GL/glext.h:11746
PFNGLGETPATHSPACINGNVPROC = CFUNCTYPE(None, GLenum, GLsizei, GLenum, POINTER(GLvoid), GLuint, GLfloat, GLfloat, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11747
PFNGLGETPATHCOLORGENIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:11748
PFNGLGETPATHCOLORGENFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11749
PFNGLGETPATHTEXGENIVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLint)) 	# GL/glext.h:11750
PFNGLGETPATHTEXGENFVNVPROC = CFUNCTYPE(None, GLenum, GLenum, POINTER(GLfloat)) 	# GL/glext.h:11751
PFNGLISPOINTINFILLPATHNVPROC = CFUNCTYPE(GLboolean, GLuint, GLuint, GLfloat, GLfloat) 	# GL/glext.h:11752
PFNGLISPOINTINSTROKEPATHNVPROC = CFUNCTYPE(GLboolean, GLuint, GLfloat, GLfloat) 	# GL/glext.h:11753
PFNGLGETPATHLENGTHNVPROC = CFUNCTYPE(GLfloat, GLuint, GLsizei, GLsizei) 	# GL/glext.h:11754
PFNGLPOINTALONGPATHNVPROC = CFUNCTYPE(GLboolean, GLuint, GLsizei, GLsizei, GLfloat, POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat), POINTER(GLfloat)) 	# GL/glext.h:11755
# AMD_pinned_memory (GL/glext.h:11758)
GL_AMD_pinned_memory = 1 	# GL/glext.h:11759
# AMD_stencil_operation_extended (GL/glext.h:11762)
GL_AMD_stencil_operation_extended = 1 	# GL/glext.h:11763
# GL/glext.h:11765
glStencilOpValueAMD = _link_function('glStencilOpValueAMD', None, [GLenum, GLuint], 'AMD_stencil_operation_extended')

PFNGLSTENCILOPVALUEAMDPROC = CFUNCTYPE(None, GLenum, GLuint) 	# GL/glext.h:11767
# AMD_vertex_shader_viewport_index (GL/glext.h:11770)
GL_AMD_vertex_shader_viewport_index = 1 	# GL/glext.h:11771
# AMD_vertex_shader_layer (GL/glext.h:11774)
GL_AMD_vertex_shader_layer = 1 	# GL/glext.h:11775
# NV_bindless_texture (GL/glext.h:11778)
GL_NV_bindless_texture = 1 	# GL/glext.h:11779
# GL/glext.h:11781
glGetTextureHandleNV = _link_function('glGetTextureHandleNV', GLuint64, [GLuint], 'NV_bindless_texture')

# GL/glext.h:11782
glGetTextureSamplerHandleNV = _link_function('glGetTextureSamplerHandleNV', GLuint64, [GLuint, GLuint], 'NV_bindless_texture')

# GL/glext.h:11783
glMakeTextureHandleResidentNV = _link_function('glMakeTextureHandleResidentNV', None, [GLuint64], 'NV_bindless_texture')

# GL/glext.h:11784
glMakeTextureHandleNonResidentNV = _link_function('glMakeTextureHandleNonResidentNV', None, [GLuint64], 'NV_bindless_texture')

# GL/glext.h:11785
glGetImageHandleNV = _link_function('glGetImageHandleNV', GLuint64, [GLuint, GLint, GLboolean, GLint, GLenum], 'NV_bindless_texture')

# GL/glext.h:11786
glMakeImageHandleResidentNV = _link_function('glMakeImageHandleResidentNV', None, [GLuint64, GLenum], 'NV_bindless_texture')

# GL/glext.h:11787
glMakeImageHandleNonResidentNV = _link_function('glMakeImageHandleNonResidentNV', None, [GLuint64], 'NV_bindless_texture')

# GL/glext.h:11788
glUniformHandleui64NV = _link_function('glUniformHandleui64NV', None, [GLint, GLuint64], 'NV_bindless_texture')

# GL/glext.h:11789
glUniformHandleui64vNV = _link_function('glUniformHandleui64vNV', None, [GLint, GLsizei, POINTER(GLuint64)], 'NV_bindless_texture')

# GL/glext.h:11790
glProgramUniformHandleui64NV = _link_function('glProgramUniformHandleui64NV', None, [GLuint, GLint, GLuint64], 'NV_bindless_texture')

# GL/glext.h:11791
glProgramUniformHandleui64vNV = _link_function('glProgramUniformHandleui64vNV', None, [GLuint, GLint, GLsizei, POINTER(GLuint64)], 'NV_bindless_texture')

# GL/glext.h:11792
glIsTextureHandleResidentNV = _link_function('glIsTextureHandleResidentNV', GLboolean, [GLuint64], 'NV_bindless_texture')

# GL/glext.h:11793
glIsImageHandleResidentNV = _link_function('glIsImageHandleResidentNV', GLboolean, [GLuint64], 'NV_bindless_texture')

PFNGLGETTEXTUREHANDLENVPROC = CFUNCTYPE(GLuint64, GLuint) 	# GL/glext.h:11795
PFNGLGETTEXTURESAMPLERHANDLENVPROC = CFUNCTYPE(GLuint64, GLuint, GLuint) 	# GL/glext.h:11796
PFNGLMAKETEXTUREHANDLERESIDENTNVPROC = CFUNCTYPE(None, GLuint64) 	# GL/glext.h:11797
PFNGLMAKETEXTUREHANDLENONRESIDENTNVPROC = CFUNCTYPE(None, GLuint64) 	# GL/glext.h:11798
PFNGLGETIMAGEHANDLENVPROC = CFUNCTYPE(GLuint64, GLuint, GLint, GLboolean, GLint, GLenum) 	# GL/glext.h:11799
PFNGLMAKEIMAGEHANDLERESIDENTNVPROC = CFUNCTYPE(None, GLuint64, GLenum) 	# GL/glext.h:11800
PFNGLMAKEIMAGEHANDLENONRESIDENTNVPROC = CFUNCTYPE(None, GLuint64) 	# GL/glext.h:11801
PFNGLUNIFORMHANDLEUI64NVPROC = CFUNCTYPE(None, GLint, GLuint64) 	# GL/glext.h:11802
PFNGLUNIFORMHANDLEUI64VNVPROC = CFUNCTYPE(None, GLint, GLsizei, POINTER(GLuint64)) 	# GL/glext.h:11803
PFNGLPROGRAMUNIFORMHANDLEUI64NVPROC = CFUNCTYPE(None, GLuint, GLint, GLuint64) 	# GL/glext.h:11804
PFNGLPROGRAMUNIFORMHANDLEUI64VNVPROC = CFUNCTYPE(None, GLuint, GLint, GLsizei, POINTER(GLuint64)) 	# GL/glext.h:11805
PFNGLISTEXTUREHANDLERESIDENTNVPROC = CFUNCTYPE(GLboolean, GLuint64) 	# GL/glext.h:11806
PFNGLISIMAGEHANDLERESIDENTNVPROC = CFUNCTYPE(GLboolean, GLuint64) 	# GL/glext.h:11807
# NV_shader_atomic_float (GL/glext.h:11810)
GL_NV_shader_atomic_float = 1 	# GL/glext.h:11811
# AMD_query_buffer_object (GL/glext.h:11814)
GL_AMD_query_buffer_object = 1 	# GL/glext.h:11815
# MESA_shader_debug (/usr/include/GL/gl.h:2093)
# MESA_packed_depth_stencil (/usr/include/GL/gl.h:2115)
# MESA_program_debug (/usr/include/GL/gl.h:2127)
# MESA_texture_array (/usr/include/GL/gl.h:2148)
# EXT_texture_array (/usr/include/GL/gl.h:2153)
# ATI_blend_equation_separate (/usr/include/GL/gl.h:2181)
# OES_EGL_image (/usr/include/GL/gl.h:2193)
# OES_EGL_image (/usr/include/GL/gl.h:2197)

__all__ = ['GL_GLEXT_VERSION', 'GL_RESCALE_NORMAL',
'GL_LIGHT_MODEL_COLOR_CONTROL', 'GL_SINGLE_COLOR',
'GL_SEPARATE_SPECULAR_COLOR', 'GL_ALIASED_POINT_SIZE_RANGE',
'GL_CONVOLUTION_1D', 'GL_CONVOLUTION_2D', 'GL_SEPARABLE_2D',
'GL_CONVOLUTION_BORDER_MODE', 'GL_CONVOLUTION_FILTER_SCALE',
'GL_CONVOLUTION_FILTER_BIAS', 'GL_REDUCE', 'GL_CONVOLUTION_FORMAT',
'GL_CONVOLUTION_WIDTH', 'GL_CONVOLUTION_HEIGHT', 'GL_MAX_CONVOLUTION_WIDTH',
'GL_MAX_CONVOLUTION_HEIGHT', 'GL_POST_CONVOLUTION_RED_SCALE',
'GL_POST_CONVOLUTION_GREEN_SCALE', 'GL_POST_CONVOLUTION_BLUE_SCALE',
'GL_POST_CONVOLUTION_ALPHA_SCALE', 'GL_POST_CONVOLUTION_RED_BIAS',
'GL_POST_CONVOLUTION_GREEN_BIAS', 'GL_POST_CONVOLUTION_BLUE_BIAS',
'GL_POST_CONVOLUTION_ALPHA_BIAS', 'GL_HISTOGRAM', 'GL_PROXY_HISTOGRAM',
'GL_HISTOGRAM_WIDTH', 'GL_HISTOGRAM_FORMAT', 'GL_HISTOGRAM_RED_SIZE',
'GL_HISTOGRAM_GREEN_SIZE', 'GL_HISTOGRAM_BLUE_SIZE',
'GL_HISTOGRAM_ALPHA_SIZE', 'GL_HISTOGRAM_LUMINANCE_SIZE', 'GL_HISTOGRAM_SINK',
'GL_MINMAX', 'GL_MINMAX_FORMAT', 'GL_MINMAX_SINK', 'GL_TABLE_TOO_LARGE',
'GL_COLOR_MATRIX', 'GL_COLOR_MATRIX_STACK_DEPTH',
'GL_MAX_COLOR_MATRIX_STACK_DEPTH', 'GL_POST_COLOR_MATRIX_RED_SCALE',
'GL_POST_COLOR_MATRIX_GREEN_SCALE', 'GL_POST_COLOR_MATRIX_BLUE_SCALE',
'GL_POST_COLOR_MATRIX_ALPHA_SCALE', 'GL_POST_COLOR_MATRIX_RED_BIAS',
'GL_POST_COLOR_MATRIX_GREEN_BIAS', 'GL_POST_COLOR_MATRIX_BLUE_BIAS',
'GL_POST_COLOR_MATRIX_ALPHA_BIAS', 'GL_COLOR_TABLE',
'GL_POST_CONVOLUTION_COLOR_TABLE', 'GL_POST_COLOR_MATRIX_COLOR_TABLE',
'GL_PROXY_COLOR_TABLE', 'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE', 'GL_COLOR_TABLE_SCALE',
'GL_COLOR_TABLE_BIAS', 'GL_COLOR_TABLE_FORMAT', 'GL_COLOR_TABLE_WIDTH',
'GL_COLOR_TABLE_RED_SIZE', 'GL_COLOR_TABLE_GREEN_SIZE',
'GL_COLOR_TABLE_BLUE_SIZE', 'GL_COLOR_TABLE_ALPHA_SIZE',
'GL_COLOR_TABLE_LUMINANCE_SIZE', 'GL_COLOR_TABLE_INTENSITY_SIZE',
'GL_CONSTANT_BORDER', 'GL_REPLICATE_BORDER', 'GL_CONVOLUTION_BORDER_COLOR',
'GL_CLIENT_ACTIVE_TEXTURE', 'GL_MAX_TEXTURE_UNITS',
'GL_TRANSPOSE_MODELVIEW_MATRIX', 'GL_TRANSPOSE_PROJECTION_MATRIX',
'GL_TRANSPOSE_TEXTURE_MATRIX', 'GL_TRANSPOSE_COLOR_MATRIX',
'GL_MULTISAMPLE_BIT', 'GL_NORMAL_MAP', 'GL_REFLECTION_MAP',
'GL_COMPRESSED_ALPHA', 'GL_COMPRESSED_LUMINANCE',
'GL_COMPRESSED_LUMINANCE_ALPHA', 'GL_COMPRESSED_INTENSITY', 'GL_COMBINE',
'GL_COMBINE_RGB', 'GL_COMBINE_ALPHA', 'GL_SOURCE0_RGB', 'GL_SOURCE1_RGB',
'GL_SOURCE2_RGB', 'GL_SOURCE0_ALPHA', 'GL_SOURCE1_ALPHA', 'GL_SOURCE2_ALPHA',
'GL_OPERAND0_RGB', 'GL_OPERAND1_RGB', 'GL_OPERAND2_RGB', 'GL_OPERAND0_ALPHA',
'GL_OPERAND1_ALPHA', 'GL_OPERAND2_ALPHA', 'GL_RGB_SCALE', 'GL_ADD_SIGNED',
'GL_INTERPOLATE', 'GL_SUBTRACT', 'GL_CONSTANT', 'GL_PRIMARY_COLOR',
'GL_PREVIOUS', 'GL_DOT3_RGB', 'GL_DOT3_RGBA', 'GL_BLEND_DST_RGB',
'GL_BLEND_SRC_RGB', 'GL_BLEND_DST_ALPHA', 'GL_BLEND_SRC_ALPHA',
'GL_POINT_FADE_THRESHOLD_SIZE', 'GL_DEPTH_COMPONENT16',
'GL_DEPTH_COMPONENT24', 'GL_DEPTH_COMPONENT32', 'GL_MIRRORED_REPEAT',
'GL_MAX_TEXTURE_LOD_BIAS', 'GL_TEXTURE_LOD_BIAS', 'GL_INCR_WRAP',
'GL_DECR_WRAP', 'GL_TEXTURE_DEPTH_SIZE', 'GL_TEXTURE_COMPARE_MODE',
'GL_TEXTURE_COMPARE_FUNC', 'GL_POINT_SIZE_MIN', 'GL_POINT_SIZE_MAX',
'GL_POINT_DISTANCE_ATTENUATION', 'GL_GENERATE_MIPMAP',
'GL_GENERATE_MIPMAP_HINT', 'GL_FOG_COORDINATE_SOURCE', 'GL_FOG_COORDINATE',
'GL_FRAGMENT_DEPTH', 'GL_CURRENT_FOG_COORDINATE',
'GL_FOG_COORDINATE_ARRAY_TYPE', 'GL_FOG_COORDINATE_ARRAY_STRIDE',
'GL_FOG_COORDINATE_ARRAY_POINTER', 'GL_FOG_COORDINATE_ARRAY', 'GL_COLOR_SUM',
'GL_CURRENT_SECONDARY_COLOR', 'GL_SECONDARY_COLOR_ARRAY_SIZE',
'GL_SECONDARY_COLOR_ARRAY_TYPE', 'GL_SECONDARY_COLOR_ARRAY_STRIDE',
'GL_SECONDARY_COLOR_ARRAY_POINTER', 'GL_SECONDARY_COLOR_ARRAY',
'GL_TEXTURE_FILTER_CONTROL', 'GL_DEPTH_TEXTURE_MODE',
'GL_COMPARE_R_TO_TEXTURE', 'GL_BUFFER_SIZE', 'GL_BUFFER_USAGE',
'GL_QUERY_COUNTER_BITS', 'GL_CURRENT_QUERY', 'GL_QUERY_RESULT',
'GL_QUERY_RESULT_AVAILABLE', 'GL_ARRAY_BUFFER', 'GL_ELEMENT_ARRAY_BUFFER',
'GL_ARRAY_BUFFER_BINDING', 'GL_ELEMENT_ARRAY_BUFFER_BINDING',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING', 'GL_READ_ONLY', 'GL_WRITE_ONLY',
'GL_READ_WRITE', 'GL_BUFFER_ACCESS', 'GL_BUFFER_MAPPED',
'GL_BUFFER_MAP_POINTER', 'GL_STREAM_DRAW', 'GL_STREAM_READ', 'GL_STREAM_COPY',
'GL_STATIC_DRAW', 'GL_STATIC_READ', 'GL_STATIC_COPY', 'GL_DYNAMIC_DRAW',
'GL_DYNAMIC_READ', 'GL_DYNAMIC_COPY', 'GL_SAMPLES_PASSED',
'GL_VERTEX_ARRAY_BUFFER_BINDING', 'GL_NORMAL_ARRAY_BUFFER_BINDING',
'GL_COLOR_ARRAY_BUFFER_BINDING', 'GL_INDEX_ARRAY_BUFFER_BINDING',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING', 'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING', 'GL_WEIGHT_ARRAY_BUFFER_BINDING',
'GL_FOG_COORD_SRC', 'GL_FOG_COORD', 'GL_CURRENT_FOG_COORD',
'GL_FOG_COORD_ARRAY_TYPE', 'GL_FOG_COORD_ARRAY_STRIDE',
'GL_FOG_COORD_ARRAY_POINTER', 'GL_FOG_COORD_ARRAY',
'GL_FOG_COORD_ARRAY_BUFFER_BINDING', 'GL_SRC0_RGB', 'GL_SRC1_RGB',
'GL_SRC2_RGB', 'GL_SRC0_ALPHA', 'GL_SRC1_ALPHA', 'GL_SRC2_ALPHA',
'GL_BLEND_EQUATION_RGB', 'GL_VERTEX_ATTRIB_ARRAY_ENABLED',
'GL_VERTEX_ATTRIB_ARRAY_SIZE', 'GL_VERTEX_ATTRIB_ARRAY_STRIDE',
'GL_VERTEX_ATTRIB_ARRAY_TYPE', 'GL_CURRENT_VERTEX_ATTRIB',
'GL_VERTEX_PROGRAM_POINT_SIZE', 'GL_VERTEX_ATTRIB_ARRAY_POINTER',
'GL_STENCIL_BACK_FUNC', 'GL_STENCIL_BACK_FAIL',
'GL_STENCIL_BACK_PASS_DEPTH_FAIL', 'GL_STENCIL_BACK_PASS_DEPTH_PASS',
'GL_MAX_DRAW_BUFFERS', 'GL_DRAW_BUFFER0', 'GL_DRAW_BUFFER1',
'GL_DRAW_BUFFER2', 'GL_DRAW_BUFFER3', 'GL_DRAW_BUFFER4', 'GL_DRAW_BUFFER5',
'GL_DRAW_BUFFER6', 'GL_DRAW_BUFFER7', 'GL_DRAW_BUFFER8', 'GL_DRAW_BUFFER9',
'GL_DRAW_BUFFER10', 'GL_DRAW_BUFFER11', 'GL_DRAW_BUFFER12',
'GL_DRAW_BUFFER13', 'GL_DRAW_BUFFER14', 'GL_DRAW_BUFFER15',
'GL_BLEND_EQUATION_ALPHA', 'GL_MAX_VERTEX_ATTRIBS',
'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED', 'GL_MAX_TEXTURE_IMAGE_UNITS',
'GL_FRAGMENT_SHADER', 'GL_VERTEX_SHADER',
'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS', 'GL_MAX_VERTEX_UNIFORM_COMPONENTS',
'GL_MAX_VARYING_FLOATS', 'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS',
'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS', 'GL_SHADER_TYPE', 'GL_FLOAT_VEC2',
'GL_FLOAT_VEC3', 'GL_FLOAT_VEC4', 'GL_INT_VEC2', 'GL_INT_VEC3', 'GL_INT_VEC4',
'GL_BOOL', 'GL_BOOL_VEC2', 'GL_BOOL_VEC3', 'GL_BOOL_VEC4', 'GL_FLOAT_MAT2',
'GL_FLOAT_MAT3', 'GL_FLOAT_MAT4', 'GL_SAMPLER_1D', 'GL_SAMPLER_2D',
'GL_SAMPLER_3D', 'GL_SAMPLER_CUBE', 'GL_SAMPLER_1D_SHADOW',
'GL_SAMPLER_2D_SHADOW', 'GL_DELETE_STATUS', 'GL_COMPILE_STATUS',
'GL_LINK_STATUS', 'GL_VALIDATE_STATUS', 'GL_INFO_LOG_LENGTH',
'GL_ATTACHED_SHADERS', 'GL_ACTIVE_UNIFORMS', 'GL_ACTIVE_UNIFORM_MAX_LENGTH',
'GL_SHADER_SOURCE_LENGTH', 'GL_ACTIVE_ATTRIBUTES',
'GL_ACTIVE_ATTRIBUTE_MAX_LENGTH', 'GL_FRAGMENT_SHADER_DERIVATIVE_HINT',
'GL_SHADING_LANGUAGE_VERSION', 'GL_CURRENT_PROGRAM',
'GL_POINT_SPRITE_COORD_ORIGIN', 'GL_LOWER_LEFT', 'GL_UPPER_LEFT',
'GL_STENCIL_BACK_REF', 'GL_STENCIL_BACK_VALUE_MASK',
'GL_STENCIL_BACK_WRITEMASK', 'GL_VERTEX_PROGRAM_TWO_SIDE', 'GL_POINT_SPRITE',
'GL_COORD_REPLACE', 'GL_MAX_TEXTURE_COORDS', 'GL_PIXEL_PACK_BUFFER',
'GL_PIXEL_UNPACK_BUFFER', 'GL_PIXEL_PACK_BUFFER_BINDING',
'GL_PIXEL_UNPACK_BUFFER_BINDING', 'GL_FLOAT_MAT2x3', 'GL_FLOAT_MAT2x4',
'GL_FLOAT_MAT3x2', 'GL_FLOAT_MAT3x4', 'GL_FLOAT_MAT4x2', 'GL_FLOAT_MAT4x3',
'GL_SRGB', 'GL_SRGB8', 'GL_SRGB_ALPHA', 'GL_SRGB8_ALPHA8',
'GL_COMPRESSED_SRGB', 'GL_COMPRESSED_SRGB_ALPHA',
'GL_CURRENT_RASTER_SECONDARY_COLOR', 'GL_SLUMINANCE_ALPHA',
'GL_SLUMINANCE8_ALPHA8', 'GL_SLUMINANCE', 'GL_SLUMINANCE8',
'GL_COMPRESSED_SLUMINANCE', 'GL_COMPRESSED_SLUMINANCE_ALPHA',
'GL_COMPARE_REF_TO_TEXTURE', 'GL_CLIP_DISTANCE0', 'GL_CLIP_DISTANCE1',
'GL_CLIP_DISTANCE2', 'GL_CLIP_DISTANCE3', 'GL_CLIP_DISTANCE4',
'GL_CLIP_DISTANCE5', 'GL_CLIP_DISTANCE6', 'GL_CLIP_DISTANCE7',
'GL_MAX_CLIP_DISTANCES', 'GL_MAJOR_VERSION', 'GL_MINOR_VERSION',
'GL_NUM_EXTENSIONS', 'GL_CONTEXT_FLAGS', 'GL_COMPRESSED_RED',
'GL_COMPRESSED_RG', 'GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT', 'GL_RGBA32F',
'GL_RGB32F', 'GL_RGBA16F', 'GL_RGB16F', 'GL_VERTEX_ATTRIB_ARRAY_INTEGER',
'GL_MAX_ARRAY_TEXTURE_LAYERS', 'GL_MIN_PROGRAM_TEXEL_OFFSET',
'GL_MAX_PROGRAM_TEXEL_OFFSET', 'GL_CLAMP_READ_COLOR', 'GL_FIXED_ONLY',
'GL_MAX_VARYING_COMPONENTS', 'GL_TEXTURE_1D_ARRAY',
'GL_PROXY_TEXTURE_1D_ARRAY', 'GL_TEXTURE_2D_ARRAY',
'GL_PROXY_TEXTURE_2D_ARRAY', 'GL_TEXTURE_BINDING_1D_ARRAY',
'GL_TEXTURE_BINDING_2D_ARRAY', 'GL_R11F_G11F_B10F',
'GL_UNSIGNED_INT_10F_11F_11F_REV', 'GL_RGB9_E5',
'GL_UNSIGNED_INT_5_9_9_9_REV', 'GL_TEXTURE_SHARED_SIZE',
'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH',
'GL_TRANSFORM_FEEDBACK_BUFFER_MODE',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS',
'GL_TRANSFORM_FEEDBACK_VARYINGS', 'GL_TRANSFORM_FEEDBACK_BUFFER_START',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE', 'GL_PRIMITIVES_GENERATED',
'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN', 'GL_RASTERIZER_DISCARD',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS', 'GL_INTERLEAVED_ATTRIBS',
'GL_SEPARATE_ATTRIBS', 'GL_TRANSFORM_FEEDBACK_BUFFER',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING', 'GL_RGBA32UI', 'GL_RGB32UI',
'GL_RGBA16UI', 'GL_RGB16UI', 'GL_RGBA8UI', 'GL_RGB8UI', 'GL_RGBA32I',
'GL_RGB32I', 'GL_RGBA16I', 'GL_RGB16I', 'GL_RGBA8I', 'GL_RGB8I',
'GL_RED_INTEGER', 'GL_GREEN_INTEGER', 'GL_BLUE_INTEGER', 'GL_RGB_INTEGER',
'GL_RGBA_INTEGER', 'GL_BGR_INTEGER', 'GL_BGRA_INTEGER', 'GL_SAMPLER_1D_ARRAY',
'GL_SAMPLER_2D_ARRAY', 'GL_SAMPLER_1D_ARRAY_SHADOW',
'GL_SAMPLER_2D_ARRAY_SHADOW', 'GL_SAMPLER_CUBE_SHADOW',
'GL_UNSIGNED_INT_VEC2', 'GL_UNSIGNED_INT_VEC3', 'GL_UNSIGNED_INT_VEC4',
'GL_INT_SAMPLER_1D', 'GL_INT_SAMPLER_2D', 'GL_INT_SAMPLER_3D',
'GL_INT_SAMPLER_CUBE', 'GL_INT_SAMPLER_1D_ARRAY', 'GL_INT_SAMPLER_2D_ARRAY',
'GL_UNSIGNED_INT_SAMPLER_1D', 'GL_UNSIGNED_INT_SAMPLER_2D',
'GL_UNSIGNED_INT_SAMPLER_3D', 'GL_UNSIGNED_INT_SAMPLER_CUBE',
'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY', 'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY',
'GL_QUERY_WAIT', 'GL_QUERY_NO_WAIT', 'GL_QUERY_BY_REGION_WAIT',
'GL_QUERY_BY_REGION_NO_WAIT', 'GL_BUFFER_ACCESS_FLAGS',
'GL_BUFFER_MAP_LENGTH', 'GL_BUFFER_MAP_OFFSET', 'GL_CLAMP_VERTEX_COLOR',
'GL_CLAMP_FRAGMENT_COLOR', 'GL_ALPHA_INTEGER', 'GL_SAMPLER_2D_RECT',
'GL_SAMPLER_2D_RECT_SHADOW', 'GL_SAMPLER_BUFFER', 'GL_INT_SAMPLER_2D_RECT',
'GL_INT_SAMPLER_BUFFER', 'GL_UNSIGNED_INT_SAMPLER_2D_RECT',
'GL_UNSIGNED_INT_SAMPLER_BUFFER', 'GL_TEXTURE_BUFFER',
'GL_MAX_TEXTURE_BUFFER_SIZE', 'GL_TEXTURE_BINDING_BUFFER',
'GL_TEXTURE_BUFFER_DATA_STORE_BINDING', 'GL_TEXTURE_BUFFER_FORMAT',
'GL_TEXTURE_RECTANGLE', 'GL_TEXTURE_BINDING_RECTANGLE',
'GL_PROXY_TEXTURE_RECTANGLE', 'GL_MAX_RECTANGLE_TEXTURE_SIZE', 'GL_RED_SNORM',
'GL_RG_SNORM', 'GL_RGB_SNORM', 'GL_RGBA_SNORM', 'GL_R8_SNORM', 'GL_RG8_SNORM',
'GL_RGB8_SNORM', 'GL_RGBA8_SNORM', 'GL_R16_SNORM', 'GL_RG16_SNORM',
'GL_RGB16_SNORM', 'GL_RGBA16_SNORM', 'GL_SIGNED_NORMALIZED',
'GL_PRIMITIVE_RESTART', 'GL_PRIMITIVE_RESTART_INDEX',
'GL_CONTEXT_CORE_PROFILE_BIT', 'GL_CONTEXT_COMPATIBILITY_PROFILE_BIT',
'GL_LINES_ADJACENCY', 'GL_LINE_STRIP_ADJACENCY', 'GL_TRIANGLES_ADJACENCY',
'GL_TRIANGLE_STRIP_ADJACENCY', 'GL_PROGRAM_POINT_SIZE',
'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS', 'GL_FRAMEBUFFER_ATTACHMENT_LAYERED',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS', 'GL_GEOMETRY_SHADER',
'GL_GEOMETRY_VERTICES_OUT', 'GL_GEOMETRY_INPUT_TYPE',
'GL_GEOMETRY_OUTPUT_TYPE', 'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS',
'GL_MAX_GEOMETRY_OUTPUT_VERTICES', 'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS',
'GL_MAX_VERTEX_OUTPUT_COMPONENTS', 'GL_MAX_GEOMETRY_INPUT_COMPONENTS',
'GL_MAX_GEOMETRY_OUTPUT_COMPONENTS', 'GL_MAX_FRAGMENT_INPUT_COMPONENTS',
'GL_CONTEXT_PROFILE_MASK', 'GL_VERTEX_ATTRIB_ARRAY_DIVISOR',
'GL_SAMPLE_SHADING', 'GL_MIN_SAMPLE_SHADING_VALUE',
'GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET',
'GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET', 'GL_TEXTURE_CUBE_MAP_ARRAY',
'GL_TEXTURE_BINDING_CUBE_MAP_ARRAY', 'GL_PROXY_TEXTURE_CUBE_MAP_ARRAY',
'GL_SAMPLER_CUBE_MAP_ARRAY', 'GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW',
'GL_INT_SAMPLER_CUBE_MAP_ARRAY', 'GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY',
'GL_TRANSPOSE_MODELVIEW_MATRIX_ARB', 'GL_TRANSPOSE_PROJECTION_MATRIX_ARB',
'GL_TRANSPOSE_TEXTURE_MATRIX_ARB', 'GL_TRANSPOSE_COLOR_MATRIX_ARB',
'GL_MULTISAMPLE_ARB', 'GL_SAMPLE_ALPHA_TO_COVERAGE_ARB',
'GL_SAMPLE_ALPHA_TO_ONE_ARB', 'GL_SAMPLE_COVERAGE_ARB',
'GL_SAMPLE_BUFFERS_ARB', 'GL_SAMPLES_ARB', 'GL_SAMPLE_COVERAGE_VALUE_ARB',
'GL_SAMPLE_COVERAGE_INVERT_ARB', 'GL_MULTISAMPLE_BIT_ARB',
'GL_NORMAL_MAP_ARB', 'GL_REFLECTION_MAP_ARB', 'GL_TEXTURE_CUBE_MAP_ARB',
'GL_TEXTURE_BINDING_CUBE_MAP_ARB', 'GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB',
'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB', 'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB',
'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB', 'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB',
'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB', 'GL_PROXY_TEXTURE_CUBE_MAP_ARB',
'GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB', 'GL_COMPRESSED_ALPHA_ARB',
'GL_COMPRESSED_LUMINANCE_ARB', 'GL_COMPRESSED_LUMINANCE_ALPHA_ARB',
'GL_COMPRESSED_INTENSITY_ARB', 'GL_COMPRESSED_RGB_ARB',
'GL_COMPRESSED_RGBA_ARB', 'GL_TEXTURE_COMPRESSION_HINT_ARB',
'GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB', 'GL_TEXTURE_COMPRESSED_ARB',
'GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB', 'GL_COMPRESSED_TEXTURE_FORMATS_ARB',
'GL_CLAMP_TO_BORDER_ARB', 'GL_POINT_SIZE_MIN_ARB', 'GL_POINT_SIZE_MAX_ARB',
'GL_POINT_FADE_THRESHOLD_SIZE_ARB', 'GL_POINT_DISTANCE_ATTENUATION_ARB',
'GL_MAX_VERTEX_UNITS_ARB', 'GL_ACTIVE_VERTEX_UNITS_ARB',
'GL_WEIGHT_SUM_UNITY_ARB', 'GL_VERTEX_BLEND_ARB', 'GL_CURRENT_WEIGHT_ARB',
'GL_WEIGHT_ARRAY_TYPE_ARB', 'GL_WEIGHT_ARRAY_STRIDE_ARB',
'GL_WEIGHT_ARRAY_SIZE_ARB', 'GL_WEIGHT_ARRAY_POINTER_ARB',
'GL_WEIGHT_ARRAY_ARB', 'GL_MODELVIEW0_ARB', 'GL_MODELVIEW1_ARB',
'GL_MODELVIEW2_ARB', 'GL_MODELVIEW3_ARB', 'GL_MODELVIEW4_ARB',
'GL_MODELVIEW5_ARB', 'GL_MODELVIEW6_ARB', 'GL_MODELVIEW7_ARB',
'GL_MODELVIEW8_ARB', 'GL_MODELVIEW9_ARB', 'GL_MODELVIEW10_ARB',
'GL_MODELVIEW11_ARB', 'GL_MODELVIEW12_ARB', 'GL_MODELVIEW13_ARB',
'GL_MODELVIEW14_ARB', 'GL_MODELVIEW15_ARB', 'GL_MODELVIEW16_ARB',
'GL_MODELVIEW17_ARB', 'GL_MODELVIEW18_ARB', 'GL_MODELVIEW19_ARB',
'GL_MODELVIEW20_ARB', 'GL_MODELVIEW21_ARB', 'GL_MODELVIEW22_ARB',
'GL_MODELVIEW23_ARB', 'GL_MODELVIEW24_ARB', 'GL_MODELVIEW25_ARB',
'GL_MODELVIEW26_ARB', 'GL_MODELVIEW27_ARB', 'GL_MODELVIEW28_ARB',
'GL_MODELVIEW29_ARB', 'GL_MODELVIEW30_ARB', 'GL_MODELVIEW31_ARB',
'GL_MATRIX_PALETTE_ARB', 'GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB',
'GL_MAX_PALETTE_MATRICES_ARB', 'GL_CURRENT_PALETTE_MATRIX_ARB',
'GL_MATRIX_INDEX_ARRAY_ARB', 'GL_CURRENT_MATRIX_INDEX_ARB',
'GL_MATRIX_INDEX_ARRAY_SIZE_ARB', 'GL_MATRIX_INDEX_ARRAY_TYPE_ARB',
'GL_MATRIX_INDEX_ARRAY_STRIDE_ARB', 'GL_MATRIX_INDEX_ARRAY_POINTER_ARB',
'GL_COMBINE_ARB', 'GL_COMBINE_RGB_ARB', 'GL_COMBINE_ALPHA_ARB',
'GL_SOURCE0_RGB_ARB', 'GL_SOURCE1_RGB_ARB', 'GL_SOURCE2_RGB_ARB',
'GL_SOURCE0_ALPHA_ARB', 'GL_SOURCE1_ALPHA_ARB', 'GL_SOURCE2_ALPHA_ARB',
'GL_OPERAND0_RGB_ARB', 'GL_OPERAND1_RGB_ARB', 'GL_OPERAND2_RGB_ARB',
'GL_OPERAND0_ALPHA_ARB', 'GL_OPERAND1_ALPHA_ARB', 'GL_OPERAND2_ALPHA_ARB',
'GL_RGB_SCALE_ARB', 'GL_ADD_SIGNED_ARB', 'GL_INTERPOLATE_ARB',
'GL_SUBTRACT_ARB', 'GL_CONSTANT_ARB', 'GL_PRIMARY_COLOR_ARB',
'GL_PREVIOUS_ARB', 'GL_DOT3_RGB_ARB', 'GL_DOT3_RGBA_ARB',
'GL_MIRRORED_REPEAT_ARB', 'GL_DEPTH_COMPONENT16_ARB',
'GL_DEPTH_COMPONENT24_ARB', 'GL_DEPTH_COMPONENT32_ARB',
'GL_TEXTURE_DEPTH_SIZE_ARB', 'GL_DEPTH_TEXTURE_MODE_ARB',
'GL_TEXTURE_COMPARE_MODE_ARB', 'GL_TEXTURE_COMPARE_FUNC_ARB',
'GL_COMPARE_R_TO_TEXTURE_ARB', 'GL_TEXTURE_COMPARE_FAIL_VALUE_ARB',
'GL_COLOR_SUM_ARB', 'GL_VERTEX_PROGRAM_ARB',
'GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB', 'GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB',
'GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB',
'GL_CURRENT_VERTEX_ATTRIB_ARB', 'GL_PROGRAM_LENGTH_ARB',
'GL_PROGRAM_STRING_ARB', 'GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB',
'GL_MAX_PROGRAM_MATRICES_ARB', 'GL_CURRENT_MATRIX_STACK_DEPTH_ARB',
'GL_CURRENT_MATRIX_ARB', 'GL_VERTEX_PROGRAM_POINT_SIZE_ARB',
'GL_VERTEX_PROGRAM_TWO_SIDE_ARB', 'GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB',
'GL_PROGRAM_ERROR_POSITION_ARB', 'GL_PROGRAM_BINDING_ARB',
'GL_MAX_VERTEX_ATTRIBS_ARB', 'GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB',
'GL_PROGRAM_ERROR_STRING_ARB', 'GL_PROGRAM_FORMAT_ASCII_ARB',
'GL_PROGRAM_FORMAT_ARB', 'GL_PROGRAM_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_INSTRUCTIONS_ARB', 'GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_TEMPORARIES_ARB', 'GL_PROGRAM_NATIVE_TEMPORARIES_ARB',
'GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB', 'GL_PROGRAM_PARAMETERS_ARB',
'GL_MAX_PROGRAM_PARAMETERS_ARB', 'GL_PROGRAM_NATIVE_PARAMETERS_ARB',
'GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB', 'GL_PROGRAM_ATTRIBS_ARB',
'GL_MAX_PROGRAM_ATTRIBS_ARB', 'GL_PROGRAM_NATIVE_ATTRIBS_ARB',
'GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB', 'GL_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB',
'GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB',
'GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB', 'GL_MAX_PROGRAM_ENV_PARAMETERS_ARB',
'GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB', 'GL_TRANSPOSE_CURRENT_MATRIX_ARB',
'GL_MATRIX0_ARB', 'GL_MATRIX1_ARB', 'GL_MATRIX2_ARB', 'GL_MATRIX3_ARB',
'GL_MATRIX4_ARB', 'GL_MATRIX5_ARB', 'GL_MATRIX6_ARB', 'GL_MATRIX7_ARB',
'GL_MATRIX8_ARB', 'GL_MATRIX9_ARB', 'GL_MATRIX10_ARB', 'GL_MATRIX11_ARB',
'GL_MATRIX12_ARB', 'GL_MATRIX13_ARB', 'GL_MATRIX14_ARB', 'GL_MATRIX15_ARB',
'GL_MATRIX16_ARB', 'GL_MATRIX17_ARB', 'GL_MATRIX18_ARB', 'GL_MATRIX19_ARB',
'GL_MATRIX20_ARB', 'GL_MATRIX21_ARB', 'GL_MATRIX22_ARB', 'GL_MATRIX23_ARB',
'GL_MATRIX24_ARB', 'GL_MATRIX25_ARB', 'GL_MATRIX26_ARB', 'GL_MATRIX27_ARB',
'GL_MATRIX28_ARB', 'GL_MATRIX29_ARB', 'GL_MATRIX30_ARB', 'GL_MATRIX31_ARB',
'GL_FRAGMENT_PROGRAM_ARB', 'GL_PROGRAM_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_TEX_INSTRUCTIONS_ARB', 'GL_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB', 'GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB',
'GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB', 'GL_MAX_TEXTURE_COORDS_ARB',
'GL_MAX_TEXTURE_IMAGE_UNITS_ARB', 'GL_BUFFER_SIZE_ARB', 'GL_BUFFER_USAGE_ARB',
'GL_ARRAY_BUFFER_ARB', 'GL_ELEMENT_ARRAY_BUFFER_ARB',
'GL_ARRAY_BUFFER_BINDING_ARB', 'GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ARRAY_BUFFER_BINDING_ARB', 'GL_NORMAL_ARRAY_BUFFER_BINDING_ARB',
'GL_COLOR_ARRAY_BUFFER_BINDING_ARB', 'GL_INDEX_ARRAY_BUFFER_BINDING_ARB',
'GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB',
'GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB',
'GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB',
'GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB',
'GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB',
'GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB', 'GL_READ_ONLY_ARB',
'GL_WRITE_ONLY_ARB', 'GL_READ_WRITE_ARB', 'GL_BUFFER_ACCESS_ARB',
'GL_BUFFER_MAPPED_ARB', 'GL_BUFFER_MAP_POINTER_ARB', 'GL_STREAM_DRAW_ARB',
'GL_STREAM_READ_ARB', 'GL_STREAM_COPY_ARB', 'GL_STATIC_DRAW_ARB',
'GL_STATIC_READ_ARB', 'GL_STATIC_COPY_ARB', 'GL_DYNAMIC_DRAW_ARB',
'GL_DYNAMIC_READ_ARB', 'GL_DYNAMIC_COPY_ARB', 'GL_QUERY_COUNTER_BITS_ARB',
'GL_CURRENT_QUERY_ARB', 'GL_QUERY_RESULT_ARB',
'GL_QUERY_RESULT_AVAILABLE_ARB', 'GL_SAMPLES_PASSED_ARB',
'GL_PROGRAM_OBJECT_ARB', 'GL_SHADER_OBJECT_ARB', 'GL_OBJECT_TYPE_ARB',
'GL_OBJECT_SUBTYPE_ARB', 'GL_FLOAT_VEC2_ARB', 'GL_FLOAT_VEC3_ARB',
'GL_FLOAT_VEC4_ARB', 'GL_INT_VEC2_ARB', 'GL_INT_VEC3_ARB', 'GL_INT_VEC4_ARB',
'GL_BOOL_ARB', 'GL_BOOL_VEC2_ARB', 'GL_BOOL_VEC3_ARB', 'GL_BOOL_VEC4_ARB',
'GL_FLOAT_MAT2_ARB', 'GL_FLOAT_MAT3_ARB', 'GL_FLOAT_MAT4_ARB',
'GL_SAMPLER_1D_ARB', 'GL_SAMPLER_2D_ARB', 'GL_SAMPLER_3D_ARB',
'GL_SAMPLER_CUBE_ARB', 'GL_SAMPLER_1D_SHADOW_ARB', 'GL_SAMPLER_2D_SHADOW_ARB',
'GL_SAMPLER_2D_RECT_ARB', 'GL_SAMPLER_2D_RECT_SHADOW_ARB',
'GL_OBJECT_DELETE_STATUS_ARB', 'GL_OBJECT_COMPILE_STATUS_ARB',
'GL_OBJECT_LINK_STATUS_ARB', 'GL_OBJECT_VALIDATE_STATUS_ARB',
'GL_OBJECT_INFO_LOG_LENGTH_ARB', 'GL_OBJECT_ATTACHED_OBJECTS_ARB',
'GL_OBJECT_ACTIVE_UNIFORMS_ARB', 'GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB',
'GL_OBJECT_SHADER_SOURCE_LENGTH_ARB', 'GL_VERTEX_SHADER_ARB',
'GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB', 'GL_MAX_VARYING_FLOATS_ARB',
'GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB',
'GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB', 'GL_OBJECT_ACTIVE_ATTRIBUTES_ARB',
'GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB', 'GL_FRAGMENT_SHADER_ARB',
'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB',
'GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB', 'GL_SHADING_LANGUAGE_VERSION_ARB',
'GL_POINT_SPRITE_ARB', 'GL_COORD_REPLACE_ARB', 'GL_MAX_DRAW_BUFFERS_ARB',
'GL_DRAW_BUFFER0_ARB', 'GL_DRAW_BUFFER1_ARB', 'GL_DRAW_BUFFER2_ARB',
'GL_DRAW_BUFFER3_ARB', 'GL_DRAW_BUFFER4_ARB', 'GL_DRAW_BUFFER5_ARB',
'GL_DRAW_BUFFER6_ARB', 'GL_DRAW_BUFFER7_ARB', 'GL_DRAW_BUFFER8_ARB',
'GL_DRAW_BUFFER9_ARB', 'GL_DRAW_BUFFER10_ARB', 'GL_DRAW_BUFFER11_ARB',
'GL_DRAW_BUFFER12_ARB', 'GL_DRAW_BUFFER13_ARB', 'GL_DRAW_BUFFER14_ARB',
'GL_DRAW_BUFFER15_ARB', 'GL_TEXTURE_RECTANGLE_ARB',
'GL_TEXTURE_BINDING_RECTANGLE_ARB', 'GL_PROXY_TEXTURE_RECTANGLE_ARB',
'GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB', 'GL_RGBA_FLOAT_MODE_ARB',
'GL_CLAMP_VERTEX_COLOR_ARB', 'GL_CLAMP_FRAGMENT_COLOR_ARB',
'GL_CLAMP_READ_COLOR_ARB', 'GL_FIXED_ONLY_ARB', 'GL_HALF_FLOAT_ARB',
'GL_TEXTURE_RED_TYPE_ARB', 'GL_TEXTURE_GREEN_TYPE_ARB',
'GL_TEXTURE_BLUE_TYPE_ARB', 'GL_TEXTURE_ALPHA_TYPE_ARB',
'GL_TEXTURE_LUMINANCE_TYPE_ARB', 'GL_TEXTURE_INTENSITY_TYPE_ARB',
'GL_TEXTURE_DEPTH_TYPE_ARB', 'GL_UNSIGNED_NORMALIZED_ARB', 'GL_RGBA32F_ARB',
'GL_RGB32F_ARB', 'GL_ALPHA32F_ARB', 'GL_INTENSITY32F_ARB',
'GL_LUMINANCE32F_ARB', 'GL_LUMINANCE_ALPHA32F_ARB', 'GL_RGBA16F_ARB',
'GL_RGB16F_ARB', 'GL_ALPHA16F_ARB', 'GL_INTENSITY16F_ARB',
'GL_LUMINANCE16F_ARB', 'GL_LUMINANCE_ALPHA16F_ARB',
'GL_PIXEL_PACK_BUFFER_ARB', 'GL_PIXEL_UNPACK_BUFFER_ARB',
'GL_PIXEL_PACK_BUFFER_BINDING_ARB', 'GL_PIXEL_UNPACK_BUFFER_BINDING_ARB',
'GL_DEPTH_COMPONENT32F', 'GL_DEPTH32F_STENCIL8',
'GL_FLOAT_32_UNSIGNED_INT_24_8_REV', 'GL_INVALID_FRAMEBUFFER_OPERATION',
'GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING',
'GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE',
'GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE', 'GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE',
'GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE', 'GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE',
'GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE',
'GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE', 'GL_FRAMEBUFFER_DEFAULT',
'GL_FRAMEBUFFER_UNDEFINED', 'GL_DEPTH_STENCIL_ATTACHMENT',
'GL_MAX_RENDERBUFFER_SIZE', 'GL_DEPTH_STENCIL', 'GL_UNSIGNED_INT_24_8',
'GL_DEPTH24_STENCIL8', 'GL_TEXTURE_STENCIL_SIZE', 'GL_TEXTURE_RED_TYPE',
'GL_TEXTURE_GREEN_TYPE', 'GL_TEXTURE_BLUE_TYPE', 'GL_TEXTURE_ALPHA_TYPE',
'GL_TEXTURE_DEPTH_TYPE', 'GL_UNSIGNED_NORMALIZED', 'GL_FRAMEBUFFER_BINDING',
'GL_DRAW_FRAMEBUFFER_BINDING', 'GL_RENDERBUFFER_BINDING',
'GL_READ_FRAMEBUFFER', 'GL_DRAW_FRAMEBUFFER', 'GL_READ_FRAMEBUFFER_BINDING',
'GL_RENDERBUFFER_SAMPLES', 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE',
'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER', 'GL_FRAMEBUFFER_COMPLETE',
'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT',
'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT',
'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER',
'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER', 'GL_FRAMEBUFFER_UNSUPPORTED',
'GL_MAX_COLOR_ATTACHMENTS', 'GL_COLOR_ATTACHMENT0', 'GL_COLOR_ATTACHMENT1',
'GL_COLOR_ATTACHMENT2', 'GL_COLOR_ATTACHMENT3', 'GL_COLOR_ATTACHMENT4',
'GL_COLOR_ATTACHMENT5', 'GL_COLOR_ATTACHMENT6', 'GL_COLOR_ATTACHMENT7',
'GL_COLOR_ATTACHMENT8', 'GL_COLOR_ATTACHMENT9', 'GL_COLOR_ATTACHMENT10',
'GL_COLOR_ATTACHMENT11', 'GL_COLOR_ATTACHMENT12', 'GL_COLOR_ATTACHMENT13',
'GL_COLOR_ATTACHMENT14', 'GL_COLOR_ATTACHMENT15', 'GL_DEPTH_ATTACHMENT',
'GL_STENCIL_ATTACHMENT', 'GL_FRAMEBUFFER', 'GL_RENDERBUFFER',
'GL_RENDERBUFFER_WIDTH', 'GL_RENDERBUFFER_HEIGHT',
'GL_RENDERBUFFER_INTERNAL_FORMAT', 'GL_STENCIL_INDEX1', 'GL_STENCIL_INDEX4',
'GL_STENCIL_INDEX8', 'GL_STENCIL_INDEX16', 'GL_RENDERBUFFER_RED_SIZE',
'GL_RENDERBUFFER_GREEN_SIZE', 'GL_RENDERBUFFER_BLUE_SIZE',
'GL_RENDERBUFFER_ALPHA_SIZE', 'GL_RENDERBUFFER_DEPTH_SIZE',
'GL_RENDERBUFFER_STENCIL_SIZE', 'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE',
'GL_MAX_SAMPLES', 'GL_INDEX', 'GL_TEXTURE_LUMINANCE_TYPE',
'GL_TEXTURE_INTENSITY_TYPE', 'GL_FRAMEBUFFER_SRGB', 'GL_LINES_ADJACENCY_ARB',
'GL_LINE_STRIP_ADJACENCY_ARB', 'GL_TRIANGLES_ADJACENCY_ARB',
'GL_TRIANGLE_STRIP_ADJACENCY_ARB', 'GL_PROGRAM_POINT_SIZE_ARB',
'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB',
'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB', 'GL_GEOMETRY_SHADER_ARB',
'GL_GEOMETRY_VERTICES_OUT_ARB', 'GL_GEOMETRY_INPUT_TYPE_ARB',
'GL_GEOMETRY_OUTPUT_TYPE_ARB', 'GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB',
'GL_MAX_VERTEX_VARYING_COMPONENTS_ARB',
'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB',
'GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB',
'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB', 'GL_HALF_FLOAT',
'GL_VERTEX_ATTRIB_ARRAY_DIVISOR_ARB', 'GL_MAP_READ_BIT', 'GL_MAP_WRITE_BIT',
'GL_MAP_INVALIDATE_RANGE_BIT', 'GL_MAP_INVALIDATE_BUFFER_BIT',
'GL_MAP_FLUSH_EXPLICIT_BIT', 'GL_MAP_UNSYNCHRONIZED_BIT',
'GL_TEXTURE_BUFFER_ARB', 'GL_MAX_TEXTURE_BUFFER_SIZE_ARB',
'GL_TEXTURE_BINDING_BUFFER_ARB', 'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB',
'GL_TEXTURE_BUFFER_FORMAT_ARB', 'GL_COMPRESSED_RED_RGTC1',
'GL_COMPRESSED_SIGNED_RED_RGTC1', 'GL_COMPRESSED_RG_RGTC2',
'GL_COMPRESSED_SIGNED_RG_RGTC2', 'GL_RG', 'GL_RG_INTEGER', 'GL_R8', 'GL_R16',
'GL_RG8', 'GL_RG16', 'GL_R16F', 'GL_R32F', 'GL_RG16F', 'GL_RG32F', 'GL_R8I',
'GL_R8UI', 'GL_R16I', 'GL_R16UI', 'GL_R32I', 'GL_R32UI', 'GL_RG8I',
'GL_RG8UI', 'GL_RG16I', 'GL_RG16UI', 'GL_RG32I', 'GL_RG32UI',
'GL_VERTEX_ARRAY_BINDING', 'GL_UNIFORM_BUFFER', 'GL_UNIFORM_BUFFER_BINDING',
'GL_UNIFORM_BUFFER_START', 'GL_UNIFORM_BUFFER_SIZE',
'GL_MAX_VERTEX_UNIFORM_BLOCKS', 'GL_MAX_GEOMETRY_UNIFORM_BLOCKS',
'GL_MAX_FRAGMENT_UNIFORM_BLOCKS', 'GL_MAX_COMBINED_UNIFORM_BLOCKS',
'GL_MAX_UNIFORM_BUFFER_BINDINGS', 'GL_MAX_UNIFORM_BLOCK_SIZE',
'GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS',
'GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS',
'GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS',
'GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT',
'GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH', 'GL_ACTIVE_UNIFORM_BLOCKS',
'GL_UNIFORM_TYPE', 'GL_UNIFORM_SIZE', 'GL_UNIFORM_NAME_LENGTH',
'GL_UNIFORM_BLOCK_INDEX', 'GL_UNIFORM_OFFSET', 'GL_UNIFORM_ARRAY_STRIDE',
'GL_UNIFORM_MATRIX_STRIDE', 'GL_UNIFORM_IS_ROW_MAJOR',
'GL_UNIFORM_BLOCK_BINDING', 'GL_UNIFORM_BLOCK_DATA_SIZE',
'GL_UNIFORM_BLOCK_NAME_LENGTH', 'GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS',
'GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES',
'GL_UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER',
'GL_UNIFORM_BLOCK_REFERENCED_BY_GEOMETRY_SHADER',
'GL_UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER', 'GL_INVALID_INDEX',
'GL_COPY_READ_BUFFER_BINDING', 'GL_COPY_READ_BUFFER',
'GL_COPY_WRITE_BUFFER_BINDING', 'GL_COPY_WRITE_BUFFER', 'GL_DEPTH_CLAMP',
'GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION', 'GL_FIRST_VERTEX_CONVENTION',
'GL_LAST_VERTEX_CONVENTION', 'GL_PROVOKING_VERTEX',
'GL_TEXTURE_CUBE_MAP_SEAMLESS', 'GL_MAX_SERVER_WAIT_TIMEOUT',
'GL_OBJECT_TYPE', 'GL_SYNC_CONDITION', 'GL_SYNC_STATUS', 'GL_SYNC_FLAGS',
'GL_SYNC_FENCE', 'GL_SYNC_GPU_COMMANDS_COMPLETE', 'GL_UNSIGNALED',
'GL_SIGNALED', 'GL_ALREADY_SIGNALED', 'GL_TIMEOUT_EXPIRED',
'GL_CONDITION_SATISFIED', 'GL_WAIT_FAILED', 'GL_SYNC_FLUSH_COMMANDS_BIT',
'GL_TIMEOUT_IGNORED', 'GL_SAMPLE_POSITION', 'GL_SAMPLE_MASK',
'GL_SAMPLE_MASK_VALUE', 'GL_MAX_SAMPLE_MASK_WORDS',
'GL_TEXTURE_2D_MULTISAMPLE', 'GL_PROXY_TEXTURE_2D_MULTISAMPLE',
'GL_TEXTURE_2D_MULTISAMPLE_ARRAY', 'GL_PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY',
'GL_TEXTURE_BINDING_2D_MULTISAMPLE',
'GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY', 'GL_TEXTURE_SAMPLES',
'GL_TEXTURE_FIXED_SAMPLE_LOCATIONS', 'GL_SAMPLER_2D_MULTISAMPLE',
'GL_INT_SAMPLER_2D_MULTISAMPLE', 'GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE',
'GL_SAMPLER_2D_MULTISAMPLE_ARRAY', 'GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY',
'GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY',
'GL_MAX_COLOR_TEXTURE_SAMPLES', 'GL_MAX_DEPTH_TEXTURE_SAMPLES',
'GL_MAX_INTEGER_SAMPLES', 'GL_SAMPLE_SHADING_ARB',
'GL_MIN_SAMPLE_SHADING_VALUE_ARB', 'GL_TEXTURE_CUBE_MAP_ARRAY_ARB',
'GL_TEXTURE_BINDING_CUBE_MAP_ARRAY_ARB',
'GL_PROXY_TEXTURE_CUBE_MAP_ARRAY_ARB', 'GL_SAMPLER_CUBE_MAP_ARRAY_ARB',
'GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW_ARB', 'GL_INT_SAMPLER_CUBE_MAP_ARRAY_ARB',
'GL_UNSIGNED_INT_SAMPLER_CUBE_MAP_ARRAY_ARB',
'GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_ARB',
'GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_ARB', 'GL_SHADER_INCLUDE_ARB',
'GL_NAMED_STRING_LENGTH_ARB', 'GL_NAMED_STRING_TYPE_ARB',
'GL_COMPRESSED_RGBA_BPTC_UNORM_ARB',
'GL_COMPRESSED_SRGB_ALPHA_BPTC_UNORM_ARB',
'GL_COMPRESSED_RGB_BPTC_SIGNED_FLOAT_ARB',
'GL_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT_ARB', 'GL_SRC1_COLOR',
'GL_ONE_MINUS_SRC1_COLOR', 'GL_ONE_MINUS_SRC1_ALPHA',
'GL_MAX_DUAL_SOURCE_DRAW_BUFFERS', 'GL_ANY_SAMPLES_PASSED',
'GL_SAMPLER_BINDING', 'GL_RGB10_A2UI', 'GL_TEXTURE_SWIZZLE_R',
'GL_TEXTURE_SWIZZLE_G', 'GL_TEXTURE_SWIZZLE_B', 'GL_TEXTURE_SWIZZLE_A',
'GL_TEXTURE_SWIZZLE_RGBA', 'GL_TIME_ELAPSED', 'GL_TIMESTAMP',
'GL_INT_2_10_10_10_REV', 'GL_DRAW_INDIRECT_BUFFER',
'GL_DRAW_INDIRECT_BUFFER_BINDING', 'GL_GEOMETRY_SHADER_INVOCATIONS',
'GL_MAX_GEOMETRY_SHADER_INVOCATIONS', 'GL_MIN_FRAGMENT_INTERPOLATION_OFFSET',
'GL_MAX_FRAGMENT_INTERPOLATION_OFFSET',
'GL_FRAGMENT_INTERPOLATION_OFFSET_BITS', 'GL_DOUBLE_VEC2', 'GL_DOUBLE_VEC3',
'GL_DOUBLE_VEC4', 'GL_DOUBLE_MAT2', 'GL_DOUBLE_MAT3', 'GL_DOUBLE_MAT4',
'GL_DOUBLE_MAT2x3', 'GL_DOUBLE_MAT2x4', 'GL_DOUBLE_MAT3x2',
'GL_DOUBLE_MAT3x4', 'GL_DOUBLE_MAT4x2', 'GL_DOUBLE_MAT4x3',
'GL_ACTIVE_SUBROUTINES', 'GL_ACTIVE_SUBROUTINE_UNIFORMS',
'GL_ACTIVE_SUBROUTINE_UNIFORM_LOCATIONS', 'GL_ACTIVE_SUBROUTINE_MAX_LENGTH',
'GL_ACTIVE_SUBROUTINE_UNIFORM_MAX_LENGTH', 'GL_MAX_SUBROUTINES',
'GL_MAX_SUBROUTINE_UNIFORM_LOCATIONS', 'GL_NUM_COMPATIBLE_SUBROUTINES',
'GL_COMPATIBLE_SUBROUTINES', 'GL_PATCHES', 'GL_PATCH_VERTICES',
'GL_PATCH_DEFAULT_INNER_LEVEL', 'GL_PATCH_DEFAULT_OUTER_LEVEL',
'GL_TESS_CONTROL_OUTPUT_VERTICES', 'GL_TESS_GEN_MODE', 'GL_TESS_GEN_SPACING',
'GL_TESS_GEN_VERTEX_ORDER', 'GL_TESS_GEN_POINT_MODE', 'GL_ISOLINES',
'GL_FRACTIONAL_ODD', 'GL_FRACTIONAL_EVEN', 'GL_MAX_PATCH_VERTICES',
'GL_MAX_TESS_GEN_LEVEL', 'GL_MAX_TESS_CONTROL_UNIFORM_COMPONENTS',
'GL_MAX_TESS_EVALUATION_UNIFORM_COMPONENTS',
'GL_MAX_TESS_CONTROL_TEXTURE_IMAGE_UNITS',
'GL_MAX_TESS_EVALUATION_TEXTURE_IMAGE_UNITS',
'GL_MAX_TESS_CONTROL_OUTPUT_COMPONENTS', 'GL_MAX_TESS_PATCH_COMPONENTS',
'GL_MAX_TESS_CONTROL_TOTAL_OUTPUT_COMPONENTS',
'GL_MAX_TESS_EVALUATION_OUTPUT_COMPONENTS',
'GL_MAX_TESS_CONTROL_UNIFORM_BLOCKS', 'GL_MAX_TESS_EVALUATION_UNIFORM_BLOCKS',
'GL_MAX_TESS_CONTROL_INPUT_COMPONENTS',
'GL_MAX_TESS_EVALUATION_INPUT_COMPONENTS',
'GL_MAX_COMBINED_TESS_CONTROL_UNIFORM_COMPONENTS',
'GL_MAX_COMBINED_TESS_EVALUATION_UNIFORM_COMPONENTS',
'GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_CONTROL_SHADER',
'GL_UNIFORM_BLOCK_REFERENCED_BY_TESS_EVALUATION_SHADER',
'GL_TESS_EVALUATION_SHADER', 'GL_TESS_CONTROL_SHADER',
'GL_TRANSFORM_FEEDBACK', 'GL_TRANSFORM_FEEDBACK_PAUSED',
'GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED', 'GL_TRANSFORM_FEEDBACK_ACTIVE',
'GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE', 'GL_TRANSFORM_FEEDBACK_BINDING',
'GL_MAX_TRANSFORM_FEEDBACK_BUFFERS', 'GL_MAX_VERTEX_STREAMS', 'GL_FIXED',
'GL_IMPLEMENTATION_COLOR_READ_TYPE', 'GL_IMPLEMENTATION_COLOR_READ_FORMAT',
'GL_LOW_FLOAT', 'GL_MEDIUM_FLOAT', 'GL_HIGH_FLOAT', 'GL_LOW_INT',
'GL_MEDIUM_INT', 'GL_HIGH_INT', 'GL_SHADER_COMPILER',
'GL_NUM_SHADER_BINARY_FORMATS', 'GL_MAX_VERTEX_UNIFORM_VECTORS',
'GL_MAX_VARYING_VECTORS', 'GL_MAX_FRAGMENT_UNIFORM_VECTORS', 'GL_RGB565',
'GL_PROGRAM_BINARY_RETRIEVABLE_HINT', 'GL_PROGRAM_BINARY_LENGTH',
'GL_NUM_PROGRAM_BINARY_FORMATS', 'GL_PROGRAM_BINARY_FORMATS',
'GL_VERTEX_SHADER_BIT', 'GL_FRAGMENT_SHADER_BIT', 'GL_GEOMETRY_SHADER_BIT',
'GL_TESS_CONTROL_SHADER_BIT', 'GL_TESS_EVALUATION_SHADER_BIT',
'GL_ALL_SHADER_BITS', 'GL_PROGRAM_SEPARABLE', 'GL_ACTIVE_PROGRAM',
'GL_PROGRAM_PIPELINE_BINDING', 'GL_MAX_VIEWPORTS',
'GL_VIEWPORT_SUBPIXEL_BITS', 'GL_VIEWPORT_BOUNDS_RANGE',
'GL_LAYER_PROVOKING_VERTEX', 'GL_VIEWPORT_INDEX_PROVOKING_VERTEX',
'GL_UNDEFINED_VERTEX', 'GL_SYNC_CL_EVENT_ARB',
'GL_SYNC_CL_EVENT_COMPLETE_ARB', 'GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB',
'GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH_ARB', 'GL_DEBUG_CALLBACK_FUNCTION_ARB',
'GL_DEBUG_CALLBACK_USER_PARAM_ARB', 'GL_DEBUG_SOURCE_API_ARB',
'GL_DEBUG_SOURCE_WINDOW_SYSTEM_ARB', 'GL_DEBUG_SOURCE_SHADER_COMPILER_ARB',
'GL_DEBUG_SOURCE_THIRD_PARTY_ARB', 'GL_DEBUG_SOURCE_APPLICATION_ARB',
'GL_DEBUG_SOURCE_OTHER_ARB', 'GL_DEBUG_TYPE_ERROR_ARB',
'GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR_ARB',
'GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR_ARB', 'GL_DEBUG_TYPE_PORTABILITY_ARB',
'GL_DEBUG_TYPE_PERFORMANCE_ARB', 'GL_DEBUG_TYPE_OTHER_ARB',
'GL_MAX_DEBUG_MESSAGE_LENGTH_ARB', 'GL_MAX_DEBUG_LOGGED_MESSAGES_ARB',
'GL_DEBUG_LOGGED_MESSAGES_ARB', 'GL_DEBUG_SEVERITY_HIGH_ARB',
'GL_DEBUG_SEVERITY_MEDIUM_ARB', 'GL_DEBUG_SEVERITY_LOW_ARB',
'GL_CONTEXT_FLAG_ROBUST_ACCESS_BIT_ARB', 'GL_LOSE_CONTEXT_ON_RESET_ARB',
'GL_GUILTY_CONTEXT_RESET_ARB', 'GL_INNOCENT_CONTEXT_RESET_ARB',
'GL_UNKNOWN_CONTEXT_RESET_ARB', 'GL_RESET_NOTIFICATION_STRATEGY_ARB',
'GL_NO_RESET_NOTIFICATION_ARB', 'GL_UNPACK_COMPRESSED_BLOCK_WIDTH',
'GL_UNPACK_COMPRESSED_BLOCK_HEIGHT', 'GL_UNPACK_COMPRESSED_BLOCK_DEPTH',
'GL_UNPACK_COMPRESSED_BLOCK_SIZE', 'GL_PACK_COMPRESSED_BLOCK_WIDTH',
'GL_PACK_COMPRESSED_BLOCK_HEIGHT', 'GL_PACK_COMPRESSED_BLOCK_DEPTH',
'GL_PACK_COMPRESSED_BLOCK_SIZE', 'GL_NUM_SAMPLE_COUNTS',
'GL_MIN_MAP_BUFFER_ALIGNMENT', 'GL_ATOMIC_COUNTER_BUFFER',
'GL_ATOMIC_COUNTER_BUFFER_BINDING', 'GL_ATOMIC_COUNTER_BUFFER_START',
'GL_ATOMIC_COUNTER_BUFFER_SIZE', 'GL_ATOMIC_COUNTER_BUFFER_DATA_SIZE',
'GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTERS',
'GL_ATOMIC_COUNTER_BUFFER_ACTIVE_ATOMIC_COUNTER_INDICES',
'GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_VERTEX_SHADER',
'GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_CONTROL_SHADER',
'GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_TESS_EVALUATION_SHADER',
'GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_GEOMETRY_SHADER',
'GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_FRAGMENT_SHADER',
'GL_MAX_VERTEX_ATOMIC_COUNTER_BUFFERS',
'GL_MAX_TESS_CONTROL_ATOMIC_COUNTER_BUFFERS',
'GL_MAX_TESS_EVALUATION_ATOMIC_COUNTER_BUFFERS',
'GL_MAX_GEOMETRY_ATOMIC_COUNTER_BUFFERS',
'GL_MAX_FRAGMENT_ATOMIC_COUNTER_BUFFERS',
'GL_MAX_COMBINED_ATOMIC_COUNTER_BUFFERS', 'GL_MAX_VERTEX_ATOMIC_COUNTERS',
'GL_MAX_TESS_CONTROL_ATOMIC_COUNTERS',
'GL_MAX_TESS_EVALUATION_ATOMIC_COUNTERS', 'GL_MAX_GEOMETRY_ATOMIC_COUNTERS',
'GL_MAX_FRAGMENT_ATOMIC_COUNTERS', 'GL_MAX_COMBINED_ATOMIC_COUNTERS',
'GL_MAX_ATOMIC_COUNTER_BUFFER_SIZE', 'GL_MAX_ATOMIC_COUNTER_BUFFER_BINDINGS',
'GL_ACTIVE_ATOMIC_COUNTER_BUFFERS', 'GL_UNIFORM_ATOMIC_COUNTER_BUFFER_INDEX',
'GL_UNSIGNED_INT_ATOMIC_COUNTER', 'GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT',
'GL_ELEMENT_ARRAY_BARRIER_BIT', 'GL_UNIFORM_BARRIER_BIT',
'GL_TEXTURE_FETCH_BARRIER_BIT', 'GL_SHADER_IMAGE_ACCESS_BARRIER_BIT',
'GL_COMMAND_BARRIER_BIT', 'GL_PIXEL_BUFFER_BARRIER_BIT',
'GL_TEXTURE_UPDATE_BARRIER_BIT', 'GL_BUFFER_UPDATE_BARRIER_BIT',
'GL_FRAMEBUFFER_BARRIER_BIT', 'GL_TRANSFORM_FEEDBACK_BARRIER_BIT',
'GL_ATOMIC_COUNTER_BARRIER_BIT', 'GL_ALL_BARRIER_BITS', 'GL_MAX_IMAGE_UNITS',
'GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS', 'GL_IMAGE_BINDING_NAME',
'GL_IMAGE_BINDING_LEVEL', 'GL_IMAGE_BINDING_LAYERED',
'GL_IMAGE_BINDING_LAYER', 'GL_IMAGE_BINDING_ACCESS', 'GL_IMAGE_1D',
'GL_IMAGE_2D', 'GL_IMAGE_3D', 'GL_IMAGE_2D_RECT', 'GL_IMAGE_CUBE',
'GL_IMAGE_BUFFER', 'GL_IMAGE_1D_ARRAY', 'GL_IMAGE_2D_ARRAY',
'GL_IMAGE_CUBE_MAP_ARRAY', 'GL_IMAGE_2D_MULTISAMPLE',
'GL_IMAGE_2D_MULTISAMPLE_ARRAY', 'GL_INT_IMAGE_1D', 'GL_INT_IMAGE_2D',
'GL_INT_IMAGE_3D', 'GL_INT_IMAGE_2D_RECT', 'GL_INT_IMAGE_CUBE',
'GL_INT_IMAGE_BUFFER', 'GL_INT_IMAGE_1D_ARRAY', 'GL_INT_IMAGE_2D_ARRAY',
'GL_INT_IMAGE_CUBE_MAP_ARRAY', 'GL_INT_IMAGE_2D_MULTISAMPLE',
'GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY', 'GL_UNSIGNED_INT_IMAGE_1D',
'GL_UNSIGNED_INT_IMAGE_2D', 'GL_UNSIGNED_INT_IMAGE_3D',
'GL_UNSIGNED_INT_IMAGE_2D_RECT', 'GL_UNSIGNED_INT_IMAGE_CUBE',
'GL_UNSIGNED_INT_IMAGE_BUFFER', 'GL_UNSIGNED_INT_IMAGE_1D_ARRAY',
'GL_UNSIGNED_INT_IMAGE_2D_ARRAY', 'GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY',
'GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE',
'GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY', 'GL_MAX_IMAGE_SAMPLES',
'GL_IMAGE_BINDING_FORMAT', 'GL_IMAGE_FORMAT_COMPATIBILITY_TYPE',
'GL_IMAGE_FORMAT_COMPATIBILITY_BY_SIZE',
'GL_IMAGE_FORMAT_COMPATIBILITY_BY_CLASS', 'GL_MAX_VERTEX_IMAGE_UNIFORMS',
'GL_MAX_TESS_CONTROL_IMAGE_UNIFORMS', 'GL_MAX_TESS_EVALUATION_IMAGE_UNIFORMS',
'GL_MAX_GEOMETRY_IMAGE_UNIFORMS', 'GL_MAX_FRAGMENT_IMAGE_UNIFORMS',
'GL_MAX_COMBINED_IMAGE_UNIFORMS', 'GL_TEXTURE_IMMUTABLE_FORMAT',
'GL_ABGR_EXT', 'GL_CONSTANT_COLOR_EXT', 'GL_ONE_MINUS_CONSTANT_COLOR_EXT',
'GL_CONSTANT_ALPHA_EXT', 'GL_ONE_MINUS_CONSTANT_ALPHA_EXT',
'GL_BLEND_COLOR_EXT', 'GL_POLYGON_OFFSET_EXT', 'GL_POLYGON_OFFSET_FACTOR_EXT',
'GL_POLYGON_OFFSET_BIAS_EXT', 'GL_ALPHA4_EXT', 'GL_ALPHA8_EXT',
'GL_ALPHA12_EXT', 'GL_ALPHA16_EXT', 'GL_LUMINANCE4_EXT', 'GL_LUMINANCE8_EXT',
'GL_LUMINANCE12_EXT', 'GL_LUMINANCE16_EXT', 'GL_LUMINANCE4_ALPHA4_EXT',
'GL_LUMINANCE6_ALPHA2_EXT', 'GL_LUMINANCE8_ALPHA8_EXT',
'GL_LUMINANCE12_ALPHA4_EXT', 'GL_LUMINANCE12_ALPHA12_EXT',
'GL_LUMINANCE16_ALPHA16_EXT', 'GL_INTENSITY_EXT', 'GL_INTENSITY4_EXT',
'GL_INTENSITY8_EXT', 'GL_INTENSITY12_EXT', 'GL_INTENSITY16_EXT',
'GL_RGB2_EXT', 'GL_RGB4_EXT', 'GL_RGB5_EXT', 'GL_RGB8_EXT', 'GL_RGB10_EXT',
'GL_RGB12_EXT', 'GL_RGB16_EXT', 'GL_RGBA2_EXT', 'GL_RGBA4_EXT',
'GL_RGB5_A1_EXT', 'GL_RGBA8_EXT', 'GL_RGB10_A2_EXT', 'GL_RGBA12_EXT',
'GL_RGBA16_EXT', 'GL_TEXTURE_RED_SIZE_EXT', 'GL_TEXTURE_GREEN_SIZE_EXT',
'GL_TEXTURE_BLUE_SIZE_EXT', 'GL_TEXTURE_ALPHA_SIZE_EXT',
'GL_TEXTURE_LUMINANCE_SIZE_EXT', 'GL_TEXTURE_INTENSITY_SIZE_EXT',
'GL_REPLACE_EXT', 'GL_PROXY_TEXTURE_1D_EXT', 'GL_PROXY_TEXTURE_2D_EXT',
'GL_TEXTURE_TOO_LARGE_EXT', 'GL_PACK_SKIP_IMAGES_EXT',
'GL_PACK_IMAGE_HEIGHT_EXT', 'GL_UNPACK_SKIP_IMAGES_EXT',
'GL_UNPACK_IMAGE_HEIGHT_EXT', 'GL_TEXTURE_3D_EXT', 'GL_PROXY_TEXTURE_3D_EXT',
'GL_TEXTURE_DEPTH_EXT', 'GL_TEXTURE_WRAP_R_EXT', 'GL_MAX_3D_TEXTURE_SIZE_EXT',
'GL_FILTER4_SGIS', 'GL_TEXTURE_FILTER4_SIZE_SGIS', 'GL_HISTOGRAM_EXT',
'GL_PROXY_HISTOGRAM_EXT', 'GL_HISTOGRAM_WIDTH_EXT', 'GL_HISTOGRAM_FORMAT_EXT',
'GL_HISTOGRAM_RED_SIZE_EXT', 'GL_HISTOGRAM_GREEN_SIZE_EXT',
'GL_HISTOGRAM_BLUE_SIZE_EXT', 'GL_HISTOGRAM_ALPHA_SIZE_EXT',
'GL_HISTOGRAM_LUMINANCE_SIZE_EXT', 'GL_HISTOGRAM_SINK_EXT', 'GL_MINMAX_EXT',
'GL_MINMAX_FORMAT_EXT', 'GL_MINMAX_SINK_EXT', 'GL_TABLE_TOO_LARGE_EXT',
'GL_CONVOLUTION_1D_EXT', 'GL_CONVOLUTION_2D_EXT', 'GL_SEPARABLE_2D_EXT',
'GL_CONVOLUTION_BORDER_MODE_EXT', 'GL_CONVOLUTION_FILTER_SCALE_EXT',
'GL_CONVOLUTION_FILTER_BIAS_EXT', 'GL_REDUCE_EXT',
'GL_CONVOLUTION_FORMAT_EXT', 'GL_CONVOLUTION_WIDTH_EXT',
'GL_CONVOLUTION_HEIGHT_EXT', 'GL_MAX_CONVOLUTION_WIDTH_EXT',
'GL_MAX_CONVOLUTION_HEIGHT_EXT', 'GL_POST_CONVOLUTION_RED_SCALE_EXT',
'GL_POST_CONVOLUTION_GREEN_SCALE_EXT', 'GL_POST_CONVOLUTION_BLUE_SCALE_EXT',
'GL_POST_CONVOLUTION_ALPHA_SCALE_EXT', 'GL_POST_CONVOLUTION_RED_BIAS_EXT',
'GL_POST_CONVOLUTION_GREEN_BIAS_EXT', 'GL_POST_CONVOLUTION_BLUE_BIAS_EXT',
'GL_POST_CONVOLUTION_ALPHA_BIAS_EXT', 'GL_COLOR_MATRIX_SGI',
'GL_COLOR_MATRIX_STACK_DEPTH_SGI', 'GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI',
'GL_POST_COLOR_MATRIX_RED_SCALE_SGI', 'GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI',
'GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI',
'GL_POST_COLOR_MATRIX_RED_BIAS_SGI', 'GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI',
'GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI', 'GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI',
'GL_COLOR_TABLE_SGI', 'GL_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_PROXY_COLOR_TABLE_SGI',
'GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI',
'GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI', 'GL_COLOR_TABLE_SCALE_SGI',
'GL_COLOR_TABLE_BIAS_SGI', 'GL_COLOR_TABLE_FORMAT_SGI',
'GL_COLOR_TABLE_WIDTH_SGI', 'GL_COLOR_TABLE_RED_SIZE_SGI',
'GL_COLOR_TABLE_GREEN_SIZE_SGI', 'GL_COLOR_TABLE_BLUE_SIZE_SGI',
'GL_COLOR_TABLE_ALPHA_SIZE_SGI', 'GL_COLOR_TABLE_LUMINANCE_SIZE_SGI',
'GL_COLOR_TABLE_INTENSITY_SIZE_SGI', 'GL_PIXEL_TEXTURE_SGIS',
'GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS', 'GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS',
'GL_PIXEL_GROUP_COLOR_SGIS', 'GL_PIXEL_TEX_GEN_SGIX',
'GL_PIXEL_TEX_GEN_MODE_SGIX', 'GL_PACK_SKIP_VOLUMES_SGIS',
'GL_PACK_IMAGE_DEPTH_SGIS', 'GL_UNPACK_SKIP_VOLUMES_SGIS',
'GL_UNPACK_IMAGE_DEPTH_SGIS', 'GL_TEXTURE_4D_SGIS',
'GL_PROXY_TEXTURE_4D_SGIS', 'GL_TEXTURE_4DSIZE_SGIS',
'GL_TEXTURE_WRAP_Q_SGIS', 'GL_MAX_4D_TEXTURE_SIZE_SGIS',
'GL_TEXTURE_4D_BINDING_SGIS', 'GL_TEXTURE_COLOR_TABLE_SGI',
'GL_PROXY_TEXTURE_COLOR_TABLE_SGI', 'GL_CMYK_EXT', 'GL_CMYKA_EXT',
'GL_PACK_CMYK_HINT_EXT', 'GL_UNPACK_CMYK_HINT_EXT', 'GL_TEXTURE_PRIORITY_EXT',
'GL_TEXTURE_RESIDENT_EXT', 'GL_TEXTURE_1D_BINDING_EXT',
'GL_TEXTURE_2D_BINDING_EXT', 'GL_TEXTURE_3D_BINDING_EXT',
'GL_DETAIL_TEXTURE_2D_SGIS', 'GL_DETAIL_TEXTURE_2D_BINDING_SGIS',
'GL_LINEAR_DETAIL_SGIS', 'GL_LINEAR_DETAIL_ALPHA_SGIS',
'GL_LINEAR_DETAIL_COLOR_SGIS', 'GL_DETAIL_TEXTURE_LEVEL_SGIS',
'GL_DETAIL_TEXTURE_MODE_SGIS', 'GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS',
'GL_LINEAR_SHARPEN_SGIS', 'GL_LINEAR_SHARPEN_ALPHA_SGIS',
'GL_LINEAR_SHARPEN_COLOR_SGIS', 'GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS',
'GL_UNSIGNED_BYTE_3_3_2_EXT', 'GL_UNSIGNED_SHORT_4_4_4_4_EXT',
'GL_UNSIGNED_SHORT_5_5_5_1_EXT', 'GL_UNSIGNED_INT_8_8_8_8_EXT',
'GL_UNSIGNED_INT_10_10_10_2_EXT', 'GL_TEXTURE_MIN_LOD_SGIS',
'GL_TEXTURE_MAX_LOD_SGIS', 'GL_TEXTURE_BASE_LEVEL_SGIS',
'GL_TEXTURE_MAX_LEVEL_SGIS', 'GL_MULTISAMPLE_SGIS',
'GL_SAMPLE_ALPHA_TO_MASK_SGIS', 'GL_SAMPLE_ALPHA_TO_ONE_SGIS',
'GL_SAMPLE_MASK_SGIS', 'GL_1PASS_SGIS', 'GL_2PASS_0_SGIS', 'GL_2PASS_1_SGIS',
'GL_4PASS_0_SGIS', 'GL_4PASS_1_SGIS', 'GL_4PASS_2_SGIS', 'GL_4PASS_3_SGIS',
'GL_SAMPLE_BUFFERS_SGIS', 'GL_SAMPLES_SGIS', 'GL_SAMPLE_MASK_VALUE_SGIS',
'GL_SAMPLE_MASK_INVERT_SGIS', 'GL_SAMPLE_PATTERN_SGIS',
'GL_RESCALE_NORMAL_EXT', 'GL_VERTEX_ARRAY_EXT', 'GL_NORMAL_ARRAY_EXT',
'GL_COLOR_ARRAY_EXT', 'GL_INDEX_ARRAY_EXT', 'GL_TEXTURE_COORD_ARRAY_EXT',
'GL_EDGE_FLAG_ARRAY_EXT', 'GL_VERTEX_ARRAY_SIZE_EXT',
'GL_VERTEX_ARRAY_TYPE_EXT', 'GL_VERTEX_ARRAY_STRIDE_EXT',
'GL_VERTEX_ARRAY_COUNT_EXT', 'GL_NORMAL_ARRAY_TYPE_EXT',
'GL_NORMAL_ARRAY_STRIDE_EXT', 'GL_NORMAL_ARRAY_COUNT_EXT',
'GL_COLOR_ARRAY_SIZE_EXT', 'GL_COLOR_ARRAY_TYPE_EXT',
'GL_COLOR_ARRAY_STRIDE_EXT', 'GL_COLOR_ARRAY_COUNT_EXT',
'GL_INDEX_ARRAY_TYPE_EXT', 'GL_INDEX_ARRAY_STRIDE_EXT',
'GL_INDEX_ARRAY_COUNT_EXT', 'GL_TEXTURE_COORD_ARRAY_SIZE_EXT',
'GL_TEXTURE_COORD_ARRAY_TYPE_EXT', 'GL_TEXTURE_COORD_ARRAY_STRIDE_EXT',
'GL_TEXTURE_COORD_ARRAY_COUNT_EXT', 'GL_EDGE_FLAG_ARRAY_STRIDE_EXT',
'GL_EDGE_FLAG_ARRAY_COUNT_EXT', 'GL_VERTEX_ARRAY_POINTER_EXT',
'GL_NORMAL_ARRAY_POINTER_EXT', 'GL_COLOR_ARRAY_POINTER_EXT',
'GL_INDEX_ARRAY_POINTER_EXT', 'GL_TEXTURE_COORD_ARRAY_POINTER_EXT',
'GL_EDGE_FLAG_ARRAY_POINTER_EXT', 'GL_GENERATE_MIPMAP_SGIS',
'GL_GENERATE_MIPMAP_HINT_SGIS', 'GL_LINEAR_CLIPMAP_LINEAR_SGIX',
'GL_TEXTURE_CLIPMAP_CENTER_SGIX', 'GL_TEXTURE_CLIPMAP_FRAME_SGIX',
'GL_TEXTURE_CLIPMAP_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX', 'GL_TEXTURE_CLIPMAP_DEPTH_SGIX',
'GL_MAX_CLIPMAP_DEPTH_SGIX', 'GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX',
'GL_NEAREST_CLIPMAP_NEAREST_SGIX', 'GL_NEAREST_CLIPMAP_LINEAR_SGIX',
'GL_LINEAR_CLIPMAP_NEAREST_SGIX', 'GL_TEXTURE_COMPARE_SGIX',
'GL_TEXTURE_COMPARE_OPERATOR_SGIX', 'GL_TEXTURE_LEQUAL_R_SGIX',
'GL_TEXTURE_GEQUAL_R_SGIX', 'GL_CLAMP_TO_EDGE_SGIS',
'GL_CLAMP_TO_BORDER_SGIS', 'GL_FUNC_ADD_EXT', 'GL_MIN_EXT', 'GL_MAX_EXT',
'GL_BLEND_EQUATION_EXT', 'GL_FUNC_SUBTRACT_EXT',
'GL_FUNC_REVERSE_SUBTRACT_EXT', 'GL_INTERLACE_SGIX',
'GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX', 'GL_PIXEL_TILE_CACHE_INCREMENT_SGIX',
'GL_PIXEL_TILE_WIDTH_SGIX', 'GL_PIXEL_TILE_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_WIDTH_SGIX', 'GL_PIXEL_TILE_GRID_HEIGHT_SGIX',
'GL_PIXEL_TILE_GRID_DEPTH_SGIX', 'GL_PIXEL_TILE_CACHE_SIZE_SGIX',
'GL_DUAL_ALPHA4_SGIS', 'GL_DUAL_ALPHA8_SGIS', 'GL_DUAL_ALPHA12_SGIS',
'GL_DUAL_ALPHA16_SGIS', 'GL_DUAL_LUMINANCE4_SGIS', 'GL_DUAL_LUMINANCE8_SGIS',
'GL_DUAL_LUMINANCE12_SGIS', 'GL_DUAL_LUMINANCE16_SGIS',
'GL_DUAL_INTENSITY4_SGIS', 'GL_DUAL_INTENSITY8_SGIS',
'GL_DUAL_INTENSITY12_SGIS', 'GL_DUAL_INTENSITY16_SGIS',
'GL_DUAL_LUMINANCE_ALPHA4_SGIS', 'GL_DUAL_LUMINANCE_ALPHA8_SGIS',
'GL_QUAD_ALPHA4_SGIS', 'GL_QUAD_ALPHA8_SGIS', 'GL_QUAD_LUMINANCE4_SGIS',
'GL_QUAD_LUMINANCE8_SGIS', 'GL_QUAD_INTENSITY4_SGIS',
'GL_QUAD_INTENSITY8_SGIS', 'GL_DUAL_TEXTURE_SELECT_SGIS',
'GL_QUAD_TEXTURE_SELECT_SGIS', 'GL_SPRITE_SGIX', 'GL_SPRITE_MODE_SGIX',
'GL_SPRITE_AXIS_SGIX', 'GL_SPRITE_TRANSLATION_SGIX', 'GL_SPRITE_AXIAL_SGIX',
'GL_SPRITE_OBJECT_ALIGNED_SGIX', 'GL_SPRITE_EYE_ALIGNED_SGIX',
'GL_TEXTURE_MULTI_BUFFER_HINT_SGIX', 'GL_POINT_SIZE_MIN_EXT',
'GL_POINT_SIZE_MAX_EXT', 'GL_POINT_FADE_THRESHOLD_SIZE_EXT',
'GL_DISTANCE_ATTENUATION_EXT', 'GL_POINT_SIZE_MIN_SGIS',
'GL_POINT_SIZE_MAX_SGIS', 'GL_POINT_FADE_THRESHOLD_SIZE_SGIS',
'GL_DISTANCE_ATTENUATION_SGIS', 'GL_INSTRUMENT_BUFFER_POINTER_SGIX',
'GL_INSTRUMENT_MEASUREMENTS_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_SGIX', 'GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX',
'GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX', 'GL_FRAMEZOOM_SGIX',
'GL_FRAMEZOOM_FACTOR_SGIX', 'GL_MAX_FRAMEZOOM_FACTOR_SGIX',
'GL_TEXTURE_DEFORMATION_BIT_SGIX', 'GL_GEOMETRY_DEFORMATION_BIT_SGIX',
'GL_GEOMETRY_DEFORMATION_SGIX', 'GL_TEXTURE_DEFORMATION_SGIX',
'GL_DEFORMATIONS_MASK_SGIX', 'GL_MAX_DEFORMATION_ORDER_SGIX',
'GL_REFERENCE_PLANE_SGIX', 'GL_REFERENCE_PLANE_EQUATION_SGIX',
'GL_DEPTH_COMPONENT16_SGIX', 'GL_DEPTH_COMPONENT24_SGIX',
'GL_DEPTH_COMPONENT32_SGIX', 'GL_FOG_FUNC_SGIS', 'GL_FOG_FUNC_POINTS_SGIS',
'GL_MAX_FOG_FUNC_POINTS_SGIS', 'GL_FOG_OFFSET_SGIX',
'GL_FOG_OFFSET_VALUE_SGIX', 'GL_IMAGE_SCALE_X_HP', 'GL_IMAGE_SCALE_Y_HP',
'GL_IMAGE_TRANSLATE_X_HP', 'GL_IMAGE_TRANSLATE_Y_HP',
'GL_IMAGE_ROTATE_ANGLE_HP', 'GL_IMAGE_ROTATE_ORIGIN_X_HP',
'GL_IMAGE_ROTATE_ORIGIN_Y_HP', 'GL_IMAGE_MAG_FILTER_HP',
'GL_IMAGE_MIN_FILTER_HP', 'GL_IMAGE_CUBIC_WEIGHT_HP', 'GL_CUBIC_HP',
'GL_AVERAGE_HP', 'GL_IMAGE_TRANSFORM_2D_HP',
'GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP',
'GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP', 'GL_IGNORE_BORDER_HP',
'GL_CONSTANT_BORDER_HP', 'GL_REPLICATE_BORDER_HP',
'GL_CONVOLUTION_BORDER_COLOR_HP', 'GL_TEXTURE_ENV_BIAS_SGIX',
'GL_VERTEX_DATA_HINT_PGI', 'GL_VERTEX_CONSISTENT_HINT_PGI',
'GL_MATERIAL_SIDE_HINT_PGI', 'GL_MAX_VERTEX_HINT_PGI', 'GL_COLOR3_BIT_PGI',
'GL_COLOR4_BIT_PGI', 'GL_EDGEFLAG_BIT_PGI', 'GL_INDEX_BIT_PGI',
'GL_MAT_AMBIENT_BIT_PGI', 'GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI',
'GL_MAT_DIFFUSE_BIT_PGI', 'GL_MAT_EMISSION_BIT_PGI',
'GL_MAT_COLOR_INDEXES_BIT_PGI', 'GL_MAT_SHININESS_BIT_PGI',
'GL_MAT_SPECULAR_BIT_PGI', 'GL_NORMAL_BIT_PGI', 'GL_TEXCOORD1_BIT_PGI',
'GL_TEXCOORD2_BIT_PGI', 'GL_TEXCOORD3_BIT_PGI', 'GL_TEXCOORD4_BIT_PGI',
'GL_VERTEX23_BIT_PGI', 'GL_VERTEX4_BIT_PGI',
'GL_PREFER_DOUBLEBUFFER_HINT_PGI', 'GL_CONSERVE_MEMORY_HINT_PGI',
'GL_RECLAIM_MEMORY_HINT_PGI', 'GL_NATIVE_GRAPHICS_HANDLE_PGI',
'GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI', 'GL_NATIVE_GRAPHICS_END_HINT_PGI',
'GL_ALWAYS_FAST_HINT_PGI', 'GL_ALWAYS_SOFT_HINT_PGI',
'GL_ALLOW_DRAW_OBJ_HINT_PGI', 'GL_ALLOW_DRAW_WIN_HINT_PGI',
'GL_ALLOW_DRAW_FRG_HINT_PGI', 'GL_ALLOW_DRAW_MEM_HINT_PGI',
'GL_STRICT_DEPTHFUNC_HINT_PGI', 'GL_STRICT_LIGHTING_HINT_PGI',
'GL_STRICT_SCISSOR_HINT_PGI', 'GL_FULL_STIPPLE_HINT_PGI',
'GL_CLIP_NEAR_HINT_PGI', 'GL_CLIP_FAR_HINT_PGI', 'GL_WIDE_LINE_HINT_PGI',
'GL_BACK_NORMALS_HINT_PGI', 'GL_COLOR_INDEX1_EXT', 'GL_COLOR_INDEX2_EXT',
'GL_COLOR_INDEX4_EXT', 'GL_COLOR_INDEX8_EXT', 'GL_COLOR_INDEX12_EXT',
'GL_COLOR_INDEX16_EXT', 'GL_TEXTURE_INDEX_SIZE_EXT',
'GL_CLIP_VOLUME_CLIPPING_HINT_EXT', 'GL_LIST_PRIORITY_SGIX',
'GL_IR_INSTRUMENT1_SGIX', 'GL_CALLIGRAPHIC_FRAGMENT_SGIX',
'GL_TEXTURE_LOD_BIAS_S_SGIX', 'GL_TEXTURE_LOD_BIAS_T_SGIX',
'GL_TEXTURE_LOD_BIAS_R_SGIX', 'GL_SHADOW_AMBIENT_SGIX',
'GL_INDEX_MATERIAL_EXT', 'GL_INDEX_MATERIAL_PARAMETER_EXT',
'GL_INDEX_MATERIAL_FACE_EXT', 'GL_INDEX_TEST_EXT', 'GL_INDEX_TEST_FUNC_EXT',
'GL_INDEX_TEST_REF_EXT', 'GL_IUI_V2F_EXT', 'GL_IUI_V3F_EXT',
'GL_IUI_N3F_V2F_EXT', 'GL_IUI_N3F_V3F_EXT', 'GL_T2F_IUI_V2F_EXT',
'GL_T2F_IUI_V3F_EXT', 'GL_T2F_IUI_N3F_V2F_EXT', 'GL_T2F_IUI_N3F_V3F_EXT',
'GL_ARRAY_ELEMENT_LOCK_FIRST_EXT', 'GL_ARRAY_ELEMENT_LOCK_COUNT_EXT',
'GL_CULL_VERTEX_EXT', 'GL_CULL_VERTEX_EYE_POSITION_EXT',
'GL_CULL_VERTEX_OBJECT_POSITION_EXT', 'GL_YCRCB_422_SGIX',
'GL_YCRCB_444_SGIX', 'GL_FRAGMENT_LIGHTING_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_SGIX', 'GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX',
'GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX', 'GL_MAX_FRAGMENT_LIGHTS_SGIX',
'GL_MAX_ACTIVE_LIGHTS_SGIX', 'GL_CURRENT_RASTER_NORMAL_SGIX',
'GL_LIGHT_ENV_MODE_SGIX', 'GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX',
'GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX',
'GL_FRAGMENT_LIGHT0_SGIX', 'GL_FRAGMENT_LIGHT1_SGIX',
'GL_FRAGMENT_LIGHT2_SGIX', 'GL_FRAGMENT_LIGHT3_SGIX',
'GL_FRAGMENT_LIGHT4_SGIX', 'GL_FRAGMENT_LIGHT5_SGIX',
'GL_FRAGMENT_LIGHT6_SGIX', 'GL_FRAGMENT_LIGHT7_SGIX',
'GL_RASTER_POSITION_UNCLIPPED_IBM', 'GL_TEXTURE_LIGHTING_MODE_HP',
'GL_TEXTURE_POST_SPECULAR_HP', 'GL_TEXTURE_PRE_SPECULAR_HP',
'GL_MAX_ELEMENTS_VERTICES_EXT', 'GL_MAX_ELEMENTS_INDICES_EXT', 'GL_PHONG_WIN',
'GL_PHONG_HINT_WIN', 'GL_FOG_SPECULAR_TEXTURE_WIN',
'GL_FRAGMENT_MATERIAL_EXT', 'GL_FRAGMENT_NORMAL_EXT', 'GL_FRAGMENT_COLOR_EXT',
'GL_ATTENUATION_EXT', 'GL_SHADOW_ATTENUATION_EXT',
'GL_TEXTURE_APPLICATION_MODE_EXT', 'GL_TEXTURE_LIGHT_EXT',
'GL_TEXTURE_MATERIAL_FACE_EXT', 'GL_TEXTURE_MATERIAL_PARAMETER_EXT',
'GL_ALPHA_MIN_SGIX', 'GL_ALPHA_MAX_SGIX', 'GL_PIXEL_TEX_GEN_Q_CEILING_SGIX',
'GL_PIXEL_TEX_GEN_Q_ROUND_SGIX', 'GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX', 'GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX',
'GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX', 'GL_BGR_EXT', 'GL_BGRA_EXT',
'GL_ASYNC_MARKER_SGIX', 'GL_ASYNC_TEX_IMAGE_SGIX',
'GL_ASYNC_DRAW_PIXELS_SGIX', 'GL_ASYNC_READ_PIXELS_SGIX',
'GL_MAX_ASYNC_TEX_IMAGE_SGIX', 'GL_MAX_ASYNC_DRAW_PIXELS_SGIX',
'GL_MAX_ASYNC_READ_PIXELS_SGIX', 'GL_ASYNC_HISTOGRAM_SGIX',
'GL_MAX_ASYNC_HISTOGRAM_SGIX', 'GL_PARALLEL_ARRAYS_INTEL',
'GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL',
'GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL', 'GL_OCCLUSION_TEST_HP',
'GL_OCCLUSION_TEST_RESULT_HP', 'GL_PIXEL_TRANSFORM_2D_EXT',
'GL_PIXEL_MAG_FILTER_EXT', 'GL_PIXEL_MIN_FILTER_EXT',
'GL_PIXEL_CUBIC_WEIGHT_EXT', 'GL_CUBIC_EXT', 'GL_AVERAGE_EXT',
'GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',
'GL_PIXEL_TRANSFORM_2D_MATRIX_EXT', 'GL_SHARED_TEXTURE_PALETTE_EXT',
'GL_LIGHT_MODEL_COLOR_CONTROL_EXT', 'GL_SINGLE_COLOR_EXT',
'GL_SEPARATE_SPECULAR_COLOR_EXT', 'GL_COLOR_SUM_EXT',
'GL_CURRENT_SECONDARY_COLOR_EXT', 'GL_SECONDARY_COLOR_ARRAY_SIZE_EXT',
'GL_SECONDARY_COLOR_ARRAY_TYPE_EXT', 'GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT',
'GL_SECONDARY_COLOR_ARRAY_POINTER_EXT', 'GL_SECONDARY_COLOR_ARRAY_EXT',
'GL_PERTURB_EXT', 'GL_TEXTURE_NORMAL_EXT', 'GL_FOG_COORDINATE_SOURCE_EXT',
'GL_FOG_COORDINATE_EXT', 'GL_FRAGMENT_DEPTH_EXT',
'GL_CURRENT_FOG_COORDINATE_EXT', 'GL_FOG_COORDINATE_ARRAY_TYPE_EXT',
'GL_FOG_COORDINATE_ARRAY_STRIDE_EXT', 'GL_FOG_COORDINATE_ARRAY_POINTER_EXT',
'GL_FOG_COORDINATE_ARRAY_EXT', 'GL_SCREEN_COORDINATES_REND',
'GL_INVERTED_SCREEN_W_REND', 'GL_TANGENT_ARRAY_EXT', 'GL_BINORMAL_ARRAY_EXT',
'GL_CURRENT_TANGENT_EXT', 'GL_CURRENT_BINORMAL_EXT',
'GL_TANGENT_ARRAY_TYPE_EXT', 'GL_TANGENT_ARRAY_STRIDE_EXT',
'GL_BINORMAL_ARRAY_TYPE_EXT', 'GL_BINORMAL_ARRAY_STRIDE_EXT',
'GL_TANGENT_ARRAY_POINTER_EXT', 'GL_BINORMAL_ARRAY_POINTER_EXT',
'GL_MAP1_TANGENT_EXT', 'GL_MAP2_TANGENT_EXT', 'GL_MAP1_BINORMAL_EXT',
'GL_MAP2_BINORMAL_EXT', 'GL_COMBINE_EXT', 'GL_COMBINE_RGB_EXT',
'GL_COMBINE_ALPHA_EXT', 'GL_RGB_SCALE_EXT', 'GL_ADD_SIGNED_EXT',
'GL_INTERPOLATE_EXT', 'GL_CONSTANT_EXT', 'GL_PRIMARY_COLOR_EXT',
'GL_PREVIOUS_EXT', 'GL_SOURCE0_RGB_EXT', 'GL_SOURCE1_RGB_EXT',
'GL_SOURCE2_RGB_EXT', 'GL_SOURCE0_ALPHA_EXT', 'GL_SOURCE1_ALPHA_EXT',
'GL_SOURCE2_ALPHA_EXT', 'GL_OPERAND0_RGB_EXT', 'GL_OPERAND1_RGB_EXT',
'GL_OPERAND2_RGB_EXT', 'GL_OPERAND0_ALPHA_EXT', 'GL_OPERAND1_ALPHA_EXT',
'GL_OPERAND2_ALPHA_EXT', 'GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE',
'GL_TRANSFORM_HINT_APPLE', 'GL_FOG_SCALE_SGIX', 'GL_FOG_SCALE_VALUE_SGIX',
'GL_UNPACK_CONSTANT_DATA_SUNX', 'GL_TEXTURE_CONSTANT_DATA_SUNX',
'GL_GLOBAL_ALPHA_SUN', 'GL_GLOBAL_ALPHA_FACTOR_SUN', 'GL_RESTART_SUN',
'GL_REPLACE_MIDDLE_SUN', 'GL_REPLACE_OLDEST_SUN', 'GL_TRIANGLE_LIST_SUN',
'GL_REPLACEMENT_CODE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_SUN',
'GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN', 'GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN',
'GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN', 'GL_R1UI_V3F_SUN',
'GL_R1UI_C4UB_V3F_SUN', 'GL_R1UI_C3F_V3F_SUN', 'GL_R1UI_N3F_V3F_SUN',
'GL_R1UI_C4F_N3F_V3F_SUN', 'GL_R1UI_T2F_V3F_SUN', 'GL_R1UI_T2F_N3F_V3F_SUN',
'GL_R1UI_T2F_C4F_N3F_V3F_SUN', 'GL_BLEND_DST_RGB_EXT', 'GL_BLEND_SRC_RGB_EXT',
'GL_BLEND_DST_ALPHA_EXT', 'GL_BLEND_SRC_ALPHA_EXT', 'GL_RED_MIN_CLAMP_INGR',
'GL_GREEN_MIN_CLAMP_INGR', 'GL_BLUE_MIN_CLAMP_INGR',
'GL_ALPHA_MIN_CLAMP_INGR', 'GL_RED_MAX_CLAMP_INGR', 'GL_GREEN_MAX_CLAMP_INGR',
'GL_BLUE_MAX_CLAMP_INGR', 'GL_ALPHA_MAX_CLAMP_INGR', 'GL_INTERLACE_READ_INGR',
'GL_INCR_WRAP_EXT', 'GL_DECR_WRAP_EXT', 'GL_422_EXT', 'GL_422_REV_EXT',
'GL_422_AVERAGE_EXT', 'GL_422_REV_AVERAGE_EXT', 'GL_NORMAL_MAP_NV',
'GL_REFLECTION_MAP_NV', 'GL_NORMAL_MAP_EXT', 'GL_REFLECTION_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_EXT', 'GL_TEXTURE_BINDING_CUBE_MAP_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT',
'GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT', 'GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT',
'GL_PROXY_TEXTURE_CUBE_MAP_EXT', 'GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT',
'GL_WRAP_BORDER_SUN', 'GL_MAX_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_FILTER_CONTROL_EXT', 'GL_TEXTURE_LOD_BIAS_EXT',
'GL_TEXTURE_MAX_ANISOTROPY_EXT', 'GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT',
'GL_MODELVIEW0_STACK_DEPTH_EXT', 'GL_MODELVIEW1_STACK_DEPTH_EXT',
'GL_MODELVIEW0_MATRIX_EXT', 'GL_MODELVIEW1_MATRIX_EXT',
'GL_VERTEX_WEIGHTING_EXT', 'GL_MODELVIEW0_EXT', 'GL_MODELVIEW1_EXT',
'GL_CURRENT_VERTEX_WEIGHT_EXT', 'GL_VERTEX_WEIGHT_ARRAY_EXT',
'GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT',
'GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT', 'GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT',
'GL_MAX_SHININESS_NV', 'GL_MAX_SPOT_EXPONENT_NV', 'GL_VERTEX_ARRAY_RANGE_NV',
'GL_VERTEX_ARRAY_RANGE_LENGTH_NV', 'GL_VERTEX_ARRAY_RANGE_VALID_NV',
'GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV', 'GL_VERTEX_ARRAY_RANGE_POINTER_NV',
'GL_REGISTER_COMBINERS_NV', 'GL_VARIABLE_A_NV', 'GL_VARIABLE_B_NV',
'GL_VARIABLE_C_NV', 'GL_VARIABLE_D_NV', 'GL_VARIABLE_E_NV',
'GL_VARIABLE_F_NV', 'GL_VARIABLE_G_NV', 'GL_CONSTANT_COLOR0_NV',
'GL_CONSTANT_COLOR1_NV', 'GL_PRIMARY_COLOR_NV', 'GL_SECONDARY_COLOR_NV',
'GL_SPARE0_NV', 'GL_SPARE1_NV', 'GL_DISCARD_NV', 'GL_E_TIMES_F_NV',
'GL_SPARE0_PLUS_SECONDARY_COLOR_NV', 'GL_UNSIGNED_IDENTITY_NV',
'GL_UNSIGNED_INVERT_NV', 'GL_EXPAND_NORMAL_NV', 'GL_EXPAND_NEGATE_NV',
'GL_HALF_BIAS_NORMAL_NV', 'GL_HALF_BIAS_NEGATE_NV', 'GL_SIGNED_IDENTITY_NV',
'GL_SIGNED_NEGATE_NV', 'GL_SCALE_BY_TWO_NV', 'GL_SCALE_BY_FOUR_NV',
'GL_SCALE_BY_ONE_HALF_NV', 'GL_BIAS_BY_NEGATIVE_ONE_HALF_NV',
'GL_COMBINER_INPUT_NV', 'GL_COMBINER_MAPPING_NV',
'GL_COMBINER_COMPONENT_USAGE_NV', 'GL_COMBINER_AB_DOT_PRODUCT_NV',
'GL_COMBINER_CD_DOT_PRODUCT_NV', 'GL_COMBINER_MUX_SUM_NV',
'GL_COMBINER_SCALE_NV', 'GL_COMBINER_BIAS_NV', 'GL_COMBINER_AB_OUTPUT_NV',
'GL_COMBINER_CD_OUTPUT_NV', 'GL_COMBINER_SUM_OUTPUT_NV',
'GL_MAX_GENERAL_COMBINERS_NV', 'GL_NUM_GENERAL_COMBINERS_NV',
'GL_COLOR_SUM_CLAMP_NV', 'GL_COMBINER0_NV', 'GL_COMBINER1_NV',
'GL_COMBINER2_NV', 'GL_COMBINER3_NV', 'GL_COMBINER4_NV', 'GL_COMBINER5_NV',
'GL_COMBINER6_NV', 'GL_COMBINER7_NV', 'GL_FOG_DISTANCE_MODE_NV',
'GL_EYE_RADIAL_NV', 'GL_EYE_PLANE_ABSOLUTE_NV', 'GL_EMBOSS_LIGHT_NV',
'GL_EMBOSS_CONSTANT_NV', 'GL_EMBOSS_MAP_NV', 'GL_COMBINE4_NV',
'GL_SOURCE3_RGB_NV', 'GL_SOURCE3_ALPHA_NV', 'GL_OPERAND3_RGB_NV',
'GL_OPERAND3_ALPHA_NV', 'GL_COMPRESSED_RGB_S3TC_DXT1_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT1_EXT', 'GL_COMPRESSED_RGBA_S3TC_DXT3_EXT',
'GL_COMPRESSED_RGBA_S3TC_DXT5_EXT', 'GL_CULL_VERTEX_IBM',
'GL_VERTEX_ARRAY_LIST_IBM', 'GL_NORMAL_ARRAY_LIST_IBM',
'GL_COLOR_ARRAY_LIST_IBM', 'GL_INDEX_ARRAY_LIST_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_IBM', 'GL_EDGE_FLAG_ARRAY_LIST_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_IBM', 'GL_SECONDARY_COLOR_ARRAY_LIST_IBM',
'GL_VERTEX_ARRAY_LIST_STRIDE_IBM', 'GL_NORMAL_ARRAY_LIST_STRIDE_IBM',
'GL_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_INDEX_ARRAY_LIST_STRIDE_IBM',
'GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM',
'GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM',
'GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM',
'GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM', 'GL_PACK_SUBSAMPLE_RATE_SGIX',
'GL_UNPACK_SUBSAMPLE_RATE_SGIX', 'GL_PIXEL_SUBSAMPLE_4444_SGIX',
'GL_PIXEL_SUBSAMPLE_2424_SGIX', 'GL_PIXEL_SUBSAMPLE_4242_SGIX',
'GL_YCRCB_SGIX', 'GL_YCRCBA_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_SGIX',
'GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX', 'GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX',
'GL_COMPRESSED_RGB_FXT1_3DFX', 'GL_COMPRESSED_RGBA_FXT1_3DFX',
'GL_MULTISAMPLE_3DFX', 'GL_SAMPLE_BUFFERS_3DFX', 'GL_SAMPLES_3DFX',
'GL_MULTISAMPLE_BIT_3DFX', 'GL_MULTISAMPLE_EXT',
'GL_SAMPLE_ALPHA_TO_MASK_EXT', 'GL_SAMPLE_ALPHA_TO_ONE_EXT',
'GL_SAMPLE_MASK_EXT', 'GL_1PASS_EXT', 'GL_2PASS_0_EXT', 'GL_2PASS_1_EXT',
'GL_4PASS_0_EXT', 'GL_4PASS_1_EXT', 'GL_4PASS_2_EXT', 'GL_4PASS_3_EXT',
'GL_SAMPLE_BUFFERS_EXT', 'GL_SAMPLES_EXT', 'GL_SAMPLE_MASK_VALUE_EXT',
'GL_SAMPLE_MASK_INVERT_EXT', 'GL_SAMPLE_PATTERN_EXT',
'GL_MULTISAMPLE_BIT_EXT', 'GL_VERTEX_PRECLIP_SGIX',
'GL_VERTEX_PRECLIP_HINT_SGIX', 'GL_CONVOLUTION_HINT_SGIX',
'GL_PACK_RESAMPLE_SGIX', 'GL_UNPACK_RESAMPLE_SGIX',
'GL_RESAMPLE_REPLICATE_SGIX', 'GL_RESAMPLE_ZERO_FILL_SGIX',
'GL_RESAMPLE_DECIMATE_SGIX', 'GL_EYE_DISTANCE_TO_POINT_SGIS',
'GL_OBJECT_DISTANCE_TO_POINT_SGIS', 'GL_EYE_DISTANCE_TO_LINE_SGIS',
'GL_OBJECT_DISTANCE_TO_LINE_SGIS', 'GL_EYE_POINT_SGIS',
'GL_OBJECT_POINT_SGIS', 'GL_EYE_LINE_SGIS', 'GL_OBJECT_LINE_SGIS',
'GL_TEXTURE_COLOR_WRITEMASK_SGIS', 'GL_DOT3_RGB_EXT', 'GL_DOT3_RGBA_EXT',
'GL_MIRROR_CLAMP_ATI', 'GL_MIRROR_CLAMP_TO_EDGE_ATI', 'GL_ALL_COMPLETED_NV',
'GL_FENCE_STATUS_NV', 'GL_FENCE_CONDITION_NV', 'GL_MIRRORED_REPEAT_IBM',
'GL_EVAL_2D_NV', 'GL_EVAL_TRIANGULAR_2D_NV', 'GL_MAP_TESSELLATION_NV',
'GL_MAP_ATTRIB_U_ORDER_NV', 'GL_MAP_ATTRIB_V_ORDER_NV',
'GL_EVAL_FRACTIONAL_TESSELLATION_NV', 'GL_EVAL_VERTEX_ATTRIB0_NV',
'GL_EVAL_VERTEX_ATTRIB1_NV', 'GL_EVAL_VERTEX_ATTRIB2_NV',
'GL_EVAL_VERTEX_ATTRIB3_NV', 'GL_EVAL_VERTEX_ATTRIB4_NV',
'GL_EVAL_VERTEX_ATTRIB5_NV', 'GL_EVAL_VERTEX_ATTRIB6_NV',
'GL_EVAL_VERTEX_ATTRIB7_NV', 'GL_EVAL_VERTEX_ATTRIB8_NV',
'GL_EVAL_VERTEX_ATTRIB9_NV', 'GL_EVAL_VERTEX_ATTRIB10_NV',
'GL_EVAL_VERTEX_ATTRIB11_NV', 'GL_EVAL_VERTEX_ATTRIB12_NV',
'GL_EVAL_VERTEX_ATTRIB13_NV', 'GL_EVAL_VERTEX_ATTRIB14_NV',
'GL_EVAL_VERTEX_ATTRIB15_NV', 'GL_MAX_MAP_TESSELLATION_NV',
'GL_MAX_RATIONAL_EVAL_ORDER_NV', 'GL_DEPTH_STENCIL_NV',
'GL_UNSIGNED_INT_24_8_NV', 'GL_PER_STAGE_CONSTANTS_NV',
'GL_TEXTURE_RECTANGLE_NV', 'GL_TEXTURE_BINDING_RECTANGLE_NV',
'GL_PROXY_TEXTURE_RECTANGLE_NV', 'GL_MAX_RECTANGLE_TEXTURE_SIZE_NV',
'GL_OFFSET_TEXTURE_RECTANGLE_NV', 'GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV',
'GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV',
'GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV', 'GL_UNSIGNED_INT_S8_S8_8_8_NV',
'GL_UNSIGNED_INT_8_8_S8_S8_REV_NV', 'GL_DSDT_MAG_INTENSITY_NV',
'GL_SHADER_CONSISTENT_NV', 'GL_TEXTURE_SHADER_NV', 'GL_SHADER_OPERATION_NV',
'GL_CULL_MODES_NV', 'GL_OFFSET_TEXTURE_MATRIX_NV',
'GL_OFFSET_TEXTURE_SCALE_NV', 'GL_OFFSET_TEXTURE_BIAS_NV',
'GL_OFFSET_TEXTURE_2D_MATRIX_NV', 'GL_OFFSET_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_TEXTURE_2D_BIAS_NV', 'GL_PREVIOUS_TEXTURE_INPUT_NV',
'GL_CONST_EYE_NV', 'GL_PASS_THROUGH_NV', 'GL_CULL_FRAGMENT_NV',
'GL_OFFSET_TEXTURE_2D_NV', 'GL_DEPENDENT_AR_TEXTURE_2D_NV',
'GL_DEPENDENT_GB_TEXTURE_2D_NV', 'GL_DOT_PRODUCT_NV',
'GL_DOT_PRODUCT_DEPTH_REPLACE_NV', 'GL_DOT_PRODUCT_TEXTURE_2D_NV',
'GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV',
'GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV',
'GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV', 'GL_HILO_NV', 'GL_DSDT_NV',
'GL_DSDT_MAG_NV', 'GL_DSDT_MAG_VIB_NV', 'GL_HILO16_NV', 'GL_SIGNED_HILO_NV',
'GL_SIGNED_HILO16_NV', 'GL_SIGNED_RGBA_NV', 'GL_SIGNED_RGBA8_NV',
'GL_SIGNED_RGB_NV', 'GL_SIGNED_RGB8_NV', 'GL_SIGNED_LUMINANCE_NV',
'GL_SIGNED_LUMINANCE8_NV', 'GL_SIGNED_LUMINANCE_ALPHA_NV',
'GL_SIGNED_LUMINANCE8_ALPHA8_NV', 'GL_SIGNED_ALPHA_NV', 'GL_SIGNED_ALPHA8_NV',
'GL_SIGNED_INTENSITY_NV', 'GL_SIGNED_INTENSITY8_NV', 'GL_DSDT8_NV',
'GL_DSDT8_MAG8_NV', 'GL_DSDT8_MAG8_INTENSITY8_NV',
'GL_SIGNED_RGB_UNSIGNED_ALPHA_NV', 'GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV',
'GL_HI_SCALE_NV', 'GL_LO_SCALE_NV', 'GL_DS_SCALE_NV', 'GL_DT_SCALE_NV',
'GL_MAGNITUDE_SCALE_NV', 'GL_VIBRANCE_SCALE_NV', 'GL_HI_BIAS_NV',
'GL_LO_BIAS_NV', 'GL_DS_BIAS_NV', 'GL_DT_BIAS_NV', 'GL_MAGNITUDE_BIAS_NV',
'GL_VIBRANCE_BIAS_NV', 'GL_TEXTURE_BORDER_VALUES_NV', 'GL_TEXTURE_HI_SIZE_NV',
'GL_TEXTURE_LO_SIZE_NV', 'GL_TEXTURE_DS_SIZE_NV', 'GL_TEXTURE_DT_SIZE_NV',
'GL_TEXTURE_MAG_SIZE_NV', 'GL_DOT_PRODUCT_TEXTURE_3D_NV',
'GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV', 'GL_VERTEX_PROGRAM_NV',
'GL_VERTEX_STATE_PROGRAM_NV', 'GL_ATTRIB_ARRAY_SIZE_NV',
'GL_ATTRIB_ARRAY_STRIDE_NV', 'GL_ATTRIB_ARRAY_TYPE_NV',
'GL_CURRENT_ATTRIB_NV', 'GL_PROGRAM_LENGTH_NV', 'GL_PROGRAM_STRING_NV',
'GL_MODELVIEW_PROJECTION_NV', 'GL_IDENTITY_NV', 'GL_INVERSE_NV',
'GL_TRANSPOSE_NV', 'GL_INVERSE_TRANSPOSE_NV',
'GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV', 'GL_MAX_TRACK_MATRICES_NV',
'GL_MATRIX0_NV', 'GL_MATRIX1_NV', 'GL_MATRIX2_NV', 'GL_MATRIX3_NV',
'GL_MATRIX4_NV', 'GL_MATRIX5_NV', 'GL_MATRIX6_NV', 'GL_MATRIX7_NV',
'GL_CURRENT_MATRIX_STACK_DEPTH_NV', 'GL_CURRENT_MATRIX_NV',
'GL_VERTEX_PROGRAM_POINT_SIZE_NV', 'GL_VERTEX_PROGRAM_TWO_SIDE_NV',
'GL_PROGRAM_PARAMETER_NV', 'GL_ATTRIB_ARRAY_POINTER_NV',
'GL_PROGRAM_TARGET_NV', 'GL_PROGRAM_RESIDENT_NV', 'GL_TRACK_MATRIX_NV',
'GL_TRACK_MATRIX_TRANSFORM_NV', 'GL_VERTEX_PROGRAM_BINDING_NV',
'GL_PROGRAM_ERROR_POSITION_NV', 'GL_VERTEX_ATTRIB_ARRAY0_NV',
'GL_VERTEX_ATTRIB_ARRAY1_NV', 'GL_VERTEX_ATTRIB_ARRAY2_NV',
'GL_VERTEX_ATTRIB_ARRAY3_NV', 'GL_VERTEX_ATTRIB_ARRAY4_NV',
'GL_VERTEX_ATTRIB_ARRAY5_NV', 'GL_VERTEX_ATTRIB_ARRAY6_NV',
'GL_VERTEX_ATTRIB_ARRAY7_NV', 'GL_VERTEX_ATTRIB_ARRAY8_NV',
'GL_VERTEX_ATTRIB_ARRAY9_NV', 'GL_VERTEX_ATTRIB_ARRAY10_NV',
'GL_VERTEX_ATTRIB_ARRAY11_NV', 'GL_VERTEX_ATTRIB_ARRAY12_NV',
'GL_VERTEX_ATTRIB_ARRAY13_NV', 'GL_VERTEX_ATTRIB_ARRAY14_NV',
'GL_VERTEX_ATTRIB_ARRAY15_NV', 'GL_MAP1_VERTEX_ATTRIB0_4_NV',
'GL_MAP1_VERTEX_ATTRIB1_4_NV', 'GL_MAP1_VERTEX_ATTRIB2_4_NV',
'GL_MAP1_VERTEX_ATTRIB3_4_NV', 'GL_MAP1_VERTEX_ATTRIB4_4_NV',
'GL_MAP1_VERTEX_ATTRIB5_4_NV', 'GL_MAP1_VERTEX_ATTRIB6_4_NV',
'GL_MAP1_VERTEX_ATTRIB7_4_NV', 'GL_MAP1_VERTEX_ATTRIB8_4_NV',
'GL_MAP1_VERTEX_ATTRIB9_4_NV', 'GL_MAP1_VERTEX_ATTRIB10_4_NV',
'GL_MAP1_VERTEX_ATTRIB11_4_NV', 'GL_MAP1_VERTEX_ATTRIB12_4_NV',
'GL_MAP1_VERTEX_ATTRIB13_4_NV', 'GL_MAP1_VERTEX_ATTRIB14_4_NV',
'GL_MAP1_VERTEX_ATTRIB15_4_NV', 'GL_MAP2_VERTEX_ATTRIB0_4_NV',
'GL_MAP2_VERTEX_ATTRIB1_4_NV', 'GL_MAP2_VERTEX_ATTRIB2_4_NV',
'GL_MAP2_VERTEX_ATTRIB3_4_NV', 'GL_MAP2_VERTEX_ATTRIB4_4_NV',
'GL_MAP2_VERTEX_ATTRIB5_4_NV', 'GL_MAP2_VERTEX_ATTRIB6_4_NV',
'GL_MAP2_VERTEX_ATTRIB7_4_NV', 'GL_MAP2_VERTEX_ATTRIB8_4_NV',
'GL_MAP2_VERTEX_ATTRIB9_4_NV', 'GL_MAP2_VERTEX_ATTRIB10_4_NV',
'GL_MAP2_VERTEX_ATTRIB11_4_NV', 'GL_MAP2_VERTEX_ATTRIB12_4_NV',
'GL_MAP2_VERTEX_ATTRIB13_4_NV', 'GL_MAP2_VERTEX_ATTRIB14_4_NV',
'GL_MAP2_VERTEX_ATTRIB15_4_NV', 'GL_TEXTURE_MAX_CLAMP_S_SGIX',
'GL_TEXTURE_MAX_CLAMP_T_SGIX', 'GL_TEXTURE_MAX_CLAMP_R_SGIX',
'GL_SCALEBIAS_HINT_SGIX', 'GL_INTERLACE_OML', 'GL_INTERLACE_READ_OML',
'GL_FORMAT_SUBSAMPLE_24_24_OML', 'GL_FORMAT_SUBSAMPLE_244_244_OML',
'GL_PACK_RESAMPLE_OML', 'GL_UNPACK_RESAMPLE_OML', 'GL_RESAMPLE_REPLICATE_OML',
'GL_RESAMPLE_ZERO_FILL_OML', 'GL_RESAMPLE_AVERAGE_OML',
'GL_RESAMPLE_DECIMATE_OML', 'GL_DEPTH_STENCIL_TO_RGBA_NV',
'GL_DEPTH_STENCIL_TO_BGRA_NV', 'GL_BUMP_ROT_MATRIX_ATI',
'GL_BUMP_ROT_MATRIX_SIZE_ATI', 'GL_BUMP_NUM_TEX_UNITS_ATI',
'GL_BUMP_TEX_UNITS_ATI', 'GL_DUDV_ATI', 'GL_DU8DV8_ATI', 'GL_BUMP_ENVMAP_ATI',
'GL_BUMP_TARGET_ATI', 'GL_FRAGMENT_SHADER_ATI', 'GL_REG_0_ATI',
'GL_REG_1_ATI', 'GL_REG_2_ATI', 'GL_REG_3_ATI', 'GL_REG_4_ATI',
'GL_REG_5_ATI', 'GL_REG_6_ATI', 'GL_REG_7_ATI', 'GL_REG_8_ATI',
'GL_REG_9_ATI', 'GL_REG_10_ATI', 'GL_REG_11_ATI', 'GL_REG_12_ATI',
'GL_REG_13_ATI', 'GL_REG_14_ATI', 'GL_REG_15_ATI', 'GL_REG_16_ATI',
'GL_REG_17_ATI', 'GL_REG_18_ATI', 'GL_REG_19_ATI', 'GL_REG_20_ATI',
'GL_REG_21_ATI', 'GL_REG_22_ATI', 'GL_REG_23_ATI', 'GL_REG_24_ATI',
'GL_REG_25_ATI', 'GL_REG_26_ATI', 'GL_REG_27_ATI', 'GL_REG_28_ATI',
'GL_REG_29_ATI', 'GL_REG_30_ATI', 'GL_REG_31_ATI', 'GL_CON_0_ATI',
'GL_CON_1_ATI', 'GL_CON_2_ATI', 'GL_CON_3_ATI', 'GL_CON_4_ATI',
'GL_CON_5_ATI', 'GL_CON_6_ATI', 'GL_CON_7_ATI', 'GL_CON_8_ATI',
'GL_CON_9_ATI', 'GL_CON_10_ATI', 'GL_CON_11_ATI', 'GL_CON_12_ATI',
'GL_CON_13_ATI', 'GL_CON_14_ATI', 'GL_CON_15_ATI', 'GL_CON_16_ATI',
'GL_CON_17_ATI', 'GL_CON_18_ATI', 'GL_CON_19_ATI', 'GL_CON_20_ATI',
'GL_CON_21_ATI', 'GL_CON_22_ATI', 'GL_CON_23_ATI', 'GL_CON_24_ATI',
'GL_CON_25_ATI', 'GL_CON_26_ATI', 'GL_CON_27_ATI', 'GL_CON_28_ATI',
'GL_CON_29_ATI', 'GL_CON_30_ATI', 'GL_CON_31_ATI', 'GL_MOV_ATI', 'GL_ADD_ATI',
'GL_MUL_ATI', 'GL_SUB_ATI', 'GL_DOT3_ATI', 'GL_DOT4_ATI', 'GL_MAD_ATI',
'GL_LERP_ATI', 'GL_CND_ATI', 'GL_CND0_ATI', 'GL_DOT2_ADD_ATI',
'GL_SECONDARY_INTERPOLATOR_ATI', 'GL_NUM_FRAGMENT_REGISTERS_ATI',
'GL_NUM_FRAGMENT_CONSTANTS_ATI', 'GL_NUM_PASSES_ATI',
'GL_NUM_INSTRUCTIONS_PER_PASS_ATI', 'GL_NUM_INSTRUCTIONS_TOTAL_ATI',
'GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI', 'GL_NUM_LOOPBACK_COMPONENTS_ATI',
'GL_COLOR_ALPHA_PAIRING_ATI', 'GL_SWIZZLE_STR_ATI', 'GL_SWIZZLE_STQ_ATI',
'GL_SWIZZLE_STR_DR_ATI', 'GL_SWIZZLE_STQ_DQ_ATI', 'GL_SWIZZLE_STRQ_ATI',
'GL_SWIZZLE_STRQ_DQ_ATI', 'GL_RED_BIT_ATI', 'GL_GREEN_BIT_ATI',
'GL_BLUE_BIT_ATI', 'GL_2X_BIT_ATI', 'GL_4X_BIT_ATI', 'GL_8X_BIT_ATI',
'GL_HALF_BIT_ATI', 'GL_QUARTER_BIT_ATI', 'GL_EIGHTH_BIT_ATI',
'GL_SATURATE_BIT_ATI', 'GL_COMP_BIT_ATI', 'GL_NEGATE_BIT_ATI',
'GL_BIAS_BIT_ATI', 'GL_PN_TRIANGLES_ATI',
'GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI', 'GL_PN_TRIANGLES_POINT_MODE_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_ATI', 'GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI',
'GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI',
'GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI', 'GL_STATIC_ATI',
'GL_DYNAMIC_ATI', 'GL_PRESERVE_ATI', 'GL_DISCARD_ATI',
'GL_OBJECT_BUFFER_SIZE_ATI', 'GL_OBJECT_BUFFER_USAGE_ATI',
'GL_ARRAY_OBJECT_BUFFER_ATI', 'GL_ARRAY_OBJECT_OFFSET_ATI',
'GL_VERTEX_SHADER_EXT', 'GL_VERTEX_SHADER_BINDING_EXT', 'GL_OP_INDEX_EXT',
'GL_OP_NEGATE_EXT', 'GL_OP_DOT3_EXT', 'GL_OP_DOT4_EXT', 'GL_OP_MUL_EXT',
'GL_OP_ADD_EXT', 'GL_OP_MADD_EXT', 'GL_OP_FRAC_EXT', 'GL_OP_MAX_EXT',
'GL_OP_MIN_EXT', 'GL_OP_SET_GE_EXT', 'GL_OP_SET_LT_EXT', 'GL_OP_CLAMP_EXT',
'GL_OP_FLOOR_EXT', 'GL_OP_ROUND_EXT', 'GL_OP_EXP_BASE_2_EXT',
'GL_OP_LOG_BASE_2_EXT', 'GL_OP_POWER_EXT', 'GL_OP_RECIP_EXT',
'GL_OP_RECIP_SQRT_EXT', 'GL_OP_SUB_EXT', 'GL_OP_CROSS_PRODUCT_EXT',
'GL_OP_MULTIPLY_MATRIX_EXT', 'GL_OP_MOV_EXT', 'GL_OUTPUT_VERTEX_EXT',
'GL_OUTPUT_COLOR0_EXT', 'GL_OUTPUT_COLOR1_EXT',
'GL_OUTPUT_TEXTURE_COORD0_EXT', 'GL_OUTPUT_TEXTURE_COORD1_EXT',
'GL_OUTPUT_TEXTURE_COORD2_EXT', 'GL_OUTPUT_TEXTURE_COORD3_EXT',
'GL_OUTPUT_TEXTURE_COORD4_EXT', 'GL_OUTPUT_TEXTURE_COORD5_EXT',
'GL_OUTPUT_TEXTURE_COORD6_EXT', 'GL_OUTPUT_TEXTURE_COORD7_EXT',
'GL_OUTPUT_TEXTURE_COORD8_EXT', 'GL_OUTPUT_TEXTURE_COORD9_EXT',
'GL_OUTPUT_TEXTURE_COORD10_EXT', 'GL_OUTPUT_TEXTURE_COORD11_EXT',
'GL_OUTPUT_TEXTURE_COORD12_EXT', 'GL_OUTPUT_TEXTURE_COORD13_EXT',
'GL_OUTPUT_TEXTURE_COORD14_EXT', 'GL_OUTPUT_TEXTURE_COORD15_EXT',
'GL_OUTPUT_TEXTURE_COORD16_EXT', 'GL_OUTPUT_TEXTURE_COORD17_EXT',
'GL_OUTPUT_TEXTURE_COORD18_EXT', 'GL_OUTPUT_TEXTURE_COORD19_EXT',
'GL_OUTPUT_TEXTURE_COORD20_EXT', 'GL_OUTPUT_TEXTURE_COORD21_EXT',
'GL_OUTPUT_TEXTURE_COORD22_EXT', 'GL_OUTPUT_TEXTURE_COORD23_EXT',
'GL_OUTPUT_TEXTURE_COORD24_EXT', 'GL_OUTPUT_TEXTURE_COORD25_EXT',
'GL_OUTPUT_TEXTURE_COORD26_EXT', 'GL_OUTPUT_TEXTURE_COORD27_EXT',
'GL_OUTPUT_TEXTURE_COORD28_EXT', 'GL_OUTPUT_TEXTURE_COORD29_EXT',
'GL_OUTPUT_TEXTURE_COORD30_EXT', 'GL_OUTPUT_TEXTURE_COORD31_EXT',
'GL_OUTPUT_FOG_EXT', 'GL_SCALAR_EXT', 'GL_VECTOR_EXT', 'GL_MATRIX_EXT',
'GL_VARIANT_EXT', 'GL_INVARIANT_EXT', 'GL_LOCAL_CONSTANT_EXT', 'GL_LOCAL_EXT',
'GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_MAX_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 'GL_MAX_VERTEX_SHADER_LOCALS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT',
'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT',
'GL_VERTEX_SHADER_INSTRUCTIONS_EXT', 'GL_VERTEX_SHADER_VARIANTS_EXT',
'GL_VERTEX_SHADER_INVARIANTS_EXT', 'GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT',
'GL_VERTEX_SHADER_LOCALS_EXT', 'GL_VERTEX_SHADER_OPTIMIZED_EXT', 'GL_X_EXT',
'GL_Y_EXT', 'GL_Z_EXT', 'GL_W_EXT', 'GL_NEGATIVE_X_EXT', 'GL_NEGATIVE_Y_EXT',
'GL_NEGATIVE_Z_EXT', 'GL_NEGATIVE_W_EXT', 'GL_ZERO_EXT', 'GL_ONE_EXT',
'GL_NEGATIVE_ONE_EXT', 'GL_NORMALIZED_RANGE_EXT', 'GL_FULL_RANGE_EXT',
'GL_CURRENT_VERTEX_EXT', 'GL_MVP_MATRIX_EXT', 'GL_VARIANT_VALUE_EXT',
'GL_VARIANT_DATATYPE_EXT', 'GL_VARIANT_ARRAY_STRIDE_EXT',
'GL_VARIANT_ARRAY_TYPE_EXT', 'GL_VARIANT_ARRAY_EXT',
'GL_VARIANT_ARRAY_POINTER_EXT', 'GL_INVARIANT_VALUE_EXT',
'GL_INVARIANT_DATATYPE_EXT', 'GL_LOCAL_CONSTANT_VALUE_EXT',
'GL_LOCAL_CONSTANT_DATATYPE_EXT', 'GL_MAX_VERTEX_STREAMS_ATI',
'GL_VERTEX_STREAM0_ATI', 'GL_VERTEX_STREAM1_ATI', 'GL_VERTEX_STREAM2_ATI',
'GL_VERTEX_STREAM3_ATI', 'GL_VERTEX_STREAM4_ATI', 'GL_VERTEX_STREAM5_ATI',
'GL_VERTEX_STREAM6_ATI', 'GL_VERTEX_STREAM7_ATI', 'GL_VERTEX_SOURCE_ATI',
'GL_ELEMENT_ARRAY_ATI', 'GL_ELEMENT_ARRAY_TYPE_ATI',
'GL_ELEMENT_ARRAY_POINTER_ATI', 'GL_QUAD_MESH_SUN', 'GL_TRIANGLE_MESH_SUN',
'GL_SLICE_ACCUM_SUN', 'GL_MULTISAMPLE_FILTER_HINT_NV', 'GL_DEPTH_CLAMP_NV',
'GL_PIXEL_COUNTER_BITS_NV', 'GL_CURRENT_OCCLUSION_QUERY_ID_NV',
'GL_PIXEL_COUNT_NV', 'GL_PIXEL_COUNT_AVAILABLE_NV', 'GL_POINT_SPRITE_NV',
'GL_COORD_REPLACE_NV', 'GL_POINT_SPRITE_R_MODE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV',
'GL_OFFSET_HILO_TEXTURE_2D_NV', 'GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV',
'GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV',
'GL_DEPENDENT_HILO_TEXTURE_2D_NV', 'GL_DEPENDENT_RGB_TEXTURE_3D_NV',
'GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV', 'GL_DOT_PRODUCT_PASS_THROUGH_NV',
'GL_DOT_PRODUCT_TEXTURE_1D_NV', 'GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV',
'GL_HILO8_NV', 'GL_SIGNED_HILO8_NV', 'GL_FORCE_BLUE_TO_ONE_NV',
'GL_STENCIL_TEST_TWO_SIDE_EXT', 'GL_ACTIVE_STENCIL_FACE_EXT',
'GL_TEXT_FRAGMENT_SHADER_ATI', 'GL_UNPACK_CLIENT_STORAGE_APPLE',
'GL_ELEMENT_ARRAY_APPLE', 'GL_ELEMENT_ARRAY_TYPE_APPLE',
'GL_ELEMENT_ARRAY_POINTER_APPLE', 'GL_DRAW_PIXELS_APPLE', 'GL_FENCE_APPLE',
'GL_VERTEX_ARRAY_BINDING_APPLE', 'GL_VERTEX_ARRAY_RANGE_APPLE',
'GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE', 'GL_VERTEX_ARRAY_STORAGE_HINT_APPLE',
'GL_VERTEX_ARRAY_RANGE_POINTER_APPLE', 'GL_STORAGE_CLIENT_APPLE',
'GL_STORAGE_CACHED_APPLE', 'GL_STORAGE_SHARED_APPLE', 'GL_YCBCR_422_APPLE',
'GL_UNSIGNED_SHORT_8_8_APPLE', 'GL_UNSIGNED_SHORT_8_8_REV_APPLE',
'GL_RGB_S3TC', 'GL_RGB4_S3TC', 'GL_RGBA_S3TC', 'GL_RGBA4_S3TC',
'GL_MAX_DRAW_BUFFERS_ATI', 'GL_DRAW_BUFFER0_ATI', 'GL_DRAW_BUFFER1_ATI',
'GL_DRAW_BUFFER2_ATI', 'GL_DRAW_BUFFER3_ATI', 'GL_DRAW_BUFFER4_ATI',
'GL_DRAW_BUFFER5_ATI', 'GL_DRAW_BUFFER6_ATI', 'GL_DRAW_BUFFER7_ATI',
'GL_DRAW_BUFFER8_ATI', 'GL_DRAW_BUFFER9_ATI', 'GL_DRAW_BUFFER10_ATI',
'GL_DRAW_BUFFER11_ATI', 'GL_DRAW_BUFFER12_ATI', 'GL_DRAW_BUFFER13_ATI',
'GL_DRAW_BUFFER14_ATI', 'GL_DRAW_BUFFER15_ATI', 'GL_TYPE_RGBA_FLOAT_ATI',
'GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI', 'GL_MODULATE_ADD_ATI',
'GL_MODULATE_SIGNED_ADD_ATI', 'GL_MODULATE_SUBTRACT_ATI',
'GL_RGBA_FLOAT32_ATI', 'GL_RGB_FLOAT32_ATI', 'GL_ALPHA_FLOAT32_ATI',
'GL_INTENSITY_FLOAT32_ATI', 'GL_LUMINANCE_FLOAT32_ATI',
'GL_LUMINANCE_ALPHA_FLOAT32_ATI', 'GL_RGBA_FLOAT16_ATI', 'GL_RGB_FLOAT16_ATI',
'GL_ALPHA_FLOAT16_ATI', 'GL_INTENSITY_FLOAT16_ATI',
'GL_LUMINANCE_FLOAT16_ATI', 'GL_LUMINANCE_ALPHA_FLOAT16_ATI', 'GL_FLOAT_R_NV',
'GL_FLOAT_RG_NV', 'GL_FLOAT_RGB_NV', 'GL_FLOAT_RGBA_NV', 'GL_FLOAT_R16_NV',
'GL_FLOAT_R32_NV', 'GL_FLOAT_RG16_NV', 'GL_FLOAT_RG32_NV',
'GL_FLOAT_RGB16_NV', 'GL_FLOAT_RGB32_NV', 'GL_FLOAT_RGBA16_NV',
'GL_FLOAT_RGBA32_NV', 'GL_TEXTURE_FLOAT_COMPONENTS_NV',
'GL_FLOAT_CLEAR_COLOR_VALUE_NV', 'GL_FLOAT_RGBA_MODE_NV',
'GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV', 'GL_FRAGMENT_PROGRAM_NV',
'GL_MAX_TEXTURE_COORDS_NV', 'GL_MAX_TEXTURE_IMAGE_UNITS_NV',
'GL_FRAGMENT_PROGRAM_BINDING_NV', 'GL_PROGRAM_ERROR_STRING_NV',
'GL_HALF_FLOAT_NV', 'GL_WRITE_PIXEL_DATA_RANGE_NV',
'GL_READ_PIXEL_DATA_RANGE_NV', 'GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV',
'GL_READ_PIXEL_DATA_RANGE_LENGTH_NV', 'GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV',
'GL_READ_PIXEL_DATA_RANGE_POINTER_NV', 'GL_PRIMITIVE_RESTART_NV',
'GL_PRIMITIVE_RESTART_INDEX_NV', 'GL_TEXTURE_UNSIGNED_REMAP_MODE_NV',
'GL_STENCIL_BACK_FUNC_ATI', 'GL_STENCIL_BACK_FAIL_ATI',
'GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI', 'GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI',
'GL_IMPLEMENTATION_COLOR_READ_TYPE_OES',
'GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES', 'GL_DEPTH_BOUNDS_TEST_EXT',
'GL_DEPTH_BOUNDS_EXT', 'GL_MIRROR_CLAMP_EXT', 'GL_MIRROR_CLAMP_TO_EDGE_EXT',
'GL_MIRROR_CLAMP_TO_BORDER_EXT', 'GL_BLEND_EQUATION_RGB_EXT',
'GL_BLEND_EQUATION_ALPHA_EXT', 'GL_PACK_INVERT_MESA',
'GL_UNSIGNED_SHORT_8_8_MESA', 'GL_UNSIGNED_SHORT_8_8_REV_MESA',
'GL_YCBCR_MESA', 'GL_PIXEL_PACK_BUFFER_EXT', 'GL_PIXEL_UNPACK_BUFFER_EXT',
'GL_PIXEL_PACK_BUFFER_BINDING_EXT', 'GL_PIXEL_UNPACK_BUFFER_BINDING_EXT',
'GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV', 'GL_MAX_PROGRAM_CALL_DEPTH_NV',
'GL_MAX_PROGRAM_IF_DEPTH_NV', 'GL_MAX_PROGRAM_LOOP_DEPTH_NV',
'GL_MAX_PROGRAM_LOOP_COUNT_NV', 'GL_INVALID_FRAMEBUFFER_OPERATION_EXT',
'GL_MAX_RENDERBUFFER_SIZE_EXT', 'GL_FRAMEBUFFER_BINDING_EXT',
'GL_RENDERBUFFER_BINDING_EXT', 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT',
'GL_FRAMEBUFFER_COMPLETE_EXT', 'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT', 'GL_FRAMEBUFFER_UNSUPPORTED_EXT',
'GL_MAX_COLOR_ATTACHMENTS_EXT', 'GL_COLOR_ATTACHMENT0_EXT',
'GL_COLOR_ATTACHMENT1_EXT', 'GL_COLOR_ATTACHMENT2_EXT',
'GL_COLOR_ATTACHMENT3_EXT', 'GL_COLOR_ATTACHMENT4_EXT',
'GL_COLOR_ATTACHMENT5_EXT', 'GL_COLOR_ATTACHMENT6_EXT',
'GL_COLOR_ATTACHMENT7_EXT', 'GL_COLOR_ATTACHMENT8_EXT',
'GL_COLOR_ATTACHMENT9_EXT', 'GL_COLOR_ATTACHMENT10_EXT',
'GL_COLOR_ATTACHMENT11_EXT', 'GL_COLOR_ATTACHMENT12_EXT',
'GL_COLOR_ATTACHMENT13_EXT', 'GL_COLOR_ATTACHMENT14_EXT',
'GL_COLOR_ATTACHMENT15_EXT', 'GL_DEPTH_ATTACHMENT_EXT',
'GL_STENCIL_ATTACHMENT_EXT', 'GL_FRAMEBUFFER_EXT', 'GL_RENDERBUFFER_EXT',
'GL_RENDERBUFFER_WIDTH_EXT', 'GL_RENDERBUFFER_HEIGHT_EXT',
'GL_RENDERBUFFER_INTERNAL_FORMAT_EXT', 'GL_STENCIL_INDEX1_EXT',
'GL_STENCIL_INDEX4_EXT', 'GL_STENCIL_INDEX8_EXT', 'GL_STENCIL_INDEX16_EXT',
'GL_RENDERBUFFER_RED_SIZE_EXT', 'GL_RENDERBUFFER_GREEN_SIZE_EXT',
'GL_RENDERBUFFER_BLUE_SIZE_EXT', 'GL_RENDERBUFFER_ALPHA_SIZE_EXT',
'GL_RENDERBUFFER_DEPTH_SIZE_EXT', 'GL_RENDERBUFFER_STENCIL_SIZE_EXT',
'GL_DEPTH_STENCIL_EXT', 'GL_UNSIGNED_INT_24_8_EXT', 'GL_DEPTH24_STENCIL8_EXT',
'GL_TEXTURE_STENCIL_SIZE_EXT', 'GL_STENCIL_TAG_BITS_EXT',
'GL_STENCIL_CLEAR_TAG_VALUE_EXT', 'GL_SRGB_EXT', 'GL_SRGB8_EXT',
'GL_SRGB_ALPHA_EXT', 'GL_SRGB8_ALPHA8_EXT', 'GL_SLUMINANCE_ALPHA_EXT',
'GL_SLUMINANCE8_ALPHA8_EXT', 'GL_SLUMINANCE_EXT', 'GL_SLUMINANCE8_EXT',
'GL_COMPRESSED_SRGB_EXT', 'GL_COMPRESSED_SRGB_ALPHA_EXT',
'GL_COMPRESSED_SLUMINANCE_EXT', 'GL_COMPRESSED_SLUMINANCE_ALPHA_EXT',
'GL_COMPRESSED_SRGB_S3TC_DXT1_EXT', 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT',
'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT',
'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT', 'GL_READ_FRAMEBUFFER_EXT',
'GL_DRAW_FRAMEBUFFER_EXT', 'GL_DRAW_FRAMEBUFFER_BINDING_EXT',
'GL_READ_FRAMEBUFFER_BINDING_EXT', 'GL_RENDERBUFFER_SAMPLES_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT', 'GL_MAX_SAMPLES_EXT',
'GL_TEXTURE_1D_STACK_MESAX', 'GL_TEXTURE_2D_STACK_MESAX',
'GL_PROXY_TEXTURE_1D_STACK_MESAX', 'GL_PROXY_TEXTURE_2D_STACK_MESAX',
'GL_TEXTURE_1D_STACK_BINDING_MESAX', 'GL_TEXTURE_2D_STACK_BINDING_MESAX',
'GL_TIME_ELAPSED_EXT', 'GL_BUFFER_SERIALIZED_MODIFY_APPLE',
'GL_BUFFER_FLUSHING_UNMAP_APPLE', 'GL_MIN_PROGRAM_TEXEL_OFFSET_NV',
'GL_MAX_PROGRAM_TEXEL_OFFSET_NV', 'GL_PROGRAM_ATTRIB_COMPONENTS_NV',
'GL_PROGRAM_RESULT_COMPONENTS_NV', 'GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV',
'GL_MAX_PROGRAM_RESULT_COMPONENTS_NV', 'GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV',
'GL_MAX_PROGRAM_GENERIC_RESULTS_NV', 'GL_LINES_ADJACENCY_EXT',
'GL_LINE_STRIP_ADJACENCY_EXT', 'GL_TRIANGLES_ADJACENCY_EXT',
'GL_TRIANGLE_STRIP_ADJACENCY_EXT', 'GL_GEOMETRY_PROGRAM_NV',
'GL_MAX_PROGRAM_OUTPUT_VERTICES_NV',
'GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV', 'GL_GEOMETRY_VERTICES_OUT_EXT',
'GL_GEOMETRY_INPUT_TYPE_EXT', 'GL_GEOMETRY_OUTPUT_TYPE_EXT',
'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT',
'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT',
'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT', 'GL_PROGRAM_POINT_SIZE_EXT',
'GL_GEOMETRY_SHADER_EXT', 'GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT',
'GL_MAX_VERTEX_VARYING_COMPONENTS_EXT', 'GL_MAX_VARYING_COMPONENTS_EXT',
'GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT',
'GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT',
'GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT',
'GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV', 'GL_SAMPLER_1D_ARRAY_EXT',
'GL_SAMPLER_2D_ARRAY_EXT', 'GL_SAMPLER_BUFFER_EXT',
'GL_SAMPLER_1D_ARRAY_SHADOW_EXT', 'GL_SAMPLER_2D_ARRAY_SHADOW_EXT',
'GL_SAMPLER_CUBE_SHADOW_EXT', 'GL_UNSIGNED_INT_VEC2_EXT',
'GL_UNSIGNED_INT_VEC3_EXT', 'GL_UNSIGNED_INT_VEC4_EXT',
'GL_INT_SAMPLER_1D_EXT', 'GL_INT_SAMPLER_2D_EXT', 'GL_INT_SAMPLER_3D_EXT',
'GL_INT_SAMPLER_CUBE_EXT', 'GL_INT_SAMPLER_2D_RECT_EXT',
'GL_INT_SAMPLER_1D_ARRAY_EXT', 'GL_INT_SAMPLER_2D_ARRAY_EXT',
'GL_INT_SAMPLER_BUFFER_EXT', 'GL_UNSIGNED_INT_SAMPLER_1D_EXT',
'GL_UNSIGNED_INT_SAMPLER_2D_EXT', 'GL_UNSIGNED_INT_SAMPLER_3D_EXT',
'GL_UNSIGNED_INT_SAMPLER_CUBE_EXT', 'GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT',
'GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT',
'GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT', 'GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT',
'GL_R11F_G11F_B10F_EXT', 'GL_UNSIGNED_INT_10F_11F_11F_REV_EXT',
'GL_RGBA_SIGNED_COMPONENTS_EXT', 'GL_TEXTURE_1D_ARRAY_EXT',
'GL_PROXY_TEXTURE_1D_ARRAY_EXT', 'GL_TEXTURE_2D_ARRAY_EXT',
'GL_PROXY_TEXTURE_2D_ARRAY_EXT', 'GL_TEXTURE_BINDING_1D_ARRAY_EXT',
'GL_TEXTURE_BINDING_2D_ARRAY_EXT', 'GL_MAX_ARRAY_TEXTURE_LAYERS_EXT',
'GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT', 'GL_TEXTURE_BUFFER_EXT',
'GL_MAX_TEXTURE_BUFFER_SIZE_EXT', 'GL_TEXTURE_BINDING_BUFFER_EXT',
'GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT', 'GL_TEXTURE_BUFFER_FORMAT_EXT',
'GL_COMPRESSED_LUMINANCE_LATC1_EXT',
'GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT',
'GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT',
'GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT',
'GL_COMPRESSED_RED_RGTC1_EXT', 'GL_COMPRESSED_SIGNED_RED_RGTC1_EXT',
'GL_COMPRESSED_RED_GREEN_RGTC2_EXT',
'GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT', 'GL_RGB9_E5_EXT',
'GL_UNSIGNED_INT_5_9_9_9_REV_EXT', 'GL_TEXTURE_SHARED_SIZE_EXT',
'GL_DEPTH_COMPONENT32F_NV', 'GL_DEPTH32F_STENCIL8_NV',
'GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV', 'GL_DEPTH_BUFFER_FLOAT_MODE_NV',
'GL_RENDERBUFFER_COVERAGE_SAMPLES_NV', 'GL_RENDERBUFFER_COLOR_SAMPLES_NV',
'GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV', 'GL_MULTISAMPLE_COVERAGE_MODES_NV',
'GL_FRAMEBUFFER_SRGB_EXT', 'GL_FRAMEBUFFER_SRGB_CAPABLE_EXT',
'GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV',
'GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV',
'GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV',
'GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV',
'GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV', 'GL_BACK_PRIMARY_COLOR_NV',
'GL_BACK_SECONDARY_COLOR_NV', 'GL_TEXTURE_COORD_NV', 'GL_CLIP_DISTANCE_NV',
'GL_VERTEX_ID_NV', 'GL_PRIMITIVE_ID_NV', 'GL_GENERIC_ATTRIB_NV',
'GL_TRANSFORM_FEEDBACK_ATTRIBS_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV', 'GL_ACTIVE_VARYINGS_NV',
'GL_ACTIVE_VARYING_MAX_LENGTH_NV', 'GL_TRANSFORM_FEEDBACK_VARYINGS_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_START_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV', 'GL_TRANSFORM_FEEDBACK_RECORD_NV',
'GL_PRIMITIVES_GENERATED_NV', 'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV',
'GL_RASTERIZER_DISCARD_NV',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV', 'GL_INTERLEAVED_ATTRIBS_NV',
'GL_SEPARATE_ATTRIBS_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV', 'GL_LAYER_NV', 'GL_NEXT_BUFFER_NV',
'GL_SKIP_COMPONENTS4_NV', 'GL_SKIP_COMPONENTS3_NV', 'GL_SKIP_COMPONENTS2_NV',
'GL_SKIP_COMPONENTS1_NV', 'GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT',
'GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT',
'GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT', 'GL_MAX_BINDABLE_UNIFORM_SIZE_EXT',
'GL_UNIFORM_BUFFER_EXT', 'GL_UNIFORM_BUFFER_BINDING_EXT', 'GL_RGBA32UI_EXT',
'GL_RGB32UI_EXT', 'GL_ALPHA32UI_EXT', 'GL_INTENSITY32UI_EXT',
'GL_LUMINANCE32UI_EXT', 'GL_LUMINANCE_ALPHA32UI_EXT', 'GL_RGBA16UI_EXT',
'GL_RGB16UI_EXT', 'GL_ALPHA16UI_EXT', 'GL_INTENSITY16UI_EXT',
'GL_LUMINANCE16UI_EXT', 'GL_LUMINANCE_ALPHA16UI_EXT', 'GL_RGBA8UI_EXT',
'GL_RGB8UI_EXT', 'GL_ALPHA8UI_EXT', 'GL_INTENSITY8UI_EXT',
'GL_LUMINANCE8UI_EXT', 'GL_LUMINANCE_ALPHA8UI_EXT', 'GL_RGBA32I_EXT',
'GL_RGB32I_EXT', 'GL_ALPHA32I_EXT', 'GL_INTENSITY32I_EXT',
'GL_LUMINANCE32I_EXT', 'GL_LUMINANCE_ALPHA32I_EXT', 'GL_RGBA16I_EXT',
'GL_RGB16I_EXT', 'GL_ALPHA16I_EXT', 'GL_INTENSITY16I_EXT',
'GL_LUMINANCE16I_EXT', 'GL_LUMINANCE_ALPHA16I_EXT', 'GL_RGBA8I_EXT',
'GL_RGB8I_EXT', 'GL_ALPHA8I_EXT', 'GL_INTENSITY8I_EXT', 'GL_LUMINANCE8I_EXT',
'GL_LUMINANCE_ALPHA8I_EXT', 'GL_RED_INTEGER_EXT', 'GL_GREEN_INTEGER_EXT',
'GL_BLUE_INTEGER_EXT', 'GL_ALPHA_INTEGER_EXT', 'GL_RGB_INTEGER_EXT',
'GL_RGBA_INTEGER_EXT', 'GL_BGR_INTEGER_EXT', 'GL_BGRA_INTEGER_EXT',
'GL_LUMINANCE_INTEGER_EXT', 'GL_LUMINANCE_ALPHA_INTEGER_EXT',
'GL_RGBA_INTEGER_MODE_EXT', 'GL_QUERY_WAIT_NV', 'GL_QUERY_NO_WAIT_NV',
'GL_QUERY_BY_REGION_WAIT_NV', 'GL_QUERY_BY_REGION_NO_WAIT_NV', 'GL_FRAME_NV',
'GL_FIELDS_NV', 'GL_CURRENT_TIME_NV', 'GL_NUM_FILL_STREAMS_NV',
'GL_PRESENT_TIME_NV', 'GL_PRESENT_DURATION_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_EXT', 'GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT',
'GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT',
'GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT', 'GL_INTERLEAVED_ATTRIBS_EXT',
'GL_SEPARATE_ATTRIBS_EXT', 'GL_PRIMITIVES_GENERATED_EXT',
'GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT', 'GL_RASTERIZER_DISCARD_EXT',
'GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT',
'GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT',
'GL_TRANSFORM_FEEDBACK_VARYINGS_EXT', 'GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT',
'GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT', 'GL_PROGRAM_MATRIX_EXT',
'GL_TRANSPOSE_PROGRAM_MATRIX_EXT', 'GL_PROGRAM_MATRIX_STACK_DEPTH_EXT',
'GL_TEXTURE_SWIZZLE_R_EXT', 'GL_TEXTURE_SWIZZLE_G_EXT',
'GL_TEXTURE_SWIZZLE_B_EXT', 'GL_TEXTURE_SWIZZLE_A_EXT',
'GL_TEXTURE_SWIZZLE_RGBA_EXT', 'GL_SAMPLE_POSITION_NV', 'GL_SAMPLE_MASK_NV',
'GL_SAMPLE_MASK_VALUE_NV', 'GL_TEXTURE_BINDING_RENDERBUFFER_NV',
'GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV', 'GL_TEXTURE_RENDERBUFFER_NV',
'GL_SAMPLER_RENDERBUFFER_NV', 'GL_INT_SAMPLER_RENDERBUFFER_NV',
'GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV', 'GL_MAX_SAMPLE_MASK_WORDS_NV',
'GL_TRANSFORM_FEEDBACK_NV', 'GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV',
'GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV', 'GL_TRANSFORM_FEEDBACK_BINDING_NV',
'GL_VBO_FREE_MEMORY_ATI', 'GL_TEXTURE_FREE_MEMORY_ATI',
'GL_RENDERBUFFER_FREE_MEMORY_ATI', 'GL_COUNTER_TYPE_AMD',
'GL_COUNTER_RANGE_AMD', 'GL_UNSIGNED_INT64_AMD', 'GL_PERCENTAGE_AMD',
'GL_PERFMON_RESULT_AVAILABLE_AMD', 'GL_PERFMON_RESULT_SIZE_AMD',
'GL_PERFMON_RESULT_AMD', 'GL_SAMPLER_BUFFER_AMD', 'GL_INT_SAMPLER_BUFFER_AMD',
'GL_UNSIGNED_INT_SAMPLER_BUFFER_AMD', 'GL_TESSELLATION_MODE_AMD',
'GL_TESSELLATION_FACTOR_AMD', 'GL_DISCRETE_AMD', 'GL_CONTINUOUS_AMD',
'GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION_EXT',
'GL_FIRST_VERTEX_CONVENTION_EXT', 'GL_LAST_VERTEX_CONVENTION_EXT',
'GL_PROVOKING_VERTEX_EXT', 'GL_ALPHA_SNORM', 'GL_LUMINANCE_SNORM',
'GL_LUMINANCE_ALPHA_SNORM', 'GL_INTENSITY_SNORM', 'GL_ALPHA8_SNORM',
'GL_LUMINANCE8_SNORM', 'GL_LUMINANCE8_ALPHA8_SNORM', 'GL_INTENSITY8_SNORM',
'GL_ALPHA16_SNORM', 'GL_LUMINANCE16_SNORM', 'GL_LUMINANCE16_ALPHA16_SNORM',
'GL_INTENSITY16_SNORM', 'GL_TEXTURE_RANGE_LENGTH_APPLE',
'GL_TEXTURE_RANGE_POINTER_APPLE', 'GL_TEXTURE_STORAGE_HINT_APPLE',
'GL_STORAGE_PRIVATE_APPLE', 'GL_HALF_APPLE', 'GL_RGBA_FLOAT32_APPLE',
'GL_RGB_FLOAT32_APPLE', 'GL_ALPHA_FLOAT32_APPLE',
'GL_INTENSITY_FLOAT32_APPLE', 'GL_LUMINANCE_FLOAT32_APPLE',
'GL_LUMINANCE_ALPHA_FLOAT32_APPLE', 'GL_RGBA_FLOAT16_APPLE',
'GL_RGB_FLOAT16_APPLE', 'GL_ALPHA_FLOAT16_APPLE',
'GL_INTENSITY_FLOAT16_APPLE', 'GL_LUMINANCE_FLOAT16_APPLE',
'GL_LUMINANCE_ALPHA_FLOAT16_APPLE', 'GL_COLOR_FLOAT_APPLE',
'GL_VERTEX_ATTRIB_MAP1_APPLE', 'GL_VERTEX_ATTRIB_MAP2_APPLE',
'GL_VERTEX_ATTRIB_MAP1_SIZE_APPLE', 'GL_VERTEX_ATTRIB_MAP1_COEFF_APPLE',
'GL_VERTEX_ATTRIB_MAP1_ORDER_APPLE', 'GL_VERTEX_ATTRIB_MAP1_DOMAIN_APPLE',
'GL_VERTEX_ATTRIB_MAP2_SIZE_APPLE', 'GL_VERTEX_ATTRIB_MAP2_COEFF_APPLE',
'GL_VERTEX_ATTRIB_MAP2_ORDER_APPLE', 'GL_VERTEX_ATTRIB_MAP2_DOMAIN_APPLE',
'GL_AUX_DEPTH_STENCIL_APPLE', 'GL_BUFFER_OBJECT_APPLE', 'GL_RELEASED_APPLE',
'GL_VOLATILE_APPLE', 'GL_RETAINED_APPLE', 'GL_UNDEFINED_APPLE',
'GL_PURGEABLE_APPLE', 'GL_PACK_ROW_BYTES_APPLE', 'GL_UNPACK_ROW_BYTES_APPLE',
'GL_RGB_422_APPLE', 'GL_VIDEO_BUFFER_NV', 'GL_VIDEO_BUFFER_BINDING_NV',
'GL_FIELD_UPPER_NV', 'GL_FIELD_LOWER_NV', 'GL_NUM_VIDEO_CAPTURE_STREAMS_NV',
'GL_NEXT_VIDEO_CAPTURE_BUFFER_STATUS_NV',
'GL_VIDEO_CAPTURE_TO_422_SUPPORTED_NV', 'GL_LAST_VIDEO_CAPTURE_STATUS_NV',
'GL_VIDEO_BUFFER_PITCH_NV', 'GL_VIDEO_COLOR_CONVERSION_MATRIX_NV',
'GL_VIDEO_COLOR_CONVERSION_MAX_NV', 'GL_VIDEO_COLOR_CONVERSION_MIN_NV',
'GL_VIDEO_COLOR_CONVERSION_OFFSET_NV', 'GL_VIDEO_BUFFER_INTERNAL_FORMAT_NV',
'GL_PARTIAL_SUCCESS_NV', 'GL_SUCCESS_NV', 'GL_FAILURE_NV',
'GL_YCBYCR8_422_NV', 'GL_YCBAYCR8A_4224_NV',
'GL_Z6Y10Z6CB10Z6Y10Z6CR10_422_NV',
'GL_Z6Y10Z6CB10Z6A10Z6Y10Z6CR10Z6A10_4224_NV',
'GL_Z4Y12Z4CB12Z4Y12Z4CR12_422_NV',
'GL_Z4Y12Z4CB12Z4A12Z4Y12Z4CR12Z4A12_4224_NV', 'GL_Z4Y12Z4CB12Z4CR12_444_NV',
'GL_VIDEO_CAPTURE_FRAME_WIDTH_NV', 'GL_VIDEO_CAPTURE_FRAME_HEIGHT_NV',
'GL_VIDEO_CAPTURE_FIELD_UPPER_HEIGHT_NV',
'GL_VIDEO_CAPTURE_FIELD_LOWER_HEIGHT_NV',
'GL_VIDEO_CAPTURE_SURFACE_ORIGIN_NV', 'GL_ACTIVE_PROGRAM_EXT',
'GL_BUFFER_GPU_ADDRESS_NV', 'GL_GPU_ADDRESS_NV',
'GL_MAX_SHADER_BUFFER_ADDRESS_NV', 'GL_VERTEX_ATTRIB_ARRAY_UNIFIED_NV',
'GL_ELEMENT_ARRAY_UNIFIED_NV', 'GL_VERTEX_ATTRIB_ARRAY_ADDRESS_NV',
'GL_VERTEX_ARRAY_ADDRESS_NV', 'GL_NORMAL_ARRAY_ADDRESS_NV',
'GL_COLOR_ARRAY_ADDRESS_NV', 'GL_INDEX_ARRAY_ADDRESS_NV',
'GL_TEXTURE_COORD_ARRAY_ADDRESS_NV', 'GL_EDGE_FLAG_ARRAY_ADDRESS_NV',
'GL_SECONDARY_COLOR_ARRAY_ADDRESS_NV', 'GL_FOG_COORD_ARRAY_ADDRESS_NV',
'GL_ELEMENT_ARRAY_ADDRESS_NV', 'GL_VERTEX_ATTRIB_ARRAY_LENGTH_NV',
'GL_VERTEX_ARRAY_LENGTH_NV', 'GL_NORMAL_ARRAY_LENGTH_NV',
'GL_COLOR_ARRAY_LENGTH_NV', 'GL_INDEX_ARRAY_LENGTH_NV',
'GL_TEXTURE_COORD_ARRAY_LENGTH_NV', 'GL_EDGE_FLAG_ARRAY_LENGTH_NV',
'GL_SECONDARY_COLOR_ARRAY_LENGTH_NV', 'GL_FOG_COORD_ARRAY_LENGTH_NV',
'GL_ELEMENT_ARRAY_LENGTH_NV', 'GL_DRAW_INDIRECT_UNIFIED_NV',
'GL_DRAW_INDIRECT_ADDRESS_NV', 'GL_DRAW_INDIRECT_LENGTH_NV',
'GL_MAX_IMAGE_UNITS_EXT',
'GL_MAX_COMBINED_IMAGE_UNITS_AND_FRAGMENT_OUTPUTS_EXT',
'GL_IMAGE_BINDING_NAME_EXT', 'GL_IMAGE_BINDING_LEVEL_EXT',
'GL_IMAGE_BINDING_LAYERED_EXT', 'GL_IMAGE_BINDING_LAYER_EXT',
'GL_IMAGE_BINDING_ACCESS_EXT', 'GL_IMAGE_1D_EXT', 'GL_IMAGE_2D_EXT',
'GL_IMAGE_3D_EXT', 'GL_IMAGE_2D_RECT_EXT', 'GL_IMAGE_CUBE_EXT',
'GL_IMAGE_BUFFER_EXT', 'GL_IMAGE_1D_ARRAY_EXT', 'GL_IMAGE_2D_ARRAY_EXT',
'GL_IMAGE_CUBE_MAP_ARRAY_EXT', 'GL_IMAGE_2D_MULTISAMPLE_EXT',
'GL_IMAGE_2D_MULTISAMPLE_ARRAY_EXT', 'GL_INT_IMAGE_1D_EXT',
'GL_INT_IMAGE_2D_EXT', 'GL_INT_IMAGE_3D_EXT', 'GL_INT_IMAGE_2D_RECT_EXT',
'GL_INT_IMAGE_CUBE_EXT', 'GL_INT_IMAGE_BUFFER_EXT',
'GL_INT_IMAGE_1D_ARRAY_EXT', 'GL_INT_IMAGE_2D_ARRAY_EXT',
'GL_INT_IMAGE_CUBE_MAP_ARRAY_EXT', 'GL_INT_IMAGE_2D_MULTISAMPLE_EXT',
'GL_INT_IMAGE_2D_MULTISAMPLE_ARRAY_EXT', 'GL_UNSIGNED_INT_IMAGE_1D_EXT',
'GL_UNSIGNED_INT_IMAGE_2D_EXT', 'GL_UNSIGNED_INT_IMAGE_3D_EXT',
'GL_UNSIGNED_INT_IMAGE_2D_RECT_EXT', 'GL_UNSIGNED_INT_IMAGE_CUBE_EXT',
'GL_UNSIGNED_INT_IMAGE_BUFFER_EXT', 'GL_UNSIGNED_INT_IMAGE_1D_ARRAY_EXT',
'GL_UNSIGNED_INT_IMAGE_2D_ARRAY_EXT',
'GL_UNSIGNED_INT_IMAGE_CUBE_MAP_ARRAY_EXT',
'GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_EXT',
'GL_UNSIGNED_INT_IMAGE_2D_MULTISAMPLE_ARRAY_EXT', 'GL_MAX_IMAGE_SAMPLES_EXT',
'GL_IMAGE_BINDING_FORMAT_EXT', 'GL_VERTEX_ATTRIB_ARRAY_BARRIER_BIT_EXT',
'GL_ELEMENT_ARRAY_BARRIER_BIT_EXT', 'GL_UNIFORM_BARRIER_BIT_EXT',
'GL_TEXTURE_FETCH_BARRIER_BIT_EXT', 'GL_SHADER_IMAGE_ACCESS_BARRIER_BIT_EXT',
'GL_COMMAND_BARRIER_BIT_EXT', 'GL_PIXEL_BUFFER_BARRIER_BIT_EXT',
'GL_TEXTURE_UPDATE_BARRIER_BIT_EXT', 'GL_BUFFER_UPDATE_BARRIER_BIT_EXT',
'GL_FRAMEBUFFER_BARRIER_BIT_EXT', 'GL_TRANSFORM_FEEDBACK_BARRIER_BIT_EXT',
'GL_ATOMIC_COUNTER_BARRIER_BIT_EXT', 'GL_ALL_BARRIER_BITS_EXT',
'GL_DOUBLE_VEC2_EXT', 'GL_DOUBLE_VEC3_EXT', 'GL_DOUBLE_VEC4_EXT',
'GL_DOUBLE_MAT2_EXT', 'GL_DOUBLE_MAT3_EXT', 'GL_DOUBLE_MAT4_EXT',
'GL_DOUBLE_MAT2x3_EXT', 'GL_DOUBLE_MAT2x4_EXT', 'GL_DOUBLE_MAT3x2_EXT',
'GL_DOUBLE_MAT3x4_EXT', 'GL_DOUBLE_MAT4x2_EXT', 'GL_DOUBLE_MAT4x3_EXT',
'GL_MAX_GEOMETRY_PROGRAM_INVOCATIONS_NV',
'GL_MIN_FRAGMENT_INTERPOLATION_OFFSET_NV',
'GL_MAX_FRAGMENT_INTERPOLATION_OFFSET_NV',
'GL_FRAGMENT_PROGRAM_INTERPOLATION_OFFSET_BITS_NV',
'GL_MIN_PROGRAM_TEXTURE_GATHER_OFFSET_NV',
'GL_MAX_PROGRAM_TEXTURE_GATHER_OFFSET_NV',
'GL_MAX_PROGRAM_SUBROUTINE_PARAMETERS_NV', 'GL_MAX_PROGRAM_SUBROUTINE_NUM_NV',
'GL_INT64_NV', 'GL_UNSIGNED_INT64_NV', 'GL_INT8_NV', 'GL_INT8_VEC2_NV',
'GL_INT8_VEC3_NV', 'GL_INT8_VEC4_NV', 'GL_INT16_NV', 'GL_INT16_VEC2_NV',
'GL_INT16_VEC3_NV', 'GL_INT16_VEC4_NV', 'GL_INT64_VEC2_NV',
'GL_INT64_VEC3_NV', 'GL_INT64_VEC4_NV', 'GL_UNSIGNED_INT8_NV',
'GL_UNSIGNED_INT8_VEC2_NV', 'GL_UNSIGNED_INT8_VEC3_NV',
'GL_UNSIGNED_INT8_VEC4_NV', 'GL_UNSIGNED_INT16_NV',
'GL_UNSIGNED_INT16_VEC2_NV', 'GL_UNSIGNED_INT16_VEC3_NV',
'GL_UNSIGNED_INT16_VEC4_NV', 'GL_UNSIGNED_INT64_VEC2_NV',
'GL_UNSIGNED_INT64_VEC3_NV', 'GL_UNSIGNED_INT64_VEC4_NV', 'GL_FLOAT16_NV',
'GL_FLOAT16_VEC2_NV', 'GL_FLOAT16_VEC3_NV', 'GL_FLOAT16_VEC4_NV',
'GL_SHADER_GLOBAL_ACCESS_BARRIER_BIT_NV', 'GL_MAX_PROGRAM_PATCH_ATTRIBS_NV',
'GL_TESS_CONTROL_PROGRAM_NV', 'GL_TESS_EVALUATION_PROGRAM_NV',
'GL_TESS_CONTROL_PROGRAM_PARAMETER_BUFFER_NV',
'GL_TESS_EVALUATION_PROGRAM_PARAMETER_BUFFER_NV', 'GL_COVERAGE_SAMPLES_NV',
'GL_COLOR_SAMPLES_NV', 'GL_DATA_BUFFER_AMD', 'GL_PERFORMANCE_MONITOR_AMD',
'GL_QUERY_OBJECT_AMD', 'GL_VERTEX_ARRAY_OBJECT_AMD', 'GL_SAMPLER_OBJECT_AMD',
'GL_MAX_DEBUG_LOGGED_MESSAGES_AMD', 'GL_DEBUG_LOGGED_MESSAGES_AMD',
'GL_DEBUG_SEVERITY_HIGH_AMD', 'GL_DEBUG_SEVERITY_MEDIUM_AMD',
'GL_DEBUG_SEVERITY_LOW_AMD', 'GL_DEBUG_CATEGORY_API_ERROR_AMD',
'GL_DEBUG_CATEGORY_WINDOW_SYSTEM_AMD', 'GL_DEBUG_CATEGORY_DEPRECATION_AMD',
'GL_DEBUG_CATEGORY_UNDEFINED_BEHAVIOR_AMD',
'GL_DEBUG_CATEGORY_PERFORMANCE_AMD', 'GL_DEBUG_CATEGORY_SHADER_COMPILER_AMD',
'GL_DEBUG_CATEGORY_APPLICATION_AMD', 'GL_DEBUG_CATEGORY_OTHER_AMD',
'GL_SURFACE_STATE_NV', 'GL_SURFACE_REGISTERED_NV', 'GL_SURFACE_MAPPED_NV',
'GL_WRITE_DISCARD_NV', 'GL_DEPTH_CLAMP_NEAR_AMD', 'GL_DEPTH_CLAMP_FAR_AMD',
'GL_TEXTURE_SRGB_DECODE_EXT', 'GL_DECODE_EXT', 'GL_SKIP_DECODE_EXT',
'GL_TEXTURE_COVERAGE_SAMPLES_NV', 'GL_TEXTURE_COLOR_SAMPLES_NV',
'GL_FACTOR_MIN_AMD', 'GL_FACTOR_MAX_AMD', 'GL_SUBSAMPLE_DISTANCE_AMD',
'GL_SYNC_X11_FENCE_EXT', 'GL_SCALED_RESOLVE_FASTEST_EXT',
'GL_SCALED_RESOLVE_NICEST_EXT', 'GL_PATH_FORMAT_SVG_NV',
'GL_PATH_FORMAT_PS_NV', 'GL_STANDARD_FONT_NAME_NV', 'GL_SYSTEM_FONT_NAME_NV',
'GL_FILE_NAME_NV', 'GL_PATH_STROKE_WIDTH_NV', 'GL_PATH_END_CAPS_NV',
'GL_PATH_INITIAL_END_CAP_NV', 'GL_PATH_TERMINAL_END_CAP_NV',
'GL_PATH_JOIN_STYLE_NV', 'GL_PATH_MITER_LIMIT_NV', 'GL_PATH_DASH_CAPS_NV',
'GL_PATH_INITIAL_DASH_CAP_NV', 'GL_PATH_TERMINAL_DASH_CAP_NV',
'GL_PATH_DASH_OFFSET_NV', 'GL_PATH_CLIENT_LENGTH_NV', 'GL_PATH_FILL_MODE_NV',
'GL_PATH_FILL_MASK_NV', 'GL_PATH_FILL_COVER_MODE_NV',
'GL_PATH_STROKE_COVER_MODE_NV', 'GL_PATH_STROKE_MASK_NV',
'GL_PATH_SAMPLE_QUALITY_NV', 'GL_PATH_STROKE_BOUND_NV',
'GL_PATH_STROKE_OVERSAMPLE_COUNT_NV', 'GL_COUNT_UP_NV', 'GL_COUNT_DOWN_NV',
'GL_PATH_OBJECT_BOUNDING_BOX_NV', 'GL_CONVEX_HULL_NV', 'GL_MULTI_HULLS_NV',
'GL_BOUNDING_BOX_NV', 'GL_TRANSLATE_X_NV', 'GL_TRANSLATE_Y_NV',
'GL_TRANSLATE_2D_NV', 'GL_TRANSLATE_3D_NV', 'GL_AFFINE_2D_NV',
'GL_PROJECTIVE_2D_NV', 'GL_AFFINE_3D_NV', 'GL_PROJECTIVE_3D_NV',
'GL_TRANSPOSE_AFFINE_2D_NV', 'GL_TRANSPOSE_PROJECTIVE_2D_NV',
'GL_TRANSPOSE_AFFINE_3D_NV', 'GL_TRANSPOSE_PROJECTIVE_3D_NV', 'GL_UTF8_NV',
'GL_UTF16_NV', 'GL_BOUNDING_BOX_OF_BOUNDING_BOXES_NV',
'GL_PATH_COMMAND_COUNT_NV', 'GL_PATH_COORD_COUNT_NV',
'GL_PATH_DASH_ARRAY_COUNT_NV', 'GL_PATH_COMPUTED_LENGTH_NV',
'GL_PATH_FILL_BOUNDING_BOX_NV', 'GL_PATH_STROKE_BOUNDING_BOX_NV',
'GL_SQUARE_NV', 'GL_ROUND_NV', 'GL_TRIANGULAR_NV', 'GL_BEVEL_NV',
'GL_MITER_REVERT_NV', 'GL_MITER_TRUNCATE_NV', 'GL_SKIP_MISSING_GLYPH_NV',
'GL_USE_MISSING_GLYPH_NV', 'GL_PATH_ERROR_POSITION_NV',
'GL_PATH_FOG_GEN_MODE_NV', 'GL_ACCUM_ADJACENT_PAIRS_NV',
'GL_ADJACENT_PAIRS_NV', 'GL_FIRST_TO_REST_NV', 'GL_PATH_GEN_MODE_NV',
'GL_PATH_GEN_COEFF_NV', 'GL_PATH_GEN_COLOR_FORMAT_NV',
'GL_PATH_GEN_COMPONENTS_NV', 'GL_PATH_STENCIL_FUNC_NV',
'GL_PATH_STENCIL_REF_NV', 'GL_PATH_STENCIL_VALUE_MASK_NV',
'GL_PATH_STENCIL_DEPTH_OFFSET_FACTOR_NV',
'GL_PATH_STENCIL_DEPTH_OFFSET_UNITS_NV', 'GL_PATH_COVER_DEPTH_FUNC_NV',
'GL_PATH_DASH_OFFSET_RESET_NV', 'GL_MOVE_TO_RESETS_NV',
'GL_MOVE_TO_CONTINUES_NV', 'GL_CLOSE_PATH_NV', 'GL_MOVE_TO_NV',
'GL_RELATIVE_MOVE_TO_NV', 'GL_LINE_TO_NV', 'GL_RELATIVE_LINE_TO_NV',
'GL_HORIZONTAL_LINE_TO_NV', 'GL_RELATIVE_HORIZONTAL_LINE_TO_NV',
'GL_VERTICAL_LINE_TO_NV', 'GL_RELATIVE_VERTICAL_LINE_TO_NV',
'GL_QUADRATIC_CURVE_TO_NV', 'GL_RELATIVE_QUADRATIC_CURVE_TO_NV',
'GL_CUBIC_CURVE_TO_NV', 'GL_RELATIVE_CUBIC_CURVE_TO_NV',
'GL_SMOOTH_QUADRATIC_CURVE_TO_NV', 'GL_RELATIVE_SMOOTH_QUADRATIC_CURVE_TO_NV',
'GL_SMOOTH_CUBIC_CURVE_TO_NV', 'GL_RELATIVE_SMOOTH_CUBIC_CURVE_TO_NV',
'GL_SMALL_CCW_ARC_TO_NV', 'GL_RELATIVE_SMALL_CCW_ARC_TO_NV',
'GL_SMALL_CW_ARC_TO_NV', 'GL_RELATIVE_SMALL_CW_ARC_TO_NV',
'GL_LARGE_CCW_ARC_TO_NV', 'GL_RELATIVE_LARGE_CCW_ARC_TO_NV',
'GL_LARGE_CW_ARC_TO_NV', 'GL_RELATIVE_LARGE_CW_ARC_TO_NV',
'GL_RESTART_PATH_NV', 'GL_DUP_FIRST_CUBIC_CURVE_TO_NV',
'GL_DUP_LAST_CUBIC_CURVE_TO_NV', 'GL_RECT_NV', 'GL_CIRCULAR_CCW_ARC_TO_NV',
'GL_CIRCULAR_CW_ARC_TO_NV', 'GL_CIRCULAR_TANGENT_ARC_TO_NV', 'GL_ARC_TO_NV',
'GL_RELATIVE_ARC_TO_NV', 'GL_BOLD_BIT_NV', 'GL_ITALIC_BIT_NV',
'GL_GLYPH_WIDTH_BIT_NV', 'GL_GLYPH_HEIGHT_BIT_NV',
'GL_GLYPH_HORIZONTAL_BEARING_X_BIT_NV',
'GL_GLYPH_HORIZONTAL_BEARING_Y_BIT_NV',
'GL_GLYPH_HORIZONTAL_BEARING_ADVANCE_BIT_NV',
'GL_GLYPH_VERTICAL_BEARING_X_BIT_NV', 'GL_GLYPH_VERTICAL_BEARING_Y_BIT_NV',
'GL_GLYPH_VERTICAL_BEARING_ADVANCE_BIT_NV', 'GL_GLYPH_HAS_KERNING_NV',
'GL_FONT_X_MIN_BOUNDS_NV', 'GL_FONT_Y_MIN_BOUNDS_NV',
'GL_FONT_X_MAX_BOUNDS_NV', 'GL_FONT_Y_MAX_BOUNDS_NV',
'GL_FONT_UNITS_PER_EM_NV', 'GL_FONT_ASCENDER_NV', 'GL_FONT_DESCENDER_NV',
'GL_FONT_HEIGHT_NV', 'GL_FONT_MAX_ADVANCE_WIDTH_NV',
'GL_FONT_MAX_ADVANCE_HEIGHT_NV', 'GL_FONT_UNDERLINE_POSITION_NV',
'GL_FONT_UNDERLINE_THICKNESS_NV', 'GL_FONT_HAS_KERNING_NV',
'GL_EXTERNAL_VIRTUAL_MEMORY_BUFFER_AMD', 'GL_SET_AMD', 'GL_REPLACE_VALUE_AMD',
'GL_STENCIL_OP_VALUE_AMD', 'GL_STENCIL_BACK_OP_VALUE_AMD',
'GL_QUERY_BUFFER_AMD', 'GL_QUERY_BUFFER_BINDING_AMD',
'GL_QUERY_RESULT_NO_WAIT_AMD', 'GLchar', 'GLintptr', 'GLsizeiptr',
'GLintptrARB', 'GLsizeiptrARB', 'GLcharARB', 'GLhandleARB', 'GLhalfARB',
'GLhalfNV', 'GLint64EXT', 'GLuint64EXT', 'GLint64', 'GLuint64', 'GLsync',
'GLDEBUGPROCARB', 'GLDEBUGPROCAMD', 'GLvdpauSurfaceNV',
'GL_VERSION_1_2_DEPRECATED', 'glColorTable', 'glColorTableParameterfv',
'glColorTableParameteriv', 'glCopyColorTable', 'glGetColorTable',
'glGetColorTableParameterfv', 'glGetColorTableParameteriv', 'glColorSubTable',
'glCopyColorSubTable', 'glConvolutionFilter1D', 'glConvolutionFilter2D',
'glConvolutionParameterf', 'glConvolutionParameterfv',
'glConvolutionParameteri', 'glConvolutionParameteriv',
'glCopyConvolutionFilter1D', 'glCopyConvolutionFilter2D',
'glGetConvolutionFilter', 'glGetConvolutionParameterfv',
'glGetConvolutionParameteriv', 'glGetSeparableFilter', 'glSeparableFilter2D',
'glGetHistogram', 'glGetHistogramParameterfv', 'glGetHistogramParameteriv',
'glGetMinmax', 'glGetMinmaxParameterfv', 'glGetMinmaxParameteriv',
'glHistogram', 'glMinmax', 'glResetHistogram', 'glResetMinmax',
'PFNGLCOLORTABLEPROC', 'PFNGLCOLORTABLEPARAMETERFVPROC',
'PFNGLCOLORTABLEPARAMETERIVPROC', 'PFNGLCOPYCOLORTABLEPROC',
'PFNGLGETCOLORTABLEPROC', 'PFNGLGETCOLORTABLEPARAMETERFVPROC',
'PFNGLGETCOLORTABLEPARAMETERIVPROC', 'PFNGLCOLORSUBTABLEPROC',
'PFNGLCOPYCOLORSUBTABLEPROC', 'PFNGLCONVOLUTIONFILTER1DPROC',
'PFNGLCONVOLUTIONFILTER2DPROC', 'PFNGLCONVOLUTIONPARAMETERFPROC',
'PFNGLCONVOLUTIONPARAMETERFVPROC', 'PFNGLCONVOLUTIONPARAMETERIPROC',
'PFNGLCONVOLUTIONPARAMETERIVPROC', 'PFNGLCOPYCONVOLUTIONFILTER1DPROC',
'PFNGLCOPYCONVOLUTIONFILTER2DPROC', 'PFNGLGETCONVOLUTIONFILTERPROC',
'PFNGLGETCONVOLUTIONPARAMETERFVPROC', 'PFNGLGETCONVOLUTIONPARAMETERIVPROC',
'PFNGLGETSEPARABLEFILTERPROC', 'PFNGLSEPARABLEFILTER2DPROC',
'PFNGLGETHISTOGRAMPROC', 'PFNGLGETHISTOGRAMPARAMETERFVPROC',
'PFNGLGETHISTOGRAMPARAMETERIVPROC', 'PFNGLGETMINMAXPROC',
'PFNGLGETMINMAXPARAMETERFVPROC', 'PFNGLGETMINMAXPARAMETERIVPROC',
'PFNGLHISTOGRAMPROC', 'PFNGLMINMAXPROC', 'PFNGLRESETHISTOGRAMPROC',
'PFNGLRESETMINMAXPROC', 'GL_VERSION_1_3_DEPRECATED', 'glClientActiveTexture',
'glMultiTexCoord1d', 'glMultiTexCoord1dv', 'glMultiTexCoord1f',
'glMultiTexCoord1fv', 'glMultiTexCoord1i', 'glMultiTexCoord1iv',
'glMultiTexCoord1s', 'glMultiTexCoord1sv', 'glMultiTexCoord2d',
'glMultiTexCoord2dv', 'glMultiTexCoord2f', 'glMultiTexCoord2fv',
'glMultiTexCoord2i', 'glMultiTexCoord2iv', 'glMultiTexCoord2s',
'glMultiTexCoord2sv', 'glMultiTexCoord3d', 'glMultiTexCoord3dv',
'glMultiTexCoord3f', 'glMultiTexCoord3fv', 'glMultiTexCoord3i',
'glMultiTexCoord3iv', 'glMultiTexCoord3s', 'glMultiTexCoord3sv',
'glMultiTexCoord4d', 'glMultiTexCoord4dv', 'glMultiTexCoord4f',
'glMultiTexCoord4fv', 'glMultiTexCoord4i', 'glMultiTexCoord4iv',
'glMultiTexCoord4s', 'glMultiTexCoord4sv', 'glLoadTransposeMatrixf',
'glLoadTransposeMatrixd', 'glMultTransposeMatrixf', 'glMultTransposeMatrixd',
'PFNGLCLIENTACTIVETEXTUREPROC', 'PFNGLMULTITEXCOORD1DPROC',
'PFNGLMULTITEXCOORD1DVPROC', 'PFNGLMULTITEXCOORD1FPROC',
'PFNGLMULTITEXCOORD1FVPROC', 'PFNGLMULTITEXCOORD1IPROC',
'PFNGLMULTITEXCOORD1IVPROC', 'PFNGLMULTITEXCOORD1SPROC',
'PFNGLMULTITEXCOORD1SVPROC', 'PFNGLMULTITEXCOORD2DPROC',
'PFNGLMULTITEXCOORD2DVPROC', 'PFNGLMULTITEXCOORD2FPROC',
'PFNGLMULTITEXCOORD2FVPROC', 'PFNGLMULTITEXCOORD2IPROC',
'PFNGLMULTITEXCOORD2IVPROC', 'PFNGLMULTITEXCOORD2SPROC',
'PFNGLMULTITEXCOORD2SVPROC', 'PFNGLMULTITEXCOORD3DPROC',
'PFNGLMULTITEXCOORD3DVPROC', 'PFNGLMULTITEXCOORD3FPROC',
'PFNGLMULTITEXCOORD3FVPROC', 'PFNGLMULTITEXCOORD3IPROC',
'PFNGLMULTITEXCOORD3IVPROC', 'PFNGLMULTITEXCOORD3SPROC',
'PFNGLMULTITEXCOORD3SVPROC', 'PFNGLMULTITEXCOORD4DPROC',
'PFNGLMULTITEXCOORD4DVPROC', 'PFNGLMULTITEXCOORD4FPROC',
'PFNGLMULTITEXCOORD4FVPROC', 'PFNGLMULTITEXCOORD4IPROC',
'PFNGLMULTITEXCOORD4IVPROC', 'PFNGLMULTITEXCOORD4SPROC',
'PFNGLMULTITEXCOORD4SVPROC', 'PFNGLLOADTRANSPOSEMATRIXFPROC',
'PFNGLLOADTRANSPOSEMATRIXDPROC', 'PFNGLMULTTRANSPOSEMATRIXFPROC',
'PFNGLMULTTRANSPOSEMATRIXDPROC', 'GL_VERSION_1_4', 'glBlendFuncSeparate',
'glMultiDrawArrays', 'glMultiDrawElements', 'glPointParameterf',
'glPointParameterfv', 'glPointParameteri', 'glPointParameteriv',
'PFNGLBLENDFUNCSEPARATEPROC', 'PFNGLMULTIDRAWARRAYSPROC',
'PFNGLMULTIDRAWELEMENTSPROC', 'PFNGLPOINTPARAMETERFPROC',
'PFNGLPOINTPARAMETERFVPROC', 'PFNGLPOINTPARAMETERIPROC',
'PFNGLPOINTPARAMETERIVPROC', 'GL_VERSION_1_4_DEPRECATED', 'glFogCoordf',
'glFogCoordfv', 'glFogCoordd', 'glFogCoorddv', 'glFogCoordPointer',
'glSecondaryColor3b', 'glSecondaryColor3bv', 'glSecondaryColor3d',
'glSecondaryColor3dv', 'glSecondaryColor3f', 'glSecondaryColor3fv',
'glSecondaryColor3i', 'glSecondaryColor3iv', 'glSecondaryColor3s',
'glSecondaryColor3sv', 'glSecondaryColor3ub', 'glSecondaryColor3ubv',
'glSecondaryColor3ui', 'glSecondaryColor3uiv', 'glSecondaryColor3us',
'glSecondaryColor3usv', 'glSecondaryColorPointer', 'glWindowPos2d',
'glWindowPos2dv', 'glWindowPos2f', 'glWindowPos2fv', 'glWindowPos2i',
'glWindowPos2iv', 'glWindowPos2s', 'glWindowPos2sv', 'glWindowPos3d',
'glWindowPos3dv', 'glWindowPos3f', 'glWindowPos3fv', 'glWindowPos3i',
'glWindowPos3iv', 'glWindowPos3s', 'glWindowPos3sv', 'PFNGLFOGCOORDFPROC',
'PFNGLFOGCOORDFVPROC', 'PFNGLFOGCOORDDPROC', 'PFNGLFOGCOORDDVPROC',
'PFNGLFOGCOORDPOINTERPROC', 'PFNGLSECONDARYCOLOR3BPROC',
'PFNGLSECONDARYCOLOR3BVPROC', 'PFNGLSECONDARYCOLOR3DPROC',
'PFNGLSECONDARYCOLOR3DVPROC', 'PFNGLSECONDARYCOLOR3FPROC',
'PFNGLSECONDARYCOLOR3FVPROC', 'PFNGLSECONDARYCOLOR3IPROC',
'PFNGLSECONDARYCOLOR3IVPROC', 'PFNGLSECONDARYCOLOR3SPROC',
'PFNGLSECONDARYCOLOR3SVPROC', 'PFNGLSECONDARYCOLOR3UBPROC',
'PFNGLSECONDARYCOLOR3UBVPROC', 'PFNGLSECONDARYCOLOR3UIPROC',
'PFNGLSECONDARYCOLOR3UIVPROC', 'PFNGLSECONDARYCOLOR3USPROC',
'PFNGLSECONDARYCOLOR3USVPROC', 'PFNGLSECONDARYCOLORPOINTERPROC',
'PFNGLWINDOWPOS2DPROC', 'PFNGLWINDOWPOS2DVPROC', 'PFNGLWINDOWPOS2FPROC',
'PFNGLWINDOWPOS2FVPROC', 'PFNGLWINDOWPOS2IPROC', 'PFNGLWINDOWPOS2IVPROC',
'PFNGLWINDOWPOS2SPROC', 'PFNGLWINDOWPOS2SVPROC', 'PFNGLWINDOWPOS3DPROC',
'PFNGLWINDOWPOS3DVPROC', 'PFNGLWINDOWPOS3FPROC', 'PFNGLWINDOWPOS3FVPROC',
'PFNGLWINDOWPOS3IPROC', 'PFNGLWINDOWPOS3IVPROC', 'PFNGLWINDOWPOS3SPROC',
'PFNGLWINDOWPOS3SVPROC', 'GL_VERSION_1_5', 'glGenQueries', 'glDeleteQueries',
'glIsQuery', 'glBeginQuery', 'glEndQuery', 'glGetQueryiv',
'glGetQueryObjectiv', 'glGetQueryObjectuiv', 'glBindBuffer',
'glDeleteBuffers', 'glGenBuffers', 'glIsBuffer', 'glBufferData',
'glBufferSubData', 'glGetBufferSubData', 'glMapBuffer', 'glUnmapBuffer',
'glGetBufferParameteriv', 'glGetBufferPointerv', 'PFNGLGENQUERIESPROC',
'PFNGLDELETEQUERIESPROC', 'PFNGLISQUERYPROC', 'PFNGLBEGINQUERYPROC',
'PFNGLENDQUERYPROC', 'PFNGLGETQUERYIVPROC', 'PFNGLGETQUERYOBJECTIVPROC',
'PFNGLGETQUERYOBJECTUIVPROC', 'PFNGLBINDBUFFERPROC', 'PFNGLDELETEBUFFERSPROC',
'PFNGLGENBUFFERSPROC', 'PFNGLISBUFFERPROC', 'PFNGLBUFFERDATAPROC',
'PFNGLBUFFERSUBDATAPROC', 'PFNGLGETBUFFERSUBDATAPROC', 'PFNGLMAPBUFFERPROC',
'PFNGLUNMAPBUFFERPROC', 'PFNGLGETBUFFERPARAMETERIVPROC',
'PFNGLGETBUFFERPOINTERVPROC', 'GL_VERSION_2_0', 'glBlendEquationSeparate',
'glDrawBuffers', 'glStencilOpSeparate', 'glStencilFuncSeparate',
'glStencilMaskSeparate', 'glAttachShader', 'glBindAttribLocation',
'glCompileShader', 'glCreateProgram', 'glCreateShader', 'glDeleteProgram',
'glDeleteShader', 'glDetachShader', 'glDisableVertexAttribArray',
'glEnableVertexAttribArray', 'glGetActiveAttrib', 'glGetActiveUniform',
'glGetAttachedShaders', 'glGetAttribLocation', 'glGetProgramiv',
'glGetProgramInfoLog', 'glGetShaderiv', 'glGetShaderInfoLog',
'glGetShaderSource', 'glGetUniformLocation', 'glGetUniformfv',
'glGetUniformiv', 'glGetVertexAttribdv', 'glGetVertexAttribfv',
'glGetVertexAttribiv', 'glGetVertexAttribPointerv', 'glIsProgram',
'glIsShader', 'glLinkProgram', 'glShaderSource', 'glUseProgram',
'glUniform1f', 'glUniform2f', 'glUniform3f', 'glUniform4f', 'glUniform1i',
'glUniform2i', 'glUniform3i', 'glUniform4i', 'glUniform1fv', 'glUniform2fv',
'glUniform3fv', 'glUniform4fv', 'glUniform1iv', 'glUniform2iv',
'glUniform3iv', 'glUniform4iv', 'glUniformMatrix2fv', 'glUniformMatrix3fv',
'glUniformMatrix4fv', 'glValidateProgram', 'glVertexAttrib1d',
'glVertexAttrib1dv', 'glVertexAttrib1f', 'glVertexAttrib1fv',
'glVertexAttrib1s', 'glVertexAttrib1sv', 'glVertexAttrib2d',
'glVertexAttrib2dv', 'glVertexAttrib2f', 'glVertexAttrib2fv',
'glVertexAttrib2s', 'glVertexAttrib2sv', 'glVertexAttrib3d',
'glVertexAttrib3dv', 'glVertexAttrib3f', 'glVertexAttrib3fv',
'glVertexAttrib3s', 'glVertexAttrib3sv', 'glVertexAttrib4Nbv',
'glVertexAttrib4Niv', 'glVertexAttrib4Nsv', 'glVertexAttrib4Nub',
'glVertexAttrib4Nubv', 'glVertexAttrib4Nuiv', 'glVertexAttrib4Nusv',
'glVertexAttrib4bv', 'glVertexAttrib4d', 'glVertexAttrib4dv',
'glVertexAttrib4f', 'glVertexAttrib4fv', 'glVertexAttrib4iv',
'glVertexAttrib4s', 'glVertexAttrib4sv', 'glVertexAttrib4ubv',
'glVertexAttrib4uiv', 'glVertexAttrib4usv', 'glVertexAttribPointer',
'PFNGLBLENDEQUATIONSEPARATEPROC', 'PFNGLDRAWBUFFERSPROC',
'PFNGLSTENCILOPSEPARATEPROC', 'PFNGLSTENCILFUNCSEPARATEPROC',
'PFNGLSTENCILMASKSEPARATEPROC', 'PFNGLATTACHSHADERPROC',
'PFNGLBINDATTRIBLOCATIONPROC', 'PFNGLCOMPILESHADERPROC',
'PFNGLCREATEPROGRAMPROC', 'PFNGLCREATESHADERPROC', 'PFNGLDELETEPROGRAMPROC',
'PFNGLDELETESHADERPROC', 'PFNGLDETACHSHADERPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYPROC', 'PFNGLENABLEVERTEXATTRIBARRAYPROC',
'PFNGLGETACTIVEATTRIBPROC', 'PFNGLGETACTIVEUNIFORMPROC',
'PFNGLGETATTACHEDSHADERSPROC', 'PFNGLGETATTRIBLOCATIONPROC',
'PFNGLGETPROGRAMIVPROC', 'PFNGLGETPROGRAMINFOLOGPROC', 'PFNGLGETSHADERIVPROC',
'PFNGLGETSHADERINFOLOGPROC', 'PFNGLGETSHADERSOURCEPROC',
'PFNGLGETUNIFORMLOCATIONPROC', 'PFNGLGETUNIFORMFVPROC',
'PFNGLGETUNIFORMIVPROC', 'PFNGLGETVERTEXATTRIBDVPROC',
'PFNGLGETVERTEXATTRIBFVPROC', 'PFNGLGETVERTEXATTRIBIVPROC',
'PFNGLGETVERTEXATTRIBPOINTERVPROC', 'PFNGLISPROGRAMPROC', 'PFNGLISSHADERPROC',
'PFNGLLINKPROGRAMPROC', 'PFNGLSHADERSOURCEPROC', 'PFNGLUSEPROGRAMPROC',
'PFNGLUNIFORM1FPROC', 'PFNGLUNIFORM2FPROC', 'PFNGLUNIFORM3FPROC',
'PFNGLUNIFORM4FPROC', 'PFNGLUNIFORM1IPROC', 'PFNGLUNIFORM2IPROC',
'PFNGLUNIFORM3IPROC', 'PFNGLUNIFORM4IPROC', 'PFNGLUNIFORM1FVPROC',
'PFNGLUNIFORM2FVPROC', 'PFNGLUNIFORM3FVPROC', 'PFNGLUNIFORM4FVPROC',
'PFNGLUNIFORM1IVPROC', 'PFNGLUNIFORM2IVPROC', 'PFNGLUNIFORM3IVPROC',
'PFNGLUNIFORM4IVPROC', 'PFNGLUNIFORMMATRIX2FVPROC',
'PFNGLUNIFORMMATRIX3FVPROC', 'PFNGLUNIFORMMATRIX4FVPROC',
'PFNGLVALIDATEPROGRAMPROC', 'PFNGLVERTEXATTRIB1DPROC',
'PFNGLVERTEXATTRIB1DVPROC', 'PFNGLVERTEXATTRIB1FPROC',
'PFNGLVERTEXATTRIB1FVPROC', 'PFNGLVERTEXATTRIB1SPROC',
'PFNGLVERTEXATTRIB1SVPROC', 'PFNGLVERTEXATTRIB2DPROC',
'PFNGLVERTEXATTRIB2DVPROC', 'PFNGLVERTEXATTRIB2FPROC',
'PFNGLVERTEXATTRIB2FVPROC', 'PFNGLVERTEXATTRIB2SPROC',
'PFNGLVERTEXATTRIB2SVPROC', 'PFNGLVERTEXATTRIB3DPROC',
'PFNGLVERTEXATTRIB3DVPROC', 'PFNGLVERTEXATTRIB3FPROC',
'PFNGLVERTEXATTRIB3FVPROC', 'PFNGLVERTEXATTRIB3SPROC',
'PFNGLVERTEXATTRIB3SVPROC', 'PFNGLVERTEXATTRIB4NBVPROC',
'PFNGLVERTEXATTRIB4NIVPROC', 'PFNGLVERTEXATTRIB4NSVPROC',
'PFNGLVERTEXATTRIB4NUBPROC', 'PFNGLVERTEXATTRIB4NUBVPROC',
'PFNGLVERTEXATTRIB4NUIVPROC', 'PFNGLVERTEXATTRIB4NUSVPROC',
'PFNGLVERTEXATTRIB4BVPROC', 'PFNGLVERTEXATTRIB4DPROC',
'PFNGLVERTEXATTRIB4DVPROC', 'PFNGLVERTEXATTRIB4FPROC',
'PFNGLVERTEXATTRIB4FVPROC', 'PFNGLVERTEXATTRIB4IVPROC',
'PFNGLVERTEXATTRIB4SPROC', 'PFNGLVERTEXATTRIB4SVPROC',
'PFNGLVERTEXATTRIB4UBVPROC', 'PFNGLVERTEXATTRIB4UIVPROC',
'PFNGLVERTEXATTRIB4USVPROC', 'PFNGLVERTEXATTRIBPOINTERPROC', 'GL_VERSION_2_1',
'glUniformMatrix2x3fv', 'glUniformMatrix3x2fv', 'glUniformMatrix2x4fv',
'glUniformMatrix4x2fv', 'glUniformMatrix3x4fv', 'glUniformMatrix4x3fv',
'PFNGLUNIFORMMATRIX2X3FVPROC', 'PFNGLUNIFORMMATRIX3X2FVPROC',
'PFNGLUNIFORMMATRIX2X4FVPROC', 'PFNGLUNIFORMMATRIX4X2FVPROC',
'PFNGLUNIFORMMATRIX3X4FVPROC', 'PFNGLUNIFORMMATRIX4X3FVPROC',
'GL_VERSION_3_0', 'glColorMaski', 'glGetBooleani_v', 'glGetIntegeri_v',
'glEnablei', 'glDisablei', 'glIsEnabledi', 'glBeginTransformFeedback',
'glEndTransformFeedback', 'glBindBufferRange', 'glBindBufferBase',
'glTransformFeedbackVaryings', 'glGetTransformFeedbackVarying',
'glClampColor', 'glBeginConditionalRender', 'glEndConditionalRender',
'glVertexAttribIPointer', 'glGetVertexAttribIiv', 'glGetVertexAttribIuiv',
'glVertexAttribI1i', 'glVertexAttribI2i', 'glVertexAttribI3i',
'glVertexAttribI4i', 'glVertexAttribI1ui', 'glVertexAttribI2ui',
'glVertexAttribI3ui', 'glVertexAttribI4ui', 'glVertexAttribI1iv',
'glVertexAttribI2iv', 'glVertexAttribI3iv', 'glVertexAttribI4iv',
'glVertexAttribI1uiv', 'glVertexAttribI2uiv', 'glVertexAttribI3uiv',
'glVertexAttribI4uiv', 'glVertexAttribI4bv', 'glVertexAttribI4sv',
'glVertexAttribI4ubv', 'glVertexAttribI4usv', 'glGetUniformuiv',
'glBindFragDataLocation', 'glGetFragDataLocation', 'glUniform1ui',
'glUniform2ui', 'glUniform3ui', 'glUniform4ui', 'glUniform1uiv',
'glUniform2uiv', 'glUniform3uiv', 'glUniform4uiv', 'glTexParameterIiv',
'glTexParameterIuiv', 'glGetTexParameterIiv', 'glGetTexParameterIuiv',
'glClearBufferiv', 'glClearBufferuiv', 'glClearBufferfv', 'glClearBufferfi',
'glGetStringi', 'PFNGLCOLORMASKIPROC', 'PFNGLGETBOOLEANI_VPROC',
'PFNGLGETINTEGERI_VPROC', 'PFNGLENABLEIPROC', 'PFNGLDISABLEIPROC',
'PFNGLISENABLEDIPROC', 'PFNGLBEGINTRANSFORMFEEDBACKPROC',
'PFNGLENDTRANSFORMFEEDBACKPROC', 'PFNGLBINDBUFFERRANGEPROC',
'PFNGLBINDBUFFERBASEPROC', 'PFNGLTRANSFORMFEEDBACKVARYINGSPROC',
'PFNGLGETTRANSFORMFEEDBACKVARYINGPROC', 'PFNGLCLAMPCOLORPROC',
'PFNGLBEGINCONDITIONALRENDERPROC', 'PFNGLENDCONDITIONALRENDERPROC',
'PFNGLVERTEXATTRIBIPOINTERPROC', 'PFNGLGETVERTEXATTRIBIIVPROC',
'PFNGLGETVERTEXATTRIBIUIVPROC', 'PFNGLVERTEXATTRIBI1IPROC',
'PFNGLVERTEXATTRIBI2IPROC', 'PFNGLVERTEXATTRIBI3IPROC',
'PFNGLVERTEXATTRIBI4IPROC', 'PFNGLVERTEXATTRIBI1UIPROC',
'PFNGLVERTEXATTRIBI2UIPROC', 'PFNGLVERTEXATTRIBI3UIPROC',
'PFNGLVERTEXATTRIBI4UIPROC', 'PFNGLVERTEXATTRIBI1IVPROC',
'PFNGLVERTEXATTRIBI2IVPROC', 'PFNGLVERTEXATTRIBI3IVPROC',
'PFNGLVERTEXATTRIBI4IVPROC', 'PFNGLVERTEXATTRIBI1UIVPROC',
'PFNGLVERTEXATTRIBI2UIVPROC', 'PFNGLVERTEXATTRIBI3UIVPROC',
'PFNGLVERTEXATTRIBI4UIVPROC', 'PFNGLVERTEXATTRIBI4BVPROC',
'PFNGLVERTEXATTRIBI4SVPROC', 'PFNGLVERTEXATTRIBI4UBVPROC',
'PFNGLVERTEXATTRIBI4USVPROC', 'PFNGLGETUNIFORMUIVPROC',
'PFNGLBINDFRAGDATALOCATIONPROC', 'PFNGLGETFRAGDATALOCATIONPROC',
'PFNGLUNIFORM1UIPROC', 'PFNGLUNIFORM2UIPROC', 'PFNGLUNIFORM3UIPROC',
'PFNGLUNIFORM4UIPROC', 'PFNGLUNIFORM1UIVPROC', 'PFNGLUNIFORM2UIVPROC',
'PFNGLUNIFORM3UIVPROC', 'PFNGLUNIFORM4UIVPROC', 'PFNGLTEXPARAMETERIIVPROC',
'PFNGLTEXPARAMETERIUIVPROC', 'PFNGLGETTEXPARAMETERIIVPROC',
'PFNGLGETTEXPARAMETERIUIVPROC', 'PFNGLCLEARBUFFERIVPROC',
'PFNGLCLEARBUFFERUIVPROC', 'PFNGLCLEARBUFFERFVPROC', 'PFNGLCLEARBUFFERFIPROC',
'PFNGLGETSTRINGIPROC', 'GL_VERSION_3_1', 'glDrawArraysInstanced',
'glDrawElementsInstanced', 'glTexBuffer', 'glPrimitiveRestartIndex',
'PFNGLDRAWARRAYSINSTANCEDPROC', 'PFNGLDRAWELEMENTSINSTANCEDPROC',
'PFNGLTEXBUFFERPROC', 'PFNGLPRIMITIVERESTARTINDEXPROC', 'GL_VERSION_3_2',
'glGetInteger64i_v', 'glGetBufferParameteri64v', 'glFramebufferTexture',
'PFNGLGETINTEGER64I_VPROC', 'PFNGLGETBUFFERPARAMETERI64VPROC',
'PFNGLFRAMEBUFFERTEXTUREPROC', 'GL_VERSION_3_3', 'glVertexAttribDivisor',
'PFNGLVERTEXATTRIBDIVISORPROC', 'GL_VERSION_4_0', 'glMinSampleShading',
'glBlendEquationi', 'glBlendEquationSeparatei', 'glBlendFunci',
'glBlendFuncSeparatei', 'PFNGLMINSAMPLESHADINGPROC',
'PFNGLBLENDEQUATIONIPROC', 'PFNGLBLENDEQUATIONSEPARATEIPROC',
'PFNGLBLENDFUNCIPROC', 'PFNGLBLENDFUNCSEPARATEIPROC', 'GL_VERSION_4_1',
'GL_VERSION_4_2', 'GL_ARB_transpose_matrix', 'glLoadTransposeMatrixfARB',
'glLoadTransposeMatrixdARB', 'glMultTransposeMatrixfARB',
'glMultTransposeMatrixdARB', 'PFNGLLOADTRANSPOSEMATRIXFARBPROC',
'PFNGLLOADTRANSPOSEMATRIXDARBPROC', 'PFNGLMULTTRANSPOSEMATRIXFARBPROC',
'PFNGLMULTTRANSPOSEMATRIXDARBPROC', 'GL_ARB_multisample',
'glSampleCoverageARB', 'PFNGLSAMPLECOVERAGEARBPROC', 'GL_ARB_texture_env_add',
'GL_ARB_texture_cube_map', 'GL_ARB_texture_compression',
'glCompressedTexImage3DARB', 'glCompressedTexImage2DARB',
'glCompressedTexImage1DARB', 'glCompressedTexSubImage3DARB',
'glCompressedTexSubImage2DARB', 'glCompressedTexSubImage1DARB',
'glGetCompressedTexImageARB', 'PFNGLCOMPRESSEDTEXIMAGE3DARBPROC',
'PFNGLCOMPRESSEDTEXIMAGE2DARBPROC', 'PFNGLCOMPRESSEDTEXIMAGE1DARBPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC', 'PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC',
'PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC', 'PFNGLGETCOMPRESSEDTEXIMAGEARBPROC',
'GL_ARB_texture_border_clamp', 'GL_ARB_point_parameters',
'glPointParameterfARB', 'glPointParameterfvARB',
'PFNGLPOINTPARAMETERFARBPROC', 'PFNGLPOINTPARAMETERFVARBPROC',
'GL_ARB_vertex_blend', 'glWeightbvARB', 'glWeightsvARB', 'glWeightivARB',
'glWeightfvARB', 'glWeightdvARB', 'glWeightubvARB', 'glWeightusvARB',
'glWeightuivARB', 'glWeightPointerARB', 'glVertexBlendARB',
'PFNGLWEIGHTBVARBPROC', 'PFNGLWEIGHTSVARBPROC', 'PFNGLWEIGHTIVARBPROC',
'PFNGLWEIGHTFVARBPROC', 'PFNGLWEIGHTDVARBPROC', 'PFNGLWEIGHTUBVARBPROC',
'PFNGLWEIGHTUSVARBPROC', 'PFNGLWEIGHTUIVARBPROC', 'PFNGLWEIGHTPOINTERARBPROC',
'PFNGLVERTEXBLENDARBPROC', 'GL_ARB_matrix_palette',
'glCurrentPaletteMatrixARB', 'glMatrixIndexubvARB', 'glMatrixIndexusvARB',
'glMatrixIndexuivARB', 'glMatrixIndexPointerARB',
'PFNGLCURRENTPALETTEMATRIXARBPROC', 'PFNGLMATRIXINDEXUBVARBPROC',
'PFNGLMATRIXINDEXUSVARBPROC', 'PFNGLMATRIXINDEXUIVARBPROC',
'PFNGLMATRIXINDEXPOINTERARBPROC', 'GL_ARB_texture_env_combine',
'GL_ARB_texture_env_crossbar', 'GL_ARB_texture_env_dot3',
'GL_ARB_texture_mirrored_repeat', 'GL_ARB_depth_texture', 'GL_ARB_shadow',
'GL_ARB_shadow_ambient', 'GL_ARB_window_pos', 'glWindowPos2dARB',
'glWindowPos2dvARB', 'glWindowPos2fARB', 'glWindowPos2fvARB',
'glWindowPos2iARB', 'glWindowPos2ivARB', 'glWindowPos2sARB',
'glWindowPos2svARB', 'glWindowPos3dARB', 'glWindowPos3dvARB',
'glWindowPos3fARB', 'glWindowPos3fvARB', 'glWindowPos3iARB',
'glWindowPos3ivARB', 'glWindowPos3sARB', 'glWindowPos3svARB',
'PFNGLWINDOWPOS2DARBPROC', 'PFNGLWINDOWPOS2DVARBPROC',
'PFNGLWINDOWPOS2FARBPROC', 'PFNGLWINDOWPOS2FVARBPROC',
'PFNGLWINDOWPOS2IARBPROC', 'PFNGLWINDOWPOS2IVARBPROC',
'PFNGLWINDOWPOS2SARBPROC', 'PFNGLWINDOWPOS2SVARBPROC',
'PFNGLWINDOWPOS3DARBPROC', 'PFNGLWINDOWPOS3DVARBPROC',
'PFNGLWINDOWPOS3FARBPROC', 'PFNGLWINDOWPOS3FVARBPROC',
'PFNGLWINDOWPOS3IARBPROC', 'PFNGLWINDOWPOS3IVARBPROC',
'PFNGLWINDOWPOS3SARBPROC', 'PFNGLWINDOWPOS3SVARBPROC',
'GL_ARB_vertex_program', 'glVertexAttrib1dARB', 'glVertexAttrib1dvARB',
'glVertexAttrib1fARB', 'glVertexAttrib1fvARB', 'glVertexAttrib1sARB',
'glVertexAttrib1svARB', 'glVertexAttrib2dARB', 'glVertexAttrib2dvARB',
'glVertexAttrib2fARB', 'glVertexAttrib2fvARB', 'glVertexAttrib2sARB',
'glVertexAttrib2svARB', 'glVertexAttrib3dARB', 'glVertexAttrib3dvARB',
'glVertexAttrib3fARB', 'glVertexAttrib3fvARB', 'glVertexAttrib3sARB',
'glVertexAttrib3svARB', 'glVertexAttrib4NbvARB', 'glVertexAttrib4NivARB',
'glVertexAttrib4NsvARB', 'glVertexAttrib4NubARB', 'glVertexAttrib4NubvARB',
'glVertexAttrib4NuivARB', 'glVertexAttrib4NusvARB', 'glVertexAttrib4bvARB',
'glVertexAttrib4dARB', 'glVertexAttrib4dvARB', 'glVertexAttrib4fARB',
'glVertexAttrib4fvARB', 'glVertexAttrib4ivARB', 'glVertexAttrib4sARB',
'glVertexAttrib4svARB', 'glVertexAttrib4ubvARB', 'glVertexAttrib4uivARB',
'glVertexAttrib4usvARB', 'glVertexAttribPointerARB',
'glEnableVertexAttribArrayARB', 'glDisableVertexAttribArrayARB',
'glProgramStringARB', 'glBindProgramARB', 'glDeleteProgramsARB',
'glGenProgramsARB', 'glProgramEnvParameter4dARB',
'glProgramEnvParameter4dvARB', 'glProgramEnvParameter4fARB',
'glProgramEnvParameter4fvARB', 'glProgramLocalParameter4dARB',
'glProgramLocalParameter4dvARB', 'glProgramLocalParameter4fARB',
'glProgramLocalParameter4fvARB', 'glGetProgramEnvParameterdvARB',
'glGetProgramEnvParameterfvARB', 'glGetProgramLocalParameterdvARB',
'glGetProgramLocalParameterfvARB', 'glGetProgramivARB',
'glGetProgramStringARB', 'glGetVertexAttribdvARB', 'glGetVertexAttribfvARB',
'glGetVertexAttribivARB', 'glGetVertexAttribPointervARB', 'glIsProgramARB',
'PFNGLVERTEXATTRIB1DARBPROC', 'PFNGLVERTEXATTRIB1DVARBPROC',
'PFNGLVERTEXATTRIB1FARBPROC', 'PFNGLVERTEXATTRIB1FVARBPROC',
'PFNGLVERTEXATTRIB1SARBPROC', 'PFNGLVERTEXATTRIB1SVARBPROC',
'PFNGLVERTEXATTRIB2DARBPROC', 'PFNGLVERTEXATTRIB2DVARBPROC',
'PFNGLVERTEXATTRIB2FARBPROC', 'PFNGLVERTEXATTRIB2FVARBPROC',
'PFNGLVERTEXATTRIB2SARBPROC', 'PFNGLVERTEXATTRIB2SVARBPROC',
'PFNGLVERTEXATTRIB3DARBPROC', 'PFNGLVERTEXATTRIB3DVARBPROC',
'PFNGLVERTEXATTRIB3FARBPROC', 'PFNGLVERTEXATTRIB3FVARBPROC',
'PFNGLVERTEXATTRIB3SARBPROC', 'PFNGLVERTEXATTRIB3SVARBPROC',
'PFNGLVERTEXATTRIB4NBVARBPROC', 'PFNGLVERTEXATTRIB4NIVARBPROC',
'PFNGLVERTEXATTRIB4NSVARBPROC', 'PFNGLVERTEXATTRIB4NUBARBPROC',
'PFNGLVERTEXATTRIB4NUBVARBPROC', 'PFNGLVERTEXATTRIB4NUIVARBPROC',
'PFNGLVERTEXATTRIB4NUSVARBPROC', 'PFNGLVERTEXATTRIB4BVARBPROC',
'PFNGLVERTEXATTRIB4DARBPROC', 'PFNGLVERTEXATTRIB4DVARBPROC',
'PFNGLVERTEXATTRIB4FARBPROC', 'PFNGLVERTEXATTRIB4FVARBPROC',
'PFNGLVERTEXATTRIB4IVARBPROC', 'PFNGLVERTEXATTRIB4SARBPROC',
'PFNGLVERTEXATTRIB4SVARBPROC', 'PFNGLVERTEXATTRIB4UBVARBPROC',
'PFNGLVERTEXATTRIB4UIVARBPROC', 'PFNGLVERTEXATTRIB4USVARBPROC',
'PFNGLVERTEXATTRIBPOINTERARBPROC', 'PFNGLENABLEVERTEXATTRIBARRAYARBPROC',
'PFNGLDISABLEVERTEXATTRIBARRAYARBPROC', 'PFNGLPROGRAMSTRINGARBPROC',
'PFNGLBINDPROGRAMARBPROC', 'PFNGLDELETEPROGRAMSARBPROC',
'PFNGLGENPROGRAMSARBPROC', 'PFNGLPROGRAMENVPARAMETER4DARBPROC',
'PFNGLPROGRAMENVPARAMETER4DVARBPROC', 'PFNGLPROGRAMENVPARAMETER4FARBPROC',
'PFNGLPROGRAMENVPARAMETER4FVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4DARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4DVARBPROC', 'PFNGLPROGRAMLOCALPARAMETER4FARBPROC',
'PFNGLPROGRAMLOCALPARAMETER4FVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERDVARBPROC',
'PFNGLGETPROGRAMENVPARAMETERFVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC',
'PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC', 'PFNGLGETPROGRAMIVARBPROC',
'PFNGLGETPROGRAMSTRINGARBPROC', 'PFNGLGETVERTEXATTRIBDVARBPROC',
'PFNGLGETVERTEXATTRIBFVARBPROC', 'PFNGLGETVERTEXATTRIBIVARBPROC',
'PFNGLGETVERTEXATTRIBPOINTERVARBPROC', 'PFNGLISPROGRAMARBPROC',
'GL_ARB_fragment_program', 'GL_ARB_vertex_buffer_object', 'glBindBufferARB',
'glDeleteBuffersARB', 'glGenBuffersARB', 'glIsBufferARB', 'glBufferDataARB',
'glBufferSubDataARB', 'glGetBufferSubDataARB', 'glMapBufferARB',
'glUnmapBufferARB', 'glGetBufferParameterivARB', 'glGetBufferPointervARB',
'PFNGLBINDBUFFERARBPROC', 'PFNGLDELETEBUFFERSARBPROC',
'PFNGLGENBUFFERSARBPROC', 'PFNGLISBUFFERARBPROC', 'PFNGLBUFFERDATAARBPROC',
'PFNGLBUFFERSUBDATAARBPROC', 'PFNGLGETBUFFERSUBDATAARBPROC',
'PFNGLMAPBUFFERARBPROC', 'PFNGLUNMAPBUFFERARBPROC',
'PFNGLGETBUFFERPARAMETERIVARBPROC', 'PFNGLGETBUFFERPOINTERVARBPROC',
'GL_ARB_occlusion_query', 'glGenQueriesARB', 'glDeleteQueriesARB',
'glIsQueryARB', 'glBeginQueryARB', 'glEndQueryARB', 'glGetQueryivARB',
'glGetQueryObjectivARB', 'glGetQueryObjectuivARB', 'PFNGLGENQUERIESARBPROC',
'PFNGLDELETEQUERIESARBPROC', 'PFNGLISQUERYARBPROC', 'PFNGLBEGINQUERYARBPROC',
'PFNGLENDQUERYARBPROC', 'PFNGLGETQUERYIVARBPROC',
'PFNGLGETQUERYOBJECTIVARBPROC', 'PFNGLGETQUERYOBJECTUIVARBPROC',
'GL_ARB_shader_objects', 'glDeleteObjectARB', 'glGetHandleARB',
'glDetachObjectARB', 'glCreateShaderObjectARB', 'glShaderSourceARB',
'glCompileShaderARB', 'glCreateProgramObjectARB', 'glAttachObjectARB',
'glLinkProgramARB', 'glUseProgramObjectARB', 'glValidateProgramARB',
'glUniform1fARB', 'glUniform2fARB', 'glUniform3fARB', 'glUniform4fARB',
'glUniform1iARB', 'glUniform2iARB', 'glUniform3iARB', 'glUniform4iARB',
'glUniform1fvARB', 'glUniform2fvARB', 'glUniform3fvARB', 'glUniform4fvARB',
'glUniform1ivARB', 'glUniform2ivARB', 'glUniform3ivARB', 'glUniform4ivARB',
'glUniformMatrix2fvARB', 'glUniformMatrix3fvARB', 'glUniformMatrix4fvARB',
'glGetObjectParameterfvARB', 'glGetObjectParameterivARB', 'glGetInfoLogARB',
'glGetAttachedObjectsARB', 'glGetUniformLocationARB', 'glGetActiveUniformARB',
'glGetUniformfvARB', 'glGetUniformivARB', 'glGetShaderSourceARB',
'PFNGLDELETEOBJECTARBPROC', 'PFNGLGETHANDLEARBPROC',
'PFNGLDETACHOBJECTARBPROC', 'PFNGLCREATESHADEROBJECTARBPROC',
'PFNGLSHADERSOURCEARBPROC', 'PFNGLCOMPILESHADERARBPROC',
'PFNGLCREATEPROGRAMOBJECTARBPROC', 'PFNGLATTACHOBJECTARBPROC',
'PFNGLLINKPROGRAMARBPROC', 'PFNGLUSEPROGRAMOBJECTARBPROC',
'PFNGLVALIDATEPROGRAMARBPROC', 'PFNGLUNIFORM1FARBPROC',
'PFNGLUNIFORM2FARBPROC', 'PFNGLUNIFORM3FARBPROC', 'PFNGLUNIFORM4FARBPROC',
'PFNGLUNIFORM1IARBPROC', 'PFNGLUNIFORM2IARBPROC', 'PFNGLUNIFORM3IARBPROC',
'PFNGLUNIFORM4IARBPROC', 'PFNGLUNIFORM1FVARBPROC', 'PFNGLUNIFORM2FVARBPROC',
'PFNGLUNIFORM3FVARBPROC', 'PFNGLUNIFORM4FVARBPROC', 'PFNGLUNIFORM1IVARBPROC',
'PFNGLUNIFORM2IVARBPROC', 'PFNGLUNIFORM3IVARBPROC', 'PFNGLUNIFORM4IVARBPROC',
'PFNGLUNIFORMMATRIX2FVARBPROC', 'PFNGLUNIFORMMATRIX3FVARBPROC',
'PFNGLUNIFORMMATRIX4FVARBPROC', 'PFNGLGETOBJECTPARAMETERFVARBPROC',
'PFNGLGETOBJECTPARAMETERIVARBPROC', 'PFNGLGETINFOLOGARBPROC',
'PFNGLGETATTACHEDOBJECTSARBPROC', 'PFNGLGETUNIFORMLOCATIONARBPROC',
'PFNGLGETACTIVEUNIFORMARBPROC', 'PFNGLGETUNIFORMFVARBPROC',
'PFNGLGETUNIFORMIVARBPROC', 'PFNGLGETSHADERSOURCEARBPROC',
'GL_ARB_vertex_shader', 'glBindAttribLocationARB', 'glGetActiveAttribARB',
'glGetAttribLocationARB', 'PFNGLBINDATTRIBLOCATIONARBPROC',
'PFNGLGETACTIVEATTRIBARBPROC', 'PFNGLGETATTRIBLOCATIONARBPROC',
'GL_ARB_fragment_shader', 'GL_ARB_shading_language_100',
'GL_ARB_texture_non_power_of_two', 'GL_ARB_point_sprite',
'GL_ARB_fragment_program_shadow', 'GL_ARB_draw_buffers', 'glDrawBuffersARB',
'PFNGLDRAWBUFFERSARBPROC', 'GL_ARB_texture_rectangle',
'GL_ARB_color_buffer_float', 'glClampColorARB', 'PFNGLCLAMPCOLORARBPROC',
'GL_ARB_half_float_pixel', 'GL_ARB_texture_float',
'GL_ARB_pixel_buffer_object', 'GL_ARB_depth_buffer_float',
'GL_ARB_draw_instanced', 'glDrawArraysInstancedARB',
'glDrawElementsInstancedARB', 'PFNGLDRAWARRAYSINSTANCEDARBPROC',
'PFNGLDRAWELEMENTSINSTANCEDARBPROC', 'GL_ARB_framebuffer_object',
'glIsRenderbuffer', 'glBindRenderbuffer', 'glDeleteRenderbuffers',
'glGenRenderbuffers', 'glRenderbufferStorage', 'glGetRenderbufferParameteriv',
'glIsFramebuffer', 'glBindFramebuffer', 'glDeleteFramebuffers',
'glGenFramebuffers', 'glCheckFramebufferStatus', 'glFramebufferTexture1D',
'glFramebufferTexture2D', 'glFramebufferTexture3D',
'glFramebufferRenderbuffer', 'glGetFramebufferAttachmentParameteriv',
'glGenerateMipmap', 'glBlitFramebuffer', 'glRenderbufferStorageMultisample',
'glFramebufferTextureLayer', 'PFNGLISRENDERBUFFERPROC',
'PFNGLBINDRENDERBUFFERPROC', 'PFNGLDELETERENDERBUFFERSPROC',
'PFNGLGENRENDERBUFFERSPROC', 'PFNGLRENDERBUFFERSTORAGEPROC',
'PFNGLGETRENDERBUFFERPARAMETERIVPROC', 'PFNGLISFRAMEBUFFERPROC',
'PFNGLBINDFRAMEBUFFERPROC', 'PFNGLDELETEFRAMEBUFFERSPROC',
'PFNGLGENFRAMEBUFFERSPROC', 'PFNGLCHECKFRAMEBUFFERSTATUSPROC',
'PFNGLFRAMEBUFFERTEXTURE1DPROC', 'PFNGLFRAMEBUFFERTEXTURE2DPROC',
'PFNGLFRAMEBUFFERTEXTURE3DPROC', 'PFNGLFRAMEBUFFERRENDERBUFFERPROC',
'PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC', 'PFNGLGENERATEMIPMAPPROC',
'PFNGLBLITFRAMEBUFFERPROC', 'PFNGLRENDERBUFFERSTORAGEMULTISAMPLEPROC',
'PFNGLFRAMEBUFFERTEXTURELAYERPROC', 'GL_ARB_framebuffer_sRGB',
'GL_ARB_geometry_shader4', 'glProgramParameteriARB',
'glFramebufferTextureARB', 'glFramebufferTextureLayerARB',
'glFramebufferTextureFaceARB', 'PFNGLPROGRAMPARAMETERIARBPROC',
'PFNGLFRAMEBUFFERTEXTUREARBPROC', 'PFNGLFRAMEBUFFERTEXTURELAYERARBPROC',
'PFNGLFRAMEBUFFERTEXTUREFACEARBPROC', 'GL_ARB_half_float_vertex',
'GL_ARB_instanced_arrays', 'glVertexAttribDivisorARB',
'PFNGLVERTEXATTRIBDIVISORARBPROC', 'GL_ARB_map_buffer_range',
'glMapBufferRange', 'glFlushMappedBufferRange', 'PFNGLMAPBUFFERRANGEPROC',
'PFNGLFLUSHMAPPEDBUFFERRANGEPROC', 'GL_ARB_texture_buffer_object',
'glTexBufferARB', 'PFNGLTEXBUFFERARBPROC', 'GL_ARB_texture_compression_rgtc',
'GL_ARB_texture_rg', 'GL_ARB_vertex_array_object', 'glBindVertexArray',
'glDeleteVertexArrays', 'glGenVertexArrays', 'glIsVertexArray',
'PFNGLBINDVERTEXARRAYPROC', 'PFNGLDELETEVERTEXARRAYSPROC',
'PFNGLGENVERTEXARRAYSPROC', 'PFNGLISVERTEXARRAYPROC',
'GL_ARB_uniform_buffer_object', 'glGetUniformIndices',
'glGetActiveUniformsiv', 'glGetActiveUniformName', 'glGetUniformBlockIndex',
'glGetActiveUniformBlockiv', 'glGetActiveUniformBlockName',
'glUniformBlockBinding', 'PFNGLGETUNIFORMINDICESPROC',
'PFNGLGETACTIVEUNIFORMSIVPROC', 'PFNGLGETACTIVEUNIFORMNAMEPROC',
'PFNGLGETUNIFORMBLOCKINDEXPROC', 'PFNGLGETACTIVEUNIFORMBLOCKIVPROC',
'PFNGLGETACTIVEUNIFORMBLOCKNAMEPROC', 'PFNGLUNIFORMBLOCKBINDINGPROC',
'GL_ARB_compatibility', 'GL_ARB_copy_buffer', 'glCopyBufferSubData',
'PFNGLCOPYBUFFERSUBDATAPROC', 'GL_ARB_shader_texture_lod',
'GL_ARB_depth_clamp', 'GL_ARB_draw_elements_base_vertex',
'glDrawElementsBaseVertex', 'glDrawRangeElementsBaseVertex',
'glDrawElementsInstancedBaseVertex', 'glMultiDrawElementsBaseVertex',
'PFNGLDRAWELEMENTSBASEVERTEXPROC', 'PFNGLDRAWRANGEELEMENTSBASEVERTEXPROC',
'PFNGLDRAWELEMENTSINSTANCEDBASEVERTEXPROC',
'PFNGLMULTIDRAWELEMENTSBASEVERTEXPROC', 'GL_ARB_fragment_coord_conventions',
'GL_ARB_provoking_vertex', 'glProvokingVertex', 'PFNGLPROVOKINGVERTEXPROC',
'GL_ARB_seamless_cube_map', 'GL_ARB_sync', 'glFenceSync', 'glIsSync',
'glDeleteSync', 'glClientWaitSync', 'glWaitSync', 'glGetInteger64v',
'glGetSynciv', 'PFNGLFENCESYNCPROC', 'PFNGLISSYNCPROC', 'PFNGLDELETESYNCPROC',
'PFNGLCLIENTWAITSYNCPROC', 'PFNGLWAITSYNCPROC', 'PFNGLGETINTEGER64VPROC',
'PFNGLGETSYNCIVPROC', 'GL_ARB_texture_multisample', 'glTexImage2DMultisample',
'glTexImage3DMultisample', 'glGetMultisamplefv', 'glSampleMaski',
'PFNGLTEXIMAGE2DMULTISAMPLEPROC', 'PFNGLTEXIMAGE3DMULTISAMPLEPROC',
'PFNGLGETMULTISAMPLEFVPROC', 'PFNGLSAMPLEMASKIPROC',
'GL_ARB_vertex_array_bgra', 'GL_ARB_draw_buffers_blend',
'glBlendEquationiARB', 'glBlendEquationSeparateiARB', 'glBlendFunciARB',
'glBlendFuncSeparateiARB', 'PFNGLBLENDEQUATIONIARBPROC',
'PFNGLBLENDEQUATIONSEPARATEIARBPROC', 'PFNGLBLENDFUNCIARBPROC',
'PFNGLBLENDFUNCSEPARATEIARBPROC', 'GL_ARB_sample_shading',
'glMinSampleShadingARB', 'PFNGLMINSAMPLESHADINGARBPROC',
'GL_ARB_texture_cube_map_array', 'GL_ARB_texture_gather',
'GL_ARB_texture_query_lod', 'GL_ARB_shading_language_include',
'glNamedStringARB', 'glDeleteNamedStringARB', 'glCompileShaderIncludeARB',
'glIsNamedStringARB', 'glGetNamedStringARB', 'glGetNamedStringivARB',
'PFNGLNAMEDSTRINGARBPROC', 'PFNGLDELETENAMEDSTRINGARBPROC',
'PFNGLCOMPILESHADERINCLUDEARBPROC', 'PFNGLISNAMEDSTRINGARBPROC',
'PFNGLGETNAMEDSTRINGARBPROC', 'PFNGLGETNAMEDSTRINGIVARBPROC',
'GL_ARB_texture_compression_bptc', 'GL_ARB_blend_func_extended',
'glBindFragDataLocationIndexed', 'glGetFragDataIndex',
'PFNGLBINDFRAGDATALOCATIONINDEXEDPROC', 'PFNGLGETFRAGDATAINDEXPROC',
'GL_ARB_explicit_attrib_location', 'GL_ARB_occlusion_query2',
'GL_ARB_sampler_objects', 'glGenSamplers', 'glDeleteSamplers', 'glIsSampler',
'glBindSampler', 'glSamplerParameteri', 'glSamplerParameteriv',
'glSamplerParameterf', 'glSamplerParameterfv', 'glSamplerParameterIiv',
'glSamplerParameterIuiv', 'glGetSamplerParameteriv',
'glGetSamplerParameterIiv', 'glGetSamplerParameterfv',
'glGetSamplerParameterIuiv', 'PFNGLGENSAMPLERSPROC',
'PFNGLDELETESAMPLERSPROC', 'PFNGLISSAMPLERPROC', 'PFNGLBINDSAMPLERPROC',
'PFNGLSAMPLERPARAMETERIPROC', 'PFNGLSAMPLERPARAMETERIVPROC',
'PFNGLSAMPLERPARAMETERFPROC', 'PFNGLSAMPLERPARAMETERFVPROC',
'PFNGLSAMPLERPARAMETERIIVPROC', 'PFNGLSAMPLERPARAMETERIUIVPROC',
'PFNGLGETSAMPLERPARAMETERIVPROC', 'PFNGLGETSAMPLERPARAMETERIIVPROC',
'PFNGLGETSAMPLERPARAMETERFVPROC', 'PFNGLGETSAMPLERPARAMETERIUIVPROC',
'GL_ARB_shader_bit_encoding', 'GL_ARB_texture_rgb10_a2ui',
'GL_ARB_texture_swizzle', 'GL_ARB_timer_query', 'glQueryCounter',
'glGetQueryObjecti64v', 'glGetQueryObjectui64v', 'PFNGLQUERYCOUNTERPROC',
'PFNGLGETQUERYOBJECTI64VPROC', 'PFNGLGETQUERYOBJECTUI64VPROC',
'GL_ARB_vertex_type_2_10_10_10_rev', 'glVertexP2ui', 'glVertexP2uiv',
'glVertexP3ui', 'glVertexP3uiv', 'glVertexP4ui', 'glVertexP4uiv',
'glTexCoordP1ui', 'glTexCoordP1uiv', 'glTexCoordP2ui', 'glTexCoordP2uiv',
'glTexCoordP3ui', 'glTexCoordP3uiv', 'glTexCoordP4ui', 'glTexCoordP4uiv',
'glMultiTexCoordP1ui', 'glMultiTexCoordP1uiv', 'glMultiTexCoordP2ui',
'glMultiTexCoordP2uiv', 'glMultiTexCoordP3ui', 'glMultiTexCoordP3uiv',
'glMultiTexCoordP4ui', 'glMultiTexCoordP4uiv', 'glNormalP3ui',
'glNormalP3uiv', 'glColorP3ui', 'glColorP3uiv', 'glColorP4ui', 'glColorP4uiv',
'glSecondaryColorP3ui', 'glSecondaryColorP3uiv', 'glVertexAttribP1ui',
'glVertexAttribP1uiv', 'glVertexAttribP2ui', 'glVertexAttribP2uiv',
'glVertexAttribP3ui', 'glVertexAttribP3uiv', 'glVertexAttribP4ui',
'glVertexAttribP4uiv', 'PFNGLVERTEXP2UIPROC', 'PFNGLVERTEXP2UIVPROC',
'PFNGLVERTEXP3UIPROC', 'PFNGLVERTEXP3UIVPROC', 'PFNGLVERTEXP4UIPROC',
'PFNGLVERTEXP4UIVPROC', 'PFNGLTEXCOORDP1UIPROC', 'PFNGLTEXCOORDP1UIVPROC',
'PFNGLTEXCOORDP2UIPROC', 'PFNGLTEXCOORDP2UIVPROC', 'PFNGLTEXCOORDP3UIPROC',
'PFNGLTEXCOORDP3UIVPROC', 'PFNGLTEXCOORDP4UIPROC', 'PFNGLTEXCOORDP4UIVPROC',
'PFNGLMULTITEXCOORDP1UIPROC', 'PFNGLMULTITEXCOORDP1UIVPROC',
'PFNGLMULTITEXCOORDP2UIPROC', 'PFNGLMULTITEXCOORDP2UIVPROC',
'PFNGLMULTITEXCOORDP3UIPROC', 'PFNGLMULTITEXCOORDP3UIVPROC',
'PFNGLMULTITEXCOORDP4UIPROC', 'PFNGLMULTITEXCOORDP4UIVPROC',
'PFNGLNORMALP3UIPROC', 'PFNGLNORMALP3UIVPROC', 'PFNGLCOLORP3UIPROC',
'PFNGLCOLORP3UIVPROC', 'PFNGLCOLORP4UIPROC', 'PFNGLCOLORP4UIVPROC',
'PFNGLSECONDARYCOLORP3UIPROC', 'PFNGLSECONDARYCOLORP3UIVPROC',
'PFNGLVERTEXATTRIBP1UIPROC', 'PFNGLVERTEXATTRIBP1UIVPROC',
'PFNGLVERTEXATTRIBP2UIPROC', 'PFNGLVERTEXATTRIBP2UIVPROC',
'PFNGLVERTEXATTRIBP3UIPROC', 'PFNGLVERTEXATTRIBP3UIVPROC',
'PFNGLVERTEXATTRIBP4UIPROC', 'PFNGLVERTEXATTRIBP4UIVPROC',
'GL_ARB_draw_indirect', 'glDrawArraysIndirect', 'glDrawElementsIndirect',
'PFNGLDRAWARRAYSINDIRECTPROC', 'PFNGLDRAWELEMENTSINDIRECTPROC',
'GL_ARB_gpu_shader5', 'GL_ARB_gpu_shader_fp64', 'glUniform1d', 'glUniform2d',
'glUniform3d', 'glUniform4d', 'glUniform1dv', 'glUniform2dv', 'glUniform3dv',
'glUniform4dv', 'glUniformMatrix2dv', 'glUniformMatrix3dv',
'glUniformMatrix4dv', 'glUniformMatrix2x3dv', 'glUniformMatrix2x4dv',
'glUniformMatrix3x2dv', 'glUniformMatrix3x4dv', 'glUniformMatrix4x2dv',
'glUniformMatrix4x3dv', 'glGetUniformdv', 'PFNGLUNIFORM1DPROC',
'PFNGLUNIFORM2DPROC', 'PFNGLUNIFORM3DPROC', 'PFNGLUNIFORM4DPROC',
'PFNGLUNIFORM1DVPROC', 'PFNGLUNIFORM2DVPROC', 'PFNGLUNIFORM3DVPROC',
'PFNGLUNIFORM4DVPROC', 'PFNGLUNIFORMMATRIX2DVPROC',
'PFNGLUNIFORMMATRIX3DVPROC', 'PFNGLUNIFORMMATRIX4DVPROC',
'PFNGLUNIFORMMATRIX2X3DVPROC', 'PFNGLUNIFORMMATRIX2X4DVPROC',
'PFNGLUNIFORMMATRIX3X2DVPROC', 'PFNGLUNIFORMMATRIX3X4DVPROC',
'PFNGLUNIFORMMATRIX4X2DVPROC', 'PFNGLUNIFORMMATRIX4X3DVPROC',
'PFNGLGETUNIFORMDVPROC', 'GL_ARB_shader_subroutine',
'glGetSubroutineUniformLocation', 'glGetSubroutineIndex',
'glGetActiveSubroutineUniformiv', 'glGetActiveSubroutineUniformName',
'glGetActiveSubroutineName', 'glUniformSubroutinesuiv',
'glGetUniformSubroutineuiv', 'glGetProgramStageiv',
'PFNGLGETSUBROUTINEUNIFORMLOCATIONPROC', 'PFNGLGETSUBROUTINEINDEXPROC',
'PFNGLGETACTIVESUBROUTINEUNIFORMIVPROC',
'PFNGLGETACTIVESUBROUTINEUNIFORMNAMEPROC', 'PFNGLGETACTIVESUBROUTINENAMEPROC',
'PFNGLUNIFORMSUBROUTINESUIVPROC', 'PFNGLGETUNIFORMSUBROUTINEUIVPROC',
'PFNGLGETPROGRAMSTAGEIVPROC', 'GL_ARB_tessellation_shader',
'glPatchParameteri', 'glPatchParameterfv', 'PFNGLPATCHPARAMETERIPROC',
'PFNGLPATCHPARAMETERFVPROC', 'GL_ARB_texture_buffer_object_rgb32',
'GL_ARB_transform_feedback2', 'glBindTransformFeedback',
'glDeleteTransformFeedbacks', 'glGenTransformFeedbacks',
'glIsTransformFeedback', 'glPauseTransformFeedback',
'glResumeTransformFeedback', 'glDrawTransformFeedback',
'PFNGLBINDTRANSFORMFEEDBACKPROC', 'PFNGLDELETETRANSFORMFEEDBACKSPROC',
'PFNGLGENTRANSFORMFEEDBACKSPROC', 'PFNGLISTRANSFORMFEEDBACKPROC',
'PFNGLPAUSETRANSFORMFEEDBACKPROC', 'PFNGLRESUMETRANSFORMFEEDBACKPROC',
'PFNGLDRAWTRANSFORMFEEDBACKPROC', 'GL_ARB_transform_feedback3',
'glDrawTransformFeedbackStream', 'glBeginQueryIndexed', 'glEndQueryIndexed',
'glGetQueryIndexediv', 'PFNGLDRAWTRANSFORMFEEDBACKSTREAMPROC',
'PFNGLBEGINQUERYINDEXEDPROC', 'PFNGLENDQUERYINDEXEDPROC',
'PFNGLGETQUERYINDEXEDIVPROC', 'GL_ARB_ES2_compatibility',
'glReleaseShaderCompiler', 'glShaderBinary', 'glGetShaderPrecisionFormat',
'glDepthRangef', 'glClearDepthf', 'PFNGLRELEASESHADERCOMPILERPROC',
'PFNGLSHADERBINARYPROC', 'PFNGLGETSHADERPRECISIONFORMATPROC',
'PFNGLDEPTHRANGEFPROC', 'PFNGLCLEARDEPTHFPROC', 'GL_ARB_get_program_binary',
'glGetProgramBinary', 'glProgramBinary', 'glProgramParameteri',
'PFNGLGETPROGRAMBINARYPROC', 'PFNGLPROGRAMBINARYPROC',
'PFNGLPROGRAMPARAMETERIPROC', 'GL_ARB_separate_shader_objects',
'glUseProgramStages', 'glActiveShaderProgram', 'glCreateShaderProgramv',
'glBindProgramPipeline', 'glDeleteProgramPipelines', 'glGenProgramPipelines',
'glIsProgramPipeline', 'glGetProgramPipelineiv', 'glProgramUniform1i',
'glProgramUniform1iv', 'glProgramUniform1f', 'glProgramUniform1fv',
'glProgramUniform1d', 'glProgramUniform1dv', 'glProgramUniform1ui',
'glProgramUniform1uiv', 'glProgramUniform2i', 'glProgramUniform2iv',
'glProgramUniform2f', 'glProgramUniform2fv', 'glProgramUniform2d',
'glProgramUniform2dv', 'glProgramUniform2ui', 'glProgramUniform2uiv',
'glProgramUniform3i', 'glProgramUniform3iv', 'glProgramUniform3f',
'glProgramUniform3fv', 'glProgramUniform3d', 'glProgramUniform3dv',
'glProgramUniform3ui', 'glProgramUniform3uiv', 'glProgramUniform4i',
'glProgramUniform4iv', 'glProgramUniform4f', 'glProgramUniform4fv',
'glProgramUniform4d', 'glProgramUniform4dv', 'glProgramUniform4ui',
'glProgramUniform4uiv', 'glProgramUniformMatrix2fv',
'glProgramUniformMatrix3fv', 'glProgramUniformMatrix4fv',
'glProgramUniformMatrix2dv', 'glProgramUniformMatrix3dv',
'glProgramUniformMatrix4dv', 'glProgramUniformMatrix2x3fv',
'glProgramUniformMatrix3x2fv', 'glProgramUniformMatrix2x4fv',
'glProgramUniformMatrix4x2fv', 'glProgramUniformMatrix3x4fv',
'glProgramUniformMatrix4x3fv', 'glProgramUniformMatrix2x3dv',
'glProgramUniformMatrix3x2dv', 'glProgramUniformMatrix2x4dv',
'glProgramUniformMatrix4x2dv', 'glProgramUniformMatrix3x4dv',
'glProgramUniformMatrix4x3dv', 'glValidateProgramPipeline',
'glGetProgramPipelineInfoLog', 'PFNGLUSEPROGRAMSTAGESPROC',
'PFNGLACTIVESHADERPROGRAMPROC', 'PFNGLCREATESHADERPROGRAMVPROC',
'PFNGLBINDPROGRAMPIPELINEPROC', 'PFNGLDELETEPROGRAMPIPELINESPROC',
'PFNGLGENPROGRAMPIPELINESPROC', 'PFNGLISPROGRAMPIPELINEPROC',
'PFNGLGETPROGRAMPIPELINEIVPROC', 'PFNGLPROGRAMUNIFORM1IPROC',
'PFNGLPROGRAMUNIFORM1IVPROC', 'PFNGLPROGRAMUNIFORM1FPROC',
'PFNGLPROGRAMUNIFORM1FVPROC', 'PFNGLPROGRAMUNIFORM1DPROC',
'PFNGLPROGRAMUNIFORM1DVPROC', 'PFNGLPROGRAMUNIFORM1UIPROC',
'PFNGLPROGRAMUNIFORM1UIVPROC', 'PFNGLPROGRAMUNIFORM2IPROC',
'PFNGLPROGRAMUNIFORM2IVPROC', 'PFNGLPROGRAMUNIFORM2FPROC',
'PFNGLPROGRAMUNIFORM2FVPROC', 'PFNGLPROGRAMUNIFORM2DPROC',
'PFNGLPROGRAMUNIFORM2DVPROC', 'PFNGLPROGRAMUNIFORM2UIPROC',
'PFNGLPROGRAMUNIFORM2UIVPROC', 'PFNGLPROGRAMUNIFORM3IPROC',
'PFNGLPROGRAMUNIFORM3IVPROC', 'PFNGLPROGRAMUNIFORM3FPROC',
'PFNGLPROGRAMUNIFORM3FVPROC', 'PFNGLPROGRAMUNIFORM3DPROC',
'PFNGLPROGRAMUNIFORM3DVPROC', 'PFNGLPROGRAMUNIFORM3UIPROC',
'PFNGLPROGRAMUNIFORM3UIVPROC', 'PFNGLPROGRAMUNIFORM4IPROC',
'PFNGLPROGRAMUNIFORM4IVPROC', 'PFNGLPROGRAMUNIFORM4FPROC',
'PFNGLPROGRAMUNIFORM4FVPROC', 'PFNGLPROGRAMUNIFORM4DPROC',
'PFNGLPROGRAMUNIFORM4DVPROC', 'PFNGLPROGRAMUNIFORM4UIPROC',
'PFNGLPROGRAMUNIFORM4UIVPROC', 'PFNGLPROGRAMUNIFORMMATRIX2FVPROC',
'PFNGLPROGRAMUNIFORMMATRIX3FVPROC', 'PFNGLPROGRAMUNIFORMMATRIX4FVPROC',
'PFNGLPROGRAMUNIFORMMATRIX2DVPROC', 'PFNGLPROGRAMUNIFORMMATRIX3DVPROC',
'PFNGLPROGRAMUNIFORMMATRIX4DVPROC', 'PFNGLPROGRAMUNIFORMMATRIX2X3FVPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X2FVPROC', 'PFNGLPROGRAMUNIFORMMATRIX2X4FVPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X2FVPROC', 'PFNGLPROGRAMUNIFORMMATRIX3X4FVPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X3FVPROC', 'PFNGLPROGRAMUNIFORMMATRIX2X3DVPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X2DVPROC', 'PFNGLPROGRAMUNIFORMMATRIX2X4DVPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X2DVPROC', 'PFNGLPROGRAMUNIFORMMATRIX3X4DVPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X3DVPROC', 'PFNGLVALIDATEPROGRAMPIPELINEPROC',
'PFNGLGETPROGRAMPIPELINEINFOLOGPROC', 'GL_ARB_vertex_attrib_64bit',
'glVertexAttribL1d', 'glVertexAttribL2d', 'glVertexAttribL3d',
'glVertexAttribL4d', 'glVertexAttribL1dv', 'glVertexAttribL2dv',
'glVertexAttribL3dv', 'glVertexAttribL4dv', 'glVertexAttribLPointer',
'glGetVertexAttribLdv', 'PFNGLVERTEXATTRIBL1DPROC',
'PFNGLVERTEXATTRIBL2DPROC', 'PFNGLVERTEXATTRIBL3DPROC',
'PFNGLVERTEXATTRIBL4DPROC', 'PFNGLVERTEXATTRIBL1DVPROC',
'PFNGLVERTEXATTRIBL2DVPROC', 'PFNGLVERTEXATTRIBL3DVPROC',
'PFNGLVERTEXATTRIBL4DVPROC', 'PFNGLVERTEXATTRIBLPOINTERPROC',
'PFNGLGETVERTEXATTRIBLDVPROC', 'GL_ARB_viewport_array', 'glViewportArrayv',
'glViewportIndexedf', 'glViewportIndexedfv', 'glScissorArrayv',
'glScissorIndexed', 'glScissorIndexedv', 'glDepthRangeArrayv',
'glDepthRangeIndexed', 'glGetFloati_v', 'glGetDoublei_v',
'PFNGLVIEWPORTARRAYVPROC', 'PFNGLVIEWPORTINDEXEDFPROC',
'PFNGLVIEWPORTINDEXEDFVPROC', 'PFNGLSCISSORARRAYVPROC',
'PFNGLSCISSORINDEXEDPROC', 'PFNGLSCISSORINDEXEDVPROC',
'PFNGLDEPTHRANGEARRAYVPROC', 'PFNGLDEPTHRANGEINDEXEDPROC',
'PFNGLGETFLOATI_VPROC', 'PFNGLGETDOUBLEI_VPROC', 'GL_ARB_cl_event',
'glCreateSyncFromCLeventARB', 'PFNGLCREATESYNCFROMCLEVENTARBPROC',
'GL_ARB_debug_output', 'glDebugMessageControlARB', 'glDebugMessageInsertARB',
'glDebugMessageCallbackARB', 'glGetDebugMessageLogARB',
'PFNGLDEBUGMESSAGECONTROLARBPROC', 'PFNGLDEBUGMESSAGEINSERTARBPROC',
'PFNGLDEBUGMESSAGECALLBACKARBPROC', 'PFNGLGETDEBUGMESSAGELOGARBPROC',
'GL_ARB_robustness', 'glGetGraphicsResetStatusARB', 'glGetnMapdvARB',
'glGetnMapfvARB', 'glGetnMapivARB', 'glGetnPixelMapfvARB',
'glGetnPixelMapuivARB', 'glGetnPixelMapusvARB', 'glGetnPolygonStippleARB',
'glGetnColorTableARB', 'glGetnConvolutionFilterARB',
'glGetnSeparableFilterARB', 'glGetnHistogramARB', 'glGetnMinmaxARB',
'glGetnTexImageARB', 'glReadnPixelsARB', 'glGetnCompressedTexImageARB',
'glGetnUniformfvARB', 'glGetnUniformivARB', 'glGetnUniformuivARB',
'glGetnUniformdvARB', 'PFNGLGETGRAPHICSRESETSTATUSARBPROC',
'PFNGLGETNMAPDVARBPROC', 'PFNGLGETNMAPFVARBPROC', 'PFNGLGETNMAPIVARBPROC',
'PFNGLGETNPIXELMAPFVARBPROC', 'PFNGLGETNPIXELMAPUIVARBPROC',
'PFNGLGETNPIXELMAPUSVARBPROC', 'PFNGLGETNPOLYGONSTIPPLEARBPROC',
'PFNGLGETNCOLORTABLEARBPROC', 'PFNGLGETNCONVOLUTIONFILTERARBPROC',
'PFNGLGETNSEPARABLEFILTERARBPROC', 'PFNGLGETNHISTOGRAMARBPROC',
'PFNGLGETNMINMAXARBPROC', 'PFNGLGETNTEXIMAGEARBPROC',
'PFNGLREADNPIXELSARBPROC', 'PFNGLGETNCOMPRESSEDTEXIMAGEARBPROC',
'PFNGLGETNUNIFORMFVARBPROC', 'PFNGLGETNUNIFORMIVARBPROC',
'PFNGLGETNUNIFORMUIVARBPROC', 'PFNGLGETNUNIFORMDVARBPROC',
'GL_ARB_shader_stencil_export', 'GL_ARB_base_instance',
'glDrawArraysInstancedBaseInstance', 'glDrawElementsInstancedBaseInstance',
'glDrawElementsInstancedBaseVertexBaseInstance',
'PFNGLDRAWARRAYSINSTANCEDBASEINSTANCEPROC',
'PFNGLDRAWELEMENTSINSTANCEDBASEINSTANCEPROC',
'PFNGLDRAWELEMENTSINSTANCEDBASEVERTEXBASEINSTANCEPROC',
'GL_ARB_shading_language_420pack', 'GL_ARB_transform_feedback_instanced',
'glDrawTransformFeedbackInstanced', 'glDrawTransformFeedbackStreamInstanced',
'PFNGLDRAWTRANSFORMFEEDBACKINSTANCEDPROC',
'PFNGLDRAWTRANSFORMFEEDBACKSTREAMINSTANCEDPROC',
'GL_ARB_compressed_texture_pixel_storage', 'GL_ARB_conservative_depth',
'GL_ARB_internalformat_query', 'glGetInternalformativ',
'PFNGLGETINTERNALFORMATIVPROC', 'GL_ARB_map_buffer_alignment',
'GL_ARB_shader_atomic_counters', 'glGetActiveAtomicCounterBufferiv',
'PFNGLGETACTIVEATOMICCOUNTERBUFFERIVPROC', 'GL_ARB_shader_image_load_store',
'glBindImageTexture', 'glMemoryBarrier', 'PFNGLBINDIMAGETEXTUREPROC',
'PFNGLMEMORYBARRIERPROC', 'GL_ARB_shading_language_packing',
'GL_ARB_texture_storage', 'glTexStorage1D', 'glTexStorage2D',
'glTexStorage3D', 'glTextureStorage1DEXT', 'glTextureStorage2DEXT',
'glTextureStorage3DEXT', 'PFNGLTEXSTORAGE1DPROC', 'PFNGLTEXSTORAGE2DPROC',
'PFNGLTEXSTORAGE3DPROC', 'PFNGLTEXTURESTORAGE1DEXTPROC',
'PFNGLTEXTURESTORAGE2DEXTPROC', 'PFNGLTEXTURESTORAGE3DEXTPROC', 'GL_EXT_abgr',
'GL_EXT_blend_color', 'glBlendColorEXT', 'PFNGLBLENDCOLOREXTPROC',
'GL_EXT_polygon_offset', 'glPolygonOffsetEXT', 'PFNGLPOLYGONOFFSETEXTPROC',
'GL_EXT_texture', 'GL_EXT_texture3D', 'glTexImage3DEXT', 'glTexSubImage3DEXT',
'PFNGLTEXIMAGE3DEXTPROC', 'PFNGLTEXSUBIMAGE3DEXTPROC',
'GL_SGIS_texture_filter4', 'glGetTexFilterFuncSGIS', 'glTexFilterFuncSGIS',
'PFNGLGETTEXFILTERFUNCSGISPROC', 'PFNGLTEXFILTERFUNCSGISPROC',
'GL_EXT_subtexture', 'glTexSubImage1DEXT', 'glTexSubImage2DEXT',
'PFNGLTEXSUBIMAGE1DEXTPROC', 'PFNGLTEXSUBIMAGE2DEXTPROC',
'GL_EXT_copy_texture', 'glCopyTexImage1DEXT', 'glCopyTexImage2DEXT',
'glCopyTexSubImage1DEXT', 'glCopyTexSubImage2DEXT', 'glCopyTexSubImage3DEXT',
'PFNGLCOPYTEXIMAGE1DEXTPROC', 'PFNGLCOPYTEXIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE1DEXTPROC', 'PFNGLCOPYTEXSUBIMAGE2DEXTPROC',
'PFNGLCOPYTEXSUBIMAGE3DEXTPROC', 'GL_EXT_histogram', 'glGetHistogramEXT',
'glGetHistogramParameterfvEXT', 'glGetHistogramParameterivEXT',
'glGetMinmaxEXT', 'glGetMinmaxParameterfvEXT', 'glGetMinmaxParameterivEXT',
'glHistogramEXT', 'glMinmaxEXT', 'glResetHistogramEXT', 'glResetMinmaxEXT',
'PFNGLGETHISTOGRAMEXTPROC', 'PFNGLGETHISTOGRAMPARAMETERFVEXTPROC',
'PFNGLGETHISTOGRAMPARAMETERIVEXTPROC', 'PFNGLGETMINMAXEXTPROC',
'PFNGLGETMINMAXPARAMETERFVEXTPROC', 'PFNGLGETMINMAXPARAMETERIVEXTPROC',
'PFNGLHISTOGRAMEXTPROC', 'PFNGLMINMAXEXTPROC', 'PFNGLRESETHISTOGRAMEXTPROC',
'PFNGLRESETMINMAXEXTPROC', 'GL_EXT_convolution', 'glConvolutionFilter1DEXT',
'glConvolutionFilter2DEXT', 'glConvolutionParameterfEXT',
'glConvolutionParameterfvEXT', 'glConvolutionParameteriEXT',
'glConvolutionParameterivEXT', 'glCopyConvolutionFilter1DEXT',
'glCopyConvolutionFilter2DEXT', 'glGetConvolutionFilterEXT',
'glGetConvolutionParameterfvEXT', 'glGetConvolutionParameterivEXT',
'glGetSeparableFilterEXT', 'glSeparableFilter2DEXT',
'PFNGLCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCONVOLUTIONFILTER2DEXTPROC',
'PFNGLCONVOLUTIONPARAMETERFEXTPROC', 'PFNGLCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLCONVOLUTIONPARAMETERIEXTPROC', 'PFNGLCONVOLUTIONPARAMETERIVEXTPROC',
'PFNGLCOPYCONVOLUTIONFILTER1DEXTPROC', 'PFNGLCOPYCONVOLUTIONFILTER2DEXTPROC',
'PFNGLGETCONVOLUTIONFILTEREXTPROC', 'PFNGLGETCONVOLUTIONPARAMETERFVEXTPROC',
'PFNGLGETCONVOLUTIONPARAMETERIVEXTPROC', 'PFNGLGETSEPARABLEFILTEREXTPROC',
'PFNGLSEPARABLEFILTER2DEXTPROC', 'GL_SGI_color_matrix', 'GL_SGI_color_table',
'glColorTableSGI', 'glColorTableParameterfvSGI', 'glColorTableParameterivSGI',
'glCopyColorTableSGI', 'glGetColorTableSGI', 'glGetColorTableParameterfvSGI',
'glGetColorTableParameterivSGI', 'PFNGLCOLORTABLESGIPROC',
'PFNGLCOLORTABLEPARAMETERFVSGIPROC', 'PFNGLCOLORTABLEPARAMETERIVSGIPROC',
'PFNGLCOPYCOLORTABLESGIPROC', 'PFNGLGETCOLORTABLESGIPROC',
'PFNGLGETCOLORTABLEPARAMETERFVSGIPROC',
'PFNGLGETCOLORTABLEPARAMETERIVSGIPROC', 'GL_SGIX_pixel_texture',
'glPixelTexGenSGIX', 'PFNGLPIXELTEXGENSGIXPROC', 'GL_SGIS_pixel_texture',
'glPixelTexGenParameteriSGIS', 'glPixelTexGenParameterivSGIS',
'glPixelTexGenParameterfSGIS', 'glPixelTexGenParameterfvSGIS',
'glGetPixelTexGenParameterivSGIS', 'glGetPixelTexGenParameterfvSGIS',
'PFNGLPIXELTEXGENPARAMETERISGISPROC', 'PFNGLPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLPIXELTEXGENPARAMETERFSGISPROC', 'PFNGLPIXELTEXGENPARAMETERFVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERIVSGISPROC',
'PFNGLGETPIXELTEXGENPARAMETERFVSGISPROC', 'GL_SGIS_texture4D',
'glTexImage4DSGIS', 'glTexSubImage4DSGIS', 'PFNGLTEXIMAGE4DSGISPROC',
'PFNGLTEXSUBIMAGE4DSGISPROC', 'GL_SGI_texture_color_table', 'GL_EXT_cmyka',
'GL_EXT_texture_object', 'glAreTexturesResidentEXT', 'glBindTextureEXT',
'glDeleteTexturesEXT', 'glGenTexturesEXT', 'glIsTextureEXT',
'glPrioritizeTexturesEXT', 'PFNGLARETEXTURESRESIDENTEXTPROC',
'PFNGLBINDTEXTUREEXTPROC', 'PFNGLDELETETEXTURESEXTPROC',
'PFNGLGENTEXTURESEXTPROC', 'PFNGLISTEXTUREEXTPROC',
'PFNGLPRIORITIZETEXTURESEXTPROC', 'GL_SGIS_detail_texture',
'glDetailTexFuncSGIS', 'glGetDetailTexFuncSGIS', 'PFNGLDETAILTEXFUNCSGISPROC',
'PFNGLGETDETAILTEXFUNCSGISPROC', 'GL_SGIS_sharpen_texture',
'glSharpenTexFuncSGIS', 'glGetSharpenTexFuncSGIS',
'PFNGLSHARPENTEXFUNCSGISPROC', 'PFNGLGETSHARPENTEXFUNCSGISPROC',
'GL_EXT_packed_pixels', 'GL_SGIS_texture_lod', 'GL_SGIS_multisample',
'glSampleMaskSGIS', 'glSamplePatternSGIS', 'PFNGLSAMPLEMASKSGISPROC',
'PFNGLSAMPLEPATTERNSGISPROC', 'GL_EXT_rescale_normal', 'GL_EXT_vertex_array',
'glArrayElementEXT', 'glColorPointerEXT', 'glDrawArraysEXT',
'glEdgeFlagPointerEXT', 'glGetPointervEXT', 'glIndexPointerEXT',
'glNormalPointerEXT', 'glTexCoordPointerEXT', 'glVertexPointerEXT',
'PFNGLARRAYELEMENTEXTPROC', 'PFNGLCOLORPOINTEREXTPROC',
'PFNGLDRAWARRAYSEXTPROC', 'PFNGLEDGEFLAGPOINTEREXTPROC',
'PFNGLGETPOINTERVEXTPROC', 'PFNGLINDEXPOINTEREXTPROC',
'PFNGLNORMALPOINTEREXTPROC', 'PFNGLTEXCOORDPOINTEREXTPROC',
'PFNGLVERTEXPOINTEREXTPROC', 'GL_EXT_misc_attribute',
'GL_SGIS_generate_mipmap', 'GL_SGIX_clipmap', 'GL_SGIX_shadow',
'GL_SGIS_texture_edge_clamp', 'GL_SGIS_texture_border_clamp',
'GL_EXT_blend_minmax', 'glBlendEquationEXT', 'PFNGLBLENDEQUATIONEXTPROC',
'GL_EXT_blend_subtract', 'GL_EXT_blend_logic_op', 'GL_SGIX_interlace',
'GL_SGIX_pixel_tiles', 'GL_SGIX_texture_select', 'GL_SGIX_sprite',
'glSpriteParameterfSGIX', 'glSpriteParameterfvSGIX', 'glSpriteParameteriSGIX',
'glSpriteParameterivSGIX', 'PFNGLSPRITEPARAMETERFSGIXPROC',
'PFNGLSPRITEPARAMETERFVSGIXPROC', 'PFNGLSPRITEPARAMETERISGIXPROC',
'PFNGLSPRITEPARAMETERIVSGIXPROC', 'GL_SGIX_texture_multi_buffer',
'GL_EXT_point_parameters', 'glPointParameterfEXT', 'glPointParameterfvEXT',
'PFNGLPOINTPARAMETERFEXTPROC', 'PFNGLPOINTPARAMETERFVEXTPROC',
'GL_SGIS_point_parameters', 'glPointParameterfSGIS', 'glPointParameterfvSGIS',
'PFNGLPOINTPARAMETERFSGISPROC', 'PFNGLPOINTPARAMETERFVSGISPROC',
'GL_SGIX_instruments', 'glGetInstrumentsSGIX', 'glInstrumentsBufferSGIX',
'glPollInstrumentsSGIX', 'glReadInstrumentsSGIX', 'glStartInstrumentsSGIX',
'glStopInstrumentsSGIX', 'PFNGLGETINSTRUMENTSSGIXPROC',
'PFNGLINSTRUMENTSBUFFERSGIXPROC', 'PFNGLPOLLINSTRUMENTSSGIXPROC',
'PFNGLREADINSTRUMENTSSGIXPROC', 'PFNGLSTARTINSTRUMENTSSGIXPROC',
'PFNGLSTOPINSTRUMENTSSGIXPROC', 'GL_SGIX_texture_scale_bias',
'GL_SGIX_framezoom', 'glFrameZoomSGIX', 'PFNGLFRAMEZOOMSGIXPROC',
'GL_SGIX_tag_sample_buffer', 'glTagSampleBufferSGIX',
'PFNGLTAGSAMPLEBUFFERSGIXPROC', 'GL_SGIX_polynomial_ffd',
'glDeformationMap3dSGIX', 'glDeformationMap3fSGIX', 'glDeformSGIX',
'glLoadIdentityDeformationMapSGIX', 'PFNGLDEFORMATIONMAP3DSGIXPROC',
'PFNGLDEFORMATIONMAP3FSGIXPROC', 'PFNGLDEFORMSGIXPROC',
'PFNGLLOADIDENTITYDEFORMATIONMAPSGIXPROC', 'GL_SGIX_reference_plane',
'glReferencePlaneSGIX', 'PFNGLREFERENCEPLANESGIXPROC', 'GL_SGIX_flush_raster',
'glFlushRasterSGIX', 'PFNGLFLUSHRASTERSGIXPROC', 'GL_SGIX_depth_texture',
'GL_SGIS_fog_function', 'glFogFuncSGIS', 'glGetFogFuncSGIS',
'PFNGLFOGFUNCSGISPROC', 'PFNGLGETFOGFUNCSGISPROC', 'GL_SGIX_fog_offset',
'GL_HP_image_transform', 'glImageTransformParameteriHP',
'glImageTransformParameterfHP', 'glImageTransformParameterivHP',
'glImageTransformParameterfvHP', 'glGetImageTransformParameterivHP',
'glGetImageTransformParameterfvHP', 'PFNGLIMAGETRANSFORMPARAMETERIHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFHPPROC', 'PFNGLIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLIMAGETRANSFORMPARAMETERFVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERIVHPPROC',
'PFNGLGETIMAGETRANSFORMPARAMETERFVHPPROC', 'GL_HP_convolution_border_modes',
'GL_SGIX_texture_add_env', 'GL_EXT_color_subtable', 'glColorSubTableEXT',
'glCopyColorSubTableEXT', 'PFNGLCOLORSUBTABLEEXTPROC',
'PFNGLCOPYCOLORSUBTABLEEXTPROC', 'GL_PGI_vertex_hints', 'GL_PGI_misc_hints',
'glHintPGI', 'PFNGLHINTPGIPROC', 'GL_EXT_paletted_texture', 'glColorTableEXT',
'glGetColorTableEXT', 'glGetColorTableParameterivEXT',
'glGetColorTableParameterfvEXT', 'PFNGLCOLORTABLEEXTPROC',
'PFNGLGETCOLORTABLEEXTPROC', 'PFNGLGETCOLORTABLEPARAMETERIVEXTPROC',
'PFNGLGETCOLORTABLEPARAMETERFVEXTPROC', 'GL_EXT_clip_volume_hint',
'GL_SGIX_list_priority', 'glGetListParameterfvSGIX',
'glGetListParameterivSGIX', 'glListParameterfSGIX', 'glListParameterfvSGIX',
'glListParameteriSGIX', 'glListParameterivSGIX',
'PFNGLGETLISTPARAMETERFVSGIXPROC', 'PFNGLGETLISTPARAMETERIVSGIXPROC',
'PFNGLLISTPARAMETERFSGIXPROC', 'PFNGLLISTPARAMETERFVSGIXPROC',
'PFNGLLISTPARAMETERISGIXPROC', 'PFNGLLISTPARAMETERIVSGIXPROC',
'GL_SGIX_ir_instrument1', 'GL_SGIX_calligraphic_fragment',
'GL_SGIX_texture_lod_bias', 'GL_SGIX_shadow_ambient', 'GL_EXT_index_texture',
'GL_EXT_index_material', 'glIndexMaterialEXT', 'PFNGLINDEXMATERIALEXTPROC',
'GL_EXT_index_func', 'glIndexFuncEXT', 'PFNGLINDEXFUNCEXTPROC',
'GL_EXT_index_array_formats', 'GL_EXT_compiled_vertex_array',
'glLockArraysEXT', 'glUnlockArraysEXT', 'PFNGLLOCKARRAYSEXTPROC',
'PFNGLUNLOCKARRAYSEXTPROC', 'GL_EXT_cull_vertex', 'glCullParameterdvEXT',
'glCullParameterfvEXT', 'PFNGLCULLPARAMETERDVEXTPROC',
'PFNGLCULLPARAMETERFVEXTPROC', 'GL_SGIX_ycrcb', 'GL_SGIX_fragment_lighting',
'glFragmentColorMaterialSGIX', 'glFragmentLightfSGIX',
'glFragmentLightfvSGIX', 'glFragmentLightiSGIX', 'glFragmentLightivSGIX',
'glFragmentLightModelfSGIX', 'glFragmentLightModelfvSGIX',
'glFragmentLightModeliSGIX', 'glFragmentLightModelivSGIX',
'glFragmentMaterialfSGIX', 'glFragmentMaterialfvSGIX',
'glFragmentMaterialiSGIX', 'glFragmentMaterialivSGIX',
'glGetFragmentLightfvSGIX', 'glGetFragmentLightivSGIX',
'glGetFragmentMaterialfvSGIX', 'glGetFragmentMaterialivSGIX',
'glLightEnviSGIX', 'PFNGLFRAGMENTCOLORMATERIALSGIXPROC',
'PFNGLFRAGMENTLIGHTFSGIXPROC', 'PFNGLFRAGMENTLIGHTFVSGIXPROC',
'PFNGLFRAGMENTLIGHTISGIXPROC', 'PFNGLFRAGMENTLIGHTIVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELFSGIXPROC', 'PFNGLFRAGMENTLIGHTMODELFVSGIXPROC',
'PFNGLFRAGMENTLIGHTMODELISGIXPROC', 'PFNGLFRAGMENTLIGHTMODELIVSGIXPROC',
'PFNGLFRAGMENTMATERIALFSGIXPROC', 'PFNGLFRAGMENTMATERIALFVSGIXPROC',
'PFNGLFRAGMENTMATERIALISGIXPROC', 'PFNGLFRAGMENTMATERIALIVSGIXPROC',
'PFNGLGETFRAGMENTLIGHTFVSGIXPROC', 'PFNGLGETFRAGMENTLIGHTIVSGIXPROC',
'PFNGLGETFRAGMENTMATERIALFVSGIXPROC', 'PFNGLGETFRAGMENTMATERIALIVSGIXPROC',
'PFNGLLIGHTENVISGIXPROC', 'GL_IBM_rasterpos_clip', 'GL_HP_texture_lighting',
'GL_EXT_draw_range_elements', 'glDrawRangeElementsEXT',
'PFNGLDRAWRANGEELEMENTSEXTPROC', 'GL_WIN_phong_shading',
'GL_WIN_specular_fog', 'GL_EXT_light_texture', 'glApplyTextureEXT',
'glTextureLightEXT', 'glTextureMaterialEXT', 'PFNGLAPPLYTEXTUREEXTPROC',
'PFNGLTEXTURELIGHTEXTPROC', 'PFNGLTEXTUREMATERIALEXTPROC',
'GL_SGIX_blend_alpha_minmax', 'GL_EXT_bgra', 'GL_SGIX_async',
'glAsyncMarkerSGIX', 'glFinishAsyncSGIX', 'glPollAsyncSGIX',
'glGenAsyncMarkersSGIX', 'glDeleteAsyncMarkersSGIX', 'glIsAsyncMarkerSGIX',
'PFNGLASYNCMARKERSGIXPROC', 'PFNGLFINISHASYNCSGIXPROC',
'PFNGLPOLLASYNCSGIXPROC', 'PFNGLGENASYNCMARKERSSGIXPROC',
'PFNGLDELETEASYNCMARKERSSGIXPROC', 'PFNGLISASYNCMARKERSGIXPROC',
'GL_SGIX_async_pixel', 'GL_SGIX_async_histogram', 'GL_INTEL_parallel_arrays',
'glVertexPointervINTEL', 'glNormalPointervINTEL', 'glColorPointervINTEL',
'glTexCoordPointervINTEL', 'PFNGLVERTEXPOINTERVINTELPROC',
'PFNGLNORMALPOINTERVINTELPROC', 'PFNGLCOLORPOINTERVINTELPROC',
'PFNGLTEXCOORDPOINTERVINTELPROC', 'GL_HP_occlusion_test',
'GL_EXT_pixel_transform', 'glPixelTransformParameteriEXT',
'glPixelTransformParameterfEXT', 'glPixelTransformParameterivEXT',
'glPixelTransformParameterfvEXT', 'PFNGLPIXELTRANSFORMPARAMETERIEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERIVEXTPROC',
'PFNGLPIXELTRANSFORMPARAMETERFVEXTPROC', 'GL_EXT_pixel_transform_color_table',
'GL_EXT_shared_texture_palette', 'GL_EXT_separate_specular_color',
'GL_EXT_secondary_color', 'glSecondaryColor3bEXT', 'glSecondaryColor3bvEXT',
'glSecondaryColor3dEXT', 'glSecondaryColor3dvEXT', 'glSecondaryColor3fEXT',
'glSecondaryColor3fvEXT', 'glSecondaryColor3iEXT', 'glSecondaryColor3ivEXT',
'glSecondaryColor3sEXT', 'glSecondaryColor3svEXT', 'glSecondaryColor3ubEXT',
'glSecondaryColor3ubvEXT', 'glSecondaryColor3uiEXT',
'glSecondaryColor3uivEXT', 'glSecondaryColor3usEXT',
'glSecondaryColor3usvEXT', 'glSecondaryColorPointerEXT',
'PFNGLSECONDARYCOLOR3BEXTPROC', 'PFNGLSECONDARYCOLOR3BVEXTPROC',
'PFNGLSECONDARYCOLOR3DEXTPROC', 'PFNGLSECONDARYCOLOR3DVEXTPROC',
'PFNGLSECONDARYCOLOR3FEXTPROC', 'PFNGLSECONDARYCOLOR3FVEXTPROC',
'PFNGLSECONDARYCOLOR3IEXTPROC', 'PFNGLSECONDARYCOLOR3IVEXTPROC',
'PFNGLSECONDARYCOLOR3SEXTPROC', 'PFNGLSECONDARYCOLOR3SVEXTPROC',
'PFNGLSECONDARYCOLOR3UBEXTPROC', 'PFNGLSECONDARYCOLOR3UBVEXTPROC',
'PFNGLSECONDARYCOLOR3UIEXTPROC', 'PFNGLSECONDARYCOLOR3UIVEXTPROC',
'PFNGLSECONDARYCOLOR3USEXTPROC', 'PFNGLSECONDARYCOLOR3USVEXTPROC',
'PFNGLSECONDARYCOLORPOINTEREXTPROC', 'GL_EXT_texture_perturb_normal',
'glTextureNormalEXT', 'PFNGLTEXTURENORMALEXTPROC', 'GL_EXT_multi_draw_arrays',
'glMultiDrawArraysEXT', 'glMultiDrawElementsEXT',
'PFNGLMULTIDRAWARRAYSEXTPROC', 'PFNGLMULTIDRAWELEMENTSEXTPROC',
'GL_EXT_fog_coord', 'glFogCoordfEXT', 'glFogCoordfvEXT', 'glFogCoorddEXT',
'glFogCoorddvEXT', 'glFogCoordPointerEXT', 'PFNGLFOGCOORDFEXTPROC',
'PFNGLFOGCOORDFVEXTPROC', 'PFNGLFOGCOORDDEXTPROC', 'PFNGLFOGCOORDDVEXTPROC',
'PFNGLFOGCOORDPOINTEREXTPROC', 'GL_REND_screen_coordinates',
'GL_EXT_coordinate_frame', 'glTangent3bEXT', 'glTangent3bvEXT',
'glTangent3dEXT', 'glTangent3dvEXT', 'glTangent3fEXT', 'glTangent3fvEXT',
'glTangent3iEXT', 'glTangent3ivEXT', 'glTangent3sEXT', 'glTangent3svEXT',
'glBinormal3bEXT', 'glBinormal3bvEXT', 'glBinormal3dEXT', 'glBinormal3dvEXT',
'glBinormal3fEXT', 'glBinormal3fvEXT', 'glBinormal3iEXT', 'glBinormal3ivEXT',
'glBinormal3sEXT', 'glBinormal3svEXT', 'glTangentPointerEXT',
'glBinormalPointerEXT', 'PFNGLTANGENT3BEXTPROC', 'PFNGLTANGENT3BVEXTPROC',
'PFNGLTANGENT3DEXTPROC', 'PFNGLTANGENT3DVEXTPROC', 'PFNGLTANGENT3FEXTPROC',
'PFNGLTANGENT3FVEXTPROC', 'PFNGLTANGENT3IEXTPROC', 'PFNGLTANGENT3IVEXTPROC',
'PFNGLTANGENT3SEXTPROC', 'PFNGLTANGENT3SVEXTPROC', 'PFNGLBINORMAL3BEXTPROC',
'PFNGLBINORMAL3BVEXTPROC', 'PFNGLBINORMAL3DEXTPROC',
'PFNGLBINORMAL3DVEXTPROC', 'PFNGLBINORMAL3FEXTPROC',
'PFNGLBINORMAL3FVEXTPROC', 'PFNGLBINORMAL3IEXTPROC',
'PFNGLBINORMAL3IVEXTPROC', 'PFNGLBINORMAL3SEXTPROC',
'PFNGLBINORMAL3SVEXTPROC', 'PFNGLTANGENTPOINTEREXTPROC',
'PFNGLBINORMALPOINTEREXTPROC', 'GL_EXT_texture_env_combine',
'GL_APPLE_specular_vector', 'GL_APPLE_transform_hint', 'GL_SGIX_fog_scale',
'GL_SUNX_constant_data', 'glFinishTextureSUNX', 'PFNGLFINISHTEXTURESUNXPROC',
'GL_SUN_global_alpha', 'glGlobalAlphaFactorbSUN', 'glGlobalAlphaFactorsSUN',
'glGlobalAlphaFactoriSUN', 'glGlobalAlphaFactorfSUN',
'glGlobalAlphaFactordSUN', 'glGlobalAlphaFactorubSUN',
'glGlobalAlphaFactorusSUN', 'glGlobalAlphaFactoruiSUN',
'PFNGLGLOBALALPHAFACTORBSUNPROC', 'PFNGLGLOBALALPHAFACTORSSUNPROC',
'PFNGLGLOBALALPHAFACTORISUNPROC', 'PFNGLGLOBALALPHAFACTORFSUNPROC',
'PFNGLGLOBALALPHAFACTORDSUNPROC', 'PFNGLGLOBALALPHAFACTORUBSUNPROC',
'PFNGLGLOBALALPHAFACTORUSSUNPROC', 'PFNGLGLOBALALPHAFACTORUISUNPROC',
'GL_SUN_triangle_list', 'glReplacementCodeuiSUN', 'glReplacementCodeusSUN',
'glReplacementCodeubSUN', 'glReplacementCodeuivSUN',
'glReplacementCodeusvSUN', 'glReplacementCodeubvSUN',
'glReplacementCodePointerSUN', 'PFNGLREPLACEMENTCODEUISUNPROC',
'PFNGLREPLACEMENTCODEUSSUNPROC', 'PFNGLREPLACEMENTCODEUBSUNPROC',
'PFNGLREPLACEMENTCODEUIVSUNPROC', 'PFNGLREPLACEMENTCODEUSVSUNPROC',
'PFNGLREPLACEMENTCODEUBVSUNPROC', 'PFNGLREPLACEMENTCODEPOINTERSUNPROC',
'GL_SUN_vertex', 'glColor4ubVertex2fSUN', 'glColor4ubVertex2fvSUN',
'glColor4ubVertex3fSUN', 'glColor4ubVertex3fvSUN', 'glColor3fVertex3fSUN',
'glColor3fVertex3fvSUN', 'glNormal3fVertex3fSUN', 'glNormal3fVertex3fvSUN',
'glColor4fNormal3fVertex3fSUN', 'glColor4fNormal3fVertex3fvSUN',
'glTexCoord2fVertex3fSUN', 'glTexCoord2fVertex3fvSUN',
'glTexCoord4fVertex4fSUN', 'glTexCoord4fVertex4fvSUN',
'glTexCoord2fColor4ubVertex3fSUN', 'glTexCoord2fColor4ubVertex3fvSUN',
'glTexCoord2fColor3fVertex3fSUN', 'glTexCoord2fColor3fVertex3fvSUN',
'glTexCoord2fNormal3fVertex3fSUN', 'glTexCoord2fNormal3fVertex3fvSUN',
'glTexCoord2fColor4fNormal3fVertex3fSUN',
'glTexCoord2fColor4fNormal3fVertex3fvSUN',
'glTexCoord4fColor4fNormal3fVertex4fSUN',
'glTexCoord4fColor4fNormal3fVertex4fvSUN', 'glReplacementCodeuiVertex3fSUN',
'glReplacementCodeuiVertex3fvSUN', 'glReplacementCodeuiColor4ubVertex3fSUN',
'glReplacementCodeuiColor4ubVertex3fvSUN',
'glReplacementCodeuiColor3fVertex3fSUN',
'glReplacementCodeuiColor3fVertex3fvSUN',
'glReplacementCodeuiNormal3fVertex3fSUN',
'glReplacementCodeuiNormal3fVertex3fvSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiColor4fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fVertex3fSUN',
'glReplacementCodeuiTexCoord2fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN',
'glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN',
'PFNGLCOLOR4UBVERTEX2FSUNPROC', 'PFNGLCOLOR4UBVERTEX2FVSUNPROC',
'PFNGLCOLOR4UBVERTEX3FSUNPROC', 'PFNGLCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLCOLOR3FVERTEX3FSUNPROC', 'PFNGLCOLOR3FVERTEX3FVSUNPROC',
'PFNGLNORMAL3FVERTEX3FSUNPROC', 'PFNGLNORMAL3FVERTEX3FVSUNPROC',
'PFNGLCOLOR4FNORMAL3FVERTEX3FSUNPROC', 'PFNGLCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FVERTEX3FSUNPROC', 'PFNGLTEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FVERTEX4FSUNPROC', 'PFNGLTEXCOORD4FVERTEX4FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4UBVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLTEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FSUNPROC',
'PFNGLTEXCOORD4FCOLOR4FNORMAL3FVERTEX4FVSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUIVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4UBVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUINORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUICOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FNORMAL3FVERTEX3FVSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FSUNPROC',
'PFNGLREPLACEMENTCODEUITEXCOORD2FCOLOR4FNORMAL3FVERTEX3FVSUNPROC',
'GL_EXT_blend_func_separate', 'glBlendFuncSeparateEXT',
'PFNGLBLENDFUNCSEPARATEEXTPROC', 'GL_INGR_blend_func_separate',
'glBlendFuncSeparateINGR', 'PFNGLBLENDFUNCSEPARATEINGRPROC',
'GL_INGR_color_clamp', 'GL_INGR_interlace_read', 'GL_EXT_stencil_wrap',
'GL_EXT_422_pixels', 'GL_NV_texgen_reflection',
'GL_SUN_convolution_border_modes', 'GL_EXT_texture_env_add',
'GL_EXT_texture_lod_bias', 'GL_EXT_texture_filter_anisotropic',
'GL_EXT_vertex_weighting', 'glVertexWeightfEXT', 'glVertexWeightfvEXT',
'glVertexWeightPointerEXT', 'PFNGLVERTEXWEIGHTFEXTPROC',
'PFNGLVERTEXWEIGHTFVEXTPROC', 'PFNGLVERTEXWEIGHTPOINTEREXTPROC',
'GL_NV_light_max_exponent', 'GL_NV_vertex_array_range',
'glFlushVertexArrayRangeNV', 'glVertexArrayRangeNV',
'PFNGLFLUSHVERTEXARRAYRANGENVPROC', 'PFNGLVERTEXARRAYRANGENVPROC',
'GL_NV_register_combiners', 'glCombinerParameterfvNV',
'glCombinerParameterfNV', 'glCombinerParameterivNV', 'glCombinerParameteriNV',
'glCombinerInputNV', 'glCombinerOutputNV', 'glFinalCombinerInputNV',
'glGetCombinerInputParameterfvNV', 'glGetCombinerInputParameterivNV',
'glGetCombinerOutputParameterfvNV', 'glGetCombinerOutputParameterivNV',
'glGetFinalCombinerInputParameterfvNV',
'glGetFinalCombinerInputParameterivNV', 'PFNGLCOMBINERPARAMETERFVNVPROC',
'PFNGLCOMBINERPARAMETERFNVPROC', 'PFNGLCOMBINERPARAMETERIVNVPROC',
'PFNGLCOMBINERPARAMETERINVPROC', 'PFNGLCOMBINERINPUTNVPROC',
'PFNGLCOMBINEROUTPUTNVPROC', 'PFNGLFINALCOMBINERINPUTNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC',
'PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC',
'PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC', 'GL_NV_fog_distance',
'GL_NV_texgen_emboss', 'GL_NV_blend_square', 'GL_NV_texture_env_combine4',
'GL_MESA_resize_buffers', 'glResizeBuffersMESA', 'PFNGLRESIZEBUFFERSMESAPROC',
'GL_MESA_window_pos', 'glWindowPos2dMESA', 'glWindowPos2dvMESA',
'glWindowPos2fMESA', 'glWindowPos2fvMESA', 'glWindowPos2iMESA',
'glWindowPos2ivMESA', 'glWindowPos2sMESA', 'glWindowPos2svMESA',
'glWindowPos3dMESA', 'glWindowPos3dvMESA', 'glWindowPos3fMESA',
'glWindowPos3fvMESA', 'glWindowPos3iMESA', 'glWindowPos3ivMESA',
'glWindowPos3sMESA', 'glWindowPos3svMESA', 'glWindowPos4dMESA',
'glWindowPos4dvMESA', 'glWindowPos4fMESA', 'glWindowPos4fvMESA',
'glWindowPos4iMESA', 'glWindowPos4ivMESA', 'glWindowPos4sMESA',
'glWindowPos4svMESA', 'PFNGLWINDOWPOS2DMESAPROC', 'PFNGLWINDOWPOS2DVMESAPROC',
'PFNGLWINDOWPOS2FMESAPROC', 'PFNGLWINDOWPOS2FVMESAPROC',
'PFNGLWINDOWPOS2IMESAPROC', 'PFNGLWINDOWPOS2IVMESAPROC',
'PFNGLWINDOWPOS2SMESAPROC', 'PFNGLWINDOWPOS2SVMESAPROC',
'PFNGLWINDOWPOS3DMESAPROC', 'PFNGLWINDOWPOS3DVMESAPROC',
'PFNGLWINDOWPOS3FMESAPROC', 'PFNGLWINDOWPOS3FVMESAPROC',
'PFNGLWINDOWPOS3IMESAPROC', 'PFNGLWINDOWPOS3IVMESAPROC',
'PFNGLWINDOWPOS3SMESAPROC', 'PFNGLWINDOWPOS3SVMESAPROC',
'PFNGLWINDOWPOS4DMESAPROC', 'PFNGLWINDOWPOS4DVMESAPROC',
'PFNGLWINDOWPOS4FMESAPROC', 'PFNGLWINDOWPOS4FVMESAPROC',
'PFNGLWINDOWPOS4IMESAPROC', 'PFNGLWINDOWPOS4IVMESAPROC',
'PFNGLWINDOWPOS4SMESAPROC', 'PFNGLWINDOWPOS4SVMESAPROC', 'GL_IBM_cull_vertex',
'GL_IBM_multimode_draw_arrays', 'glMultiModeDrawArraysIBM',
'glMultiModeDrawElementsIBM', 'PFNGLMULTIMODEDRAWARRAYSIBMPROC',
'PFNGLMULTIMODEDRAWELEMENTSIBMPROC', 'GL_IBM_vertex_array_lists',
'glColorPointerListIBM', 'glSecondaryColorPointerListIBM',
'glEdgeFlagPointerListIBM', 'glFogCoordPointerListIBM',
'glIndexPointerListIBM', 'glNormalPointerListIBM', 'glTexCoordPointerListIBM',
'glVertexPointerListIBM', 'PFNGLCOLORPOINTERLISTIBMPROC',
'PFNGLSECONDARYCOLORPOINTERLISTIBMPROC', 'PFNGLEDGEFLAGPOINTERLISTIBMPROC',
'PFNGLFOGCOORDPOINTERLISTIBMPROC', 'PFNGLINDEXPOINTERLISTIBMPROC',
'PFNGLNORMALPOINTERLISTIBMPROC', 'PFNGLTEXCOORDPOINTERLISTIBMPROC',
'PFNGLVERTEXPOINTERLISTIBMPROC', 'GL_SGIX_subsample', 'GL_SGIX_ycrcba',
'GL_SGIX_ycrcb_subsample', 'GL_SGIX_depth_pass_instrument',
'GL_3DFX_texture_compression_FXT1', 'GL_3DFX_multisample', 'GL_3DFX_tbuffer',
'glTbufferMask3DFX', 'PFNGLTBUFFERMASK3DFXPROC', 'GL_EXT_multisample',
'glSampleMaskEXT', 'glSamplePatternEXT', 'PFNGLSAMPLEMASKEXTPROC',
'PFNGLSAMPLEPATTERNEXTPROC', 'GL_SGIX_vertex_preclip',
'GL_SGIX_convolution_accuracy', 'GL_SGIX_resample',
'GL_SGIS_point_line_texgen', 'GL_SGIS_texture_color_mask',
'glTextureColorMaskSGIS', 'PFNGLTEXTURECOLORMASKSGISPROC',
'GL_SGIX_igloo_interface', 'glIglooInterfaceSGIX',
'PFNGLIGLOOINTERFACESGIXPROC', 'GL_EXT_texture_env_dot3',
'GL_ATI_texture_mirror_once', 'GL_NV_fence', 'glDeleteFencesNV',
'glGenFencesNV', 'glIsFenceNV', 'glTestFenceNV', 'glGetFenceivNV',
'glFinishFenceNV', 'glSetFenceNV', 'PFNGLDELETEFENCESNVPROC',
'PFNGLGENFENCESNVPROC', 'PFNGLISFENCENVPROC', 'PFNGLTESTFENCENVPROC',
'PFNGLGETFENCEIVNVPROC', 'PFNGLFINISHFENCENVPROC', 'PFNGLSETFENCENVPROC',
'GL_NV_evaluators', 'glMapControlPointsNV', 'glMapParameterivNV',
'glMapParameterfvNV', 'glGetMapControlPointsNV', 'glGetMapParameterivNV',
'glGetMapParameterfvNV', 'glGetMapAttribParameterivNV',
'glGetMapAttribParameterfvNV', 'glEvalMapsNV', 'PFNGLMAPCONTROLPOINTSNVPROC',
'PFNGLMAPPARAMETERIVNVPROC', 'PFNGLMAPPARAMETERFVNVPROC',
'PFNGLGETMAPCONTROLPOINTSNVPROC', 'PFNGLGETMAPPARAMETERIVNVPROC',
'PFNGLGETMAPPARAMETERFVNVPROC', 'PFNGLGETMAPATTRIBPARAMETERIVNVPROC',
'PFNGLGETMAPATTRIBPARAMETERFVNVPROC', 'PFNGLEVALMAPSNVPROC',
'GL_NV_packed_depth_stencil', 'GL_NV_register_combiners2',
'glCombinerStageParameterfvNV', 'glGetCombinerStageParameterfvNV',
'PFNGLCOMBINERSTAGEPARAMETERFVNVPROC',
'PFNGLGETCOMBINERSTAGEPARAMETERFVNVPROC', 'GL_NV_texture_compression_vtc',
'GL_NV_texture_rectangle', 'GL_NV_texture_shader', 'GL_NV_texture_shader2',
'GL_NV_vertex_array_range2', 'GL_NV_vertex_program',
'glAreProgramsResidentNV', 'glBindProgramNV', 'glDeleteProgramsNV',
'glExecuteProgramNV', 'glGenProgramsNV', 'glGetProgramParameterdvNV',
'glGetProgramParameterfvNV', 'glGetProgramivNV', 'glGetProgramStringNV',
'glGetTrackMatrixivNV', 'glGetVertexAttribdvNV', 'glGetVertexAttribfvNV',
'glGetVertexAttribivNV', 'glGetVertexAttribPointervNV', 'glIsProgramNV',
'glLoadProgramNV', 'glProgramParameter4dNV', 'glProgramParameter4dvNV',
'glProgramParameter4fNV', 'glProgramParameter4fvNV',
'glProgramParameters4dvNV', 'glProgramParameters4fvNV',
'glRequestResidentProgramsNV', 'glTrackMatrixNV', 'glVertexAttribPointerNV',
'glVertexAttrib1dNV', 'glVertexAttrib1dvNV', 'glVertexAttrib1fNV',
'glVertexAttrib1fvNV', 'glVertexAttrib1sNV', 'glVertexAttrib1svNV',
'glVertexAttrib2dNV', 'glVertexAttrib2dvNV', 'glVertexAttrib2fNV',
'glVertexAttrib2fvNV', 'glVertexAttrib2sNV', 'glVertexAttrib2svNV',
'glVertexAttrib3dNV', 'glVertexAttrib3dvNV', 'glVertexAttrib3fNV',
'glVertexAttrib3fvNV', 'glVertexAttrib3sNV', 'glVertexAttrib3svNV',
'glVertexAttrib4dNV', 'glVertexAttrib4dvNV', 'glVertexAttrib4fNV',
'glVertexAttrib4fvNV', 'glVertexAttrib4sNV', 'glVertexAttrib4svNV',
'glVertexAttrib4ubNV', 'glVertexAttrib4ubvNV', 'glVertexAttribs1dvNV',
'glVertexAttribs1fvNV', 'glVertexAttribs1svNV', 'glVertexAttribs2dvNV',
'glVertexAttribs2fvNV', 'glVertexAttribs2svNV', 'glVertexAttribs3dvNV',
'glVertexAttribs3fvNV', 'glVertexAttribs3svNV', 'glVertexAttribs4dvNV',
'glVertexAttribs4fvNV', 'glVertexAttribs4svNV', 'glVertexAttribs4ubvNV',
'PFNGLAREPROGRAMSRESIDENTNVPROC', 'PFNGLBINDPROGRAMNVPROC',
'PFNGLDELETEPROGRAMSNVPROC', 'PFNGLEXECUTEPROGRAMNVPROC',
'PFNGLGENPROGRAMSNVPROC', 'PFNGLGETPROGRAMPARAMETERDVNVPROC',
'PFNGLGETPROGRAMPARAMETERFVNVPROC', 'PFNGLGETPROGRAMIVNVPROC',
'PFNGLGETPROGRAMSTRINGNVPROC', 'PFNGLGETTRACKMATRIXIVNVPROC',
'PFNGLGETVERTEXATTRIBDVNVPROC', 'PFNGLGETVERTEXATTRIBFVNVPROC',
'PFNGLGETVERTEXATTRIBIVNVPROC', 'PFNGLGETVERTEXATTRIBPOINTERVNVPROC',
'PFNGLISPROGRAMNVPROC', 'PFNGLLOADPROGRAMNVPROC',
'PFNGLPROGRAMPARAMETER4DNVPROC', 'PFNGLPROGRAMPARAMETER4DVNVPROC',
'PFNGLPROGRAMPARAMETER4FNVPROC', 'PFNGLPROGRAMPARAMETER4FVNVPROC',
'PFNGLPROGRAMPARAMETERS4DVNVPROC', 'PFNGLPROGRAMPARAMETERS4FVNVPROC',
'PFNGLREQUESTRESIDENTPROGRAMSNVPROC', 'PFNGLTRACKMATRIXNVPROC',
'PFNGLVERTEXATTRIBPOINTERNVPROC', 'PFNGLVERTEXATTRIB1DNVPROC',
'PFNGLVERTEXATTRIB1DVNVPROC', 'PFNGLVERTEXATTRIB1FNVPROC',
'PFNGLVERTEXATTRIB1FVNVPROC', 'PFNGLVERTEXATTRIB1SNVPROC',
'PFNGLVERTEXATTRIB1SVNVPROC', 'PFNGLVERTEXATTRIB2DNVPROC',
'PFNGLVERTEXATTRIB2DVNVPROC', 'PFNGLVERTEXATTRIB2FNVPROC',
'PFNGLVERTEXATTRIB2FVNVPROC', 'PFNGLVERTEXATTRIB2SNVPROC',
'PFNGLVERTEXATTRIB2SVNVPROC', 'PFNGLVERTEXATTRIB3DNVPROC',
'PFNGLVERTEXATTRIB3DVNVPROC', 'PFNGLVERTEXATTRIB3FNVPROC',
'PFNGLVERTEXATTRIB3FVNVPROC', 'PFNGLVERTEXATTRIB3SNVPROC',
'PFNGLVERTEXATTRIB3SVNVPROC', 'PFNGLVERTEXATTRIB4DNVPROC',
'PFNGLVERTEXATTRIB4DVNVPROC', 'PFNGLVERTEXATTRIB4FNVPROC',
'PFNGLVERTEXATTRIB4FVNVPROC', 'PFNGLVERTEXATTRIB4SNVPROC',
'PFNGLVERTEXATTRIB4SVNVPROC', 'PFNGLVERTEXATTRIB4UBNVPROC',
'PFNGLVERTEXATTRIB4UBVNVPROC', 'PFNGLVERTEXATTRIBS1DVNVPROC',
'PFNGLVERTEXATTRIBS1FVNVPROC', 'PFNGLVERTEXATTRIBS1SVNVPROC',
'PFNGLVERTEXATTRIBS2DVNVPROC', 'PFNGLVERTEXATTRIBS2FVNVPROC',
'PFNGLVERTEXATTRIBS2SVNVPROC', 'PFNGLVERTEXATTRIBS3DVNVPROC',
'PFNGLVERTEXATTRIBS3FVNVPROC', 'PFNGLVERTEXATTRIBS3SVNVPROC',
'PFNGLVERTEXATTRIBS4DVNVPROC', 'PFNGLVERTEXATTRIBS4FVNVPROC',
'PFNGLVERTEXATTRIBS4SVNVPROC', 'PFNGLVERTEXATTRIBS4UBVNVPROC',
'GL_SGIX_texture_coordinate_clamp', 'GL_SGIX_scalebias_hint',
'GL_OML_interlace', 'GL_OML_subsample', 'GL_OML_resample',
'GL_NV_copy_depth_to_color', 'GL_ATI_envmap_bumpmap',
'glTexBumpParameterivATI', 'glTexBumpParameterfvATI',
'glGetTexBumpParameterivATI', 'glGetTexBumpParameterfvATI',
'PFNGLTEXBUMPPARAMETERIVATIPROC', 'PFNGLTEXBUMPPARAMETERFVATIPROC',
'PFNGLGETTEXBUMPPARAMETERIVATIPROC', 'PFNGLGETTEXBUMPPARAMETERFVATIPROC',
'GL_ATI_fragment_shader', 'glGenFragmentShadersATI',
'glBindFragmentShaderATI', 'glDeleteFragmentShaderATI',
'glBeginFragmentShaderATI', 'glEndFragmentShaderATI', 'glPassTexCoordATI',
'glSampleMapATI', 'glColorFragmentOp1ATI', 'glColorFragmentOp2ATI',
'glColorFragmentOp3ATI', 'glAlphaFragmentOp1ATI', 'glAlphaFragmentOp2ATI',
'glAlphaFragmentOp3ATI', 'glSetFragmentShaderConstantATI',
'PFNGLGENFRAGMENTSHADERSATIPROC', 'PFNGLBINDFRAGMENTSHADERATIPROC',
'PFNGLDELETEFRAGMENTSHADERATIPROC', 'PFNGLBEGINFRAGMENTSHADERATIPROC',
'PFNGLENDFRAGMENTSHADERATIPROC', 'PFNGLPASSTEXCOORDATIPROC',
'PFNGLSAMPLEMAPATIPROC', 'PFNGLCOLORFRAGMENTOP1ATIPROC',
'PFNGLCOLORFRAGMENTOP2ATIPROC', 'PFNGLCOLORFRAGMENTOP3ATIPROC',
'PFNGLALPHAFRAGMENTOP1ATIPROC', 'PFNGLALPHAFRAGMENTOP2ATIPROC',
'PFNGLALPHAFRAGMENTOP3ATIPROC', 'PFNGLSETFRAGMENTSHADERCONSTANTATIPROC',
'GL_ATI_pn_triangles', 'glPNTrianglesiATI', 'glPNTrianglesfATI',
'PFNGLPNTRIANGLESIATIPROC', 'PFNGLPNTRIANGLESFATIPROC',
'GL_ATI_vertex_array_object', 'glNewObjectBufferATI', 'glIsObjectBufferATI',
'glUpdateObjectBufferATI', 'glGetObjectBufferfvATI', 'glGetObjectBufferivATI',
'glFreeObjectBufferATI', 'glArrayObjectATI', 'glGetArrayObjectfvATI',
'glGetArrayObjectivATI', 'glVariantArrayObjectATI',
'glGetVariantArrayObjectfvATI', 'glGetVariantArrayObjectivATI',
'PFNGLNEWOBJECTBUFFERATIPROC', 'PFNGLISOBJECTBUFFERATIPROC',
'PFNGLUPDATEOBJECTBUFFERATIPROC', 'PFNGLGETOBJECTBUFFERFVATIPROC',
'PFNGLGETOBJECTBUFFERIVATIPROC', 'PFNGLFREEOBJECTBUFFERATIPROC',
'PFNGLARRAYOBJECTATIPROC', 'PFNGLGETARRAYOBJECTFVATIPROC',
'PFNGLGETARRAYOBJECTIVATIPROC', 'PFNGLVARIANTARRAYOBJECTATIPROC',
'PFNGLGETVARIANTARRAYOBJECTFVATIPROC', 'PFNGLGETVARIANTARRAYOBJECTIVATIPROC',
'GL_EXT_vertex_shader', 'glBeginVertexShaderEXT', 'glEndVertexShaderEXT',
'glBindVertexShaderEXT', 'glGenVertexShadersEXT', 'glDeleteVertexShaderEXT',
'glShaderOp1EXT', 'glShaderOp2EXT', 'glShaderOp3EXT', 'glSwizzleEXT',
'glWriteMaskEXT', 'glInsertComponentEXT', 'glExtractComponentEXT',
'glGenSymbolsEXT', 'glSetInvariantEXT', 'glSetLocalConstantEXT',
'glVariantbvEXT', 'glVariantsvEXT', 'glVariantivEXT', 'glVariantfvEXT',
'glVariantdvEXT', 'glVariantubvEXT', 'glVariantusvEXT', 'glVariantuivEXT',
'glVariantPointerEXT', 'glEnableVariantClientStateEXT',
'glDisableVariantClientStateEXT', 'glBindLightParameterEXT',
'glBindMaterialParameterEXT', 'glBindTexGenParameterEXT',
'glBindTextureUnitParameterEXT', 'glBindParameterEXT',
'glIsVariantEnabledEXT', 'glGetVariantBooleanvEXT', 'glGetVariantIntegervEXT',
'glGetVariantFloatvEXT', 'glGetVariantPointervEXT',
'glGetInvariantBooleanvEXT', 'glGetInvariantIntegervEXT',
'glGetInvariantFloatvEXT', 'glGetLocalConstantBooleanvEXT',
'glGetLocalConstantIntegervEXT', 'glGetLocalConstantFloatvEXT',
'PFNGLBEGINVERTEXSHADEREXTPROC', 'PFNGLENDVERTEXSHADEREXTPROC',
'PFNGLBINDVERTEXSHADEREXTPROC', 'PFNGLGENVERTEXSHADERSEXTPROC',
'PFNGLDELETEVERTEXSHADEREXTPROC', 'PFNGLSHADEROP1EXTPROC',
'PFNGLSHADEROP2EXTPROC', 'PFNGLSHADEROP3EXTPROC', 'PFNGLSWIZZLEEXTPROC',
'PFNGLWRITEMASKEXTPROC', 'PFNGLINSERTCOMPONENTEXTPROC',
'PFNGLEXTRACTCOMPONENTEXTPROC', 'PFNGLGENSYMBOLSEXTPROC',
'PFNGLSETINVARIANTEXTPROC', 'PFNGLSETLOCALCONSTANTEXTPROC',
'PFNGLVARIANTBVEXTPROC', 'PFNGLVARIANTSVEXTPROC', 'PFNGLVARIANTIVEXTPROC',
'PFNGLVARIANTFVEXTPROC', 'PFNGLVARIANTDVEXTPROC', 'PFNGLVARIANTUBVEXTPROC',
'PFNGLVARIANTUSVEXTPROC', 'PFNGLVARIANTUIVEXTPROC',
'PFNGLVARIANTPOINTEREXTPROC', 'PFNGLENABLEVARIANTCLIENTSTATEEXTPROC',
'PFNGLDISABLEVARIANTCLIENTSTATEEXTPROC', 'PFNGLBINDLIGHTPARAMETEREXTPROC',
'PFNGLBINDMATERIALPARAMETEREXTPROC', 'PFNGLBINDTEXGENPARAMETEREXTPROC',
'PFNGLBINDTEXTUREUNITPARAMETEREXTPROC', 'PFNGLBINDPARAMETEREXTPROC',
'PFNGLISVARIANTENABLEDEXTPROC', 'PFNGLGETVARIANTBOOLEANVEXTPROC',
'PFNGLGETVARIANTINTEGERVEXTPROC', 'PFNGLGETVARIANTFLOATVEXTPROC',
'PFNGLGETVARIANTPOINTERVEXTPROC', 'PFNGLGETINVARIANTBOOLEANVEXTPROC',
'PFNGLGETINVARIANTINTEGERVEXTPROC', 'PFNGLGETINVARIANTFLOATVEXTPROC',
'PFNGLGETLOCALCONSTANTBOOLEANVEXTPROC',
'PFNGLGETLOCALCONSTANTINTEGERVEXTPROC', 'PFNGLGETLOCALCONSTANTFLOATVEXTPROC',
'GL_ATI_vertex_streams', 'glVertexStream1sATI', 'glVertexStream1svATI',
'glVertexStream1iATI', 'glVertexStream1ivATI', 'glVertexStream1fATI',
'glVertexStream1fvATI', 'glVertexStream1dATI', 'glVertexStream1dvATI',
'glVertexStream2sATI', 'glVertexStream2svATI', 'glVertexStream2iATI',
'glVertexStream2ivATI', 'glVertexStream2fATI', 'glVertexStream2fvATI',
'glVertexStream2dATI', 'glVertexStream2dvATI', 'glVertexStream3sATI',
'glVertexStream3svATI', 'glVertexStream3iATI', 'glVertexStream3ivATI',
'glVertexStream3fATI', 'glVertexStream3fvATI', 'glVertexStream3dATI',
'glVertexStream3dvATI', 'glVertexStream4sATI', 'glVertexStream4svATI',
'glVertexStream4iATI', 'glVertexStream4ivATI', 'glVertexStream4fATI',
'glVertexStream4fvATI', 'glVertexStream4dATI', 'glVertexStream4dvATI',
'glNormalStream3bATI', 'glNormalStream3bvATI', 'glNormalStream3sATI',
'glNormalStream3svATI', 'glNormalStream3iATI', 'glNormalStream3ivATI',
'glNormalStream3fATI', 'glNormalStream3fvATI', 'glNormalStream3dATI',
'glNormalStream3dvATI', 'glClientActiveVertexStreamATI',
'glVertexBlendEnviATI', 'glVertexBlendEnvfATI', 'PFNGLVERTEXSTREAM1SATIPROC',
'PFNGLVERTEXSTREAM1SVATIPROC', 'PFNGLVERTEXSTREAM1IATIPROC',
'PFNGLVERTEXSTREAM1IVATIPROC', 'PFNGLVERTEXSTREAM1FATIPROC',
'PFNGLVERTEXSTREAM1FVATIPROC', 'PFNGLVERTEXSTREAM1DATIPROC',
'PFNGLVERTEXSTREAM1DVATIPROC', 'PFNGLVERTEXSTREAM2SATIPROC',
'PFNGLVERTEXSTREAM2SVATIPROC', 'PFNGLVERTEXSTREAM2IATIPROC',
'PFNGLVERTEXSTREAM2IVATIPROC', 'PFNGLVERTEXSTREAM2FATIPROC',
'PFNGLVERTEXSTREAM2FVATIPROC', 'PFNGLVERTEXSTREAM2DATIPROC',
'PFNGLVERTEXSTREAM2DVATIPROC', 'PFNGLVERTEXSTREAM3SATIPROC',
'PFNGLVERTEXSTREAM3SVATIPROC', 'PFNGLVERTEXSTREAM3IATIPROC',
'PFNGLVERTEXSTREAM3IVATIPROC', 'PFNGLVERTEXSTREAM3FATIPROC',
'PFNGLVERTEXSTREAM3FVATIPROC', 'PFNGLVERTEXSTREAM3DATIPROC',
'PFNGLVERTEXSTREAM3DVATIPROC', 'PFNGLVERTEXSTREAM4SATIPROC',
'PFNGLVERTEXSTREAM4SVATIPROC', 'PFNGLVERTEXSTREAM4IATIPROC',
'PFNGLVERTEXSTREAM4IVATIPROC', 'PFNGLVERTEXSTREAM4FATIPROC',
'PFNGLVERTEXSTREAM4FVATIPROC', 'PFNGLVERTEXSTREAM4DATIPROC',
'PFNGLVERTEXSTREAM4DVATIPROC', 'PFNGLNORMALSTREAM3BATIPROC',
'PFNGLNORMALSTREAM3BVATIPROC', 'PFNGLNORMALSTREAM3SATIPROC',
'PFNGLNORMALSTREAM3SVATIPROC', 'PFNGLNORMALSTREAM3IATIPROC',
'PFNGLNORMALSTREAM3IVATIPROC', 'PFNGLNORMALSTREAM3FATIPROC',
'PFNGLNORMALSTREAM3FVATIPROC', 'PFNGLNORMALSTREAM3DATIPROC',
'PFNGLNORMALSTREAM3DVATIPROC', 'PFNGLCLIENTACTIVEVERTEXSTREAMATIPROC',
'PFNGLVERTEXBLENDENVIATIPROC', 'PFNGLVERTEXBLENDENVFATIPROC',
'GL_ATI_element_array', 'glElementPointerATI', 'glDrawElementArrayATI',
'glDrawRangeElementArrayATI', 'PFNGLELEMENTPOINTERATIPROC',
'PFNGLDRAWELEMENTARRAYATIPROC', 'PFNGLDRAWRANGEELEMENTARRAYATIPROC',
'GL_SUN_mesh_array', 'glDrawMeshArraysSUN', 'PFNGLDRAWMESHARRAYSSUNPROC',
'GL_SUN_slice_accum', 'GL_NV_multisample_filter_hint', 'GL_NV_depth_clamp',
'GL_NV_occlusion_query', 'glGenOcclusionQueriesNV',
'glDeleteOcclusionQueriesNV', 'glIsOcclusionQueryNV',
'glBeginOcclusionQueryNV', 'glEndOcclusionQueryNV', 'glGetOcclusionQueryivNV',
'glGetOcclusionQueryuivNV', 'PFNGLGENOCCLUSIONQUERIESNVPROC',
'PFNGLDELETEOCCLUSIONQUERIESNVPROC', 'PFNGLISOCCLUSIONQUERYNVPROC',
'PFNGLBEGINOCCLUSIONQUERYNVPROC', 'PFNGLENDOCCLUSIONQUERYNVPROC',
'PFNGLGETOCCLUSIONQUERYIVNVPROC', 'PFNGLGETOCCLUSIONQUERYUIVNVPROC',
'GL_NV_point_sprite', 'glPointParameteriNV', 'glPointParameterivNV',
'PFNGLPOINTPARAMETERINVPROC', 'PFNGLPOINTPARAMETERIVNVPROC',
'GL_NV_texture_shader3', 'GL_NV_vertex_program1_1', 'GL_EXT_shadow_funcs',
'GL_EXT_stencil_two_side', 'glActiveStencilFaceEXT',
'PFNGLACTIVESTENCILFACEEXTPROC', 'GL_ATI_text_fragment_shader',
'GL_APPLE_client_storage', 'GL_APPLE_element_array', 'glElementPointerAPPLE',
'glDrawElementArrayAPPLE', 'glDrawRangeElementArrayAPPLE',
'glMultiDrawElementArrayAPPLE', 'glMultiDrawRangeElementArrayAPPLE',
'PFNGLELEMENTPOINTERAPPLEPROC', 'PFNGLDRAWELEMENTARRAYAPPLEPROC',
'PFNGLDRAWRANGEELEMENTARRAYAPPLEPROC', 'PFNGLMULTIDRAWELEMENTARRAYAPPLEPROC',
'PFNGLMULTIDRAWRANGEELEMENTARRAYAPPLEPROC', 'GL_APPLE_fence',
'glGenFencesAPPLE', 'glDeleteFencesAPPLE', 'glSetFenceAPPLE',
'glIsFenceAPPLE', 'glTestFenceAPPLE', 'glFinishFenceAPPLE',
'glTestObjectAPPLE', 'glFinishObjectAPPLE', 'PFNGLGENFENCESAPPLEPROC',
'PFNGLDELETEFENCESAPPLEPROC', 'PFNGLSETFENCEAPPLEPROC',
'PFNGLISFENCEAPPLEPROC', 'PFNGLTESTFENCEAPPLEPROC',
'PFNGLFINISHFENCEAPPLEPROC', 'PFNGLTESTOBJECTAPPLEPROC',
'PFNGLFINISHOBJECTAPPLEPROC', 'GL_APPLE_vertex_array_object',
'glBindVertexArrayAPPLE', 'glDeleteVertexArraysAPPLE',
'glGenVertexArraysAPPLE', 'glIsVertexArrayAPPLE',
'PFNGLBINDVERTEXARRAYAPPLEPROC', 'PFNGLDELETEVERTEXARRAYSAPPLEPROC',
'PFNGLGENVERTEXARRAYSAPPLEPROC', 'PFNGLISVERTEXARRAYAPPLEPROC',
'GL_APPLE_vertex_array_range', 'glVertexArrayRangeAPPLE',
'glFlushVertexArrayRangeAPPLE', 'glVertexArrayParameteriAPPLE',
'PFNGLVERTEXARRAYRANGEAPPLEPROC', 'PFNGLFLUSHVERTEXARRAYRANGEAPPLEPROC',
'PFNGLVERTEXARRAYPARAMETERIAPPLEPROC', 'GL_APPLE_ycbcr_422', 'GL_S3_s3tc',
'GL_ATI_draw_buffers', 'glDrawBuffersATI', 'PFNGLDRAWBUFFERSATIPROC',
'GL_ATI_pixel_format_float', 'GL_ATI_texture_env_combine3',
'GL_ATI_texture_float', 'GL_NV_float_buffer', 'GL_NV_fragment_program',
'glProgramNamedParameter4fNV', 'glProgramNamedParameter4dNV',
'glProgramNamedParameter4fvNV', 'glProgramNamedParameter4dvNV',
'glGetProgramNamedParameterfvNV', 'glGetProgramNamedParameterdvNV',
'PFNGLPROGRAMNAMEDPARAMETER4FNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DNVPROC',
'PFNGLPROGRAMNAMEDPARAMETER4FVNVPROC', 'PFNGLPROGRAMNAMEDPARAMETER4DVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERFVNVPROC',
'PFNGLGETPROGRAMNAMEDPARAMETERDVNVPROC', 'GL_NV_half_float', 'glVertex2hNV',
'glVertex2hvNV', 'glVertex3hNV', 'glVertex3hvNV', 'glVertex4hNV',
'glVertex4hvNV', 'glNormal3hNV', 'glNormal3hvNV', 'glColor3hNV',
'glColor3hvNV', 'glColor4hNV', 'glColor4hvNV', 'glTexCoord1hNV',
'glTexCoord1hvNV', 'glTexCoord2hNV', 'glTexCoord2hvNV', 'glTexCoord3hNV',
'glTexCoord3hvNV', 'glTexCoord4hNV', 'glTexCoord4hvNV', 'glMultiTexCoord1hNV',
'glMultiTexCoord1hvNV', 'glMultiTexCoord2hNV', 'glMultiTexCoord2hvNV',
'glMultiTexCoord3hNV', 'glMultiTexCoord3hvNV', 'glMultiTexCoord4hNV',
'glMultiTexCoord4hvNV', 'glFogCoordhNV', 'glFogCoordhvNV',
'glSecondaryColor3hNV', 'glSecondaryColor3hvNV', 'glVertexWeighthNV',
'glVertexWeighthvNV', 'glVertexAttrib1hNV', 'glVertexAttrib1hvNV',
'glVertexAttrib2hNV', 'glVertexAttrib2hvNV', 'glVertexAttrib3hNV',
'glVertexAttrib3hvNV', 'glVertexAttrib4hNV', 'glVertexAttrib4hvNV',
'glVertexAttribs1hvNV', 'glVertexAttribs2hvNV', 'glVertexAttribs3hvNV',
'glVertexAttribs4hvNV', 'PFNGLVERTEX2HNVPROC', 'PFNGLVERTEX2HVNVPROC',
'PFNGLVERTEX3HNVPROC', 'PFNGLVERTEX3HVNVPROC', 'PFNGLVERTEX4HNVPROC',
'PFNGLVERTEX4HVNVPROC', 'PFNGLNORMAL3HNVPROC', 'PFNGLNORMAL3HVNVPROC',
'PFNGLCOLOR3HNVPROC', 'PFNGLCOLOR3HVNVPROC', 'PFNGLCOLOR4HNVPROC',
'PFNGLCOLOR4HVNVPROC', 'PFNGLTEXCOORD1HNVPROC', 'PFNGLTEXCOORD1HVNVPROC',
'PFNGLTEXCOORD2HNVPROC', 'PFNGLTEXCOORD2HVNVPROC', 'PFNGLTEXCOORD3HNVPROC',
'PFNGLTEXCOORD3HVNVPROC', 'PFNGLTEXCOORD4HNVPROC', 'PFNGLTEXCOORD4HVNVPROC',
'PFNGLMULTITEXCOORD1HNVPROC', 'PFNGLMULTITEXCOORD1HVNVPROC',
'PFNGLMULTITEXCOORD2HNVPROC', 'PFNGLMULTITEXCOORD2HVNVPROC',
'PFNGLMULTITEXCOORD3HNVPROC', 'PFNGLMULTITEXCOORD3HVNVPROC',
'PFNGLMULTITEXCOORD4HNVPROC', 'PFNGLMULTITEXCOORD4HVNVPROC',
'PFNGLFOGCOORDHNVPROC', 'PFNGLFOGCOORDHVNVPROC',
'PFNGLSECONDARYCOLOR3HNVPROC', 'PFNGLSECONDARYCOLOR3HVNVPROC',
'PFNGLVERTEXWEIGHTHNVPROC', 'PFNGLVERTEXWEIGHTHVNVPROC',
'PFNGLVERTEXATTRIB1HNVPROC', 'PFNGLVERTEXATTRIB1HVNVPROC',
'PFNGLVERTEXATTRIB2HNVPROC', 'PFNGLVERTEXATTRIB2HVNVPROC',
'PFNGLVERTEXATTRIB3HNVPROC', 'PFNGLVERTEXATTRIB3HVNVPROC',
'PFNGLVERTEXATTRIB4HNVPROC', 'PFNGLVERTEXATTRIB4HVNVPROC',
'PFNGLVERTEXATTRIBS1HVNVPROC', 'PFNGLVERTEXATTRIBS2HVNVPROC',
'PFNGLVERTEXATTRIBS3HVNVPROC', 'PFNGLVERTEXATTRIBS4HVNVPROC',
'GL_NV_pixel_data_range', 'glPixelDataRangeNV', 'glFlushPixelDataRangeNV',
'PFNGLPIXELDATARANGENVPROC', 'PFNGLFLUSHPIXELDATARANGENVPROC',
'GL_NV_primitive_restart', 'glPrimitiveRestartNV',
'glPrimitiveRestartIndexNV', 'PFNGLPRIMITIVERESTARTNVPROC',
'PFNGLPRIMITIVERESTARTINDEXNVPROC', 'GL_NV_texture_expand_normal',
'GL_NV_vertex_program2', 'GL_ATI_map_object_buffer', 'glMapObjectBufferATI',
'glUnmapObjectBufferATI', 'PFNGLMAPOBJECTBUFFERATIPROC',
'PFNGLUNMAPOBJECTBUFFERATIPROC', 'GL_ATI_separate_stencil',
'glStencilOpSeparateATI', 'glStencilFuncSeparateATI',
'PFNGLSTENCILOPSEPARATEATIPROC', 'PFNGLSTENCILFUNCSEPARATEATIPROC',
'GL_ATI_vertex_attrib_array_object', 'glVertexAttribArrayObjectATI',
'glGetVertexAttribArrayObjectfvATI', 'glGetVertexAttribArrayObjectivATI',
'PFNGLVERTEXATTRIBARRAYOBJECTATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTFVATIPROC',
'PFNGLGETVERTEXATTRIBARRAYOBJECTIVATIPROC', 'GL_OES_read_format',
'GL_EXT_depth_bounds_test', 'glDepthBoundsEXT', 'PFNGLDEPTHBOUNDSEXTPROC',
'GL_EXT_texture_mirror_clamp', 'GL_EXT_blend_equation_separate',
'glBlendEquationSeparateEXT', 'PFNGLBLENDEQUATIONSEPARATEEXTPROC',
'GL_MESA_pack_invert', 'GL_MESA_ycbcr_texture', 'GL_EXT_pixel_buffer_object',
'GL_NV_fragment_program_option', 'GL_NV_fragment_program2',
'GL_NV_vertex_program2_option', 'GL_NV_vertex_program3',
'GL_EXT_framebuffer_object', 'glIsRenderbufferEXT', 'glBindRenderbufferEXT',
'glDeleteRenderbuffersEXT', 'glGenRenderbuffersEXT',
'glRenderbufferStorageEXT', 'glGetRenderbufferParameterivEXT',
'glIsFramebufferEXT', 'glBindFramebufferEXT', 'glDeleteFramebuffersEXT',
'glGenFramebuffersEXT', 'glCheckFramebufferStatusEXT',
'glFramebufferTexture1DEXT', 'glFramebufferTexture2DEXT',
'glFramebufferTexture3DEXT', 'glFramebufferRenderbufferEXT',
'glGetFramebufferAttachmentParameterivEXT', 'glGenerateMipmapEXT',
'PFNGLISRENDERBUFFEREXTPROC', 'PFNGLBINDRENDERBUFFEREXTPROC',
'PFNGLDELETERENDERBUFFERSEXTPROC', 'PFNGLGENRENDERBUFFERSEXTPROC',
'PFNGLRENDERBUFFERSTORAGEEXTPROC', 'PFNGLGETRENDERBUFFERPARAMETERIVEXTPROC',
'PFNGLISFRAMEBUFFEREXTPROC', 'PFNGLBINDFRAMEBUFFEREXTPROC',
'PFNGLDELETEFRAMEBUFFERSEXTPROC', 'PFNGLGENFRAMEBUFFERSEXTPROC',
'PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE1DEXTPROC',
'PFNGLFRAMEBUFFERTEXTURE2DEXTPROC', 'PFNGLFRAMEBUFFERTEXTURE3DEXTPROC',
'PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC',
'PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC',
'PFNGLGENERATEMIPMAPEXTPROC', 'GL_GREMEDY_string_marker',
'glStringMarkerGREMEDY', 'PFNGLSTRINGMARKERGREMEDYPROC',
'GL_EXT_packed_depth_stencil', 'GL_EXT_stencil_clear_tag',
'glStencilClearTagEXT', 'PFNGLSTENCILCLEARTAGEXTPROC', 'GL_EXT_texture_sRGB',
'GL_EXT_framebuffer_blit', 'glBlitFramebufferEXT',
'PFNGLBLITFRAMEBUFFEREXTPROC', 'GL_EXT_framebuffer_multisample',
'glRenderbufferStorageMultisampleEXT',
'PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC', 'GL_MESAX_texture_stack',
'GL_EXT_timer_query', 'glGetQueryObjecti64vEXT', 'glGetQueryObjectui64vEXT',
'PFNGLGETQUERYOBJECTI64VEXTPROC', 'PFNGLGETQUERYOBJECTUI64VEXTPROC',
'GL_EXT_gpu_program_parameters', 'glProgramEnvParameters4fvEXT',
'glProgramLocalParameters4fvEXT', 'PFNGLPROGRAMENVPARAMETERS4FVEXTPROC',
'PFNGLPROGRAMLOCALPARAMETERS4FVEXTPROC', 'GL_APPLE_flush_buffer_range',
'glBufferParameteriAPPLE', 'glFlushMappedBufferRangeAPPLE',
'PFNGLBUFFERPARAMETERIAPPLEPROC', 'PFNGLFLUSHMAPPEDBUFFERRANGEAPPLEPROC',
'GL_NV_gpu_program4', 'glProgramLocalParameterI4iNV',
'glProgramLocalParameterI4ivNV', 'glProgramLocalParametersI4ivNV',
'glProgramLocalParameterI4uiNV', 'glProgramLocalParameterI4uivNV',
'glProgramLocalParametersI4uivNV', 'glProgramEnvParameterI4iNV',
'glProgramEnvParameterI4ivNV', 'glProgramEnvParametersI4ivNV',
'glProgramEnvParameterI4uiNV', 'glProgramEnvParameterI4uivNV',
'glProgramEnvParametersI4uivNV', 'glGetProgramLocalParameterIivNV',
'glGetProgramLocalParameterIuivNV', 'glGetProgramEnvParameterIivNV',
'glGetProgramEnvParameterIuivNV', 'PFNGLPROGRAMLOCALPARAMETERI4INVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4IVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERSI4IVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4UINVPROC',
'PFNGLPROGRAMLOCALPARAMETERI4UIVNVPROC',
'PFNGLPROGRAMLOCALPARAMETERSI4UIVNVPROC', 'PFNGLPROGRAMENVPARAMETERI4INVPROC',
'PFNGLPROGRAMENVPARAMETERI4IVNVPROC', 'PFNGLPROGRAMENVPARAMETERSI4IVNVPROC',
'PFNGLPROGRAMENVPARAMETERI4UINVPROC', 'PFNGLPROGRAMENVPARAMETERI4UIVNVPROC',
'PFNGLPROGRAMENVPARAMETERSI4UIVNVPROC',
'PFNGLGETPROGRAMLOCALPARAMETERIIVNVPROC',
'PFNGLGETPROGRAMLOCALPARAMETERIUIVNVPROC',
'PFNGLGETPROGRAMENVPARAMETERIIVNVPROC',
'PFNGLGETPROGRAMENVPARAMETERIUIVNVPROC', 'GL_NV_geometry_program4',
'glProgramVertexLimitNV', 'glFramebufferTextureEXT',
'glFramebufferTextureLayerEXT', 'glFramebufferTextureFaceEXT',
'PFNGLPROGRAMVERTEXLIMITNVPROC', 'PFNGLFRAMEBUFFERTEXTUREEXTPROC',
'PFNGLFRAMEBUFFERTEXTURELAYEREXTPROC', 'PFNGLFRAMEBUFFERTEXTUREFACEEXTPROC',
'GL_EXT_geometry_shader4', 'glProgramParameteriEXT',
'PFNGLPROGRAMPARAMETERIEXTPROC', 'GL_NV_vertex_program4',
'glVertexAttribI1iEXT', 'glVertexAttribI2iEXT', 'glVertexAttribI3iEXT',
'glVertexAttribI4iEXT', 'glVertexAttribI1uiEXT', 'glVertexAttribI2uiEXT',
'glVertexAttribI3uiEXT', 'glVertexAttribI4uiEXT', 'glVertexAttribI1ivEXT',
'glVertexAttribI2ivEXT', 'glVertexAttribI3ivEXT', 'glVertexAttribI4ivEXT',
'glVertexAttribI1uivEXT', 'glVertexAttribI2uivEXT', 'glVertexAttribI3uivEXT',
'glVertexAttribI4uivEXT', 'glVertexAttribI4bvEXT', 'glVertexAttribI4svEXT',
'glVertexAttribI4ubvEXT', 'glVertexAttribI4usvEXT',
'glVertexAttribIPointerEXT', 'glGetVertexAttribIivEXT',
'glGetVertexAttribIuivEXT', 'PFNGLVERTEXATTRIBI1IEXTPROC',
'PFNGLVERTEXATTRIBI2IEXTPROC', 'PFNGLVERTEXATTRIBI3IEXTPROC',
'PFNGLVERTEXATTRIBI4IEXTPROC', 'PFNGLVERTEXATTRIBI1UIEXTPROC',
'PFNGLVERTEXATTRIBI2UIEXTPROC', 'PFNGLVERTEXATTRIBI3UIEXTPROC',
'PFNGLVERTEXATTRIBI4UIEXTPROC', 'PFNGLVERTEXATTRIBI1IVEXTPROC',
'PFNGLVERTEXATTRIBI2IVEXTPROC', 'PFNGLVERTEXATTRIBI3IVEXTPROC',
'PFNGLVERTEXATTRIBI4IVEXTPROC', 'PFNGLVERTEXATTRIBI1UIVEXTPROC',
'PFNGLVERTEXATTRIBI2UIVEXTPROC', 'PFNGLVERTEXATTRIBI3UIVEXTPROC',
'PFNGLVERTEXATTRIBI4UIVEXTPROC', 'PFNGLVERTEXATTRIBI4BVEXTPROC',
'PFNGLVERTEXATTRIBI4SVEXTPROC', 'PFNGLVERTEXATTRIBI4UBVEXTPROC',
'PFNGLVERTEXATTRIBI4USVEXTPROC', 'PFNGLVERTEXATTRIBIPOINTEREXTPROC',
'PFNGLGETVERTEXATTRIBIIVEXTPROC', 'PFNGLGETVERTEXATTRIBIUIVEXTPROC',
'GL_EXT_gpu_shader4', 'glGetUniformuivEXT', 'glBindFragDataLocationEXT',
'glGetFragDataLocationEXT', 'glUniform1uiEXT', 'glUniform2uiEXT',
'glUniform3uiEXT', 'glUniform4uiEXT', 'glUniform1uivEXT', 'glUniform2uivEXT',
'glUniform3uivEXT', 'glUniform4uivEXT', 'PFNGLGETUNIFORMUIVEXTPROC',
'PFNGLBINDFRAGDATALOCATIONEXTPROC', 'PFNGLGETFRAGDATALOCATIONEXTPROC',
'PFNGLUNIFORM1UIEXTPROC', 'PFNGLUNIFORM2UIEXTPROC', 'PFNGLUNIFORM3UIEXTPROC',
'PFNGLUNIFORM4UIEXTPROC', 'PFNGLUNIFORM1UIVEXTPROC',
'PFNGLUNIFORM2UIVEXTPROC', 'PFNGLUNIFORM3UIVEXTPROC',
'PFNGLUNIFORM4UIVEXTPROC', 'GL_EXT_draw_instanced',
'glDrawArraysInstancedEXT', 'glDrawElementsInstancedEXT',
'PFNGLDRAWARRAYSINSTANCEDEXTPROC', 'PFNGLDRAWELEMENTSINSTANCEDEXTPROC',
'GL_EXT_packed_float', 'GL_EXT_texture_array', 'GL_EXT_texture_buffer_object',
'glTexBufferEXT', 'PFNGLTEXBUFFEREXTPROC', 'GL_EXT_texture_compression_latc',
'GL_EXT_texture_compression_rgtc', 'GL_EXT_texture_shared_exponent',
'GL_NV_depth_buffer_float', 'glDepthRangedNV', 'glClearDepthdNV',
'glDepthBoundsdNV', 'PFNGLDEPTHRANGEDNVPROC', 'PFNGLCLEARDEPTHDNVPROC',
'PFNGLDEPTHBOUNDSDNVPROC', 'GL_NV_fragment_program4',
'GL_NV_framebuffer_multisample_coverage',
'glRenderbufferStorageMultisampleCoverageNV',
'PFNGLRENDERBUFFERSTORAGEMULTISAMPLECOVERAGENVPROC',
'GL_EXT_framebuffer_sRGB', 'GL_NV_geometry_shader4',
'GL_NV_parameter_buffer_object', 'glProgramBufferParametersfvNV',
'glProgramBufferParametersIivNV', 'glProgramBufferParametersIuivNV',
'PFNGLPROGRAMBUFFERPARAMETERSFVNVPROC',
'PFNGLPROGRAMBUFFERPARAMETERSIIVNVPROC',
'PFNGLPROGRAMBUFFERPARAMETERSIUIVNVPROC', 'GL_EXT_draw_buffers2',
'glColorMaskIndexedEXT', 'glGetBooleanIndexedvEXT', 'glGetIntegerIndexedvEXT',
'glEnableIndexedEXT', 'glDisableIndexedEXT', 'glIsEnabledIndexedEXT',
'PFNGLCOLORMASKINDEXEDEXTPROC', 'PFNGLGETBOOLEANINDEXEDVEXTPROC',
'PFNGLGETINTEGERINDEXEDVEXTPROC', 'PFNGLENABLEINDEXEDEXTPROC',
'PFNGLDISABLEINDEXEDEXTPROC', 'PFNGLISENABLEDINDEXEDEXTPROC',
'GL_NV_transform_feedback', 'glBeginTransformFeedbackNV',
'glEndTransformFeedbackNV', 'glTransformFeedbackAttribsNV',
'glBindBufferRangeNV', 'glBindBufferOffsetNV', 'glBindBufferBaseNV',
'glTransformFeedbackVaryingsNV', 'glActiveVaryingNV',
'glGetVaryingLocationNV', 'glGetActiveVaryingNV',
'glGetTransformFeedbackVaryingNV', 'glTransformFeedbackStreamAttribsNV',
'PFNGLBEGINTRANSFORMFEEDBACKNVPROC', 'PFNGLENDTRANSFORMFEEDBACKNVPROC',
'PFNGLTRANSFORMFEEDBACKATTRIBSNVPROC', 'PFNGLBINDBUFFERRANGENVPROC',
'PFNGLBINDBUFFEROFFSETNVPROC', 'PFNGLBINDBUFFERBASENVPROC',
'PFNGLTRANSFORMFEEDBACKVARYINGSNVPROC', 'PFNGLACTIVEVARYINGNVPROC',
'PFNGLGETVARYINGLOCATIONNVPROC', 'PFNGLGETACTIVEVARYINGNVPROC',
'PFNGLGETTRANSFORMFEEDBACKVARYINGNVPROC',
'PFNGLTRANSFORMFEEDBACKSTREAMATTRIBSNVPROC', 'GL_EXT_bindable_uniform',
'glUniformBufferEXT', 'glGetUniformBufferSizeEXT', 'glGetUniformOffsetEXT',
'PFNGLUNIFORMBUFFEREXTPROC', 'PFNGLGETUNIFORMBUFFERSIZEEXTPROC',
'PFNGLGETUNIFORMOFFSETEXTPROC', 'GL_EXT_texture_integer',
'glTexParameterIivEXT', 'glTexParameterIuivEXT', 'glGetTexParameterIivEXT',
'glGetTexParameterIuivEXT', 'glClearColorIiEXT', 'glClearColorIuiEXT',
'PFNGLTEXPARAMETERIIVEXTPROC', 'PFNGLTEXPARAMETERIUIVEXTPROC',
'PFNGLGETTEXPARAMETERIIVEXTPROC', 'PFNGLGETTEXPARAMETERIUIVEXTPROC',
'PFNGLCLEARCOLORIIEXTPROC', 'PFNGLCLEARCOLORIUIEXTPROC',
'GL_GREMEDY_frame_terminator', 'glFrameTerminatorGREMEDY',
'PFNGLFRAMETERMINATORGREMEDYPROC', 'GL_NV_conditional_render',
'glBeginConditionalRenderNV', 'glEndConditionalRenderNV',
'PFNGLBEGINCONDITIONALRENDERNVPROC', 'PFNGLENDCONDITIONALRENDERNVPROC',
'GL_NV_present_video', 'glPresentFrameKeyedNV', 'glPresentFrameDualFillNV',
'glGetVideoivNV', 'glGetVideouivNV', 'glGetVideoi64vNV', 'glGetVideoui64vNV',
'PFNGLPRESENTFRAMEKEYEDNVPROC', 'PFNGLPRESENTFRAMEDUALFILLNVPROC',
'PFNGLGETVIDEOIVNVPROC', 'PFNGLGETVIDEOUIVNVPROC', 'PFNGLGETVIDEOI64VNVPROC',
'PFNGLGETVIDEOUI64VNVPROC', 'GL_EXT_transform_feedback',
'glBeginTransformFeedbackEXT', 'glEndTransformFeedbackEXT',
'glBindBufferRangeEXT', 'glBindBufferOffsetEXT', 'glBindBufferBaseEXT',
'glTransformFeedbackVaryingsEXT', 'glGetTransformFeedbackVaryingEXT',
'PFNGLBEGINTRANSFORMFEEDBACKEXTPROC', 'PFNGLENDTRANSFORMFEEDBACKEXTPROC',
'PFNGLBINDBUFFERRANGEEXTPROC', 'PFNGLBINDBUFFEROFFSETEXTPROC',
'PFNGLBINDBUFFERBASEEXTPROC', 'PFNGLTRANSFORMFEEDBACKVARYINGSEXTPROC',
'PFNGLGETTRANSFORMFEEDBACKVARYINGEXTPROC', 'GL_EXT_direct_state_access',
'glClientAttribDefaultEXT', 'glPushClientAttribDefaultEXT',
'glMatrixLoadfEXT', 'glMatrixLoaddEXT', 'glMatrixMultfEXT',
'glMatrixMultdEXT', 'glMatrixLoadIdentityEXT', 'glMatrixRotatefEXT',
'glMatrixRotatedEXT', 'glMatrixScalefEXT', 'glMatrixScaledEXT',
'glMatrixTranslatefEXT', 'glMatrixTranslatedEXT', 'glMatrixFrustumEXT',
'glMatrixOrthoEXT', 'glMatrixPopEXT', 'glMatrixPushEXT',
'glMatrixLoadTransposefEXT', 'glMatrixLoadTransposedEXT',
'glMatrixMultTransposefEXT', 'glMatrixMultTransposedEXT',
'glTextureParameterfEXT', 'glTextureParameterfvEXT', 'glTextureParameteriEXT',
'glTextureParameterivEXT', 'glTextureImage1DEXT', 'glTextureImage2DEXT',
'glTextureSubImage1DEXT', 'glTextureSubImage2DEXT', 'glCopyTextureImage1DEXT',
'glCopyTextureImage2DEXT', 'glCopyTextureSubImage1DEXT',
'glCopyTextureSubImage2DEXT', 'glGetTextureImageEXT',
'glGetTextureParameterfvEXT', 'glGetTextureParameterivEXT',
'glGetTextureLevelParameterfvEXT', 'glGetTextureLevelParameterivEXT',
'glTextureImage3DEXT', 'glTextureSubImage3DEXT', 'glCopyTextureSubImage3DEXT',
'glMultiTexParameterfEXT', 'glMultiTexParameterfvEXT',
'glMultiTexParameteriEXT', 'glMultiTexParameterivEXT', 'glMultiTexImage1DEXT',
'glMultiTexImage2DEXT', 'glMultiTexSubImage1DEXT', 'glMultiTexSubImage2DEXT',
'glCopyMultiTexImage1DEXT', 'glCopyMultiTexImage2DEXT',
'glCopyMultiTexSubImage1DEXT', 'glCopyMultiTexSubImage2DEXT',
'glGetMultiTexImageEXT', 'glGetMultiTexParameterfvEXT',
'glGetMultiTexParameterivEXT', 'glGetMultiTexLevelParameterfvEXT',
'glGetMultiTexLevelParameterivEXT', 'glMultiTexImage3DEXT',
'glMultiTexSubImage3DEXT', 'glCopyMultiTexSubImage3DEXT',
'glBindMultiTextureEXT', 'glEnableClientStateIndexedEXT',
'glDisableClientStateIndexedEXT', 'glMultiTexCoordPointerEXT',
'glMultiTexEnvfEXT', 'glMultiTexEnvfvEXT', 'glMultiTexEnviEXT',
'glMultiTexEnvivEXT', 'glMultiTexGendEXT', 'glMultiTexGendvEXT',
'glMultiTexGenfEXT', 'glMultiTexGenfvEXT', 'glMultiTexGeniEXT',
'glMultiTexGenivEXT', 'glGetMultiTexEnvfvEXT', 'glGetMultiTexEnvivEXT',
'glGetMultiTexGendvEXT', 'glGetMultiTexGenfvEXT', 'glGetMultiTexGenivEXT',
'glGetFloatIndexedvEXT', 'glGetDoubleIndexedvEXT', 'glGetPointerIndexedvEXT',
'glCompressedTextureImage3DEXT', 'glCompressedTextureImage2DEXT',
'glCompressedTextureImage1DEXT', 'glCompressedTextureSubImage3DEXT',
'glCompressedTextureSubImage2DEXT', 'glCompressedTextureSubImage1DEXT',
'glGetCompressedTextureImageEXT', 'glCompressedMultiTexImage3DEXT',
'glCompressedMultiTexImage2DEXT', 'glCompressedMultiTexImage1DEXT',
'glCompressedMultiTexSubImage3DEXT', 'glCompressedMultiTexSubImage2DEXT',
'glCompressedMultiTexSubImage1DEXT', 'glGetCompressedMultiTexImageEXT',
'glNamedProgramStringEXT', 'glNamedProgramLocalParameter4dEXT',
'glNamedProgramLocalParameter4dvEXT', 'glNamedProgramLocalParameter4fEXT',
'glNamedProgramLocalParameter4fvEXT', 'glGetNamedProgramLocalParameterdvEXT',
'glGetNamedProgramLocalParameterfvEXT', 'glGetNamedProgramivEXT',
'glGetNamedProgramStringEXT', 'glNamedProgramLocalParameters4fvEXT',
'glNamedProgramLocalParameterI4iEXT', 'glNamedProgramLocalParameterI4ivEXT',
'glNamedProgramLocalParametersI4ivEXT', 'glNamedProgramLocalParameterI4uiEXT',
'glNamedProgramLocalParameterI4uivEXT',
'glNamedProgramLocalParametersI4uivEXT',
'glGetNamedProgramLocalParameterIivEXT',
'glGetNamedProgramLocalParameterIuivEXT', 'glTextureParameterIivEXT',
'glTextureParameterIuivEXT', 'glGetTextureParameterIivEXT',
'glGetTextureParameterIuivEXT', 'glMultiTexParameterIivEXT',
'glMultiTexParameterIuivEXT', 'glGetMultiTexParameterIivEXT',
'glGetMultiTexParameterIuivEXT', 'glProgramUniform1fEXT',
'glProgramUniform2fEXT', 'glProgramUniform3fEXT', 'glProgramUniform4fEXT',
'glProgramUniform1iEXT', 'glProgramUniform2iEXT', 'glProgramUniform3iEXT',
'glProgramUniform4iEXT', 'glProgramUniform1fvEXT', 'glProgramUniform2fvEXT',
'glProgramUniform3fvEXT', 'glProgramUniform4fvEXT', 'glProgramUniform1ivEXT',
'glProgramUniform2ivEXT', 'glProgramUniform3ivEXT', 'glProgramUniform4ivEXT',
'glProgramUniformMatrix2fvEXT', 'glProgramUniformMatrix3fvEXT',
'glProgramUniformMatrix4fvEXT', 'glProgramUniformMatrix2x3fvEXT',
'glProgramUniformMatrix3x2fvEXT', 'glProgramUniformMatrix2x4fvEXT',
'glProgramUniformMatrix4x2fvEXT', 'glProgramUniformMatrix3x4fvEXT',
'glProgramUniformMatrix4x3fvEXT', 'glProgramUniform1uiEXT',
'glProgramUniform2uiEXT', 'glProgramUniform3uiEXT', 'glProgramUniform4uiEXT',
'glProgramUniform1uivEXT', 'glProgramUniform2uivEXT',
'glProgramUniform3uivEXT', 'glProgramUniform4uivEXT', 'glNamedBufferDataEXT',
'glNamedBufferSubDataEXT', 'glMapNamedBufferEXT', 'glUnmapNamedBufferEXT',
'glMapNamedBufferRangeEXT', 'glFlushMappedNamedBufferRangeEXT',
'glNamedCopyBufferSubDataEXT', 'glGetNamedBufferParameterivEXT',
'glGetNamedBufferPointervEXT', 'glGetNamedBufferSubDataEXT',
'glTextureBufferEXT', 'glMultiTexBufferEXT', 'glNamedRenderbufferStorageEXT',
'glGetNamedRenderbufferParameterivEXT', 'glCheckNamedFramebufferStatusEXT',
'glNamedFramebufferTexture1DEXT', 'glNamedFramebufferTexture2DEXT',
'glNamedFramebufferTexture3DEXT', 'glNamedFramebufferRenderbufferEXT',
'glGetNamedFramebufferAttachmentParameterivEXT', 'glGenerateTextureMipmapEXT',
'glGenerateMultiTexMipmapEXT', 'glFramebufferDrawBufferEXT',
'glFramebufferDrawBuffersEXT', 'glFramebufferReadBufferEXT',
'glGetFramebufferParameterivEXT', 'glNamedRenderbufferStorageMultisampleEXT',
'glNamedRenderbufferStorageMultisampleCoverageEXT',
'glNamedFramebufferTextureEXT', 'glNamedFramebufferTextureLayerEXT',
'glNamedFramebufferTextureFaceEXT', 'glTextureRenderbufferEXT',
'glMultiTexRenderbufferEXT', 'glProgramUniform1dEXT', 'glProgramUniform2dEXT',
'glProgramUniform3dEXT', 'glProgramUniform4dEXT', 'glProgramUniform1dvEXT',
'glProgramUniform2dvEXT', 'glProgramUniform3dvEXT', 'glProgramUniform4dvEXT',
'glProgramUniformMatrix2dvEXT', 'glProgramUniformMatrix3dvEXT',
'glProgramUniformMatrix4dvEXT', 'glProgramUniformMatrix2x3dvEXT',
'glProgramUniformMatrix2x4dvEXT', 'glProgramUniformMatrix3x2dvEXT',
'glProgramUniformMatrix3x4dvEXT', 'glProgramUniformMatrix4x2dvEXT',
'glProgramUniformMatrix4x3dvEXT', 'PFNGLCLIENTATTRIBDEFAULTEXTPROC',
'PFNGLPUSHCLIENTATTRIBDEFAULTEXTPROC', 'PFNGLMATRIXLOADFEXTPROC',
'PFNGLMATRIXLOADDEXTPROC', 'PFNGLMATRIXMULTFEXTPROC',
'PFNGLMATRIXMULTDEXTPROC', 'PFNGLMATRIXLOADIDENTITYEXTPROC',
'PFNGLMATRIXROTATEFEXTPROC', 'PFNGLMATRIXROTATEDEXTPROC',
'PFNGLMATRIXSCALEFEXTPROC', 'PFNGLMATRIXSCALEDEXTPROC',
'PFNGLMATRIXTRANSLATEFEXTPROC', 'PFNGLMATRIXTRANSLATEDEXTPROC',
'PFNGLMATRIXFRUSTUMEXTPROC', 'PFNGLMATRIXORTHOEXTPROC',
'PFNGLMATRIXPOPEXTPROC', 'PFNGLMATRIXPUSHEXTPROC',
'PFNGLMATRIXLOADTRANSPOSEFEXTPROC', 'PFNGLMATRIXLOADTRANSPOSEDEXTPROC',
'PFNGLMATRIXMULTTRANSPOSEFEXTPROC', 'PFNGLMATRIXMULTTRANSPOSEDEXTPROC',
'PFNGLTEXTUREPARAMETERFEXTPROC', 'PFNGLTEXTUREPARAMETERFVEXTPROC',
'PFNGLTEXTUREPARAMETERIEXTPROC', 'PFNGLTEXTUREPARAMETERIVEXTPROC',
'PFNGLTEXTUREIMAGE1DEXTPROC', 'PFNGLTEXTUREIMAGE2DEXTPROC',
'PFNGLTEXTURESUBIMAGE1DEXTPROC', 'PFNGLTEXTURESUBIMAGE2DEXTPROC',
'PFNGLCOPYTEXTUREIMAGE1DEXTPROC', 'PFNGLCOPYTEXTUREIMAGE2DEXTPROC',
'PFNGLCOPYTEXTURESUBIMAGE1DEXTPROC', 'PFNGLCOPYTEXTURESUBIMAGE2DEXTPROC',
'PFNGLGETTEXTUREIMAGEEXTPROC', 'PFNGLGETTEXTUREPARAMETERFVEXTPROC',
'PFNGLGETTEXTUREPARAMETERIVEXTPROC', 'PFNGLGETTEXTURELEVELPARAMETERFVEXTPROC',
'PFNGLGETTEXTURELEVELPARAMETERIVEXTPROC', 'PFNGLTEXTUREIMAGE3DEXTPROC',
'PFNGLTEXTURESUBIMAGE3DEXTPROC', 'PFNGLCOPYTEXTURESUBIMAGE3DEXTPROC',
'PFNGLMULTITEXPARAMETERFEXTPROC', 'PFNGLMULTITEXPARAMETERFVEXTPROC',
'PFNGLMULTITEXPARAMETERIEXTPROC', 'PFNGLMULTITEXPARAMETERIVEXTPROC',
'PFNGLMULTITEXIMAGE1DEXTPROC', 'PFNGLMULTITEXIMAGE2DEXTPROC',
'PFNGLMULTITEXSUBIMAGE1DEXTPROC', 'PFNGLMULTITEXSUBIMAGE2DEXTPROC',
'PFNGLCOPYMULTITEXIMAGE1DEXTPROC', 'PFNGLCOPYMULTITEXIMAGE2DEXTPROC',
'PFNGLCOPYMULTITEXSUBIMAGE1DEXTPROC', 'PFNGLCOPYMULTITEXSUBIMAGE2DEXTPROC',
'PFNGLGETMULTITEXIMAGEEXTPROC', 'PFNGLGETMULTITEXPARAMETERFVEXTPROC',
'PFNGLGETMULTITEXPARAMETERIVEXTPROC',
'PFNGLGETMULTITEXLEVELPARAMETERFVEXTPROC',
'PFNGLGETMULTITEXLEVELPARAMETERIVEXTPROC', 'PFNGLMULTITEXIMAGE3DEXTPROC',
'PFNGLMULTITEXSUBIMAGE3DEXTPROC', 'PFNGLCOPYMULTITEXSUBIMAGE3DEXTPROC',
'PFNGLBINDMULTITEXTUREEXTPROC', 'PFNGLENABLECLIENTSTATEINDEXEDEXTPROC',
'PFNGLDISABLECLIENTSTATEINDEXEDEXTPROC', 'PFNGLMULTITEXCOORDPOINTEREXTPROC',
'PFNGLMULTITEXENVFEXTPROC', 'PFNGLMULTITEXENVFVEXTPROC',
'PFNGLMULTITEXENVIEXTPROC', 'PFNGLMULTITEXENVIVEXTPROC',
'PFNGLMULTITEXGENDEXTPROC', 'PFNGLMULTITEXGENDVEXTPROC',
'PFNGLMULTITEXGENFEXTPROC', 'PFNGLMULTITEXGENFVEXTPROC',
'PFNGLMULTITEXGENIEXTPROC', 'PFNGLMULTITEXGENIVEXTPROC',
'PFNGLGETMULTITEXENVFVEXTPROC', 'PFNGLGETMULTITEXENVIVEXTPROC',
'PFNGLGETMULTITEXGENDVEXTPROC', 'PFNGLGETMULTITEXGENFVEXTPROC',
'PFNGLGETMULTITEXGENIVEXTPROC', 'PFNGLGETFLOATINDEXEDVEXTPROC',
'PFNGLGETDOUBLEINDEXEDVEXTPROC', 'PFNGLGETPOINTERINDEXEDVEXTPROC',
'PFNGLCOMPRESSEDTEXTUREIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDTEXTUREIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDTEXTUREIMAGE1DEXTPROC',
'PFNGLCOMPRESSEDTEXTURESUBIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDTEXTURESUBIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDTEXTURESUBIMAGE1DEXTPROC',
'PFNGLGETCOMPRESSEDTEXTUREIMAGEEXTPROC',
'PFNGLCOMPRESSEDMULTITEXIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXIMAGE1DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXSUBIMAGE3DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXSUBIMAGE2DEXTPROC',
'PFNGLCOMPRESSEDMULTITEXSUBIMAGE1DEXTPROC',
'PFNGLGETCOMPRESSEDMULTITEXIMAGEEXTPROC', 'PFNGLNAMEDPROGRAMSTRINGEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4DEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4DVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4FEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETER4FVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERDVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERFVEXTPROC',
'PFNGLGETNAMEDPROGRAMIVEXTPROC', 'PFNGLGETNAMEDPROGRAMSTRINGEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERS4FVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4IEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4IVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERSI4IVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERI4UIVEXTPROC',
'PFNGLNAMEDPROGRAMLOCALPARAMETERSI4UIVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERIIVEXTPROC',
'PFNGLGETNAMEDPROGRAMLOCALPARAMETERIUIVEXTPROC',
'PFNGLTEXTUREPARAMETERIIVEXTPROC', 'PFNGLTEXTUREPARAMETERIUIVEXTPROC',
'PFNGLGETTEXTUREPARAMETERIIVEXTPROC', 'PFNGLGETTEXTUREPARAMETERIUIVEXTPROC',
'PFNGLMULTITEXPARAMETERIIVEXTPROC', 'PFNGLMULTITEXPARAMETERIUIVEXTPROC',
'PFNGLGETMULTITEXPARAMETERIIVEXTPROC', 'PFNGLGETMULTITEXPARAMETERIUIVEXTPROC',
'PFNGLPROGRAMUNIFORM1FEXTPROC', 'PFNGLPROGRAMUNIFORM2FEXTPROC',
'PFNGLPROGRAMUNIFORM3FEXTPROC', 'PFNGLPROGRAMUNIFORM4FEXTPROC',
'PFNGLPROGRAMUNIFORM1IEXTPROC', 'PFNGLPROGRAMUNIFORM2IEXTPROC',
'PFNGLPROGRAMUNIFORM3IEXTPROC', 'PFNGLPROGRAMUNIFORM4IEXTPROC',
'PFNGLPROGRAMUNIFORM1FVEXTPROC', 'PFNGLPROGRAMUNIFORM2FVEXTPROC',
'PFNGLPROGRAMUNIFORM3FVEXTPROC', 'PFNGLPROGRAMUNIFORM4FVEXTPROC',
'PFNGLPROGRAMUNIFORM1IVEXTPROC', 'PFNGLPROGRAMUNIFORM2IVEXTPROC',
'PFNGLPROGRAMUNIFORM3IVEXTPROC', 'PFNGLPROGRAMUNIFORM4IVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2FVEXTPROC', 'PFNGLPROGRAMUNIFORMMATRIX3FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2X3FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X2FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2X4FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X2FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X4FVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X3FVEXTPROC', 'PFNGLPROGRAMUNIFORM1UIEXTPROC',
'PFNGLPROGRAMUNIFORM2UIEXTPROC', 'PFNGLPROGRAMUNIFORM3UIEXTPROC',
'PFNGLPROGRAMUNIFORM4UIEXTPROC', 'PFNGLPROGRAMUNIFORM1UIVEXTPROC',
'PFNGLPROGRAMUNIFORM2UIVEXTPROC', 'PFNGLPROGRAMUNIFORM3UIVEXTPROC',
'PFNGLPROGRAMUNIFORM4UIVEXTPROC', 'PFNGLNAMEDBUFFERDATAEXTPROC',
'PFNGLNAMEDBUFFERSUBDATAEXTPROC', 'PFNGLMAPNAMEDBUFFEREXTPROC',
'PFNGLUNMAPNAMEDBUFFEREXTPROC', 'PFNGLMAPNAMEDBUFFERRANGEEXTPROC',
'PFNGLFLUSHMAPPEDNAMEDBUFFERRANGEEXTPROC',
'PFNGLNAMEDCOPYBUFFERSUBDATAEXTPROC', 'PFNGLGETNAMEDBUFFERPARAMETERIVEXTPROC',
'PFNGLGETNAMEDBUFFERPOINTERVEXTPROC', 'PFNGLGETNAMEDBUFFERSUBDATAEXTPROC',
'PFNGLTEXTUREBUFFEREXTPROC', 'PFNGLMULTITEXBUFFEREXTPROC',
'PFNGLNAMEDRENDERBUFFERSTORAGEEXTPROC',
'PFNGLGETNAMEDRENDERBUFFERPARAMETERIVEXTPROC',
'PFNGLCHECKNAMEDFRAMEBUFFERSTATUSEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURE1DEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURE2DEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURE3DEXTPROC',
'PFNGLNAMEDFRAMEBUFFERRENDERBUFFEREXTPROC',
'PFNGLGETNAMEDFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC',
'PFNGLGENERATETEXTUREMIPMAPEXTPROC', 'PFNGLGENERATEMULTITEXMIPMAPEXTPROC',
'PFNGLFRAMEBUFFERDRAWBUFFEREXTPROC', 'PFNGLFRAMEBUFFERDRAWBUFFERSEXTPROC',
'PFNGLFRAMEBUFFERREADBUFFEREXTPROC', 'PFNGLGETFRAMEBUFFERPARAMETERIVEXTPROC',
'PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC',
'PFNGLNAMEDRENDERBUFFERSTORAGEMULTISAMPLECOVERAGEEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTUREEXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTURELAYEREXTPROC',
'PFNGLNAMEDFRAMEBUFFERTEXTUREFACEEXTPROC', 'PFNGLTEXTURERENDERBUFFEREXTPROC',
'PFNGLMULTITEXRENDERBUFFEREXTPROC', 'PFNGLPROGRAMUNIFORM1DEXTPROC',
'PFNGLPROGRAMUNIFORM2DEXTPROC', 'PFNGLPROGRAMUNIFORM3DEXTPROC',
'PFNGLPROGRAMUNIFORM4DEXTPROC', 'PFNGLPROGRAMUNIFORM1DVEXTPROC',
'PFNGLPROGRAMUNIFORM2DVEXTPROC', 'PFNGLPROGRAMUNIFORM3DVEXTPROC',
'PFNGLPROGRAMUNIFORM4DVEXTPROC', 'PFNGLPROGRAMUNIFORMMATRIX2DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3DVEXTPROC', 'PFNGLPROGRAMUNIFORMMATRIX4DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2X3DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX2X4DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X2DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX3X4DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X2DVEXTPROC',
'PFNGLPROGRAMUNIFORMMATRIX4X3DVEXTPROC', 'GL_EXT_vertex_array_bgra',
'GL_EXT_texture_swizzle', 'GL_NV_explicit_multisample',
'glGetMultisamplefvNV', 'glSampleMaskIndexedNV', 'glTexRenderbufferNV',
'PFNGLGETMULTISAMPLEFVNVPROC', 'PFNGLSAMPLEMASKINDEXEDNVPROC',
'PFNGLTEXRENDERBUFFERNVPROC', 'GL_NV_transform_feedback2',
'glBindTransformFeedbackNV', 'glDeleteTransformFeedbacksNV',
'glGenTransformFeedbacksNV', 'glIsTransformFeedbackNV',
'glPauseTransformFeedbackNV', 'glResumeTransformFeedbackNV',
'glDrawTransformFeedbackNV', 'PFNGLBINDTRANSFORMFEEDBACKNVPROC',
'PFNGLDELETETRANSFORMFEEDBACKSNVPROC', 'PFNGLGENTRANSFORMFEEDBACKSNVPROC',
'PFNGLISTRANSFORMFEEDBACKNVPROC', 'PFNGLPAUSETRANSFORMFEEDBACKNVPROC',
'PFNGLRESUMETRANSFORMFEEDBACKNVPROC', 'PFNGLDRAWTRANSFORMFEEDBACKNVPROC',
'GL_ATI_meminfo', 'GL_AMD_performance_monitor', 'glGetPerfMonitorGroupsAMD',
'glGetPerfMonitorCountersAMD', 'glGetPerfMonitorGroupStringAMD',
'glGetPerfMonitorCounterStringAMD', 'glGetPerfMonitorCounterInfoAMD',
'glGenPerfMonitorsAMD', 'glDeletePerfMonitorsAMD',
'glSelectPerfMonitorCountersAMD', 'glBeginPerfMonitorAMD',
'glEndPerfMonitorAMD', 'glGetPerfMonitorCounterDataAMD',
'PFNGLGETPERFMONITORGROUPSAMDPROC', 'PFNGLGETPERFMONITORCOUNTERSAMDPROC',
'PFNGLGETPERFMONITORGROUPSTRINGAMDPROC',
'PFNGLGETPERFMONITORCOUNTERSTRINGAMDPROC',
'PFNGLGETPERFMONITORCOUNTERINFOAMDPROC', 'PFNGLGENPERFMONITORSAMDPROC',
'PFNGLDELETEPERFMONITORSAMDPROC', 'PFNGLSELECTPERFMONITORCOUNTERSAMDPROC',
'PFNGLBEGINPERFMONITORAMDPROC', 'PFNGLENDPERFMONITORAMDPROC',
'PFNGLGETPERFMONITORCOUNTERDATAAMDPROC', 'GL_AMD_texture_texture4',
'GL_AMD_vertex_shader_tesselator', 'glTessellationFactorAMD',
'glTessellationModeAMD', 'PFNGLTESSELLATIONFACTORAMDPROC',
'PFNGLTESSELLATIONMODEAMDPROC', 'GL_EXT_provoking_vertex',
'glProvokingVertexEXT', 'PFNGLPROVOKINGVERTEXEXTPROC', 'GL_EXT_texture_snorm',
'GL_AMD_draw_buffers_blend', 'glBlendFuncIndexedAMD',
'glBlendFuncSeparateIndexedAMD', 'glBlendEquationIndexedAMD',
'glBlendEquationSeparateIndexedAMD', 'PFNGLBLENDFUNCINDEXEDAMDPROC',
'PFNGLBLENDFUNCSEPARATEINDEXEDAMDPROC', 'PFNGLBLENDEQUATIONINDEXEDAMDPROC',
'PFNGLBLENDEQUATIONSEPARATEINDEXEDAMDPROC', 'GL_APPLE_texture_range',
'glTextureRangeAPPLE', 'glGetTexParameterPointervAPPLE',
'PFNGLTEXTURERANGEAPPLEPROC', 'PFNGLGETTEXPARAMETERPOINTERVAPPLEPROC',
'GL_APPLE_float_pixels', 'GL_APPLE_vertex_program_evaluators',
'glEnableVertexAttribAPPLE', 'glDisableVertexAttribAPPLE',
'glIsVertexAttribEnabledAPPLE', 'glMapVertexAttrib1dAPPLE',
'glMapVertexAttrib1fAPPLE', 'glMapVertexAttrib2dAPPLE',
'glMapVertexAttrib2fAPPLE', 'PFNGLENABLEVERTEXATTRIBAPPLEPROC',
'PFNGLDISABLEVERTEXATTRIBAPPLEPROC', 'PFNGLISVERTEXATTRIBENABLEDAPPLEPROC',
'PFNGLMAPVERTEXATTRIB1DAPPLEPROC', 'PFNGLMAPVERTEXATTRIB1FAPPLEPROC',
'PFNGLMAPVERTEXATTRIB2DAPPLEPROC', 'PFNGLMAPVERTEXATTRIB2FAPPLEPROC',
'GL_APPLE_aux_depth_stencil', 'GL_APPLE_object_purgeable',
'glObjectPurgeableAPPLE', 'glObjectUnpurgeableAPPLE',
'glGetObjectParameterivAPPLE', 'PFNGLOBJECTPURGEABLEAPPLEPROC',
'PFNGLOBJECTUNPURGEABLEAPPLEPROC', 'PFNGLGETOBJECTPARAMETERIVAPPLEPROC',
'GL_APPLE_row_bytes', 'GL_APPLE_rgb_422', 'GL_NV_video_capture',
'glBeginVideoCaptureNV', 'glBindVideoCaptureStreamBufferNV',
'glBindVideoCaptureStreamTextureNV', 'glEndVideoCaptureNV',
'glGetVideoCaptureivNV', 'glGetVideoCaptureStreamivNV',
'glGetVideoCaptureStreamfvNV', 'glGetVideoCaptureStreamdvNV',
'glVideoCaptureNV', 'glVideoCaptureStreamParameterivNV',
'glVideoCaptureStreamParameterfvNV', 'glVideoCaptureStreamParameterdvNV',
'PFNGLBEGINVIDEOCAPTURENVPROC', 'PFNGLBINDVIDEOCAPTURESTREAMBUFFERNVPROC',
'PFNGLBINDVIDEOCAPTURESTREAMTEXTURENVPROC', 'PFNGLENDVIDEOCAPTURENVPROC',
'PFNGLGETVIDEOCAPTUREIVNVPROC', 'PFNGLGETVIDEOCAPTURESTREAMIVNVPROC',
'PFNGLGETVIDEOCAPTURESTREAMFVNVPROC', 'PFNGLGETVIDEOCAPTURESTREAMDVNVPROC',
'PFNGLVIDEOCAPTURENVPROC', 'PFNGLVIDEOCAPTURESTREAMPARAMETERIVNVPROC',
'PFNGLVIDEOCAPTURESTREAMPARAMETERFVNVPROC',
'PFNGLVIDEOCAPTURESTREAMPARAMETERDVNVPROC', 'GL_NV_copy_image',
'glCopyImageSubDataNV', 'PFNGLCOPYIMAGESUBDATANVPROC',
'GL_EXT_separate_shader_objects', 'glUseShaderProgramEXT',
'glActiveProgramEXT', 'glCreateShaderProgramEXT',
'PFNGLUSESHADERPROGRAMEXTPROC', 'PFNGLACTIVEPROGRAMEXTPROC',
'PFNGLCREATESHADERPROGRAMEXTPROC', 'GL_NV_parameter_buffer_object2',
'GL_NV_shader_buffer_load', 'glMakeBufferResidentNV',
'glMakeBufferNonResidentNV', 'glIsBufferResidentNV',
'glMakeNamedBufferResidentNV', 'glMakeNamedBufferNonResidentNV',
'glIsNamedBufferResidentNV', 'glGetBufferParameterui64vNV',
'glGetNamedBufferParameterui64vNV', 'glGetIntegerui64vNV', 'glUniformui64NV',
'glUniformui64vNV', 'glGetUniformui64vNV', 'glProgramUniformui64NV',
'glProgramUniformui64vNV', 'PFNGLMAKEBUFFERRESIDENTNVPROC',
'PFNGLMAKEBUFFERNONRESIDENTNVPROC', 'PFNGLISBUFFERRESIDENTNVPROC',
'PFNGLMAKENAMEDBUFFERRESIDENTNVPROC', 'PFNGLMAKENAMEDBUFFERNONRESIDENTNVPROC',
'PFNGLISNAMEDBUFFERRESIDENTNVPROC', 'PFNGLGETBUFFERPARAMETERUI64VNVPROC',
'PFNGLGETNAMEDBUFFERPARAMETERUI64VNVPROC', 'PFNGLGETINTEGERUI64VNVPROC',
'PFNGLUNIFORMUI64NVPROC', 'PFNGLUNIFORMUI64VNVPROC',
'PFNGLGETUNIFORMUI64VNVPROC', 'PFNGLPROGRAMUNIFORMUI64NVPROC',
'PFNGLPROGRAMUNIFORMUI64VNVPROC', 'GL_NV_vertex_buffer_unified_memory',
'glBufferAddressRangeNV', 'glVertexFormatNV', 'glNormalFormatNV',
'glColorFormatNV', 'glIndexFormatNV', 'glTexCoordFormatNV',
'glEdgeFlagFormatNV', 'glSecondaryColorFormatNV', 'glFogCoordFormatNV',
'glVertexAttribFormatNV', 'glVertexAttribIFormatNV', 'glGetIntegerui64i_vNV',
'PFNGLBUFFERADDRESSRANGENVPROC', 'PFNGLVERTEXFORMATNVPROC',
'PFNGLNORMALFORMATNVPROC', 'PFNGLCOLORFORMATNVPROC', 'PFNGLINDEXFORMATNVPROC',
'PFNGLTEXCOORDFORMATNVPROC', 'PFNGLEDGEFLAGFORMATNVPROC',
'PFNGLSECONDARYCOLORFORMATNVPROC', 'PFNGLFOGCOORDFORMATNVPROC',
'PFNGLVERTEXATTRIBFORMATNVPROC', 'PFNGLVERTEXATTRIBIFORMATNVPROC',
'PFNGLGETINTEGERUI64I_VNVPROC', 'GL_NV_texture_barrier', 'glTextureBarrierNV',
'PFNGLTEXTUREBARRIERNVPROC', 'GL_AMD_shader_stencil_export',
'GL_AMD_seamless_cubemap_per_texture', 'GL_AMD_conservative_depth',
'GL_EXT_shader_image_load_store', 'glBindImageTextureEXT',
'glMemoryBarrierEXT', 'PFNGLBINDIMAGETEXTUREEXTPROC',
'PFNGLMEMORYBARRIEREXTPROC', 'GL_EXT_vertex_attrib_64bit',
'glVertexAttribL1dEXT', 'glVertexAttribL2dEXT', 'glVertexAttribL3dEXT',
'glVertexAttribL4dEXT', 'glVertexAttribL1dvEXT', 'glVertexAttribL2dvEXT',
'glVertexAttribL3dvEXT', 'glVertexAttribL4dvEXT', 'glVertexAttribLPointerEXT',
'glGetVertexAttribLdvEXT', 'glVertexArrayVertexAttribLOffsetEXT',
'PFNGLVERTEXATTRIBL1DEXTPROC', 'PFNGLVERTEXATTRIBL2DEXTPROC',
'PFNGLVERTEXATTRIBL3DEXTPROC', 'PFNGLVERTEXATTRIBL4DEXTPROC',
'PFNGLVERTEXATTRIBL1DVEXTPROC', 'PFNGLVERTEXATTRIBL2DVEXTPROC',
'PFNGLVERTEXATTRIBL3DVEXTPROC', 'PFNGLVERTEXATTRIBL4DVEXTPROC',
'PFNGLVERTEXATTRIBLPOINTEREXTPROC', 'PFNGLGETVERTEXATTRIBLDVEXTPROC',
'PFNGLVERTEXARRAYVERTEXATTRIBLOFFSETEXTPROC', 'GL_NV_gpu_program5',
'glProgramSubroutineParametersuivNV', 'glGetProgramSubroutineParameteruivNV',
'PFNGLPROGRAMSUBROUTINEPARAMETERSUIVNVPROC',
'PFNGLGETPROGRAMSUBROUTINEPARAMETERUIVNVPROC', 'GL_NV_gpu_shader5',
'glUniform1i64NV', 'glUniform2i64NV', 'glUniform3i64NV', 'glUniform4i64NV',
'glUniform1i64vNV', 'glUniform2i64vNV', 'glUniform3i64vNV',
'glUniform4i64vNV', 'glUniform1ui64NV', 'glUniform2ui64NV',
'glUniform3ui64NV', 'glUniform4ui64NV', 'glUniform1ui64vNV',
'glUniform2ui64vNV', 'glUniform3ui64vNV', 'glUniform4ui64vNV',
'glGetUniformi64vNV', 'glProgramUniform1i64NV', 'glProgramUniform2i64NV',
'glProgramUniform3i64NV', 'glProgramUniform4i64NV', 'glProgramUniform1i64vNV',
'glProgramUniform2i64vNV', 'glProgramUniform3i64vNV',
'glProgramUniform4i64vNV', 'glProgramUniform1ui64NV',
'glProgramUniform2ui64NV', 'glProgramUniform3ui64NV',
'glProgramUniform4ui64NV', 'glProgramUniform1ui64vNV',
'glProgramUniform2ui64vNV', 'glProgramUniform3ui64vNV',
'glProgramUniform4ui64vNV', 'PFNGLUNIFORM1I64NVPROC',
'PFNGLUNIFORM2I64NVPROC', 'PFNGLUNIFORM3I64NVPROC', 'PFNGLUNIFORM4I64NVPROC',
'PFNGLUNIFORM1I64VNVPROC', 'PFNGLUNIFORM2I64VNVPROC',
'PFNGLUNIFORM3I64VNVPROC', 'PFNGLUNIFORM4I64VNVPROC',
'PFNGLUNIFORM1UI64NVPROC', 'PFNGLUNIFORM2UI64NVPROC',
'PFNGLUNIFORM3UI64NVPROC', 'PFNGLUNIFORM4UI64NVPROC',
'PFNGLUNIFORM1UI64VNVPROC', 'PFNGLUNIFORM2UI64VNVPROC',
'PFNGLUNIFORM3UI64VNVPROC', 'PFNGLUNIFORM4UI64VNVPROC',
'PFNGLGETUNIFORMI64VNVPROC', 'PFNGLPROGRAMUNIFORM1I64NVPROC',
'PFNGLPROGRAMUNIFORM2I64NVPROC', 'PFNGLPROGRAMUNIFORM3I64NVPROC',
'PFNGLPROGRAMUNIFORM4I64NVPROC', 'PFNGLPROGRAMUNIFORM1I64VNVPROC',
'PFNGLPROGRAMUNIFORM2I64VNVPROC', 'PFNGLPROGRAMUNIFORM3I64VNVPROC',
'PFNGLPROGRAMUNIFORM4I64VNVPROC', 'PFNGLPROGRAMUNIFORM1UI64NVPROC',
'PFNGLPROGRAMUNIFORM2UI64NVPROC', 'PFNGLPROGRAMUNIFORM3UI64NVPROC',
'PFNGLPROGRAMUNIFORM4UI64NVPROC', 'PFNGLPROGRAMUNIFORM1UI64VNVPROC',
'PFNGLPROGRAMUNIFORM2UI64VNVPROC', 'PFNGLPROGRAMUNIFORM3UI64VNVPROC',
'PFNGLPROGRAMUNIFORM4UI64VNVPROC', 'GL_NV_shader_buffer_store',
'GL_NV_tessellation_program5', 'GL_NV_vertex_attrib_integer_64bit',
'glVertexAttribL1i64NV', 'glVertexAttribL2i64NV', 'glVertexAttribL3i64NV',
'glVertexAttribL4i64NV', 'glVertexAttribL1i64vNV', 'glVertexAttribL2i64vNV',
'glVertexAttribL3i64vNV', 'glVertexAttribL4i64vNV', 'glVertexAttribL1ui64NV',
'glVertexAttribL2ui64NV', 'glVertexAttribL3ui64NV', 'glVertexAttribL4ui64NV',
'glVertexAttribL1ui64vNV', 'glVertexAttribL2ui64vNV',
'glVertexAttribL3ui64vNV', 'glVertexAttribL4ui64vNV',
'glGetVertexAttribLi64vNV', 'glGetVertexAttribLui64vNV',
'glVertexAttribLFormatNV', 'PFNGLVERTEXATTRIBL1I64NVPROC',
'PFNGLVERTEXATTRIBL2I64NVPROC', 'PFNGLVERTEXATTRIBL3I64NVPROC',
'PFNGLVERTEXATTRIBL4I64NVPROC', 'PFNGLVERTEXATTRIBL1I64VNVPROC',
'PFNGLVERTEXATTRIBL2I64VNVPROC', 'PFNGLVERTEXATTRIBL3I64VNVPROC',
'PFNGLVERTEXATTRIBL4I64VNVPROC', 'PFNGLVERTEXATTRIBL1UI64NVPROC',
'PFNGLVERTEXATTRIBL2UI64NVPROC', 'PFNGLVERTEXATTRIBL3UI64NVPROC',
'PFNGLVERTEXATTRIBL4UI64NVPROC', 'PFNGLVERTEXATTRIBL1UI64VNVPROC',
'PFNGLVERTEXATTRIBL2UI64VNVPROC', 'PFNGLVERTEXATTRIBL3UI64VNVPROC',
'PFNGLVERTEXATTRIBL4UI64VNVPROC', 'PFNGLGETVERTEXATTRIBLI64VNVPROC',
'PFNGLGETVERTEXATTRIBLUI64VNVPROC', 'PFNGLVERTEXATTRIBLFORMATNVPROC',
'GL_NV_multisample_coverage', 'GL_AMD_name_gen_delete', 'glGenNamesAMD',
'glDeleteNamesAMD', 'glIsNameAMD', 'PFNGLGENNAMESAMDPROC',
'PFNGLDELETENAMESAMDPROC', 'PFNGLISNAMEAMDPROC', 'GL_AMD_debug_output',
'glDebugMessageEnableAMD', 'glDebugMessageInsertAMD',
'glDebugMessageCallbackAMD', 'glGetDebugMessageLogAMD',
'PFNGLDEBUGMESSAGEENABLEAMDPROC', 'PFNGLDEBUGMESSAGEINSERTAMDPROC',
'PFNGLDEBUGMESSAGECALLBACKAMDPROC', 'PFNGLGETDEBUGMESSAGELOGAMDPROC',
'GL_NV_vdpau_interop', 'glVDPAUInitNV', 'glVDPAUFiniNV',
'glVDPAURegisterVideoSurfaceNV', 'glVDPAURegisterOutputSurfaceNV',
'glVDPAUIsSurfaceNV', 'glVDPAUUnregisterSurfaceNV', 'glVDPAUGetSurfaceivNV',
'glVDPAUSurfaceAccessNV', 'glVDPAUMapSurfacesNV', 'glVDPAUUnmapSurfacesNV',
'PFNGLVDPAUINITNVPROC', 'PFNGLVDPAUFININVPROC',
'PFNGLVDPAUREGISTERVIDEOSURFACENVPROC',
'PFNGLVDPAUREGISTEROUTPUTSURFACENVPROC', 'PFNGLVDPAUISSURFACENVPROC',
'PFNGLVDPAUUNREGISTERSURFACENVPROC', 'PFNGLVDPAUGETSURFACEIVNVPROC',
'PFNGLVDPAUSURFACEACCESSNVPROC', 'PFNGLVDPAUMAPSURFACESNVPROC',
'PFNGLVDPAUUNMAPSURFACESNVPROC', 'GL_AMD_transform_feedback3_lines_triangles',
'GL_AMD_depth_clamp_separate', 'GL_EXT_texture_sRGB_decode',
'GL_NV_texture_multisample', 'glTexImage2DMultisampleCoverageNV',
'glTexImage3DMultisampleCoverageNV', 'glTextureImage2DMultisampleNV',
'glTextureImage3DMultisampleNV', 'glTextureImage2DMultisampleCoverageNV',
'glTextureImage3DMultisampleCoverageNV',
'PFNGLTEXIMAGE2DMULTISAMPLECOVERAGENVPROC',
'PFNGLTEXIMAGE3DMULTISAMPLECOVERAGENVPROC',
'PFNGLTEXTUREIMAGE2DMULTISAMPLENVPROC',
'PFNGLTEXTUREIMAGE3DMULTISAMPLENVPROC',
'PFNGLTEXTUREIMAGE2DMULTISAMPLECOVERAGENVPROC',
'PFNGLTEXTUREIMAGE3DMULTISAMPLECOVERAGENVPROC', 'GL_AMD_blend_minmax_factor',
'GL_AMD_sample_positions', 'glSetMultisamplefvAMD',
'PFNGLSETMULTISAMPLEFVAMDPROC', 'GL_EXT_x11_sync_object', 'glImportSyncEXT',
'PFNGLIMPORTSYNCEXTPROC', 'GL_AMD_multi_draw_indirect',
'glMultiDrawArraysIndirectAMD', 'glMultiDrawElementsIndirectAMD',
'PFNGLMULTIDRAWARRAYSINDIRECTAMDPROC',
'PFNGLMULTIDRAWELEMENTSINDIRECTAMDPROC',
'GL_EXT_framebuffer_multisample_blit_scaled', 'GL_NV_path_rendering',
'glGenPathsNV', 'glDeletePathsNV', 'glIsPathNV', 'glPathCommandsNV',
'glPathCoordsNV', 'glPathSubCommandsNV', 'glPathSubCoordsNV',
'glPathStringNV', 'glPathGlyphsNV', 'glPathGlyphRangeNV', 'glWeightPathsNV',
'glCopyPathNV', 'glInterpolatePathsNV', 'glTransformPathNV',
'glPathParameterivNV', 'glPathParameteriNV', 'glPathParameterfvNV',
'glPathParameterfNV', 'glPathDashArrayNV', 'glPathStencilFuncNV',
'glPathStencilDepthOffsetNV', 'glStencilFillPathNV', 'glStencilStrokePathNV',
'glStencilFillPathInstancedNV', 'glStencilStrokePathInstancedNV',
'glPathCoverDepthFuncNV', 'glPathColorGenNV', 'glPathTexGenNV',
'glPathFogGenNV', 'glCoverFillPathNV', 'glCoverStrokePathNV',
'glCoverFillPathInstancedNV', 'glCoverStrokePathInstancedNV',
'glGetPathParameterivNV', 'glGetPathParameterfvNV', 'glGetPathCommandsNV',
'glGetPathCoordsNV', 'glGetPathDashArrayNV', 'glGetPathMetricsNV',
'glGetPathMetricRangeNV', 'glGetPathSpacingNV', 'glGetPathColorGenivNV',
'glGetPathColorGenfvNV', 'glGetPathTexGenivNV', 'glGetPathTexGenfvNV',
'glIsPointInFillPathNV', 'glIsPointInStrokePathNV', 'glGetPathLengthNV',
'glPointAlongPathNV', 'PFNGLGENPATHSNVPROC', 'PFNGLDELETEPATHSNVPROC',
'PFNGLISPATHNVPROC', 'PFNGLPATHCOMMANDSNVPROC', 'PFNGLPATHCOORDSNVPROC',
'PFNGLPATHSUBCOMMANDSNVPROC', 'PFNGLPATHSUBCOORDSNVPROC',
'PFNGLPATHSTRINGNVPROC', 'PFNGLPATHGLYPHSNVPROC', 'PFNGLPATHGLYPHRANGENVPROC',
'PFNGLWEIGHTPATHSNVPROC', 'PFNGLCOPYPATHNVPROC',
'PFNGLINTERPOLATEPATHSNVPROC', 'PFNGLTRANSFORMPATHNVPROC',
'PFNGLPATHPARAMETERIVNVPROC', 'PFNGLPATHPARAMETERINVPROC',
'PFNGLPATHPARAMETERFVNVPROC', 'PFNGLPATHPARAMETERFNVPROC',
'PFNGLPATHDASHARRAYNVPROC', 'PFNGLPATHSTENCILFUNCNVPROC',
'PFNGLPATHSTENCILDEPTHOFFSETNVPROC', 'PFNGLSTENCILFILLPATHNVPROC',
'PFNGLSTENCILSTROKEPATHNVPROC', 'PFNGLSTENCILFILLPATHINSTANCEDNVPROC',
'PFNGLSTENCILSTROKEPATHINSTANCEDNVPROC', 'PFNGLPATHCOVERDEPTHFUNCNVPROC',
'PFNGLPATHCOLORGENNVPROC', 'PFNGLPATHTEXGENNVPROC', 'PFNGLPATHFOGGENNVPROC',
'PFNGLCOVERFILLPATHNVPROC', 'PFNGLCOVERSTROKEPATHNVPROC',
'PFNGLCOVERFILLPATHINSTANCEDNVPROC', 'PFNGLCOVERSTROKEPATHINSTANCEDNVPROC',
'PFNGLGETPATHPARAMETERIVNVPROC', 'PFNGLGETPATHPARAMETERFVNVPROC',
'PFNGLGETPATHCOMMANDSNVPROC', 'PFNGLGETPATHCOORDSNVPROC',
'PFNGLGETPATHDASHARRAYNVPROC', 'PFNGLGETPATHMETRICSNVPROC',
'PFNGLGETPATHMETRICRANGENVPROC', 'PFNGLGETPATHSPACINGNVPROC',
'PFNGLGETPATHCOLORGENIVNVPROC', 'PFNGLGETPATHCOLORGENFVNVPROC',
'PFNGLGETPATHTEXGENIVNVPROC', 'PFNGLGETPATHTEXGENFVNVPROC',
'PFNGLISPOINTINFILLPATHNVPROC', 'PFNGLISPOINTINSTROKEPATHNVPROC',
'PFNGLGETPATHLENGTHNVPROC', 'PFNGLPOINTALONGPATHNVPROC',
'GL_AMD_pinned_memory', 'GL_AMD_stencil_operation_extended',
'glStencilOpValueAMD', 'PFNGLSTENCILOPVALUEAMDPROC',
'GL_AMD_vertex_shader_viewport_index', 'GL_AMD_vertex_shader_layer',
'GL_NV_bindless_texture', 'glGetTextureHandleNV',
'glGetTextureSamplerHandleNV', 'glMakeTextureHandleResidentNV',
'glMakeTextureHandleNonResidentNV', 'glGetImageHandleNV',
'glMakeImageHandleResidentNV', 'glMakeImageHandleNonResidentNV',
'glUniformHandleui64NV', 'glUniformHandleui64vNV',
'glProgramUniformHandleui64NV', 'glProgramUniformHandleui64vNV',
'glIsTextureHandleResidentNV', 'glIsImageHandleResidentNV',
'PFNGLGETTEXTUREHANDLENVPROC', 'PFNGLGETTEXTURESAMPLERHANDLENVPROC',
'PFNGLMAKETEXTUREHANDLERESIDENTNVPROC',
'PFNGLMAKETEXTUREHANDLENONRESIDENTNVPROC', 'PFNGLGETIMAGEHANDLENVPROC',
'PFNGLMAKEIMAGEHANDLERESIDENTNVPROC', 'PFNGLMAKEIMAGEHANDLENONRESIDENTNVPROC',
'PFNGLUNIFORMHANDLEUI64NVPROC', 'PFNGLUNIFORMHANDLEUI64VNVPROC',
'PFNGLPROGRAMUNIFORMHANDLEUI64NVPROC', 'PFNGLPROGRAMUNIFORMHANDLEUI64VNVPROC',
'PFNGLISTEXTUREHANDLERESIDENTNVPROC', 'PFNGLISIMAGEHANDLERESIDENTNVPROC',
'GL_NV_shader_atomic_float', 'GL_AMD_query_buffer_object']
# END GENERATED CONTENT (do not edit above this line)






